<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\Sm00001000DocumentoPropostaFactory as DocumentoFactory;
use Teknisa\SM\Util\Utilities;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\SM\Util\PHPWord\SMTemplateProcessor;
use Teknisa\SM\Service\SmDocumentosService;

class Sm00001000DocumentoPropostaService {
    
    protected $connection;
    protected $documentoFactory;
    protected $fileServerBaseUrl;
    protected $smdocumentos;
    
    private $tempDir;
    
    public function __construct(EntityManager $entityManager, DocumentoFactory $documentoFactory, $fileServerBaseUrl, SmDocumentosService $smdocumentos) {
        $this->connection        = $entityManager->getConnection();
        $this->documentoFactory  = $documentoFactory;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
        $this->smdocumentos      = $smdocumentos;
        
        $this->tempDir = sys_get_temp_dir();
    }
    
    public function getDadosAnexo($nrseqorcamento, $nrtipoorcamento, $idSituacao, $nrorg) {
        $proposta = $this->getDadosProposta($nrseqorcamento, $nrorg);
        $cdidiom = $proposta["CDIDIOM"];
        $dadosAnexo = $this->getDocumentoProposta($nrseqorcamento, $nrtipoorcamento[0], $idSituacao, $cdidiom);
        return $dadosAnexo;
    }
    
    /**
     * Gera a mensagem de erro quando não forem encontratos documentos
    **/
    public function getErrorMessage($idSituacao, $nmidioma) {
        switch ($idSituacao) {
            case "ELABORACAO": 
                $idSituacao = "Em Elaboração";
                break;
            case "ACEITO": 
                $idSituacao = "Aceita";
                break;
            case "ENVIADO": 
                $idSituacao = "Documento Enviado";
                break;
            case "CONCLUIDO": 
                $idSituacao = "Concluída";
                break;
            case "CANCELADO": 
                $idSituacao = "Cancelada";
                break;
        }
        
        return "Operação bloqueada! <br/><br/>Essa proposta não possui documento cadastrado para a situação '$idSituacao' com o idioma '$nmidioma'.";
    }

    /**
     * Gera o documento da proposta.
     */
    public function gerarDocumentoProposta($nrseqorcamento, $nrtipoorcamento, $nrorg, $idSituacao, $idpermitedocx) {
        try {
            $nomeArquivoNovo = null;
            $ds = DIRECTORY_SEPARATOR;

            $proposta = $this->getDadosProposta($nrseqorcamento, $nrorg);
            $cdidiom = $proposta["CDIDIOM"];
            
            $nomeArquivos = array();
            $documentosAux = $this->getDocumentoProposta($nrseqorcamento, $nrtipoorcamento, $idSituacao, $cdidiom);
          
            if ((count($documentosAux)) == 0) {
                $nmidioma = $proposta["NMIDIOMA"];
                throw new \Exception($this->getErrorMessage($idSituacao, $nmidioma));
            }            
            
            foreach ($documentosAux as $doc) {
                $microtime = microtime(true);
                $microtime = str_replace(array(".", "&"), "_", $microtime);
                
                if ($doc["DSTIPO"] == "application/pdf") {
                    if(!in_array($doc["DSCAMINHOARQ"], $nomeArquivos)) {
                        $fileName = $microtime . "_" . $doc["NMARQUIVO"];
                        $caminhoFisicoArquivo = $this->smdocumentos->downloadDocFileServer($this->fileServerBaseUrl . $doc["DSCAMINHOARQ"], $fileName, $nrorg);
                        
                        $nomeArquivos[$doc["NRSEQORCADOC"]] = array(
                            "CAMINHO"        => "/downloadArquivo?requestType=Row&row[FILENAME]=" . urlencode(base64_encode($fileName)) . "&row[FILETYPE]=" . $doc["DSTIPO"],
                            "NMTIPOORCAMENTO"=> null,
                            "DADOSDOCUMENTO" => array(
                                "NMARQUIVO"    => $doc["NMARQUIVO"],
                                "DSCAMINHOARQ" => $caminhoFisicoArquivo,
                                "TYPE"         => $doc["DSTIPO"]
                            )
                        );
                    }
                } else {
                    $nrtipoorcamento = $doc["NRTIPOORCAMENTO"];
                    if (!empty($doc["DSCAMINHOARQ"])) {
                        $fileName = $microtime . "_" . $doc["NMARQUIVO"];
                        $nomeArquivoDocx = $this->smdocumentos->downloadDocFileServer($this->fileServerBaseUrl . $doc["DSCAMINHOARQ"], $fileName, $nrorg);
                        
                        if (is_file($nomeArquivoDocx)) {
                            $templateProcessor = new SMTemplateProcessor($nomeArquivoDocx);
    
                            //pega os dados principais da proposta
                            //$proposta = $this->getDadosProposta($nrseqorcamento, $nrorg);
    
                            //substitui os dados da proposta no documento
                            $this->substituirDadosPropostaDocumento($proposta, $templateProcessor);
    
                            //substitui a data inicial de faturamento
                            $this->substituirDiaMinimoDePagamentoDocumento($nrseqorcamento, $nrorg, $templateProcessor);
    
                            //substitui o logotipo do cliente
                            $caminhoLogotipo = $this->getCaminhoLogotipoParceiro($proposta["NRPARCNEGOCIO"], $nrorg);
                            if (!empty($caminhoLogotipo)) {
                                $this->substituirLogotipoDocumento($caminhoLogotipo, $templateProcessor, $doc["VRTAMANHOLOGO"]);
                            } else {
                                $templateProcessor->setValue("[LOGOTIPO]", "");
                            }
                            //busca os dados do comprador da proposta
                            $comprador = $this->getDadosCompradorProposta($proposta["NRPARCNEGOCIO"], $nrorg);
                            
                            //substitui os dados do comprador da proposta no documento
                            $this->substituirDadosCompradorPropostaDocumento($comprador, $templateProcessor);
                            
                            //retorna o endereço principal do parceiro (cliente) da proposta
                            $enderecoPrincipal = $this->getEnderecoParceiro($proposta["NRPARCNEGOCIO"], $nrorg, "PRINCIPAL");
                            
                            //substitui o endereço principal do comprador da proposta no documento
                            $this->substituirDadosEnderecoPrincipalCompradorPropostaDocumento($enderecoPrincipal, $templateProcessor);
                        
                            //busca o endereço de entrega do comprador da proposta
                            $enderecoEntregaComprador = $this->getEnderecoParceiro($proposta["NRPARCNEGOCIO"], $nrorg, "ENTREGA");
                            
                            //substitui o endereço de entrega do comprador da proposta no documento
                            $this->substituirDadosEnderecoEntregaCompradorPropostaDocumento($enderecoEntregaComprador, $templateProcessor);
                            
                            //retorna o endereço de entrega da proposta
                            $enderecoEntrega = $this->getEnderecoDeEntregaProposta($proposta["NRPARCNEGOCIO"], $nrorg);
                            
                            //substitui os dados do endereço de entrega no documento
                            $this->substituirEnderecoDeEntregaDocumento($enderecoEntrega, $enderecoPrincipal, $templateProcessor);
                            
                            //retorna o endereço principal do parceiro (cliente) da proposta
                            $enderecoPrincipal = $this->getEnderecoParceiro($proposta["NRPARCNEGOCIO"], $nrorg, "PRINCIPAL");
                            
                            //substitui os dados do cliente no documento
                            $this->substituirDadosClienteDocumento($proposta, $enderecoPrincipal, $templateProcessor);
                            
                            //retorna o vendedor principal da proposta
                            $vendedorPrincipal = $this->getVendedorPrincipalProposta($nrseqorcamento, $nrorg);
                            
                            //substitui os dados do vendedor no documento
                            $this->substituirDadosVendedorDocumento($vendedorPrincipal, $templateProcessor);
                            
                            //busca os dados do revenda da proposta
                            $revenda = $this->getDadosRevenda($proposta["NRREVENDA"], $nrorg);
                            
                            //retorna o endereço principal do revendedor da proposta
                            $enderecoRevenda = $this->getEnderecoParceiro($proposta["NRREVENDA"], $nrorg, "PRINCIPAL");

                            //substitui os dados do revenda da proposta no documento
                            $this->substituirDadosRevendaDocumento($revenda, $enderecoRevenda, $templateProcessor);

                            $documento = $templateProcessor->getTempDocumentMainPart();
                            
                            $gerarDocumento = $this->validarExistenciaItensDocumento($documento, $nrseqorcamento, $nrtipoorcamento, $nrorg);
                            
                            if(!$gerarDocumento) {
                                $nomeArquivos[$doc["NRSEQORCADOC"]] = array(
                                    "CAMINHO"        => null,
                                    "NMTIPOORCAMENTO"=> $this->getNomeEscopo($nrtipoorcamento, $nrorg),
                                    "DADOSDOCUMENTO" => array (
                                        "NMARQUIVO" => null,
                                        "DSCAMINHOARQ" => null,
                                        "TYPE"         => $doc["DSTIPO"]
                                    ),
                                );
                            } else {
                                //monta os mnemônicos com índice da parte de itens, elementos e adicionais no documento, para serem substituidos pelos valores depois
                                $documento = $this->gerarDocumentoParteDeItens($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento);
                                
                                //substitui os dados das Assinaturas da proposta.
                                $documento = $this->gerarDocumentoAssinaturas($nrseqorcamento, $nrorg, $documento);
                                
                                //substitui a descricao dos elementos
                                $documento = $this->gerarDocumentoParteDeDescricaoElementos($nrseqorcamento, $nrorg, $documento);
                                
                                //monta os mnemônicos da parte de previsão de pagamento no documento.
                                $documento = $this->gerarDocumentoParteDePrevisaoDePagamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento);
                                
                                //monta os mnemônicos da parte de parcelas das negociações de valores.
                                $documento = $this->gerarDocumentoParteDeNegociacoesValoresDeParcelamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento);
                                
                                //monta os mnemônicos da parte de formação de preço de serviço no documento.
                                $documento = $this->gerarDocumentoParteDeFormacoesDePreco($nrseqorcamento, $nrorg, $documento);
                                
                                //monta a arvore de elementos adquiridos na proposta
                                $documento = $this->gerarDocumentoParteElementoPaiComposicaoServico($nrseqorcamento, $nrorg, $documento);
                               
                                //gera a tabela de variações de preço da proposta
                                $documento = $this->gerarDocumentoParteVariacaoPreco($nrseqorcamento, $nrorg, $idSituacao, $nrtipoorcamento, $documento);
        
                                //monta os mnemônicos da parte das negociações de valores
                                $documento = $this->gerarDocumentoParteDeNegociacaoValores($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento);
        
                                //monta os mnemônicos da parte de desconto por período das negociações
                                $documento = $this->gerarDocumentoParteDeNegociacoesValoresParaDescontoPorPeriodo($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento);
                                
                                //Substitui os dados de alteração contratual
                                $documento = $this->substituirDadosAlteracaoContratual($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento);
                                
                                //Substitui os dados de revenda
                                $documento = $this->substituirDadosRevenda($revenda, $documento);

                                $valorTotalItens = 0;
                                $valorTotalTodosAdicionais = 0;
                                
                                $somaQuantidadeItens = 0;
                                $somaQuantidadeMedidaItens = 0;
                                $somaQuantidadeMedidaElementosItens = 0;
                                $somaValorTabelaPrecoItens = 0;
                                $somaValorUnitarioItens = 0;
                                $somaValorDescontoItens = 0;
                                $somaValorAcrescimoItens = 0;
                                $somaValorAjusteItens = 0;
                                $somaValorSemDescontoItens = 0;
                                $somaValorSemAjusteItens = 0;
                                $somaValorTotalItens = 0;
        
                                $itens = $this->getItensProposta($nrseqorcamento, $nrtipoorcamento, $nrorg);
                                foreach ($itens as $item) {
                                    //substitui os dados dos itens da proposta no documento
                                    $documento = $this->substituirDadosItensDocumento($item, $documento);
        
                                    $somaQuantidadeItens += $item["QTITEMORCAMENTO"];
                                    $somaQuantidadeMedidaItens += $item["QTMEDIDAORCA"];
                                    $somaQuantidadeMedidaElementosItens += $item["QTMEDSERVORCA"];
                                    $valorTabelaPreco = ($item["QTMEDSERVORCA"] == 0) ? 0 : (($item["VRTOTITEMSEMDESC"] - $item["VRACRESCIMO"]) / $item["QTMEDSERVORCA"]);
                                    $somaValorTabelaPrecoItens += $valorTabelaPreco;
                                    $somaValorUnitarioItens += $item["VRUNITITEMORCA"];
                                    $somaValorDescontoItens += $item["VRDESCITEM"];
                                    $somaValorAcrescimoItens += $item["VRACRESCIMO"];
                                    $vrAjuste = $item["VRACRESCIMO"] - $item["VRDESCITEM"];
                                    $somaValorAjusteItens += $vrAjuste;
                                    $somaValorSemDescontoItens += $item["VRTOTITEMSEMDESC"];
                                    $somaValorSemAjusteItens += $item["VRTOTITEMSEMDESC"];
                                    $somaValorTotalItens += $item["VRTOTITEMCOMDESC"];
                                    
                                    $elementos = $this->getElementosProposta($item["CODITEM"], $nrseqorcamento, $nrtipoorcamento, $nrorg);
                                    foreach ($elementos as $elemento) {
                                        //substitui os dados dos elementos da proposta no documento
                                        $documento = $this->substituirDadosElementosDocumento($item["NRITEMORCAMENTO"], $elemento, $documento);
                                    }
                                    
                                    $valorTotalAdicionaisDoItem = 0;
        
                                    $itensAdicionais = $this->getItensAdicionaisProposta($nrseqorcamento, $nrtipoorcamento, $item["NRITEMORCAMENTO"], $nrorg);
                                    foreach ($itensAdicionais as $itemAdicional) {
                                        //substitui os dados dos itens adicionais da proposta no documento
                                        $documento = $this->substituirDadosItensAdicionaisDocumento($item, $itemAdicional, $documento);
                                        
                                        $valorTotalAdicionaisDoItem += (float)$itemAdicional["VRTOTALITEMORCA"];
                                    }
                                    
                                    $valorTotalTodosAdicionais += $valorTotalAdicionaisDoItem;
                                    $documento = str_replace("[TOTITEMAD" . $item["NRITEMORCAMENTO"] . "1]", Utilities::formatarNumero($valorTotalAdicionaisDoItem), $documento);
                                    
                                    $valorTotalItens += (float)$item["VRTOTITEMCOMDESC"];
                                }
        
                                if (count($itens) == 0) {
                                    $documento = str_replace("[SUMQNT]", "", $documento);
                                    $documento = str_replace("[SUMQTMED]", "", $documento);
                                    $documento = str_replace("[SUMQTMEDELITENS]", "", $documento);
                                    $documento = str_replace("[SUMVRTBLPRECO]", "", $documento);
                                    $documento = str_replace("[SUMVRUNIT]", "", $documento);
                                    $documento = str_replace("[SUMVRDESC]", "", $documento);
                                    $documento = str_replace("[SUMVRACRE]", "", $documento);
                                    $documento = str_replace("[SUMVRAJUSTE]", "", $documento);
                                    $documento = str_replace("[SUMVRSEMDESC]", "", $documento);
                                    $documento = str_replace("[SUMVRSEMAJUSTE]", "", $documento);
                                    $documento = str_replace("[SUMVRTOT]", "", $documento);
                                } else {
                                    $documento = str_replace("[SUMQNT]", Utilities::formatarNumero($somaQuantidadeItens), $documento);
                                    $documento = str_replace("[SUMQTMED]", Utilities::formatarNumero($somaQuantidadeMedidaItens), $documento);
                                    $documento = str_replace("[SUMQTMEDELITENS]", Utilities::formatarNumero($somaQuantidadeMedidaElementosItens), $documento);
                                    $documento = str_replace("[SUMVRTBLPRECO]", Utilities::formatarNumero($somaValorTabelaPrecoItens), $documento);
                                    $documento = str_replace("[SUMVRUNIT]", Utilities::formatarNumero($somaValorUnitarioItens), $documento);
                                    $documento = str_replace("[SUMVRDESC]", Utilities::formatarNumero($somaValorDescontoItens), $documento);
                                    $documento = str_replace("[SUMVRACRE]", Utilities::formatarNumero($somaValorAcrescimoItens), $documento);
                                    $documento = str_replace("[SUMVRAJUSTE]", Utilities::formatarNumero($somaValorAjusteItens), $documento);
                                    $documento = str_replace("[SUMVRSEMDESC]", Utilities::formatarNumero($somaValorSemDescontoItens), $documento);
                                    $documento = str_replace("[SUMVRSEMAJUSTE]", Utilities::formatarNumero($somaValorSemAjusteItens), $documento);
                                    $documento = str_replace("[SUMVRTOT]", Utilities::formatarNumero($somaValorTotalItens), $documento);
                                }
                                
                                $idfaturacontr = "'A', 'X'";
                                $itensPropostaPeriodicoOuApuracao = $this->documentoFactory->getItensPropostaComParametrosFaturamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $idfaturacontr);
                                
                                foreach ($itensPropostaPeriodicoOuApuracao as $item) {
                                    $competencia = $this->getPrimeiraCompetenciaNegociacaoProposta($item["NRNEGOCIACAOPROPOSTA"], $nrorg);
                                    if ($competencia) {
                                        $dataIniFat = $this->calcularDataInicioFaturamentoCompetencia($item, $competencia);
                                        $diaVencimento = $this->calcularDiaVencimentoCompetencia($competencia, $dataIniFat);
                                        
                                        //substitui os dados dos itens da previsão de pagamento
                                        $documento = $this->substituirDadosItensPrevisaoDePagamentoPeriodicoOuApuracao($item, $dataIniFat, $diaVencimento, $documento);
                                    }
                                }
                                
                                $idfaturacontr = "'P'";
                                $itensPropostaNaoPeriodico = $this->documentoFactory->getItensPropostaComParametrosFaturamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $idfaturacontr);
                                foreach ($itensPropostaNaoPeriodico as $item) {
                                    //substitui os dados dos itens da previsão de pagamento
                                    $documento = $this->substituirDadosItensPrevisaoDePagamentoNaoPeriodico($item, $documento);
                                }
                                
                                $formacoesPreco = $this->getFormacoesPrecoProposta($nrseqorcamento, $nrorg);
                                foreach ($formacoesPreco as $formacao) {
                                    //substitui os dados da formação de preço da proposta no documento
                                    $documento = $this->substituirDadosFormacoesPrecoDocumento($formacao, $documento);
                                    
                                    $itensFormacaoPreco = $this->getItensFormacaoPrecoProposta($formacao["NRFORMPRECOSERVICO"], $nrorg);
                                    foreach ($itensFormacaoPreco as $itemFormacao) {
                                        //substitui os dados dos elementos da proposta no documento
                                        $documento = $this->substituirDadosItensFormacaoPrecoDocumento($formacao["NRFORMPRECOSERVICO"], $itemFormacao, $documento);
                                    }
                                }
        
                                $negociacoesValores = $this->getNegociacoesValores($nrseqorcamento, $nrtipoorcamento, $nrorg, $cdidiom);
                                
                                $somaValorTotalNegociacao = 0;
                                $somaValorDescontoNegociacao = 0;
                                $somaValorTotalComDescontoNegociacao = 0;
                                
                                foreach ($negociacoesValores as $negociacao) {
                                    //substitui os dados das negociações de valores da proposta no documento
                                    $documento = $this->substituirDadosNegociacoesValoresDocumento($negociacao, $documento);
        
                                    //calcula os totalizadores da negociação de valoes
                                    $somaValorTotalNegociacao += $negociacao["VRTOTITEM"];
                                    $somaValorDescontoNegociacao += $negociacao["VRDESCITEM"];
                                    $somaValorTotalComDescontoNegociacao += $negociacao["VRTOTITEMCOMDESC"];
                                }
        
                                //substitui os totalizadores da negociação de valores
                                if (count($negociacoesValores) == 0) {
                                    $documento = str_replace("[SUMVRTOTNV]", "", $documento);
                                    $documento = str_replace("[SUMVRDESCNV]", "", $documento);
                                    $documento = str_replace("[SUMVRTOTCOMDESCNV]", "", $documento);
                                } else {
                                    $documento = str_replace("[SUMVRTOTNV]", Utilities::formatarNumero($somaValorTotalNegociacao), $documento);
                                    $documento = str_replace("[SUMVRDESCNV]", Utilities::formatarNumero($somaValorDescontoNegociacao), $documento);
                                    $documento = str_replace("[SUMVRTOTCOMDESCNV]", Utilities::formatarNumero($somaValorTotalComDescontoNegociacao), $documento);
                                }
        
                                $negociacoesDeParcelamento = $this->getNegociacoesValoresDeParcelamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $cdidiom);
                                
                                foreach ($negociacoesDeParcelamento as $negociacaoParc) {
                                    //substitui os dados das negociações de valores da proposta no documento
                                    $documento = $this->substituirDadosNegociacoesValoresDeParcelamentoDocumento($negociacaoParc, $documento);    
                                }
                                
                                /* ========================================================================== */
        
                                $negociacoesParaDescontoPorPeriodo = $this->getNegociacoesValoresParaDescontoPorPeriodo($nrseqorcamento, $nrtipoorcamento, $nrorg, $cdidiom);
                                foreach ($negociacoesParaDescontoPorPeriodo as $negociacaoDescPer) {
                                    //substitui os dados das negociações de valores para desconto por período
                                    $documento = $this->substituirDadosNegociacoesValoresParaDescontoPorPeriodoDocumento($negociacaoDescPer, $documento);
                                    
                                    $descontosPorPeriodo = $this->getDescontosPorPeriodoNegociacaoValores($negociacaoDescPer["NRNEGOCIACAOPROPOSTA"], $nrorg);
                                    foreach ($descontosPorPeriodo as $descPer) {
                                        //substitui os dados dos descontos por período das negociações
                                        $documento = $this->substituirDadosDescontoPorPeriodoNegociacaoValores($negociacaoDescPer["NRNEGOCIACAOPROPOSTA"], $descPer, $documento);
                                    }
                                }
        
                                /* ========================================================================== */
                                
                                $valorTotalTodosAdicionais = Utilities::formatarNumero($valorTotalTodosAdicionais);
                                $valorTotalItens = Utilities::formatarNumero($valorTotalItens);
                                
                                $totorca = $this->getValorTotalProposta($nrseqorcamento, $nrtipoorcamento, $nrorg);
                                $totorca = Utilities::formatarNumero($totorca);
                                
                                $documento = str_replace("[TOTGERAD]", $valorTotalTodosAdicionais, $documento);
                                $documento = str_replace("[TOTITEM]", $valorTotalItens, $documento);
                                $documento = str_replace("[TOTORCA]", $totorca, $documento);
                                
                                $documento = str_replace("[LN]", "", $documento);
                                $documento = str_replace("[/LN]", "", $documento);
                                
                                //insere o documento na tabela temporária
                                //$this->inserirDocumentoTemporario($nrseqorcamento, $nrtipoorcamento, $documento, $nrorg);
        
                                $templateProcessor->setTempDocumentMainPart($documento);
                                
                                $nmtipoorcamento = $this->getNomeEscopo($nrtipoorcamento, $nrorg);
                               
                                $fileName = $microtime . "_" . str_replace(array("/", "\\"), "_", $proposta["NRORCAMENTO"]) . " - " . $nmtipoorcamento . ".docx";
                                $fileName = str_replace(array(",", "&"), "_", $fileName);
                                $nomeArquivoNovo = $this->tempDir . $ds . $fileName;
                               
                                $templateProcessor->saveAs($nomeArquivoNovo);
                                
                                if (!is_file($nomeArquivoNovo)) {
                                    throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoNovo."\" não foi encontrado.");
                                }
                                
                                if($idpermitedocx == 'D' && $this->validaExtensaoDocx($nrseqorcamento, $nrorg, $nrtipoorcamento) == "S"){
                                    //gerando docx
                                    $nomeArquivos[$doc["NRSEQORCADOC"]] = array(
                                        "CAMINHO"        => "/downloadArquivo?requestType=Row&row[FILENAME]=" . urlencode(base64_encode($fileName)) . "&row[FILETYPE]=application/vnd.openxmlformats-officedocument.wordprocessingml.document",
                                        "NMTIPOORCAMENTO"=> null,
                                        "DADOSDOCUMENTO" => array(
                                            "NMARQUIVO" => $fileName,
                                            "DSCAMINHOARQ" => $this->tempDir . $ds . $fileName,
                                            "TYPE"         => $doc["DSTIPO"]
                                        )
                                    );
                                } else {
                                    //gerando pdf
                                    $cmd = "export HOME=" .$this->tempDir. " && soffice --headless --convert-to pdf --outdir " .$this->tempDir. " \"" .$nomeArquivoNovo. "\" 2>&1";
                                     
                                    $resultadoCmd = shell_exec($cmd);
                                    
                                    if (strpos(strtolower($resultadoCmd), "error") !== false) {
                                        throw new \Exception("Falha ao gerar o arquivo PDF. Provavelmente o arquivo 'docx' foi gerado com erros, sendo impossível sua leitura para a convertê-lo em PDF.<br/>Erro: ".$resultadoCmd);
                                    } else {
                                        if (is_file($nomeArquivoNovo)) 
                                            unlink($nomeArquivoNovo);
                                        
                                        if (is_file($caminhoLogotipo)) 
                                            unlink($caminhoLogotipo);
                                        
                                        $fileName = $microtime . "_" . str_replace(array("/", "\\"), "_", $proposta["NRORCAMENTO"]) . " - " . $nmtipoorcamento . ".pdf";
                                        $fileName = str_replace(",", "_", $fileName);
                                        $nomeArquivoNovo = $this->tempDir . $ds . $fileName;
                                        
                                        if (is_file($nomeArquivoNovo)) {
                                            $nomeArquivos[$doc["NRSEQORCADOC"]] = array(
                                                "CAMINHO"        => "/downloadArquivo?requestType=Row&row[FILENAME]=" .  urlencode(base64_encode($fileName)) . "&row[FILETYPE]=application/pdf",
                                                "NMTIPOORCAMENTO"=> null,
                                                "DADOSDOCUMENTO" => array(
                                                    "NMARQUIVO" => $fileName,
                                                    "DSCAMINHOARQ" => $this->tempDir . $ds . $fileName,
                                                    "TYPE"         => $doc["DSTIPO"]
                                                )
                                            );
                                        } else {
                                            throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoNovo."\" não foi encontrado.<br/>Erro: ".$resultadoCmd);
                                        }
                                    }
                                }
                            }
                        } else {
                            throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoDocx."\" não foi encontrado.");
                        }
                        
                        if (is_file($nomeArquivoDocx)) 
                            unlink($nomeArquivoDocx);
                    }
                }
            }
            
            return $nomeArquivos;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Retorna se o documento sendo gerado pode ser gerado em docx
     **/
    public function validaExtensaoDocx($nrseqorcamento, $nrorg, $nrtipoorcamento){
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );

        $result = $this->documentoFactory->validaExtensaoDocx($params);
        return !empty($result) ? $result[0]["IDPERMITEDOCX"] : "N";
    }
    /**
     * Busca e substitui a data inicial do faturamento dos itens
    **/
    public function substituirDiaMinimoDePagamentoDocumento($nrseqorcamento, $nrorg, $templateProcessor){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        $dtInicial = $this->documentoFactory->getMenorDataInicioFatItens($params);
        $templateProcessor->setValue("[DTINIFATORC]", $dtInicial);
    }
    /**
     * Monta os mnemônicos com índice da parte de itens e seus adicionais no documento, 
     * para serem substituidos pelos valores depois.
     */
    private function gerarDocumentoParteDeItens($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[IND]" no documento inteiro
        $posicaoInd = strpos($documento, "[IND]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[IND]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
            $posicaoFechaLN = strpos($documento, "[/LN]", $posicaoInd);
            
            //posicao do primeiro "[LN]" que antecede o "[IND]"
            $posicaoPrimeiroLN = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoFechaLN) * -1));
            
            
            //verifica se existe mnemonico de item adicional no documento
            $existeMnemonicoAdicional = strpos($documento, "[INDAD]");
            
            //verifica se existe mnemonico de elementos no documento
            $existeMnemonicoElemento = strpos($documento, "[INDEL]");
            
            
            if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN !== false) {
                if ($posicaoPrimeiroTr < $posicaoPrimeiroLN) {
                    $result = "C";
                
                    if ($existeMnemonicoAdicional !== false || $existeMnemonicoElemento !== false) {
                        //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoInd
                        $posFin = strpos($documento, "</w:tbl>", $posicaoInd);
                        
                        //posicao do primeiro "<w:tbl" que antecede o "[IND]"
                        $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                        
                        $tagFecha = "</w:tbl>";
                    }
                    else {
                        $posIni = $posicaoPrimeiroTr;
                        $posFin = $posicaoFechaTr;
                        $tagFecha = "</w:tr>";
                    }
                }
                else {
                    $result = "L";
                
                    //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "[/LN]", $posicaoInd);
                    
                    //posicao do primeiro "[LN]" que antecede o "[IND]"
                    $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "[/LN]";
                }
            }
            else if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN === false) {
                $result = "C";
                
                if ($existeMnemonicoAdicional !== false || $existeMnemonicoElemento !== false) {
                    //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "</w:tbl>", $posicaoInd);
                    
                    //posicao do primeiro "<w:tbl" que antecede o "[IND]"
                    $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "</w:tbl>";
                }
                else {
                    $posIni = $posicaoPrimeiroTr;
                    $posFin = $posicaoFechaTr;
                    $tagFecha = "</w:tr>";
                }
            }
            else {
                $result = "L";
                
                //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "[/LN]", $posicaoInd);
                
                //posicao do primeiro "[LN]" que antecede o "[IND]"
                $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "[/LN]";
            }
            
            $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
            $clobLinha = substr($documento, $posIni, $posFin - $posIni);
            $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
            
            $linhasCodigoItens = $this->getCodigoItensProposta($nrseqorcamento, $nrtipoorcamento, $nrorg);
            $qtdItens = count($linhasCodigoItens);
            
            foreach ($linhasCodigoItens as $linhaCodigoItem) {
                //coloca o índice nos mnemônicos dos itens
                $clobLinhaSubstituida = $this->colocarIndiceMnemonicoItens($linhaCodigoItem["IND"], $clobLinha);
                
                //monta a parte dos elementos raiz no documento
                $clobLinhaSubstituida = $this->gerarDocumentoParteDeElementos($nrorg, $linhaCodigoItem["NRITEMORCAMENTO"], $clobLinhaSubstituida);
                
                //monta a parte de itens adicionais no documento
                $clobLinhaSubstituida = $this->gerarDocumentoParteDeAdicionais($nrseqorcamento, $nrtipoorcamento, $nrorg, $linhaCodigoItem["NRITEMORCAMENTO"], $clobLinhaSubstituida);
                
                if ($result == "C") {
                    if ($existeMnemonicoAdicional !== false || $existeMnemonicoElemento !== false) {
                        $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;//."<w:br/>";
                        //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1||'</w:tbl>';
                    }
                    else {
                        $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                        //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1||'</w:tr>';
                    }
                }
                else {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1;
                }
            }
            
            if ($qtdItens > 0) {
                //retira os mnemônicos dos itens
                $clobLinhaSubstituida = $this->retirarMnemonicosItens($clobLinha);
                
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
        }
        
        return $documento;
    }
    
    /**
     * Retorna o documento da proposta para o escopo informado.
     */
    private function getDocumentoProposta($nrseqorcamento, $nrtipoorcamento, $idSituacao, $cdidiom) {
        $params = array(
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "NRTIPOORCAMENTO"     => $nrtipoorcamento,
            "CDIDIOM"             => $cdidiom,
            "IDSITUACAOORCAMENTO" => $idSituacao
        );
        
        return $this->documentoFactory->getDocumentoProposta($params);
    }
    
    /**
     * Retorna os dados principais da proposta.
     */
    private function getDadosProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosProposta($params);
        return $dataset[0];
    }
    
    /**
     * Substitui os dados da proposta no documento.
     */
    private function substituirDadosPropostaDocumento($proposta, $templateProcessor) {
        $dataAtual = date_create();
        
        $templateProcessor->setValue("[ORCLIE]", $proposta["NMPARCNEGOCIO"]);
        $templateProcessor->setValue("[ORFILI]", $proposta["CDFILIAL"]);
        $templateProcessor->setValue("[ORCOD]", $proposta["NRORCAMENTO"]);
        $templateProcessor->setValue("[ORVERS]", $proposta["NRVERSAOORCAMENTO"]);
        $templateProcessor->setValue("[ORDTREF]", $proposta["DTREFORCAMENTO"]);
        $templateProcessor->setValue("[ORMODPAG]", $proposta["DSMODPGTO"]);
        $templateProcessor->setValue("[ORINTER]", $proposta["NMPARCINTERLOCUTOR"]);
        $templateProcessor->setValue("[ORREFE]", $proposta["DSREFERENCIAORCA"]);
        $templateProcessor->setValue("[ORDIASVD]", $proposta["QTDIASVALIDADE"]);
        $templateProcessor->setValue("[ORDTVAL]", $proposta["DTVALIDADE"]);
        $templateProcessor->setValue("[ORCAREN]", $proposta["NRDIASCARENCIA"]);
        $templateProcessor->setValue("[QTDIASCT]", $proposta["QTDIASCONTRATO"]);
        $templateProcessor->setValue("[OREMAIL]", $proposta["DSEMAIL"]);
        $templateProcessor->setValue("[ORTEL]", $proposta["NRTELEFONE"]);
        $templateProcessor->setValue("[NRISSUE]", $proposta["NRISSUE"]);
        $templateProcessor->setValue("[NRSOLISERV]", $proposta["NRSOLISERV"]);
        $templateProcessor->setValue("[NRISSUESOL]", $proposta["NRISSUESOL"]);
        $templateProcessor->setValue("[TPISSUESOL]", $proposta["TPISSUESOL"]);
        $templateProcessor->setValue("[OROBS]", $proposta["DSOBSERVACAOORCA"]);
        $templateProcessor->setValue("[ORVRORC]", Utilities::formatarNumero($proposta["VRORCAMENTO"]));
        $templateProcessor->setValue("[DTHOJE]", $dataAtual->format("d/m/Y"));
        $templateProcessor->setValue("[DIAAT]", $dataAtual->format("d"));
        $templateProcessor->setValue("[MESAT]", $dataAtual->format("m"));
        $templateProcessor->setValue("[ANOAT]", $dataAtual->format("Y"));
        
        $nomeMes = Utilities::getNomeMesPortugues($dataAtual);
        $templateProcessor->setValue("[NMMESAT]", $nomeMes);
    }
    
    /**
     * Busca o endereço de um determinado tipo do parceiro de negócio informado.
     */
    public function getEnderecoParceiro($nrparcnegocio, $nrorg, $cdtipoendereco) {
        $params = array(
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "NRORG"          => $nrorg,
            "CDTIPOENDERECO" => $cdtipoendereco
        );
        
        $dataset = $this->documentoFactory->getEnderecoParceiro($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Substitui os dados do cliente no documento.
     */
    private function substituirDadosClienteDocumento($proposta, $enderecoPrincipal, $templateProcessor) {
        $inscricao = Utilities::formatarCpfCnpj($proposta["NRINSCRICAOPARC"]);
        
        $templateProcessor->setValue("[ORPARC]", $inscricao);
        $templateProcessor->setValue("[ORNMPARC]", $proposta["NMPARCNEGOCIO"]);
        $templateProcessor->setValue("[ORIE]", $proposta["NRINSESTCLIE"]);
        
        if (!empty($enderecoPrincipal)) {
            $cep = Utilities::formatarCep($enderecoPrincipal["NRCEPENDERECO"]);

            $templateProcessor->setValue("[ORENDE]", $enderecoPrincipal["ENDERECO"]);
            $templateProcessor->setValue("[ORDSEND]", $enderecoPrincipal["DSENDERECO"]);
            $templateProcessor->setValue("[ORMUNIC]", $enderecoPrincipal["NMMUNICIPIO"]);
            $templateProcessor->setValue("[ORBAIRRO]", $enderecoPrincipal["NMBAIRROENDERECO"]);
            $templateProcessor->setValue("[ORESTADO]", $enderecoPrincipal["SGESTADO"]);
            $templateProcessor->setValue("[ORCEP]", $cep);
        }
        else {
            $templateProcessor->setValue("[ORENDE]", "");
            $templateProcessor->setValue("[ORDSEND]", "");
            $templateProcessor->setValue("[ORMUNIC]", "");
            $templateProcessor->setValue("[ORBAIRRO]", "");
            $templateProcessor->setValue("[ORESTADO]", "");
            $templateProcessor->setValue("[ORCEP]", "");
        }
    }
    
    /**
     * Exclui os documentos temporários a ser utilizado para gerar o relatório.
     */
    public function excluirDocumentosTemporarios($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
            
        $this->documentoFactory->excluirDocumentosTemporarios($params);
    }
    
    /**
     * Retorna o código dos itens da proposta.
     */
    private function getCodigoItensProposta($nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        return $this->documentoFactory->getCodigoItensProposta($params);
    }
    
    /**
     * Coloca o índice nos mnemônicos dos itens da proposta.
     */
    private function colocarIndiceMnemonicoItens($codigoItem, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[IND]", "[IND" . $codigoItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[COD]", "[COD" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QNT]", "[QNT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESC]", "[DESC" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTMED]", "[QTMED" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[MEDIDA]", "[MEDIDA" . $codigoItem . "]", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[TPSERV]", "[TPSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[QTTPSERV]", "[QTTPSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[MODPAG]", "[MODPAG" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRUNIT]", "[VRUNIT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRDESC]", "[VRDESC" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRACRE]", "[VRACRE" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRAJUSTE]", "[VRAJUSTE" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRSEMDESC]", "[VRSEMDESC" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRSEMAJUSTE]", "[VRSEMAJUSTE" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOT]", "[VRTOT" . $codigoItem . "]", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[TOTPER]", "[TOTPER" . $codigoItem . "]", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[TPHIERQ]", "[TPHIERQ" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OBSERV]", "[OBSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[SUMQTMEDEL]", "[SUMQTMEDEL" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTBLPRECO]", "[VRTBLPRECO" . $codigoItem . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Retira os mnemônicos dos itens da proposta.
     */
    private function retirarMnemonicosItens($clobLinha) {
        $clobLinhaSubstituida = str_replace("[IND]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[COD]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QNT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTMED]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[MEDIDA]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTTPSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[MODPAG]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRUNIT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRDESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRACRE]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRAJUSTE]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRSEMDESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRSEMAJUSTE]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TOTPER]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPHIERQ]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OBSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[SUMQTMEDEL]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTBLPRECO]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Monta a parte dos elementos raiz no documento.
     */
    private function gerarDocumentoParteDeElementos($nrorg, $nritemorcamento, $clobLinhaSubstituida) {
        $posIniElemento = -1;
        $posFinElemento = -1;
        
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        
        //posicao do primeiro "[INDEL]" dentro de $clobLinhaSubstituida
        $posicaoIndElemento = strpos($clobLinhaSubstituida, "[INDEL]");
        
        if ($posicaoIndElemento !== false) {
            //posicao do primeiro "</w:tr>" dentro de $clobLinhaSubstituida, contando a partir de $posicaoIndElemento
            $posicaoFechaTrElemento = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoIndElemento);
            
            //posicao do primeiro "<w:tr" que antecede o "[INDEL]" dentro de $clobLinhaSubstituida
            $posicaoPrimeiroTrElemento = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrElemento) * -1));
            
            if (!$posicaoPrimeiroTrElemento) {
                $posicaoPrimeiroTrElemento = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrElemento) * -1));
            }
            
            if ($posicaoPrimeiroTrElemento !== false) {
                $posIniElemento = $posicaoPrimeiroTrElemento;
                $posFinElemento = $posicaoFechaTrElemento;
                $tagFecha = "</w:tr>";
                
                $clobInicialElemento = substr($clobLinhaSubstituida, 0, $posFinElemento + strlen($tagFecha));
                $clobLinhaElemento = substr($clobLinhaSubstituida, $posIniElemento, $posFinElemento - $posIniElemento);
                $clobFinalElemento = substr($clobLinhaSubstituida, $posFinElemento, $tamanhoClobLinhaSubstituida);
                
                $linhasCodigoElementosFilhos = $this->getCodigoElementosRaizProposta($nritemorcamento, $nrorg);
                
                $qtdElementos = count($linhasCodigoElementosFilhos);
                foreach ($linhasCodigoElementosFilhos as $linhaCodigoElementoFilho) {
                    //coloca o índice nos mnemônicos dos elementos
                    $clobLinhaElementoSubstituida = $this->colocarIndiceMnemonicosElementos($nritemorcamento, $linhaCodigoElementoFilho["IND"], $clobLinhaElemento);
                    
                    $clobInicialElemento = $clobInicialElemento . $clobLinhaElementoSubstituida . $tagFecha;
                }
                
                if ($qtdElementos > 0) {
                    //retira os mnemônicos dos elementos
                    $clobLinhaElementoSubstituida = $this->retirarMnemonicosElementos($clobLinhaElemento);
                    
                    $clobLinhaSubstituida = $clobInicialElemento . substr($clobFinalElemento, strlen($tagFecha));
                }
                
                $clobLinhaSubstituida = str_replace($clobLinhaElemento.$tagFecha, "", $clobLinhaSubstituida);
            }
        }
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Retorna o código dos elementos raiz da proposta.
     */
    private function getCodigoElementosRaizProposta($nritemorcamento, $nrorg) {
        $params = array(
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getCodigoElementosRaizProposta($params);
    }
    
    /**
     * Coloca o índice nos mnemônicos dos elementos da proposta.
     */
    private function colocarIndiceMnemonicosElementos($codigoItem, $indiceElemento, $clobLinhaElemento) {
        $clobLinhaElementoSubstituida = str_replace("[INDEL]", "[INDEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElemento);
        $clobLinhaElementoSubstituida = str_replace("[DESCELCONTRATUAL]", "[DESCELCONTRATUAL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[DESCEL]", "[DESCEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRUNITEL]", "[VRUNITEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[MEDIDAVAR]", "[MEDIDAVAR" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[QTMEDVAR]", "[QTMEDVAR" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[MEDIDAEL]", "[MEDIDAEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[QTMEDEL]", "[QTMEDEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[QTFRANQEL]", "[QTFRANQEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRPERCDESCEL]", "[VRPERCDESCEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRDESCEL]", "[VRDESCEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRTOTELSEMDESC]", "[VRTOTELSEMDESC" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRTOTEL]", "[VRTOTEL" . $codigoItem . $indiceElemento . "]", $clobLinhaElementoSubstituida);
        
        return $clobLinhaElementoSubstituida;
    }
    
    /**
     * Retira os mnemônicos dos elementos da proposta.
     */
    private function retirarMnemonicosElementos($clobLinhaElemento) {
        $clobLinhaElementoSubstituida = str_replace("[INDEL]", "", $clobLinhaElemento);
        $clobLinhaElementoSubstituida = str_replace("[DESCELCONTRATUAL]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[DESCEL]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[MEDIDAVAR]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRUNITEL]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[QTMEDVAR]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[QTMEDEL]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[QTFRANQEL]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRPERCDESCEL]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRDESCEL]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRTOTELSEMDESC]", "", $clobLinhaElementoSubstituida);
        $clobLinhaElementoSubstituida = str_replace("[VRTOTEL]", "", $clobLinhaElementoSubstituida);
        
        return $clobLinhaElementoSubstituida;
    }
    
    /**
     * Busca os dados do comprador da proposta.
     */
    private function getDadosCompradorProposta($nrparcnegocio, $nrorg) {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosCompradorProposta($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Substitui os dados do comprador da proposta no documento.
     */
    public function substituirDadosCompradorPropostaDocumento($comprador, $templateProcessor) {
        $inscricao = Utilities::formatarCpfCnpj($comprador["NRINSCRICAOPARC"]);
        
        $templateProcessor->setValue("[CPARC]", $inscricao);
        $templateProcessor->setValue("[CNMPARC]", $comprador["NMPRINCIPALPARC"]);
        $templateProcessor->setValue("[CIE]", $comprador["CDINSCESTA"]);
    }
    
    /**
     * Substitui o endereço principal do comprador da proposta no documento.
     */
    public function substituirDadosEnderecoPrincipalCompradorPropostaDocumento($endereco, $templateProcessor) {

        if (!empty($endereco)) {
            $cep = Utilities::formatarCep($endereco["NRCEPENDERECO"]);
       
            $templateProcessor->setValue("[CENDE]", $endereco["ENDERECO"]);
            $templateProcessor->setValue("[CDSEND]", $endereco["DSENDERECO"]);
            $templateProcessor->setValue("[CMUNIC]", $endereco["NMMUNICIPIO"]);
            $templateProcessor->setValue("[CBAIRRO]", $endereco["NMBAIRROENDERECO"]);
            $templateProcessor->setValue("[CESTADO]", $endereco["SGESTADO"]);
            $templateProcessor->setValue("[CNMESTADO]", $endereco["NMESTADO"]);
            $templateProcessor->setValue("[CCEP]", $cep);
        }
        else {
            $templateProcessor->setValue("[CENDE]", "");
            $templateProcessor->setValue("[CDSEND]", "");
            $templateProcessor->setValue("[CMUNIC]", "");
            $templateProcessor->setValue("[CBAIRRO]", "");
            $templateProcessor->setValue("[CESTADO]", "");
            $templateProcessor->setValue("[CNMESTADO]", "");
            $templateProcessor->setValue("[CCEP]", "");
        }
    }
    
    /**
     * Substitui o endereço de entrega do comprador da proposta no documento.
     */
    public function substituirDadosEnderecoEntregaCompradorPropostaDocumento($endereco, $templateProcessor) {
       
        if (!empty($endereco)) {
            $cep = Utilities::formatarCep($endereco["NRCEPENDERECO"]);

            $templateProcessor->setValue("[CENDE2]", $endereco["ENDERECO"]);
            $templateProcessor->setValue("[CDSEND2]", $endereco["DSENDERECO"]);
            $templateProcessor->setValue("[CMUNIC2]", $endereco["NMMUNICIPIO"]);
            $templateProcessor->setValue("[CBAIRRO2]", $endereco["NMBAIRROENDERECO"]);
            $templateProcessor->setValue("[CESTADO2]", $endereco["SGESTADO"]);
            $templateProcessor->setValue("[CNMESTADO2]", $endereco["NMESTADO"]);
            $templateProcessor->setValue("[CCEP2]", $cep);
        }
        else {
            $templateProcessor->setValue("[CENDE2]", "");
            $templateProcessor->setValue("[CDSEND2]", "");
            $templateProcessor->setValue("[CMUNIC2]", "");
            $templateProcessor->setValue("[CBAIRRO2]", "");
            $templateProcessor->setValue("[CESTADO2]", "");
            $templateProcessor->setValue("[CNMESTADO2]", "");
            $templateProcessor->setValue("[CCEP2]", "");
        }
    }
    
    /**
     * Retorna o endereço de entrega da proposta.
     */
    private function getEnderecoDeEntregaProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->documentoFactory->getEnderecoDeEntregaProposta($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Substitui os dados do endereço de entrega no documento. Caso não haja endereço de entrega, é utilizado o endereço principal.
     */
    private function substituirEnderecoDeEntregaDocumento($enderecoEntrega, $enderecoPrincipal, $templateProcessor) {
        $enderecoParaSubstituir = null;
        
        if (!empty($enderecoEntrega)) {
            $enderecoParaSubstituir = $enderecoEntrega;
        }
        else if (!empty($enderecoPrincipal)) {
            $enderecoParaSubstituir = $enderecoPrincipal;
        }
        
        if (!empty($enderecoParaSubstituir)) {
            $cep = Utilities::formatarCep($enderecoParaSubstituir["NRCEPENDERECO"]);
    
            $templateProcessor->setValue("[ORENDE2]", $enderecoParaSubstituir["ENDERECO"]);
            $templateProcessor->setValue("[ORDSEND2]", $enderecoParaSubstituir["DSENDERECO"]);
            $templateProcessor->setValue("[ORMUNIC2]", $enderecoParaSubstituir["NMMUNICIPIO"]);
            $templateProcessor->setValue("[ORBAIRRO2]", $enderecoParaSubstituir["NMBAIRROENDERECO"]);
            $templateProcessor->setValue("[ORESTADO2]", $enderecoParaSubstituir["SGESTADO"]);
            $templateProcessor->setValue("[ORCEP2]", $cep);
        }
        else {
            $templateProcessor->setValue("[ORENDE2]", "");
            $templateProcessor->setValue("[ORDSEND2]", "");
            $templateProcessor->setValue("[ORMUNIC2]", "");
            $templateProcessor->setValue("[ORBAIRRO2]", "");
            $templateProcessor->setValue("[ORESTADO2]", "");
            $templateProcessor->setValue("[ORCEP2]", "");
        }
    }
    
    /**
     * Retorna o vendedor principal da proposta.
     */
    private function getVendedorPrincipalProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->documentoFactory->getVendedorPrincipalProposta($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Substitui os dados do vendedor principal da proposta no documento.
     */
    private function substituirDadosVendedorDocumento($vendedorPrincipal, $templateProcessor) {
        if (!empty($vendedorPrincipal)) {
            $templateProcessor->setValue("[NMVENDPRI]", $vendedorPrincipal["NMPARCVENDEDOR"]);
            $templateProcessor->setValue("[NMGERENTPRI]", $vendedorPrincipal["NMPARCGERENTE"]);
            $templateProcessor->setValue("[PARTVENDPRI]", $vendedorPrincipal["VRPEPARTICIPAVENDA"]);
        }
        else {
            $templateProcessor->setValue("[NMVENDPRI]", "");
            $templateProcessor->setValue("[NMGERENTPRI]", "");
            $templateProcessor->setValue("[PARTVENDPRI]", "");
        }
    }
    
    /**
     * Monta os mnemônicos da parte de previsão de pagamento no documento
     */
    private function gerarDocumentoParteDePrevisaoDePagamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento) {
        $documento = $this->gerarDocumentoParteDePrevisaoDePagamentoNaoPeriodico($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento);
        $documento = $this->gerarDocumentoParteDePrevisaoDePagamentoPeriodicoOuApuracao($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento);
        
        return $documento;
    }
    
    /**
     * Monta a parte de previsão de pagamento dos itens com Tipo de Faturamento "Não Periódico".
     */
    private function gerarDocumentoParteDePrevisaoDePagamentoNaoPeriodico($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDPAGNP]" no documento inteiro
        $posicaoIndPagNP = strpos($documento, "[INDPAGNP]");
        
        if ($posicaoIndPagNP !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoIndPagNP
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoIndPagNP);
            
            //posicao do primeiro "<w:tr>" que antecede o "[INDPAGNP]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoIndPagNP
                $posFin = strpos($documento, "</w:tbl>", $posicaoIndPagNP);
                
                //posicao do primeiro "<w:tbl>" que antecede o "[INDPAGNP]"
                $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "</w:tbl>";
            
            
                $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($documento, $posIni, $posFin - $posIni);
                $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
                
                $itensPrevisaoPagamento = $this->getItensPrevisaoPagamentoNaoPeriodico($nrseqorcamento, $nrtipoorcamento, $nrorg);
                $qtdItens = count($itensPrevisaoPagamento);
                
                foreach ($itensPrevisaoPagamento as $itemPrevisaoPgto) {
                    if ($itemPrevisaoPgto["VRTOTITEMCOMDESC"]) {
                        //coloca o índice nos mnemônicos dos itens da previsão de pagamento do tipo de faturamento "Não Periódico"
                        $clobLinhaSubstituida = $this->colocarIndiceMnemonicoItensPrevisaoDePagamentoNaoPeriodico($itemPrevisaoPgto["IND"], $clobLinha);
                        
                        //monta a parte de itens da previsão de pagamento do tipo de faturamento "Não Periódico"
                        $clobLinhaSubstituida = $this->gerarDocumentoParteDeItensPrevisaoDePagamentoNaoPeriodico($nrorg, $itemPrevisaoPgto, $clobLinhaSubstituida);
                        
                        $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;//."<w:br/>";
                    }
                }
                
                if ($qtdItens > 0) {
                    //retira os mnemônicos dos itens
                    $clobLinhaSubstituida = $this->retirarMnemonicosItensPrevisaoDePagamentoNaoPeriodico($clobLinha);
                    
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        
        return $documento;
    }
    
    /**
     * Retorna os itens da proposta com tipo de faturamento igual a "Não Periódico"
     */
    private function getItensPrevisaoPagamentoNaoPeriodico($nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getItensPrevisaoPagamentoNaoPeriodico($params);
    }
    
    private function colocarIndiceMnemonicoItensPrevisaoDePagamentoNaoPeriodico($codigoItem, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDPAGNP]", "[INDPAGNP" . $codigoItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCPAGNP]", "[DESCPAGNP" . $codigoItem . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Monta a parte de itens da previsão de pagamento do tipo de faturamento "Não Periódico"
     */
    private function gerarDocumentoParteDeItensPrevisaoDePagamentoNaoPeriodico($nrorg, $itemPrevisaoPgto, $clobLinhaSubstituida) {
        $posIniParcela = -1;
        $posFinParcela = -1;
        
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        
        //posicao do primeiro "[INDPARCPAGNP]" dentro de $clobLinhaSubstituida
        $posicaoIndParcela = strpos($clobLinhaSubstituida, "[INDPARCPAGNP]");
        
        if ($posicaoIndParcela !== false) {
            //posicao do primeiro "</w:tr>" dentro de $clobLinhaSubstituida, contando a partir de $posicaoIndParcela
            $posicaoFechaTrParcela = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoIndParcela);
            
            //posicao do primeiro "<w:tr>" que antecede o "[INDPARCPAGNP]" dentro de $clobLinhaSubstituida
            $posicaoPrimeiroTrParcela = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrParcela) * -1));
            
            if (!$posicaoPrimeiroTrParcela) {
                $posicaoPrimeiroTrParcela = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrParcela) * -1));
            }
            
            if ($posicaoPrimeiroTrParcela !== false) {
                $posIniParcela = $posicaoPrimeiroTrParcela;
                $posFinParcela = $posicaoFechaTrParcela;
                $tagFecha = "</w:tr>";
            
                $clobInicialParcela = substr($clobLinhaSubstituida, 0, $posFinParcela + strlen($tagFecha));
                $clobLinhaParcela = substr($clobLinhaSubstituida, $posIniParcela, $posFinParcela - $posIniParcela);
                $clobFinalParcela = substr($clobLinhaSubstituida, $posFinParcela, $tamanhoClobLinhaSubstituida);
                
                $parcelasNegociacao = $this->getParcelasNegociacao($itemPrevisaoPgto["NRNEGOCIACAOPROPOSTA"], $itemPrevisaoPgto["NRITNEGOCIACAOPROPOSTA"], $nrorg);
                if (count($parcelasNegociacao) > 0) {
                    
                    $dataAceite = $itemPrevisaoPgto["DTACEITEORCA"];
                    if ($dataAceite) {
                        $dataAceite = DateUtil::getDataDeString($dataAceite);
                    }
                    else {
                        $dataAceite = DateUtil::getDataAtual(true);
                    }
                    
                    $informacoesParcelas = array();
                    $sequencial = 1;

                    foreach ($parcelasNegociacao as $parcela) {
                        $percDescontoNegociacao = $parcela["VRPERCDESCNEG"];
                        $percDescontoParcela    = $parcela["VRPERCDESCPARC"];
                        $valorItem              = $parcela["VRITEM"];
                        $qtdDiasFaturamento     = $parcela["QTDIASFATURAMENTO"];
                        $diaVencimento          = $parcela["NRDIAVENCIMENTO"];

                        $dataReferencia = DateUtil::adicionaIntervalo($dataAceite, $qtdDiasFaturamento);

                        if ($percDescontoNegociacao > 0) {
                            $valorItem = $valorItem - ($valorItem * $percDescontoNegociacao / 100);
                        }
                        
                        if ($percDescontoParcela > 0) {
                            $valorItem = $valorItem - ($valorItem * $percDescontoParcela / 100);
                        }

                        if ($diaVencimento) {
                            $diaVencimento = str_pad($diaVencimento, 2, "0", STR_PAD_LEFT);
                            $dataVencimento = DateUtil::getDataDeString($diaVencimento . "/" . $dataReferencia->format("m/Y"));
                            if (DateUtil::getQtdeDiasEntre($dataVencimento, $dataReferencia) > 0) {
                                $dataVencimento = DateUtil::adicionaIntervalo(DateUtil::getUltimoDiaDoMes($dataVencimento), 1);
                                $dataVencimento = DateUtil::getDataDeString($diaVencimento . "/" . $dataVencimento->format("m/Y"));
                            }
                        }
                        else {
                            $idtpcalcvenc  = $parcela["IDTPCALCVENC"];
                            $nrdiacalcvenc = str_pad($parcela["NRDIACALCVENC"], 2, "0", STR_PAD_LEFT);
                            
                            if ($idtpcalcvenc == "1") { //dia fixo
                                $idmescalcvenc = $parcela["IDMESCALCVENC"];

                                $dataVencimento = DateUtil::getDataDeString($nrdiacalcvenc . "/" . $dataReferencia->format("m/Y"));
                                if ($idmescalcvenc == "1") {
                                    $dataVencimento = DateUtil::subtraiIntervalo($dataVencimento, 1, DateUtil::MES);
                                }
                                else if ($idmescalcvenc == "3") {
                                    $dataVencimento = DateUtil::adicionaIntervalo($dataVencimento, 1, DateUtil::MES);
                                }
                            }
                            else { //qtd dias apos emissao
                                $dataVencimento = DateUtil::adicionaIntervalo($dataReferencia, $nrdiacalcvenc);
                            }
                        }

                        $informacoesParcelas[] = array(
                            "NRPARCELA"    => $sequencial,
                            "DTVENCIMENTO" => $dataVencimento->format("d/m/Y"),
                            "VRVENCIMENTO" => $valorItem
                        );

                        $sequencial++;
                    }
                }
                else {
                    throw new \Exception("Operação bloqueada. <br/>Não existem parcelas definidas para o item \"" . $itemPrevisaoPgto["NMITEMORCAMENTO"] . "\".");
                }
                
                $qtdInfoParcelas = count($informacoesParcelas);
                    
                foreach ($informacoesParcelas as $infoParcela) {
                    //coloca o índice nos mnemônicos das parcelas de previsão de pagamento
                    $clobLinhaParcelaSubstituida = $this->colocarIndiceMnemonicoParcelasPrevisaoPagamento($itemPrevisaoPgto["NRITEMORCAMENTO"], $infoParcela["NRPARCELA"], $clobLinhaParcela);
                    
                    $clobInicialParcela = $clobInicialParcela . $clobLinhaParcelaSubstituida . $tagFecha;
                }
                
                if ($qtdInfoParcelas > 0) {
                    //retira os mnemônicos das parcelas de previsão de pagamento
                    $clobLinhaParcelaSubstituida = $this->retirarMnemonicosParcelasPrevisaoPagamento($clobLinhaParcela);
                }
                
                $clobLinhaSubstituida = $clobInicialParcela . substr($clobFinalParcela, strlen($tagFecha));
                $clobLinhaSubstituida = str_replace($clobLinhaParcela.$tagFecha, "", $clobLinhaSubstituida);
                
                foreach ($informacoesParcelas as $infoParcela) {
                    //substitui os dados das parcelas de previsão de pagamento no documento
                    $clobLinhaSubstituida = $this->substituirDadosParcelasPrevisaoPagamento($itemPrevisaoPgto["NRITEMORCAMENTO"], $infoParcela, $clobLinhaSubstituida);
                }
            }
        }
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Retira os mnemônicos dos itens de previsão de pagamento do tipo "Não Periódico" do documento
     */
    private function retirarMnemonicosItensPrevisaoDePagamentoNaoPeriodico($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDPAGNP]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCPAGNP]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Retorna o prazo em dias e o percentual de cada parcela da forma de pagamento variável
     */
    private function getParcelasFormaPagamentoVariavel($cdmodpgto, $nrorg) {
        $params = array(
            "CDMODPGTO" => $cdmodpgto,
            "NRORG"     => $nrorg
        );
        
        return $this->documentoFactory->getParcelasFormaPagamentoVariavel($params);
    }
    
    /**
     * Chama a procedure FCT_CALCULA_DIAPGTO
     */
    private function calcularDiaPgto($dataVencimento, $idtpperiodo, $cdformpgto) {
        $params = array(
            "DTVENCIMENTO" => $dataVencimento,
            "IDTPPERIODO"  => $idtpperiodo,
            "CDFORMPGTO"   => $cdformpgto
        );
        
        $dataset = $this->documentoFactory->calcularDiaPgto($params);
        return $dataset[0]["FCT_CALCULA_DIAPGTO"];
    }
    
    /**
     * Coloca o índice nos mnemônicos das parcelas de previsão de pagamento da proposta
     */
    private function colocarIndiceMnemonicoParcelasPrevisaoPagamento($codigoItem, $indiceParcela, $clobLinhaParcela) {
        $clobLinhaParcelaSubstituida = str_replace("[INDPARCPAGNP]", "[INDPARCPAGNP" . $codigoItem . $indiceParcela . "]", $clobLinhaParcela);
        $clobLinhaParcelaSubstituida = str_replace("[DTVENCPARCPAGNP]", "[DTVENCPARCPAGNP" . $codigoItem . $indiceParcela . "]", $clobLinhaParcelaSubstituida);
        $clobLinhaParcelaSubstituida = str_replace("[VRPARCPAGNP]", "[VRPARCPAGNP" . $codigoItem . $indiceParcela . "]", $clobLinhaParcelaSubstituida);
        
        return $clobLinhaParcelaSubstituida;
    }
    
    /**
     * Retira os mnemônicos das parcelas de previsão de pagamento
     */
    private function retirarMnemonicosParcelasPrevisaoPagamento($clobLinhaParcela) {
        $clobLinhaParcelaSubstituida = str_replace("[INDPARCPAGNP]", "", $clobLinhaParcela);
        $clobLinhaParcelaSubstituida = str_replace("[DTVENCPARCPAGNP]", "", $clobLinhaParcelaSubstituida);
        $clobLinhaParcelaSubstituida = str_replace("[VRPARCPAGNP]", "", $clobLinhaParcelaSubstituida);
        
        return $clobLinhaParcelaSubstituida;
    }
    
    /**
     * Substitui os dados das parcelas de previsão de pagamento no documento.
     */
    private function substituirDadosParcelasPrevisaoPagamento($nritemorcamento, $infoParcela, $clobLinhaSubstituida) {
        $clobLinhaSubstituida = str_replace("[INDPARCPAGNP" . $nritemorcamento . $infoParcela["NRPARCELA"] . "]", str_pad($infoParcela["NRPARCELA"], 2, "0", STR_PAD_LEFT), $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DTVENCPARCPAGNP" . $nritemorcamento . $infoParcela["NRPARCELA"] . "]", $infoParcela["DTVENCIMENTO"], $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPARCPAGNP" . $nritemorcamento . $infoParcela["NRPARCELA"] . "]", Utilities::formatarNumero($infoParcela["VRVENCIMENTO"]), $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Monta a parte de previsão de pagamento dos itens com Tipo de Faturamento "Periódico" ou "Apuração".
     */
    private function gerarDocumentoParteDePrevisaoDePagamentoPeriodicoOuApuracao($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDPAG]" no documento inteiro
        $posicaoIndPag = strpos($documento, "[INDPAG]");
        
        if ($posicaoIndPag !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoIndPag
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoIndPag);
            
            //posicao do primeiro "<w:tr" que antecede o "[INDPAG]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoIndPag
            $posicaoFechaLN = strpos($documento, "[/LN]", $posicaoIndPag);
            
            //posicao do primeiro "[LN]" que antecede o "[INDPAG]"
            $posicaoPrimeiroLN = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoFechaLN) * -1));
            
            
            if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN !== false) {
                
                if ($posicaoPrimeiroTr < $posicaoPrimeiroLN) {
                    $result = "C";
                
                    $posIni = $posicaoPrimeiroTr;
                    $posFin = $posicaoFechaTr;
                    $tagFecha = "</w:tr>";
                }
                else {
                    $result = "L";
                
                    $posIni = $posicaoPrimeiroLN;
                    $posFin = $posicaoFechaLN;
                    $tagFecha = "[/LN]";
                }
            }
            else if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN === false) {
                $result = "C";
                
                $posIni = $posicaoPrimeiroTr;
                $posFin = $posicaoFechaTr;
                $tagFecha = "</w:tr>";
            }
            else {
                $result = "L";
                
                $posIni = $posicaoPrimeiroLN;
                $posFin = $posicaoFechaLN;
                $tagFecha = "[/LN]";
            }
            
            $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
            $clobLinha = substr($documento, $posIni, $posFin - $posIni);
            $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
            
            //$linhasCodigoItensPrevisaoPagamento = $this->getCodigoItensPrevisaoPagamentoPeriodicoOuApuracao($nrseqorcamento, $nrtipoorcamento, $nrorg);
            $idfaturacontr = "'A', 'X'";
            $linhasCodigoItensPrevisaoPagamento = $this->documentoFactory->getItensPropostaComParametrosFaturamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $idfaturacontr);
            
            $qtdItens = count($linhasCodigoItensPrevisaoPagamento);
            
            foreach ($linhasCodigoItensPrevisaoPagamento as $linhaCodigoItem) {
                //if ($linhaCodigoItem["VRTOTITEMCOMDESC"] > 0) {
                    //coloca o índice nos mnemônicos dos itens da previsão de pagamento do tipo de faturamento "Periódico" ou "Apuração"
                    $clobLinhaSubstituida = $this->colocarIndiceMnemonicoItensPrevisaoDePagamentoPeriodicoOuApuracao($linhaCodigoItem["IND"], $clobLinha);
                    
                    if ($result == "C") {
                        $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                    }
                    else {
                        $clobInicial = $clobInicial . $clobLinhaSubstituida;
                    }
                //}
            }
            
            if ($qtdItens > 0) {
                //retira os mnemônicos dos itens
                $clobLinhaSubstituida = $this->retirarMnemonicosItensPrevisaoDePagamentoPeriodicoOuApuracao($clobLinha);
                
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
        }
        
        return $documento;
    }
    
    /**
     * Retorna o código dos itens da proposta com tipo de faturamento igual a "Periódico" ou "Apuração"
     */
    private function getCodigoItensPrevisaoPagamentoPeriodicoOuApuracao($nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getCodigoItensPrevisaoPagamentoPeriodicoOuApuracao($params);
    }
    
    /**
     * Coloca o índice nos mnemônicos dos itens da previsão de pagamento do tipo de faturamento "Periódico" ou "Apuração"
     */
    private function colocarIndiceMnemonicoItensPrevisaoDePagamentoPeriodicoOuApuracao($codigoItem, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDPAG]", "[INDPAG" . $codigoItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCPAG]", "[DESCPAG" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DIAVENCPAG]", "[DIAVENCPAG" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DTINICIOPAG]", "[DTINICIOPAG" . $codigoItem . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Retira os mnemônicos dos itens de previsão de pagamento do tipo "Periódico" ou "Apuração" do documento
     */
    private function retirarMnemonicosItensPrevisaoDePagamentoPeriodicoOuApuracao($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDPAG]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCPAG]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DIAVENCPAG]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DTINICIOPAG]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Retorna os itens da proposta.
     */
    private function getItensProposta($nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getItensProposta($params);
    }
    
    /**
     * Substitui os dados dos itens da proposta no documento.
     */
    private function substituirDadosItensDocumento($item, $documento) {
        //$documento = str_replace("[IND".$item["IND"]."]", $item["IND"], $documento);
        $documento = str_replace("[IND".$item["IND"]."]", "", $documento);
        
        $documento = str_replace("[COD".$item["IND"]."]", $item["CODITEM"], $documento);
        $documento = str_replace("[QNT".$item["IND"]."]", Utilities::formatarNumero($item["QTITEMORCAMENTO"],0), $documento);
        $documento = str_replace("[DESC".$item["IND"]."]", str_replace("&", "&amp;", $item["NMITEMORCAMENTO"]), $documento);
        $documento = str_replace("[QTMED".$item["IND"]."]", Utilities::formatarNumero($item["QTMEDIDAORCA"],0), $documento);
        $documento = str_replace("[MEDIDA".$item["IND"]."]", str_replace("&", "&amp;", $item["DSTPMEDIDASERV"]), $documento);
        
        $documento = str_replace("[VRUNIT".$item["IND"]."]", Utilities::formatarNumero($item["VRUNITITEMORCA"]), $documento);
        
        /*$vrDesconto = $item["VRITEMSEMDESC"] - $item["VRTOTALITEMORCA"];
        if ($vrDesconto < 0) $vrDesconto = 0;
        $documento = str_replace("[VRDESC".$item["IND"]."]", Utilities::formatarNumero($vrDesconto), $documento);*/
        $documento = str_replace("[VRDESC".$item["IND"]."]", Utilities::formatarNumero($item["VRDESCITEM"]), $documento);
        
        /*$vrAcrescimo = $item["VRTOTALITEMORCA"] - $item["VRITEMSEMDESC"];
        if ($vrAcrescimo < 0) $vrAcrescimo = 0;
        $documento = str_replace("[VRACRE".$item["IND"]."]", Utilities::formatarNumero($vrAcrescimo), $documento);*/
        $documento = str_replace("[VRACRE".$item["IND"]."]", Utilities::formatarNumero($item["VRACRESCIMO"]), $documento);
        
        //$vrAjuste = $item["VRTOTITEMCOMDESC"] - $item["VRTOTITEMSEMDESC"];
        $vrAjuste = $item["VRACRESCIMO"] - $item["VRDESCITEM"];
        $documento = str_replace("[VRAJUSTE".$item["IND"]."]", Utilities::formatarNumero($vrAjuste), $documento);
        
        $documento = str_replace("[VRSEMDESC".$item["IND"]."]", Utilities::formatarNumero($item["VRTOTITEMSEMDESC"]), $documento);
        $documento = str_replace("[VRSEMAJUSTE".$item["IND"]."]", Utilities::formatarNumero($item["VRTOTITEMSEMDESC"]), $documento);
        $documento = str_replace("[VRTOT".$item["IND"]."]", Utilities::formatarNumero($item["VRTOTITEMCOMDESC"]), $documento);
        $documento = str_replace("[SUMQTMEDEL".$item["IND"]."]", str_replace(".", ",", $item["QTMEDSERVORCA"]), $documento);
        
        $valorTabelaPreco = ($item["QTMEDSERVORCA"] == 0) ? 0 : (($item["VRTOTITEMSEMDESC"] - $item["VRACRESCIMO"]) / $item["QTMEDSERVORCA"]);
        $documento = str_replace("[VRTBLPRECO".$item["IND"]."]", Utilities::formatarNumero($valorTabelaPreco), $documento);
        
        return $documento;
    }
    
    /**
     * Retorna os elementos da proposta.
     */
    private function getElementosProposta($cdservorca, $nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "CDSERVORCA"      => $cdservorca,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg,
            "NRSEQORCAMENTO"  => $nrseqorcamento
        );
        
        return $this->documentoFactory->getElementosProposta($params);
    }

    /**
     * Substitui os dados dos elementos da proposta no documento.
     */
    private function substituirDadosElementosDocumento($nritemorcamento, $elemento, $documento) {
        /*$valorUnitarioElemento = $elemento["VRUNITSERVORCA"];
        if (!is_null($valorUnitarioElemento)) {
            $valorUnitarioElemento = Utilities::formatarNumero($valorUnitarioElemento);
        }*/
        
        //$documento = str_replace("[INDEL" . $nritemorcamento . $elemento["IND"] . "]", $elemento["IND"], $documento);
        $documento = str_replace("[INDEL" . $nritemorcamento . $elemento["IND"] . "]", "", $documento);
        
        $documento = str_replace("[DESCELCONTRATUAL" . $nritemorcamento . $elemento["IND"] . "]", str_replace("&", "&amp;", $elemento["NMCONTRATUALITEMORCA"]), $documento);
        $documento = str_replace("[DESCEL" . $nritemorcamento . $elemento["IND"] . "]", str_replace("&", "&amp;", $elemento["NMELEMENTORAIZ"]), $documento);
        $documento = str_replace("[VRUNITEL" . $nritemorcamento . $elemento["IND"] . "]", Utilities::formatarNumero($elemento["VRUNITITEMORCA"]), $documento);
        $documento = str_replace("[MEDIDAVAR" . $nritemorcamento . $elemento["IND"] . "]", str_replace("&", "&amp;", $elemento["DSTPMEDIDAVARIAVEL"]), $documento);
        $documento = str_replace("[QTMEDVAR" . $nritemorcamento . $elemento["IND"] . "]", Utilities::formatarNumero($elemento["QTMEDIDAVARIAVEL"], 0), $documento);
        $documento = str_replace("[MEDIDAEL" . $nritemorcamento . $elemento["IND"] . "]", str_replace("&", "&amp;", $elemento["DSTPMEDIDASERV"]), $documento);
        $documento = str_replace("[QTMEDEL" . $nritemorcamento . $elemento["IND"] . "]", Utilities::formatarNumero($elemento["QTMEDIDAORCA"], 0), $documento);
        $documento = str_replace("[QTFRANQEL" . $nritemorcamento . $elemento["IND"] . "]", Utilities::formatarNumero($elemento["QTFRANQUIA"], 0), $documento);
        $documento = str_replace("[VRPERCDESCEL" . $nritemorcamento . $elemento["IND"] . "]", Utilities::formatarNumero($elemento["VRPERCDESCITEM"]), $documento);
        $documento = str_replace("[VRDESCEL" . $nritemorcamento . $elemento["IND"] . "]", Utilities::formatarNumero($elemento["VRDESCITEM"]), $documento);
        $documento = str_replace("[VRTOTELSEMDESC" . $nritemorcamento . $elemento["IND"] . "]", Utilities::formatarNumero($elemento["VRTOTSERVORCASEMDESC"]), $documento);
        $documento = str_replace("[VRTOTEL" . $nritemorcamento . $elemento["IND"] . "]", Utilities::formatarNumero($elemento["VRTOTSERVORCA"]), $documento);
        
        return $documento;
    }
    
    /**
     * Retorna os itens adicionais da proposta.
     */
    private function getItensAdicionaisProposta($nrseqorcamento, $nrtipoorcamento, $nritemorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getItensAdicionaisProposta($params);
    }
    
    /**
     * Substitui os dados dos itens adicionais da proposta no documento.
     */
    private function substituirDadosItensAdicionaisDocumento($item, $itemAdicional, $documento) {
        //$documento = str_replace("[INDAD" . $item["NRITEMORCAMENTO"] . $itemAdicional["IND"] . "]", $itemAdicional["IND"], $documento);
        $documento = str_replace("[INDAD" . $item["NRITEMORCAMENTO"] . $itemAdicional["IND"] . "]", "", $documento);
        
        $documento = str_replace("[DESCAD" . $item["NRITEMORCAMENTO"] . $itemAdicional["IND"] . "]", str_replace("&", "&amp;", $itemAdicional["NMITEMORCAMENTO"]), $documento);
        $documento = str_replace("[QNTAD" . $item["NRITEMORCAMENTO"] . $itemAdicional["IND"] . "]", Utilities::formatarNumero($itemAdicional["QTITEMORCAMENTO"],0), $documento);
        $documento = str_replace("[VRUNITAD" . $item["NRITEMORCAMENTO"] . $itemAdicional["IND"] . "]", Utilities::formatarNumero($itemAdicional["VRUNITITEMORCA"]), $documento);
        $documento = str_replace("[MEDIDAAD" . $item["NRITEMORCAMENTO"] . $itemAdicional["IND"] . "]", str_replace("&", "&amp;", $itemAdicional["DSTPMEDIDASERV"]), $documento);
        $documento = str_replace("[QTMEDAD" . $item["NRITEMORCAMENTO"] . $itemAdicional["IND"] . "]", Utilities::formatarNumero($itemAdicional["QTMEDIDAORCA"],0), $documento);
        $documento = str_replace("[VRTOTAD" . $item["NRITEMORCAMENTO"] . $itemAdicional["IND"] . "]", Utilities::formatarNumero($itemAdicional["VRTOTALITEMORCA"]), $documento);
        
        return $documento;
    }
    
    /**
     * Substitui os dados dos itens da previsão de pagamento do tipo "Periódico" ou "Apuração"
     */
    private function substituirDadosItensPrevisaoDePagamentoPeriodicoOuApuracao($item, $dataIniFat, $diaVencimento, $documento) {
        $documento = str_replace("[INDPAG".$item["IND"]."]", "", $documento);
        $documento = str_replace("[DESCPAG".$item["IND"]."]", str_replace("&", "&amp;", $item["NMITEMORCAMENTO"]), $documento);
        $documento = str_replace("[DIAVENCPAG".$item["IND"]."]", $diaVencimento, $documento);
        $documento = str_replace("[DTINICIOPAG".$item["IND"]."]", $dataIniFat, $documento);
        
        return $documento;
    }
    
    /**
     * Substitui os dados dos itens da previsão de pagamento do tipo "Não Periódico"
     */
    private function substituirDadosItensPrevisaoDePagamentoNaoPeriodico($item, $txDocument) {
        $txDocument = str_replace("[INDPAGNP".$item["IND"]."]", "", $txDocument);
        $txDocument = str_replace("[DESCPAGNP".$item["IND"]."]", str_replace("&", "&amp;", $item["NMITEMORCAMENTO"]), $txDocument);
        
        return $txDocument;
    }
    
    /**
     * Retorna o valor total da proposta.
     */
    public function getValorTotalProposta($nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->documentoFactory->getValorTotalProposta($params);
        return (float)$dataset[0]["TOTAL"];
    }
    
    /**
     * Insere o documento na tabela temporária.
     */
    private function inserirDocumentoTemporario($nrseqorcamento, $nrtipoorcamento, $txDocument, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "TXDOCUMENTO"     => $txDocument,
            "NRORG"           => $nrorg
        );
        
        $this->documentoFactory->inserirDocumentoTemporario($params);
    }
    
    /**
     * Salva o relatório do documento da proposta na pasta "/tmp" e retorna seu caminho completo.
     */
    public function salvarRelatorio($nomeArquivo, $reportUrl) {
        try {
            $report = file_get_contents($reportUrl);
        }
        catch (\Exception $e) {
            return false;
        }

        $ds = DIRECTORY_SEPARATOR;

        if (is_dir($this->tempDir)) {
            $reportPath = $this->tempDir . $ds . "documento_proposta" . $nomeArquivo . ".pdf";
        }
        else {
            $reportPath = __DIR__ . $ds . "documento_proposta" . $nomeArquivo . ".pdf";
        }
        file_put_contents($reportPath, $report);
        return $reportPath;
    }
    
    public function getCaminhoLogotipoParceiro($nrparcnegocio, $nrorg) {
        $url = $this->getUrlFileServerLogotipoParceiro($nrparcnegocio, $nrorg);
        if (!empty($url)) {
            $logo = $this->smdocumentos->getDocumentoFileServer($this->fileServerBaseUrl . $url, $nrorg);
            if ($logo !== false) {
                $posPonto = strrpos($url, ".");
                $extensaoArquivo = substr($url, $posPonto);
                
                $caminhoLogotipo = $this->tempDir . DIRECTORY_SEPARATOR ."logo_cliente_" . $nrparcnegocio . $extensaoArquivo;
                file_put_contents($caminhoLogotipo, $logo);
                
                return $caminhoLogotipo;
            }
            else {
                return null;
            }
        }
        else {
            return null;
        }
    }
    
    public function getUrlFileServerLogotipoParceiro($nrparcnegocio, $nrorg) {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getUrlFileServerLogotipoParceiro($params);
        return $dataset[0]["DSDIRLOGOMARCA"];
    }
    
    public function substituirLogotipoDocumento($caminhoLogotipo, $templateProcessor, $tamanhoLogo = null) {
        $imagem = array(
            "src" => $caminhoLogotipo,
            "swh" => "400"
        );

        if ($tamanhoLogo)
            $imagem["swh"] = $tamanhoLogo;
        
        $templateProcessor->setImg("[LOGOTIPO]", $imagem);
    }
    
    public function getNomeEscopo($nrtipoorcamento, $nrorg) {
        if ($nrtipoorcamento != 0) {
            $params = array(
                "NRTIPOORCAMENTO" => $nrtipoorcamento,
                "NRORG"           => $nrorg
            );
            
            $dataset = $this->documentoFactory->getNomeEscopo($params);
            return $dataset[0]["NMTIPOORCAMENTO"];
        }
        else {
            return "0";
        }
    }
    
    private function gerarDocumentoParteDeAdicionais($nrseqorcamento, $nrtipoorcamento, $nrorg, $nritemorcamento, $clobLinhaSubstituida) {
        $posIniAd = -1;
        $posFinAd = -1;
        
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        
        //posicao do primeiro "[INDAD]" dentro de $clobLinhaSubstituida
        $posicaoIndAd = strpos($clobLinhaSubstituida, "[INDAD]");
        
        if ($posicaoIndAd !== false) {
            //posicao do primeiro "</w:tr>" dentro de $clobLinhaSubstituida, contando a partir de $posicaoIndAd
            $posicaoFechaTrAd = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoIndAd);
            
            //posicao do primeiro "<w:tr" que antecede o "[INDAD]" dentro de $clobLinhaSubstituida
            $posicaoPrimeiroTrAd = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrAd) * -1));
            
            if (!$posicaoPrimeiroTrAd) {
                $posicaoPrimeiroTrAd = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrAd) * -1));
            }
            
            if ($posicaoPrimeiroTrAd !== false) {
                $posIniAd = $posicaoPrimeiroTrAd;
                $posFinAd = $posicaoFechaTrAd;
                $tagFecha = "</w:tr>";
                
                $clobInicialAd = substr($clobLinhaSubstituida, 0, $posFinAd + strlen($tagFecha));
                $clobLinhaAd = substr($clobLinhaSubstituida, $posIniAd, $posFinAd - $posIniAd);
                $clobFinalAd = substr($clobLinhaSubstituida, $posFinAd, $tamanhoClobLinhaSubstituida);
                
                $linhasCodigoItensAdicionais = $this->getCodigoItensAdicionaisProposta($nrseqorcamento, $nrtipoorcamento, $nritemorcamento, $nrorg);
                
                $qtdAdicionais = count($linhasCodigoItensAdicionais);
                
                foreach ($linhasCodigoItensAdicionais as $linhaCodigoItemAdicional) {
                    
                    //coloca o índice nos mnemônicos dos itens adicionais
                    $clobLinhaAdSubstituida = $this->colocarIndiceMnemonicosItensAdicionais($nritemorcamento, $linhaCodigoItemAdicional["IND"], $clobLinhaAd);
                    
                    $clobInicialAd = $clobInicialAd . $clobLinhaAdSubstituida . $tagFecha;
                }
                
                if ($qtdAdicionais > 0) {
                    //retira os mnemônicos dos itens adicionais
                    $clobLinhaAdSubstituida = $this->retirarMnemonicosItensAdicionais($clobLinhaAd);
                    
                    $clobLinhaSubstituida = $clobInicialAd . substr($clobFinalAd, strlen($tagFecha));
                }
                
                $clobLinhaSubstituida = str_replace($clobLinhaAd.$tagFecha, "", $clobLinhaSubstituida);
            }
        }
        
        return $clobLinhaSubstituida;
    }
    
    private function getCodigoItensAdicionaisProposta($nrseqorcamento, $nrtipoorcamento, $nritemorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getCodigoItensAdicionaisProposta($params);
    }
    
    private function colocarIndiceMnemonicosItensAdicionais($nritemorcamento, $indice, $clobLinhaAd) {
        $clobLinhaAdSubstituida = str_replace("[INDAD]", "[INDAD" . $nritemorcamento . $indice . "]", $clobLinhaAd);
        $clobLinhaAdSubstituida = str_replace("[DESCAD]", "[DESCAD" . $nritemorcamento . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[QNTAD]", "[QNTAD" . $nritemorcamento . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[VRUNITAD]", "[VRUNITAD" . $nritemorcamento . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[MEDIDAAD]", "[MEDIDAAD" . $nritemorcamento . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[QTMEDAD]", "[QTMEDAD" . $nritemorcamento . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[VRTOTAD]", "[VRTOTAD" . $nritemorcamento . $indice . "]", $clobLinhaAdSubstituida);
        
        return $clobLinhaAdSubstituida;
    }
    
    private function retirarMnemonicosItensAdicionais($clobLinhaAd) {
        $clobLinhaAdSubstituida = str_replace("[INDAD]", "", $clobLinhaAd);
        $clobLinhaAdSubstituida = str_replace("[DESCAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[QNTAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[VRUNITAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[MEDIDAAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[QTMEDAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[VRTOTAD]", "", $clobLinhaAdSubstituida);
        
        return $clobLinhaAdSubstituida;
    }
    
    /**
     * Monta os mnemônicos da parte de formação de preço de serviço no documento.
     */
    private function gerarDocumentoParteDeFormacoesDePreco($nrseqorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDFP]" no documento inteiro
        $posicaoInd = strpos($documento, "[INDFP]");
        
        if ($posicaoInd !== false) {
            $quebralinha = strpos($documento, "[QBRPAGFP]");
            
            if ($quebralinha !== false){
                
                $pageBreak = '<w:p>
                                    <w:r>
                                      <w:br w:type="page" />
                                    </w:r>
                                  </w:p>';
                
                $documento = str_replace("[QBRPAGFP]", "", $documento);
            }
            else{
                $pageBreak = '<w:br/>';
            }
            
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr>" que antecede o "[INDFP]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "</w:tbl>", $posicaoInd);
                
                //posicao do primeiro "<w:tbl>" que antecede o "[INDFP]"
                $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "</w:tbl>";
            
            
                $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($documento, $posIni, $posFin - $posIni);
                $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
                
                
                $linhasCodigoFormacoesPreco = $this->getCodigoFormacoesPrecoProposta($nrseqorcamento, $nrorg);
                $qtdFormacoes = count($linhasCodigoFormacoesPreco);
                
                foreach ($linhasCodigoFormacoesPreco as $linhaCodigoFormacao) {
                    //coloca o índice nos mnemônicos dos itens da previsão de pagamento do tipo de faturamento "Periódico" ou "Apuração"
                    $clobLinhaSubstituida = $this->colocarIndiceMnemonicoFormacoesPreco($linhaCodigoFormacao["IND"], $clobLinha);
                    
                    //monta a parte dos itens da formação de preço
                    //$clobLinhaSubstituida = $this->gerarDocumentoParteDeItensFormacaoPreco($nrorg, $linhaCodigoFormacao["NRFORMPRECOSERVICO"], $clobLinhaSubstituida);
                    
                    $clobLinhaSubstituida = $this->gerarDocumentoParteDeItensFormacaoPrecoPorIdentificacao($nrorg, $linhaCodigoFormacao, $clobLinhaSubstituida);
                    
                    
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha . $pageBreak;
                }
                
                if ($qtdFormacoes > 0) {
                    //retira os mnemônicos das formações de preço
                    $clobLinhaSubstituida = $this->retirarMnemonicosFormacoesPreco($clobLinha);
                    
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        
        return $documento;
    }
    
    public function getCodigoFormacoesPrecoProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        return $this->documentoFactory->getCodigoFormacoesPrecoProposta($params);
    }
    
    public function colocarIndiceMnemonicoFormacoesPreco($indiceFormacao, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDFP]", "[INDFP" . $indiceFormacao . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCFP]", "[DESCFP" . $indiceFormacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESCSINDFP]", "[DESCSINDFP" . $indiceFormacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESCSRVFP]", "[DESCSRVFP" . $indiceFormacao . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    private function retirarMnemonicosFormacoesPreco($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDFP]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCFP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESCSINDFP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESCSRVFP]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    public function gerarDocumentoParteDeItensFormacaoPreco($nrorg, $nrformprecoservico, $clobLinhaSubstituida) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        
        //posicao do primeiro "[INDITFP]" dentro de $clobLinhaSubstituida
        $posicaoInd = strpos($clobLinhaSubstituida, "[INDITFP]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" dentro de $clobLinhaSubstituida, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[INDITFP]" dentro de $clobLinhaSubstituida
            $posicaoPrimeiroTr = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                $posIni = $posicaoPrimeiroTr;
                $posFin = $posicaoFechaTr;
                $tagFecha = "</w:tr>";
                
                $clobInicial = substr($clobLinhaSubstituida, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($clobLinhaSubstituida, $posIni, $posFin - $posIni);
                $clobFinal = substr($clobLinhaSubstituida, $posFin, $tamanhoClobLinhaSubstituida);
                
                $linhasCodigoItensFormacaoPreco = $this->getCodigoItensFormacaoPrecoProposta($nrformprecoservico, $nrorg);
                
                $qtdItensFormacao = count($linhasCodigoItensFormacaoPreco);
                
                foreach ($linhasCodigoItensFormacaoPreco as $linhaCodigoItensFormacao) {
                    
                    //coloca o índice nos mnemônicos dos elementos
                    $clobLinhaItemFormacaoSubstituida = $this->colocarIndiceMnemonicosItensFormacaoPreco($nrformprecoservico, $linhaCodigoItensFormacao["IND"], $clobLinha);
                    
                    $clobInicial = $clobInicial . $clobLinhaItemFormacaoSubstituida . $tagFecha;
                }
                
                if ($qtdItensFormacao > 0) {
                    //retira os mnemônicos dos elementos
                    $clobLinhaItemFormacaoSubstituida = $this->retirarMnemonicosItensFormacaoPreco($clobLinha);
                    
                    $clobLinhaSubstituida = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $clobLinhaSubstituida = str_replace($clobLinha.$tagFecha, "", $clobLinhaSubstituida);
            }
        }
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Verifica na tabela de alterações da formação de preço se há a formação procurada 
    **/
    public function getItensFormacaoAuxiliarProposta($nrorg, $nrformprecoservico, $nritemorcamento, $nrseqorcamento){
        $params = array(
            "NRSEQORCAMENTO"     => $nrseqorcamento,
            "NRITEMORCAMENTO"    => $nritemorcamento,
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRORG"              => $nrorg
        );
        
        return $this->documentoFactory->getItensFormacaoAuxiliarProposta($params);
    }

    
    public function gerarDocumentoParteDeItensFormacaoPrecoPorIdentificacao($nrorg, $formprecoservico, $clobLinhaSubstituida) {
        $itensFormacaoPreco = array();
        $itensFormacaoPreco = $this->getItensFormacaoAuxiliarProposta($nrorg, $formprecoservico["NRFORMPRECOSERVICO"], $formprecoservico["NRITEMORCAMENTO"], $formprecoservico["NRSEQORCAMENTO"]);
        
        if(count($itensFormacaoPreco) <= 0){
            $itensFormacaoPreco = $this->getItensFormacaoPrecoProposta($formprecoservico["NRFORMPRECOSERVICO"], $nrorg);
        }
        
        $itensFormacaoPreco = $this->formatarItensFormacaoPrecoPorIdentificador($itensFormacaoPreco);
        $identificadores = array_keys($itensFormacaoPreco);
        foreach ($identificadores as $identificador) {
            
            $campos = array(
                "_DESC"     => array(
                    "NOME"            => "NMELEMENTO",
                    "FORMATAR_NUMERO" => false
                ),
                "_VRPERC"   => array(
                    "NOME"            => "VRPERCCALCULO",
                    "FORMATAR_NUMERO" => true
                ),
                "_VRTOT"    => array(
                    "NOME"            => "VRTOTAL",
                    "FORMATAR_NUMERO" => true
                ),
                "_QTMEDIDA" => array(
                    "NOME"            => "QTMEDIDACALC",
                    "FORMATAR_NUMERO" => true
                ),
                "_VRUNIT"   => array(
                    "NOME"            => "VRUNITARIO",
                    "FORMATAR_NUMERO" => true
                ),
                "_MEDIDA"    => array(
                    "NOME"            => "DSTPMEDIDASERV",
                    "FORMATAR_NUMERO" => false
                )
            );
            
            foreach ($campos as $chave => $campo) {
                $mnemonico = "[ID_" . $identificador . $chave . "]";
                $posicao = strpos($clobLinhaSubstituida, $mnemonico);
                
                if ($posicao !== false) {
                    $valor = substr($clobLinhaSubstituida, $posicao, strlen($mnemonico));
                    $substituicao = $itensFormacaoPreco[$identificador][$campo["NOME"]];
                    
                    if ($campo["FORMATAR_NUMERO"] && !empty($substituicao)) {
                        $substituicao = Utilities::formatarNumero($substituicao);
                    }
                    
                    $clobLinhaSubstituida = str_replace($valor, $substituicao, $clobLinhaSubstituida);
                }
            }
        }
        
        $clobLinhaSubstituida = $this->retirarMnemonicosItensFormacaoPrecoPorIdentificacao($clobLinhaSubstituida);
        return $clobLinhaSubstituida;
    }
    
    public function getCodigoItensFormacaoPrecoProposta($nrformprecoservico, $nrorg) {
        $params = array(
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRORG"              => $nrorg
        );
        
        return $this->documentoFactory->getCodigoItensFormacaoPrecoProposta($params);
    }
    
    public function colocarIndiceMnemonicosItensFormacaoPreco($nrformprecoservico, $indiceItem, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDITFP]", "[INDITFP" . $nrformprecoservico . $indiceItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCITFP]", "[DESCITFP" . $nrformprecoservico . $indiceItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPERCITFP]", "[VRPERCITFP" . $nrformprecoservico . $indiceItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTITFP]", "[VRTOTITFP" . $nrformprecoservico . $indiceItem . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    public function retirarMnemonicosItensFormacaoPreco($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDITFP]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCITFP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPERCITFP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTITFP]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    public function getFormacoesPrecoProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        return $this->documentoFactory->getFormacoesPrecoProposta($params);
    }
    
    public function substituirDadosFormacoesPrecoDocumento($formacao, $documento) {
        //$documento = str_replace("[INDFP".$formacao["IND"]."]", $formacao["IND"], $documento);
        $documento = str_replace("[INDFP".$formacao["IND"]."]", "", $documento);
        $documento = str_replace("[DESCFP".$formacao["IND"]."]", str_replace("&", "&amp;", $formacao["NMFORMPRECOSERVICO"]), $documento);
        $documento = str_replace("[DESCSINDFP".$formacao["IND"]."]", str_replace("&", "&amp;", $formacao["DSSINDICATO"]), $documento);
        $documento = str_replace("[DESCSRVFP".$formacao["IND"]."]", str_replace("&", "&amp;", $formacao["NMSERVORCA"]), $documento);
        
        return $documento;
    }
    
    public function getItensFormacaoPrecoProposta($nrformprecoservico, $nrorg) {
        $params = array(
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRORG"              => $nrorg
        );
        
        return $this->documentoFactory->getItensFormacaoPrecoProposta($params);
    }
    
    public function substituirDadosItensFormacaoPrecoDocumento($nrformprecoservico, $itemFormacao, $documento) {
        $level = $itemFormacao["LEVEL"];
        $identacao = str_repeat(" ", ($level-1) * 4);
        
        
        $percentual = $itemFormacao["VRPERCCALCULO"];
        if (empty($percentual) || $percentual == 0) {
            $percentual = "";
        }
        else {
            $percentual = Utilities::formatarNumero($percentual);
        }
        
        $qtFilhos = $itemFormacao["QTFILHOS"];
        if ((int)$qtFilhos != 0) {
            $documento = $this->colocarEmNegrito($documento, "[DESCITFP" . $nrformprecoservico . $itemFormacao["IND"] . "]");
            $documento = $this->colocarEmNegrito($documento, "[VRPERCITFP" . $nrformprecoservico . $itemFormacao["IND"] . "]");
            $documento = $this->colocarEmNegrito($documento, "[VRTOTITFP" . $nrformprecoservico . $itemFormacao["IND"] . "]");
        }
        
        //$documento = str_replace("[INDITFP" . $nrformprecoservico . $itemFormacao["IND"] . "]", $itemFormacao["IND"], $documento);
        $documento = str_replace("[INDITFP" . $nrformprecoservico . $itemFormacao["IND"] . "]", "", $documento);
        $documento = str_replace("[DESCITFP" . $nrformprecoservico . $itemFormacao["IND"] . "]", $identacao . str_replace("&", "&amp;", $itemFormacao["NMELEMENTO"]), $documento);
        $documento = str_replace("[VRPERCITFP" . $nrformprecoservico . $itemFormacao["IND"] . "]", $percentual, $documento);
        $documento = str_replace("[VRTOTITFP" . $nrformprecoservico . $itemFormacao["IND"] . "]", Utilities::formatarNumero($itemFormacao["VRTOTAL"]), $documento);
        
        return $documento;
    }
    
    public function colocarEmNegrito($documento, $chave) {
        $tamanhoDocumento = strlen($documento);
        
        $posicaoChave = strpos($documento, $chave);
        if ($posicaoChave !== false) {
            //posicao do primeiro "</w:t>" no documento, contando a partir de $posicaoChave
            $posFin = strpos($documento, "</w:t>", $posicaoChave);
            
            //posicao do primeiro "<w:t>" que antecede $posicaoChave
            $posIni = strrpos($documento, "<w:t>", (($tamanhoDocumento - $posFin) * -1));
            
            $teste = substr($documento, $posIni, $posFin - $posIni);
            $testeNovo = "<w:rPr><w:b/></w:rPr>".$teste;
            $documento = str_replace($teste, $testeNovo, $documento);
        }
        
        return $documento;
    }
    
    /* Preserva os espaços*/
    public function preservarEspacos($documento, $chave){
        $tamanhoDocumento = strlen($documento);
        
        $posicaoChave = strpos($documento, $chave);
        if($posicaoChave !== false){
            $posFin = strpos($documento, "</w:t>", $posicaoChave);
            
            $posIni = strrpos($documento, "<w:t>", (($tamanhoDocumento - $posFin) * -1));
            $atual = substr($documento, $posIni, $posFin - $posIni);
            $novo = str_replace("<w:t>","<w:t xml:space='preserve'>", $atual);
            
            $documento = str_replace($atual, $novo, $documento);
        }
        
        return $documento;
    }
    
    public function formatarItensFormacaoPrecoPorIdentificador($itensFormacaoPreco) {
        $novosItens = array();
        foreach ($itensFormacaoPreco as $item) {
            $novosItens[$item["IDITEMCALCULO"]] = $item;
        }
        
        return $novosItens;
    }
    
    public function retirarMnemonicosItensFormacaoPrecoPorIdentificacao($clobLinhaSubstituida) {
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        $posicaoId = strpos($clobLinhaSubstituida, "[ID_");
        
        while ($posicaoId !== false) {
            
            $posicaoFechaTr = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoId);
            $posicaoPrimeiroTr = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTr) * -1));
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTr) * -1));
            }
            
            $posIni = $posicaoPrimeiroTr;
            $posFin = $posicaoFechaTr;
            $tagFecha = "</w:tr>";

            $clobLinha = substr($clobLinhaSubstituida, $posIni, $posFin - $posIni);
            $clobLinha .= $tagFecha;
            
            $clobLinhaSubstituida = str_replace($clobLinha, "", $clobLinhaSubstituida);
            $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
            $posicaoId = strpos($clobLinhaSubstituida, "[ID_");
        }
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Gera e substitui as tabelas de elementos da composição de serviço adquirida
    **/
    private function gerarDocumentoParteElementoPaiComposicaoServico($nrseqorcamento, $nrorg, $documento){
        $posicaoInicio = -1;
        $posicaoFinal  = -1;
        $aux           = array();
        
        $tamanhoDocumento = strlen($documento);
        
        $posicaoIdPai = strpos($documento, "[INDELPAICS]");
        if($posicaoIdPai !== false) {
            
            $quebralinha = strpos($documento, "[QBRPAGCS]");
            if ($quebralinha !== false){
                $pageBreak = '<w:p>
                                    <w:r>
                                      <w:br w:type="page" />
                                    </w:r>
                                  </w:p>';
                
                $documento = str_replace("[QBRPAGCS]", "", $documento);
            }
            else{
                $pageBreak = '';
            }
            
            $posicaoFinalTR   = strpos($documento, "</w:tr>", $posicaoIdPai);
            
            $posicaoInicialTR = strrpos($documento, "<w:tr", (($tamanhoDocumento - $posicaoFinalTR) * -1));    
            
            if(!$posicaoInicialTR){
                $posicaoInicialTR = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFinalTR) * -1));
            }
            
            $posicaoFinal  = strpos($documento, "</w:tbl>", $posicaoIdPai);
            $posicaoInicio = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posicaoFinal) * -1));
            $tagFecha = "</w:tbl>";
            
            $clobInicial = substr($documento, 0, $posicaoFinal + strlen($tagFecha));
            $clobLinha   = substr($documento, $posicaoInicio, $posicaoFinal - $posicaoInicio);
            $clobFinal   = substr($documento, $posicaoFinal, $tamanhoDocumento);

            $semElementos = (strpos($clobLinha, "[SEM_ELEMENTO]") > 0);
            
            $elementosPaiCompServ = $this->getElementoPaiComposicaoServProposta($nrseqorcamento, $nrorg);
            $qtdDados = count($elementosPaiCompServ);
            
            $verificaUltimo = 0;
            $tagparagrafo = "<w:p/>";
        
            foreach ($elementosPaiCompServ as $elementoPai) {
                if($verificaUltimo == ($qtdDados - 1)) {
                    $tagparagrafo = "";
                }
                
                $clobSubstituida = $this->colocarIndiceMnemonicosElementoPaiCompServic($elementoPai["INDELPAICS"], $clobLinha);
                $clobSubstituida = $this->substituirDadosElementosPaiCompServDocumento($elementoPai, $clobSubstituida);
                
                //monta a parte de composição de serviço
                $clobSubstituida = $this->gerarDocumentoParteElementoComposicaoServico($nrseqorcamento, $nrorg, $elementoPai["NRCOMPSERVICO"], $clobSubstituida, $semElementos);
                
                if (empty($pageBreak)) {
                    $clobInicial = $clobInicial . $clobSubstituida . $tagFecha . $tagparagrafo;
                }
                else {
                    $clobInicial = $clobInicial . $clobSubstituida . $tagFecha . $pageBreak;
                }
                
                $verificaUltimo++;
            }
            
            if($qtdDados > 0){
               $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha . $tagFecha, "", $documento);
        }

        return $documento;
    }
    
    /**
     * Gera os mnemonicos dos elementos pai adquiridos pelo cliente.
    **/
    private function colocarIndiceMnemonicosElementoPaiCompServic($idelementopai, $linhaDocumento){
        $linhaSubstituida = str_replace("[INDELPAICS]",  "[INDELPAICS"  . $idelementopai . "]", $linhaDocumento);
        $linhaSubstituida = str_replace("[DESCELPAICS]", "[DESCELPAICS" . $idelementopai . "]", $linhaSubstituida);
        $linhaSubstituida = str_replace("[SEM_ELEMENTO]", "", $linhaSubstituida);
        //$linhaSubstituida = str_replace("[VRTOTELPAICS]", "[VRTOTELPAICS" . $idelementopai . "]", $linhaSubstituida);
        //$linhaSubstituida = str_replace("[VRTOTSAJELPAICS]", "[VRTOTSAJELPAICS" . $idelementopai . "]", $linhaSubstituida);
        return $linhaSubstituida;
    }
    
    /**
     * Substitui os mnemonicos dos elementos pai adquiridos.
    **/
    private function substituirDadosElementosPaiCompServDocumento($elementoPai, $linhaDocumento){
        $linhaDocumento = str_replace("[INDELPAICS"        . $elementoPai["INDELPAICS"] . "]", "", $linhaDocumento);
        $linhaDocumento = str_replace("[DESCELPAICS"       . $elementoPai["INDELPAICS"] . "]", str_replace("&", "&amp;", $elementoPai["NMCOMPSERVICO"]), $linhaDocumento);
        //$linhaSubstituida = str_replace("[VRTOTELPAICS"    . $elementoPai["INDELPAICS"] . "]", $elementoPai["VRTOTSERVORCA"], $linhaSubstituida);
        //$linhaSubstituida = str_replace("[VRTOTSAJELPAICS" . $elementoPai["INDELPAICS"] . "]", $elementoPai["VRTOTSAJUSTE"], $linhaSubstituida);
        return $linhaDocumento;
    }

    /*
    * Gera e substitui com os dados as linhas de elementos nas tabela de composição de serviço 
    */
    private function gerarDocumentoParteElementoComposicaoServico($nrseqorcamento, $nrorg, $elementoPai, $documento, $semElementos = false) {
        $posIniCompServico = -1;
        $posFinCompServico = -1;
        $aux = array();

        $tamanhoDocumento = strlen($documento);
        
        $posicaoIdCompServico = strpos($documento, "[INDELCS]");

        if($posicaoIdCompServico !== false) {
            $posicaoFechaTrCompServico = strpos($documento, "</w:tr>", $posicaoIdCompServico);
            
            $posicaoAbreTrCompServico = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTrCompServico) * -1));
            
            if(!$posicaoAbreTrCompServico){
                $posicaoAbreTrCompServico = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTrCompServico) * -1));
            }
            
            $posFinCompServico = $posicaoFechaTrCompServico;
            $posIniCompServico = $posicaoAbreTrCompServico;
            $tagFecha = "</w:tr>";
            
            $clobInicialCompServico = substr($documento, 0, $posFinCompServico + strlen($tagFecha));
            $clobLinhaCompServico   = substr($documento, $posIniCompServico, $posFinCompServico - $posIniCompServico);
            $clobFinalCompServico   = substr($documento, $posFinCompServico, $tamanhoDocumento);
            
            $linhasDadosCompServico = $this->getElementoComposicaoServProposta($nrseqorcamento, $nrorg, $elementoPai, $semElementos);
            $qtdDadosCompServico = count($linhasDadosCompServico);
            
            foreach($linhasDadosCompServico as $linhaDadoCompServico){
                
                $clobLinhaDadosCompServSubstituida = $this->colocarIndiceMnemonicosElementosCompServico($linhaDadoCompServico["INDELCS"], $clobLinhaCompServico) ;
                $clobLinhaDadosCompServSubstituida = $this->substituirDadosComposicaoServico($linhaDadoCompServico, $clobLinhaDadosCompServSubstituida);
                $clobInicialCompServico = $clobInicialCompServico . $clobLinhaDadosCompServSubstituida . $tagFecha;
            }  
            
            if($qtdDadosCompServico > 0) {
               $documento = $clobInicialCompServico . substr($clobFinalCompServico, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinhaCompServico . $tagFecha, "", $documento);
        }
        return $documento;
        
    }
    
    /**
     * Retorna o elemento pai das composicoes de servico da proposta.
     */
    private function getElementoPaiComposicaoServProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );

        return $this->documentoFactory->getElementoPaiComposicaoServProposta($params);
    }
    
    /**
     * Retorna os dados das composicoes de servico da proposta.
     */
    private function getElementoComposicaoServProposta($nrseqorcamento, $nrorg, $nrcompservico, $semElementos) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg,
            "NRCOMPSERVICO"  => $nrcompservico,
            "TIPO"           => $semElementos ? "C" : null
        );

        return $this->documentoFactory->getElementoComposicaoServProposta($params);
    }


    /**
    * Coloca o índice nos mnemônicos dos elementos da proposta.
    */
    private function colocarIndiceMnemonicosElementosCompServico($codigoItem, $clobLinhaCompServico) {
        $clobLinhaCompServicoSubstituida = str_replace("[INDELCS]", "[INDELCS"   . $codigoItem . "]", $clobLinhaCompServico);
        $clobLinhaCompServicoSubstituida = str_replace("[DESCELCS]", "[DESCELCS" . $codigoItem . "]", $clobLinhaCompServicoSubstituida);
        $clobLinhaCompServicoSubstituida = str_replace("[TPCS]", "[TPCS" . $codigoItem . "]", $clobLinhaCompServicoSubstituida);
        $clobLinhaCompServicoSubstituida = str_replace("[IDOBRICS]", "[IDOBRICS" . $codigoItem . "]", $clobLinhaCompServicoSubstituida);
        $clobLinhaCompServicoSubstituida = str_replace("[VRTOTELCS]", "[VRTOTELCS" . $codigoItem . "]", $clobLinhaCompServicoSubstituida);
        $clobLinhaCompServicoSubstituida = str_replace("[VRTOTSAJELCS]", "[VRTOTSAJELCS" . $codigoItem . "]", $clobLinhaCompServicoSubstituida);
        return $clobLinhaCompServicoSubstituida;
    }
    
    /*
    * Substitui os mnêmonicos de composição de serviço pelos seus respectivos dados
    */
    private function substituirDadosComposicaoServico($dadoCompServ, $documento){
        $tipo         = $dadoCompServ["TIPO"];
        $obrigatorio  = $dadoCompServ["IDSERVOBRIGCOMP"];
        $vrTotElCS    = $dadoCompServ["VRTOTSERVORCA"];
        $vrTotSAjElCS = $dadoCompServ["VRTOTSAJUSTE"];
        
        if($tipo == "C"){
            $documento = $this->colocarEmNegrito($documento, "[DESCELCS" . $dadoCompServ["INDELCS"] . "]");
            $tipo = "Composição";
            
        } else {
            if($tipo == "E"){
                $tipo = "Elemento";
            } else{
                $tipo = "";
            }
        }
        
        if($obrigatorio == "S"){
            $obrigatorio = "Sim";
        } else {
            if($obrigatorio == "N"){
                $obrigatorio = "Não";
            } else {
                $obrigatorio = "";
            }
        }
        
        if (empty($vrTotElCS) || $vrTotElCS == 0) {
            $vrTotElCS = "";
            $vrTotSAjElCS = "";
        } else {
            $vrTotElCS = Utilities::formatarNumero($vrTotElCS);
            $vrTotSAjElCS = Utilities::formatarNumero($vrTotSAjElCS);
        }
        
        $documento = $this->preservarEspacos($documento, "[DESCELCS" . $dadoCompServ["INDELCS"] . "]");

        $documento = str_replace("[INDELCS"  . $dadoCompServ["INDELCS"] . "]", "", $documento);
        $documento = str_replace("[TPCS"     . $dadoCompServ["INDELCS"] . "]", $tipo, $documento);
        $documento = str_replace("[IDOBRICS" . $dadoCompServ["INDELCS"] . "]", $obrigatorio, $documento);
        $documento = str_replace("[DESCELCS" . $dadoCompServ["INDELCS"] . "]", str_replace("&", "&amp;", $dadoCompServ["NMELEMENTO"]), $documento);
        $documento = str_replace("[VRTOTELCS". $dadoCompServ["INDELCS"] . "]", $vrTotElCS, $documento);
        $documento = str_replace("[VRTOTSAJELCS". $dadoCompServ["INDELCS"] . "]", $vrTotSAjElCS, $documento);
       
        return $documento;
    }
    
    /**
     * Retorna os dados das medidas da proposta
    **/
    public function getItensPropostaComVariacaoPreco($nrseqorcamento, $nrtipoorcamento, $nrorg, $idSituacao) {
        $params = array(
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "NRTIPOORCAMENTO"     => $nrtipoorcamento,
            "NRORG"               => $nrorg,
            "IDSITUACAOORCAMENTO" => $idSituacao
        );

        return $this->documentoFactory->getItensPropostaComVariacaoPreco($params);
    }

    /**
     * Retorna os dados da variação de preço da proposta
    **/
    public function getVariacoesPrecoProposta($nrseqorcamento, $nrtipoorcamento, $nrorg, $cdservorca, $idSituacao) {
        $params = array(
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "NRTIPOORCAMENTO"     => $nrtipoorcamento,
            "NRORG"               => $nrorg,
            "CDSERVORCA"          => $cdservorca,
            "IDSITUACAOORCAMENTO" => $idSituacao
        );
          
        return $this->documentoFactory->getVariacoesPrecoProposta($params);
    }

    /**
     * Coloca na linha os dados da medida 
    **/
    public function substituirDadosItensPropostaComVariacaoPreco($item, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDITVARPREC".$item['IND']."]", "",  $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCITVARPREC".$item['IND']."]", $item["NMITEMORCAMENTO"], $clobLinhaSubstituida);
        return $clobLinhaSubstituida;
    }
    /**
     * Cria a row para ser subtituida pelos dados 
    **/
    public function colocarIndiceItensPropostaComVariacaoPreco($indice, $clobLinha){
        $clobLinhaSubstituida = str_replace("[INDITVARPREC]", "[INDITVARPREC".$indice."]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCITVARPREC]", "[DESCITVARPREC".$indice."]",  $clobLinhaSubstituida);
        return $clobLinhaSubstituida;
    }
    /**
     * Cria a row para ser subtituida pelos dados 
    **/
    public function colocarIndiceMnemonicoVariacaoPreco($indice, $clobLinha){
        $clobLinhaSubstituida = str_replace("[INDVARPREC]",        "[INDVARPREC".$indice."]",        $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCMEDIDAVARPREC]", "[DESCMEDIDAVARPREC".$indice."]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTINIVARPREC]",      "[QTINIVARPREC".$indice."]",      $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTFINVARPREC]",      "[QTFINVARPREC".$indice."]",      $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPRECOVARIACAO]",   "[VRPRECOVARIACAO".$indice."]",   $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTFRANQUIA]",        "[QTFRANQUIA".$indice."]",        $clobLinhaSubstituida);
        return $clobLinhaSubstituida;
    }
    /**
     * Coloca na linha os dados da variação de preço
    **/
    public function substituirDadosVariacaoPreco($varPreco, $clobLinha){
        $clobLinhaSubstituida = str_replace("[INDVARPREC".$varPreco["IND"]."]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCMEDIDAVARPREC".$varPreco["IND"]."]", $varPreco["NMTPMEDIDAVARIAVEL"], $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTINIVARPREC".$varPreco["IND"]."]", $varPreco["QTMEDIDAINIVARIACAO"], $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTFINVARPREC".$varPreco["IND"]."]",    $varPreco["QTMEDIDAFIMVARIACAO"], $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPRECOVARIACAO".$varPreco["IND"]."]",  Utilities::formatarNumero($varPreco["VRPRECOVARIACAO"]),     $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTFRANQUIA".$varPreco["IND"]."]",      $varPreco["QTFRANQUIA"],          $clobLinhaSubstituida);
        return $clobLinhaSubstituida;
    }
    /**
     * Gera a tabela de variação de preço no documento 
    **/
    public function gerarDocumentoParteVariacaoPreco($nrseqorcamento, $nrorg, $idSituacao, $nrtipoorcamento, $documento) {
        $tamanhoDocumento = strlen($documento);
        $posicaoIdPai = strpos($documento, "[INDITVARPREC]");
        if($posicaoIdPai !== false){
            $posicaoFinal  = strpos($documento, "</w:tbl>", $posicaoIdPai);
            $posicaoInicio = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posicaoFinal) * -1));
            $tagFecha = "</w:tbl>";
            //$tagParagrafo = "<w:p/>";
            
            $clobInicial = substr($documento, 0, $posicaoFinal + strlen($tagFecha));
            $clobLinha   = substr($documento, $posicaoInicio, $posicaoFinal - $posicaoInicio);
            $clobFinal   = substr($documento, $posicaoFinal, $tamanhoDocumento);
            $itens       = $this->getItensPropostaComVariacaoPreco($nrseqorcamento, $nrtipoorcamento, $nrorg, $idSituacao);
            foreach ($itens as $item) {
                $clobLinhaSubstituida = $this->colocarIndiceItensPropostaComVariacaoPreco($item["IND"], $clobLinha);
                $clobLinhaSubstituida = $this->substituirDadosItensPropostaComVariacaoPreco($item, $clobLinhaSubstituida);
                $clobLinhaSubstituida = $this->gerarDocumentoParteFaixasVariacaoPreco($item, $nrseqorcamento, $nrtipoorcamento, $nrorg, $idSituacao, $clobLinhaSubstituida);
                $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha; //.$tagParagrafo;
            }
            
            if (count($itens) > 0) {
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
              
            $documento = str_replace($clobLinha . $tagFecha, "", $documento);
        }
        return $documento;
    }
    /**
     * Gera as faixas de variação de preço de cada medida
    **/
    public function gerarDocumentoParteFaixasVariacaoPreco($item, $nrseqorcamento, $nrtipoorcamento, $nrorg, $idSituacao, $linhaDocumento){
        $tamanhoLinha  = strlen($linhaDocumento);
        $posicaoIndice = strpos($linhaDocumento, "[INDVARPREC]");
        if($posicaoIndice !== false){
            $posicaoFechaTr = strpos($linhaDocumento, "</w:tr>", $posicaoIndice);
            $posicaoAbreTr  = strrpos($linhaDocumento, "<w:tr ", (($tamanhoLinha - $posicaoFechaTr) * -1));
            
            if(!$posicaoAbreTr)
                $posicaoAbreTr  = strrpos($linhaDocumento, "<w:tr>", (($tamanhoLinha - $posicaoFechaTr) * -1));
            
            if($posicaoAbreTr !== false){
                $posInicial = $posicaoAbreTr;
                $posFinal   = $posicaoFechaTr;
                $tagFecha   = "</w:tr>";
                
                $clobInicial = substr($linhaDocumento, 0, $posFinal + strlen($tagFecha));
                $clobLinha   = substr($linhaDocumento, $posInicial, $posFinal - $posInicial);
                $clobFinal   = substr($linhaDocumento, $posFinal, $tamanhoLinha);
                
                $varPrecos = $this->getVariacoesPrecoProposta($nrseqorcamento, $nrtipoorcamento, $nrorg, $item["CDSERVORCA"], $idSituacao);
                foreach ($varPrecos as $varPreco) {
                    $clobSubstituida = $this->colocarIndiceMnemonicoVariacaoPreco($varPreco["IND"], $clobLinha);
                    $clobSubstituida = $this->substituirDadosVariacaoPreco($varPreco, $clobSubstituida);
                    $clobInicial     = $clobInicial . $clobSubstituida . $tagFecha;
                }
                
                if (count($varPrecos) > 0) {
                    //$clobLinhaSubstituida = $this->retirarMnemonicosNegociacoesValores($clobLinha);
                    $linhaDocumento = $clobInicial.substr($clobFinal, strlen($tagFecha));
                }
                  
                $linhaDocumento = str_replace($clobLinha.$tagFecha, "", $linhaDocumento);
            }
        }

        return $linhaDocumento;
    }

    private function getParcelasNegociacao($nrnegociacaoproposta, $nritnegociacaoproposta, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA"   => $nrnegociacaoproposta,
            "NRITNEGOCIACAOPROPOSTA" => $nritnegociacaoproposta,
            "NRORG"                  => $nrorg
        );
        return $this->documentoFactory->getParcelasNegociacao($params);
    }

    private function getPrimeiraCompetenciaNegociacaoProposta($nrnegociacaoproposta, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );
        
        $dataset = $this->documentoFactory->getPrimeiraCompetenciaNegociacaoProposta($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }

    private function calcularDataInicioFaturamentoCompetencia(array $item, array $competencia) {
        if ($item["DTFINALVALIDADEDESC"]) {
            $dataReferencia = DateUtil::getDataDeString($item["DTFINALVALIDADEDESC"]);
        } else if (!$item["QTMESESDESC"]) {
            $dataReferencia = $item["DTACEITEORCA"];
            if ($dataReferencia) 
                $dataReferencia = DateUtil::getDataDeString($dataReferencia);
            else
                $dataReferencia = DateUtil::getDataAtual(true);
        } else {
            $dataReferencia = DateUtil::getDataAtual(true);
            $qtdParcelas    = $item["QTMESESDESC"];

            $dataAux = explode("/", $dataReferencia->format("d/m/Y"));
           
            if (checkdate($dataAux[1], $competencia["NRDIACOMPETFIN"], $dataAux[2])) {
                $dataReferencia = DateUtil::getDataDeString(str_pad($competencia["NRDIACOMPETFIN"], 2, "0", STR_PAD_LEFT) . "/" . $dataAux[1] . "/" . $dataAux[2]);
            } else {
                $dataReferencia = DateUtil::getDataDeString("01/" . $dataAux[1] . "/" . $dataAux[2]);
                $dataReferencia = DateUtil::getUltimoDiaDoMes($dataReferencia);
            }
            
            if ($competencia["IDMESCOMPETFIN"] == "3") {
                if (checkdate($dataReferencia->format("d"), $dataReferencia->format("m")+1, $dataReferencia->format("Y"))) {
                    $dataReferencia = DateUtil::getDataDeString($dataReferencia->format("d") . "/" . str_pad((int)$dataReferencia->format("m")+1, 2, "0", STR_PAD_LEFT) . "/" . $dataReferencia->format("Y"));
                } else {
                    $dataReferencia = DateUtil::getDataDeString("01/" . str_pad((int)$dataReferencia->format("m")+1, 2, "0", STR_PAD_LEFT) . "/" . $dataReferencia->format("Y"));
                    $dataReferencia = DateUtil::getUltimoDiaDoMes($dataReferencia);
                }
            }

            $dataReferencia = DateUtil::getPrimeiroDiaDoMes($dataReferencia);
            $dataReferencia = Utilities::adicionarMes($dataReferencia, $qtdParcelas);
        }
        
        return $dataReferencia->format("d/m/Y");
    }

    public function calcularDiaVencimentoCompetencia(array $competencia, $dataIniFatura) {
        $idtpcalcvenc  = $competencia["IDTPCALCVENC"];
        $nrdiacalcvenc = str_pad($competencia["NRDIACALCVENC"], 2, "0", STR_PAD_LEFT);
        
        if ($idtpcalcvenc == "1") { //dia fixo
            return $nrdiacalcvenc;
        }
        else { //qtd dias apos emissao
            $qtdiasapuracao = $competencia["QTDIASAPURACAO"];
            $dataIniFatura = DateUtil::getDataDeString($dataIniFatura);
            $dataVencimento = DateUtil::adicionaIntervalo($dataIniFatura, $qtdiasapuracao);
            $dataVencimento = DateUtil::adicionaIntervalo($dataVencimento, $nrdiacalcvenc);
            return $dataVencimento->format("d");
        }
    }

    private function gerarDocumentoParteDeNegociacaoValores($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDNV]" no documento inteiro
        $posicaoInd = strpos($documento, "[INDNV]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[INDNV]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
            $posicaoFechaLN = strpos($documento, "[/LN]", $posicaoInd);
            
            //posicao do primeiro "[LN]" que antecede o "[INDNV]"
            $posicaoPrimeiroLN = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoFechaLN) * -1));
            
            
            if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN !== false) {
                if ($posicaoPrimeiroTr < $posicaoPrimeiroLN) {
                    $result = "C";
                
                    $posIni = $posicaoPrimeiroTr;
                    $posFin = $posicaoFechaTr;
                    $tagFecha = "</w:tr>";
                }
                else {
                    $result = "L";
                
                    //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "[/LN]", $posicaoInd);
                    
                    //posicao do primeiro "[LN]" que antecede o "[INDNV]"
                    $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "[/LN]";
                }
            }
            else if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN === false) {
                $result = "C";
                
                $posIni = $posicaoPrimeiroTr;
                $posFin = $posicaoFechaTr;
                $tagFecha = "</w:tr>";
            }
            else {
                $result = "L";
                
                //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "[/LN]", $posicaoInd);
                
                //posicao do primeiro "[LN]" que antecede o "[INDNV]"
                $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "[/LN]";
            }
            
            $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
            $clobLinha = substr($documento, $posIni, $posFin - $posIni);
            $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
            
            $linhasCodigoNegociacoes = $this->getCodigoNegociacoesValores($nrseqorcamento, $nrtipoorcamento, $nrorg);
            $qtdItens = count($linhasCodigoNegociacoes);
            
            foreach ($linhasCodigoNegociacoes as $linhaCodigoNegociacao) {
                //coloca o índice nos mnemônicos dos itens
                $clobLinhaSubstituida = $this->colocarIndiceMnemonicoNegociacoesValores($linhaCodigoNegociacao["IND"], $clobLinha);
                
                if ($result == "C") {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                }
                else {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida;
                }
            }
            
            if ($qtdItens > 0) {
                //retira os mnemônicos dos itens
                $clobLinhaSubstituida = $this->retirarMnemonicosNegociacoesValores($clobLinha);
                
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
        }
        
        return $documento;
    }

    public function getCodigoNegociacoesValores($nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );

        return $this->documentoFactory->getCodigoNegociacoesValores($params);
    }

    private function colocarIndiceMnemonicoNegociacoesValores($indiceNegociacao, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDNV]", "[INDNV" . $indiceNegociacao . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCNV]", "[DESCNV" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTNV]", "[VRTOTNV" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPERCDESCNV]", "[VRPERCDESCNV" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRDESCNV]", "[VRDESCNV" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTCOMDESCNV]", "[VRTOTCOMDESCNV" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function retirarMnemonicosNegociacoesValores($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDNV]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCNV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTNV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPERCDESCNV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRDESCNV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTCOMDESCNV]", "", $clobLinhaSubstituida);

        return $clobLinhaSubstituida;
    }

    private function getNegociacoesValores($nrseqorcamento, $nrtipoorcamento, $nrorg, $cdidiom) {
        $params = array(
            "IDIOMA"          => $cdidiom,
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );

        return $this->documentoFactory->getNegociacoesValores($params);
    }

    private function substituirDadosNegociacoesValoresDocumento($negociacao, $documento) {
        $documento = str_replace("[INDNV".$negociacao["IND"]."]", "", $documento);
        $documento = str_replace("[DESCNV".$negociacao["IND"]."]", str_replace("&", "&amp;", $negociacao["DSNOMEAGRUPAMENTO"]), $documento);
        $documento = str_replace("[VRTOTNV".$negociacao["IND"]."]", Utilities::formatarNumero($negociacao["VRTOTITEM"]), $documento);
        $documento = str_replace("[VRPERCDESCNV".$negociacao["IND"]."]", Utilities::formatarNumero($negociacao["VRPERCDESCITEM"]), $documento);
        $documento = str_replace("[VRDESCNV".$negociacao["IND"]."]", Utilities::formatarNumero($negociacao["VRDESCITEM"]), $documento);
        $documento = str_replace("[VRTOTCOMDESCNV".$negociacao["IND"]."]", Utilities::formatarNumero($negociacao["VRTOTITEMCOMDESC"]), $documento);

        return $documento;
    }

    private function gerarDocumentoParteDeNegociacoesValoresDeParcelamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDNVPARC]" no documento inteiro
        $posicaoInd = strpos($documento, "[INDNVPARC]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr>" que antecede o "[INDNVPARC]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "</w:tbl>", $posicaoInd);
                
                //posicao do primeiro "<w:tbl>" que antecede o "[INDNVPARC]"
                $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "</w:tbl>";
            
            
                $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($documento, $posIni, $posFin - $posIni);
                $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
                
                $negociacoes = $this->getCodigoNegociacoesValoresDeParcelamento($nrseqorcamento, $nrtipoorcamento, $nrorg);
                $qtdNegociacoes = count($negociacoes);
                
                foreach ($negociacoes as $negociacao) {
                    if ($negociacao["VRTOTITEMCOMDESC"]) {
                        //coloca o índice nos mnemônicos das negociações de valores para parcelamento
                        $clobLinhaSubstituida = $this->colocarIndiceMnemonicoNegociacoesValoresDeParcelamento($negociacao["IND"], $clobLinha);
                        
                        //monta a parte de parcelas da negociação de valores
                        $clobLinhaSubstituida = $this->gerarDocumentoParteDeParcelasNegociacoesDeValores($nrorg, $negociacao, $clobLinhaSubstituida);
                        
                        $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                    }
                }
                
                if ($qtdNegociacoes > 0) {
                    //retira os mnemônicos das negociações de valores de parcelamento
                    $clobLinhaSubstituida = $this->retirarMnemonicosNegociacoesValoresDeParcelamento($clobLinha);
                    
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        
        return $documento;
    }

    private function getCodigoNegociacoesValoresDeParcelamento($nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        return $this->documentoFactory->getCodigoNegociacoesValoresDeParcelamento($params);
    }

    private function colocarIndiceMnemonicoNegociacoesValoresDeParcelamento($indiceNegociacao, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDNVPARC]", "[INDNVPARC" . $indiceNegociacao . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCNVPARC]", "[DESCNVPARC" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTCOMDESCNVPARC]", "[VRTOTCOMDESCNVPARC" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function retirarMnemonicosNegociacoesValoresDeParcelamento($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDNVPARC]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCNVPARC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTCOMDESCNVPARC]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function gerarDocumentoParteDeParcelasNegociacoesDeValores($nrorg, $negociacao, $clobLinhaSubstituida) {
        $posIniParcela = -1;
        $posFinParcela = -1;
        
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        
        //posicao do primeiro "[INDPARCNV]" dentro de $clobLinhaSubstituida
        $posicaoIndParcela = strpos($clobLinhaSubstituida, "[INDPARCNV]");
        
        if ($posicaoIndParcela !== false) {
            //posicao do primeiro "</w:tr>" dentro de $clobLinhaSubstituida, contando a partir de $posicaoIndParcela
            $posicaoFechaTrParcela = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoIndParcela);
            
            //posicao do primeiro "<w:tr>" que antecede o "[INDPARCNV]" dentro de $clobLinhaSubstituida
            $posicaoPrimeiroTrParcela = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrParcela) * -1));
            
            if (!$posicaoPrimeiroTrParcela) {
                $posicaoPrimeiroTrParcela = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrParcela) * -1));
            }
            
            if ($posicaoPrimeiroTrParcela !== false) {
                $posIniParcela = $posicaoPrimeiroTrParcela;
                $posFinParcela = $posicaoFechaTrParcela;
                $tagFecha = "</w:tr>";
            
                $clobInicialParcela = substr($clobLinhaSubstituida, 0, $posFinParcela + strlen($tagFecha));
                $clobLinhaParcela = substr($clobLinhaSubstituida, $posIniParcela, $posFinParcela - $posIniParcela);
                $clobFinalParcela = substr($clobLinhaSubstituida, $posFinParcela, $tamanhoClobLinhaSubstituida);
                
                $parcelasNegociacao = $this->getParcelasNegociacaoValores($negociacao["NRNEGOCIACAOPROPOSTA"], $nrorg);
                if (count($parcelasNegociacao) > 0) {
                    
                    $dataAceite = $negociacao["DTACEITEORCA"];
                    if ($dataAceite) {
                        $dataAceite = DateUtil::getDataDeString($dataAceite);
                    }
                    else {
                        $dataAceite = DateUtil::getDataAtual(true);
                    }
                    
                    $informacoesParcelas = array();
                    $sequencial = 1;

                    foreach ($parcelasNegociacao as $parcela) {
                        $valorParcela       = $parcela["VRPARCELA"];
                        $qtdDiasFaturamento = $parcela["QTDIASFATURAMENTO"];
                        $diaVencimento      = $parcela["NRDIAVENCIMENTO"];

                        $dataReferencia = DateUtil::adicionaIntervalo($dataAceite, $qtdDiasFaturamento);

                        if ($diaVencimento) {
                            $diaVencimento = str_pad($diaVencimento, 2, "0", STR_PAD_LEFT);
                            if (checkdate((int)$dataReferencia->format("m"), (int)$diaVencimento, (int)$dataReferencia->format("Y"))) {
                                $dataVencimento = DateUtil::getDataDeString($diaVencimento . "/" . $dataReferencia->format("m/Y"));
                            }
                            else {
                                $dataVencimento = DateUtil::getDataDeString("01/" . $dataReferencia->format("m/Y"));
                                $dataVencimento = DateUtil::getUltimoDiaDoMes($dataVencimento);
                            }
                            
                            if (DateUtil::getQtdeDiasEntre($dataVencimento, $dataReferencia) > 0) {
                                $dataVencimento = DateUtil::adicionaIntervalo(DateUtil::getUltimoDiaDoMes($dataVencimento), 1);
                                $dataVencimento = DateUtil::getDataDeString($diaVencimento . "/" . $dataVencimento->format("m/Y"));
                            }
                        }
                        else {
                            $paramsFatItensNego = $this->getParametrosFaturamentoItensNegociacao($negociacao["NRNEGOCIACAOPROPOSTA"], $nrorg);
                            if ($paramsFatItensNego) {
                                $idtpcalcvenc  = $paramsFatItensNego["IDTPCALCVENC"];
                                $nrdiacalcvenc = str_pad($paramsFatItensNego["NRDIACALCVENC"], 2, "0", STR_PAD_LEFT);
                                $idmescalcvenc = $paramsFatItensNego["IDMESCALCVENC"];
                            }
                            else {
                                $idtpcalcvenc  = $parcela["IDTPCALCVENC"];
                                $nrdiacalcvenc = str_pad($parcela["NRDIACALCVENC"], 2, "0", STR_PAD_LEFT);
                                $idmescalcvenc = $parcela["IDMESCALCVENC"];
                            }

                            //$idtpcalcvenc  = $parcela["IDTPCALCVENC"];
                            //$nrdiacalcvenc = str_pad($parcela["NRDIACALCVENC"], 2, "0", STR_PAD_LEFT);
                            
                            if ($idtpcalcvenc == "1") { //dia fixo
                                //$idmescalcvenc = $parcela["IDMESCALCVENC"];

                                $dataVencimento = DateUtil::getDataDeString($nrdiacalcvenc . "/" . $dataReferencia->format("m/Y"));
                                if ($idmescalcvenc == "1") {
                                    $dataVencimento = DateUtil::subtraiIntervalo($dataVencimento, 1, DateUtil::MES);
                                }
                                else if ($idmescalcvenc == "3") {
                                    $dataVencimento = DateUtil::adicionaIntervalo($dataVencimento, 1, DateUtil::MES);
                                }
                            }
                            else { //qtd dias apos emissao
                                $dataVencimento = DateUtil::adicionaIntervalo($dataReferencia, $nrdiacalcvenc);
                            }
                        }

                        $informacoesParcelas[] = array(
                            "NRPARCELA"    => $sequencial,
                            "DTVENCIMENTO" => $dataVencimento->format("d/m/Y"),
                            "VRVENCIMENTO" => $valorParcela
                        );

                        $sequencial++;
                    }
                }
                else {
                    throw new \Exception("Operação bloqueada. <br/>Algumas Negociações de Valores do tipo \"Parcelamento\" não possuem parcelas definidas.");
                }
                
                $qtdInfoParcelas = count($informacoesParcelas);
                    
                foreach ($informacoesParcelas as $infoParcela) {
                    //coloca o índice nos mnemônicos das parcelas de negociação de valores
                    $clobLinhaParcelaSubstituida = $this->colocarIndiceMnemonicosParcelasNegociacaoValores($negociacao["NRNEGOCIACAOPROPOSTA"], $infoParcela["NRPARCELA"], $clobLinhaParcela);
                    
                    $clobInicialParcela = $clobInicialParcela . $clobLinhaParcelaSubstituida . $tagFecha;
                }
                
                if ($qtdInfoParcelas > 0) {
                    //retira os mnemônicos das parcelas de negociação de valores
                    $clobLinhaParcelaSubstituida = $this->retirarMnemonicosParcelasNegociacaoValores($clobLinhaParcela);
                }
                
                $clobLinhaSubstituida = $clobInicialParcela . substr($clobFinalParcela, strlen($tagFecha));
                $clobLinhaSubstituida = str_replace($clobLinhaParcela.$tagFecha, "", $clobLinhaSubstituida);
                
                foreach ($informacoesParcelas as $infoParcela) {
                    //substitui os dados das parcelas de negociação de valores no documento
                    $clobLinhaSubstituida = $this->substituirDadosParcelasNegociacaoValores($negociacao["NRNEGOCIACAOPROPOSTA"], $infoParcela, $clobLinhaSubstituida);
                }
            }
        }
        
        return $clobLinhaSubstituida;
    }

    private function getParcelasNegociacaoValores($nrnegociacaoproposta, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );
        return $this->documentoFactory->getParcelasNegociacaoValores($params);
    }

    private function colocarIndiceMnemonicosParcelasNegociacaoValores($nrnegociacaoproposta, $indice, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDPARCNV]", "[INDPARCNV" . $nrnegociacaoproposta . "_" . $indice . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DTVENCPARCNV]", "[DTVENCPARCNV" . $nrnegociacaoproposta . "_" . $indice . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPARCNV]", "[VRPARCNV" . $nrnegociacaoproposta . "_" . $indice . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function retirarMnemonicosParcelasNegociacaoValores($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDPARCNV]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DTVENCPARCNV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPARCNV]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function getNegociacoesValoresDeParcelamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $cdidiom) {
        $params = array(
            "IDIOMA"          => $cdidiom,
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        return $this->documentoFactory->getNegociacoesValoresDeParcelamento($params);    
    }

    private function substituirDadosNegociacoesValoresDeParcelamentoDocumento($negociacaoParc, $documento) {
        $documento = str_replace("[INDNVPARC".$negociacaoParc["IND"]."]", "", $documento);
        $documento = str_replace("[DESCNVPARC".$negociacaoParc["IND"]."]", str_replace("&", "&amp;", $negociacaoParc["DSNOMEAGRUPAMENTO"]), $documento);
        $documento = str_replace("[VRTOTCOMDESCNVPARC".$negociacaoParc["IND"]."]", Utilities::formatarNumero($negociacaoParc["VRTOTITEMCOMDESC"]), $documento);
        
        return $documento;
    }

    private function substituirDadosParcelasNegociacaoValores($nrnegociacaoproposta, $infoParcela, $clobLinhaSubstituida) {
        $clobLinhaSubstituida = str_replace("[INDPARCNV" . $nrnegociacaoproposta . "_" . $infoParcela["NRPARCELA"] . "]", str_pad($infoParcela["NRPARCELA"], 2, "0", STR_PAD_LEFT), $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DTVENCPARCNV" . $nrnegociacaoproposta . "_" . $infoParcela["NRPARCELA"] . "]", $infoParcela["DTVENCIMENTO"], $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRPARCNV" . $nrnegociacaoproposta . "_" . $infoParcela["NRPARCELA"] . "]", Utilities::formatarNumero($infoParcela["VRVENCIMENTO"]), $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function gerarDocumentoParteDeNegociacoesValoresParaDescontoPorPeriodo($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDNVDP]" no documento inteiro
        $posicaoInd = strpos($documento, "[INDNVDP]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr>" que antecede o "[INDNVDP]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "</w:tbl>", $posicaoInd);
                
                //posicao do primeiro "<w:tbl>" que antecede o "[INDNVDP]"
                $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "</w:tbl>";
            
            
                $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($documento, $posIni, $posFin - $posIni);
                $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
                
                $negociacoes = $this->getCodigoNegociacoesValoresParaDescontoPorPeriodo($nrseqorcamento, $nrtipoorcamento, $nrorg);
                $qtdNegociacoes = count($negociacoes);
                
                foreach ($negociacoes as $negociacao) {
                    //coloca o índice nos mnemônicos das negociações de valores para parcelamento
                    $clobLinhaSubstituida = $this->colocarIndiceMnemonicoNegociacoesValoresParaDescontoPorPeriodo($negociacao["IND"], $clobLinha);
                    
                    //monta a parte de parcelas da negociação de valores
                    $clobLinhaSubstituida = $this->gerarDocumentoParteDeDescontoPorPeriodoNegociacoesDeValores($nrorg, $negociacao["NRNEGOCIACAOPROPOSTA"], $clobLinhaSubstituida);
                    
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                }
                
                if ($qtdNegociacoes > 0) {
                    //retira os mnemônicos das negociações de valores de parcelamento
                    $clobLinhaSubstituida = $this->retirarMnemonicosNegociacoesValoresParaDescontoPorPeriodo($clobLinha);
                    
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        
        return $documento;
    }

    private function getCodigoNegociacoesValoresParaDescontoPorPeriodo($nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        return $this->documentoFactory->getCodigoNegociacoesValoresParaDescontoPorPeriodo($params);
    }

    private function colocarIndiceMnemonicoNegociacoesValoresParaDescontoPorPeriodo($indiceNegociacao, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDNVDP]", "[INDNVDP" . $indiceNegociacao . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCNVDP]", "[DESCNVDP" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTCOMDESCNVDP]", "[VRTOTCOMDESCNVDP" . $indiceNegociacao . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function retirarMnemonicosNegociacoesValoresParaDescontoPorPeriodo($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDNVDP]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCNVDP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTCOMDESCNVDP]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function gerarDocumentoParteDeDescontoPorPeriodoNegociacoesDeValores($nrorg, $nrnegociacaoproposta, $clobLinhaSubstituida) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        
        //posicao do primeiro "[INDDP]" dentro de $clobLinhaSubstituida
        $posicaoInd = strpos($clobLinhaSubstituida, "[INDDP]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" dentro de $clobLinhaSubstituida, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[INDDP]" dentro de $clobLinhaSubstituida
            $posicaoPrimeiroTr = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                $posIni = $posicaoPrimeiroTr;
                $posFin = $posicaoFechaTr;
                $tagFecha = "</w:tr>";
                
                $clobInicial = substr($clobLinhaSubstituida, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($clobLinhaSubstituida, $posIni, $posFin - $posIni);
                $clobFinal = substr($clobLinhaSubstituida, $posFin, $tamanhoClobLinhaSubstituida);
                
                $linhasCodigoDescontosPorPeriodo = $this->getCodigoDescontosPorPeriodoNegociacaoValores($nrnegociacaoproposta, $nrorg);
                
                $qtdDescontos = count($linhasCodigoDescontosPorPeriodo);
                foreach ($linhasCodigoDescontosPorPeriodo as $linhaCodigoDesconto) {
                    //coloca o índice nos mnemônicos das parcelas por periodo
                    $clobLinhaSubstituida = $this->colocarIndiceDescontoPorPeriodoNegociacaoValores($nrnegociacaoproposta, $linhaCodigoDesconto["IND"], $clobLinha);
                    
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                }
                
                if ($qtdDescontos > 0) {
                    //retira os mnemônicos das parcelas por periodo
                    $clobLinhaSubstituida = $this->retirarMnemonicosDescontoPorPeriodoNegociacaoValores($clobLinha);
                    
                    $clobLinhaSubstituida = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $clobLinhaSubstituida = str_replace($clobLinha.$tagFecha, "", $clobLinhaSubstituida);
            }
        }
        
        return $clobLinhaSubstituida;
    }

    private function getCodigoDescontosPorPeriodoNegociacaoValores($nrnegociacaoproposta, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg,
        );
        return $this->documentoFactory->getCodigoDescontosPorPeriodoNegociacaoValores($params);
    }

    /**
     * Coloca o índice nos mnemônicos dos elementos da proposta.
     */
    private function colocarIndiceDescontoPorPeriodoNegociacaoValores($nrnegociacaoproposta, $indiceDesconto, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDDP]", "[INDDP" . $nrnegociacaoproposta . "_" . $indiceDesconto . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[VRPERCDESCDP]", "[VRPERCDESCDP" . $nrnegociacaoproposta . "_" . $indiceDesconto . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRDESCDP]", "[VRDESCDP" . $nrnegociacaoproposta . "_" . $indiceDesconto . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTCOMDESCDP]", "[VRTOTCOMDESCDP" . $nrnegociacaoproposta . "_" . $indiceDesconto . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTPARCDP]", "[QTPARCDP" . $nrnegociacaoproposta . "_" . $indiceDesconto . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ORDEMDP]", "[ORDEMDP" . $nrnegociacaoproposta . "_" . $indiceDesconto . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function retirarMnemonicosDescontoPorPeriodoNegociacaoValores($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDDP]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[VRPERCDESCDP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRDESCDP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOTCOMDESCDP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTPARCDP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ORDEMDP]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function getNegociacoesValoresParaDescontoPorPeriodo($nrseqorcamento, $nrtipoorcamento, $nrorg, $cdidiom) {
        $params = array(
            "IDIOMA"          => $cdidiom,
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        return $this->documentoFactory->getNegociacoesValoresParaDescontoPorPeriodo($params);
    }

    private function substituirDadosNegociacoesValoresParaDescontoPorPeriodoDocumento($negociacaoDescPer, $documento) {
        $documento = str_replace("[INDNVDP".$negociacaoDescPer["IND"]."]", "", $documento);
        $documento = str_replace("[DESCNVDP".$negociacaoDescPer["IND"]."]", str_replace("&", "&amp;", $negociacaoDescPer["DSNOMEAGRUPAMENTO"]), $documento);
        $documento = str_replace("[VRTOTCOMDESCNVDP".$negociacaoDescPer["IND"]."]", Utilities::formatarNumero($negociacaoDescPer["VRTOTITEMCOMDESC"]), $documento);
        
        return $documento;
    }

    private function getDescontosPorPeriodoNegociacaoValores($nrnegociacaoproposta, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );
        return $this->documentoFactory->getDescontosPorPeriodoNegociacaoValores($params);
    }

    private function substituirDadosDescontoPorPeriodoNegociacaoValores($nrnegociacaoproposta, $descPer, $documento) {
        $documento = str_replace("[INDDP" . $nrnegociacaoproposta . "_" . $descPer["IND"] . "]", "", $documento);
        $documento = str_replace("[VRPERCDESCDP" . $nrnegociacaoproposta . "_" . $descPer["IND"] . "]", Utilities::formatarNumero($descPer["VRPERCDESCITEM"]), $documento);
        $documento = str_replace("[VRDESCDP" . $nrnegociacaoproposta . "_" . $descPer["IND"] . "]", Utilities::formatarNumero($descPer["VRDESCITEM"]), $documento);
        $documento = str_replace("[VRTOTCOMDESCDP" . $nrnegociacaoproposta . "_" . $descPer["IND"] . "]", Utilities::formatarNumero($descPer["VRTOTITEMCOMDESC"]), $documento);
        $documento = str_replace("[QTPARCDP" . $nrnegociacaoproposta . "_" . $descPer["IND"] . "]", $descPer["QTMESESDESC"], $documento);
        $documento = str_replace("[ORDEMDP" . $nrnegociacaoproposta . "_" . $descPer["IND"] . "]", $descPer["NRORDEMAPLICADESC"], $documento);
        
        return $documento;
    }

    public function getParametrosFaturamentoItensNegociacao($nrnegociacaoproposta, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );

        $dataset = $this->documentoFactory->getParametrosFaturamentoItensNegociacao($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Valida se o documento mostrará os itens da proposta e se eles existem
    **/
    public function validarExistenciaItensDocumento($documento, $nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $posicaoInd = strpos($documento, "[IND]");
        if ($posicaoInd !== false) {
            $itens = $this->getItensProposta($nrseqorcamento, $nrtipoorcamento, $nrorg);
            return count($itens) > 0;
        }
        
        return true;
    }
    
    /**
     * Monta a parte de Assinaturas da proposta.
     */
    private function gerarDocumentoAssinaturas($nrseqorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDSIGN]" no documento inteiro
        $posicaoIndSign = strpos($documento, "[INDSIGN]");
        
        if ($posicaoIndSign !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoIndSign
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoIndSign);
            
            //posicao do primeiro "<w:tr>" que antecede o "[INDSIGN]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoIndSign
                $posFin = strpos($documento, "</w:tbl>", $posicaoIndSign);
                
                //posicao do primeiro "<w:tbl>" que antecede o "[INDSIGN]"
                $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "</w:tbl>";
            
            
                $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($documento, $posIni, $posFin - $posIni);
                $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
                
                $assinaturas = $this->getAssinaturasProposta($nrseqorcamento, $nrorg);
                $qtdItens = count($assinaturas);
                
                foreach ($assinaturas as $assinatura) {

                    //coloca os indices nos mnemônicos das assinaturas da proposta
                    $clobLinhaSubstituida = $this->colocarIndiceAssinaturasProposta($assinatura["IND"], $clobLinha);
                    
                    //monta a parte das assinaturas da proposta
                    $clobLinhaSubstituida = $this->gerarDocumentoAssinaturasProposta($nrorg, $assinatura, $clobLinhaSubstituida);
                    
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                }
                
                if ($qtdItens > 0) {
                    //Retira os mnemônicos das assinaturas da proposta do documento
                    $clobLinhaSubstituida = $this->retirarMnemonicosAssinaturasProposta($clobLinha);
                    
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        
        return $documento;
    }
    
      /**
     * Retorna as assinaturas da proposta
     */
    private function getAssinaturasProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getAssinaturasProposta($params);
    }
    
      /**
     * coloca os indices nos mnemônicos das assinaturas da proposta
     */
    private function colocarIndiceAssinaturasProposta($indice, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDSIGN]",   "[INDSIGN"   . $indice . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[NMSIGNAT]",  "[NMSIGNAT"  . $indice . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[CPFSIGNAT]", "[CPFSIGNAT" . $indice . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPASSINAT]", "[TPASSINAT" . $indice . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSIGNAT]",  "[TPSIGNAT"  . $indice . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    
    /**
     * Monta a parte das assinaturas da proposta
     */
    private function gerarDocumentoAssinaturasProposta($nrorg, $assinatura, $clobLinhaSubstituida) {
        
        $nomeSignatario = $assinatura["NMSIGNATARIO"];
        $cpfSignatario  = Utilities::formatarCpfCnpj($assinatura["NRCPFSIGNATARIO"]);
        $tipoAssinatura = $assinatura["IDTIPOASSINATURA"];
        $tipoSignatario = $assinatura["IDTESTEMUNHA"];
        
        if($tipoAssinatura == "E"){
            $tipoAssinatura = "Eletrônica";
        } else {
            $tipoAssinatura = "Digital";
        }
        
        if($tipoSignatario == "S"){
            $tipoSignatario = "Testemunha";
        } else {
            $tipoSignatario = "Signatário";
        }
        
        $clobLinhaSubstituida = str_replace("[INDSIGN"   . $assinatura["IND"] . "]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[NMSIGNAT"  . $assinatura["IND"] . "]", $nomeSignatario, $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[CPFSIGNAT" . $assinatura["IND"] . "]", $cpfSignatario, $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPASSINAT" . $assinatura["IND"] . "]", $tipoAssinatura, $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSIGNAT"  . $assinatura["IND"] . "]", $tipoSignatario, $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    
    }
    
    /**
     * Retira os mnemônicos das assinaturas da proposta do documento
     */
    private function retirarMnemonicosAssinaturasProposta($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDSIGN]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[NMSIGNAT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[CPFSIGNAT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPASSINAT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSIGNAT]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Substitui os dados de alteração contratual
     */
    public function substituirDadosAlteracaoContratual($nrseqorcamento, $nrtipoorcamento, $nrorg, $documento) {
        
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosAlteracaoContratual($params);
        
        $altContratual = (count($dataset) > 0) ? $dataset[0] : null;
        
        if (!empty($altContratual["DSALTERACONTRATUAIS"])) {
            
            $documento =  str_replace("[INIALTERCONTR]", "", $documento);
            $documento =  str_replace("[ALTERCONTR]", $altContratual["DSALTERACONTRATUAIS"], $documento);
            $documento =  str_replace("[FIMALTERCONTR]", "", $documento);
        }
        else{
            
            $posicaoIniAltContr = strpos($documento, "[INIALTERCONTR]");
            $posicaoFimAltContr = strpos($documento, "[FIMALTERCONTR]");
            
            if (($posicaoIniAltContr != false) && ($posicaoFimAltContr != false)) {
            
                $posicaoFimAltContr = ($posicaoFimAltContr - $posicaoIniAltContr)+15;
                
                $AlteraContr = substr($documento, $posicaoIniAltContr, $posicaoFimAltContr);
                
                $documento =  str_replace($AlteraContr, "", $documento);
            }
        }
            
        return $documento;
    }
    
    private function gerarDocumentoParteDeDescricaoElementos($nrseqorcamento, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDELEMENTO]" no documento inteiro
        $posicaoIndElemento = strpos($documento, "[INDELEMENTO]");
        
        if ($posicaoIndElemento !== false) {
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoIndElemento
            $posicaoFechaLn = strpos($documento, "[/LN]", $posicaoIndElemento);
            
            //posicao do primeiro "[LN]" que antecede o "[INDELEMENTO]"
            $posicaoPrimeiroLn = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoFechaLn) * -1));
            
            if ($posicaoPrimeiroLn !== false) {
                //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoIndElemento
                $posFin = strpos($documento, "[/LN]", $posicaoIndElemento);
                
                //posicao do primeiro "[LN]" que antecede o "[INDELEMENTO]"
                $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "[/LN]";
            
            
                $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($documento, $posIni, $posFin - $posIni);
                $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
                
                $descricoes = $this->getDescricaoElementos($nrseqorcamento, $nrorg);
                $qtdItens = count($descricoes);
                
                foreach ($descricoes as $descricao) {

                    //substitui os mnemonicos da descricao dos elementos
                    $clobLinhaSubstituida = $this->substituirDadosDescricaoElemento($descricao, $clobLinha);
                    
                    $clobInicial = $clobInicial . $clobLinhaSubstituida;
                }
                
                if ($qtdItens > 0) {
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        
        return $documento;
    }
    
    private function getDescricaoElementos($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getDescricaoElementos($params);
    }
    
      /**
     * substitui os mnemonicos da descricao dos elementos
     */
    private function substituirDadosDescricaoElemento($descricao, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDELEMENTO]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[NMELEMENTO]", $descricao["NMELEMENTO"], $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DSELEMENTO]", $descricao["DSELEMENTO"], $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Substitui os dados do Revenda da proposta no documento.
     */
    public function substituirDadosRevendaDocumento($revenda, $enderecoRevenda, $templateProcessor) {
        if (!empty($revenda)) {
            
            $templateProcessor->setValue("[INIREVENDA]", "");
            $templateProcessor->setValue("[FIMREVENDA]", "");

            $templateProcessor->setValue("[NMPRINCIPALPARC]", $revenda["NMPRINCIPALPARC"]);
            $templateProcessor->setValue("[RENDERECO]", $enderecoRevenda["ENDERECO"]);
            
            $cep = Utilities::formatarCep($enderecoRevenda["NRCEPENDERECO"]);
            $templateProcessor->setValue("[RCEP]", $cep);
            
            $inscricao = Utilities::formatarCpfCnpj($revenda["NRINSCRICAOPARC"]);
            $templateProcessor->setValue("[NRINSCRICAOPARC]", $inscricao);
        } else {
            $templateProcessor->setValue("[NMPRINCIPALPARC]", "");
            $templateProcessor->setValue("[RENDERECO]", "");
            $templateProcessor->setValue("[NRINSCRICAOPARC]", "");
        }
    }

    /**
     * Substitui os dados de Revenda quando o respectivo campo está vazio
     */
    public function substituirDadosRevenda($revenda, $documento) {
        $posicaoIniRevenda = strpos($documento, "[INIREVENDA]");
        $posicaoFimRevenda = strpos($documento, "[FIMREVENDA]");
        
        if ((empty($revenda)) && ($posicaoIniRevenda != false) && ($posicaoFimRevenda != false) ) {
            $posicaoFimRevenda = ($posicaoFimRevenda - $posicaoIniRevenda) + 12; 
            $descRevenda = substr($documento, $posicaoIniRevenda, $posicaoFimRevenda);
            $documento =  str_replace($descRevenda, "", $documento);
        }
          
        return $documento;
    }

     /**
     * Busca os dados do Revenda da proposta.
     */
    private function getDadosRevenda($nrparcnegocio, $nrorg) {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosRevenda($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
}
?>