// FILE: ./assets/js/util/CRM_Utilities.js
function CRM_Utilities(ScreenService, templateManager, ZHPromise, restEngine, General, ValidationEngine) {

/*function CRM_Utilities(ScreenService, templateManager, Operador_Repository, ZHPromise, restEngine, General, ValidationEngine) {
*/

    /**
     * Função para limpar todos os valores de uma widget recebida
     * como parâmetro.
     * 
     * @param {obejct} widget widget na qual será removido os valores
     * de todas as fields.
     * 
    */
    this.cleanWidgetFields = function(widget){
        widget.fields.forEach(function(field){
            field.value("");    
        });   
    };
    
    
    this.CATCH_EXCEPTION_ERROR = function(error) {
        return 'Ocorreu um erro inesperado, contate o administrador informando o trecho abaixo: <br><br>' + error + '<br><br>';
    };

    
    /**
     * Função para retornar as informações do
     * operador logado.
     * 
     * @return {object} loggedOperatorInfos objeto
     * que contém todas as informações do operador
     * logado.
    */
    this.getLoggedOperatorInfos = function(){
        
        var loggedOperatorInfos = templateManager.LOGGEDOPERATOR;
        
        return loggedOperatorInfos;
            
    };
	
	/**
     * Função para setar as informações do operador 
     * logado em um array na widget recebida.
     * 
     * @param {object} destinationObject objeto destino 
     * das informações do operador logado.
     * 
    */
    /*
	this.setOperatorLoggedInfosInCurrentRow = function(destinationObject){
        
        return Operador_Repository.getOperadorLogado().then(function(operator){
        
            if(!operator[0]){
                
                ScreenService.showMessage("Não foi possível carregar os dados do operador logado. Entre em contato com um administrador sobre o erro.");
                return;
            }
        
            destinationObject.OPERATORLOGGED = operator[0];
            
            }, 
            function(error){
                ScreenService.showMessage("Não foi possível carregar os dados do operador logado. Entre em contato com um administrador sobre o erro.");
            }
        );  
    }; 
    */
    /**
     * Retorna o index da widget localizada, pelo nome
     * recebido como parâmetro da função, no array de widgets,
     * também recebido como parâmetro.
     * 
     * @param {object} widgets array de widgets onde é feito
     * a busca.
     * 
     * @ param {string} widgetName nome da widget a ser
     * localizada.
     * 
     * @param {int} indexWidgetObject index da widget localizada
     * dentro do array de wigets.
    */
    this.getWidgetPosition = function(widgets, widgetName){
        
        var indexWidgetObject;
        
        widgets.forEach(function(widget, indexWidget){
            
            if(widget.name === widgetName){
                indexWidgetObject = indexWidget;
                return;
            }
                
        });
        
        return indexWidgetObject;
            
    };
    
    /**
     * Função para identificar o index de um objeto
     * dentro de um dataSourceFilter, de acordo com 
     * seu nome.
     * 
     * @param {object} dataSourceFilter objeto da busca.
     * 
     * @param {String} name propriedade no objeto pelo qual
     * é feito a busca.
     * 
     * @return {int} indexFilterObject index Do objeto 
     * encontrado.
     * 
    */
    this.getFilterPosition = function(dataSourceFilter, name){
        var indexFilterObject;
        
        dataSourceFilter.forEach(function(value,i){
            if(value.name === name){
                indexFilterObject = i;
                return;
            }    
        });
        return indexFilterObject;
    };
    
    /**
     * Função para identificar o index de um objeto
     * dentro de um FieldGroups, de acordo com 
     * seu nome.
     * 
     * @param {object} fieldGroup objeto da busca.
     * 
     * @param {String} name propriedade no objeto 
     * pelo qual é feito a busca.
     * 
     * @return {int} indexFieldGroupObject index Do objeto 
     * encontrado.
    */
    this.getFieldGroupPosition = function(fieldGroups, name){
      
        var indexFieldGroupObject;
      
        fieldGroups.forEach(function(value, i){
            if(value.name === name){
                indexFieldGroupObject = i;
            } 
        });
        
        return indexFieldGroupObject;
        
    };
    
    /**
     * Procedimento para recuperar a posição
     * de um objeto dentro de um array.
     * 
     * @param {object} array array onde será feito
     * a varredura.
     * 
     * @param {object} objectName nome do objeto a ser
     * buscado.
    */
    this.getObjectPositionInArrayByName = function(array, objectName){
        
        var indexFieldObject;
      
        array.forEach(function(value, i){
            if(value.name === objectName){
                indexFieldObject = i;
            } 
        });
        
        return indexFieldObject;
        
    };
    
    /**
     * Limpa um field de acordo com seu template.
     * 
     * @param {object} field terá seu valor limpo
    */
    this.clearFieldValue = function(field){
        
        var templateField = field.template;
        
        if(templateField === "field/select.html"){
            
            if(field.value() && field.value() !== ""){
                
                field.clearByOutData();
            }
            
        }else{
            
            field.clearValue();
            
        }
        
    };
    
     /**
     * Função que habilita a criação de um novo código
     * para o registro que será salvo.
     * 
     * @param {object(widget)} widget widget ativa que
     * terá seu parâmetro alterados
     * 
     * @param {String} tableName tabela referência para
     * gerar o novo código
     * 
     * @param {String} tablePk nome da Primary Key da tabela
     * que recebera o código gerado.
    */
    this.enableNewNumericalCode = function(widget, tableName, tablePk){
        
        if(widget.currentRow.__is_new){
            
            var paramsNewCode = widget.paramsNewCode;
            
            paramsNewCode.__is_new_code = true;
            paramsNewCode.__new_code_pk = tableName;
            paramsNewCode.__table_pk = tablePk;
            
        }
        
    };
    
     /**
     * Define os parâmetros para gerar um novo código
     * para o registro a ser salvo.
     * 
     * @param {object} row objeto que será persistido.
     * 
     * @param {String} tableName tabela referência para
     * gerar o novo código
     * 
     * @param {String} tablePk nome da Primary Key da tabela
     * que recebera o código gerado.
     * 
     * @param {string} columnToReturn coluna a ser retornada.
    */
    this.setColumnToReturnParamsOnRow = function(row, tableName, tablePk, columnToReturn){
        
        row.__is_new_code = true;
        row.__new_code_pk = tableName;
        row.__table_pk = tablePk;
        row.__new_code_length = 20;
        row.__columns_to_return = [columnToReturn];
        
    };
    
    /**
     * Função que desabilita a criação de um novo código.
     * 
     * @param {object(widget)} widget widget ativa
     * que terá seus parâmetros desabilitados.
    */
    this.desableNewNumericalCode = function(widget){
    
        var paramsNewCode = widget.paramsNewCode;
        
        paramsNewCode.__is_new_code = false;
        paramsNewCode.__new_code_pk = "";
        paramsNewCode.__table_pk = "";
        
    };
    
    /**
     * Função para setar a coluna a ser retornada
     * na resposta do 'save'.
     * 
     * @param {object} currentRow currentRow que receberá
     * a propriedade com o nome da coluna a ser retornada.
     * 
     * @param {string} columnName nome da coluna a ser
     * retornada.
    */
    this.setColumnToReturnInCurrentRow = function(currentRow, columnName){
    
        currentRow.__columns_to_return = [columnName];

    };
    
    /**
     * Função para remover todos os caracteres
     * não numéricos de um determinado valor,
     * caso o valor recebido seja vazio, uma
     * String vazia é retornada.
     * 
     * @param {String} valueWithCurrencyMask valor que será retornado
     * sem caracteres não numéricos.
     * 
     * @return {String} valueWithoutSymbols valor
     * recebido como parâmetro sem caracteres não
     * numéricos.
     * 
    */
    this.removeValueMask = function(valueWithMask){
        
        if(valueWithMask){
        
            var valueWithoutSymbols = valueWithMask.replace(/[^0-9]/g, "");
            
            return valueWithoutSymbols;
        
        }else{
            
            return "";
            
        }
    };
    
    /**
     * Função para removar mascarás de moedas do valor
     * recebido como parâmetro.
     * 
     * @param {String} valueWithCurrencyMask valor que 
     * será retornado sem máscara de moeda.
     * 
     * @return {String} valueWithoutSymbols valor
     * recebido como parâmetro sem máscara de moeda.
     * 
    */
    this.removeValueCurrencyMask = function(valueWithCurrencyMask){
        
        if(valueWithCurrencyMask){
            if(typeof valueWithCurrencyMask === "string") {
                var valuesWithoutCurrencyMask = $.currency.unmaskString(valueWithCurrencyMask, {"thousands": "."});    
                return valuesWithoutCurrencyMask;
            }else{
                //Retorna o pŕoprio valor se não vier uma string
                return valueWithCurrencyMask;
            }
        }else{
            
            return "";
            
        }
        
            
    };
    
    /**
     * Função para localizar uma widget dentro
     * de um array de widgets pelo seu nome;
     * 
     * @param {object} widgets arrays de widgets, que
     * será percorrido pela busca.
     * 
     * @param {string} widgetName nome da widget a ser
     * procurada.
     * 
     * @return {object} foundWidget widget encontrada
     * no array de widgets recebido como parâmetro.
    */
    this.getWidgetByName = function(widgets, widgetName){
        
        var foundWidget;
        
        widgets.map(function(widget){
            
            if(widget.name === widgetName){
                foundWidget = widget;
            }
            else if(widget.widgets && widget.widgets.length > 0 && !foundWidget){
                
                foundWidget = this.getWidgetByName(widget.widgets, widgetName);
                
            }
            
        }.bind(this));
        
        return foundWidget;
        
    };
    
    /**
     * Função para setar o filtro do número da organização
     * padrão e do número da organização do operador logado
     * na field recebida como parâmetro, e logo em seguida,
     * recarregar a field.
     * 
     * @param {object} field field que receberá o filtro.
    */
    this.setDefaultOrgFilterAndOperatorOrgFilter = function(field){
        
        var fieldFilter = field.dataSourceFilter;
        var loggedOperatorInfos = this.getLoggedOperatorInfos();
        var valueWithDefaultOrgAndJobOrg = [getLocalVar("CRM_NRORGPADRAO"), loggedOperatorInfos.NRORGTRAB];
        
        if(!fieldFilter){
            
            field.dataSourceFilter = [];
            fieldFilter = field.dataSourceFilter;
            
        }
        
        var nrorgFilterIndex = this.getFilterPosition(fieldFilter, "NRORG");
        
        if(nrorgFilterIndex >= 0){
            fieldFilter[nrorgFilterIndex].value = valueWithDefaultOrgAndJobOrg;
        }else{
            fieldFilter.push(
                {
                    "name": "NRORG",
                    "operator": "IN",
                    "value": valueWithDefaultOrgAndJobOrg
                }
            );
        }
        
    };
    
    /**
     * Função para setar o filtro de organização padrão
     * para atribuir o valor do parâmetro 'P_NRORGPADRAO'
     * nas querys do dataSource em caso.
     * 
     * @param {object} filterDestination destino onde o filtro
     * será definido.
    */
    this.setDefaultOrgNumberFilterValue = function(filterDestination){
        
        var filterDestinationFilter = filterDestination.dataSourceFilter;
        
        if(!filterDestinationFilter){
            
            filterDestination.dataSourceFilter = [];
            filterDestinationFilter = filterDestination.dataSourceFilter;
            
        }
        
        filterDestinationFilter.push(
            {
                "name": "P_NRORGPADRAO",
                "operator": "IN",
                "value": [getLocalVar("CRM_NRORGPADRAO")]
            }
        );
        
    };
    
    /**
     * Função para remover um objeto do array(filtro)
     * recebido como parâmetro.
     * 
     * @param {object} filter array com os objetos(filtros).
     * 
     * @param {int} objectIndex índice do objecto a ser removido
     * do array(filtro).
    */
    this.deleteFilterObject = function(filter, objectIndex){
        
        if(objectIndex === 0){
            filter.shift();    
        }else if(objectIndex === filter.length - 1){
            filter.pop();
        }else{
            filter = filter.slice(0,objectIndex).concat(filter.slice(objectIndex + 1));    
        }
        
        return filter;
    };
   
    /**
     * Função para retornar a data e hora atual.
     * 
     * @return {string} data e hora atual.
    */
    this.getDataHoraAtual = function() {
        return moment().format('DD/MM/YYYY HH:mm');            
    };
    
    /**
     * Função para retornar a data atual.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDate = function(){
        return moment().format('DD/MM/YYYY');
    };
    
    /**
     * Função para retornar a hora atual.
     * 
     * @return {string} hora atual.
    */
    this.getCurrentTime = function(){
        return moment().format('HH:ss');
    };
    
    /**
     * Função para retornar a data atual com
     * o horário máximo.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDateWithMaxTime = function(){
        return moment().format('DD/MM/YYYY 23:59');
    };
    
    /**
     * Função para retornar a data atual com
     * o horário mínimo.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDateWithMinTime = function(){
        return moment().format('DD/MM/YYYY 00:00');
    };
    
    /**
     * Procedimento para retornar uma data
     * subtraida pela quantidade de dias recebida
     * como parâmetro.
     * 
     * @param {string} date data s ser subtraida.
     * 
     * @param {number} numberOfDays dias como subtraendo.
     * 
     * @return {string} data subtraida.
    */
    this.subtractDateByDays = function(date,numberOfDays){
        return moment(date, 'DD/MM/YYYY').subtract(numberOfDays, 'day').format('DD/MM/YYYY');
    };
    
    /**
     * Procedimento para retornar uma data
     * subtraida pela quantidade de mêses recebido
     * como parâmetro.
     * 
     * @param {string} date data s ser subtraida.
     * 
     * @param {number} numberOfMonth mêses como subtraendo.
     * 
     * @return {string} data subtraida.
    */
    this.subtractDateByMonth = function(date,numberOfMonth){
        return moment(date, 'DD/MM/YYYY').subtract(numberOfMonth, 'month').format('DD/MM/YYYY');
    };
    
    /**
     * Adiciona um número de dias a uma data.
     * 
     * @param {string} date data se ser modificada.
     * 
     * @param {number} numberOfDays número de dias a ser
     * adicionado.
     * 
     * @param {string} format formato da data recebida.
     * 
     * @return  {string} data com os dias adicionado.
    */
    this.addDaysToDate = function(date, numberOfDays, format){
        return moment(date, format).add(numberOfDays, 'day').format(format);
    };
    
    /**
     * Retornar a data de uma string no formato
     * 'dateTime'.
     * 
     * @param {string} dateTime string no formato
     * 'dateTime'.
    */
    this.getDateInDateTime = function(dateTime){
        return moment(dateTime, 'DD/MM/YYYY HH:ss').format('DD/MM/YYYY');
    };
    
    /**
     * Retornar o horário de uma string no formato
     * 'dateTime'.
     * 
     * @param {string} dateTime string no formato
     * 'dateTime'.
    */
    this.getTimeInDateTime = function(dateTime){
        return moment(dateTime, 'DD/MM/YYYY HH:ss').format('HH:ss');
    };
    
    /**
     * Altera o formato da data de acordo com o
     * formato recebido como parâmetro.
     * 
     * @param {string} date data que será modificada.
     * 
     * @param {string} currentFormat formato atual da
     * data recebida.
     * 
     * @param {string} newFormat formato que será aplicado
     * na data recebida.
     * 
     * @return {string} data recebida formatada de acordo com
     * o novo padrão também recebido.
    */
    this.changeDateFormat = function(date, currentFormat, newFormat){
        return moment(date, currentFormat).format('YYYY-MM-DD');
    };
    
    /**
     * Função para filtrar os valores de um
     * campo de localização pelo seu local de origem
     * (Ex.:Minas Gerais (estado selecionado)
     * Cidades disponíveis: Contagem, Belo Horizonte, Betin).
     * 
     * @param {object} widget widget onde se localiza o campo
     * de localidade.
     * 
     * @param {object} field campo de localidade que receberá
     * o filtro pelo seu local de origem.
    */
    this.setPlaceFilterByPlaceOrigin = function(widget, field){
        
        var simplifiedFieldName = this.getPartOfTheExpressionFromLeftToRight(field.name, 4);
        var currentRow = widget.currentRow;
        var filters = [];
        var filterReference;
        
        switch(simplifiedFieldName){
            
            case "NMES":
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
            case "NMMU":
                filterReference = "SGESTADO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.SGESTADO));
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
            case "NMBA":
                filterReference = "CDMUNICIPIO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDMUNICIPIO));
                filterReference = "SGESTADO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.SGESTADO));
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
        }
        
        if(filters){
            filters.forEach(function(filter){
                var placeNameFilterIndex = this.getFilterPosition(field.dataSourceFilter, filter.name);
            
                if(placeNameFilterIndex >= 0){
                    field.dataSourceFilter[placeNameFilterIndex].value = currentRow[filter.name];
                }else{
                    field.dataSourceFilter.push(filter);   
                }
            }.bind(this));
        }
    };
    
    /**
     * Função que recebe uma expressão e retorna
     * a mesma expressão simplicada(o número de 
     * caracteres recebido como parâmetro, começando
     * da esquerda para a direita.)
     * 
     * @param {string} string expressão a ser simplificada.
     * 
     * @para {number} numbersOfChars número de caracteres
     * que será retornado da expressão recebida com parâmetro.
    */
    this.getPartOfTheExpressionFromLeftToRight = function(expression, numbersOfChars){
        
        eval('var regEx = /(\\w{'+numbersOfChars+'})(\\w*)/');
        
        var newExpression = expression.replace(regEx, "$1");
        return newExpression;
    };
    
    /**
     * Função que retorna um 'objeto filtro'
     * com o operador de igualdade.
     * 
     * @param {string} name nome de referência para
     * o filtro.
     * 
     * @param {string/number} value valor buscado pelo
     * filtro.
    */
    this.createFilterWithEqualityOperator = function(name,value){
        
        var filter = {
            "name": name,
            "operator": "=",
            "value": value
        };
            
        return filter;
    };

    /**
     * Valida a inscrição informada pelo usuário
     * e seta na currentRow da widget o código do
     * tipo da inscrição e a máscara da inscrição,
     * caso a inscrição seja inválida
     * ele limpa os campos e manda uma mensagem.
     * 
     * @param {object} widget        : widget que possui a currentRow
     *                                  a ser modificada
     * 
     * @param {string} inscricao     : string com o nome do field 
     *                                  que possui a Inscrição
     * 
     * @param {string} tipoInscricao : string com o nome do field 
     *                                  do tipo de inscrição
    */
    this.validaInscricao = function(widget, inscricao, tipoInscricao){
        
        var currentRow = widget.currentRow;
        
        var resultValidate = General.inscricaoFormat(currentRow[inscricao]);
        
        if(resultValidate.valid && resultValidate.value){
            
            currentRow[tipoInscricao] = resultValidate.tipo.cod;
            
            currentRow[inscricao] = resultValidate.value;
            
        }else{
            
            ScreenService.showMessage('Número de inscrição inválido');
            
            currentRow[inscricao]     = "";
            currentRow[tipoInscricao] = "";
        }    
        
    };
    
    this.validaCPF = function(widget, inscricao){
        
        var currentRow = widget.currentRow;
        
        var resultValidate = General.validateCPF(currentRow[inscricao]);
        
        if(resultValidate){
            resultValidate = General.inscricaoFormat(currentRow[inscricao]);
            currentRow[inscricao] = resultValidate.value;
            
        }else{
            
            ScreenService.showMessage('CPF inválido');
            
            currentRow[inscricao]     = "";
        }    
        
    };
    
    /**
     * Busca as parametrizações do sistema
     * de acordo com o número da organização
     * do operador logado no template Manager.
     * 
     * @return {object} array com as parametrizações
     * da organização do operador logado.
    */
    this.getCrmParametrization = function(){
        
        return templateManager.CRM_PARAMETRIZACAO; 
      
    };
    
    /**
     * Esconde as ações funcionais do container
     * ativo.
    */
    this.hideFunctionalActions = function(){
        
        var containerActions = templateManager.container.actions;
        
        containerActions.forEach(function(action){
                
            if(action.label !== "Voltar"){
                action.isVisible = false;
                action.disable = true;
            }
            
        });    
    
    };
    
    /**
     * Retorna as configurações padrão das ações
     * do container ativo.
     * 
     * @param {object} widget em caso.
    */
    this.returnContainerDefaultActions = function(widget){
        
        var containerActions = templateManager.container.actions;

        containerActions.forEach(function(action){
            
            if(Boolean(action.disable)){
                action.isVisible = true;
                delete action.disable;
            }
            
        });
        
    };
    
    /**
     * Define o filtro(regra) nos fields definidos de acordo
     * com a revenda do operador logado.
     * 
     * @param {object} objects array com os objetos a receberem
     * a regra.
    */
    this.setResaleRulesInRestrictObjectsByLoggedOperatorResale = function(objects) {
        
        objects.forEach(function(field){
            this.setObjectFilterWhenResaleLoggedOperator(field);    
        }.bind(this));
       
    };
    
    /**
     * Defini os filtros de revenda do obejto recebido
     * como parâmetro (field || widget).
     * 
     * @param {object} objeto object a ser modificado.
    */
    this.setObjectFilterWhenResaleLoggedOperator = function(object) {
        
        if(this.isResaleLoggedOperator()) {
            
            this.changeObjectOperTypeFilterToResaleOper(object);
            this.setObjectResaleFilter(object);
        }
        
    };
    
    /**
     * Retorna um valor lógico apontando se o operador
     * logado pertence a uma revenda.
     * 
    */
    this.isResaleLoggedOperator = function(){
        
        var loggedOperatorInfos = this.getLoggedOperatorInfos();
        
        return (Boolean(loggedOperatorInfos.NRREVENDA) && loggedOperatorInfos.IDTPOPERADOR === 'R');
        
    };
    
    /**
     * Altera o filtro de tipo de operador do objeto
     * recebido como parâmetro.
     * 
     * @param {object} object objeto a ser modificado.
    */
    this.changeObjectOperTypeFilterToResaleOper = function(object) {
        
        var objectFilter = object.dataSourceFilter;
        var idtooperadorFilterIndex = this.getObjectPositionInArrayByName(objectFilter, 'IDTPOPERADOR');
        var resaleTypeOperator = 'R';
        
        if(idtooperadorFilterIndex === 0 || Boolean(idtooperadorFilterIndex)){
            objectFilter[idtooperadorFilterIndex].value = resaleTypeOperator;    
        }
      
    };
    
    /**
     * Define um filtro no objeto recebido como parâmetro
     * atribuindo o número da revenda o qual o operador
     * logado pertence.
     * 
     * @param {object} object objeto a ser modificado.
    */
    this.setObjectResaleFilter = function(object) {
        
        var loggedOperatorInfos = this.getLoggedOperatorInfos();    
        
        var resaleFilter = {
            name: "NRREVENDA",
            operator: "=",
            value: loggedOperatorInfos.NRREVENDA
        };
        
        object.dataSourceFilter.push(resaleFilter);
        
    };
    
        
    /*
     * Deixa primeira letra das palavras de uma frase maíusculas
    */
    this.capitalizeFirstLetter = function(string){
        string = string.split(' ');
        string.forEach(function(partOfString, index, string){
            partOfString = partOfString.charAt(0).toUpperCase() + partOfString.toLowerCase().slice(1);    
            string[index] = partOfString;
        });
        return string.join(' ');
    };
    
    /*
     * registra a validação do tipo 'phone'
    */
    this.registerPhoneValidation = function(){
        
        var phoneValidation = function(value, validationAttrs) {
            var retorno;
            if(value === null){
                retorno =  {"valid":true};
            }else{    
                var numberSizeReceived = value.replace(/[^0-9]/g,'').length;
                retorno = {
                    "valid": (numberSizeReceived === 10 || numberSizeReceived === 11 || numberSizeReceived === 12 || numberSizeReceived === 0),
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Invalid phone.")
                };    
            }
            return retorno;
        };
            	      
        ValidationEngine.registerValidation('phone', phoneValidation);
    };
    
    this.registerMinCurrencyValidation = function(){
        
        var minCurrency = function(value, validationAttrs) {
            var limit = parseFloat(validationAttrs.limit);
    		value = parseFloat(value.replace(/[^0-9]/g, '').slice(0, -2));
    		
    		return {
    			"valid": limit <= value || (isNaN(limit) && isNaN(value)),
    			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Minimum value: ") + validationAttrs.limit
    		};
        };
            	      
        ValidationEngine.registerValidation('minCurrency', minCurrency);
    };
    
    /**
     * Inicializa as actions da widget e suas 
     * filhas recursivamente.
     * 
     * @param {object} widget widget de evolução.
     * 
    */    
    this.factoryWidgetActions = function(widget){
        if(!Util.isEmptyOrBlank(widget.widgets)){
            widget.widgets.forEach(function(inWidget){
                this.factoryWidgetActions(inWidget);
            }.bind(this));
        }
        widget.actions.forEach(function(action){
           metaDataFactory.actionFactory(action, widget);
        });
        widget.activate();
    };
    
    /**
     * Seta o datasource filter das tabs dinamicas 
     * baseado num campo da widget principal.
     * 
     * @param {object} widget widget dinamica
     * @param {object} widget widget principal
    */
    this.setDataSourceFilterOnDynamicTabs = function(tabWidget, activeWidget){
        
        var NRSEQCONTATO = activeWidget.currentRow.NRSEQCONTATO;
        tabWidget.dataSourceFilter[0].value = NRSEQCONTATO;
        if(Util.isEmptyOrBlank(tabWidget.currentRow.NRSEQCONTATO))
            tabWidget.currentRow.NRSEQCONTATO = NRSEQCONTATO;
        tabWidget.reload();
        
    };
    
    /**
     * Seta um elemento em uma posição definida dentro de um
     * array.
     * 
     * @param {all} element elemento a ser inserido no array.
     * 
     * @param {object} array array a ser modificado.
     * 
     * @param {int} position posição que o elemento entrará no
     * array.
     * 
     * @return {object} array modificado.
    */
    this.setElementOnArrayDefinedPosition = function(element, array, position){
        
        if(!Boolean(position)){
            array.push(element);
        } else {
            array.splice(position - 1, 0, element);    
        }
        
        return array;
        
    };
    
    /**
     * Verifica se a primeira data recebida como parâmtro
     * é maior que a segunda data, também recebida como
     * parâmetro.
     * Valida um intervalo de datas.
     * 
     * @param {string} initDate data inicial do intervalo.
     * 
     * @param {string} endDate data final do intervalo.
     * 
     * @param {string} format formato das datas recebidas.
     * 
     * @retur {boolean} valor lógico indicando se a primeira
     * data é maior ou não que a segunda.
    */
    this.validateIntervalDates = function(initDate, endDate, format){
        
        if(moment(initDate, format).format(format) > moment(endDate, format).format(format)){
            return false;
        }
        
        return true;
    };
    
    /**
     * Define a propriedade 'readOnly' do campo recebido como
     * parâmetro.
     * 
     * @param {object} field campo a ser modificado.
     * 
     * @param {boolean} isReadOnly valor lógico definindo a 
     * propriedade do campo.
    */
    this.setFieldReadOnlyProperty = function(field, isReadOnly){
        field.readOnly = isReadOnly;  
    };
    
    /**
     * Aplica o método de inserção na widget recebida como parâmetro e
     * seta na currentRow a coluna retornada.
     * 
     * @param {object} widgtet widget que sofrerá a rotina de inserção.
     * 
     * @param {string} columnToReturn coluna a ser retornada na currentRow.
    */
    this.saveAndSetColumnToBeRetunrOnCurrentRow = function(widget, columnToReturn){
        
        var currentRow = widget.currentRow;
        
        this.setColumnToReturnInCurrentRow(currentRow, columnToReturn);
        
        return General.widgetSave(widget).then(function(response){
            currentRow[columnToReturn] = response.dataset.columnsToReturn[0][columnToReturn];    
        });
        
    };
    
    /**
     * Sobreescreve a função relacionada a uma ação
     * da widget recebida como parâmetro.
     * 
     * @param {object} widget widget que terá sua ação
     * modificadas.
     * 
     * @param {string} actionName nome da ação a ser 
     * modificada.
     * 
     * @param {string} functionCall chamada da função a ser colocada
     * na action.
     * EX.: ExemploController.exmploFuncao();
     * Vale lembrar que a chamada da função é atribuida no JSON da
     * widget, sendo assim, os parâmetros devem seguir as regras de
     * chamdas em JSON.
    */
    this.overwriteAction = function(widget, actionName, functionCall){
        
        if(!Boolean(widget.getAction(actionName).events[0].defaultCode)){
            widget.getAction(actionName).events[0].defaultCode = widget.getAction(actionName).events[0].code;
        }
        
        widget.getAction(actionName).events[0].code = functionCall;
        
    };
    
    /**
     * Retorna a função padrão de uma ação (quando a mesma
     * foi sobrescrita).
     * 
     * @param {object} widget widget que terá sua ação 
     * modificada.
     * 
     * @param {string} actionName nome da ação a ser 
     * modificada.
    */
    this.setActionDefaultCode = function(widget, actionName){
        
        widget.getAction(actionName).events[0].code = widget.getAction(actionName).events[0].defaultCode;
        
    };
    /* Função que verifica se o caractere digitado é do tipo NUMBER ou STRING,
    caso seja STRING, substitui o valor para vazio, caso seja numero, passa o valor digitado*/ 
    this.onlyNumbers = function(field) {
		const fieldValue = field.value();
		if (fieldValue) {
			var fieldElements = fieldValue.split("");
			fieldElements.forEach(function (value, index) {
				if (isNaN(parseInt(value))) {
					fieldElements.splice(index,1);
					field.value(fieldElements.join(''));
				}
			});
		}
	};
	
	this.changeVisibilityOrgTrab = function(nrOrgRegistro, widget, actions, fields) {

        var visible = true;
        var nrOrgTrab = getLocalVar("CRM_NRORGTRAB");

        if (nrOrgRegistro == nrOrgTrab || widget.currentRow.__is_new) {
            visible = true;
        }
        else {
            visible = false;
        }

        if (!Util.isUndefinedOrNull(actions)) {
            actions.forEach(function(action) {
                if (widget.getAction(action)) {
                    widget.getAction(action).isVisible = visible;
                }
            });
        }

        if (!Util.isUndefinedOrNull(fields)) {
            fields.forEach(function(field) {
                if (widget.getField(field)) {
                    widget.getField(field).readOnly = !visible;
                }
            });
        }
    };
    
    /**
     * Faz com que o select-native respeite o outData, setando 
     * os capos na currentRow
    */
    this.outDataNativeFields = function(args){
        var field      = args.owner.field;
	    field.dataSource.data.forEach(function(option){
	        if(option[field.valueField] === args.row[field.name]){
	            //Limpa a antiga row
	            Object.keys(field.outData).forEach(function(key){
                    if(args.row[field.outData[key]] !== undefined) delete args.row[field.outData[key]];    	                     
	            });
	            Object.keys(option).forEach(function(key){
	                if(field.outData[key] !== undefined){
	                    args.row[field.outData[key]] = option[key];
	                }
	            });
	        }      
	    });
	};
	
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('CRM_Utilities', CRM_Utilities);
});

// FILE: ./assets/js/util/PartnerInfosModalController.js
function PartnerInfosModalController(CRM_Utilities, ScreenService, templateManager, metaDataFactory){
    
    /**
     * Builda o modal.
     * 
     * @param {object} widget widget que irá receber o modal.
     * 
     * @param {string} nrparcnegocio nome do campo com o número
     * do parceiro de negócio.
     * 
     * @return widget modal construida.
     * 
    */
    this.factoryModalWidget = function(activeWidget, nrparcnegocio) {
        
        /*Modificado em 22/09 por Arthur
            Adicionada propriedade 'preventHide',
            que garante que a widget Dados do Parceiro 
            não tenha suas actions setadas como invisíveis em chamados já 
            concluidos
        */
        var modalWidgetJson = {
            "id":"17223197583113",
            "name": "WDGT_MODAL_PARTNER_INFOS",
            "label": "Informações do Parceiro",
            "container": activeWidget.container,
            "parent": activeWidget,
            "template": "widget/tabbedWidget.html",
            "defaultTab": "WDGT_PARTNER_DATA",
            "showAsModal": true,
            "preventHide":true,
            "showAsTab":false,
            "isVisible": true,
            "events":[
                {
                    "id":"17223197583113124",
                    "name":"WidgetOnEnterOnce",
                    "code":"AbaInterlocutorController.setInterlocutorWidgetOnActiveWidget(args.owner);ABALogotipoParceiroController.setLogoWidgetOnActiveWidget(args.owner, 5);PartnerInfosModalController.changeTemplateTabInterlocutor(args.owner);"
                }
            ],
            "widgets":[
                {
                    "id":"17223197583113123",
                    "name": "WDGT_PARTNER_DATA",
                    "label": "Dados",
                    "container": activeWidget.container,
                    "template": "widget/form-view.html",
                    "isVisible": true,
                    "dataSource": {
                        "name": "product#/crm_candidato_prospect_parcnegocio",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id":"17223197583113125",
                            "name": "WidgetOnEnter",
                            "code": "PartnerInfosModalController.setModalFilterByActivePartner(args.owner.parent, templateManager.container.getWidget('" + activeWidget.name + "'), '" + nrparcnegocio + "');"
                        },
                        {
                            "id":"17223197583113126",
                            "name": "WidgetAfterLoadDataSource",
                            "code": "args.owner.moveToFirst();"
                        }
                    ],
                    "fields": [
                        {
                            "id":"17223197583113127",
                            "name": "NOMEPRINCIPAL",
                            "label": "Nome",
                            "template": "field/text-edit.html",
                            "class": "12",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113128",
                            "name": "NOMEFANTASIA",
                            "label": "Nome Fantasia",
                            "template": "field/text-edit.html",
                            "class": "12",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113129",
                            "name": "INSCRICAO",
                            "label": "Inscrição",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113130",
                            "name": "TIPOINSC",
                            "label": "Tipo Inscrição",
                            "template": "field/select-native.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true,
                            "valueField": "value",
                            "descriptionField": "name",
                            "validations": {},
                            "events": [],
                            "dataSource": {
                                "data": [
                                    {
                                        "name": "Física",
                                        "value": "F"
                                    },
                                    {
                                        "name": "Jurídica",
                                        "value": "J"
                                    },
                                    {
                                        "name": "Outros",
                                        "value": "O"
                                    },
                                    {
                                        "name": "Inscrição",
                                        "value": "I"
                                    }
                                ]
                            }
                        },
                        {
                            "id":"17223197583113131",
                            "name": "NMOPERRESPONSAVEL",
                            "label": "Operador Responsável",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113132",
                            "name": "TIPOPARCEIRO",
                            "label": "Tipo Parceiro",
                            "template": "field/select-native.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true,
                            "valueField": "value",
                            "descriptionField": "name",
                            "dataSource": {
                                "data": [
                                    {
                                        "name": "Lead",
                                        "value": "L"
                                    },
                                    {
                                        "name": "Prospect",
                                        "value": "P"
                                    },
                                    {
                                        "name": "Cliente",
                                        "value": "C"
                                    }
                                ]
                            }
                        },
                        {
                            "id":"17223197583113133",
                            "name": "DSSEGMER",
                            "label": "Segmento de Mercado",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113134",
                            "name": "DSATIVCLI",
                            "label": "Ramo de Atividade",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113135",
                            "name": "DSTPATIVCLIENTE",
                            "label": "Tipo de Atividade",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113136",
                            "name": "NMMODODIVULG",
                            "label": "Como Chegou Até Nós",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        },
                        {
                            "id":"17223197583113137",
                            "name": "IDATIVO",
                            "label": "Ativo",
                            "template": "field/select-native.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true,
                            "valueField": "value",
                            "descriptionField": "name",
                            "validations": {},
                            "dataSource": {
                                "data": [
                                    {
                                        "name": "Sim",
                                        "value": "S"
                                    },
                                    {
                                        "name": "Não",
                                        "value": "N"
                                    }
                                ]
                            }
                        },
                        {
                            "id":"17223197583113138",
                            "name": "NRFUNCIONARIOS",
                            "label": "Nr. Funcionários",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnView": true
                        }
                    ],
                    "widgets":[]
                },
                {
                    "id": "14418982341257778598851",
                    "name": "WDGT_DADOS_ADICIONAIS",
                    "container": activeWidget.container,
                    "label": "Dados Adicionais",
                    "showCheckbox": false,
                    "isVisible": true,
                    "itemsPerPage": 30,
                    "template": "widget/form-view.html",
                    "templateToForm":"widget/form.html",
                    "showDescriptionOrder": false,
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "product#/crud/ger_parcnegocioinfo",
                        "itemsPerPage": 100
                    },
                    "dataSourceFilter": [
                        {
                            "name": "P_NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        },
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": "@templateManager.LOGGEDOPERATOR.NRORG@"
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id": "14418982343495474723876",
                            "name": "WidgetOnEnter",
                            "code": "PartnerInfosModalController.setCurrentRowDadosAdicionais(args.owner)"
                        }
                    ],
                    "fields": [   
                        {
                            "id":"17223197583113139",
                            "name": "NMRESPGERAL",
                            "label": "Responsável Geral",
                            "class":"12",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113140",
                            "name": "NMPONTOATENDIMENTO",
                            "label": "Ponto de Atendimento",
                            "class":"12",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113141",
                            "name": "NRPONTUACAO",
                            "label": "Pontuação",
                            "class":"6",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113142",
                            "name": "CDINTEGRACAO",
                            "label": "Núm. Integração",
                            "class":"6",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113143",
                            "name": "DTINIRELACIONA",
                            "label": "Data de Início do Relacionamento",
                            "class":"6",
                            "isVisible": true,
                            "showOnForm": true,
                            "dateFormat": "DD/MM/YYYY",
                            "template": "field/calendar.html"
                        },
                        {
                            "id":"17223197583113143",
                            "name": "NMMODORECEBINFO",
                            "label": "Como Deseja Receber Informações",
                            "class":"6",
                            "isVisible": true,
                            "showOnForm": true,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "widgets": []
                },
                {
                    "id":"17223197583113144",
                    "name": "WDGT_PARTNER_ADDRESS",
                    "label": "Endereços",
                    "container": activeWidget.container,
                    "template": "widget/grid.html",
                    "isVisible": true,
                    "dataSource": {
                        "name": "product#/crud/enderecoparc_localizacao_tipoendereco",
                        "rest": true,
                        "itemsPerPage": 50,
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id":"17223197583113144",
                            "name": "WidgetOnEnter",
                            "code": "args.owner.reload();PartnerInfosModalController.resizeWindow()"
                        }
                    ],
                    "fields": [
                        {
                            "id":"17223197583113145",
                            "label": "Tipo de Endereço",
                            "name": "NMTIPOENDERECO",
                            "template": "field/text-edit.html",
                            "isVisible": true,
                            "showOnForm": true,
                            "showOnList": true,
                            "valueField": "CDTIPOENDERECO",
                            "descriptionField": "NMTIPOENDERECO",
                            "dataSource": {},
                            "events": []
                        },
                        {
                            "id":"17223197583113146",
                            "name": "NRCEPENDERECO",
                            "label": "CEP",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnForm": true,
                            "events": [],
                            "mask": {
                                "type": "fix",
                                "params": {
                                    "cleanMaskOnModel": true,
                                    "mask": "99999-999"
                                }
                            }
                        },
                        {
                            "id":"17223197583113147",
                            "name": "DSENDERECO",
                            "label": "Endereço",
                            "validations": {
                                "required": true
                            },
                            "maxlength": 250,
                            "class": "10",
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113147",
                            "name": "NRIMOVELENDERECO",
                            "label": "Número",
                            "class": "2",
                            "maxlength": 10,
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113148",
                            "name": "DSCOMPLEENDERECO",
                            "label": "Complemento",
                            "class": "12",
                            "maxlength": 100,
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113149",
                            "name": "NMPAIS",
                            "label": "País",
                            "template": "field/select.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "dataSource": {
                                "name": "product#/pais",
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [],
                            "outData": {
                                "CDPAIS": "CDPAIS",
                                "NMPAIS": "NMPAIS"
                            },
                            "fields": [
                                {
                                    "id":"17223197583113150",
                                    "label": "Código",
                                    "name": "CDPAIS",
                                    "class": "4",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                },
                                {
                                    "id":"17223197583113151",
                                    "label": "País",
                                    "name": "NMPAIS",
                                    "class": "8",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                }
                            ],
                            "events": []
                        },
                        {
                            "id":"17223197583113152",
                            "name": "NMESTADO",
                            "label": "Estado",
                            "template": "field/select.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "dataSource": {
                                "name": "product#/estado",
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [],
                            "outData": {
                                "SGESTADO": "SGESTADO",
                                "NMESTADO": "NMESTADO"
                            },
                            "fields": [
                                {
                                    "id":"17223197583113153",
                                    "label": "Sigla",
                                    "name": "SGESTADO",
                                    "class": "4",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                },
                                {
                                    "id":"17223197583113153",
                                    "label": "Estado",
                                    "name": "NMESTADO",
                                    "class": "8",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                }
                            ],
                            "events": []
                        },
                        {
                            "id":"17223197583113154",
                            "name": "NMMUNICIPIO",
                            "label": "Município",
                            "template": "field/select.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "dataSource": {
                                "name": "product#/municipio",
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [],
                            "outData": {
                                "CDMUNICIPIO": "CDMUNICIPIO",
                                "NMMUNICIPIO": "NMMUNICIPIO"
                            },
                            "fields": [
                                {
                                    "id":"17223197583113155",
                                    "label": "Código",
                                    "name": "CDMUNICIPIO",
                                    "class": "4",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                },
                                {
                                    "id":"17223197583113156",
                                    "label": "Município",
                                    "name": "NMMUNICIPIO",
                                    "class": "8",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                }
                            ],
                            "events": []
                        },
                        {
                            "id":"17223197583113157",
                            "name": "NMBAIRRO",
                            "label": "Bairro",
                            "template": "field/select.html",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "dataSource": {
                                "name": "product#/bairro",
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [],
                            "outData": {
                                "CDBAIRRO": "CDBAIRRO",
                                "NMBAIRRO": "NMBAIRRO"
                            },
                            "events": []
                        }
                    ],
                    "widgets": []
                },
                {
                    "id":"17223197583113158",
                    "name": "WDGT_PARTNER_CONTACTS",
                    "label": "Contatos",
                    "container": activeWidget.container,
                    "template": "widget/grid.html",
                    "isVisible": false,
                    "dataSource": {
                        "rest": true,
                        "name": "product#/crm_relacionaparc",
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORG')@"
                        },
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORG')@"
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id":"17223197583113159",
                            "name": "WidgetOnEnter",
                            "code": "args.owner.reload();PartnerInfosModalController.resizeWindow()"
                        }
                    ],
                    "fields": [
                        {
                            "id":"17223197583113160",
                            "name": "NOMECONT",
                            "label": "Nome",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnForm": true,
                            "validations": {
                                "required": true
                            },
                            "class": 10,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id":"17223197583113161",
                            "name": "EMAIL",
                            "label": "Email",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "showOnForm": true,
                            "class": 6,
                            "template": "field/text-edit.html",
                            "events": []
                        },
                        {
                            "id":"17223197583113162",
                            "name": "TELEFONE",
                            "label": "Telefone",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "showOnForm": true,
                            "class": 6,
                            "template": "field/text-edit.html",
                            "events": []
                        },
                        {
                            "id":"17223197583113163",
                            "name": "NMTIPOPARCEIRO",
                            "label": "Tipo de Parceiro",
                            "template": "field/select.html",
                            "isVisible": true,
                            "showOnList": true,
                            "class": "12",
                            "valueField": "TIPODEPARCEIRO",
                            "descriptionField": "NMTIPODEPARCEIRO",
                            "dataSource": {
                                "rest": true,
                                "name": "product#/crm_tipoParceiro",
                                "lazyLoad": true
                            },
                            "outData": {
                                "NMTIPODEPARCEIRO": "NMTIPODEPARCEIRO",
                                "TIPODEPARCEIRO": "TIPODEPARCEIRO"
                            },
                            "dataSourceFilter": [],
                            "events": []
                        }
                    ],
                    "widgets": []
                },
                {
                    "id": "14418982341257778598852",
                    "name": "WDGT_PARTNER_COMU",
                    "container": activeWidget.container,
                    "label": "Comunicação",
                    "showCheckbox": false,
                    "isVisible": true,
                    "itemsPerPage": 30,
                    "template": "widget/master_detail/grid.html",
                    "showDescriptionOrder": false,
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "product#/crm_comunicaparc",
                        "itemsPerPage": 100
                    },
                    "dataSourceFilter": [
                        {
                            "name": "P_NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        },
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": "@templateManager.LOGGEDOPERATOR.NRORG@"
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@templateManager.LOGGEDOPERATOR.NRORG@"
                        }
                    ],
                    "actions": [],
                    "events": [
                        {
                            "id": "14418982343495474723875",
                            "name": "WidgetOnEnter",
                            "code": "args.owner.reload();PartnerInfosModalController.resizeWindow()"
                        }
                    ],
                    "fields": [
                        {
                            "id": "14418982341628670838852",
                            "name": "IDATIVO",
                            "label": "Ativo",
                            "isVisible": false,
                            "showOnList": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982342800647217853",
                            "name": "NRCOMUNICAPARC",
                            "label": "NRCOMUNICAPARC",
                            "isVisible": false,
                            "showOnList": false,
                            "showOnForm": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982341061414688854",
                            "name": "NRPARCNEGOCIO",
                            "label": "NRPARCNEGOCIO",
                            "isVisible": false,
                            "showOnList": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982344140770823855",
                            "name": "NRENDERECOPARC",
                            "label": "NRENDERECOPARC",
                            "isVisible": false,
                            "showOnList": false,
                            "showOnForm": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "1441898234664443810856",
                            "name": "CDCOMPLECOMUPARC",
                            "label": "CDCOMPLECOMUPARC",
                            "isVisible": false,
                            "showOnList": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982341567386213857",
                            "name": "CDFORMACOMU",
                            "label": "Forma de Comunicação",
                            "isVisible": false,
                            "showOnList": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "1441898234614649406858",
                            "name": "NMFORMACOMU",
                            "label": "Forma de Comunicação",
                            "class": "12",
                            "order": "CDFORMACOMU",
                            "maxlength": 4,
                            "isVisible": true,
                            "showOnList": true,
                            "showOnForm": true,
                            "readOnly": false,
                            "defaultValue": null,
                            "template": "field/select.html",
                            "showSelectAs": "grid",
                            "validations": {
                                "required": true
                            },
                            "newLine": true,
                            "placeholder": null,
                            "dataSource": {
                                "rest": true,
                                "localStorage": true,
                                "name": "product#/formacomu",
                                "itemsPerPage": 50,
                                "lazyLoad": true
                            },
                            "dataSourceFilter": [
                                {
                                    "name": "NRORG",
                                    "operator": "IN",
                                    "value": "@[getLocalVar('CRM_NRORGTRAB'),getLocalVar('CRM_NRORGPADRAO') ]@"
                                }
                            ],
                            "outData": {
                                "CDFORMACOMU": "CDFORMACOMU",
                                "NMFORMACOMU": "NMFORMACOMU"
                            },
                            "fields": [
                                {
                                    "id": "14418982344259089695859",
                                    "label": "Código",
                                    "name": "CDFORMACOMU",
                                    "class": "4",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                },
                                {
                                    "id": "14418982341859185194860",
                                    "label": "Forma de Comunicação",
                                    "name": "NMFORMACOMU",
                                    "class": "8",
                                    "showOnList": true,
                                    "template": "field/text-edit.html"
                                }
                            ],
                            "events": []
                        },
                        {
                            "id": "14418982341691438344861",
                            "name": "CDPREFIXCOMUPARC",
                            "label": "Prefixo",
                            "class": "2",
                            "isVisible": true,
                            "showOnList": true,
                            "maxlength": 10,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "14418982342324818248862",
                            "name": "CDCOMUNICAPARC",
                            "label": "Descrição",
                            "class": "10",
                            "isVisible": true,
                            "showOnList": true,
                            "validations": {
                                "required": true
                            },
                            "maxlength": 250,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "widgets": []
                }
			],
			"actions": [
			    {
			        "id":"17223197583113164",
                    "name": "BACK_MODAL",
                    "label": "Voltar",
                    "icon": "back",
                    "isVisible": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "id":"17223197583113165",
                           	"name": "ActionEvent",
                            "code": "ScreenService.closePopup(true)"
                        }
                    ]
                }
            ]
        };
        
        var modalWidget = metaDataFactory.widgetFactory(modalWidgetJson);
        
        return modalWidget;
        
    };
    
    /**
     * Abri o modal com as informações do parceiro.
    */
    this.openModal = function(){
        
        ScreenService.openPopup(templateManager.container.getWidget('WDGT_MODAL_PARTNER_INFOS')).then(function(){
            var widget = templateManager.container.getWidget('WDGT_PARTNER_DATA');
            //this.setDataWidgetCurrentRow(widget);
        }.bind(this));
        
    };
    
    /**
     * Define o filtro das widgets do modal informativo
     * de acordo com o parceiro selectionado na widget
     * ativa.
     * 
     * @param {object} modalWidget widget do modal informativo.
     * 
     * @param {object} activeWidget widget ativa onde o parceiro
     * foi selecionado.
     * 
     * @param {string} partnerFieldName nome do campo referente ao
     * parceiro de negócio.
    */
    this.setModalFilterByActivePartner = function(modalWidget, activeWidget, partnerFieldName){
        
        var selectedPartner = activeWidget.currentRow[partnerFieldName];
        
        var modalWidgets = modalWidget.widgets;
        var dataWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_PARTNER_DATA");
        var addressWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_PARTNER_ADDRESS");     
        var contactsWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_PARTNER_CONTACTS");
        var comunicationWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_PARTNER_COMU");
        var additionalWidgetIndex = CRM_Utilities.getObjectPositionInArrayByName(modalWidgets, "WDGT_DADOS_ADICIONAIS");
        var dataWidget = modalWidgets[dataWidgetIndex]; 
        var addressWidget = modalWidgets[addressWidgetIndex];     
        var contactsWidget = modalWidgets[contactsWidgetIndex];
        var comunicationWidget = modalWidgets[comunicationWidgetIndex];
        var additionalWidget = modalWidgets[additionalWidgetIndex];
        
        var dataWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(dataWidget.dataSourceFilter, "NRPARCNEGOCIO");
        var addressWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(addressWidget.dataSourceFilter, "NRPARCNEGOCIO");
        var contactsWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(contactsWidget.dataSourceFilter, "NRPARCNEGOCIO");
        var comunicationWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(comunicationWidget.dataSourceFilter, "P_NRPARCNEGOCIO");
        var additionalWidgetNrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(additionalWidget.dataSourceFilter, "P_NRPARCNEGOCIO");
        
        dataWidget.dataSourceFilter[dataWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        addressWidget.dataSourceFilter[addressWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        contactsWidget.dataSourceFilter[contactsWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        comunicationWidget.dataSourceFilter[comunicationWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        additionalWidget.dataSourceFilter[additionalWidgetNrparcnegocioFilterIndex].value = selectedPartner;
        
        dataWidget.reload();
        
    };
    
    /**
     * Atribui o parceiro retornado no data source do modal
     * na currentRow da widget de informações do parceiro
     * ('WDGT_PARTNER_DATA').
     * 
     * @param {object} widget widget de dados do parceiro.
     * ('WDGT_PARTNER_DATA').
    */
    this.setDataWidgetCurrentRow = function(widget){

        widget.moveToFirst();
        
    };
    
    /**
     * Insere a widget modal na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá o modal.
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */
    this.setModalWidgetOnActiveWidget = function(widget, nrparcnegocio){
        
        this.createModalAction(widget);
        widget.widgets.push(this.factoryModalWidget(widget, nrparcnegocio));
        
    };
    
    /**
     * Cria a action para abrir o modal com os dados
     *  do parceiro na widget.
     * 
     * @param {object} widget widget que receberá o modal.
     * 
    */
    this.createModalAction = function(widget){
        /*Modificado em 22/09 por Arthur
            Adicionada propriedade 'preventHide',
            que garante que a action de Dados do Parceiro 
            não seja setada como invisível em chamados já 
            concluidos
        */
        var modalAction = {
            "id":"17223197583113166",
            "name": "ACTION_MODAL_INFOS_PARCEIRO",
            "label": "Consultar Dados do Parceiro",
            "isVisible": true,
            "preventHide":true,
            "showAsAction": "never_view",
            "events": [
                {
                    "name": "ActionEvent",
                    "code": "PartnerInfosModalController.openModal();"
                }
            ]
        };
        
        if((widget.template.search("master_detail") === -1) && widget.template.search("kanban") === -1){
            modalAction.showAsAction = "never_list";
        }
        
        metaDataFactory.actionFactory(modalAction, widget, true);
        
    };
    
    
    /**
     * Muda o template da tab de interlocutor, necessário
     * para evitar o click na row, que abria o swipe.
     * 
     * @param {object} widget widget modal.
     * 
    */
    this.changeTemplateTabInterlocutor = function(widget){
        var interlocutorTab = templateManager.container.getWidget('WDGT_INTERLOCUTOR');
        interlocutorTab.template = 'widget/grid.html';
        templateManager.updateTemplate();
        interlocutorTab.activate();
    };
    
    this.setCurrentRowDadosAdicionais = function(widget){
        widget.reload().then(function(){
            if(!Util.isEmptyOrBlank(widget.dataSource.data))
                widget.currentRow = widget.dataSource.data[0];
        });
    }
    
    /**
     * Evita Bug de tamanho de colunas inferior ao tamanho do registro
    */
    this.resizeWindow = function(){
        $(window).resize();
    };
	
}
Configuration(function(ContextRegister){
    ContextRegister.register('PartnerInfosModalController', PartnerInfosModalController);
});

// FILE: ./assets/js/util/SelectUtil.js
function SelectUtil(ScreenService, metaDataFactory, EventChildPropagation) {
    
    var self = this;
    
    var CROSSTAB_TEMPLATE = 'crosstab.html';
	var TREE_TEMPLATE = 'tree/grid.html';
	var TREE_LIST_TEMPLATE = 'tree/list.html';
	var GANTT_TEMPLATE = 'tree/gantt.html';
	
	var _isTemplate = function(widget, template) {
		return widget.template && widget.template.indexOf(template) !== -1;
	};
	
	this.isTree = function(widget) {
		return _isTemplate(widget,TREE_TEMPLATE) || _isTemplate(widget, TREE_LIST_TEMPLATE);
	};
	
	var isLastChild = function(row){
		return row._childCount === 0;
	};
	
	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};
    
    this.construirWidgetSelect = function(field, newActions) {
        if (!field.selectWidget) {
            var actions = [{
    			"label": ScreenService.i18n("Cancel"),
    			"showAsAction": "back",
    			"isVisible": true,
    			"showOnForm": true,
    			"showOnList": true,
    			"hideIcon": true,
    			"events": [{
    				"name": "ActionEvent",
    				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
    				"id": 9998
    			}]
    		}];
    
    		if (field.showAddButtonAs == 'action') {
    			actions.push({
    				"icon": "plus",
    				"label": field.addButtonText,
    				"showAsAction": "always|edit|checked_rows|view",
    				"isVisible": true,
    				"showOnForm": true,
    				"showOnList": true,
    				"events": [{
    					"name": "ActionEvent",
    					"code": "args.owner.widget.onAdd()",
    					"id": 9999
    				}]
    			});
    		}
    		
    		actions = actions.concat(newActions);

    		var template;
    		if (!field.showSelectAs) {
    			template = "widget/list.html";
    		} else if (field.showSelectAs === "tree") {
    			template = "widget/tree/grid.html";
    		} else {
    			template = "widget/" + field.showSelectAs + ".html";
    		}
    
    		var fields;
    		if (field.fields && field.fields.length) {
    			fields = field.fields;
    		} else {
    			fields = [{
    				"label": field.sourceLabel,
    				"name": field.descriptionField,
    				"showOnList": true,
    				"mask": field.mask
    			}];
    		}
            
			var widgetEvents = [{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.setSelected(args);",
				"id": 9999
			}];
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 9009,
				"name": "Widget",
				"label": field.label,
				"template": template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile(),
					"defaultSearchField": field.defaultSearchField || field.descriptionField,
					"pdfAction": false,
					"xlsAction": false,
					"csvAction": false,
					"searchAction": true,
					"customizationAction": {
						"filter": false,
						"layout": false,
						"view": false
					}
				},

				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,

				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"orderFromGroups": field.orderFromGroups,
				"selectField": field,
				"showCheckbox": false,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"fields": fields,
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"itemsPerPage": field.itemsPerPage,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"allowSelectOnlyLastChild": field.allowSelectOnlyLastChild,
				"searchCriteria": field.searchCriteria,

				"actions": actions,
				"events": EventChildPropagation.propagate(widgetEvents, field),

				"styles": field.selectWidgetStyles
			});
			
			field.selectWidget.setSelected = this.setSelected;
			field.selectWidget.afterClosePopup = this.afterClosePopup;
		}
    };
    
    this.setSelected = function(args) {
		var field = args.owner.selectField;
		if(!field.allowSelectOnlyLastChild || (field.allowSelectOnlyLastChild && self.isTree(field.selectWidget) && isLastChild(args.row))){
			ScreenService.closePopup();

			var row = args.row;
			
			if (field.element) {
				field.element.focus();
			}

			(field.outData ? self.changeByOut : self.changeByName)(field, row);
			field.dataSource.clearCheckedRows();
			field.dataSource.addCheckedRows(row);
			setChanged(field);

			if (field._afterSelected) {
				field._afterSelected(row);
			}
			if (field.change) {
				field.change();
			}
		}
	};
	
	this.changeByOut = function(field, row) {
		if (field.widget.currentRow) {
			var outData = field.outData || {};

			Object.keys(outData).forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey] = row[key];
					});
				} else {
					field.widget.currentRow[outData[key]] = row[key];
				}
			});
		}
	};
	
	this.changeByName = function(field, row) {
      var valueField = field.valueField || field.name;
      var descriptionField = field.descriptionField || field.name;
      field.widget.currentRow[valueField] = row[valueField];
      field.widget.currentRow[field.name] = row[descriptionField];
      field.widget.inlineEdit();
    };
    
    this.afterClosePopup = function(field) {
		if (field.element) {
			field.element.focus();
		}
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register("SelectUtil", SelectUtil);
});

// FILE: ./assets/js/util/Utilities.js
function Utilities(ScreenService, General, restEngine, ZHPromise, templateManager, ZhFilterPreferences, ZhUtil) {

    /**
     * Carrega os registros da widget passada quando ela estiver em modo list.
     * 
     * @param {object} widget
     */
    this.carregarWidgetOnEnterTab = function (widget) {
        if (widget.mode == "list") widget.reload();
    };

    /**
     * Carrega na currentRow os dados do dataSource em uma 
     * widget com template form.html
     * @params{Object} widget - widget de form
     * @params{bool} alterarAction - determina se as actions de add e edit também serão validadas
    **/
    this.carregarFormOnEnterTab = function (widget, alterarAction) {
        if (widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0)
            widget.currentRow = widget.dataSource.data[0];
        else if (widget.active)
            this.formWidgetOnEnterTab(widget, alterarAction);
    };

    this.setValoresPadroesParaCampoSelectMultiple = function (field, value, index) {
        var data = this.procurarDataSource(field, value, index);
        field.value(this.criarArrayPeloDataSet(data, field.valueField));
    };

    this.procurarDataSource = function (field, values, index) {
        if (!field.dataSource || !field.dataSource.data) {
            return [];
        } else {
            return field.dataSource.data.filter(function (row) {
                var found = false;
                values.forEach(function (value) {
                    if (row[index] == value) {
                        found = true;
                    }
                });
                return found;
            });
        }
    };

    this.criarArrayPeloDataSet = function (dataSet, index) {
        var array = [];
        dataSet.forEach(function (row) {
            array.push(row[index]);
        });
        return array;
    };
    /**
     * Cria uma linha na widget para cada registro selecionado no campo select-multiple passado, 
     * a fim de inserir múltiplos registros de uma só vez na tabela.
     * 
     * @param {object} widget - Widget da tela
     * @param {string} nomeCampo - Nome do campo (field) select-multiple
     * 
     * @return {object[]}
     */
    this.gerarLinhasParaInsertMultiplo = function (widget, nomeCampo, camposOutData, usaNovoCodigo) {
        if (!camposOutData) {
            camposOutData = widget.getField(nomeCampo).outData;

            if (!camposOutData) {
                camposOutData = { nomeCampo: nomeCampo };
            }
        }

        var registros = widget.currentRow[nomeCampo];
        var rows = [];

        for (var i = 0; i < registros.length; i++) {
            var currentRow = angular.copy(widget.currentRow);
            currentRow.__is_new = true;

            if (usaNovoCodigo && !currentRow.__new_code_pk) {
                if (widget.paramsNewCode) {
                    currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
                    currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
                    currentRow.__table_pk = widget.paramsNewCode.__table_pk;
                    currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                    currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
                    if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                        widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
                }
            }

            var keys = Object.keys(camposOutData);
            var values = Object.values(camposOutData);

            for (var j = 0; j < keys.length; j++) {
                currentRow[values[j]] = currentRow[values[j]][i];
            }

            rows.push(currentRow);
        }

        return rows;
    };

    /**
     * Insere registros múltiplos.
     * 
     * @param {object} widget
     * @param {string} nomeCampo - Nome do campo (field) select-multiple na current row
     * @param {string} rota - Rota de "save" da widget
     * 
     * @return promise
     */
    this.inserirMultiplos = function (widget, nomeCampo, camposOutData, rota) {
        if (!rota)
            rota = widget.dataSource.name + "/save";

        var rows = this.gerarLinhasParaInsertMultiplo(widget, nomeCampo, camposOutData, false);
        return General.callRestEngine(rota, "DataSet", rows);
    };

    this.inserirMultiplosComNovoCodigo = function (widget, nomeCampo, camposOutData, rota) {
        if (!rota)
            rota = widget.dataSource.name + "/save";

        var rows = this.gerarLinhasParaInsertMultiplo(widget, nomeCampo, camposOutData, true);
        return General.callRestEngine(rota, "DataSet", rows);
    };

    /**
     * Ao selecionar algum item da árvore, seus itens pai também são selecionados.
     * 
     * @param {object} widget
     */
    this.selecionarTodosItensPai = function (widget) {
        var currentRow = widget.selectedRow;
        this.selecionarPai(widget, currentRow);
    };

    /**
     * Seleciona a linha pai da que foi passada.
     * 
     * @param {object} widget
     * @param {object} row
     */
    this.selecionarPai = function (widget, row) {
        widget.selectAll([row]);
        if (row._parent) {
            this.selecionarPai(widget, row._parent);
        }
    };

    /**
     * Retorna todos os itens pai de um elemento de uma árvore.
     * 
     * @param {object} row
     * 
     * @return {object[]}
     */
    this.getItensPai = function (row) {
        var itensPai = [];
        return this.getItensPaiAux(itensPai, row);
    };

    /**
     * Função auxiliar da "getItensPai", que vai adicionando os itens pai recursivamente em um array.
     * 
     * @param {object[]} itensPai
     * @param {object} row
     * 
     * @return {object[]}
     */
    this.getItensPaiAux = function (itensPai, row) {
        if (row._parent) {
            itensPai.push(row._parent);
            itensPai = this.getItensPaiAux(itensPai, row._parent);
        }
        return itensPai;
    };

    /**
     * Retorna o maior item pai (item mais acima) de um elemento de uma árvore.
     * 
     * @param {object} row
     * @return {object}
     */
    this.getMaiorItemPai = function (row) {
        return (row._parent) ? this.getMaiorItemPai(row._parent) : row;
    };

    /**
     * Retorna os itens raíz da árvore, ou seja, os itens que não tem item pai.
     * 
     * @param {object[]} dataSource
     * @return {object[]}
     */
    this.getItensRaizArvore = function (rows) {
        return rows.filter(function (row) {
            return !row._parent;
        });
    };

    /**
     * Abre um relatório BIRT
     */
    this.openReport = function (route, reportName, params) {
        params.__report_name = reportName;

        var reportParams = {
            "requestType": "Row",
            "row": params
        };

        return restEngine.post(route, reportParams, null, null, null, null, 300000).then(function (response) {
            if (response && response.messages) {
                var report = response.messages.pop();
                window.open(report.message);
            }
        }).catch(function (response) {
            if (response.data && response.data.error) {
                ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
            }
        });
    };

    /**
     * Se o NRORG do registro for igual a zero, o registro fica inalterável.
     */
    this.validarNrorgZero = function (widget) {
        if (!widget.currentRow.__is_new) {
            var actions = [
                widget.getAction("edit"),
                widget.getAction("delete"),
            ];

            var nrorg = widget.currentRow.NRORG;
            if (nrorg !== undefined) {
                var visivel = (nrorg == 0) ? false : true;

                actions.forEach(function (act) {
                    act.isVisible = visivel;
                });
            }
        }
    };

    /**
     * Função para identificar o index de um objeto
     * dentro de um FieldGroups, de acordo com 
     * seu nome.
     * 
     * @param {object} fieldGroup objeto da busca.
     * 
     * @param {String} name propriedade no objeto 
     * pelo qual é feito a busca.
     * 
     * @return {int} indexFieldGroupObject index Do objeto 
     * encontrado.
    */
    this.getFieldGroupPosition = function (fieldGroups, name) {
        var indexFieldGroupObject;

        fieldGroups.forEach(function (value, i) {
            if (value.name === name) {
                indexFieldGroupObject = i;
            }
        });
        return indexFieldGroupObject;
    };

    /**
     * Função para identificar o index de um objeto
     * dentro de um dataSourceFilter, de acordo com 
     * seu nome.
     * 
     * @param {object} dataSourceFilter objeto da busca.
     * 
     * @param {String} name propriedade no objeto pelo qual
     * é feito a busca.
     * 
     * @return {int} indexFilterObject index Do objeto 
     * encontrado.
     * 
    */
    this.getFilterPosition = function (dataSourceFilter, name) {
        var indexFilterObject;

        dataSourceFilter.forEach(function (value, i) {
            if (value.name === name) {
                indexFilterObject = i;
                return;
            }
        });

        return indexFilterObject;
    };

    this.callRestEngineWithTimeout = function (route, requestType, requestValue, miliseconds) {
        var defer = ZHPromise.defer();
        var params = {};

        switch (requestType) {
            case "Row":
                params = {
                    "requestType": "Row",
                    "row": requestValue
                };
                break;

            case "DataSet":
                params = {
                    "requestType": "DataSet",
                    "dataset": requestValue
                };
                break;

            case "FilterData":
                params = {
                    "requestType": "FilterData",
                    "filter": requestValue
                };
                break;

            default:
                return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
        }

        if (miliseconds)
            return restEngine.post(route, params, null, null, null, null, miliseconds);
        else
            return restEngine.post(route, params);
    };


    /**
    * Verifica se o e-mail é válido.
    *
    * retorna true para e-mail válido, false para inválido
    */

    this.verificaEmail = function (email) {
        var valida = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
        var retorno = valida.test(email);

        return retorno;
    };

    /**
     * Valida no banco se o dados que estao sendo inseridos ja não existem nos campos dos quais estão sendo gravados
     * 
     * Retona true se nao existe ou false se já existir na promise
     * 
     * @param {object} currenteRow da tela.
     * 
     * @param {Array} Lista de campos que serão verificados
     * 
     * @return {String} Rota do dataSource
     */
    this.validarCamposDuplicados = function (widget, pkFieldNames, validationFieldNames, findRoute) {
        var promise = ZHPromise.defer();

        if (!findRoute)
            findRoute = widget.dataSource.name;

        var params = {
            "DATASOURCE": findRoute,
            "COLUNAS_PK": pkFieldNames,
            "COLUNAS_VALIDACAO": validationFieldNames,
            "ROW": widget.currentRow
        };
        General.callRestEngine("product#/validacoes_gerais/validarRegistrosDuplicados", "Row", params).then(function (response) {

            if (response.dataset.EXISTE[0]) {
                promise.resolve(true);
            } else {
                promise.resolve(false);
            }
        })
            .catch(function (error) {
                promise.reject(error);
            });
        return promise;
    };

    /**
     * Constroi a função Criteria que nao existe no app.js para ser concatenado ao arquivo
     * (fundametal para o funcionamento do calendário e do filtro do mapa)
     * 
     * @param {object} widget widget 
    */
    this.buildCriteria = function (widget) {

        widget.setSearchCriteria = function (searchCriteria, forceReload) {
            if (!forceReload) {
                widget._noReloadFromSearchCriteria = true;
            }
            widget.searchCriteria = {};
            widget.searchList = "";
            if (searchCriteria) {
                searchCriteria.forEach(function (field) {
                    var fieldValue = field.value;
                    if (fieldValue[fieldValue.length - 1] == '%') {
                        fieldValue = fieldValue.substring(0, (fieldValue.length - 1));
                    }
                    if (fieldValue[0] == "%") {
                        fieldValue = fieldValue.substring(1, fieldValue.length);
                    }
                    this.widget.searchCriteria[field.name] = fieldValue;
                });
            }
        };
    };

    /**
     * Retorna o valor do field na widget informada. 
     * Caso não informe a widget, será considerada a widget ativa.
     */

    this.getFieldValue = function (name, widgetName) {
		var widget;
		if (widgetName) {
		    widget = templateManager.container.getWidget(widgetName);
		}
	    else {
	        widget = templateManager.container.getActiveWidget();
	    }
		    
		if (widget && widget.currentRow) {
			return widget.currentRow[name];
		}
		else {
			return "";
		}
	};
	
	 /* Função para confirmar e setar na currentRow 
     * do endereço em cadastro, o endereço retornado
     * da pesquisa pelo CEP no banco.
     * 
     * @param {string} CEP cep que será utilizado 
     *  para encontrar o endereço
     * 
    */
    this.buscarCep = function(CEP) {

        CEP = CEP.replace(/[\D]+/g, '');

        var params = {
            'CEP': CEP
        };

        if (CEP.length == 8) {
            return General.callRestEngine('/findAddress', "Row", params);
        } else {
            ScreenService.showMessage('CEP Inválido.');
        }
    };

    this.transformaHorasEmSegundos = function (horas) {
        horas = horas.split(':');
        return horas[0] * 3600 + horas[1] * 60;
    };


    /**
     * Mostra mensagem de erro, sendo ela uma string ou um objeto de erro de promise.
     */
    this.showErrorMessage = function (error) {
        console.error(error);
        var message = error;
        if (typeof error === 'object') {
            if (error.data && error.data.error) {
                message = error.data.error;
            }
            else if (error.message) {
                message = error.message;
            }
        }

        if (typeof message === 'object' && message.data == null) {
            message = 'Tempo limite excedido. Por favor, tente novamente.';
        }
        if (!message) {
            message = 'Ocorreu um erro inesperado. Entre em contato com o administrador do sistema.';
        }
        else if (message.indexOf('Exception: Exception: ') != -1) {
            message = message.replace('Exception: Exception: ', '');
        }

        ScreenService.showMessage(message, 'error');
    };

    /**
     * Salva a currentRow da widget e seta os valores da "__columns_to_return" na currentRow.
     * O parâmetro "showError" define se, caso ocorra algum erro na requisição do save, o erro será exibido.
     */
    this.widgetSaveAndSetColumnsToReturn = function(widget, showError) {
        if (showError == undefined) {
            showError = true;
        }

        var deferred = ZHPromise.defer();

        var columnsToReturn = null;
        if (widget.paramsNewCode && widget.paramsNewCode.__columns_to_return) {
            columnsToReturn = widget.paramsNewCode.__columns_to_return;
        }
        else {
            columnsToReturn = widget.currentRow.__columns_to_return;
        }

        General.widgetSave(widget).then(function (response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                if (Array.isArray(columnsToReturn)) {
                    columnsToReturn.forEach(function (column) {
                        widget.currentRow[column] = response.dataset.columnsToReturn[0][column];
                    });
                }
            }

            deferred.resolve(response);
        },
            function (error) {
                if (showError !== false) {
                    this.showErrorMessage(error);
                }

                deferred.reject(error);
            }.bind(this));

        return deferred.promise;
    };
    /**
     * Manda buscar e coloca na row as informações 
     * em uma widget de template form.html
     * @params{Object} widget - widget de form
     * @params{bool} alterarAction - determina se as actions de add e edit também serão validadas
    **/
    this.formWidgetOnEnterTab = function (widget, alteraActions) {
        widget.list();
        var actAdicionar = widget.getAction("add");
        var actEditar = widget.getAction("edit");
        var actExcluir = widget.getAction("delete");
        var deferred = ZHPromise.defer();

        widget.reload().then(function () {
            if (widget.dataSource.data.length > 0) {
                widget.currentRow = widget.dataSource.data[0];
                if (alteraActions) {
                    actAdicionar.isVisible = false;
                    actEditar.isVisible = true;
                    if (actExcluir) actExcluir.isVisible = true;
                }
            } else {
                widget.currentRow = {};
                if (alteraActions) {
                    actAdicionar.isVisible = true;
                    actEditar.isVisible = false;
                    if (actExcluir) actExcluir.isVisible = false;
                }
            }

            widget.view();

            deferred.resolve();
        })
            .catch(function (error) {
                this.showErrorMessage(error);
                deferred.reject(error);
            });

        return deferred.promise;

    };

    this.formWidgetNewRow = function (widget) {
        widget.newRow();
        widget.template = 'widget/form.html';
    };

    this.formWidgetEdit = function (widget) {
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

        //faz backup da currentRow para permitir a recuperação dos dados
        widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
        widget.edit();
        widget.template = 'widget/form.html';
    };

    this.formWidgetCancel = function (widget) {
        if (widget.currentRow.__is_new) {
            widget.dataSource.data.pop();
            widget.currentRow = {};
        } else {
            Object.keys(widget.backupRow).forEach(function (key) {
                widget.currentRow[key] = widget.backupRow[key];
            });
        }
        widget.view();
        widget.template = 'widget/form-view.html';
    };

    this.formWidgetSaveAndSetColumnsToReturn = function(widget, showError) {
        if (showError == undefined) {
            showError = true;
        }

        var deferred = ZHPromise.defer();

        var columnsToReturn = null;
        if (widget.paramsNewCode && widget.paramsNewCode.__columns_to_return) {
            columnsToReturn = widget.paramsNewCode.__columns_to_return;
        }
        else {
            columnsToReturn = widget.currentRow.__columns_to_return;
        }

        General.widgetSave(widget).then(function (response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                if (Array.isArray(columnsToReturn)) {
                    columnsToReturn.forEach(function (column) {
                        widget.currentRow[column] = response.dataset.columnsToReturn[0][column];
                    });
                }
            }

            widget.view();
            widget.template = "widget/form-view.html";

            deferred.resolve(response);
        })
            .catch(function (error) {
                if (showError !== false) {
                    this.showErrorMessage(error);
                }

                deferred.reject(error);
            }.bind(this));

        return deferred.promise;
    };

    /**
     * Abre a tela com a arvore de elementos aberta
     * @params{Object} widget widget que será aberta
    **/
    this.abreArvoreElementos = function (widget) {
        var deferred = ZHPromise.defer();
        if (widget.mode === "list") {
            widget.reload().then(function () {
                try {
                    widget.dataSource.data.map(function (dado) {
                        dado._opened = true;
                    });

                    templateManager.updateTemplate();
                    deferred.resolve();
                } catch (e) {
                    deferred.reject();
                }
            }).catch(function (error) {
                deferred.reject();
            });
        }
        return deferred.promise;
    };

    this.tornaCamposSelectMultipleObrigatorios = function (widget, grupoDeCamposEscolhidos) {
        var vazio = true;
        var contem = false;
        var contA = 0;
        var contC = 0;
        var contD = 0;
        grupoDeCamposEscolhidos.forEach(function (campoNome) {
            widget.fields.forEach(function (field) {
                if (campoNome == field.name) {
                    if (!field.validations.required)
                        contA += 1;
                    else
                        contC += 1;
                }
            });
        });

        grupoDeCamposEscolhidos.forEach(function (campo) {
            if (Array.isArray(widget.getField(campo).value())) {
                if (widget.getField(campo).value().length > 0) {
                    contB += 1;
                    contem = true;
                }
                else
                    contB -= 1;

                if (widget.getField(campo).value().length == 0)
                    contD += 1;
            } else {
                if (widget.getField(campo).value())
                    contB += 1;
                else
                    contB -= 1;

                if (!widget.getField(campo).value())
                    contD += 1;
            }
        });

        if (contB == 0 || contD == grupoDeCamposEscolhidos.length)
            vazio = false;

        if (contA == grupoDeCamposEscolhidos.length) {
            grupoDeCamposEscolhidos.forEach(function (campo) {
                widget.getField(campo).validations = { "required": true };
            });
            contA = 1;
        }
        if (contC == grupoDeCamposEscolhidos.length && !vazio && !contem) {
            grupoDeCamposEscolhidos.forEach(function (campo) {
                widget.getField(campo).validations = {};
            });
        }
    };
    /**
    * Instrução de Uso:
    * 
    * Seta validation como obrigatorio para os fields que foram escolhidos e passado seus nomes em um Array 
    * Para o objetivo completo desta função é nescessario chamar estes metodo em cada field escolida através 
    * do evento FieldOnChange destes Fields.
    * Para o tipo de campo que esta na Widget verificar se existe campo Select Multiplo e seta na variavel true ou false
    * 
    * @params{Object, Array} widget widget que contem as fields e "grupoDeCamposEscolhidos" um array com os nomes dod fields.
   **/
    var contB = 0;
    this.tornaCamposEscolhidosObrigatorios = function (widget, grupoDeCamposEscolhidos, _is_multSearch) {
        if (_is_multSearch) {
            this.tornaCamposSelectMultipleObrigatorios(widget, grupoDeCamposEscolhidos);
        } else {
            var vazio = true;
            var contA = 0;
            var contC = 0;
            var contD = 0;
            grupoDeCamposEscolhidos.forEach(function (campoNome) {
                widget.fields.forEach(function (field) {
                    if (campoNome == field.name) {
                        if (!field.validations.required)
                            contA += 1;
                        else
                            contC += 1;
                    }
                });
            });

            grupoDeCamposEscolhidos.forEach(function (campo) {
                if (widget.getField(campo).value())
                    contB += 1;
                else
                    contB -= 1;

                if (!widget.getField(campo).value())
                    contD += 1;
            });

            if (contB == 0 || contD == grupoDeCamposEscolhidos.length)
                vazio = false;

            if (contA == grupoDeCamposEscolhidos.length) {
                grupoDeCamposEscolhidos.forEach(function (campo) {
                    widget.getField(campo).validations = { "required": true };
                });
                contA = 1;
            }
            if (contC == grupoDeCamposEscolhidos.length && !vazio) {
                grupoDeCamposEscolhidos.forEach(function (campo) {
                    widget.getField(campo).validations = {};
                });
            }
        }
    };

    /**
   * Instrução de Uso:
   * 
   * verifica se os campos que estão visiveis na widget estão prenchidos para ser ultilizado na validação da ação de save da tela
   * 
   * @params{Object} widget widget que contem as fields para filtrar os campos visiveis
  **/

    this.verificaCamposPrenchidos = function (widget) {
        var retorno = false;
        if (!retorno) {
            widget.fields.forEach(function (field) {
                if (field.isVisible && field.showOnForm) {
                    if (Array.isArray(field.value())) {
                        if (field.value().length > 0)
                            retorno = true;
                    } else
                        if (field.value())
                            retorno = true;
                }
            });
        }
        return retorno;
    };

    this.masterDetailNewRow = function () {
        var widget = templateManager.container.getActiveWidget();

        this.changeWidgetVisibility(widget.widgets, false);

        widget.newRow();
    };

    this.changeWidgetVisibility = function (widgets, isVisible) {
        widgets.forEach(function (widget) {
            widget.isVisible = isVisible;
        });
    };

    this.masterDetailSave = function () {
        var widget = templateManager.container.getActiveWidget();

        this.changeWidgetVisibility(widget.widgets, true);
        this.formChangeTemplate();

        if (widget.isValid()) {
            widget.currentRow.changed = true;

            return widget.saveRow().then(function (response) {
                widget.currentRow.changed = false;
                widget.currentRow.__is_new = false;
                ScreenService.showMessage("Registro salvo com sucesso");
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            });
        }
    };

    this.formChangeTemplate = function () {
        var widget = templateManager.container.getActiveWidget();

        if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

        if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
        if (!widget.templateToView) widget.templateToForm = "widget/form-view.html";

        if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
        else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

        templateManager.updateTemplate();

        return true;
    };

    this.masterDetailGoBackToList = function () {
        var widget = templateManager.container.getActiveWidget();

        ScreenService.closeSwipe();
        widget.list();
    };

    this.masterDetailCancelForm = function () {
        var widget = templateManager.container.getActiveWidget();

        this.changeWidgetVisibility(widget.widgets, true);

        if (widget.currentRow.__is_new) {
            var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
            if (indexOfCurrentRow !== -1) {
                widget.dataSource.data.pop();
            }
            if (!this.formChangeTemplate()) widget.list();
            else widget.list();

            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backupRow);
            this.formChangeTemplate();
            widget.view();
        }
    };

    this.masterDetailEdit = function () {
        var widget = templateManager.container.getActiveWidget();

        this.changeWidgetVisibility(widget.widgets, false);
        this.formChangeTemplate();

        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();
    };

    this.masterDetailDelete = function () {
        try {
            var widget = templateManager.container.getActiveWidget();
            ScreenService.confirmMessage("Deseja deletar o registro?", "CONFIRMATION", function () {
                widget.remove();
                this.masterDetailGoBackToList();
                ScreenService.showMessage("Registro excluído com sucesso");
            }.bind(this));
        } catch (e) {
            ScreenService.showMessage(e, 'ERROR');
        }
    };

    /**
     * Aplica o filtro dos valores default na tela 
     * durante a primeira entrada
    **/
    this.filterAfterApply = function(args, fieldNames, filter) {
        if(filter == {} || filter == undefined) {
            if(fieldNames.length <= 0){
                args.owner.fields.forEach(function(field){
                    fieldNames.push(field.name);
                });
            }
            filter = this.getRowDefaultValuesFields(args.owner, fieldNames);
        }

        ZhFilterPreferences.setOriginalFilter(args.owner.id, filter);
        ZhFilterPreferences.applyOriginalFilter(args.owner);
    };

    /**
     * Habilita a field de select multiplo apenas quando 
     * o usuário estiver inserindo um novo registro
     * @param {Object} widget widget que contem a field e a row
     * @param {String} nameFieldMultiple nome da field de select multiple
     * @param {String} nameField nome da field que deverá aparecer no lugar da field de select multiple
    **/
    this.EdicaoFieldSelectMultiple = function (widget, nameFieldMultiple, nameField) {
        if (widget.currentRow.__is_new) {
            widget.getField(nameFieldMultiple).isVisible = true;
            widget.getField(nameFieldMultiple).showOnForm = true;
            widget.getField(nameField).showOnForm = false;
        } else {
            widget.getField(nameFieldMultiple).isVisible = false;
            widget.getField(nameFieldMultiple).showOnForm = false;
            widget.getField(nameField).showOnForm = true;
        }
    };

    /**
     * Substitui a visualização de uma widget por outra
     * @param {Object} widget widget que ficará invisível
     * @param {String} nameWidget nome da widget que ficará visivel
    **/
    this.alteraModoVisualizacaoWdg = function (widget, nameWidget) {
        var wdgAlteracao = templateManager.container.getWidget(nameWidget);
        widget.isVisible = false;
        wdgAlteracao.isVisible = true;
        templateManager.updateTemplate();
        wdgAlteracao.activate();
        wdgAlteracao.reload();
    };

    this.iniciarAutocomplete = function (field) {
        field.initializeField();
    };
    /**
     * Retira os valores repetidos de um array
     * @params{Array} arrayData - array de dados
    **/
    this.retirarValoresDuplicados = function (arrayData) {
        return arrayData.filter(function (value, index) {
            return arrayData.indexOf(value) === index;
        });
    };

    /**
     * Retira a mascara de formatação dos valores 
     * e o inicializa com 0 caso seja null
     * @params{string} vrUnitario - valor a ser formatado 
     * @params{number} casas - casas decimais do numero a ser retornado
    **/
    this.tirarMascaraValores = function(vrUnitario, casas) {
        if (casas == undefined) {
            casas = 2;
        }

        if (!vrUnitario) {
            vrUnitario = 0;
        } else {
            vrUnitario = vrUnitario.toString();
            if (vrUnitario.includes(",")) {
                vrUnitario = vrUnitario.replace(/\./g, "").replace(",", ".");
            }
        }

        if (typeof vrUnitario === 'string' || vrUnitario instanceof String) {
            vrUnitario = parseFloat(vrUnitario);
        }

        return parseFloat(vrUnitario.toFixed(casas));
    };

    this.calcularValoresDesconto = function (widget, nomeCampoValorTotal, nomeCampoPercDesconto, nomeCampoValorDesconto, nomeCampoValorTotalComDesconto, campoAlterado) {
        var valorTotal = this.tirarMascaraValores(widget.currentRow[nomeCampoValorTotal]);
        var valorPercDesconto = this.tirarMascaraValores(widget.currentRow[nomeCampoPercDesconto]);
        var valorDesconto = this.tirarMascaraValores(widget.currentRow[nomeCampoValorDesconto]);
        var valorTotalComDesconto = 0;

        if (valorPercDesconto || valorDesconto) {
            if (campoAlterado == nomeCampoPercDesconto) {
                valorDesconto = valorTotal * valorPercDesconto / 100;
                widget.currentRow[nomeCampoValorDesconto] = valorDesconto.toFixed(2);
                valorTotalComDesconto = valorTotal - valorDesconto;
            }
            else {
                valorPercDesconto = valorDesconto / valorTotal * 100;
                widget.currentRow[nomeCampoPercDesconto] = valorPercDesconto.toFixed(2);
                valorTotalComDesconto = valorTotal - valorDesconto;
            }

            widget.currentRow[nomeCampoValorTotalComDesconto] = valorTotalComDesconto.toFixed(2);
        }
        else {
            widget.currentRow[nomeCampoValorTotalComDesconto] = valorTotal;
        }
    };

    this.widgetCancelForm = function (widget) {
        widget.currentRow.changed = false;
        General.widgetCancelForm(widget);
    };

    /**
     * Seta o idativo no filtro das widgets
     * @params{Object} widget - widget que terá o filtro inicializado
     * @params{bool} idUtilizaDefaultValue - identifica se utilizará o valor do defaulvalue das fields
     * @params{array} nmFields - array de nome das fields que irão compor o filtro
     *  - caso esse parametro venha null e o idUtilizaDefaultValue = true ele usará todas as fields visiveis da widget
    **/
    this.setRowFilterWidgets = function (widget, idUtilizaDefaultValue, nmFields) {
        var currentRow = {};
        if (idUtilizaDefaultValue) {
            if (!nmFields) {
                widget.fields.forEach(function (field) {
                    nmFields.push(field.name);
                });
            }
            currentRow = this.getRowDefaultValuesFields(widget, nmFields);
        }
        ZhFilterPreferences.openFilterWidget(widget, currentRow);
    };

    /**
     * Retorna uma row com todos os valores do defaultvalue das fields descritas que irão aparecer na tela
     * @params{Object} widget - widget solicitante
     * @params{array} nmFields - array com os nomes das fields que irão compor o row
    **/
    this.getRowDefaultValuesFields = function (widget, nmFields) {
        var row = {};
        widget.fields.forEach(function (field) {
            if (nmFields.includes(field.name)) {
                var name = field.name;
                var fieldTemplate = field.template;
                var isVisible = (field.filterProperties && field.filterProperties.isVisible) ? field.filterProperties.isVisible : field.isVisible;
                if (field.filterProperties && isVisible && field.filterProperties.defaultValue) {
                    if (field.filterProperties.name) name = field.filterProperties.name;
                    if (field.filterProperties.template) fieldTemplate = field.filterProperties.template;

                    row[name] = ZhUtil.evaluate(field.filterProperties.defaultValue);
                } else {
                    if (field.isVisible && field.showOnForm && field.defaultValue)
                        row[field.name] = ZhUtil.evaluate(field.defaultValue);
                }

                if (fieldTemplate === "field/select-multiple.html" && !Array.isArray(row[name]))
                    row[name] = [row[name]];
            }
        });
        return row;
    };
    /**
     * Recebe um número de qualquer tipo e retorna uma string do mesmo 
     * formatado
    **/
    this.formatarNumero = function (numero) {
        return parseFloat(numero).toLocaleString('pt-BR', { minimumFractionDigits: 2 });
    };
    /**
     * Retorna o tipo de um arquivo
     * @params{str} type - extensao do arquivo
    **/
    this.getMimeTypesFile = function (type) {
        var extencoes = {
            ".pdf": "application/pdf",
            ".xlsx": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            ".xls": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            ".doc": "application/msword",
            ".docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
            ".png": "image/png",
            ".jpg": "image/jpeg",
            ".zip": "application/zip"
        };

        if (extencoes["." + type.replace(".", "")])
            return extencoes["." + type.replace(".", "")];

        return type;
    };

    /**
     * Faz o download de um arquivo salvo no servidor 
     * @params{string} nmarquivo - nome do arquivo
     * @params{string} dscaminho - caminho do arquivo no fileServer
     * @params{string} dstipo - tipo do arquivo
    **/
    this.downloadFile = function (nmarquivo, dscaminho, dstipo) {
        var params = {
            "DSCAMINHOARQ": dscaminho,
            "NMARQUIVO": nmarquivo,
            "DSTIPO": this.getMimeTypesFile(dstipo)
        };

        General.callRestEngine("product#/documentos/getUrlAnexos", "Row", params).then(function (success) {
            var file = success.dataset.URLDOCUMENTO;
            window.open(window.backendIndexUrl + file);
            ScreenService.closePopup();
        }).catch(function (error) {
            this.showErrorMessage();
        }.bind(this));
    };
    /**
     * Adiciona valores a row do filtro sem mante-los como default nas fields
     * @params{Object} widget     - widget que terá o filtro inicializado
     * @params{array} nmFields    - array de nome das fields que irão compor o filtro
     * @params{array} valueFields - valores que serão aplicados nas rows
    **/
    this.setRowFilterWidgetSemDefaultValue = function (widget, nameFields, valueFields) {
        var currentRow = {};
        for (var i = 0; i < nameFields.length; i++) {
            if (valueFields.length >= i)
                currentRow[nameFields[i]] = valueFields[i];
        }

        return ZhFilterPreferences.openFilterWidget(widget, currentRow);
    };

    /**
     * Retorna um período de doze meses anteriores a data atual
    **/
    this.getUltimosDozeMesesPeriodoInclusivo = function () {
        var dtAtual = moment().endOf('month');
        var periodo = {};

        periodo.end = dtAtual.format('DD/MM/YYYY');
        periodo.start = dtAtual.subtract(1, 'year').format('DD/MM/YYYY');
        return periodo;
    };

    /**
     * Retorna um período de doze meses anteriores a data atual
    **/
    this.getUltimosDozeMesesPeriodoExclusivo = function () {
        var dtAtual = moment().subtract(1, 'months').endOf('month');
        var periodo = {};

        periodo.end = dtAtual.format('DD/MM/YYYY');
        periodo.start = dtAtual.subtract(1, 'year').format('DD/MM/YYYY');
        return periodo;
    };

    /**
     * Valida se o percetual fornecido está entre 0 e 100
     * 
     * @params{Object} widget - widget que contem o campo de percentual
     * @params{String} nmCampoPercentual - nome do campo de percentual
    **/
    this.validarPercentual = function (widget, nmCampoPercentual) {
        if (widget.currentRow[nmCampoPercentual]) {
            var percentual = this.tirarMascaraValores(widget.currentRow[nmCampoPercentual]);
            if (percentual < 0 || percentual > 100) {
                ScreenService.showMessage("O percentual informado deve ser um número entre 0 e 100.");
                widget.currentRow[nmCampoPercentual] = 0;
            }
        }
    };

    /**
     * Retorna um período de 7 dias anteriores a data atual
    **/
    this.getUltimaSemana = function () {
        var dtAtual = moment().subtract(1, 'months').endOf('month');
        var periodo = {};

        periodo.end = dtAtual.format('DD/MM/YYYY');
        periodo.start = dtAtual.subtract(7, 'days').format('DD/MM/YYYY');
        return periodo;
    };

    /**
     * Retorna um período de 1 ano atrás até ontem.
     */
    this.getUltimoAnoAteOntem = function () {
        var dtFinal = moment().subtract(1, 'days');
        var periodo = {};

        periodo.end = dtFinal.format('DD/MM/YYYY');
        periodo.start = dtFinal.subtract(1, 'year').add(1, 'days').format('DD/MM/YYYY');
        return periodo;
    };

    /**
     * Compara se 2 arrays são iguais.
     * @returns {boolean}
     */
    this.arraysSaoIguais = function (arr1, arr2) {
        // if the other array is a falsy value, return
        if (!arr2)
            return false;

        // compare lengths - can save a lot of time 
        if (arr1.length != arr2.length)
            return false;

        for (var i = 0, l = arr1.length; i < l; i++) {
            // Check if we have nested arrays
            if (arr1[i] instanceof Array && arr2[i] instanceof Array) {
                // recurse into the nested arrays
                if (!arr1[i].equals(arr2[i]))
                    return false;
            }
            else if (arr1[i] != arr2[i]) {
                // Warning - two different object instances will never be equal: {x:20} != {x:20}
                return false;
            }
        }
        return true;
    };

    this.getDiasEntreDatas = function (dataIni, dataFin) {
        var prazo = null;
        if (dataIni.length > 0 && dataFin.length > 0) {
            dataIni = moment(dataIni, "DD/MM/YYYY");
            dataFin = moment(dataFin, "DD/MM/YYYY");
            prazo = dataFin.diff(dataIni, "days");
        }
        return prazo;
    };

    this.limparChangedDataSource = function (widget) {
        widget.dataSource.data.forEach(function (row) {
            row.changed = false;
        });
    };

    this.widgetNewRow = function (widget) {
        this.limparChangedDataSource(widget);
        General.widgetNewRow(widget);
    };


    this.widgetEdit = function (widget) {
        this.limparChangedDataSource(widget);
        General.widgetEdit(widget);
    };

    /**
     * Limpa o campo de data
     * 
     * @param {string} no do campo, mesmo nome presente no current row.
    */
    this.clearCalendarField = function(nomeDoCampo){
        setTimeout(function(){
            var campo = document.querySelector('.zh-field-' + nomeDoCampo);
            if(campo){
                var filhos = campo.children;
                if (filhos) {
                    var fechar = filhos[1].children[0];
                    if (fechar) {
                        fechar.click();
                    }
                }
            }
        }, 250);

    };

    /**
     * Retorna a diferença em dias entre 2 datas.
     * @param {string} dataini 
     * @param {string} datafin 
     * @returns int
     */
    this.calculaDiferencaDatasEmDias = function (dataini, datafin) {
        if (dataini && datafin) {
            dataini = moment(dataini.split(" ")[0], "DD/MM/YYYY");
            datafin = moment(datafin.split(" ")[0], "DD/MM/YYYY");

            return datafin.diff(dataini, "days");
        }
        return null;
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register("Utilities", Utilities);
});


// FILE: ./assets/js/util/UtilitiesMetas.js
function UtilitiesMetas(General, ScreenService, Utilities, templateManager){
    this.corSituacaoMeta = function (element, row) {
        if(row.REALIZADO < row.VRMETATPPERFIL){
            element.find('span').css('color', '#FF0000');
        } else {
            element.find('span').css('color', '#000');
        }
    }

    /**
     * Retorna a quantidade de dias de acordo 
     * com o código do período fixo
     * @param {Object} widget widget tabbed principal
     **/
    this.getDiasPeriodo = function(widget){
        switch (widget.currentRow.CDPERIODOFIXO) {
            case '2': return 7;
            case '3': return 15;
            case '4': return 30;
            case '5': return 90;
            case '6': return 180;
            case '7': return 365;
            default: return 1;
        }
    } 
    
    /**
     * Calcula a quantidade de períodos há a partir 
     * das datas passadas pelo usuário
     * @param {Object} widget widget tabbed principal
     **/
    this.calculaQtPeriodos = function (widget){
        var periodoIni = widget.currentRow.PERIODO.start ? moment(widget.currentRow.PERIODO.start, "DD/MM/YYYY") : moment().subtract(30, "days");
        var periodoFin = widget.currentRow.PERIODO.end ? moment(widget.currentRow.PERIODO.end, "DD/MM/YYYY") : moment();
        var dias       = periodoFin.diff(periodoIni, "days");
        var qtperiodos = 1;
        if(widget.currentRow.TPPERIODOMETA === "1")
            qtperiodos =  dias/this.getDiasPeriodo(widget);
        else 
            qtperiodos = dias/widget.currentRow.QTDIASCORRIDOS;
        if(qtperiodos < 1)
            return 1;
        else 
            return parseInt(qtperiodos);
    }

    /**
     * Coloca os valores do filtro nas widgets
     **/
    this.filtraDadosTela = function(){
        var wdgPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var filter = [];

        if(wdgPrincipal.currentRow.NRPARAMSMETAS){
            if(!wdgPrincipal.currentRow.PERIODO && wdgPrincipal.currentRow.NRPARAMSMETAS){
                wdgPrincipal.currentRow.PERIODO = {
                    "end"   : moment().endOf('month').format('DD/MM/YYYY'),
                    "start" : moment().startOf('month').format('DD/MM/YYYY'),
                    "period": ""
                };
            }

            var qtperiodo = this.calculaQtPeriodos(wdgPrincipal);
            filter = [
                {
                    name: "NRPARAMSMETAS",
                    operator: "=",
                    value: wdgPrincipal.currentRow.NRPARAMSMETAS
                },
                {
                    name: "PERIODOINI",
                    operator: wdgPrincipal.currentRow.PERIODOINI ? "=" : "IS_NULL",
                    value: wdgPrincipal.currentRow.PERIODO.start 
                },
                {
                    name: "PERIODOFIN",
                    operator: wdgPrincipal.currentRow.PERIODOFIN ? "=" : "IS_NULL",
                    value: wdgPrincipal.currentRow.PERIODO.end
                },
                {
                    name: "QTPERIODOS",
                    operator: "=",
                    value: qtperiodo
                }
            ];
        }

        wdgPrincipal.widgets.forEach(function(widget){
            widget.dataSourceFilter = [widget.dataSourceFilter[0], widget.dataSourceFilter[1]];
            widget.dataSourceFilter = widget.dataSourceFilter.concat(filter);
        });
    };

    /**
     * Valida as datas do período buscado
     * @param {Object} widget widget tabbed principal
     * @param {Object} field field de data que será validada
    **/
    this.validarData = function(widget){
        if(widget.currentRow.PERIODO){
            if(moment(widget.currentRow.PERIODO.end, "DD/MM/YYYY").diff(moment(widget.currentRow.PERIODO.start, "DD/MM/YYYY")) < 0){
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br>A data final do período precisa ser maior do que a data inicial.")).then(function(){
                    widget.currentRow[field.name] = "";
                });
            }
        }
    }

    this.alteraViewValores = function(wdgTabbed){
        wdgTabbed.widgets.forEach(function(widget){
            if(widget.template === 'zh-charts#widget/zh-chart.html'){
                widget.chartSeries[0].serieYAxisFormat = (wdgTabbed.currentRow.TPMEDIDAMETA === '1') ? "currency" : "number"; 
                if(widget.isActive())
                    widget.updateChart();
            } else {
                widget.getField("REALIZADO").mask.params.showSymbol  = (wdgTabbed.currentRow.TPMEDIDAMETA === '2') ? false : "R$";
                widget.getField("DIFERENCA").mask.params.showSymbol  = (wdgTabbed.currentRow.TPMEDIDAMETA === '2') ? false : "R$";
                widget.getField("VRMETATPPERFIL").mask.params.showSymbol  = (wdgTabbed.currentRow.TPMEDIDAMETA === '2') ? false : "R$";
            }
        });        
        templateManager.container.getActiveWidget().reload();
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('UtilitiesMetas', UtilitiesMetas);
});

// FILE: ./assets/js/util/UtilitiesPropostaContrato.js
/**
 * Utilities de funções iguais entre o contrato e a proposta
**/
function UtilitiesPropostaContrato(ScreenService, templateManager, General, Utilities, ZHPromise) {
        
    /**
     * Abre a widget que permite criar um novo serviço durante a 
     * seleceção do serviço para o item de formação de preço 
     * na proposta ou no contrato
     * @params{String} wdgName nome da widget que abrirá o popup
    **/
    this.criarNovoServicoSelecao = function(wdgName){
        ScreenService.closePopup();
        var widServico       = templateManager.container.getWidget(wdgName);
        widServico.isVisible = true;
        ScreenService.openPopup(widServico).then(function(){
            widServico.edit();
            widServico.newRow();
            widServico.activate();
        });
    };

    /**
     * Salva o novo serviço inserido durante a busca 
     * na proposta ou no contrato
     * @params{Object} widget widget popup servico proposta
    **/
    this.saveService = function(widget){
        var servico = widget.currentRow.NMSERVORCA;
        var widgetItens = templateManager.container.getWidget("itemFormacaoPreco");
        widget.currentRow.__columns_to_return = ["CDSERVORCA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if(response.dataset){
                widgetItens.currentRow.CDSERVORCA = response.dataset.columnsToReturn[0].CDSERVORCA;
                widgetItens.currentRow.NMSERVORCA = servico;
            }

            widget.isVisible = false;
            ScreenService.closePopup();
            widgetItens.activate();
        }.bind(this))
        .catch(function(error){
            console.log(error);
        });
    };
    
    /**
     * Habilita a widget para permitir a edição de um item do cálculo de 
     * formção de preço
     * @params{Object} widget widget editar calculo de formação de preço
    **/
    this.editarItensFormacaoPrec = function(widget){
        if(!widget.currentRow.IDALTERAFORMPREC || widget.currentRow.IDALTERAFORMPREC === "N"){
            ScreenService.showMessage("Elemento não pode ser alterado.<br>");
        } else {
            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
    
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
            widget.template = "widget/form.html";
        }
    };
    
        
    /**
     * Volta para a widget de item de formação de preço
     * @params{Object} widget widget editar cálculo de formação de preço
     * @params{String} actionName nome da action que está sendo utilizada
    **/
    this.voltarItensFormacaoPrec = function(widget, actionName){
        if(actionName === "VoltarWdgFormPrec"){
            var tabbedFormacao = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
            var itemFormacao   = templateManager.container.getWidget("itemFormacaoPreco");

            tabbedFormacao.isVisible = false;
            widget.isVisible = false;
            widget.dataSource.clearCheckedRows();
            widget.dataSource.data = [];
   
            this.habilitaEdicaoWidgetItemFormacao(itemFormacao, false);
            ScreenService.closeSwipe().then(function(){
                itemFormacao.activate();
            });  
        } else {
            if(actionName === "voltar_grid_arvore"){
                widget.template = "widget/tree/grid.html";
                widget.list();
            } else {
                if(!widget.currentRow.__is_new) {
                    Object.keys(widget.backupRow).forEach(function(key) {
                        widget.currentRow[key] = widget.backupRow[key];
                    });
                    
                    widget.changed  = false;
                    widget.template = "widget/form-view.html";
                    widget.view();
                } else {
                    if(widget.dataSource.data.indexOf(widget.currentRow) !== -1)
                        widget.dataSource.data.pop();
                    
                    widget.template = "widget/tree/grid.html";
                    widget.list();
                }
            }
        }
    };
    
    /**
     * Apaga um item de formação de preço da árvore auxiliar
     * @params{Object} widget widget editar formação de preço
    **/
    this.excluiItensArvore = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            if(widget.dataSource.name === "product#/fct_calcformprecserv_widget"){
                var index = widget.dataSource.data.indexOf(widget.currentRow);
                widget.dataSource.data.splice(index, 1);
                this.voltarItensFormacaoPrec(widget, "voltar_grid_arvore");
            } else {
                widget.removeRow().then(function() {
                    widget.template = "widget/tree/grid.html";
                    widget.list();

                    var params = {};
                    if(widget.dataSource.name === "product#/crud/fct_calcformprecserv_itpro_widget")
                        params = {
                            "NRFORMPRECOSERVICO" : widget.currentRow.NRFORMPRECOSERVICO,
                            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO",
                            "NRITEMORCAMENTO"    :  null
                        };
                    else 
                        params = {
                            "NRFORMPRECOSERVICO" : widget.currentRow.NRFORMPRECOSERVICO,
                            "TABELA"             : "FCT_CALCFORMPRECSERV_ITCONT",
                            "NRITEMOCONTRATO"    :  null
                        };
                    General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function(){
                        Utilities.abreArvoreElementos(widget);
                    }.bind(this))
                    .catch(function(){
                        ScreenService.showMessage("Falha ao recalcular o valor da Formação de Preço.");
                    });
                });
            }
        }.bind(this), 
        function (error) {
			Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * habilita o modo view da widget sem abrim um novo swipe
     * @params{Object} widget widget editar itens formação
    **/
    this.habilitarViewItensFormacao = function(widget) {
        widget.template = "widget/form-view.html";
        widget.view();
        widget.activate();
    };
    
    /**
     * Seleciona todos os dados da árvore para salvá-los
     * @params{Object} widget widget editar formação de preço
     * @params{Array} rows array de rows selecionadas
    **/
    this.buscaArvoreFormacaoCompleta = function(widget, rows){
        widget.dataSource.data.forEach(function(data){
            rows.filter(function(row){
                if(row === data._parent) {
                    if (rows.indexOf(data) === -1) {
                        rows.push(data);
                    }
                }
            });
        });
        
        return rows;
    };
    
    this.calculaItem = function(widget, tableName, nritemorcamento, nritemocontrato){
        var defered = ZHPromise.defer();
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        if (tipoCalculo == 'E') 
            this.calculaExpressao(widget, tableName, nritemorcamento, nritemocontrato).then(function(){
                this.calculaValorTotal(widget);
                defered.resolve();
            }.bind(this));
        else 
            if (tipoCalculo == 'S'){
                defered.resolve();
            } else {
                this.calculaValorTotal(widget);
                defered.resolve();
            }
        return defered;
    };

    /**
     * Realiza o cálculo descrito no campo "Expressão" e retorna para o campo "Valor Unitário"
     * 
     * @param {object} widget widget
     */
    this.calculaExpressao = function(widget, tableName, nritemorcamento, nritemocontrato){
        var defered = ZHPromise.defer();
        var NrFormPrecoServico = widget.currentRow.NRFORMPRECOSERVICO;
        var expressao = widget.currentRow.DSEXPRESSAOCALC;
        var percentual = widget.currentRow.VRPERCCALCULO;
        var vrunit = 0;
        var params = {
            "NRFORMPRECOSERVICO" : NrFormPrecoServico,
            "TABELA"             : tableName,
            "NRITEMORCAMENTO"    : nritemorcamento,
            "NRITEMOCONTRATO"    : nritemocontrato
        };
        General.callRestEngine('product#/fct_calcformprecserv_widget/buscaIdentValores', "Row", params).then(function(response){
            var identVal = response.dataset.IDENTVAL;
            identVal.forEach(function(identfica){
                var identificador = '['+identfica.IDITEMCALCULO+']';
                var valor = identfica.VRTOTAL;
                expressao = expressao.split(identificador).join(valor);    
            });
            expressao = expressao.split("%").join("/100");
            if (percentual != null){
               vrunit = eval(expressao);
               vrunit = (vrunit*percentual)/100;
            } else {
                vrunit = eval(expressao);
            }
            widget.currentRow.VRUNITARIO =  vrunit;
            defered.resolve();
        }, function(error) {
            Utilities.showErrorMessage(error);
            defered.reject();
        });
        return defered;
    };

    /**
     * Realiza o cálculo do Valor Total
     * 
     * @param {object} widget widget
     */
     
    this.calculaValorTotal = function(widget){
        var vrUnit = widget.currentRow.VRUNITARIO;
        if (vrUnit) {
            vrUnit = vrUnit.toString();
            var primeiraPos = vrUnit.indexOf(".");
            var ultimaPos = vrUnit.lastIndexOf(".");
            if (primeiraPos == ultimaPos) {
                if (vrUnit.includes(",")) {
                    vrUnit = parseFloat(vrUnit.replace(".", "").replace(",", "."));
                } else {
                    vrUnit = parseFloat(vrUnit.replace(",", "."));
                }
            } else {
                var parteInicial = vrUnit.substr(0, primeiraPos + 1);
                var parteFinal = vrUnit.substr(primeiraPos + 1);
                vrUnit = parseFloat(parteInicial.replace(".", "") + parteFinal);
            }
        } else {
            vrUnit = 0;
        }
        var qtCalc    = widget.currentRow.QTCALCULO;
        var usamedida = widget.currentRow.IDUSAMEDIDA;
        var qtmedida  = widget.currentRow.QTMEDIDACALC;
        var vrTot     = 0;
        if (usamedida == 'S'){
            vrTot = (vrUnit*qtCalc)*qtmedida ; 
        } else {
            vrTot = vrUnit*qtCalc;
        }
        widget.currentRow.VRTOTAL = vrTot;
    };
    
    /**
     * Valida se os escopos escolhidos possuem os dados necessários 
     * a geração da negociação de valores da proposta.
     * @params{Object} widget - widget de proposta
     * @param{String} nmFieldEscopo - nome do campo de escopo
     * @params{String} tipoDeEscopo - tipo de escopo (ORC, CONT)
    **/
    this.validarParamsFaturamentoEscopo = function(widget, nmFieldEscopo, tipoDeEscopo) {
        var nrtipoorcamento  = (tipoDeEscopo == "ORC") ? "NRTIPOORCAMENTO" : "NRTIPOORCAMENTO_CONT";
        var nmtipoorcamento  = (tipoDeEscopo == "ORC") ? "NMTIPOORCAMENTO" : "NMTIPOORCAMENTO_CONT";
        var fieldIsArray     = (["NRTIPOORCAMENTO", "NRTIPOORCAMENTO_CONT"].includes(nmFieldEscopo));
 
        if((!fieldIsArray && widget.currentRow[nrtipoorcamento]) || (fieldIsArray && widget.currentRow[nrtipoorcamento].length > 0)) {
            var params = { "NRTIPOORCAMENTO": widget.currentRow[nrtipoorcamento] };
            General.callRestEngine("product#/fct_tipoorcamento_widget/validarParamsFaturamentoEscopo", "Row", params)
            .then().catch(function(error) {
                Utilities.showErrorMessage(error);
                
                widget.currentRow[nrtipoorcamento] = fieldIsArray ? [] : null;
                widget.currentRow[nmtipoorcamento] = fieldIsArray ? [] : null;
                widget.getField(nmFieldEscopo).readOnly = false;
            });
        }
    };
    
    /**
     * valida se o operador logado pode diminuir a quantidade de medida dos itens
    **/
    this.validarPermissaoAlteracaoQtMedida = function() {
        var deferred = ZHPromise.defer();
        General.callRestEngine("product#/fct_orcamento_widget/validarPermissaoAlteracaoQtMedida", "Row", {}).then(function(response) {
            var permiteAlteracao = (response && response.dataset && response.dataset.fct_orcamento_widget.PERMITEDIMINUICAO);
            deferred.resolve(permiteAlteracao);
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject();
        });
        
        return deferred.promise;
    };

    this.setarMedidaVariavelEscopo = function(wdgItens, propostaOuContrato) {
        var nrtipoorcamento = (propostaOuContrato == "P") ? wdgItens.currentRow.NRTIPOORCAMENTO : templateManager.container.getWidget("gridContrato").currentRow.NRTIPOORCAMENTO;
        var tipoprodserv =  (propostaOuContrato == "P") ? wdgItens.currentRow.TIPOPRODSERV : wdgItens.currentRow.IDPRODSERV;
        var cdprodserv = wdgItens.currentRow.CDPRODSERV;
        var nomeFieldQtdMedida = (propostaOuContrato == "P") ? "QTMEDIDAORCA" : "QTMEDIDACONTR";

        if (nrtipoorcamento && tipoprodserv == "S" && cdprodserv) {
            var params = {
                "NRTIPOORCAMENTO": nrtipoorcamento,
                "CDSERVORCA"     : cdprodserv
            };

            General.callRestEngine("product#/fct_itemorcamento_widget/getMedidaVariavelEscopo", "Row", params).then(function(response) {
                if (response.dataset && response.dataset.retorno) {
                    wdgItens.currentRow.CDTPMEDIDASERV                = response.dataset.retorno.CDTPMEDIDAVARIAVEL;
                    wdgItens.currentRow.CDTPMEDIDAVARIAVEL            = response.dataset.retorno.CDTPMEDIDAVARIAVEL;
                    wdgItens.currentRow.DSTPMEDIDASERV                = response.dataset.retorno.DSTPMEDIDAVARIAVEL;
                    wdgItens.currentRow.IDMULTMEDVRTOT                = "S";
                    wdgItens.getField("DSTPMEDIDASERV").readOnly      = true;
                    wdgItens.getField("DSTPMEDIDASERV").validations   = {required: true};
                    wdgItens.getField("IDMULTMEDVRTOT").readOnly      = true;
                    wdgItens.getField(nomeFieldQtdMedida).readOnly    = false;
                    wdgItens.getField(nomeFieldQtdMedida).validations = {required: true};
                    wdgItens.getField(nomeFieldQtdMedida).validations = {required: true};
                    wdgItens.currentRow.INSERIR_MEDIDA                = true;
                } else {
                    wdgItens.currentRow.CDTPMEDIDASERV           = "";
                    wdgItens.currentRow.CDTPMEDIDAVARIAVEL       = "";
                    wdgItens.currentRow.DSTPMEDIDASERV           = "";
                    wdgItens.getField("DSTPMEDIDASERV").readOnly = false;
                    wdgItens.getField("IDMULTMEDVRTOT").readOnly = false;
                    wdgItens.currentRow.INSERIR_MEDIDA           = false;
                }
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            wdgItens.currentRow.CDTPMEDIDASERV = "";
            wdgItens.currentRow.CDTPMEDIDAVARIAVEL = "";
            wdgItens.currentRow.DSTPMEDIDASERV = "";
            wdgItens.getField("DSTPMEDIDASERV").readOnly = false;
            wdgItens.getField("IDMULTMEDVRTOT").readOnly = false;
            wdgItens.currentRow.INSERIR_MEDIDA = false;
        }
    };
    
    /**
     * Controla o readOnly das fields na widget de itens de formação de preço
     * para manter a coerencia durante a edição 
     * @params{Object} widget - widget de itens de formação
     * @params{bool} readOnly - se o campo será habilitado ou não
    **/
    this.habilitaEdicaoWidgetItemFormacao = function(widget, readOnly) {
        widget.fields.forEach(function(field) {
            if(field.name != "NMTIPOORCAMENTO")
                field.readOnly = readOnly;
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("UtilitiesPropostaContrato", UtilitiesPropostaContrato);
});

// FILE: ./assets/js/util/ValidacoesParametrosFaturamentoUtil.js
function ValidacoesParametrosFaturamentoUtil(General, ZHPromise, Utilities, ScreenService, templateManager) {
    
    this.validarSePeriodoDeCompetenciaExiste = function(widget, nomeTabela, nomeCampoPK, nomeCampoPai) {
        var deferred = ZHPromise.defer();
        
        var valorCampoPK = widget.currentRow[nomeCampoPK];
        if (Util.isEmptyOrBlank(valorCampoPK))
            valorCampoPK = -1;
        
        var valorCampoPai = widget.currentRow[nomeCampoPai];
        
        var params = {
            "NOME_TABELA"    : nomeTabela,
            "NOME_CAMPO_PK"  : nomeCampoPK,
            "VALOR_CAMPO_PK" : valorCampoPK,
            "NOME_CAMPO_PAI" : nomeCampoPai,
            "VALOR_CAMPO_PAI": valorCampoPai,
            "NRDIACOMPETINI" : widget.currentRow.NRDIACOMPETINI,
            "NRDIACOMPETFIN" : widget.currentRow.NRDIACOMPETFIN,
            "IDMESCOMPETINI" : widget.currentRow.IDMESCOMPETINI,
            "IDMESCOMPETFIN" : widget.currentRow.IDMESCOMPETFIN
        };
        
        General.callRestEngine("product#/validacoes_parametros_faturamento/validarSePeriodoDeCompetenciaExiste", "Row", params)
        .then(function(success) {
            deferred.resolve(success.dataset.resultado.existe);
        })
        .catch(function(error) {
            deferred.reject(error);
        });
        
        return deferred.promise;
    };
    
    /* ========================== FUNCOES NOVO FATURAMENTO ========================== */
    
    this.controlarTipoCalculoVencimento = function(wdgParametrosFaturamento) {
        var idtpcalcvenc = wdgParametrosFaturamento.currentRow.IDTPCALCVENC;
        var fieldDiaVencimento = wdgParametrosFaturamento.getField("NRDIACALCVENC");
        var fieldMesCalcVenc = wdgParametrosFaturamento.getField("IDMESCALCVENC");
        var fieldDiasMinimosFatVenc = wdgParametrosFaturamento.getField("QTDIASFATUVENC");
        
        if (idtpcalcvenc == "1") { //Dia fixo para a data de vencimento
            fieldDiaVencimento.label = "Dia de Vencimento";
            
            fieldMesCalcVenc.readOnly = false;
            fieldMesCalcVenc.validations = {"required": true};
            
            fieldDiasMinimosFatVenc.readOnly = false;
            fieldDiasMinimosFatVenc.validations = {"required": true};
        }
        else if (idtpcalcvenc == "2") { //Quantidade de dias após a data de emissão
            fieldDiaVencimento.label = "Quantidade de Dias";
            
            fieldMesCalcVenc.readOnly = true;
            fieldMesCalcVenc.value("");
            fieldMesCalcVenc.validations = {};
            
            fieldDiasMinimosFatVenc.readOnly = true;
            fieldDiasMinimosFatVenc.value("");
            fieldDiasMinimosFatVenc.validations = {};
        }
    };
    
    this.wdgParametrosFaturamentoNewRow = function(wdgParametrosFaturamento, nameWdgCompetencia) {
        General.widgetNewRow(wdgParametrosFaturamento);
        this.controlarTipoCalculoVencimento(wdgParametrosFaturamento);
        
        if(nameWdgCompetencia)
            this.controlarCamposTipoFaturamento(wdgParametrosFaturamento, nameWdgCompetencia);
    };
    
    this.validarPeriodoCompetencia = function(wdgParametrosFaturamento) {
        var diaInicial = wdgParametrosFaturamento.currentRow.NRDIACOMPETINI;
        var diaFinal   = wdgParametrosFaturamento.currentRow.NRDIACOMPETFIN;
        var mesInicial = parseInt(wdgParametrosFaturamento.currentRow.IDMESCOMPETINI);
        var mesFinal   = parseInt(wdgParametrosFaturamento.currentRow.IDMESCOMPETFIN);
        
        if (!Util.isEmptyOrBlank(diaInicial) && !Util.isEmptyOrBlank(diaFinal)) {
            if (mesInicial == mesFinal && diaInicial > diaFinal) {
                ScreenService.showMessage("Operação bloqueada.<br/>O dia da competência inicial deve anterior que o dia final.");
                wdgParametrosFaturamento.currentRow.NRDIACOMPETINI = "";
                wdgParametrosFaturamento.currentRow.NRDIACOMPETFIN = "";
            }
            else if (mesInicial > mesFinal) {
                ScreenService.showMessage("Operação bloqueada.<br/>O dia da competência inicial deve anterior que o dia final.");
                wdgParametrosFaturamento.currentRow.NRDIACOMPETINI = "";
                wdgParametrosFaturamento.currentRow.NRDIACOMPETFIN = "";
            }
        }
    };
    
    this.controlarCamposTipoFaturamento = function(wdgParametrosFaturamento, nameWdgCompetencia) {
        var idtpfatura = wdgParametrosFaturamento.currentRow.IDTPFATURA;
        
        var fieldTipoMedidaVar = wdgParametrosFaturamento.getField("DSTPMEDIDAVARIAVEL");
        if (fieldTipoMedidaVar) {
            if (idtpfatura == "X") {
                fieldTipoMedidaVar.readOnly = false;
            }
            else {
                Object.values(fieldTipoMedidaVar.outData).forEach(function(field) {
                    wdgParametrosFaturamento.currentRow[field] = null;
                });
                fieldTipoMedidaVar.readOnly = true;
            }
        }

        if(nameWdgCompetencia) {
            var wdgCompetencia = templateManager.container.getWidget(nameWdgCompetencia);
            wdgCompetencia.isVisible = (idtpfatura && idtpfatura != "P");
            wdgParametrosFaturamento.getField("QTDIASAPURACAO").isVisible = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("IDTPCALCVENC").isVisible   = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("NRDIACALCVENC").isVisible  = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("IDMESCALCVENC").isVisible  = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("QTDIASFATUVENC").isVisible = (idtpfatura == "P");
            wdgParametrosFaturamento.getField("QTPARCELASFATU").isVisible = (idtpfatura == "P");
            
            templateManager.updateTemplate();
        } else {
            var campoQtdParcelas = wdgParametrosFaturamento.getField("QTPARCELASFATU");
            if (idtpfatura == "P") { //Parcelamento (Quantidade de Parcelas)
                campoQtdParcelas.readOnly = false;
                campoQtdParcelas.validations = {"required": true};
            } else { //Periódico
                campoQtdParcelas.readOnly = true;
                campoQtdParcelas.value("");
                campoQtdParcelas.validations = {};
            }
        }
    };
    
    this.wdgCapaParametrosFaturamentoOnEnter = function(wdgCapaParametrosFaturamento, nameWdgCompetencia) {
        wdgCapaParametrosFaturamento.list();
        var actAdicionar = wdgCapaParametrosFaturamento.getAction("add");
        var actEditar    = wdgCapaParametrosFaturamento.getAction("edit");
        var actExcluir   = wdgCapaParametrosFaturamento.getAction("delete");
        
        wdgCapaParametrosFaturamento.reload().then(function() {
            if (wdgCapaParametrosFaturamento.dataSource.data.length > 0) {
                wdgCapaParametrosFaturamento.currentRow = wdgCapaParametrosFaturamento.dataSource.data[0];
                actAdicionar.isVisible = false;
                actEditar.isVisible    = true;
                if (actExcluir) actExcluir.isVisible = true;
            } else {
                wdgCapaParametrosFaturamento.currentRow = {};
                actAdicionar.isVisible = true;
                actEditar.isVisible    = false;
                if (actExcluir) actExcluir.isVisible = false;
            }

            wdgCapaParametrosFaturamento.view();
            if(nameWdgCompetencia){
                this.controlarCamposTipoFaturamento(wdgCapaParametrosFaturamento, nameWdgCompetencia);
                this.controlarTipoCalculoVencimento(wdgCapaParametrosFaturamento);
            }
            
        }.bind(this));
    };
    
    this.wdgCapaParametrosFaturamentoNewRow = function(wdgCapaParametrosFaturamento, nomeWidgetPai, nomeWidgetParamFatTabbed, nameWdgCompetencia) {
        altararAcessibilidadeWidgetsFaturamento(wdgCapaParametrosFaturamento.name, nomeWidgetPai, nomeWidgetParamFatTabbed, false);
        Utilities.formWidgetNewRow(wdgCapaParametrosFaturamento);
        templateManager.container.getWidget(nomeWidgetParamFatTabbed).edit();
        this.controlarCamposTipoFaturamento(wdgCapaParametrosFaturamento, nameWdgCompetencia);
    };
    
    this.wdgCapaParametrosFaturamentoEdit = function(wdgCapaParametrosFaturamento, nomeWidgetPai, nomeWidgetParamFatTabbed, nameWdgCompetencia) {
        altararAcessibilidadeWidgetsFaturamento(wdgCapaParametrosFaturamento.name, nomeWidgetPai, nomeWidgetParamFatTabbed, false);
        Utilities.formWidgetEdit(wdgCapaParametrosFaturamento);
        templateManager.container.getWidget(nomeWidgetParamFatTabbed).edit();
        this.controlarCamposTipoFaturamento(wdgCapaParametrosFaturamento, nameWdgCompetencia);
    };
    
    this.wdgCapaParametrosFaturamentoCancel = function(wdgCapaParametrosFaturamento, nomeWidgetPai, nomeWidgetParamFatTabbed) {
        altararAcessibilidadeWidgetsFaturamento(wdgCapaParametrosFaturamento.name, nomeWidgetPai, nomeWidgetParamFatTabbed, true);
        Utilities.formWidgetCancel(wdgCapaParametrosFaturamento);
        templateManager.container.getWidget(nomeWidgetParamFatTabbed).list();
    };
    
    this.wdgCapaParametrosFaturamentoDelete = function(widget, wdgNameWidgetTabbed) {
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
			widget.removeRow().then(function () {
				this.wdgCapaParametrosFaturamentoOnEnter(widget, wdgNameWidgetTabbed);
			}.bind(this),
			function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
				}
				else {
					Utilities.showErrorMessage(error);
				}
			});
		}.bind(this));
    };
    
    this.wdgCapaParametrosFaturamentoSave = function(wdgCapaParametrosFaturamento, nomeWidgetPai, nomeWidgetParamFatTabbed, nameWidgetCompetencia) {
        Utilities.formWidgetSaveAndSetColumnsToReturn(wdgCapaParametrosFaturamento).then(function() {
            altararAcessibilidadeWidgetsFaturamento(wdgCapaParametrosFaturamento.name, nomeWidgetPai, nomeWidgetParamFatTabbed, true);
            this.wdgCapaParametrosFaturamentoOnEnter(wdgCapaParametrosFaturamento, nameWidgetCompetencia);
            templateManager.container.getWidget(nomeWidgetParamFatTabbed).list();
        }.bind(this));
    };
    
    var altararAcessibilidadeWidgetsFaturamento = function(nomeWidgetCapa, nomeWidgetPai, nomeWidgetParamFatTabbed, visibilidade) {
        var wdgModeloContrato = templateManager.container.getWidget(nomeWidgetPai);
        
        var widgets = wdgModeloContrato.widgets.filter(function(wid) {
            return wid.name != nomeWidgetParamFatTabbed;
        }).concat(wdgModeloContrato);
        
        var wdgParametrosFaturamentoTabbed = templateManager.container.getWidget(nomeWidgetParamFatTabbed);
        widgets = widgets.concat(wdgParametrosFaturamentoTabbed.widgets.filter(function(wid) {
            return wid.name != nomeWidgetCapa;
        }));
        
        General.changeWidgetAcessibility(widgets, visibilidade);
    };
    
    this.wdgParametrosFaturamentoSave = function(wdgParametrosFaturamento, nomeCampoPai) {
        if (wdgParametrosFaturamento.isValid()) {
            var nomeTabela = wdgParametrosFaturamento.paramsNewCode.__new_code_pk;
            var nomeCampoPK = wdgParametrosFaturamento.paramsNewCode.__table_pk;
            
            this.validarSePeriodoDeCompetenciaExiste(wdgParametrosFaturamento, nomeTabela, nomeCampoPK, nomeCampoPai)
            .then(function(existe) {
                if (existe === false) {
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgParametrosFaturamento);
                }
                else {
                    Utilities.showErrorMessage("Operação bloqueada. <br/>Já existe um registro para o período de competência informado.");
                }
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("ValidacoesParametrosFaturamentoUtil", ValidacoesParametrosFaturamentoUtil);
});

// FILE: ./assets/js/util/evolutionTabController.js
function evolutionTabController(ScreenService, templateManager, metaDataFactory, ZHPromise, General){
    
    /**
     * Tranforma o JSON da tab em uma widget de fato.
     * 
     * @param {object} widget widget que irá receber a tab.
     * 
     * @param {string} nrparcnegocio nome do campo com o número
     * do parceiro de negócio.
     * 
     * @return widget evolucao construida.
     * 
    */
    function _factoryEvolutionTab(activeWidget) {
        
        
        var evolutionTabWidgetJson = {
            "id": "1722319758311315390513",
            "container":activeWidget.container,
            "parent":activeWidget,
            "name": "WDGT_EVOLUCAO",
            "label": "Evoluções",
            "isVisible": true,
            "showDescriptionOrder": false,
            "template": "widget/master_detail/grid.html",
            "dataSource": {
                "name": "/evolcont_contato",
                "rest": true,
                "lazyLoad": true
            },
            "fieldGroups": [
                {
                    "name": "FG_AGENDAR_TAREFA",
                    "label": "Próxima Tarefa",
                    "collapsible": true,
                    "opened": true
                }
            ],
            "dataSourceFilter": [
                {
                    "name": "NRSEQCONTATO",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": "@templateManager.LOGGEDOPERATOR.NRORG@"
                }
            ],
            "fields": [
                {
                    "id": "02",
                    "name": "NRSEQCONTATO",
                    "label": "Contato",
                    "template": "field/datetime.html",
                    "showOnForm": false,
                    "showOnList": false,
                    "isVisible": false,
                    "readOnly": false,
                    "class": "6",
                    "defaultValue":"@General.getFieldValue('NRSEQCONTATO', '"+activeWidget.name+"')@",
                    "events": []
                },
                {
                    "id": "03",
                    "name": "NRORG",
                    "label": "Org",
                    "template": "field/datetime.html",
                    "showOnForm": false,
                    "showOnList": false,
                    "isVisible": false,
                    "readOnly": false,
                    "class": "6",
                    "defaultValue":"@getLocalVar('CRM_NRORGTRAB')@",
                    "events": []
                },
                {
                    "id": "04",
                    "name": "DTHREVOLCONT",
                    "label": "Data da Evolução",
                    "template": "field/datetime.html",
                    "showOnForm": true,
                    "showOnList": true,
                    "isVisible": true,
                    "readOnly": false,
                    "defaultValue":"@CRM_Utilities.getDataHoraAtual()@",
                    "class": "6",
                    "events": []
                },
                {
                    "id": "05",
                    "name": "IDSENEVOCONT",
                    "label": "Sentido",
                    "template": "field/select-native.html",
                    "showOnForm": true,
                    "showOnList": true,
                    "class": "6",
                    "valueField": "IDSENEVOCONT",
                    "descriptionField": "NMSENEVOLCONT",
                    "defaultValue": "S",
                    "isVisible": true,
                    "events": [],
                    "validations":{
                        "required":true
                    },
                    "filterProperties":{
                        "validations":{}
                    },
                    "hideEmptyValue":true,
                    "dataSource": {
                        "data": [
                            {
                                "NMSENEVOLCONT": "Entrada",
                                "IDSENEVOCONT":"E"
                            },
                            {
                                "NMSENEVOLCONT": "Saída",
                                "IDSENEVOCONT":"S"
                            }
                        ]
                    }
                },
                {
                    "id": "06",
                    "name": "NMOPERADOR",
                    "label": "Operador Responsável",
                    "template": "field/select.html",
                    "class": "12",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "readOnly": false,
                    "dataSource": {
                        "name": "product#/operador",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "defaultValue": "@getLocalVar('CRM_NMOPERADOR')@",
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        },
                        {
                            "name": "IDATIVO",
                            "operator": "=",
                            "value": "S"
                        }
                    ],
                    "descriptionField": "NMOPERADOR",
                    "valueField": "CDOPEREVOL",
                    "outData":{
                        "CDOPERADOR" : "CDOPEREVOL",
                        "NMOPERADOR" : "NMOPERADOR"
                    },
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            
                            "name": "NMOPERADOR",
                            "label": "Operador",
                            "showOnList": true
                        }
                    ],
                    "validations": {},
                    "events": []
                },
                {
                    "id": "07",
                    "name": "CDOPEREVOL",
                    "label": "Código Operador",
                    "template": "field/datetime.html",
                    "showOnForm": false,
                    "showOnList": false,
                    "isVisible": false,
                    "readOnly": false,
                    "class": "6",
                    "defaultValue":"@templateManager.LOGGEDOPERATOR.CDOPERADOR@",
                    "events": []
                },
                {
                    "id": "08",
                    "name": "NMTIPOCONT",
                    "label": "Tipo de Contato",
                    "template": "field/select.html",
                    "showSelectAs": "grid",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": "6",
                    "readOnly": false,
                    "dataSource": {
                        "name": "product#/crud/tipocont",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        },
                        {
                            "name": "IDATIVO",
                            "operator": "=",
                            "value": "S"
                        }
                    ],
                    "descriptionField": "NMTIPOCONT",
                    "valueField": "CDTIPOCONT",
                    "fields": [
                        {
                            
                            "name": "NMTIPOCONT",
                            "label": "Tipo de Contato",
                            "showOnList": true
                        }
                    ],
                    "validations":{  
                        "required":true
                    },
                    "events": []
                },
                {
                    "id": "09",
                    "name": "NMMOTIEVOCON",
                    "label": "Motivo da Evolução",
                    "template": "field/select.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "readOnly": false,
                    "dataSource": {
                        "name": "product#/motievolcont_crm_perfiresult_crm_perfioper",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        }
                    ],
                    "outData": {
                        "NMMOTIEVOCON": "NMMOTIEVOCON",
                        "CDMOTIEVOCON": "CDMOTIEVOCON",
                        "IDENVIAEMAIL": "IDENVIAEMAIL",
                        "NMANEXOPADRAO": "NMANEXOPADRAO",
                        "DSCAMINHOANEXOPADRAO": "DSCAMINHOANEXOPADRAO"
                    },
                    "descriptionField": "NMMOTIEVOCON",
                    "valueField": "CDMOTIEVOCON",
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            
                            "name": "NMMOTIEVOCON",
                            "label": "Motivo da Evolução",
                            "showOnList": true
                        }
                    ],
                    "validations":{  
                        "required":true
                    },
                    "events":[
                        {
                            "name":"FieldOnChange",
                            "code":"evolutionTabController.checkMotievolcont(args.owner.field.widget)"
                        }
                    ]
                },
                {
                    "id": "010",
                    "name": "NMINTERLOCUTOR",
                    "label": "Interlocutor",
                    "template": "field/select.html",
                    "showSelectAs": "grid",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "readOnly": false,
                    "class": "6",
                    "valueField": "NRSEQINTERLOCUTOR",
                    "descriptionField": "NMINTERLOCUTOR",
                    "dataSource": {
                        "name": "product#/crm_interlocutor",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                   
                        {
                            "name": "NRPARCNEGOCIO",
                            "operator": "=",
                            "value": "@General.getFieldValue('NRPARCNEGOCIO', '"+activeWidget.name+"')@"
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        }
                    ],
                    "outData": {
                        "NRSEQINTERLOCUTOR": "NRSEQINTERLOCUTOR",
                        "EMAILINTERLOC": "EMAILINTERLOC",
                        "NRTELEFONE1": "NRTELEFONE1",
                        "NMINTERLOCUTOR": "NMINTERLOCUTOR"
                    },
                    "fields": [
                        {
                            
                            "name": "NMINTERLOCUTOR",
                            "label": "Nome Interlocutor",
                            "isVisible": true,
                            "showOnList": true
                        }
                    ],
                    "events": [
                        {
                            
                            "name": "FieldAfterSelectOpen",
                            "code": "args.owner.field.reload()"
                        }
                    ],
                    "filterProperties": {
                        "isVisible": false,
                        "showOnForm": false
                    }
                },
                {
                    "id": "12932085129618812781121755128",
                    "name": "ESTADOINTERLOCUTOR",
                    "label": "Estado (Interlocutor)",
                    "template": "field/text-edit.html",
                    "showOnForm": false,
                    "showOnView": false,
                    "showOnList": true,
                    "isVisible": true,
                    "events": []
                },
                {
                    "id": "12932085112296181287811127559",
                    "name": "CIDADEINTERLOCUTOR",
                    "label": "Cidade (Interlocutor)",
                    "template": "field/text-edit.html",
                    "showOnForm": false,
                    "showOnView": false,
                    "showOnList": true,
                    "isVisible": true,
                    "events": []
                },
                {
                    "id": "011",
                    "name": "NMSEQRESULEVOL",
                    "label": "Resultado",
                    "template": "field/select.html",
                    "showSelectAs": "grid",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": "6",
                    "dataSource": {
                        "name": "product#/crm_resultevol_crm_perfiresult_crm_perfioper",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {  
                            "name":"P_NRORG",
                            "operator":"=",
                            "value":"@getLocalVar('CRM_NRORGTRAB')@"
                        },
                        {  
                            "name":"IDRESULTEVOLUCAO",
                            "operator":"=",
                            "value":"SIM"
                        }
                    ],
                    "descriptionField": "DESCRESULEVOL",
                    "valueField": "NRSEQRESULEVOL",
                    "fields": [
                        {
                            
                            "name": "DESCRESULEVOL",
                            "label": "Resultado da Evolução",
                            "showOnList": true
                        }
                    ],
                    "outData": {
                        "IDGERPROXATIV": "NEWTASK",
                        "DESCRESULEVOL": "NMSEQRESULEVOL",
                        "NRSEQRESULEVOL":"NRSEQRESULEVOL"
                    },
                    "validations":{  
                        "required":true
                    },
                    "events": [
                        {
                            
                            "name": "FieldOnChange",
                            "code": "EvolucaoController.checkNewTask(args.owner.field.widget)"
                        }
                    ]
                },
                {
                    "id": "012",
                    "name": "TXEVOLCONT",
                    "label": "Descrição",
                    "template": "field/textarea.html",
                    "isVisible": true,
                    "showOnForm": true,
                    "class": "12",
                    "validations":{  
                        "required":true
                    },
                    "textRows": 5
                },
                {
                    "id": "013",
                    "name": "DTPLANFILACONTA",
                    "label": "Data da Próxima Tarefa",
                    "template": "field/datetime.html",
                    "showOnForm": true,
                    "showOnList": false,
                    "isVisible": true,
                    "readOnly": true,
                    "class": "6",
                    "fieldGroup": "FG_AGENDAR_TAREFA",
                    "validations": {},
                    "events": []
                },
                {
                    "id": "014",
                    "name": "NMOPERADORREAL",
                    "label": "Operador",
                    "template": "field/select.html",
                    "showSelectAs": "grid",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnView": true,
                    "showOnList": false,
                    "readOnly": true,
                    "fieldGroup": "FG_AGENDAR_TAREFA",
                    "class": "6",
                    "dataSource": {
                        "name": "product#/operador",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "defaultValue": "@getLocalVar('CRM_NMOPERADOR')@",
                    "itemsPerPage": 50,
                    "dataSourceFilter": [
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": "@getLocalVar('CRM_NRORGTRAB')@"
                        },
                        {
                            "name": "IDATIVO",
                            "operator": "=",
                            "value": "S"
                        }
                    ],
                    "descriptionField": "NMOPERADORREAL",
                    "valueField": "CDOPERREAL",
                    "fields": [
                        {
                            
                            "name": "NMOPERADOR",
                            "label": "Operador",
                            "showOnList": true
                        }
                    ],
                    "outData": {
                        "NMOPERADOR": "NMOPERADORREAL",
                        "CDOPERADOR": "CDOPERREAL"
                    },
                    "validations": {},
                    "events": []
                },
                {
                    "id": "015",
                    "name": "CDOPERREAL",
                    "label": "Código Operador",
                    "template": "field/datetime.html",
                    "showOnForm": false,
                    "showOnList": false,
                    "isVisible": false,
                    "readOnly": false,
                    "class": "6",
                    "defaultValue":"@templateManager.LOGGEDOPERATOR.CDOPERADOR@",
                    "events": []
                },
                {
                    "id": "016",
                    "name": "DSFILACONTATO",
                    "label": "Assunto da Tarefa",
                    "template": "field/text-edit.html",
                    "showOnForm": true,
                    "fieldGroup": "FG_AGENDAR_TAREFA",
                    "readOnly": true,
                    "showOnList": true,
                    "isVisible": true,
                    "defaultValue": "RETORNAR CONTATO",
                    "class": "12",
                    "events": []
                }
            ],
            "actions": [
                {
                    "id": "017",
                    "name": "BACK_VIEW",
                    "label": "Voltar",
                    "isVisible": true,
                    "preventHide":true,
                    "showAsAction": "back_view",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetGoBackToList(args.owner.widget);"
                        }
                    ]
                },
                {
                    "id": "018",
                    "name": "BACK_LIST",
                    "label": "Voltar",
                    "isVisible": true,
                    "preventHide":true,
                    "showAsAction": "back_list",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetGoBackToParent(args.owner.widget);"
                        }
                    ]
                },
                {
                    "id": "019",
                    "name": "BACK_EDIT",
                    "label": "Cancelar",
                    "isVisible": true,
                    "showAsAction": "back_edit",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetCancelForm(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "020",
                    "name": "EDIT",
                    "label": "Editar",
                    "icon": "pencil",
                    "isVisible": true,
                    "showAsAction": "view",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetEdit(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "021",
                    "name": "ADD",
                    "label": "Adicionar",
                    "icon": "plus",
                    "showAsAction": "always|list",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "General.widgetNewRow(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "022",
                    "name": "SAVE",
                    "label": "Salvar",
                    "showAsAction": "never_edit",
                    "events": [
                        {
                            
                            "name": "ActionEvent",
                            "code": "EvolucaoController.checkEvolution(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "023",
                    "name": "DELETE",
                    "label": "Excluir",
                    "icon": "trash",
                    "isVisible": false,
                    "preventHide": true,
                    "showAsAction": "view",
                    "events": [
                        {
                            "id": "024",
                            "name": "ActionEvent",
                            "code": "General.deleteCrud(args.owner.widget);"
                        }
                    ]
                }
            ],
            "events": [
                {
                    "id": "025",
                    "name": "WidgetOnEnterOnce",
                    "code": "CRM_Utilities.factoryWidgetActions(args.owner)"
                },
                {
                    "id": "026",
                    "name": "WidgetOnEnter",
                    "code": "CRM_Utilities.setDataSourceFilterOnDynamicTabs(args.owner, templateManager.container.getWidget('" + activeWidget.name + "'))"
                }
            ],
            "widgets": [
                {
                    "id": "1722319758311315390514",
                    "container":activeWidget.container,
                    "name": "WDGT_ANEXOS_EVOLUCAO",
                    "label": "Anexos",
                    "isVisible": true,
                    "showDescriptionOrder": false,
                    "template": "widget/master_detail/grid.html",
                    "dataSource": {
                        "name": "/crm_anexocontato",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "NRSEQCONTATO",
                            "operator": "=",
                            "value": "@General.getFieldValue('NRSEQCONTATO', 'WDGT_EVOLUCAO')@"
                        },
                        {
                            "name": "NRORIGEM",
                            "operator": "=",
                            "value": "@General.getFieldValue('NRSEQEVOLCONT', 'WDGT_EVOLUCAO')@"
                        }
                    ],
                    "events": [
                        {
                            "id": "17223197583113153905171",
                            "name": "WidgetOnEnter",
                            "code": "args.owner.reload()"
                        }
                    ],
                    "actions": [
                        {
                            "id": "1722319758311315390516",
                            "name": "DOWNLOAD",
                            "label": "Download",
                            "isVisible": true,
                            "showAsAction": "never_view",
                            "events": [
                                {
                                    "id": "030",
                                    "name": "ActionEvent",
                                    "code": "EvolucaoController.downloadFile(args.owner.widget);"
                                }
                            ]
                        },
                        {
                            "id": "1722319758311315390517",
                            "name": "ADD",
                            "label": "Adicionar",
                            "icon": "plus",
                            "showAsAction": "always|list",
                            "events": [
                                {
                                    "id": "032",
                                    "name": "ActionEvent",
                                    "code": "EvolucaoController.openAttachSwipe();"
                                }
                            ]
                        },
                        {
                            "id": "1722319758311315390518",
                            "name": "BACK_LIST",
                            "label": "Voltar",
                            "isVisible": true,
                            "preventHide":true,
                            "showAsAction": "back_list",
                            "events": [
                                {
                                    "id": "035",
                                    "name": "ActionEvent",
                                    "code": "General.widgetGoBackToParent(args.owner.widget);"
                                }
                            ]
                        },
                        {
                            "id": "172231975831131539051911",
                            "name": "BACK_VIEW",
                            "label": "Voltar",
                            "isVisible": true,
                            "preventHide":true,
                            "showAsAction": "back_view",
                            "events": [
                                {
                                    "id": "037",
                                    "name": "ActionEvent",
                                    "code": "General.widgetGoBackToList(args.owner.widget);"
                                }
                            ]
                        },
                        {
                            "id": "1722319758311315390591",
                            "name": "DELETE",
                            "label": "Excluir",
                            "icon": "trash",
                            "showAsAction": "view",
                            "events": [
                                {
                                    "id": "039",
                                    "name": "ActionEvent",
                                    "code": "General.deleteCrud(args.owner.widget);"
                                }
                            ]
                        }
                    ],
                    "fields": [
                        {
                            "id": "1722319758311315390592",
                            "name": "NRORIGEM",
                            "label": "Código",
                            "template": "field/text-edit.html",
                            "class": "6",
                            "showOnList": false,
                            "isVisible": false,
                            "readOnly": false
                        },
                        {
                            "id": "1722319758311315390593",
                            "name": "NMARQUIVO",
                            "label": "Nome do arquivo",
                            "class": "12",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "readOnly": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "1722319758311315390594",   
                            "name": "DSTIPO",
                            "label": "Extensão",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "readOnly": false,
                            "template": "field/text-edit.html"
                        },
                        {
                            "id": "1722319758311315390595",
                            "name": "VRTAMANHO",
                            "label": "Tamanho do arquivo(bytes)",
                            "class": "6",
                            "isVisible": true,
                            "showOnList": true,
                            "showOnView": true,
                            "readOnly": false,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "widgets": [
                        {
                            "id": "1722319758311315390515",
                            "container":activeWidget.container,
                            "name": "WDGT_SELECAO_ARQUIVOS",
                            "label": "Anexar",
                            "isVisible": false,
                            "template": "widget/form.html",
                            "dataSource": {},
                            "actions": [
                                {
                                    "id": "1722319758311315390519112",
                                    "name": "uploadFile",
                                    "isVisible": true,
                                    "label": "Anexar",
                                    "showAsAction": "never",
                                    "events": [
                                        {
                                            "id": "046",
                                            "name": "ActionEvent",
                                            "code": "EvolucaoController.uploadEvolutionFiles(args.owner.widget);"
                                        }
                                    ]
                                },
                                {
                                    "id": "1722319758311315390519113",
                                    "name": "BACK",
                                    "label": "Cancelar",
                                    "isVisible": true,
                                    "showAsAction": "back",
                                    "events": [
                                        {
                                            "id": "1722319758311315390519114",
                                            "name": "ActionEvent",
                                            "code": "EvolucaoController.closeAttachSwipe(args.owner.widget);"
                                        }
                                    ]
                                }
                            ],
                            "events": [
                                {
                                    "id": "1722319758311315390597",
                                    "name": "WidgetOnEnter",
                                    "code": "args.owner.newRow();"
                                }
                            ],
                            "fields": [
                                {
                                    "id": "1722319758311315390598",
                                    "name": "DSCAMINHO",
                                    "placeholder": "Anexar arquivo",
                                    "class": "10 col-xs-offset-1",
                                    "isVisible": true,
                                    "readOnly": false,
                                    "descriptionOrder": 1,
                                    "validations": {
                                        "required": true
                                    },
                                    "template": "field/file-upload.html",
                                    "isMultiple": true
                                }
                            ]
                        }
                    ]
                }
            ]
        };
        
        var evolutionTabWidget = metaDataFactory.widgetFactory(evolutionTabWidgetJson);
        
        return evolutionTabWidget;
        
    }
    
    
    
    /**
     * Insere a widget de evoluções na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá a tab.
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */
    this.setEvolutionTabOnActiveWidget = function(widget){
        
        widget.widgets.push(_factoryEvolutionTab(widget));
        
    };
    
    /**
     * Sobreescreve qualquer action da tab.
     * 
     * @param {string} actionName nome da action que será sobrescrita.
     *   BACK_VIEW -> Voltar para o grid da tab.
     *   BACK_LIST -> Voltar para o grid principal.
     *   BACK_EDIT -> Cancelar Edição.
     *   EDIT      -> Editar.
     *   ADD       -> Nova Evolução.
     *   SAVE      -> Salvar.
     *   DELETE    -> Excluir.
     * 
     * @param {string} code nome do controller + nome do metodo a ser executado.
     * Ex.:XYController.xyMethod()
    */
    this.overwriteAction = function(actionName, code){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        if(!evolutionWidget.getAction(actionName).events[0].defaultCode)
            evolutionWidget.getAction(actionName).events[0].defaultCode = evolutionWidget.getAction(actionName).events[0].code;
        templateManager.container.getWidget('WDGT_EVOLUCAO').getAction(actionName).events[0].code = code;    
    };
    
    /**
     * Volta para o evento defalut de qualquer action.
     * 
     * @param {string} actionName nome da action que será sobrescrita.
     *   BACK_VIEW -> Voltar para o grid da tab.
     *   BACK_LIST -> Voltar para o grid principal.
     *   BACK_EDIT -> Cancelar Edição.
     *   EDIT      -> Editar.
     *   ADD       -> Nova Evolução.
     *   SAVE      -> Salvar.
     *   DELETE    -> Excluir.
     * 
    */
    this.defaultAction = function(actionName){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        if(evolutionWidget.getAction(actionName).events[0].defaultCode)
            evolutionWidget.getAction(actionName).events[0].code = evolutionWidget.getAction(actionName).events[0].defaultCode;
    };
    
    /**
     * Executa comandos após toda a rotina save da evolução.
     * 
     * @param {array} arrayOfMethods array de strings, cada posição 
     * um comando a ser executado.
     * Ex.:[
     *      "ScreenService.closeSwipe(true)",
     *      "XYController.xyMethod()"
     *    ]
    */
    this.setMethodsAfterSave = function(arrayOfMethods){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        if(evolutionWidget.afterSave)
            evolutionWidget.afterSave.push(arrayOfMethods);
        else
            evolutionWidget.afterSave = arrayOfMethods;
    };
    
    
    /**
     * Builda novos fieldGroups na tab e chama o metodo de inserção de fields.
     * 
     * @param {array}  arrayOfFields array de fields(json).
     * @param {string} fieldGroupName nome do fieldGroup a ser inserido.
     * 
    */
    this.buildDynamicFieldGroup = function(arrayOfFields, fieldGroupName, fieldGroupLabel){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        
        var fieldGroup = {
            "name":fieldGroupName,
            "label":fieldGroupLabel,
            "colapsible":true,
            "opened":true,
            "isVisible":true,
            "dynamic":true
        };
        if(this.checkFieldGroup(fieldGroup, evolutionWidget.fieldGroups) === -1){
            if(evolutionWidget.fieldGroups)
                evolutionWidget.fieldGroups.push(fieldGroup);
            else
                evolutionWidget.fieldGroups = [fieldGroup];
            
            evolutionWidget.fieldGroups = evolutionWidget.fieldGroups.map(function(item) {
    			item.widget = evolutionWidget;
    			return Util.extend(new FieldGroup(), item);
    		});
    		//Chama o metodo de inserção
            _insertFields(arrayOfFields, evolutionWidget);
        }
    };
    
    this.checkFieldGroup = function(fieldGroup, fieldGroups){
        var indexField = -1;
        fieldGroups.forEach(function(item, index){
            if(item.name === fieldGroup.name)
                indexField = index;
        });    
        return indexField;
    }
    
    /**
     * Builda os novos Fields na widget
     * 
     * @param {array}  arrayOfFields array de fields(json).
     * @param {string} fieldGroupName nome do fieldGroup a ser inserido.
     * 
    */
    function _insertFields(arrayOfFields, widget){
        arrayOfFields.forEach(function(field){
            field.dynamic = true;
            metaDataFactory.fieldFactory(field, widget, true);  
        });
    }
    
    /**
     * Esconde determinados fieldGroups
     * 
     * @param {string/true} fieldGroupName nome do field
     * group a ser escondido, se = true, esconde todos.
     * 
    */
    this.hideFieldGroup = function(fieldGroupName){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        
        if(typeof fieldGroupName === "string"){
            evolutionWidget.fieldGroups.forEach(function(fg){
                if(fg.name === fieldGroupName && fg.dynamic){
                    fg.isVisible = false;
                }
            });    
        }else if(fieldGroupName && typeof fieldGroupName === "boolean"){
            evolutionWidget.fieldGroups.forEach(function(fg){
                if(fg.dynamic)
                    fg.isVisible = false;
            });   
        }
        
        evolutionWidget.fields.forEach(function(field){
            if(field.fieldGroup && (field.fieldGroup === fieldGroupName || fieldGroupName === true)){
                if(field.dynamic)
                    field.isVisible = false;
            }
        });
    };
    
    /**
     * Mostra determinados fieldGroups
     * 
     * @param {string/true} fieldGroupName nome do field
     * group a ser mostrado, se = true, mostra todos.
     * 
    */
    this.showFieldGroup = function(fieldGroupName){
        var evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        
        if(typeof fieldGroupName === "string"){
            evolutionWidget.fieldGroups.forEach(function(fg){
                if(fg.name === fieldGroupName && fg.dynamic)
                    fg.isVisible = true;
            });    
        }else if(fieldGroupName && typeof fieldGroupName === "boolean"){
            evolutionWidget.fieldGroups.forEach(function(fg){
                if(fg.dynamic)
                    fg.isVisible = true;
            });   
        }
        
        evolutionWidget.fields.forEach(function(field){
            if(field.fieldGroup && (field.fieldGroup === fieldGroupName || fieldGroupName === true)){
                if(field.dynamic)
                    field.isVisible = true;
            }
        });
    };
    
    /**
     * Verifica se o motivo selecionado permite enviar email
     * para o interlocutor
     * 
     * @param {object} evolWidget  Widget de Evolução
     * 
    */
    this.checkMotievolcont = function(evolWidget){
        widgetMail.fields.forEach(function(field, index){
            if(field.name === 'DSCAMINHO')
                widgetMail.fields.splice(index,1);
        });
        if(evolWidget.currentRow.IDENVIAEMAIL === 'S'){
            //Salva o Anexo default no popup de email
            var file = {
                name: evolWidget.currentRow.NMANEXOPADRAO || '',
                path: evolWidget.currentRow.DSCAMINHOANEXOPADRAO || ''
            };
            this.insertAttachField(file);    
        }
    };
    
    this.insertAttachField = function(file){
        var field = {
            "name": "DSCAMINHO",
            "placeholder": "Anexar arquivo",
            "class": "12",
            "isVisible": true,
            "readOnly": false,
            "validations": {},
            "template": "field/file-upload.html",
            "isMultiple": true,
            "file":file
        };
        widgetMail.fields.push(field);     
    };
    
    var widgetMail = 
        {
            "name": "mensagens",
            "label": "Notificação de Interlocutor",
            "isVisible": false,
            "showAsTab": false,
            "template": "widget/form.html",
            "events": [
                {
                    "id": "6182828512039293853658",
                    "name": "WidgetOnEnter",
                    "code": "args.owner.reload();"
                }
            ],
            "actions": [
                {
                    "id": "618282851568652067654",
                    "name": "enviar",
                    "isVisible": true,
                    "label": "Enviar",
                    "showAsAction": "never",
                    "events": [
                        {
                            "id": "6182828514145614549655",
                            "name": "ActionEvent",
                            "code": "evolutionTabController.notificarInterlocutor(args.owner.widget, args.owner.widget.backuprow)"
                        }
                    ]
                },
                {
                    "id": "6182828511372890396656",
                    "name": "cancel",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "events": [
                        {
                            "id": "6182828512205401251657",
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup(args.owner.widget);evolutionTabController.removeMessagesWidget(args.owner.widget.parent);"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "id": "6182828513466738669652",
                    "name": "DSTITULOMENSAGEM",
                    "label": "Assunto da Mensagem",
                    "isVisible": true,
                    "class": 12,
                    "descriptionOrder": 1,
                    "events": [],
                    "showOnList": true,
                    "showOnForm": true,
                    "showOnView": true,
                    "template": "field/text-edit.html",
                    "validations": {
                        "required": {},
                        "maxLength": {
                            "length": 100
                        }
                    }
                },
                {
                    "id": "6182828511660973795653",
                    "name": "DSMENSAGEM",
                    "label": "Mensagem",
                    "class": 12,
                    "placeholder": null,
                    "isVisible": true,
                    "showOnList": false,
                    "showOnForm": true,
                    "showOnView": true,
                    "template": "zh-rich-text#field/rich-text.html",
                    "height": 150,
                    "toolbar": {
                        "fontname": true
                    },
                    "validations": {
                        "required": {}
                    },
                    "events": [],
                    "filterProperty": {
                        "isVisible": false,
                        "showOnForm": false
                    }
                }
            ]
        };

    this.removeMessagesWidget = function(widget){
        widget.widgets.forEach(function(wid, index){
            if(wid.name === "mensagens")
                widget.widgets.splice(index, 1);
        });      
        widget.activate();
    };
    
    this.openWidgetSendMail = function(widget){
        var promise = ZHPromise.defer();
        if(widget.currentRow.EMAILINTERLOC){
            
            widgetMail.container = widget.container;
            widgetMail = metaDataFactory.widgetFactory(widgetMail);
            widgetMail.parent = widget;
            widget.widgets.push(widgetMail);
            
            widgetMail.currentRow.DSMENSAGEM = '<table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee" style="background-color: #eeeeee;"> <tbody> <tr> <td> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="0274BA" class="container590" style="background-color: #0274ba; text-align: center;"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="center" valign="middle"> <table border="0" align="middle" cellpadding="0" cellspacing="0" style="border-collapse:collapse; mso-table-lspace:0pt; mso-table-rspace:0pt;" class=""> <tbody> <tr> <td align="center" valign="middle"> <p style="color: #ffffff; font-size: 25px; padding:20px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 25px; padding-top:45px;">Notificação de Evolução</p> <p style="color: #ffffff; font-size: 16px; font-family: "Questrial", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"> [DATAEVOLUCAO] </p> <p style="color: #ffffff; font-size: 16px; font-family: "Questrial", Helvetica, sans-serif; line-height: 15px; padding-top:15px;">&nbsp;</p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"> <br> </p> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> <table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee"> <tbody> <tr> <td align="center"> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="ffffff" class="container590 bg_color"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="left" style="color: #39434b; font-size: 20px; font-family: " Montserrat ", Helvetica, sans-serif; font-weight: 400; padding-bottom:15px; padding-top:20px; line-height: 35px;" class="title_color"> <div style="padding:20px; line-height: 22px"> Prezado(a) [INTERLOCUTOR]</div> </td> </tr> <tr> <td align="left" class="title_color" style="color: #848999; font-size: 14px; font-family: " Questrial ", sans-serif; mso-line-height-rule: exactly; line-height: 24px; padding-bottom:15px;"> <div style="line-height: 24px;"> <br> </div> </td> </tr> <tr> <td height="10" style="font-size: 10px; line-height: 10px;">&nbsp;</td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> ';
            //Seta o anexo default no field de upload
            widgetMail.currentRow.DSCAMINHO  = [widgetMail.getField('DSCAMINHO').file];
            setTimeout(function(){
                widgetMail.getField('DSCAMINHO').fileList = widgetMail.currentRow.DSCAMINHO;
            },100);
            widgetMail.currentRow.DATAEVOLUCAO = widget.currentRow.DTHREVOLCONT;
            widgetMail.backuprow = angular.copy(widget.currentRow);
            ScreenService.openPopup(widgetMail).then(function(){
                promise.resolve();
                widgetMail.activate();
            });
        }else{
            ScreenService.showMessage('Interlocutor não possui email cadastrado!').then(function(){
               promise.resolve(); 
            });
        }
        return promise.promise;
    };
    
    this.notificarInterlocutor = function(widget, row){
        var evolWidget = templateManager.container.getWidget('evolucao');
        if(!evolWidget){
            evolWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
            evolWidget.currentRow = row;    
        }
        
        var messagesWidget = templateManager.container.getWidget('mensagens');
        messagesWidget.fields.forEach(function(field, index){
            if(field.name === 'DSCAMINHO')
                messagesWidget.fields.splice(index,1);
        }); 
        
        var files = messagesWidget.currentRow.DSCAMINHO;
        var rows = [];
        
        files.forEach(function(file, fileIndex){
            rows.push(this.setValuesForFileUpload({DSCAMINHO:widget.currentRow.DSCAMINHO}, fileIndex, evolWidget.currentRow));
            //Se for o anexo default
            if(!Util.isEmptyOrBlank(file.path)){
                rows[0].DEFAULTATTACH = true;
            }
            
        }.bind(this));
        
        General.callRestEngine("product#/saveAttachEvol", "DataSet", rows).then(function() {
            
                ScreenService.closePopup(true);
                widget.currentRow.NMINTERLOCUTOR  = evolWidget.currentRow.NMINTERLOCUTOR;
                widget.currentRow.DSEMAILINTERLOC = evolWidget.currentRow.EMAILINTERLOC;
                widget.currentRow.NRORG           = templateManager.LOGGEDOPERATOR.NRORG;
                widget.currentRow.NMOPERADORRESP  = evolWidget.currentRow.NMOPERADOR;
                widget.currentRow.NRSEQEVOLCONT   = evolWidget.currentRow.NRSEQEVOLCONT;
                General.callRestEngine("product#/notificaInterlocutorEvolucao", "Row", widget.currentRow);
                this.removeMessagesWidget(evolWidget);
                evolWidget.activate();
                
        }.bind(this));
        if(templateManager.promise)
            templateManager.promise.resolve();
    };
    
    /**
     * Função para retornar uma row configurada
     * com as informações do anexo referente
     * ao ID, recebido como parâmetro, no array
     * de anexos na currentRow.
     * 
     * @param {object} widget widget onde foi selecionado
     * os anexos.Array
     * 
     * @param {int} fileIndex index do anexo no array de 
     * anexos presente na currentRow.
     * 
     * @return {object} currentRow row configurada com as informações
     * do anexo referente ao ID recebido.
    */
    this.setValuesForFileUpload = function(row, fileIndex){
        
        var currentRow = angular.copy(row);
        var evolutionWidget = templateManager.container.getWidget('evolucao');
        if(!evolutionWidget){
            evolutionWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        }
        var loggedOperatorInfos = templateManager.LOGGEDOPERATOR;
        
        currentRow.NRSEQCONTATO = evolutionWidget.currentRow.NRSEQCONTATO;
        currentRow.NMCOMPLETO = currentRow.DSCAMINHO[fileIndex].name;
        currentRow.NMARQUIVO = this.getFileNameWithoutExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.DSTIPO = this.getFileExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.VRTAMANHO = currentRow.DSCAMINHO[fileIndex].size;
        currentRow.IDATIVO = "S";
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        currentRow.CDOPERADOR = loggedOperatorInfos.CDOPERADOR;
        currentRow.IDORIGEM = "EVOLUCAO";
        currentRow.NRORIGEM = evolutionWidget.currentRow.NRSEQEVOLCONT;
        
        return currentRow;
        
    };
    
    /**
     * Função para retornar a extenção de um arquivo
     * extraida do seu nome.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileExtension = function(fileName){
        
        return fileName.replace(/^(.*)\.(.*)$/, "$2");
        
    };
    
    /**
     * Função para retornar o nome do arquivo sem
     * sua extenção.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileNameWithoutExtension = function(fileName){
        
        return fileName.replace(/^(.*)\.(.*)$/, "$1");
            
    };
	
}
Configuration(function(ContextRegister){
    ContextRegister.register('evolutionTabController', evolutionTabController);
});

// FILE: ./assets/js/repositories/Operador_Repository.js
var Operador_Repository = function(localStorageEngine) {
    
    this.getOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];
        
        return localStorageEngine.download("product#/crud/operador", filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Operador_Repository", Operador_Repository);
});

// FILE: ./assets/js/repositories/Sm00001000_PropostaAlimenticia_Repository.js
var Sm00001000_PropostaAlimenticia_Repository = function(localStorageEngine) {
    /**
     * Salva no localStorage o dataSource de uma tela
     * @params{String} name - nome da widget
     * @params{Array} data - dataSource
    **/
    this.saveDataLocalStorageEngine = function(name, data) {
        localStorageEngine._setItem(name, data);
    };
    /**
     * retorna do localStorage um dataSource de uma tela
     * @params{String} name - nome do data na sessao
    **/
    this.getDataLocalStorageEngine = function(name) {
        return localStorageEngine.getDataSourceItems(name);
    };
    /**
     * deleta no localStorage um dataSource de uma tela
     * @params{String} name - nome do data na sessao
    **/
    this.removeDataLocalStorageEngine = function(name) {
        localStorageEngine._removeAll(name);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001000_PropostaAlimenticia_Repository", Sm00001000_PropostaAlimenticia_Repository);
});

// FILE: ./assets/js/repositories/Sm00002000_Modelo_Proposta_Repository.js
function Sm00002000_Modelo_Proposta_Repository(restEngine, ScreenService){
   
    /**
     * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo.
     */
    
    this.validaEscopoProposta = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relacescopomodelo_widget/validaEscopoProposta', paramsRestEngine);
        
    };
    
    /**
     * A função "validaCompProposta" verifica se a composição 
     * que está sendo relacionada já está relacionada ao modelo.
     */
    
    this.validaCompProposta = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relorcamodcomposicao_widget/validaCompProposta', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00002000_Modelo_Proposta_Repository", Sm00002000_Modelo_Proposta_Repository);
});


// FILE: ./assets/js/repositories/Sm00003000_Escopo_Proposta_Repository.js
function Sm00003000_Escopo_Proposta_Repository(restEngine, ScreenService){
   
     /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
     
    this.validaServicoEscopo = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_itescopoproposta_widget/validaServicoEscopo', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00003000_Escopo_Proposta_Repository", Sm00003000_Escopo_Proposta_Repository);
});


// FILE: ./assets/js/repositories/Sm00004000_Contrato_Repository.js
function Sm00004000_Contrato_Repository(restEngine, ScreenService){
   
     /**
     * A função "copiaVersionaContrato" executa a procedure
     * "FCT_COPIA_VERSIONA_CONTRATO".
     */
     
    this.copiaVersionaContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/copiaVersionaContrato', paramsRestEngine);
        
    };
    
     /**
     * A função "encerraContrato" executa a procedure
     * "FCT_ENCERRA_CONTRATO".
     */
     
    this.encerraContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/encerraContrato', paramsRestEngine);
        
    };
    
    
     /**
     * A função "geraParcelasContrato" executa a procedure
     * "FCT_VENCPREV_CONT".
     */
     
    this.geraParcelasContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/geraParcelasContrato', paramsRestEngine);
        
    };
    
     /**
     * A função "cancelaParcelasContrato" executa a procedure
     * "FCT_CANC_VENCPREV_CONT".
     */
     
    this.cancelaParcelasContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/cancelaParcelasContrato', paramsRestEngine);
        
    };
    
    /**
     * A função "alteraVigenciaContrato" altera os dados de vigência do contrato
     */
     
    this.alteraVigenciaContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_prorrogcontrato_widget/alteraVigenciaContrato', paramsRestEngine);
        
    };
    
    
    /**
     * A função "validaDadosProrrog" valida se ja existem alterações de vigência para o contrato
     */
     
    this.validaDadosProrrog = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_prorrogcontrato_widget/validaDadosProrrog', paramsRestEngine);
        
    };
    
    /**
     * A função "alteraDadosProrrog" inativa as alterações de vigência do contrato
     */
     
    this.alteraDadosProrrog = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_prorrogcontrato_widget/alteraDadosProrrog', paramsRestEngine);
        
    };
    
    /**
     * A função "recebeParamFatModCont" recebe os dados de Faturamento do Modelo de Contrato
     */
     
    this.recebeParamFatModCont = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_paramfatmodcont_widget/recebeParamFatModCont', paramsRestEngine);
        
    };
    
    /**
     * A função "recebeModReajustContr" recebe os dados de Reajuste do Modelo de Contrato
     */
     
    this.recebeModReajustContr = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_modreajustcontr_operador/recebeModReajustContr', paramsRestEngine);
        
    };
    
    /**
     * A função "recebeModIndContr" recebe os dados de Indice de Reajuste do Modelo de Contrato
     */
     
    this.recebeModIndContr = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_modindcontr_widget/recebeModIndContr', paramsRestEngine);
        
    };
    
    /**
     * A função "recebeModeloDocContr" recebe os dados de Documento do Modelo de Contrato
     */
     
    this.recebeModeloDocContr = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_modelodoccontr_widget/recebeModeloDocContr', paramsRestEngine);
        
    };
    
    /**
     * A função "geraCodigoContrato" executa a procedure  "CRIA_MNEMONICO".
     */
     
    this.geraCodigoContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/geraCodigoContrato', paramsRestEngine);
        
    };
    
    /**
     * A função "geraDocumentoContrato" executa a procedure "FCT_SUB_MNE_DOC".
     */
     
    this.geraDocumentoContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_contrato_widget/geraDocumentoContrato', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00004000_Contrato_Repository", Sm00004000_Contrato_Repository);
});


// FILE: ./assets/js/repositories/Sm00004005_Parametrizacao_Contrato_Repository.js
function Sm00004005_Parametrizacao_Contrato_Repository(restEngine, ScreenService){
   
     /**
     * A função "insereDadosContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Contrato (FCT_CONTRATO).
     */
    
    this.insereDadosContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_paramfatcont_widget/insereDadosContrato', paramsRestEngine);
        
    };
    
    
    this.insereDadosItContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_paramfatitcont_widget/atualizarDadosItContrato', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00004005_Parametrizacao_Contrato_Repository", Sm00004005_Parametrizacao_Contrato_Repository);
});


// FILE: ./assets/js/repositories/Sm00006000_Elemento_Repository.js
function Sm00006000_Elemento_Repository(restEngine, ScreenService){
   
    /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
    
    this.validaServicoRelacionado = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relacservico_widget/validaServicoRelacionado', paramsRestEngine);
        
    };
    
     /**
     * A função "validaServicoRelacionadoUnit" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço para inclusão 
     * de um só item.
     */
    
    this.validaServicoRelacionadoUnit = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relacservico_widget/validaServicoRelacionadoUnit', paramsRestEngine);
        
    };
    
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006000_Elemento_Repository", Sm00006000_Elemento_Repository);
});


// FILE: ./assets/js/repositories/Sm00006000_Servicos_Repository.js
function Sm00006000_Servicos_Repository(restEngine, ScreenService){
   
     /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
     
    this.validaServicoRelacionado = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_relacservico_widget/validaServicoRelacionado', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006000_Servicos_Repository", Sm00006000_Servicos_Repository);
});


// FILE: ./assets/js/repositories/Sm00006001_Tabela_Preco_Servico_Repository.js
function Sm00006001_Tabela_Preco_Servico_Repository(restEngine, ScreenService){
   
     /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
     
    this.validaServicoTabela = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_itprecbaseserv_widget/validaServicoTabela', paramsRestEngine);
        
    };
    
      /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
     
    this.validaTipoMedidaVariavel = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_variacaoprecbaseserv_widget/validaTipoMedidaVariavel', paramsRestEngine);
        
    };
    
     /**
     * A função "sincronizarVariacaoPreco" executa a procedure "FCT_SINCRONIZAR_VARIACAO_PRECO".
     */
     
    this.sincronizarVariacaoPreco = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_itprecbaseserv_widget/sincronizarVariacaoPreco', paramsRestEngine);
        
    };
    
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006001_Tabela_Preco_Servico_Repository", Sm00006001_Tabela_Preco_Servico_Repository);
});


// FILE: ./assets/js/repositories/Sm00006002_Tipo_de_Medida_Repository.js
function Sm00006002_Tipo_de_Medida_Repository(restEngine){
   
     /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
     
    this.validaTipoMedidaVariavel = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_variacaotipomedida_fct_tipomedidaserv/validaTipoMedidaVariavel', paramsRestEngine);
        
    };
    
    /**
     * A função "validaExsiteMedidaVariavel" verifica se existem medidas variaveis 
     * para o tipo de medida atual.
     */
     
    this.validaExsiteMedidaVariavel = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_variacaotipomedida_fct_tipomedidaserv/validaExsiteMedidaVariavel', paramsRestEngine);
        
    };
    
     /**
     * A função "validaExsiteVariacao" verifica se o tipo de medida variavel esá sendo 
     * utilizado.
     */
     
    this.validaExsiteVariacao = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_variacaotipomedida_fct_tipomedidaserv/validaExsiteVariacao', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006002_Tipo_de_Medida_Repository", Sm00006002_Tipo_de_Medida_Repository);
});


// FILE: ./assets/js/repositories/Sm00009000_Contrato_Modelo_Repository.js
function Sm00009000_Contrato_Modelo_Repository(restEngine, ScreenService){
   
     /**
     * A função "insereDadosModContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Modelo de Contrato (FCT_MODELOCONTRATO).
     */
    
    this.insereDadosModContrato = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_paramfatmodcont_widget/insereDadosModContrato', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00009000_Contrato_Modelo_Repository", Sm00009000_Contrato_Modelo_Repository);
});


// FILE: ./assets/js/repositories/Sm00034000_Maximo_Carencia_Proposta_Repository.js
function Sm00034000_Maximo_Carencia_Proposta_Repository(restEngine, ScreenService){
   
    /**
     * A função "validaCarenciaOperador" verifica se já existe 
     * uma carencia cadastrada para o operador indicado.
     */
    
    this.validaCarenciaOperador = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_carenciaoper_parcnegocio/validaCarenciaOperador', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00034000_Maximo_Carencia_Proposta_Repository", Sm00034000_Maximo_Carencia_Proposta_Repository);
});


// FILE: ./assets/js/repositories/Sm_Percentual_Desconto_Operador_Repository.js
function Sm_Percentual_Desconto_Operador_Repository(restEngine, ScreenService){
   
    this.validaPercentDescontoOper = function(paramsRestEngine){
    
        return restEngine.post('product#/fct_percdescoper_widget/ValidaPercentDescontoOper', paramsRestEngine);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm_Percentual_Desconto_Operador_Repository", Sm_Percentual_Desconto_Operador_Repository);
});


// FILE: ./assets/js/repositories/ger00004500_Prospect_Repository.js
function ger00004500_Prospect_Repository(restEngine){
    
    this.callInsertTaskProspect = function(paramsRestEngine, currentRow){
        
        return restEngine.post('/insertNextTaskProspect', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
    };
    
    this.selectContactProspect = function(paramsRestEngine, currentRow){
        
        return restEngine.post('/selectProspectNrContato', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
    };
    
    this.requestForForInsertNrseqcontatoForNewOpportunity = function(paramsRestEngine){
        
        return restEngine.post('/insertNrseqcontatoForNewOpportunity', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForUpdatePartnerType = function(paramsRestEngine){
        
        return restEngine.post('/updatePartnerType', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForInsertPartnershipOfTypeClient = function(paramsRestEngine){
    
        return restEngine.post('/insertPartnershipOfTypeClient', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForInsertClient = function(paramsRestEngine){
        
        return restEngine.post('/insertClient', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForDeactivateProspect = function(paramsRestEngine){
        
        return restEngine.post('/deactivateProspect', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
          
    };
    
    this.requestForInsertClientAddress = function(paramsRestEngine){
    
        return restEngine.post('/insertClientAddress', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForSelectProspectAddress = function(paramsRestEngine){
      
        return restEngine.post('/selectProspectAddress', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
         
    };
    
    this.requestForSetTaskEndDate = function(paramsRestEngine){
      
        return restEngine.post('/setTaskEndDate/Prospect', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
         
    };
    
    this.requestForDeleteEvolutionFilesByOriginNumber = function(paramsRestEngine){
        
        return restEngine.post('/deleteFilesByOriginNumber', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
        
    };
    
    this.requestForDeactivateTask = function(paramsRestEngine){
      
        return restEngine.post('/deactivateProspectTask', paramsRestEngine,
            function(success){
                
            },undefined,
            function(error){
                
            },undefined,undefined
        );
         
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("ger00004500_Prospect_Repository", ger00004500_Prospect_Repository);
});


// FILE: ./assets/js/services/Sm00001000_LancamentoProposta_Service.js
function Sm00001000_LancamentoProposta_Service(General, ZHPromise, Utilities) {
    
    /**
     * 
     */
    this.validarElementosParaConclusao = function (row) {
        var params = {
            "NRSEQORCAMENTO"  : row.NRSEQORCAMENTO,
            "NRORG"           : row.NRORG,
            "NRORCAMENTO"     : row.NRORCAMENTO,
            "PERIODOEXTENDIDO": true,
            "DTFINVIGCONTR"   : row.DTFINPROPOSTA

        };
        return General.callRestEngine("product#/fct_orcamentoaliment_widget/concluirProposta", "Row", params);

    };

    this.validarPropostaParaConclusao = function(currentRow) {
        var msgErro = "";
        
        // Início da validação da Proposta
        
        if (!currentRow.NMPARCINTERLOCUTOR) {
            msgErro += "Interlocutor deve ser informado.<br/><br/>";
        }
        else {
            if (!currentRow.DSEMAIL) {
                msgErro += "O Interlocutor informado não possui e-mail cadastrado.<br/><br/>";
            }else{
                
                var validaEmail = Utilities.verificaEmail(currentRow.DSEMAIL);  
                
                if (validaEmail == false){
                    
                     msgErro += "O e-mail informado para o interlocutor não é valido.<br/><br/>";
                }
            }
        }
        
        if (!currentRow.DTVALIDADE) {
            msgErro += "Validade da Proposta (Data) deve ser informada.<br/><br/>";
        }
        
        if (!currentRow.QTDIASVALIDADE && currentRow.QTDIASVALIDADE != 0) {
            msgErro += "Validade da Proposta (Dias) deve ser informada.<br/><br/>";
        }
        
        // Fim da validação da Proposta
        
        if (msgErro.length > 0) {
            msgErro = msgErro.substring(0, msgErro.length-10);
        }
        
        return msgErro;
    };

    this.validarMoedaItens = function(nrseqorcamento) {
        var deferred = ZHPromise.defer();
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/getQuantidadeItensMoedaDiferente", "Row", params).then(function(response) {
            if (response.dataset.retorno.qtd > 0) {
                deferred.reject("Operação bloqueada! <br><br>Os itens da proposta foram inseridos com uma diferente da moeda atual da proposta. Favor atualizar o valor dos itens.");
            }
            else {
                deferred.resolve();
            }
        }).catch(function(error) {
            deferred.reject(error);
        });

        return deferred.promise;
    };
    
    /**
     * 
     */
    this.validarItensPropostaParaConclusao = function(nrseqorcamento, nrorg) {
        var deferred = ZHPromise.defer();
        var msgErro = "";
        
        var filtro = [
            {
                "name": "NRSEQORCAMENTO",
                "operator": "=",
                "value": nrseqorcamento
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": nrorg
            }
        ];
        
        General.callRestEngine("product#/crud/fct_itemorcamento_widget", "FilterData", filtro).then(function(success) {
            if (success.dataset) {
                
                var itens = success.dataset.fct_itemorcamento_widget;
                
                // Início da validação dos Itens
                for (var i = 0; i < itens.length; i++) {
                    
                    var tipo = (itens[i].IDADICITEMORC == "S") ? "Adicional " : "Item ";
                    
                    if (itens[i].IDFATURAORCA == "O") {
                        if (!itens[i].CDMODPGTO) {
                            msgErro += tipo + itens[i].NMITEMORCAMENTO + ": Modalidade de Pagamento deve ser informada.<br/><br/>";
                        }
                    }
                    
                    if (itens[i].IDFATURAORCA == "P") {
                        if (!itens[i].QTPARCELASFATU) {
                            msgErro += tipo + itens[i].NMITEMORCAMENTO + ": Quantidade de Parcelas deve ser informado.<br/><br/>";
                        }
                        
                        if (!itens[i].NRDIAVENCPARCELASFATU) {
                            msgErro += tipo + itens[i].NMITEMORCAMENTO + ": Dia para Vencimento deve ser informado.<br/><br/>";
                        }
                    }

                    if (!itens[i].VRTOTALITEMORCA && itens[i].VRTOTALITEMORCA != 0) {
                        msgErro += tipo + itens[i].NMITEMORCAMENTO + ": Valor Total deve ser informado.<br/><br/>";
                    }
                }
                // Fim da validação dos Itens
            }
            
            if (msgErro) {
                msgErro = msgErro.substring(0, msgErro.length-10);
                deferred.reject(msgErro);
            }
            else {
                deferred.resolve();
            }
        },
        function(error) {
            console.log(error);
            deferred.reject(error.data.error);
        });
        
        return deferred.promise;
    };
    
    this.getComposicaoDoElemento = function(row) {
        if (row._parent.TIPO == "C") {
            return row._parent;
        }
        else {
            return this.getComposicaoDoElemento(row._parent);
        }
    };
    
    
    
    this.validaPropReneg = function(nrseqorcamento){
        
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento

         };
         
         General.callRestEngine("product#/fct_orcamento_widget/validaPropReneg", "Row", params).then(function(success){ 
            
             var validaPropReneg = success.dataset.PROPOSTA.VALIDA;    
             
             return validaPropReneg;
             
         }.bind(this),
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
        });
    }
    
    
    /**
     * Validação que obriga o usuário selecionar os elementos da composição que são obrigatórios.
     * 
     * @param {object[]} rows
     * 
     * @return {string}
     */
    this.validarObrigatoriedadeItensComposicao = function(rows) {
        var promessa = ZHPromise.defer();
        var msg      = null;
        
        var existeLinhaEscolhida = rows.some(function(row) {
            return row.__isSelected == true;
        });
        
        if (existeLinhaEscolhida) {
            var composicoesObrigatoriasNaoEscolhidas = rows.filter(function(row) {
                if (row.TIPO == "C" && row.IDSERVOBRIGCOMP == "S" && row.__isSelected == false) {
                    if (row._parent) {
                        var composicao = this.getComposicaoDoElemento(row);
                        return composicao.__isSelected;
                    }
                    else {
                        return true;
                    }
                }
            }.bind(this));
            
            if (composicoesObrigatoriasNaoEscolhidas.length == 0) {
                
                var elementosObrigatoriosNaoEscolhidos = rows.filter(function(row) {
                    if (row.TIPO == "E" && row._childCount == 0 && row.IDSERVOBRIGCOMP == "S" && row.__isSelected == false) {
                        if (row._parent) {
                            var composicao = this.getComposicaoDoElemento(row);
                            return composicao.__isSelected;
                        }
                        else {
                            return true;
                        }
                    }
                    else {
                        return false;
                    }
                }.bind(this));
                
                if (elementosObrigatoriosNaoEscolhidos.length == 0) {
                    promessa.resolve(null);
                }
                else {
                    msg = "Os seguintes elementos são obrigatórios:<br/><br/>";
                    
                    elementosObrigatoriosNaoEscolhidos.forEach(function(campo) {
                        var itensPai = Utilities.getItensPai(campo);
                        
                        if (itensPai.length > 0) {
                            itensPai.reverse();
                            itensPai.forEach(function(pai) {
                                msg += pai.NMELEMENTO + " > ";
                            });
                        }
                        msg += "<u>" + campo.NMELEMENTO + "</u><br/><br/>";
                    });
                    
                    promessa.resolve(msg.substr(0, msg.length - 10));
                }
                
            }
            else {
                msg = "As seguintes composições são obrigatórias:<br/><br/>";
                composicoesObrigatoriasNaoEscolhidas.forEach(function(item) {
                    var itensPai = Utilities.getItensPai(item);
                        
                    if (itensPai.length > 0) {
                        itensPai.reverse();
                        itensPai.forEach(function(pai) {
                            msg += pai.NMELEMENTO + " > ";
                        });
                    }
                    
                    msg += "<u>" + item.NMELEMENTO + "</u><br/><br/>";
                });
                
                promessa.resolve(msg.substr(0, msg.length - 10));
            }
        
        } else {
            promessa.resolve("Operação bloqueada.<br/><br/>Nenhum elemento foi selecionado.");
        }
        
        return promessa.promise;
    };
    
    this.validarVendedor = function(nrseqorcamento, nrvendedororca, nrparcvendedor) {
        if (!nrvendedororca)
            nrvendedororca = -1;
        
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento,
            "NRVENDEDORORCA": nrvendedororca,
            "NRPARCVENDEDOR": nrparcvendedor
        };
        
        return General.callRestEngine("product#/fct_vendedororca_widget/validarVendedor", "Row", params);
    };
    
    this.getPorcentagemVendedores = function(nrseqorcamento, nrorg) {
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento,
            "NRORG": nrorg
        };
        
        return General.callRestEngine("product#/fct_vendedororca_widget/getPorcentagemVendedores", "Row", params);
    };
    
    this.validarTipoEndereco = function(nrseqorcamento, nrenderecoorca, cdtipoendereco) {
        if (!nrenderecoorca)
            nrenderecoorca = -1;
        
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento,
            "NRENDERECOORCA": nrenderecoorca,
            "CDTIPOENDERECO": cdtipoendereco
        };
        
        return General.callRestEngine("product#/fct_enderecoorca_widget/validarTipoEndereco", "Row", params);
    };

    this.validarNegociacaoValores = function(nrseqorcamento) {
        var promise = ZHPromise.defer();
        var params = {
            "NRSEQORCAMENTO": nrseqorcamento
        };

        General.callRestEngine("product#/fct_orcamento_widget/validarNegociacaoValores", "Row", params).then(function(response) {
            promise.resolve(response.dataset.validacao.resultado);
        })
        .catch(function(error) {
            console.error(error);
            promise.reject(error.data.error);
        });

        return promise.promise;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001000_LancamentoProposta_Service", Sm00001000_LancamentoProposta_Service);
});

// FILE: ./assets/js/services/Sm00002000_Modelo_Proposta_Service.js
function Sm00002000_Modelo_Proposta_Service(Sm00002000_Modelo_Proposta_Repository){
    
    /**
     * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo.
     */
    
    this.validaEscopoProposta = function(tipoOrcamento, orcaModelo){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRTIPOORCAMENTO': tipoOrcamento,
                    'NRORCAMENTOMODELO': orcaModelo
            }
        };
        
        var requestPromisse = Sm00002000_Modelo_Proposta_Repository.validaEscopoProposta(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
    /**
     * A função "validaCompProposta" verifica se a composição 
     * que está sendo relacionada já está relacionada ao modelo.
     */
    
    this.validaCompProposta = function(compServico, orcaModelo){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRCOMPSERVICO': compServico,
                    'NRORCAMENTOMODELO': orcaModelo
            }
        };
        
        var requestPromisse = Sm00002000_Modelo_Proposta_Repository.validaCompProposta(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00002000_Modelo_Proposta_Service", Sm00002000_Modelo_Proposta_Service);
});


// FILE: ./assets/js/services/Sm00003000_Escopo_Proposta_Service.js
function Sm00003000_Escopo_Proposta_Service(Sm00003000_Escopo_Proposta_Repository){
    
    /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
    
    this.validaServicoEscopo = function(servOrca, escopoOrca){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {
                'CDSERVORCA': servOrca,
                'NRTIPOORCAMENTO': escopoOrca
            }
        };
        
        var requestPromisse = Sm00003000_Escopo_Proposta_Repository.validaServicoEscopo(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00003000_Escopo_Proposta_Service", Sm00003000_Escopo_Proposta_Service);
});


// FILE: ./assets/js/services/Sm00004000_Contrato_Service.js
function Sm00004000_Contrato_Service(Sm00004000_Contrato_Repository) {

    /**
     * A função "copiaVersionaContrato" executa a procedure
     * "FCT_COPIA_VERSIONA_CONTRATO".
     */
    this.copiaVersionaContrato = function(nrSeqContrato, idCopiaVersiona, adicionaAgrupamento, idParcelaConsiderada, idReplicarNegociacao, dtRenegociaCont, dtFimDoNovoContrato) {
        
        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO'       : nrSeqContrato,
                'TIPO'                : idCopiaVersiona,
                'ADCAGRUPAMENTO'      : adicionaAgrupamento,
                'IDPARCELACONSIDERADA': idParcelaConsiderada,
                'IDREPLICARNEGOCIACAO': idReplicarNegociacao,
                'DTRENEGOCIACONT'     : dtRenegociaCont,
                "DTFIMCONTRATO"       : dtFimDoNovoContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.copiaVersionaContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * Executa a procedure "FCT_ENCERRA_CONTRATO".
     */
    this.encerraContrato = function(nrSeqContrato, idSituacao, dtrecisao, idtpexclusaoparcela, nrtiporescisao, idDocX, dtpedrescisaocontr, nrmotivorescisao, dsjustificaencerra, vrpendenciafin) {
        
        if(!dtrecisao)
            dtrecisao = moment().format("DD/MM/YYYY");
            
        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO'       : nrSeqContrato,
                "IDSITUACAOCONTRATO"  : idSituacao,
                "IDTPEXCLUSAOPARCELA" : idtpexclusaoparcela,
                "DTRECISAOCONTR"      : dtrecisao,
                "NRTIPORESCISAO"      : nrtiporescisao,
                "IDDOCX"              : idDocX,
                "DTPEDRESCISAOCONTR"  : dtpedrescisaocontr,
                "NRMOTIVORESCISAO"    : nrmotivorescisao,
                "DSJUSTIFICAENCERRA"  : dsjustificaencerra,
                "VRPENDENCIAFIN"      : vrpendenciafin
            }
        };
        
        var requestPromisse = Sm00004000_Contrato_Repository.encerraContrato(paramsRestEngine);
        return requestPromisse;
    };

    /**
     * Executa a procedure "FCT_VENCPREV_CONT".
     */
    this.geraParcelasContrato = function(nrSeqContrato) {

        var paramsRestEngine = {
            "requestType": "Row",
            "row": {
                "NRSEQCONTRATO": nrSeqContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.geraParcelasContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "cancelaParcelasContrato" executa a procedure
     * "FCT_CANC_VENCPREV_CONT".
     */
    this.cancelaParcelasContrato = function(nrSeqContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO': nrSeqContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.cancelaParcelasContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "alteraVigenciaContrato" altera os dados de vigência do contrato
     */
    this.alteraVigenciaContrato = function(dtInicVigContr, dtFinVigContr, seqContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'DTINIVIGENCONT': dtInicVigContr,
                'DTFINVIGENCONT': dtFinVigContr,
                'NRSEQCONTRATO': seqContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.alteraVigenciaContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "validaDadosProrrog" valida se ja existem alterações de vigência para o contrato
     */
    this.validaDadosProrrog = function(seqContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': { 'NRSEQCONTRATO': seqContrato }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.validaDadosProrrog(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "alteraDadosProrrog" inativa as alterações de vigência do contrato
     */

    this.alteraDadosProrrog = function(seqContrato, seqprorrog) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO': seqContrato,
                'NRSEQPRORRCONT': seqprorrog
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.alteraDadosProrrog(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "recebeParamFatModCont" recebe os dados de Faturamento do Modelo de Contrato
     */

    this.recebeParamFatModCont = function(seqModContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': { 'NRSEQMODCONTRA': seqModContrato }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.recebeParamFatModCont(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "recebeModReajustContr" recebe os dados de Reajuste do Modelo de Contrato
     */

    this.recebeModReajustContr = function(seqModContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': { 'NRSEQMODCONTRA': seqModContrato }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.recebeModReajustContr(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "recebeModIndContr" recebe os dados de Indice de Reajuste do Modelo de Contrato
     */

    this.recebeModIndContr = function(seqModContrato, seqModReajus) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQMODCONTRA': seqModContrato,
                'NRSEQMODREAJUS': seqModReajus
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.recebeModIndContr(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "recebeModeloDocContr" recebe os dados de Documento do Modelo de Contrato
     */

    this.recebeModeloDocContr = function(seqModContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': { 'NRSEQMODCONTRA': seqModContrato }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.recebeModeloDocContr(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "geraCodigoContrato" executa a procedure "CRIA_MNEMONICO".
     */

    this.geraCodigoContrato = function(seqContrato) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO': seqContrato
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.geraCodigoContrato(paramsRestEngine);

        return requestPromisse;
    };

    /**
     * A função "geraDocumentoContrato" executa a procedure "FCT_SUB_MNE_DOC".
     */

    this.geraDocumentoContrato = function(seqContrato, nrSeqDocCont) {

        var paramsRestEngine = {
            'requestType': 'Row',
            'row': {
                'NRSEQCONTRATO': seqContrato,
                'NRSEQCONTDOC': nrSeqDocCont
            }
        };

        var requestPromisse = Sm00004000_Contrato_Repository.geraDocumentoContrato(paramsRestEngine);

        return requestPromisse;
    };


}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00004000_Contrato_Service", Sm00004000_Contrato_Service);
});


// FILE: ./assets/js/services/Sm00004005_Parametrizacao_Contrato_Service.js
function Sm00004005_Parametrizacao_Contrato_Service(Sm00004005_Parametrizacao_Contrato_Repository){
    
  /**
     * A função "insereDadosContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Contrato (FCT_CONTRATO).
     */
    
    this.insereDadosContrato = function(IdFaturaContr, CdModPgto, QtParcelasFatu, NrDiaVencParcelasFatu, dtInicFatContr, idQtDiasCompCon, nrQtDiasCompCon, idCompetCont, nrDiaCompetCont, idTpDtEmissNf, nrDiaDtEmissNf, idTpDtVenc, nrDiaVenParcFat, idTpFatuIncomp, seqContrato){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': { 
                     'IDFATURACONTR' : IdFaturaContr, 
                     'CDMODPGTO': CdModPgto,
                     'QTPARCELASFATU': QtParcelasFatu,
                     'NRDIAVENCPARCELASFATU': NrDiaVencParcelasFatu,
                     'DTINICFATCONTR': dtInicFatContr,
                     'IDQTDIASCOMPCON': idQtDiasCompCon,
                     'NRQTDIASCOMPCON': nrQtDiasCompCon,
                     'IDTPCOMPETCONT': idCompetCont,
                     'NRDIACOMPETCONT': nrDiaCompetCont,
                     'IDTPDTEMISSNF': idTpDtEmissNf,
                     'NRDIADTEMISSNF': nrDiaDtEmissNf,
                     'IDTPDTVENC': idTpDtVenc,
                     'NRDIAVENPARCFAT': nrDiaVenParcFat,
                     'IDTPFATUINCOMP':idTpFatuIncomp,
                     'NRSEQCONTRATO': seqContrato
            }
        };
        
        var requestPromisse = Sm00004005_Parametrizacao_Contrato_Repository.insereDadosContrato(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
    this.insereDadosItContrato = function(IdFaturaContr, CdModPgto, QtParcelasFatu, NrDiaVencParcelasFatu, seqContrato, seqitContrato){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': { 
                     'IDFATURACONTR' : IdFaturaContr, 
                     'CDMODPGTO': CdModPgto,
                     'QTPARCELASFATU': QtParcelasFatu,
                     'NRDIAVENCPARCELASFATU': NrDiaVencParcelasFatu,
                     'NRSEQCONTRATO': seqContrato,
                     'NRITEMOCONTRATO': seqitContrato
            }
        };
        
        var requestPromisse = Sm00004005_Parametrizacao_Contrato_Repository.insereDadosItContrato(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00004005_Parametrizacao_Contrato_Service", Sm00004005_Parametrizacao_Contrato_Service);
});


// FILE: ./assets/js/services/Sm00006000_Elemento_Service.js
function Sm00006000_Elemento_Service(Sm00006000_Elemento_Repository){
    
    /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
    
    this.validaServicoRelacionado = function(servOrcaRelac, servOrca, tipoMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDSERVORCARELAC': servOrcaRelac,
                    'CDSERVORCA': servOrca,
                    'CDTPMEDIDASERV': tipoMedida
            }
        };
        
        var requestPromisse = Sm00006000_Elemento_Repository.validaServicoRelacionado(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
     /**
     * A função "validaServicoRelacionadoUnit" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço para inclusão 
     * de um só item.
     */
    
    this.validaServicoRelacionadoUnit = function(servOrcaRelac, servOrca, tipoMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDSERVORCARELAC': servOrcaRelac,
                    'CDSERVORCA': servOrca,
                    'CDTPMEDIDASERV': tipoMedida
            }
        };
        
        var requestPromisse = Sm00006000_Elemento_Repository.validaServicoRelacionadoUnit(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006000_Elemento_Service", Sm00006000_Elemento_Service);
});


// FILE: ./assets/js/services/Sm00006000_Servicos_Service.js
function Sm00006000_Servicos_Service(Sm00006000_Servicos_Repository){
    
    /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
     
    this.validaServicoRelacionado = function(servOrcaRelac, servOrca, tipoMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDSERVORCARELAC': servOrcaRelac,
                    'CDSERVORCA': servOrca,
                    'CDTPMEDIDASERV': tipoMedida
            }
        };
        
        var requestPromisse = Sm00006000_Servicos_Repository.validaServicoRelacionado(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006000_Servicos_Service", Sm00006000_Servicos_Service);
});


// FILE: ./assets/js/services/Sm00006001_Tabela_Preco_Servico_Service.js
function Sm00006001_Tabela_Preco_Servico_Service(Sm00006001_Tabela_Preco_Servico_Repository){
    
    /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
    
    this.validaServicoTabela = function(servOrca, precBaseServ, tipoMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDSERVORCA': servOrca,
                    'CDPRECBASESERV': precBaseServ,
                    'CDTPMEDIDASERV': tipoMedida
            }
        };
        
        var requestPromisse = Sm00006001_Tabela_Preco_Servico_Repository.validaServicoTabela(paramsRestEngine);
        
        return requestPromisse;
    };
    
     /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
    
    this.validaTipoMedidaVariavel = function(nrSeqItPrecBServ, cdTpMedidaServ, CdtpMedidaVariavel, nrVaricacaoPrecBaseServ){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRSEQITPRECBSERV' : nrSeqItPrecBServ,
                    'CDTPMEDIDASERV' : cdTpMedidaServ,
                    'CDTPMEDIDAVARIAVEL' : CdtpMedidaVariavel,
                    'NRVARIACAOPRECBASESERV': nrVaricacaoPrecBaseServ,
            }
        };
        
        var requestPromisse = Sm00006001_Tabela_Preco_Servico_Repository.validaTipoMedidaVariavel(paramsRestEngine);
        
        return requestPromisse;
    };
    
     /**
     * A função "sincronizarVariacaoPreco" executa a procedure "FCT_SINCRONIZAR_VARIACAO_PRECO".
     */
    
    this.sincronizarVariacaoPreco = function(nrSeqItPrecBServ, tipo){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRSEQITPRECBSERV': nrSeqItPrecBServ,
                    'TIPO': tipo
            }
        };
        
        var requestPromisse = Sm00006001_Tabela_Preco_Servico_Repository.sincronizarVariacaoPreco(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006001_Tabela_Preco_Servico_Service", Sm00006001_Tabela_Preco_Servico_Service);
});


// FILE: ./assets/js/services/Sm00006002_Tipo_de_Medida_Service.js
function Sm00006002_Tipo_de_Medida_Service(Sm00006002_Tipo_de_Medida_Repository){
    
    /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
    
    this.validaTipoMedidaVariavel = function(cdTpMedidaServ, CdtpMedidaVariavel, nrVaricacaoTpMedida){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDTPMEDIDASERV': cdTpMedidaServ,
                    'CDTPMEDIDAVARIAVEL': CdtpMedidaVariavel,
                    'NRVARIACAOTPMEDIDA': nrVaricacaoTpMedida
            }
        };
        
        var requestPromisse = Sm00006002_Tipo_de_Medida_Repository.validaTipoMedidaVariavel(paramsRestEngine);
        
        return requestPromisse;
    };
    
    /**
     * A função "validaExsiteMedidaVariavel" verifica se existem medidas variaveis 
     * para o tipo de medida atual.
     */
     
    this.validaExsiteMedidaVariavel = function(cdTpMedidaServ){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDTPMEDIDASERV': cdTpMedidaServ
            }
        };
        
        var requestPromisse = Sm00006002_Tipo_de_Medida_Repository.validaExsiteMedidaVariavel(paramsRestEngine);
        
        return requestPromisse;
    };
    
    
     /**
     * A função "validaExsiteVariacao" verifica se o tipo de medida variavel esá sendo 
     * utilizado.
     */
     
    this.validaExsiteVariacao = function(CdtpMedidaVariavel){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'CDTPMEDIDAVARIAVEL': CdtpMedidaVariavel
            }
        };
        
        var requestPromisse = Sm00006002_Tipo_de_Medida_Repository.validaExsiteVariacao(paramsRestEngine);
        
        return requestPromisse;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00006002_Tipo_de_Medida_Service", Sm00006002_Tipo_de_Medida_Service);
});


// FILE: ./assets/js/services/Sm00009000_Contrato_Modelo_Service.js
function Sm00009000_Contrato_Modelo_Service(Sm00009000_Contrato_Modelo_Repository){
    
  /**
     * A função "insereDadosModContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Modelo de Contrato (FCT_MODELOCONTRATO).
     */
    
    this.insereDadosModContrato = function(dtInicFatContr, idQtDiasCompCon, nrQtDiasCompCon, idCompetCont, nrDiaCompetCont, idTpDtEmissNf, nrDiaDtEmissNf, idTpDtVenc, nrDiaVenParcFat, idTpFatuIncomp, seqModContrato){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': { 'DTINICFATCONTR': dtInicFatContr,
                     'IDQTDIASCOMPCON': idQtDiasCompCon,
                     'NRQTDIASCOMPCON': nrQtDiasCompCon,
                     'IDTPCOMPETCONT': idCompetCont,
                     'NRDIACOMPETCONT': nrDiaCompetCont,
                     'IDTPDTEMISSNF': idTpDtEmissNf,
                     'NRDIADTEMISSNF': nrDiaDtEmissNf,
                     'IDTPDTVENC': idTpDtVenc,
                     'NRDIAVENPARCFAT': nrDiaVenParcFat,
                     'IDTPFATUINCOMP':idTpFatuIncomp,
                     'NRSEQMODCONTRA': seqModContrato
            }
        };
        
        var requestPromisse = Sm00009000_Contrato_Modelo_Repository.insereDadosModContrato(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00009000_Contrato_Modelo_Service", Sm00009000_Contrato_Modelo_Service);
});


// FILE: ./assets/js/services/Sm00034000_Maximo_Carencia_Proposta_Service.js
function Sm00034000_Maximo_Carencia_Proposta_Service(Sm00034000_Maximo_Carencia_Proposta_Repository){
    
    /**
     * A função "validaCarenciaOperador" verifica se já existe 
     * uma carencia cadastrada para o operador indicado.
     */
    
    this.validaCarenciaOperador = function(parcOperador){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRPARCNEGOCIO': parcOperador}
        };
        
        var requestPromisse = Sm00034000_Maximo_Carencia_Proposta_Repository.validaCarenciaOperador(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm00034000_Maximo_Carencia_Proposta_Service", Sm00034000_Maximo_Carencia_Proposta_Service);
});


// FILE: ./assets/js/services/Sm_Percentual_Desconto_Operador_Service.js
function Sm_Percentual_Desconto_Operador_Service(Sm_Percentual_Desconto_Operador_Repository){
    
    
    this.validaPercentDescontoOper = function(parcOperador){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRPARCNEGOCIO': parcOperador}
        };
        
        var requestPromisse = Sm_Percentual_Desconto_Operador_Repository.validaPercentDescontoOper(paramsRestEngine);
        
        return requestPromisse;
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("Sm_Percentual_Desconto_Operador_Service", Sm_Percentual_Desconto_Operador_Service);
});


// FILE: ./assets/js/services/ger00004500_Propspect_Service.js
function ger00004500_Prospect_Service(ger00004500_Prospect_Repository, CRM_Utilities){
    
    this.callForInsertProspect = function(currentRow){
        
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        
        return ger00004500_Prospect_Repository.callInsertTaskProspect(paramsRestEngine, currentRow);
        
    };
    
    this.callForSelectProspect = function(currentRow){
        
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        
        return ger00004500_Prospect_Repository.selectContactProspect(paramsRestEngine, currentRow);
        
    };
    
    this.callForInsertNrseqcontatoForNewOpportunity = function(currentRow){
        
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        
        ger00004500_Prospect_Repository.requestForForInsertNrseqcontatoForNewOpportunity(paramsRestEngine);
        
    };
    
    this.callForUpdatePartnerType = function(currentRow){
    
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForUpdatePartnerType(paramsRestEngine);
        
    };
    
    this.callForInsertPartnershipOfTypeClient = function(currentRow){
    
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForInsertPartnershipOfTypeClient(paramsRestEngine);
        
    };
    
    this.callForInsertClient = function(currentRow){
        
        currentRow.NRINSJURPROS = CRM_Utilities.removeValueMask(currentRow.NRINSJURPROS);
        currentRow.CDCLIENTE = this.concatInscriptionTypeWithInscription(currentRow.IDTPIJURPROS, currentRow.NRINSJURPROS);
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForInsertClient(paramsRestEngine);
        
    };
    
    this.concatInscriptionTypeWithInscription = function(inscriptionType, inscription){
        
        return inscriptionType + inscription;
            
    };
    
    this.callForDeactivateProspect = function(currentRow){
    
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForDeactivateProspect(paramsRestEngine);
    };
    
    this.callForInsertClientAddress = function(currentRow){
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForInsertClientAddress(paramsRestEngine);
        
    };
    
    this.callForSelectProspectAddress = function(currentRow){
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForSelectProspectAddress(paramsRestEngine);
    };
    
    this.callForSetTaskEndDate = function(widget){
        
        var currentRow = widget.currentRow;
        
        currentRow.NRSEQFILACONTA = widget.NRSEQFILACONTA;
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForSetTaskEndDate(paramsRestEngine);
          
    };
    
    this.callForDeleteEvolutionFilesByOriginNumber = function(widget){
        
        var currentRow = widget.currentRow;
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForDeleteEvolutionFilesByOriginNumber(paramsRestEngine);
        
    };
    
    this.callForDeactivateTask = function(widget){
        
        var currentRow = widget.currentRow;
        
        var paramsRestEngine = {
            
            "requestType": "Row",
            "row": currentRow
            
        };
        
        return ger00004500_Prospect_Repository.requestForDeactivateTask(paramsRestEngine);
          
    };
    
}
Configuration(function (ContextRegister) {
    ContextRegister.register("ger00004500_Prospect_Service", ger00004500_Prospect_Service);
});


// FILE: ./assets/js/controllers/ABEvolucaoController.js
function ABEvolucaoController(CRM_Utilities, templateManager, restEngine, ScreenService, ZHPromise) {
    
    /**
     * Procedimento para definir a permissão para editar
     * o campo 'Data da Evolução' no ambiente de criação
     * de uma evolução.
     * 
     * @para {widget} widget widget referente a tela de
     * evolução.
    */
    
    this.setEvolDateChangePermission = function(widget){
        var currentRow = widget.currentRow;
        
        this.getNrDiasLanceEvol(widget).then(function(response){
            var nrDiasLanceEvol = templateManager.NRDIASLANCEVOL;
            
            if(nrDiasLanceEvol > 0 && currentRow.__is_new){
                widget.getField('DTHREVOLCONT').readOnly = false;
                this.configureEvolDateChangePermission(widget, nrDiasLanceEvol);
            }else{
                widget.getField('DTHREVOLCONT').readOnly = true;
            }
        }.bind(this));
    };
    
    /**
     * Procedimento para buscar, e setar no templateManager
     * (se não já existir no templateManager),
     * o número de dias permitido para uma 
     * evolução ser lançada posteriomento ao dia real.
     * 
     * @param {object} widget widget referente a
     * tela de evoluções.
     * 
     * @return {number} retorna o número de dias pemitido
     * para lançar uma evolução posterior a data real.
    */
    this.getNrDiasLanceEvol = function(widget){
        var nrDiasLanceEvol = templateManager.NRDIASLANCEVOL;
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        var deferred = ZHPromise.defer();
        
        if(nrDiasLanceEvol){
            deferred.resolve();
            
            return deferred;
        }else{
            
            var paramsRestEngine = {
            "requestType": "Row",
            "row": {NRORG: loggedOperatorInfos.NRORG}
            };
        
            return restEngine.post('/getNrDiasLanceEvol', paramsRestEngine,
            function(success){
                templateManager.NRDIASLANCEVOL = success.dataset.DADOS.NRDIASLANCEVOL[0].NRDIASLANCEVOL;
            },undefined,
            function(error){
                CRM_Utilities.CATCH_EXCEPTION_ERROR(error.data.error);
            },undefined,undefined
        );
            
        }
    };
    
    /**
     * Procedimento para configurar as permissões
     * de data máxima e data mínima para lançar
     * uma evolução.
     * 
     * @param {object} widget widget referente a
     * tela de evoluções.
     * 
     * @param {number} numberOfDays número máximo de dias
     * para se lançar uma evolução posteriormente.
    */
    this.configureEvolDateChangePermission = function(widget, numberOfDays){
        var evolDateField = widget.getField('DTHREVOLCONT');
        
        evolDateField.minDate = CRM_Utilities.subtractDateByDays(CRM_Utilities.getCurrentDateWithMinTime(),numberOfDays);
        evolDateField.maxDate = CRM_Utilities.getCurrentDateWithMaxTime();
    };
    
    /**
     * //TODO
    */
    this.configureEvolTimeShowPermission = function(widget){
        var currentRow = widget.currentRow;
        var selectedDateWithoutTime = moment(currentRow.DTHREVOLCONT, 'DD/MM/YYYY HH:mm').format('DD/MM/YYYY');    
        
        if(selectedDateWithoutTime === CRM_Utilities.getCurrentDate()){
            currentRow.DTHREVOLCONT = selectedDateWithoutTime + ' ' + CRM_Utilities.getCurrentTime();
        }else{
            currentRow.DTHREVOLCONT = selectedDateWithoutTime + ' 00:00';
        }
    };
    
    /**
     * Define o filtro da widget de evolução
     * pelo número do parceiro de negócio selecionado
     * anteriormente.
     * 
     * @param {object} widgetName nome da widget de evolução
     * em caso.
    */
    this.setEvolutionWidgetFilterByNrParcNegocio = function(widgetName){
        
        var evolutionWidget = templateManager.container.getWidget(widgetName);
        
        var widgetFilter = evolutionWidget.dataSourceFilter;
        var parentWidgetCurrentRow = evolutionWidget.parent.currentRow;
        var nrParcNegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(widgetFilter, "P_NRPARCNEGOCIO");
        
        widgetFilter[nrParcNegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;
        
        evolutionWidget.reload();
        
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ABEvolucaoController', ABEvolucaoController);
});

// FILE: ./assets/js/controllers/Sm00001000_LancamentoPropostaAlimenticia_Controller.js
/**
 * Controller JS para a tela de lançamento de propostas alimentícias
**/
function Sm00001000_LancamentoPropostaAlimenticia_Controller(Operador_Repository, ZHPromise, General, Utilities, templateManager,ScreenService, Sm00001000_LancamentoProposta_Service, UtilitiesPropostaContrato, Sm00001000_PropostaAlimenticia_Repository){
    
    var opLogado                = null;
    var nomeOperador            = null;
    var idUtilizaHierOper       = null;
    var idOperMaster            = null;
    var idAlteraPropConc        = null;
    var onClickMobra            = null;
    var onClickMobraItens       = null;
    var nrparcnegocioOperLogado = null;
    var tpOperador              = null;
    var nrOrgOper               = null;
    var codeRelacFormacao       = null;
    var countMObra              = -1;
    var idItensMObraAlterado    = false;
    /**
     * Coloca na currentRow os parametrôs de novo código
     * para a inserção de novos registros
     * @params{Object} widget - widget que irá inserir um novo registro
    **/
    this.criarParametrosNewCode = function(widget){
        if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
            widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
            if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
        }
    };
    /**
     * Busca as informações do operador logado e 
     * verifica se o sistema utiliza hierarquia de operadores 
    **/
    this.buscaOperadorLogado = function(){
        var deferred = ZHPromise.defer();
        Operador_Repository.getOperadorLogado().then(function(success) {
            opLogado                = success[0].CDOPERADOR;
            nomeOperador            = success[0].NMOPERADOR;
            nrparcnegocioOperLogado = success[0].NRPARCNEGOCIO;
            tpOperador              = success[0].IDTPOPERADOR;
            nrOrgOper               = success[0].NRORG;
            this.buscaInformacaoHierarquia();
            deferred.resolve(success);
        }.bind(this));
        return deferred.promise;
    };
    /**
     * Verifica se o sistema utiliza hierarquia de operadores e 
     * caso use busca sua posição
     * @params{Object} widgetPropostaGrid - widget grid de propostas 
     * @params{Object} widget - dados da proposta
     **/
    this.verificaHierarquia = function(widgetPropostaGrid, widget) {
        if(idOperMaster === null && idUtilizaHierOper === null){
            this.buscaInformacaoHierarquia();
        }
        widget.currentRow.CDOPERRESPONSAVEL = opLogado;
        widget.currentRow.CDNOVOOPERRESP    = opLogado;
        widget.currentRow.NMOPERRESP        = nomeOperador;
    };
    /**
     * busca as informações da hierarquia de operadores 
     * ligada ao usuário logado.
    **/
    this.buscaInformacaoHierarquia = function() {
        General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", {}).then(function(success) {
            if (success.dataset.fct_parametrogeral && success.dataset.fct_parametrogeral.length > 0) {
                var paramsFilter = [
                    {
                        name: "CDOPERADOR",
                        operator: "=",
                        value: opLogado
                    }
                ];
                idUtilizaHierOper = success.dataset.fct_parametrogeral[0].IDUTILIZAHIEROPER;
                General.callRestEngine("product#/crud/crm_hierarquiaoper", "FilterData", paramsFilter).then(function(success) {
                    if (success.dataset.crm_hierarquiaoper && success.dataset.crm_hierarquiaoper.length > 0) 
                        idOperMaster = success.dataset.crm_hierarquiaoper[0].IDOPERMASTER;
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Abre a widget de Dados da Proposta (view/form)
     **/
    this.abrirSwipeProposta = function() {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            wdgTabbedPrincipal.openTab("wdgPropostaTabbed");
            wdgProposta.list();
            wdgProposta.reload().then(function() {
                wdgTabbedPrincipal.view();
                wdgProposta.view();
                wdgProposta.activate();
                this.controlarEscopoProposta(wdgProposta, 'S');
                //if(wdgProposta.getField("NMTIPOORCAMENTO").isVisible && wdgProposta.currentRow.NMTIPOORCAMENTO instanceof Array)
                //    wdgProposta.currentRow.NMTIPOORCAMENTO = wdgProposta.currentRow.NMTIPOORCAMENTO[0];
            }.bind(this));
        }.bind(this));
    };
    /**
     * verifica se o sistema permite a alteração de propostas já concluídas
     * @params{Object} widget - widget de proposta grid
    **/
    this.verificaParamAlteraPropConc = function(widget){
        var deferred = ZHPromise.defer();
        if(idAlteraPropConc){
            deferred.resolve();
        } else {
            var paramsGeral = [
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("PROPOSTA_NRORG")
                }
            ];
            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", paramsGeral).then(function(success) {
                if(success.dataset.fct_parametrogeral.length !== 0)
                    idAlteraPropConc = success.dataset.fct_parametrogeral[0].IDALTERAPROPCONC;
                deferred.resolve();
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
                deferred.reject();
            });
        }
        return deferred.promise;
    };
    /**
     * Faz o controle geral da tela de Proposta de acordo com o registro selecionado.
     * @param {object} widget - Widget de Proposta (grid)
     */
    this.validacoesGeraisProposta = function(wdgPropostaGrid) {
        var situacao    = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        this.controlarAcoesDeCrud(wdgProposta, situacao);
        this.controlarAcoesDeOperacao(wdgProposta, situacao);
    };
    /**
     * Verifica a situação da Proposta para permitir/bloquear sua edição e/ou exclusão.
     * @param {object} wdgProposta - Widget de Dados da Proposta 
     * @param {String} situacao - Situação da Proposta
     */
    this.controlarAcoesDeCrud = function(wdgProposta, situacao) {
        var actDelete = wdgProposta.getAction("delete");
        var actEdit = wdgProposta.getAction("edit");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        if(situacao == "CONCLUIDO"){
            wdgProposta.getAction("edit").isVisible = (idAlteraPropConc === "S") ? true : false;
            actDelete.isVisible = false;
            this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, false, situacao);
        } else {
            if (situacao == "ELABORACAO") {
                actDelete.isVisible = true;
                actEdit.isVisible   = true;
                this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, true, situacao);
            } else {
                actEdit.isVisible   = false;
                actDelete.isVisible = (situacao == "CANCELADO");
                this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, false, situacao);
            }
        }
    };
    /**
     * Muda a visibilidade das ações de crud das widgets filhas de proposta
     * @params {Object} widget - widget tabbed principal
     * @params {boolean} visivel - estado em que a visibilidade das actions devem ficar
     * @params {String} situacao - situação da proposta
     **/
    this.controlarAcoesDeCrudAux = function(widget, visivel, situacao) {
        var widgetsIgnoradas = ["wdgProposta", "wdgPropostaTabbed"];
        widget.widgets.filter(function(wid) {
                return widgetsIgnoradas.indexOf(wid.name) == -1;
            }).forEach(function(wid) {
                this.setVisibilidadeDasAcoesDeCrud(wid, visivel, situacao);
                if (wid.widgets.length > 0)
                    this.controlarAcoesDeCrudAux(wid, visivel, situacao);
            }.bind(this));
    };
    /**
     * Ativa/desativa todas as actions novo, editar e excluir de uma widget.
     * @param {object} widget - widget que terá as ações alteradas
     * @param {boolean} visivel - estado em que a visibilidade das actions devem ficar
     * @params {String} situacao - situação da proposta
     */
    this.setVisibilidadeDasAcoesDeCrud = function(widget, visivel, situacao) {
        var actions = ["add", "edit", "delete"];
        if(situacao === "CONCLUIDO"){
            actions.forEach(function(act){
                if(widget.getAction(act))
                    widget.getAction(act).isVisible = (idAlteraPropConc === "S") ? true : false;   
            });
            actions = [];
        }
        
        widget.actions.forEach(function(action) {
            if(actions.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
    };
    /**
     * Executa validações da proposta ao trocar de linha.
     * @param {object} wdgPropostaGrid - Widget de Proposta (grid)
     */
    this.wdgPropostaGridAfterMoveRow = function(wdgPropostaGrid) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbedPrincipal.descriptionOrder = wdgPropostaGrid.descriptionOrder;
        setLocalVar("PROPOSTA_NRSEQORCAMENTO", wdgPropostaGrid.currentRow.NRSEQORCAMENTO);
        setLocalVar("PROPOSTA_NRORG", wdgPropostaGrid.currentRow.NRORG);
        this.abrirSwipeProposta();
        this.verificaParamAlteraPropConc(wdgPropostaGrid).then(function(){
            this.validacoesGeraisProposta(wdgPropostaGrid);
        }.bind(this));
        this.desobrigaCamposEvolucaoEmail();
    };
    /**
     * Controla as ações do canto direito da tela de Proposta.
     * @param {object} widget - Widget de Proposta
     * @param string situacao - Situação da Proposta
     */
    this.controlarAcoesDeOperacao = function(widget, situacao) {
        var actConcluir       = widget.getAction("concluir_proposta");
        var actAceitar        = widget.getAction("aceitar_proposta");
        var actCopiar         = widget.getAction("copiar_proposta");
        var actVersionar      = widget.getAction("versionar_proposta");
        var actCancelar       = widget.getAction("cancelar_proposta");
        switch (situacao) {
            case "ELABORACAO": 
                actConcluir.isVisible       = true;
                actAceitar.isVisible        = false;
                actCopiar.isVisible         = true;
                actVersionar.isVisible      = true;
                actCancelar.isVisible       = true;
            break;
            case "CANCELADO":
               actConcluir.isVisible       = false;
               actAceitar.isVisible        = false;
               actCopiar.isVisible         = true;
               actVersionar.isVisible      = true;
               actCancelar.isVisible       = false;
            break;
            case "ACEITO":
                actConcluir.isVisible       = false;
                actAceitar.isVisible        = false;
                actCopiar.isVisible         = true;
                actVersionar.isVisible      = true;
                actCancelar.isVisible       = true;
            break;
            default: //CONCLUIDO ou ENVIADO
                actConcluir.isVisible       = false;
                actAceitar.isVisible        = true;
                actCopiar.isVisible         = true;
                actVersionar.isVisible      = true;
                actCancelar.isVisible       = true;
            break;
        }
    };
    this.desobrigaCamposEvolucaoEmail = function() {
        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        var is_required    = this.alternaCampo(wdgEnviarEmail, false);
        wdgEnviarEmail.reload();
    };
    /**
     * Alterna o requerimento dos campos de evolução da tela de enviar e-mail
     * de acordo com o preenchimento
     * @params{Object} widget  - widget Enviar E-mail
     * @params{boolean} setObg - status da obrigatoriedade dos campos
    **/
    this.alternaCampo = function(widget, setObrg){
         var campoPreenchido = false;
         if(widget.getField("NMTIPOCONT").value() || widget.getField("NMMOTIEVOCON").value() || widget.getField("NMSEQRESULEVOL").value())
            campoPreenchido = true;
         if(!setObrg){
            widget.getField("NMTIPOCONT").validations = {};
            widget.getField("NMMOTIEVOCON").validations = {};
            widget.getField("NMSEQRESULEVOL").validations = {};
         } else {
            if(campoPreenchido){
                widget.getField("NMTIPOCONT").validations.required     = setObrg;
                widget.getField("NMMOTIEVOCON").validations.required   = setObrg;
                widget.getField("NMSEQRESULEVOL").validations.required = setObrg;
            }
        }
        return setObrg;
    };
    /**
     * Habilita a tela para a inserção de uma nova proposta
     **/
    this.novaProposta = function() {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        setLocalVar("PROPOSTA_NRSEQORCAMENTO", "");
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            this.alterarAcessibilidadeAbasProposta(false);
            
            wdgTabbedPrincipal.edit();
            wdgProposta.getAction("avancar").isVisible = true;
            wdgProposta.getAction("save").isVisible    = false;
            wdgProposta.newRow();
            wdgProposta.template = "widget/form.html";
            
            if(!wdgProposta.currentRow.IDPROPALIMENTICIA)
                wdgProposta.currentRow.IDPROPALIMENTICIA = "S";
                
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            this.verificaHierarquia(wdgPropostaGrid, wdgProposta);
            wdgPropostaGrid.currentRow = {};
            
            var wdgItens = templateManager.container.getWidget("wdgTabbedPrincipalItens");
            wdgItens.isVisible = false;
            
            var wdgEscopo = templateManager.container.getWidget("wdgEscoposProposta");
            wdgEscopo.isVisible = false;
            
            this.validacoesGeraisProposta(wdgPropostaGrid);
            this.controlaCodProposta(wdgProposta);
            this.removerDataLocalStorageLancamento();
        }.bind(this));
    };
    
    /**
     * Controla a acessibilidade das abas filhas de Proposta, desabilitando-as em 
     * modo de inserção e habilitando-as em modo de edição.
     * @param {boolean} enabled - status da acessibilidade das abas
     */
    this.alterarAcessibilidadeAbasProposta = function(enabled) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbedPrincipal.widgets.filter(function(wid) {
            return wid.name != "wdgPropostaTabbed";
        }).forEach(function(widget) {
            widget.enabled = enabled;
        });
        
        templateManager.updateTemplate();
    };
    /**
     * Altera a visibilidade das widgets de inserção
    **/
    this.alterarVisibilidadesWidgets = function(){
        var wdgCusto       = templateManager.container.getWidget("wdgCustoProposta");
        var wdgMObra       = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgDespesa     = templateManager.container.getWidget("wdgDespesas");
        var wdgInvest      = templateManager.container.getWidget("wdgInvestimentosInserindo");
        var wdgTaxas       = templateManager.container.getWidget("wdgTaxasInserindo");
        var wdgServico     = templateManager.container.getWidget("wdgTabbedServico");
        var wdgItens       = templateManager.container.getWidget("wdgTabbedPrincipalItens");
        var wdgMObraTabbed = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        var wdgCustoFinal  = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        var wdgEditarServ  = templateManager.container.getWidget("wdgEditarServico");
        var tabbedPrinc    = templateManager.container.getWidget("wdgTabbedPrincipal");
        var Propostatabbed = templateManager.container.getWidget("wdgPropostaTabbed");
        var wdgImposto     = templateManager.container.getWidget("wdgImpostoProposta");
        var wdgEscopo      = templateManager.container.getWidget("wdgEscoposProposta");

        wdgServico.isVisible          = false;
        wdgEditarServ.label           = "";
        wdgEditarServ.currentRow      = {};
        wdgEditarServ.dataSource.data = [];
        wdgEditarServ.fields.forEach(function(field){
            field.readOnly = true;
        });

        wdgCusto.isVisible = false;
        
        wdgMObraTabbed.isVisible  = false;
        wdgMObra.dataSource.data  = [];
        wdgMObraTabbed.currentRow = {};

        wdgDespesa.isVisible = false;
        wdgDespesa.dataSource.clearCheckedRows();

        wdgInvest.isVisible = false;
        wdgInvest.dataSource.clearCheckedRows();

        wdgTaxas.isVisible = false;
        wdgTaxas.dataSource.clearCheckedRows();
        
        wdgImposto.isVisible = false;
        wdgImposto.dataSource.clearCheckedRows();

        wdgItens.isVisible     = true;
        wdgEscopo.isVisible    = false;
        tabbedPrinc.enabled    = true;
        Propostatabbed.enabled = true;
        tabbedPrinc.view();
        
        wdgCustoFinal.mode = "list";
    };
    /**
     * Controla o campo de Código na aba de dados da proposta
     * @params{Object} widget - widget dados da proposta
     **/
    this.controlaCodProposta = function(widget) {
        var fieldCodigo = widget.getField("NRORCAMENTO");
        if (widget.currentRow.NMORCAMENTOMODELO && widget.currentRow.__is_new) {
            fieldCodigo.readOnly = true;
            widget.currentRow.NRORCAMENTO = "";
            fieldCodigo.validations = {};
        } else {
            fieldCodigo.readOnly = false;
            fieldCodigo.validations = { "required": true } ;
        }
    };
    /**
     * Seta a currentRow com o registro retornado do banco de dados.
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.setDadosProposta = function(wdgProposta) {
        wdgProposta.currentRow = wdgProposta.dataSource.data[0];
    };
    /**
     * Volta para o grid da tela de proposta.
     **/
    this.voltarParaPropostaGrid = function() {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgItens    = templateManager.container.getWidget("wdgTabbedPrincipalItens");
        wdgProposta.dataSource.data = [];
        wdgProposta.currentRow      = {};
        
        this.removerDataLocalStorageLancamento();
        ScreenService.closeSwipe().then(function() {
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            wdgPropostaGrid.activate();
            wdgItens.enabled   = true;
            wdgItens.isVisible = true;
            wdgPropostaGrid.refreshWidget();
        });
    };
    /**
     * Coloca a tela de Proposta em modo de edição.
     * @param {object} wdgProposta - Widget de Proposta
     **/
    this.prepararEdicaoProposta = function(wdgProposta) {
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
        wdgTabbed.edit();
        
        wdgProposta.backupRow = Util.clone(wdgProposta.currentRow, ignoredKeys);
        wdgProposta.getAction("avancar").isVisible    = false;
        wdgProposta.getAction("save").isVisible       = true;
        wdgProposta.getField("VRPERCMARKUP").readOnly = true;
        wdgProposta.edit();
        wdgProposta.template = "widget/form.html";
        
        this.alterarAcessibilidadeAbasProposta(false);
        this.controlaCodProposta(wdgProposta);
        this.controlarEscopoProposta(wdgProposta, 'S');
    };
    /**
     * Exclui a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.excluirProposta = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function() {
            Utilities.validarCamposDuplicados(wdgProposta, null, ["NRSEQCONTATO", "NRORG"], "product#/evolcont_parcnegocio").then(function(validacaoCampo){
                if(!validacaoCampo){
                    ScreenService.confirmMessage("Não é possível excluir a proposta porque ela possui registros filhos. Deseja inativar a proposta?", "question", function() {
                        General.callRestEngine("product#/fct_orcamento_widget/inativaPropContato", "Row", wdgProposta.currentRow)
                        .then().catch(function(error){
                            console.error(error);
                            ScreenService.showMessage("Não foi possível tornar a proposta inativa.");
                        });
                    });
                } else {
                    wdgProposta.removeRow().then(function() {
                        this.voltarParaPropostaGrid();
                    }.bind(this),
                    function(error) {
                        if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                            ScreenService.confirmMessage("Não é possível excluir a proposta porque ela possui registros filhos. Deseja inativar a proposta?", "question", function() {
                                this.inativarProposta(wdgProposta);
                            }.bind(this), null);
                        } else {
                            Utilities.showErrorMessage(error);
                        }
                    }.bind(this));
                }
            }.bind(this));
        }.bind(this));
    };
    /**
     * Inativa proposta
    **/
    this.inativarProposta = function(wdgProposta) {
        wdgProposta.currentRow.changed = true;
        wdgProposta.currentRow.IDATIVO = "N";
        General.widgetSave(wdgProposta).then(function() {
            this.voltarParaPropostaGrid();
        }.bind(this),
        function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };
    /**
     * Seta o Modelo e os Escopos de Proposta para o filtro da Composição.
     * @param {object} widget - Widget de Serviço (inserindo)
     */
    this.setFiltroComposicao = function(widget) {
        var wdgProposta       = templateManager.container.getWidget("wdgProposta");
        var nmorcamentomodelo = widget.dataSourceFilter[0].value;
        var escopo            = widget.dataSourceFilter[1].value;
        var oldQuantidade     = getLocalVar("QTDIASVALIDADE_PROPOSTA");
        
        if(wdgProposta.currentRow.NRORCAMENTOMODELO !== nmorcamentomodelo || wdgProposta.currentRow.NRTIPOORCAMENTO !== escopo) {
            widget.dataSourceFilter[0].value = wdgProposta.currentRow.NRORCAMENTOMODELO;
            widget.dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
            widget.reload();
        } else {
            Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgServicosInserindo").then(function(response) {
                if(response && JSON.parse(response).length > 0) { 
                    var wdgServico = templateManager.container.getWidget("wdgServicosInserindo");
                    wdgServico.dataSource.data = JSON.parse(response);
                } else {
                    widget.dataSourceFilter[0].value = wdgProposta.currentRow.NRORCAMENTOMODELO;
                    widget.dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
                    widget.reload();
                }
            }.bind(this));
        } 
    };
    /**
     * Refaz o cálculo da quantidade de serviços por período quando 
     * a quantidade de dias da proposta é alterado
     * @params{Object} widget - widget de serviço
    **/
    /*this.atualizarQuantidadePeriodoServico = function(widget) {
        var rowsAlteradas = widget.dataSource.data.filter(function(servico) {
            return servico._childCount == 0 && servico.QTMENSAL > 0;
        });
        
        var wdgEdicaoServico = templateManager.container.getWidget("wdgEditarServico");
        rowsAlteradas.forEach(function(row) {
            wdgEdicaoServico.currentRow = row;
            this.salvarEdicaoServico(wdgEdicaoServico);
        }.bind(this));
    };*/
    /**
     * Avançar para a inserção dos serviços na proposta
     * @params{Object} widget - widget dados da proposta
    **/
    this.avancarServico = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgPropostaTabbed     = templateManager.container.getWidget("wdgPropostaTabbed");
            wdgPropostaTabbed.enabled = false;

            var wdgServico = templateManager.container.getWidget("wdgServicosInserindo");
            var wdgTabbed  = templateManager.container.getWidget("wdgTabbedServico");
            wdgTabbed.isVisible = true;
            wdgTabbed.enabled   = true;
            wdgTabbed.isVisible = true;
            wdgTabbed.mode      = "edit";
            wdgTabbed.activate().then(function() {
                this.setFiltroComposicao(wdgServico);
            }.bind(this));
        }
    };
    /** 
     * Valida o campo de código da proposta
     * @params{Object} widget - dados da proposta
    **/
    this.validaCodPropostas = function(widget, nameAction){
        if(!widget.currentRow.NRORCAMENTO){
            this.avancarServico(widget);
        } else {
            Utilities.validarCamposDuplicados(widget, ["NRSEQORCAMENTO"], ["NRORG", "NRORCAMENTO"], "product#/crud/fct_orcamento").then(function(response){
                if(response)
                    if(nameAction === "save")
                        this.validarEdicaoProposta(widget);
                    else 
                        this.avancarServico(widget);
                else 
                    ScreenService.showMessage("Operação bloqueada.<br/>Já existe uma proposta com o código informado.");
            }.bind(this));
        }
    };
    /**
     * Verifica se o percentual de desconto oferecido permitido 
     * ao operador logado
     * @params{Object} widget - widget dados da proposta
     **/
    this.validaCarenciaOperOuPerf = function(widget, nameAction) {
        if(widget.currentRow.NRDIASCARENCIA === null){
            this.validaCodPropostas(widget, nameAction);
        } else {
            var totalMaxCarencia = null;
            var tipoCarencia     = null;
            var contexto         = "";
            General.callRestEngine("product#/fct_carenciaoper_parcnegocio/validaCarenciaOperOuPerf", "Row", {}).then(function (success) {
                if (!success.dataset.VALIDAOPERPERF.VALIDA) {
                    this.validaCodPropostas(widget, nameAction);
                } else {
                    totalMaxCarencia = success.dataset.VALIDAOPERPERF.VALIDA.NRDIAMAXIMO;
                    tipoCarencia = success.dataset.VALIDAOPERPERF.VALIDA.TIPO;
                    if(totalMaxCarencia >= widget.currentRow.NRDIASCARENCIA){
                        this.validaCodPropostas(widget, nameAction);
                    } else {
                        if(tipoCarencia == 1)
                            contexto = "operador logado.";
                        if(tipoCarencia == 2)
                            contexto = "perfil de acesso deste operador logado.";
                        ScreenService.showMessage("Quantidade de dias de carência informado está acima do permitido para o "+ contexto +" (Carência máxima permitida: "+ totalMaxCarencia +" dias).");
                    }
                }
            }.bind(this));
        }
    };
    /**
     * Cancela o salvamento da Proposta.
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.cancelarSalvamentoProposta = function(wdgProposta) {
        var wdgTabbed = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");
        wdgPropostaTabbed.enabled = true;
        wdgTabbed.view();
        wdgProposta.currentRow.changed = false;
        this.alterarAcessibilidadeAbasProposta(true);
        this.alterarVisibilidadesWidgets();
        this.controlarEscopoProposta(wdgProposta, 'S');
        wdgProposta.template = "widget/form-view.html";
        wdgProposta.getAction("save").isVisible       = false;
        wdgProposta.getAction("avancar").isVisible    = true;
        wdgProposta.getField("VRPERCMARKUP").readOnly = false;
        
        if (wdgProposta.currentRow.__is_new) {
            if (wdgProposta.dataSource.data.indexOf(wdgProposta.currentRow) !== -1)
                wdgProposta.dataSource.data.pop();
                
            codeRelacFormacao = null;
            wdgProposta.list();
            this.voltarParaPropostaGrid();
        } else { 
            //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
            Object.keys(wdgProposta.backupRow).forEach(function(key) {
                wdgProposta.currentRow[key] = wdgProposta.backupRow[key];
            }); //copia dados coluna por coluna para não perder referência do objeto da currentRow
            wdgProposta.view();
        }
        
    };
    
    /**
     * Função disparada ao modificar o campo Modelo de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.modeloOnChange = function(widget) {
        this.verificarModeloProposta(widget);
        this.calcularDataValidadeProposta(widget);
        this.escoposOnChange(widget);
        this.controlaCodProposta(widget);
        this.controlaEscopoModelo(widget);
    };
    /**
     * Ao carregar um modelo, é verificado se seus dados não poderão ser alterados e se será
     * obrigatório informar uma oportunidade de negócio.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.verificarModeloProposta = function(widget) {
        var idbloqalteradados = widget.currentRow.IDBLOQALTERADADOS;
        var readOnly = (idbloqalteradados == "SIM") ? true : false;
        //this.manipularCamposModeloProposta(widget, readOnly);

        var utilizaComposicao = (widget.currentRow.IDUTILIZACOMP == "S");
        this.controlarEscopoProposta(widget, utilizaComposicao);
    };
    /**
     * Controla a field de escopo de acordo com o modelo de orcamento
     * escolhido. 
     * @param {Object} widget widget propostas
    **/
    this.controlaEscopoModelo = function(widget){
        var params = [
            {
                name     : "NRORCAMENTOMODELO",
                operator : "=",
                value    : widget.currentRow.NRORCAMENTOMODELO
            }
        ];
        
        General.callRestEngine("product#/crud/fct_relacescopomodelo_widget", "FilterData", params).then(function(response){
            if(response.dataset && response.dataset.fct_relacescopomodelo_widget){
                var fieldEscopo  = (widget.currentRow.IDUTILIZACOMP == "S") ? widget.getField("NRTIPOORCAMENTO"): widget.getField("NMTIPOORCAMENTO");
                var fieldIsArray = fieldEscopo.template === "field/select-multiple.html"; //(widget.currentRow[fieldEscopo.name] instanceof Array);
                if(response.dataset.fct_relacescopomodelo_widget.length == 1){
                    fieldEscopo.readOnly = true;
                    widget.currentRow.NRTIPOORCAMENTO = fieldIsArray ? [response.dataset.fct_relacescopomodelo_widget[0].NRTIPOORCAMENTO] : response.dataset.fct_relacescopomodelo_widget[0].NRTIPOORCAMENTO;
                    widget.currentRow.NMTIPOORCAMENTO = fieldIsArray ? [response.dataset.fct_relacescopomodelo_widget[0].NMTIPOORCAMENTO] : response.dataset.fct_relacescopomodelo_widget[0].NMTIPOORCAMENTO;
                } else {
                    fieldEscopo.readOnly = false;
                    widget.currentRow.NRTIPOORCAMENTO = fieldIsArray ? [] : "";
                    widget.currentRow.NMTIPOORCAMENTO = fieldIsArray ? [] : "";
                }
            }
        }.bind(this));
    };
    
     /**
     * Calcula a data de validade da proposta a partir dos dias.
     * @param {object} widget - Widget de Proposta
     */
    this.calcularDataValidadeProposta = function(widget) {
        var qtDias = widget.currentRow.QTDIASVALIDADE;
        if (qtDias != null && qtDias >= 0) {
            var dataAtual = moment();
            widget.currentRow.DTVALIDADE = dataAtual.add(qtDias, "day").format("DD/MM/YYYY");
        } else {
            widget.currentRow.DTVALIDADE = "";
            widget.currentRow.QTDIASVALIDADE = "";
        }
    };
    
    /**
     * Seta o campo IDUTILIZAVARPRECO de acordo com os escopos selecionados.
     * @param {object} widget - Widget de Proposta
     */
    this.escoposOnChange = function(widget) {
        var fieldEscopo = widget.getField("NRTIPOORCAMENTO");
        var nrtipoorcamento = fieldEscopo.value();
        var escopos = fieldEscopo.dataSource.data;

        var idutilizavarpreco = [];
        escopos.forEach(function(row) {
            if (nrtipoorcamento.indexOf(row.NRTIPOORCAMENTO) > -1) {
                idutilizavarpreco.push(row.IDUTILIZAVARPRECO);
            }
        });

        if (idutilizavarpreco.length == 0)
            idutilizavarpreco = "";

        widget.currentRow.IDUTILIZAVARPRECO = idutilizavarpreco;
    };
    
    /**
     * Controla o campo de Escopo de Proposta de acordo com o Modelo (se utiliza composição de serviços ou não).
     * 
     * @param {object} widget - Widget de Proposta
     * @param {boolean} utilizaComposicao
     */
    this.controlarEscopoProposta = function(widget, utilizaComposicao, limparCampos) {
        if (limparCampos == undefined) {
            limparCampos = true;
        }

        var fieldCodEscopo    = widget.getField("NRTIPOORCAMENTO");
        var fieldNomeEscopo   = widget.getField("NMTIPOORCAMENTO");
        var fieldUtilVariacao = widget.getField("IDUTILIZAVARPRECO");
        fieldNomeEscopo.readOnly   = false;
        fieldCodEscopo.readOnly    = false;
        if (utilizaComposicao) {
            this.filtroEscopo(widget);
            fieldNomeEscopo.isVisible  = false;
            fieldNomeEscopo.showOnForm = false;
            fieldNomeEscopo.template   = "field/text-edit.html";
            fieldCodEscopo.template    = "field/select-multiple.html";
            fieldCodEscopo.validations = {"required": true};
            fieldCodEscopo.isVisible   = true;
            fieldCodEscopo.showOnForm  = true;
            fieldCodEscopo.reload().then(function() {
                if (fieldCodEscopo.dataSource.data && fieldCodEscopo.dataSource.data.length == 1) {
                    fieldCodEscopo.value([fieldCodEscopo.dataSource.data[0].NRTIPOORCAMENTO]);
                    fieldNomeEscopo.value([fieldCodEscopo.dataSource.data[0].NMTIPOORCAMENTO]);
                    fieldUtilVariacao.value([fieldCodEscopo.dataSource.data[0].IDUTILIZAVARPRECO]);
                }
            });
        } else {
            fieldCodEscopo.isVisible   = false;
            fieldCodEscopo.showOnForm  = false;
            fieldCodEscopo.template    = "field/text-edit.html";
            fieldCodEscopo.validations = {};
            fieldNomeEscopo.template   = "zh-select-autocomplete#field/select-autocomplete.html";
            fieldNomeEscopo.isVisible  = true;
            fieldNomeEscopo.showOnForm = true;
            Utilities.iniciarAutocomplete(fieldNomeEscopo);
        }

        if (limparCampos) {
            fieldNomeEscopo.value("");
            fieldCodEscopo.value("");
        }
    };
    /**
     * filtra o campo Escopo de Proposta quando tem Modelo.
     * @param {object} widget - Widget de Proposta
     */
    this.filtroEscopo = function(widget) {
        var fieldEscopo       = widget.getField("NRTIPOORCAMENTO");
        var nrorcamentomodelo = widget.currentRow.NRORCAMENTOMODELO;
        fieldEscopo.dataSourceFilter[1].value = nrorcamentomodelo;
        fieldEscopo.reload();
    };
    /**
     * Recarrega o field caso o dataSource tenha sido alterado 
     * @params{Object} fieldNmOp - field Nome Operador
     **/
    this.recarregaDadosField = function(fieldNmOp) {
        var widgetAtiva = templateManager.container.getActiveWidget();
        var fieldParc  =  widgetAtiva.getField("NRPARCNEGOCIO");
        if (widgetAtiva.name === "filterPopup") {
            fieldParc.dataSourceFilter[1] = {
                name: "P_CDOPERADOR",
                operator: "=",
                value: opLogado
            };
        }
        fieldNmOp.reload();
    };
    /**
     * Manda recalcular a arvore de formação
     * @params{Object} widget widget formação de preço
     * @params{int} nrformprecoservico nr da formação de preço
    **/
    this.recalculaFormacaoItens = function(widget, nrformprecoservico, nritemorcamento){
        var deferred = ZHPromise.defer();
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO",
            "NRITEMORCAMENTO"    : nritemorcamento
        };
        
        General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function() {
            Utilities.abreArvoreElementos(widget).then(function(){
                ScreenService.showMessage("Processo realizado com sucesso!");
                deferred.resolve();
            });
        })
        .catch(function(error) {
            console.log(error);
            deferred.reject();
        });
        
        return deferred.promise;
    };
    /**
     * Recalcula a árvore de elementos
     * @params{Object} widget widget editar itens de formação de preço
     * .parent substituido pela widget da grid principal
    **/
    this.recalcularArvore = function(widget) {
        var nrformprecoservico;
        if(widget.name === "editaCalculoItensFormPreco"){
            var wdgMObraIserindo = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            nrformprecoservico   = wdgMObraIserindo.currentRow.NRFORMPRECOSERVICO;
            this.recalculaFormacaoItens(widget, nrformprecoservico, wdgMObraIserindo.currentRow.NRRELACFORMACAOPROP);
        } else {
            var wdgMObra = templateManager.container.getWidget("wdgItensMaoObra");
            nrformprecoservico = wdgMObra.currentRow.NRFORMPRECOSERVICO;
            this.recalculaFormacaoItens(widget, nrformprecoservico, wdgMObra.currentRow.NRRELACFORMACAOPROP).then(function(){
                Utilities.abreArvoreElementos(widget);
                this.atualizarValorItem();
            }.bind(this));
        }
    };
    /**
     * Volta para a aba de Proposta.
     * @param {object} widget - Widget de Serviço (inserindo)
     */
    this.voltarComposicaoProposta = function(widget) {
        widget.enabled = false;
        Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgServicosInserindo", templateManager.container.getWidget("wdgServicosInserindo").dataSource.data);
        setLocalVar("QTDIASVALIDADE_PROPOSTA", templateManager.container.getWidget("wdgProposta").currentRow.QTDIASVALIDADE);
        
        var wdgPropostaTabbed     = templateManager.container.getWidget("wdgPropostaTabbed");
        wdgPropostaTabbed.enabled = true;
        wdgPropostaTabbed.activate();
    };
    /**
     * habilita a edição das quantidades relacionadas aos serviços
     * @params{widget} widget servico grid
     * @params{String} event - nome do local em que a row estará
    **/
    this.editarQuantidadeServicos = function(widget, event){
        var wdgEditServ = templateManager.container.getWidget("wdgEditarServico");
        var wdgTabbed   = templateManager.container.getWidget("wdgTabbedServico");
        
        wdgEditServ.currentRow = Util.clone(((event === 'checkedRow') ? widget.getLastSelectedRow() : widget.currentRow), []);
        if(wdgEditServ.currentRow._childCount !== 0){
            wdgEditServ.fields.forEach(function(field){
                field.readOnly = true;
                field.value(null);
            }); 
        } else {
            wdgEditServ.fields.forEach(function(field){
                field.readOnly = false;
            }); 
    
            var label = "";
            if(wdgEditServ.currentRow._parent){
                label = this.getArvoreServico(widget, wdgEditServ.currentRow, [], false).filter(function(servico) {
                    return !servico._parent;
                });
                label = (label.length == 0) ? "" : label[0]["NMELEMENTO"] + " -> ";
            }
            
            wdgEditServ.label = label + wdgEditServ.currentRow.NMELEMENTO;    
            templateManager.updateTemplate();
        }
        
        wdgTabbed.activate();
    };
    /**
     * Avança para a widget de custo do serviço escolhido
     * @params{Object} widget - widget imposto inserindo
    **/
    this.avancarCusto = function(widget){
        if (!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            widget.enabled = false;
            Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgImpostoProposta", widget.dataSource.data);
            var wdgCusto = templateManager.container.getWidget("wdgCustoProposta");
            wdgCusto.enabled   = true;
            wdgCusto.isVisible = true;
            wdgCusto.mode = "edit";
            wdgCusto.activate().then(function() {
                this.setFiltroCustoServico(wdgCusto);
            }.bind(this));
        }
    };
    /**
     * Aplica um filtro widget de custo durante a inserção da proposta
     * @params{Object} widget - widget serviços inserindo
    **/
    this.setFiltroCustoServico = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgServico  = templateManager.container.getWidget("wdgServicosInserindo");
        var composicoes = [];
        wdgServico.dataSource.getCheckedRows().forEach(function(serv) {
            composicoes.push(serv.NRCOMPSERVICO);
        });
        var params = {
            "NRTIPOORCAMENTO": wdgProposta.currentRow.NRTIPOORCAMENTO,
            "NRCOMPSERVICO": composicoes
        };
        General.callRestEngine("/fct_orcamentoaliment_widget/getComposicoesServicos", "Row", params).then(function(response) {
            widget.dataSource.data = response.dataset.fct_custocomposicao;
            widget.dataSource.data.forEach(function(custo){
                custo.NRCOMPSERVICO = this.retirarValoresDuplicados(custo.NRCOMPSERVICO.split("_"));
            }.bind(this));
            
            widget.getAction("save").isVisible   = false;
            widget.getAction("cancel").isVisible = false;
            this.setQuantidadesServicos();
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Volta para a aba de serviços.
     * @param {object} widget - Widget de custo (inserindo)
     */
    this.voltarServicoProposta = function(widget) {
        widget.enabled  = false;
        var wdgServicos = templateManager.container.getWidget("wdgTabbedServico");
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgServicosInserindo").then(function(response) {
            if(response) {
                var wdgServicoGrid = templateManager.container.getWidget("wdgServicosInserindo");
                wdgServicoGrid.dataSource.data = JSON.parse(response);
            }
            
            wdgServicos.enabled = true;
            wdgServicos.activate();
        }.bind(this));
    };
    /**
     * Avança para a widget de Mão de obra durante a inserção
     * @params{Object} widget - widget custo inserindo
    **/
    this.avancarMaoObra = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            widget.enabled = false;
            Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgCustoProposta", widget.dataSource.data);
            var wdgMObraProposta = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
            var wdgMObraGrid     = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            wdgMObraProposta.enabled   = true;
            wdgMObraProposta.isVisible = true;
            wdgMObraGrid.mode          = "edit";
            wdgMObraProposta.activate().then(function(){
                var params = {
                    "NRSEQORCAMENTO" : 0,
                    "NRITEMORCAMENTO": -1
                };
                
                General.callRestEngine("product#/fct_relacestruturaprop_widget/deletaAlteracoesInutilizadas", "Row", params).then(function(){
                    wdgMObraProposta.getField('NRCALCFORMPRECSERV').readOnly = false;
                });
            });
        }
    };
    /**
     * filtra o campo "Itens da Formação de Preço" de acordo com as 
     * formações de preço selecionadas
     * @params{Object} widget - widget mão de obra
     **/
    this.filtrarFieldItemFormPreco = function(widget) {
        var formPreco = widget.currentRow.NRFORMPRECOSERVICO;
        widget.getField('NRCALCFORMPRECSERV').dataSourceFilter[0].value = formPreco;
        widget.getField('NRCALCFORMPRECSERV').reload();
    };
    /***
     * retorna um serviço para cada formação
    **/
    this.getDataFormacaoServico = function(widget){
        var fieldFormacao = widget.getField("NRFORMPRECOSERVICO");
        var fieldServico  = widget.getField("CDSERVORCA");
        var data          = [];
        fieldServico.dataSource.getCheckedRows().forEach(function(servico){
            fieldFormacao.dataSource.getCheckedRows().forEach(function(formacao){
                data.push({
                    "NRFORMPRECOSERVICO": formacao.NRFORMPRECOSERVICO,
                    "NMSERVORCA"        : servico.NMSERVORCA,
                    "CDSERVORCA"        : servico.CDSERVORCA
                });
            });
        });
        
        if(data.length <= 0)
            data = fieldFormacao.dataSource.getCheckedRows();
            
        return data;
    };
    /**
     * Adiciona no grid as mão de obras selecionadas 
     * @params{Object} widget- widget mão de obra 
    **/
    this.addMaoObraProposta = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var mObraGrid    = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            var tabbedMObra  = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
            var dataFormacao  = this.getDataFormacaoServico(widget);
            var fieldItemForm = widget.getField("NRCALCFORMPRECSERV");
            var newData       = [];
            var itemPai       = null;
            
            this.salvarAlteracoesCalculo(widget, 0, dataFormacao).then(function(sucess){
                widget.currentRow.QTFORMACAO = "0";
                dataFormacao.forEach(function(formacao) {
                    var idformacao      = 0;
                    var valorTotal      = 0;
                    var nrrelacformacao = null;
                    sucess.filter(function(idForm){
                        if(idForm.NRFORMPRECOSERVICO === formacao.NRFORMPRECOSERVICO && idForm.CDSERVORCA === formacao.CDSERVORCA){
                            idformacao      = idForm.IDFORMACAO;
                            nrrelacformacao = idForm.NRRELACFORMACAOPROP;
                        }
                    });
                    fieldItemForm.dataSource.getCheckedRows().forEach(function(item){
                        if(item.NRFORMPRECOSERVICO === formacao.NRFORMPRECOSERVICO && !item._parent){
                            valorTotal += item.VRTOTAL;
                            itemPai     = item.NRCALCFORMPRECSERV;
                        }
                    });
                    newData.push({
                        "NMFORMPRECOSERVICO"    : formacao.NMSERVORCA,
                        "NRFORMPRECOSERVICO"    : formacao.NRFORMPRECOSERVICO,
                        "VRUNITARIO"            : valorTotal,
                        "VRTOTAL"               : valorTotal * parseInt(widget.currentRow.QTFORMACAO),
                        "QTFORMACAO"            : parseInt(widget.currentRow.QTFORMACAO),
                        "IDPOSICAOFORM"         : formacao.NRFORMPRECOSERVICO + "_" + idformacao,
                        "IDALTERADO"            : "N",
                        "NRCALCFORMPRECSERV"    : itemPai,
                        "IDFORMACAO"            : idformacao,
                        "NRRELACFORMACAOPROP"   : nrrelacformacao,
                        "CDSERVORCA"            : formacao.CDSERVORCA
                    });
                });
                mObraGrid.dataSource.data = mObraGrid.dataSource.data.concat(newData);
                fieldItemForm.dataSourceFilter[0].value = [];
                templateManager.updateTemplate();
        
                widget.currentRow.NRFORMPRECOSERVICO = [];
                widget.currentRow.NRCALCFORMPRECSERV = [];
                widget.currentRow.CDSERVORCA         = [];
    
                fieldItemForm.readOnly = false;
                fieldItemForm.reload();
                tabbedMObra.activate().then(function(){
                    setTimeout(function() {
                        mObraGrid.getAction("save").isVisible = false;
                        mObraGrid.getAction("cancel").isVisible = false;
                    }, 100);
                });
            }.bind(this));   
        }
    };
    /**
     * Recupera o valor total de uma árvore de elementos
     * @params{Object} widget - widget mão de obra 
    *
    this.recuperarValorMaoObra = function(widget){
        var fieldItens = widget.getField("NRCALCFORMPRECSERV");
        var itensSelecionados = fieldItens.dataSource.getCheckedRows();
        fieldItens.dataSource.data.forEach(function(item){
           
        });
    };
    */
    /**
     * Volta para a widget de custo durante a inserção da proposta
     * @params{Object} widget - widget mão de obra
    **/
    this.voltarCustoServico = function(){
        var wdgTabbed     = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        wdgTabbed.enabled = false;
        wdgTabbed.getField("NRCALCFORMPRECSERV").dataSourceFilter[0].value = [];

        var wdgCusto = templateManager.container.getWidget("wdgCustoProposta");
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgCustoProposta").then(function(response) {
            if(response)
                wdgCusto.dataSource.data = JSON.parse(response);
                
            wdgCusto.enabled = true;
            wdgCusto.activate();
        }.bind(this));
    };
    /**
     * Deleta a mão de obra escolhida durante a inserção da proposta
     * @params{Object} widget - widget mão de obra
    **/
    this.deletarMaoObraEscolhida = function(widget){
        ScreenService.confirmMessage("Retirar Mão de Obra da Proposta?", "question", function(){
            var params = {
                "NRSEQORCAMENTO" : 0,
                "NRITEMORCAMENTO": -1,
                "NRCALCFORMPRECSERV": widget.currentRow.NRCALCFORMPRECSERV, 
                "NRFORMPRECOSERVICO": widget.currentRow.NRFORMPRECOSERVICO
            };
            
            General.callRestEngine("product#/fct_relacestruturaprop_widget/deletarArvoreFormPrecoItPro", "Row", params).then(function(){
                var fieldFormacao = templateManager.container.getWidget("wdgTabbedPrincipalMObra").getField("NRFORMPRECOSERVICO");
                var index         = widget.dataSource.data.indexOf(widget.currentRow);
                widget.dataSource.data.splice(index, 1);
                
                if(fieldFormacao.dataSourceFilter.length > 0) {
                    index = fieldFormacao.dataSourceFilter[0].value.indexOf(widget.currentRow.NRFORMPRECOSERVICO);
                    fieldFormacao.dataSourceFilter[0].value.splice(index, 1);
                    fieldFormacao.reload();
                }
            }.bind(this));  
        });
    };
    /**
     * Abre o grid de edição das formações de preço 
     * durante a inserção da proposta
     * @params{Object} widget - widget mão de obra
    **/
    this.abrirWdgEditarFormacaoPreco = function(widget){
        if(!onClickMobra || onClickMobra !== widget.currentRow.NRFORMPRECOSERVICO){
            onClickMobra = widget.currentRow.NRFORMPRECOSERVICO;
        } else{
            var tabbedFormacao = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
            var itensCalculo   = templateManager.container.getWidget("editaCalculoItensFormPreco");
            var wdgTabbedMObra = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
            var maoObraGrid    = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            var ignoredKeys    = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
            
            tabbedFormacao.mode      = 'edit';
            tabbedFormacao.isVisible = true;
            itensCalculo.isVisible   = true;
            widget.enabled           = false;
            wdgTabbedMObra.enabled   = false;
            idItensMObraAlterado     = false;
            
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.currentRow.VRUNITARIO = 0;
            widget.currentRow.VRTOTAL    = 0;
        
            if(!widget.currentRow.IDALTERADO || widget.currentRow.IDALTERADO === 'N'){
                if(itensCalculo.dataSourceFilter.length === 5)
                    itensCalculo.dataSourceFilter.pop();
            } else {
                itensCalculo.dataSourceFilter[4] = {
                    name     : "NRCALCFORMPRECSERV",
                    operator : "IN",
                    value    : maoObraGrid.currentRow.NRCALCFORMPRECSERVS
                }; 
            }

            ScreenService.openSwipe(tabbedFormacao).then(function() {
                itensCalculo.dataSourceFilter[0].value = maoObraGrid.currentRow.NRFORMPRECOSERVICO;
                itensCalculo.dataSourceFilter[3].value = maoObraGrid.currentRow.IDFORMACAO;

                Utilities.abreArvoreElementos(itensCalculo);
                onClickMobra = 0;
            }.bind(this));    
        }
    };
    /**
     * Vusca o valor do último código inserido na tabela fct_relacformacaoprop 
    **/
    this.getCodeRelacFormacaoProp = function(){
        var deferred = ZHPromise.defer();
        if(codeRelacFormacao){
            codeRelacFormacao++;
            deferred.resolve();
        } else {
            General.callRestEngine("product#/fct_orcamentoaliment_widget/getCodeRelacFormacaoProp", "Row", {}).then(function(response) {
                if(response.dataset && response.dataset.fct_relacformacaoprop && response.dataset.fct_relacformacaoprop.NRULTRELACFORM)
                    codeRelacFormacao = response.dataset.fct_relacformacaoprop.NRULTRELACFORM;
                deferred.resolve();
            }).catch(function(error) {
                deferred.reject();
            });
        }
        
        return deferred.promise;
    };
    /**
     * Salva as alterações feitas nos elementos do cálculo
     * @params{Object} widget - widget editar itens formação de preço
     * @params{boolean} isNew - currentRow.__is_new
     * @params{Array} nrItensProp - array de itens da proposta
    **/
    this.salvarAlteracoesCalculo = function(widget, nrseqorcamento, dataFormacao) {
        var deferred  = ZHPromise.defer();
        var formprecoservico = [];
        var idFormacoes      = [];
        this.getCodeRelacFormacaoProp().then(function(){
            dataFormacao.forEach(function(formacao){
                codeRelacFormacao++;
                idFormacoes.push({
                    "NRFORMPRECOSERVICO" : formacao.NRFORMPRECOSERVICO,
                    "IDFORMACAO"         : codeRelacFormacao,
                    "NRRELACFORMACAOPROP": codeRelacFormacao,
                    "CDSERVORCA"         : formacao.CDSERVORCA
                });
                widget.getField("NRCALCFORMPRECSERV").dataSource.getCheckedRows().forEach(function(elemento){
                    if(formacao.NRFORMPRECOSERVICO === elemento.NRFORMPRECOSERVICO){
                        formprecoservico.push({
                            "NRFORMPRECOSERVICO": formacao.NRFORMPRECOSERVICO,
                            "NRCALCFORMPRECSERV": "_" + elemento.NRCALCFORMPRECSERV + "_",
                            "IDFORMACAO"        : codeRelacFormacao
                        });
                    }
                });
            });
        
            var params = {
                "FORMPRECOSERVICOS" : formprecoservico,
                "NRSEQORCAMENTO"    : nrseqorcamento,
                "NRITEMORCAMENTO"   : -1
            };
            
            General.callRestEngine("product#/fct_itemorcamentoaliment_widget/insereMaoObraFormacao", "Row", params).then(function(){
                deferred.resolve(idFormacoes);
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
                deferred.reject(error);
            });
        }.bind(this));
        return deferred.promise;
    };
    /**
     * habilita / desabilita o campo "Expressão"
     * @param {object} widget - widget editar item formação
     **/
    this.habilitaExpressao = function(widget){
        var isNew = widget.currentRow.__is_new;
        widget.currentRow.IDTIPOCALCULO = (widget.currentRow.IDTIPOCALCULO) ? widget.currentRow.IDTIPOCALCULO : "V";
        if (widget.currentRow.IDTIPOCALCULO == 'E') {
            widget.getField("DSEXPRESSAOCALC").readOnly    = false;
            widget.getField("DSEXPRESSAOCALC").validations = {'required':true};
            widget.getField("VRPERCCALCULO").readOnly      = false;
            widget.getField("VRPERCCALCULO").validations   = {};
            widget.getField("VRUNITARIO").readOnly         = true;
            widget.getField("VRUNITARIO").validations      = {};
            widget.getField("VRTOTAL").validations         = {};
            
            if(isNew) {
                widget.currentRow.VRUNITARIO = null;
                widget.currentRow.VRTOTAL    = null;
            }
        } else if (widget.currentRow.IDTIPOCALCULO == 'S'){
            widget.getField("DSEXPRESSAOCALC").readOnly    = true;
            widget.getField("DSEXPRESSAOCALC").validations = {};
            widget.currentRow.DSEXPRESSAOCALC              = null;
            widget.getField("VRPERCCALCULO").readOnly      = true;
            widget.getField("VRPERCCALCULO").validations   = {};
            widget.currentRow.VRPERCCALCULO                = null;
            widget.getField("VRUNITARIO").readOnly         = true;
            widget.getField("VRUNITARIO").validations      = {};
            widget.getField("VRTOTAL").validations         = {};
            
            if(isNew) {
                widget.currentRow.VRUNITARIO = null;
                widget.currentRow.VRTOTAL    = null;
            }
        } else {
            widget.getField("DSEXPRESSAOCALC").readOnly    = true;
            widget.getField("VRPERCCALCULO").readOnly      = true;
            widget.getField("DSEXPRESSAOCALC").validations = {};
            widget.getField("VRPERCCALCULO").validations   = {};
            widget.currentRow.DSEXPRESSAOCALC              = null;
            widget.currentRow.VRPERCCALCULO                = null;
            widget.getField("VRUNITARIO").readOnly         = false;
            widget.getField("VRUNITARIO").validations      = {'required':true};
            widget.getField("VRTOTAL").validations         = {'required':true};
        }
    };

     /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * @param {object} widget - widget editar item formação
     **/
    this.bloquearDadosMedida = function(widget){
        var usamedida = widget.getField('IDUSAMEDIDA').value();
        if (usamedida == 'N') {
            widget.getField('DSTPMEDIDASERV').readOnly    = true;
            widget.getField('DSTPMEDIDASERV').validations = {};
            widget.getField('CDTPMEDIDASERV').validations = {};
            widget.getField('QTMEDIDACALC').readOnly      = true;
            widget.getField('QTMEDIDACALC').validations   = {};
            widget.getField('DSTPMEDIDASERV').setValue(null);
            widget.getField('CDTPMEDIDASERV').setValue(null);
            widget.getField('QTMEDIDACALC').setValue(null);
            
        } else {
            widget.getField('DSTPMEDIDASERV').readOnly    = false;
            widget.getField('DSTPMEDIDASERV').validations = {"required": true};
            widget.getField('QTMEDIDACALC').readOnly      = false;
            widget.getField('QTMEDIDACALC').validations   = {"required": true};
        }
    };
    /**
     * Volta para a widget de mão de obra durante a inserção da proposta
     * @params{Object} widget - widget editar itens formação de preço
    **/
    this.voltarWdgMObraProposta = function(widget) {
        var rowsAlteradas = widget.dataSource.data.filter(function(data) {
            return data.changed;
        });
        
        if(rowsAlteradas.length == 0 && !idItensMObraAlterado) {
            this.utilizarItensAlteradosFormPreco(widget);
        } else {
            ScreenService.confirmMessage("Deseja salvar as alterações realizadas?", "question", function(){
                this.utilizarItensAlteradosFormPreco(widget);
            }.bind(this), function() {
                var wdgTabbed    = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
                var wdgGridMObra = templateManager.container.getWidget("wdgMaoObraGridInserindo");
                
                wdgGridMObra.currentRow.VRTOTAL    = wdgGridMObra.backupRow.VRTOTAL;
                wdgGridMObra.currentRow.VRUNITARIO = wdgGridMObra.backupRow.VRUNITARIO;
                
                wdgTabbed.isVisible = false;
                this.voltarBackupItensMaoObra(widget);
            }.bind(this));
        }
    };
    /**
     * Volta no banco de dados os valores antigos da formação de preço
     * @params{Object} widget - widget formação de preco
    **/
    this.voltarBackupItensMaoObra  = function(widget) {
        var wdgGridMObra = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgTabbed    = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        var params = {
            "NRRELACFORMACAOPROP": wdgGridMObra.currentRow.NRRELACFORMACAOPROP, 
            "NRFORMPRECOSERVICO" : wdgGridMObra.currentRow.NRFORMPRECOSERVICO,
            "NRCALCFORMPRECSERV" : "_" + wdgGridMObra.currentRow.NRCALCFORMPRECSERV + "_",
            "IDFORMACAO"         : wdgGridMObra.currentRow.IDFORMACAO
        };
        
        General.callRestEngine("product#/fct_orcamentoaliment_widget/voltarBackupItensMaoObra", "Row", params);
        ScreenService.closeSwipe().then(function(){
            widget.dataSource.data = [];
            widget.dataSource.clearCheckedRows();
            
            wdgTabbed.enabled    = true;
            wdgGridMObra.enabled = true;
            wdgTabbed.activate();
        });
        
        templateManager.updateTemplate();
    };
    /**
     * Habilita a tela para receber um novo item de formação de preço
     * e seta o número de ordenação na tela
     * @params{Object} widget widget editar formação de preço
    **/
    this.adicionarItensFormacao = function(widget){
        if(widget.mode !== "view"){
            widget.newRow();
            this.bloquearDadosMedida(widget);
        } else {
            var nmElementoSup = widget.currentRow.DSREFECALCULO;
            var nrCalcServPai = widget.currentRow.NRCALCFORMPRECSERV;
            widget.newRow();
            this.bloquearDadosMedida(widget);
            widget.currentRow.DSREFECALCULOPAI      = nmElementoSup;
            widget.currentRow.NRCALCFORMPRECSERVPAI = nrCalcServPai;
        }
        
        var widgetParent = null;
        if(widget.name === "wdgItensFormacaoPrecoProposta"){
            widgetParent = templateManager.container.getWidget("wdgItensMaoObra");
        } else {
            widgetParent = templateManager.container.getWidget("wdgMaoObraGridInserindo");
            widget.template = "widget/form.html";
        }
        
        var nritemorcamento    = widgetParent.currentRow.NRRELACFORMACAOPROP;
        var nrformprecoservico = widgetParent.currentRow.NRFORMPRECOSERVICO;
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "NRITEMORCAMENTO"    : nritemorcamento,
            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO"
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/recebeOrdenacao', "Row", params).then(function(response){
            if(!response.dataset.ORDENACAO){
                widget.currentRow.IDORDENACAO = 10;
            } else {
                var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;
                ordenacao = ordenacao ? (ordenacao + 10) : 10;
                widget.currentRow.IDORDENACAO        = ordenacao;
                widget.currentRow.NRFORMPRECOSERVICO = nrformprecoservico;
                widget.currentRow.NRITEMORCAMENTO    = nritemorcamento;
            }
        }, function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Filtra a field de elemento superior para aperecer apenas os elementos
     * já inseridos na árvore
     * @params{Object} widget - widget editar formaçao de preço
    **/
    this.filtraElementoSuperiorItensFormacao = function(widget){
        if(!widget.currentRow.DSREFECALCULOPAI){
            var nrformprecoservico = null;
            widget.currentRow.DSREFECALCULOPAI = "";
            nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
            if(widget.currentRow.NMELEMENTO){
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[0].value = nrformprecoservico;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[1].value = (widget.currentRow.NRCALCFORMPRECSERV) ? widget.currentRow.NRCALCFORMPRECSERV : -1;
                
                if(widget.name === "wdgItensFormacaoPrecoProposta")
                    widget.getField("DSREFECALCULOPAI").dataSourceFilter[2].value = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;
                else 
                    widget.getField("DSREFECALCULOPAI").dataSourceFilter[4].value = templateManager.container.getWidget('wdgMaoObraGridInserindo').currentRow.NRRELACFORMACAOPROP;
                
                widget.getField("DSREFECALCULOPAI").reload();
            }
        }
    };
    /**
     * Realiza a validação  do campo "identificador". 
     * Não podem haver dois identificadores iguais em uma mesma composição.
     * @param {object} widget widget
     **/
    this.validaIdentificador = function(widget){
        var widgetParent = null;
        if(widget.name === "wdgItensFormacaoPrecoProposta")
            widgetParent = templateManager.container.getWidget("wdgItensMaoObra");
        else 
            widgetParent = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var nritemorcamento    = widgetParent.currentRow.NRRELACFORMACAOPROP;
        var nrformprecoservico = widgetParent.currentRow.NRFORMPRECOSERVICO;
        var IdItemCalculo      = widget.currentRow.IDITEMCALCULO;
        var nrCalcFormPrecServ = widget.currentRow.NRCALCFORMPRECSERV;
        var params = {
            "NRFORMPRECOSERVICO"   : nrformprecoservico,
            "NRITEMORCAMENTO"      : nritemorcamento,
            "IDITEMCALCULO"        : IdItemCalculo,
            "NRCALCFORMPRECSERV"   : nrCalcFormPrecServ,
            "TABELA"               : "FCT_CALCFORMPRECSERV_ITPRO"
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/validaIdentificador', "Row", params).then(function(response){
            var identificador = response.dataset.IDENTIFICADOR.VALIDA;
            if (identificador != 0){
                widget.currentRow.NMELEMENTO = "",
                widget.currentRow.IDITEMCALCULO = null;
                ScreenService.showMessage('O identificador "'+IdItemCalculo+'" já está sendo utilizado para outro elemento.'); 
            }
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
    };
    /**
     * Realiza o controle de Percentual do field "VRPERCCALCULO"
     * @param {object} widget - widget mão de obra
    */
    this.Validapercentual = function(widget){
        var vrdesconto = widget.getField('VRPERCCALCULO').value();
        if (vrdesconto < 0 || vrdesconto > 100) 
            widget.currentRow.VRPERCCALCULO = null;
    };
    /**
     * Realiza o cálculo do Valor Total
     * @param {object} widget - widget mão de obra
     **/
    this.calculaValorTotal = function(widget){
        var vrUnit = widget.currentRow.VRUNITARIO;
        if (!vrUnit) {
            vrUnit = 0;
        } else {
            vrUnit = vrUnit.toString();
            var primeiraPos = vrUnit.indexOf(".");
            var ultimaPos   = vrUnit.lastIndexOf(".");
            if (primeiraPos == ultimaPos) {
                if (vrUnit.includes(",")) 
                    vrUnit = parseFloat(vrUnit.replace(".", "").replace(",", "."));
                else 
                    vrUnit = parseFloat(vrUnit.replace(",", "."));
            } else {
                var parteInicial = vrUnit.substr(0, primeiraPos + 1);
                var parteFinal   = vrUnit.substr(primeiraPos + 1);
                vrUnit = parseFloat(parteInicial.replace(".", "") + parteFinal);
            }
        }
        
        var qtCalc    = widget.currentRow.QTCALCULO;
        var usamedida = widget.currentRow.IDUSAMEDIDA;
        var qtmedida  = widget.currentRow.QTMEDIDACALC;
        var vrTot     = 0;
        if (usamedida == 'S')
            vrTot = (vrUnit * qtCalc) * qtmedida ; 
        else 
            vrTot = vrUnit * qtCalc;
        widget.currentRow.VRTOTAL = vrTot;
    };
    /**
     * Salva as alterações realizadas na tela de edição dos 
     * itens de formação de preço
     * @params{Object} widget - widget editar itens de formação (mão de obra)
    **/
    this.salvarAlteracoesItemCalculo = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            idItensMObraAlterado = true;
            
            var wdgParent;
            var isnew       = widget.currentRow.__is_new;
            var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
            widget.currentRow.NRSEQORCAMENTO  = (widget.currentRow.NRSEQORCAMENTO) ? widget.currentRow.NRSEQORCAMENTO : 0;
            widget.currentRow.NRITEMORCAMENTO = (widget.name === "wdgItensFormacaoPrecoProposta") ? 0 : -1;
            if(!widget.currentRow.DSREFECALCULO)
                widget.currentRow.DSREFECALCULO = widget.currentRow.IDITEMCALCULO + " - " + widget.currentRow.NMELEMENTO;
            
            if(widget.name === "editaCalculoItensFormPreco" || isnew){
                wdgParent = templateManager.container.getWidget("wdgMaoObraGridInserindo");
                widget.currentRow.NRSEQORCAMENTO = 0;
            } else {
                widget.dataSource.name = "product#/crud/fct_calcformprecserv_itpro_widget";
                wdgParent       = templateManager.container.getWidget("wdgItensMaoObra");
                var wdgProposta = templateManager.container.getWidget("wdgProposta");
                widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
            }
                
            widget.currentRow.NRRELACFORMACAOPROP = wdgParent.currentRow.NRRELACFORMACAOPROP;
            this.criarParametrosNewCode(widget);
            widget.saveRow().then(function(response) {
                if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0)
                    widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
                
                if (tipoCalculo == 'S'){
                    var params = {
                        "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                        "NRITEMORCAMENTO"    : widget.currentRow.NRITEMORCAMENTO,
                        "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO"
                    };
                    
                    General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);  
                }
                
                ScreenService.confirmMessage("Deseja recalcular a Formação de Preço?", 'question', function (success) {
                    if(widget.name === "wdgItensFormacaoPrecoProposta"){
                        widget.dataSource.name = "product#/crud/fct_calcformprecservtelaitensprop_widget";
                        ScreenService.closeSwipe();
                    } else {
                        widget.template = "widget/tree/grid.html";
                    }
                    
                    widget.list();
                    this.recalcularArvore(widget);
                }.bind(this), function(){
                    if(widget.name === "wdgItensFormacaoPrecoProposta"){
                        widget.dataSource.name = "product#/crud/fct_calcformprecservtelaitensprop_widget";
                        if(!isnew){
                            widget.view();
                        } else { 
                            ScreenService.closeSwipe();
                            widget.list();
                        }
                    } else {
                        if(!isnew){
                            UtilitiesPropostaContrato.habilitarViewItensFormacao(widget);
                        } else {
                            widget.template = "widget/tree/grid.html";
                            widget.list();
                        }
                    }
                });    
            }.bind(this))
            .catch(function(error){
                if(widget.name === "wdgItensFormacaoPrecoProposta")
                    widget.dataSource.name = "product#/crud/fct_calcformprecservtelaitensprop_widget";
                
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Adiciona na tela de mão de obra os itens selecionado
     * @params{Object} widget - widget editar itens formação de preço (mão de obra)
    **/
    this.utilizarItensAlteradosFormPreco = function(widget){
        ScreenService.closePopup();
        var wdgTabbed     = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        var wdgMobraGrid  = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var rows          = widget.getCheckedRows();
        if(widget.dataSource.data.length !== rows.length){
            if(rows.length == 0)
                rows = widget.dataSource.data;
            else 
                rows = UtilitiesPropostaContrato.buscaArvoreFormacaoCompleta(widget, rows);
        }
        this.criarParametrosNewCode(widget);
        rows.forEach(function(row){row.NRITEMORCAMENTO = -1;});
        General.callRestEngine("product#/fct_orcamentoaliment_widget/utilizarItensFormacaoProp", "DataSet", rows).then(function(){
            rows.forEach(function(row) {
                wdgMobraGrid.dataSource.data.forEach(function(item){
                    if(row.NRFORMPRECOSERVICO === item.NRFORMPRECOSERVICO && wdgMobraGrid.currentRow.NRRELACFORMACAOPROP === item.NRRELACFORMACAOPROP){
                        if(!item.NRCALCFORMPRECSERVS)
                            item.NRCALCFORMPRECSERVS = [];
                            
                        item.NRCALCFORMPRECSERVS.push(row.NRCALCFORMPRECSERV);
                        if(row._parent == null){
                            item.IDALTERADO = "S";
                            item.VRUNITARIO += row.VRTOTAL;
                            item.VRTOTAL    += parseFloat(row.VRTOTAL) * parseFloat(item.QTFORMACAO);
                        }
                    }
                });
            });
            
            ScreenService.closeSwipe().then(function(){
                widget.dataSource.data = [];
                widget.dataSource.clearCheckedRows();
                
                wdgTabbed.enabled    = true;
                wdgMobraGrid.enabled = true;
                wdgTabbed.activate();
            });
            
            templateManager.updateTemplate();
        });
    };
    /**
     * Deletas os itens de árvores da formação de preço 
     * não selecionadas 
     * @params{Object} widget - widget editar formação de preço
    **/
    this.deletarElementosNaoUtilizados = function(widget){
        var itensNSelects = widget.dataSource.data.filter(function(item){
            return !item.__isSelected && !item._parent;
        });
        
        UtilitiesPropostaContrato.buscaArvoreFormacaoCompleta(widget, itensNSelects).forEach(function(itemNSelect){
            widget.currentRow = itemNSelect;
            widget.removeRow().then(function() {
                widget.template = "widget/tree/grid.html";
                widget.list();
                var params = {
                    "NRFORMPRECOSERVICO" : widget.currentRow.NRFORMPRECOSERVICO,
                    "NRITEMORCAMENTO"    : widget.currentRow.NRRELACFORMACAOPROP,
                    "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO",
                };
                    
                General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function(){
                    Utilities.abreArvoreElementos(widget);
                }.bind(this))
                .catch(function(){
                    ScreenService.showMessage("Falha ao recalcular o valor da Formação de Preço.");
                });
            });
        });
    };
    /**
     * Avançar para widget de despesas
     * @params{Object} widget - widget despesas propostas
    **/
    this.avancarDespesas = function(widget){
        templateManager.container.getWidget("wdgTabbedPrincipalMObra").enabled = false;
        var wdgDespesa = templateManager.container.getWidget("wdgDespesas");
        wdgDespesa.enabled   = true;
        wdgDespesa.isVisible = true;
        wdgDespesa.mode = "edit";
        wdgDespesa.activate().then(function() {
            this.setFiltroDespesas(wdgDespesa);
        }.bind(this));
    };
    /**
     * Adiciona à tela de despesas o filtro de formação de preço e servico
     * @params{Object} widget    - widget despesas
     * @params{Object} wdgFilter - widget de filtro (servico ou formação)
     * @params{String} nameCampo - nome do campo que será filtrado 
    **/
    this.setFiltroDespesas = function(widget){
        var servicos  = [];
        var formacoes = [];
        templateManager.container.getWidget("wdgServicosInserindo").dataSource.getCheckedRows().forEach(function(serv){
            servicos.push(serv.NRCOMPSERVICO);
        });
        templateManager.container.getWidget("wdgMaoObraGridInserindo").dataSource.data.forEach(function(form){
            formacoes.push(form.NRFORMPRECOSERVICO);
        });
        servicos  = this.retirarValoresDuplicados(servicos);
        formacoes = this.retirarValoresDuplicados(formacoes);
        
        widget.dataSourceFilter[0].value = servicos;
        widget.dataSourceFilter[1].value = formacoes;
        widget.reload().then(function(){
            setTimeout(function(){
                widget.dataSource.data.forEach(function(despesa){
                    despesa.__isSelected = true;
                    widget.dataSource.addCheckedRows(despesa);
                }.bind(this));
                
                widget.getAction("save").isVisible   = false;
                widget.getAction("cancel").isVisible = false;
                templateManager.updateTemplate();
                this.calcularDespesaPorPeriodo(widget);
            }.bind(this), 300);
        }.bind(this));
    };
    /**
     * Verifica o período da despesa e cálcula o seu valor mensal
     * @params{Object} widget - widget despesas proposta inserindo
    **/
    this.validarECalcularPeriodo = function(rowDespesa, nameVrDesp, nameCampoVrTotal, nameCampoPeriodo){
        var vrDespesa = this.tirarMascaraValores(rowDespesa[nameVrDesp]);
        switch(rowDespesa[nameCampoPeriodo]){
                case '1': //Diária
                    rowDespesa[nameCampoVrTotal] = vrDespesa * 30;
                break;
                case '2': //Semanalmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa * (30/7);
                break;
                case '3': //Quinzenalmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa * 2;
                break;
                case '4': //Mensalmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa;
                break;
                case '5': //Trimestralmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa / 3;
                break;
                case '6': //Semestralmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa / 6;
                break;
                case '7': //Anualmente
                    rowDespesa[nameCampoVrTotal] = vrDespesa / 12;
                break;
            }
    };
    /**
     * Verifica qual widget está mandando calcular a despesa e envia a row certa
     * @params{Object} widget - widget despesas (item ou inserindo)
    **/
    this.calcularDespesaPorPeriodo = function(widget){
        var nameVrTotal = (widget.name === "wdgItensDespesasProposta") ? "VRDESPESAORCA" : "VRTOTDESPPERIODO";
        var namePeriodo = (widget.name === "wdgItensDespesasProposta") ? "CDPERIODODESPPROP" : "CDPERIODO";
        var nameVrDesp  = (widget.name === "wdgItensDespesasProposta") ? "VRUNITDESPESA" : "VRDESPESA";
        if(widget.name === "wdgItensDespesasProposta" && widget.mode === "edit"){
            this.validarECalcularPeriodo(widget.currentRow, "VRUNITDESPESA", "VRDESPESAORCA", "CDPERIODODESPPROP");
        } else {
            widget.dataSource.data.forEach(function(despesa){
                this.validarECalcularPeriodo(despesa, nameVrDesp, nameVrTotal, namePeriodo);
            }.bind(this));
        }
    };
    /**
     * Retira os valores repetidos de um array
     * @params{Array} arrayData - array de dados
    **/
    this.retirarValoresDuplicados = function(arrayData){
        return arrayData.filter(function(value, index){
            return arrayData.indexOf(value) === index;
        });
    };
    /**
     * Volta para a widget de mão de obra
     * durante a inserção da proposta
     * @params{Object} widget - widget despesas
    **/
    this.voltarDespesaMObra = function(widget){
        widget.enabled = false;
        var wdgMObra     = templateManager.container.getWidget("wdgTabbedPrincipalMObra");
        wdgMObra.enabled = true;
        wdgMObra.mode    = "edit";
        wdgMObra.activate();
    };
    /**
     * Abre um popup que permitirá a inserção de uma nova despesa
    **/
    this.adicionarNovaDespesa = function(widget){
        var despesas = [];
        var popupDespesa = templateManager.container.getWidget("wdgPopupInsereDesp");
        widget.dataSource.data.forEach(function(despesa){
            if(despesa.CDDESPESA)
                despesas.push(despesa.CDDESPESA);
        });
        
        popupDespesa.isVisible = true;
        ScreenService.openPopup(popupDespesa).then(function(){
            popupDespesa.getField("CDDESPESA").dataSourceFilter[0].value = despesas.length > 0 ? despesas : [-1];
            popupDespesa.getField("CDDESPESA").reload();
            popupDespesa.edit();
            popupDespesa.newRow();
            popupDespesa.activate();
        });
    };
    /**
     * Cancela a inserção de uma nova despesa
     * na tela de despesas durante a criação de uma proposta
    **/
    this.cancelarInsercaoNovaDespesa = function(){
        ScreenService.closePopup();
        var wdgDespesa  = templateManager.container.getWidget("wdgDespesas");
        wdgDespesa.enabled = true;
        wdgDespesa.mode    = "edit";
        wdgDespesa.activate().then(function(){
            setTimeout(function() {
                wdgDespesa.getAction("save").isVisible   = false;
                wdgDespesa.getAction("cancel").isVisible = false;
            }, 300);
        });
    };
    /**
     * Salva uma nova despesa criada durante a inserção da proposta
     * @params{Object} widget - widget popup despesas
    **/
    this.salvarNovaDespesa = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgDespesa = templateManager.container.getWidget("wdgDespesas");
            widget.currentRow.CDDESPESA.forEach(function(despesa){
                widget.getField("CDDESPESA").dataSource.data.forEach(function(valueDespesa){
                    if(valueDespesa.CDDESPESA === despesa){
                        var row = {
                            "DSDESPESA"       : valueDespesa.DSDESPESA,
                            "CDDESPESA"       : despesa,
                            "VRDESPESA"       : valueDespesa.VRDESPESA ? valueDespesa.VRDESPESA : "0",
                            "CDPERIODO"       : valueDespesa.CDPERIODO,
                            "VRTOTDESPPERIODO": 0,
                            "NRCOMPSERVICO"   : "",
                            "__is_new"        : false,
                            "__isSelected"    : true
                        };
                        wdgDespesa.dataSource.data.push(row);
                        wdgDespesa.dataSource.addCheckedRows(row);
                    }
                });
            });
            this.calcularDespesaPorPeriodo(wdgDespesa);
            this.cancelarInsercaoNovaDespesa();
        }
    };
    /**
     * Avançar para widget de investimento
     * @params{Object} widget - widget despesas propostas
    **/
    this.avancarInvestimento = function(widget){
        this.calcularWidgetDespesaInserindo(widget);
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgDespesas", widget.dataSource.data);
        
        widget.enabled = false;
        var wdgInvest  = templateManager.container.getWidget("wdgInvestimentosInserindo");
        wdgInvest.enabled   = true;
        wdgInvest.isVisible = true;
        wdgInvest.mode = "edit";
        wdgInvest.activate().then(function() {
            this.setFiltroInvestimento(wdgInvest);
        }.bind(this));
    };
    /**
     * Aplica um filtro de servico e formação na widget de investimento
     * além de calcular o valor total
     * @params{Object} widget - widget investimento
    **/
    this.setFiltroInvestimento = function(widget){
        var servicos  = [];
        var formacoes = [];
        templateManager.container.getWidget("wdgServicosInserindo").dataSource.getCheckedRows().forEach(function(serv){
            servicos.push(serv.NRCOMPSERVICO);
        });
        templateManager.container.getWidget("wdgMaoObraGridInserindo").dataSource.data.forEach(function(form){
            formacoes.push(form.NRFORMPRECOSERVICO);
        });
        
        servicos  = this.retirarValoresDuplicados(servicos);
        formacoes = this.retirarValoresDuplicados(formacoes);
        widget.dataSourceFilter[0].value = servicos;
        widget.dataSourceFilter[1].value = formacoes;
        widget.reload().then(function(){
            widget.dataSource.data.forEach(function(investimento){
                investimento.VRTOTINVESTIMENTO = investimento.VRINVESTIMENTO;
                investimento.QTINVESTIMENTO    = "1",
                investimento.__isSelected      = true;
                widget.dataSource.addCheckedRows(investimento);
            });
            widget.getAction("save").isVisible   = false;
            widget.getAction("cancel").isVisible = false;
        }.bind(this));
    };
    /**
     * Calcula o valor total do investimento
     * @params{Object} widget - widget investimento
    **/
    this.calcularVrTotInvestimento = function(rowInvestimento, vrunitario, vrTotal, qtInvestimento){
        var qtAmortizacao = (rowInvestimento["QTMESARMOTIZACAO_inline"])  ? rowInvestimento["QTMESARMOTIZACAO_inline"]  : rowInvestimento["QTMESARMOTIZACAO"];
        vrunitario     = (rowInvestimento[vrunitario + "_inline"])     ? rowInvestimento[vrunitario + "_inline"]     : rowInvestimento[vrunitario];
        qtInvestimento = (rowInvestimento[qtInvestimento + "_inline"]) ? rowInvestimento[qtInvestimento + "_inline"] : rowInvestimento[qtInvestimento];
        qtInvestimento = qtInvestimento ? qtInvestimento : 1;
            
        rowInvestimento[qtInvestimento] = (rowInvestimento[qtInvestimento] && rowInvestimento[qtInvestimento] !== 0) ? parseFloat(rowInvestimento[qtInvestimento]) : 1;
        rowInvestimento[vrTotal]        = parseFloat(qtInvestimento) * parseFloat(vrunitario);
        rowInvestimento.VRPARCELA       = parseFloat(rowInvestimento[vrTotal])/parseFloat(qtAmortizacao);
    };
    /**
     * Valida se o que está sendo trabalhado é o datasource da tela ou 
     * a currentRow
     * @params{Object} widget - widget investimento (itens ou inserindo)
    **/
    this.validarParaCalculoInvestimento = function(widget){
        var vrUnitario    = (widget.name = "wdgItensInvestimentosProp") ? "VRUNITINVESTORCA" : "VRINVESTIMENTO"; 
        var vrTotal       = (widget.name = "wdgItensInvestimentosProp") ? "VRTOTINVETORCA" : "VRTOTINVESTIMENTO"; 
        var quantidade    = (widget.name = "wdgItensInvestimentosProp") ? "QTINVESTORCA" : "QTINVESTIMENTO";

        if(widget.name === "wdgItensInvestimentosProp" && widget.mode !== "list"){
            this.calcularVrTotInvestimento(widget.currentRow, "VRUNITINVESTORCA", "VRTOTINVETORCA", "QTINVESTORCA", "QTMESARMOTIZACAO");
        } else {
            widget.dataSource.data.forEach(function(investimento){
                this.calcularVrTotInvestimento(widget.currentRow, vrUnitario, vrTotal, quantidade);
            });
        }
    };
    /**
     * Volta para a aba de despesas durante a inserção da proposta
     * @params{Object} widget - widget investimento proposta
    **/
    this.voltarInvestDepesas = function(widget){
        widget.enabled = false;
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgDespesas").then(function(response) {
            if(response) 
                wdgDespesas.dataSource.data = JSON.parse(response);
    
            wdgDespesas.enabled = true;
            wdgDespesas.mode    = "edit";
            wdgDespesas.activate();
        }.bind(this));
    };
    /**
     * Avançar para widget de taxas
     * @params{Object} widget - widget investimento
    **/
    this.avancarTaxas = function(widget){
        this.calcularWidgetInvetimentoInserindo(widget);
        Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgInvestimentosInserindo", widget.dataSource.data);
        widget.enabled = false;
        
        var wdgTaxas = templateManager.container.getWidget("wdgTaxasInserindo");
        wdgTaxas.enabled   = true;
        wdgTaxas.isVisible = true;
        wdgTaxas.mode = "edit";
        wdgTaxas.activate().then(function() {
            wdgTaxas.reload().then(function(){
                setTimeout(function() {
                    wdgTaxas.getAction("save").isVisible   = false;
                    wdgTaxas.getAction("cancel").isVisible = false;
                }, 300);
            });
        }.bind(this));
    };
    /**
     * Volta para a widget de investimento durante a inserção da proposta
     * @params{Object} widget - widget taxas proposta
    **/
    this.voltarTaxasInvest = function(widget){
        widget.enabled = false;
        var wdgInvest = templateManager.container.getWidget("wdgInvestimentosInserindo");
        Sm00001000_PropostaAlimenticia_Repository.getDataLocalStorageEngine("wdgInvestimentosInserindo").then(function(response) {
            if(response)
                wdgInvest.dataSource.data = JSON.parse(response);
                
            wdgInvest.enabled = true;
            wdgInvest.mode    = "edit";
            wdgInvest.activate();
        }.bind(this));
    };
    /**
     * Avança para a aba de custo final durante a inserção da proposta
     * @params{Object} widget - widget taxas proposta
    **/
    this.avancarCustoFinal = function(widget){
        widget.enabled    = false;
        var wdgCustoFinal = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        wdgCustoFinal.enabled   = true;
        wdgCustoFinal.isVisible = true;
        wdgCustoFinal.mode = "edit";
        wdgCustoFinal.activate().then(function() {
            this.setDadosCustoFinal();
        }.bind(this));
    };
    /**
     * Voltar para a widget de taxa durante a inserção da proposta
     * @params{Object} widget - widget curto final
    **/
    this.voltarCFinalTaxa = function(widget){
        widget.enabled = false;
        var wdgTaxas   = templateManager.container.getWidget("wdgTaxasInserindo");
        wdgTaxas.enabled = true;
        wdgTaxas.mode    = "edit";
        wdgTaxas.activate();
    };
    /**
     * Coloca a lista de custos financeiros em negrito
     * @params{Object} widget - widget grid financeiro 
    **/
    this.negritarListaCusto = function(element) {
        if (element.field && (["VRFINANCEIRO", "DSFINANCEIRO"].includes(element.field.name))) {
            var style = element.context.children[0].getAttribute("style");
            if (!style) {
                element.context.children[0].setAttribute("style", " font-weight: bold;");
            } else if (style.indexOf("font-weight: bold;") === -1) {
                element.context.children[0].setAttribute("style", style + " font-weight: bold;");
            }
        }
    };
    /**
     * Efetua os cálculos da widget de custo final
    **/
    this.setDadosCustoFinal = function(){
        var wdgFinanceiro = templateManager.container.getWidget("wdgResumoGrid");
        var wdgRServico   = templateManager.container.getWidget("wdgResumoServico");
        this.calcularValoresServico(wdgRServico);
        this.calcularFinanceiro(wdgFinanceiro, true);
    };
    /**
     * Retorna todos os serviços relacionados a árvore de serviço a partir de um 
     * elemento selecionado
     * @params{Object} rowElemento - elemento selecionado
    **/
    this.getServicosRelacionados = function(rowElemento){
        var wdgCusto      = templateManager.container.getWidget("wdgCustoProposta");
        var wdgServico    = templateManager.container.getWidget("wdgServicosInserindo");
        var cdcompservico = [];
        var cdServRelac   = [];
        
        this.getArvoreServico(wdgServico, rowElemento, [], false).forEach(function(servico) {
            cdcompservico.push(servico.NRCOMPSERVICO);
        });
        cdcompservico = this.retirarValoresDuplicados(cdcompservico);
        wdgCusto.dataSource.data.forEach(function(custo){
            custo.NRCOMPSERVICO.forEach(function(compCusto){
                if(!cdServRelac.includes(custo.CDSERVORCARELAC) && custo.CDSERVORCARELAC)
                    cdServRelac.push(custo.CDSERVORCARELAC);
            });
        });
        
        return cdServRelac;
    };
    /**
     * Retorna o valor total do custo de um serviço
     * @params{Objects} wdgCusto - widget de custo inserindo
     * @params{Objects} wdgServico - widget de servico inserindo 
     * @params{Objects} rowServico - row do elemento que terá o valor calculado
    **/
    this.getCustoServicoTotal = function(wdgCusto, wdgServico, rowServico){
        var vrCustoTot    = 0;
        var cdcomposicoes = [];
        var eleSomados    = [];
        var elementoRaiz  = [];
        var nrCompCusto   = [];
        var elementoAtual = null;
        
        this.getArvoreServico(wdgServico, rowServico, [], false).forEach(function(eleArvore){
            elementoRaiz.push(eleArvore.IDELEMENTO);
            cdcomposicoes.push(eleArvore.NRCOMPSERVICO);
		});
		
		elementoRaiz  = this.retirarValoresDuplicados(elementoRaiz);
		cdcomposicoes = this.retirarValoresDuplicados(cdcomposicoes);
		wdgCusto.dataSource.data.forEach(function(custo){
            elementoAtual = custo.IDELEMENTO; 
            if(elementoAtual.split("_").length < 3)
	            elementoAtual += "_" + custo.CDSERVORCARELAC;
	            
	        if(!eleSomados.includes(elementoAtual) && custo.CDSERVORCARELAC && elementoRaiz.includes(custo.IDELEMENTORAIZ)){
        		nrCompCusto = custo.NRCOMPSERVICO.filter(function (element, index) {
                    if(cdcomposicoes.indexOf(element) == -1)
                        return element;
                });
                if(nrCompCusto && nrCompCusto.length){
                    vrCustoTot += parseFloat(custo.VRCUSTOSERV);
                    eleSomados.push(elementoAtual);
                }
	        }
        });
        return vrCustoTot;
    };
    /**
     * Calcula os valores dos serviços da proposta
     * @params{Object} widget - widget de resumo do serviço
    **/
    this.calcularValoresServico = function(widget){
        widget.dataSource.data = [];
        var wdgCusto    = templateManager.container.getWidget("wdgCustoProposta");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgServico  = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgTaxas    = templateManager.container.getWidget("wdgTaxasInserindo");
        var elCalculado = [];
        var qtServicos  = 0;
        var nmElemento  = "";
        
        var totalOrcamento = this.calcularValorPropSemTaxa();
        totalOrcamento *= (1 + (this.getSomatorioDataSource(wdgTaxas, "VRPERCTAXA", true)/100));
        totalOrcamento *= (1 + (wdgProposta.currentRow.VRPERCMARKUP ? (wdgProposta.currentRow.VRPERCMARKUP/100) : 0));

        var elementoRaiz = wdgServico.dataSource.getCheckedRows().filter(function(serv) {
            if(serv.TIPO === "C" && !serv.IDARVOREPAI)    
                qtServicos += (serv.QTMENSAL <= 0) ? 1 : parseFloat(serv.QTMENSAL);
            return serv.TIPO === "C";
        });
        
        qtServicos = qtServicos <= 0 ? 1 : qtServicos;
        wdgCusto.dataSource.data.forEach(function(custoServ){
            custoServ.NRCOMPSERVICO.forEach(function(nrcompCusto){
                elementoRaiz.forEach(function(servico){
                    if(custoServ.NRCOMPSERVICO.includes(servico.NRCOMPSERVICO.toString()) && !elCalculado.includes(custoServ.IDELEMENTORAIZ)){
                        var vrPercServico  = this.getVrPercentualServico(null, servico);
                        var vrcusto        = this.getCustoServicoTotal(wdgCusto, wdgServico, servico);
                        var qtElemento = this.getArvoreServico(wdgServico, servico, [], false).filter(function(serv){
                            if(!serv.IDARVOREPAI)
                                nmElemento = serv.NMELEMENTO;
                            return !serv.IDARVOREPAI;
                        })[0]["QTMENSAL"];
                        var vrTotalServico = this.distribuiValorServico(vrPercServico/100, qtElemento, vrcusto);
                        widget.dataSource.data.push({
                            "NRCOMPSERVICO"       : servico.NRCOMPSERVICO,
                            "IDELEMENTO"          : custoServ.IDELEMENTORAIZ,
                            "CDSERVORCA"          : this.getServicosRelacionados(servico),
                            "NMELEMENTO"          : nmElemento,
                            "VRCUSTO"             : vrcusto,
                            "VRPERCSERVICO"       : vrPercServico,
                            "VRPRECOIDEAL"        : vrTotalServico / qtElemento,
                            "VRUNITITEMORCA"      : vrTotalServico / qtElemento,
                            "VRUNITITEMORCA_GRID" : vrTotalServico / qtElemento,
                            "VRTOTALITEMORCA"     : vrTotalServico,
                            "VRTOTIDEALSERV"      : vrTotalServico,
                            "QTITEMORCAMENTO"     : qtElemento,
                            "NRTIPOORCAMENTO"     : custoServ.NRTIPOORCAMENTO
                        });
                        elCalculado.push(custoServ.IDELEMENTORAIZ);
                    }
                }.bind(this));
            }.bind(this));
        }.bind(this));
    };
    /**
     * Distribui o valor dos demais componentes do serviço
     * de acordo com a porcentagem do custo alimentar de cada um
     * @params{float} vrpercservico - valor do percentual do servico
    **/
    this.distribuiValorServico = function(vrpercservico, qtElemento, vrcusto){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgTaxas    = templateManager.container.getWidget("wdgTaxasInserindo");
        var totalCustoAliment = this.getVrTotalServico(null);
        var qtMesesProposta   = wdgProposta.currentRow.QTDIASVALIDADE/30;
        
        var totalOrcamento = this.calcularValorPropSemTaxa();
        totalOrcamento *= (1 + (this.getSomatorioDataSource(wdgTaxas, "VRPERCTAXA", true)/100));
        totalOrcamento *= (1 + (wdgProposta.currentRow.VRPERCMARKUP ? (wdgProposta.currentRow.VRPERCMARKUP/100) : 0));
        
        var valorSCustoAlimentar = totalOrcamento - totalCustoAliment;
        var totalServico = (qtElemento * vrcusto * qtMesesProposta) + (valorSCustoAlimentar * vrpercservico);
        return totalServico;
    };
    /**
     * Cálculo financeiro ligados a proposta
     * @params{Object} widget - widget de resumo financeiro
     * @params{Boolean} calcularPrecoIdeal - necessidade do calculo do preço ideal 
    **/
    this.calcularFinanceiro = function(widget, calcularPrecoIdeal){
        var wdgMObra        = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgDespesas     = templateManager.container.getWidget("wdgDespesas");
        var wdgInvestimento = templateManager.container.getWidget("wdgInvestimentosInserindo");
        var wdgTaxas        = templateManager.container.getWidget("wdgTaxasInserindo");
        var wdgProposta     = templateManager.container.getWidget("wdgProposta");
        var wdgImpostos     = templateManager.container.getWidget("wdgImpostoProposta");

        var totalOrcaBruto = this.calcularValorPropSemTaxa();
        var totalTaxa      = totalOrcaBruto * (this.getSomatorioDataSource(wdgTaxas, "VRPERCTAXA", true)/100);
        totalOrcaBruto     += totalTaxa;
        
        var totalCmTaxa    = totalOrcaBruto;
        var totalMarkup    = totalOrcaBruto * (wdgProposta.currentRow.VRPERCMARKUP ? (wdgProposta.currentRow.VRPERCMARKUP/100) : 0);
        totalOrcaBruto     += totalMarkup;
        
        var totalImposto   = totalOrcaBruto * (this.getSomatorioDataSource(wdgImpostos, "VRPRECIMPOSTO", true)/100);
        var totalOrcamento = totalOrcaBruto - totalImposto;
        
        var totalMObra     = this.getSomatorioDataSource(wdgMObra, "VRTOTAL", false);
        var totalDespesa   = this.getSomatorioDataSource(wdgDespesas, "VRTOTDESPPERIODO", true);
        var totalInvest    = this.getSomatorioDataSource(wdgInvestimento, "VRPARCELA", true);
        var totalServico   = this.getVrTotalServico(null);
        var totalSTaxa     = this.calcularValorPropSemTaxa();
        
        this.setValoresWdgRFinanceiro(widget, totalOrcaBruto, totalImposto, totalOrcamento, totalServico, totalCmTaxa, totalMObra, totalDespesa, totalInvest, totalTaxa, totalMarkup, totalSTaxa, totalOrcaBruto, totalImposto, totalOrcamento, totalMarkup);
    };
    /**
     * Retorna o percentual de um servico relacionado 
     * @params{int} cdServRelac - servico que terá o percentual calculado
     * @params{Object} rowServico - row do elemento que terá o percential calculado
    **/
    this.getVrPercentualServico = function(cdServRelac, rowServico){
        var wdgServico = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgCusto   = templateManager.container.getWidget("wdgCustoProposta");
        var totalServico  = 0;
        var totalSrvRelac = 0;
        wdgServico.dataSource.getCheckedRows().filter(function(serv) {
            return serv.TIPO === "C" && !serv.IDARVOREPAI;
        }).forEach(function(elementoRaiz){
            totalServico  += this.getCustoServicoTotal(wdgCusto, wdgServico, elementoRaiz);
        }.bind(this));
        if(cdServRelac)
            totalSrvRelac = this.getVrTotalServico(cdServRelac);
        else
            totalSrvRelac = this.getCustoServicoTotal(wdgCusto, wdgServico, rowServico);
      
        return totalSrvRelac * 100 / totalServico;
    };
    /**
     * Retorna o cálculo do valor do serviço da proposta
     * @param{int} cdServRelac - caso não seja nulo irá retornar o valor do servico para um determinado servico
    **/
    this.getVrTotalServico = function(cdServRelac){
        var totalServico = 0;
        var wdgCusto     = templateManager.container.getWidget("wdgCustoProposta");
        var wdgServico   = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgProposta  = templateManager.container.getWidget("wdgProposta");
        var qtMeses      = wdgProposta.currentRow.QTDIASVALIDADE ? wdgProposta.currentRow.QTDIASVALIDADE/30 : 1;
        var cdcomposicoes = [];

        wdgServico.dataSource.getCheckedRows().filter(function(servico){
            return servico._childCount === 0 && servico.QTMENSAL;    
        }).forEach(function(servicoCusto){
            cdcomposicoes = []; 
            this.getArvoreServico(wdgServico, servicoCusto, [], false).forEach(function(servSelect){
                cdcomposicoes.push(servSelect.NRCOMPSERVICO);
            });
            cdcomposicoes = this.retirarValoresDuplicados(cdcomposicoes);
            wdgCusto.dataSource.data.forEach(function(custo){
                custo.NRCOMPSERVICO.forEach(function(nrcomposicao){
                    if(cdcomposicoes.includes(parseInt(nrcomposicao))){
                        if((!cdServRelac && custo.CDSERVORCARELAC)){
                            totalServico += (parseFloat(servicoCusto.QTMENSAL) * parseFloat(custo.VRCUSTOSERV) * qtMeses); 
                        } else {
                            if(cdServRelac && custo.CDSERVORCARELAC === cdServRelac){
                                totalServico += (parseFloat(custo.VRCUSTOSERV) * qtMeses);
                            }
                        }
                    }
                }.bind(this));
            }.bind(this));
        }.bind(this));
        
        return totalServico;
    };
    /**
     * Retorna o cálculo do valor bruto da proposta
    **/
    this.calcularValorPropSemTaxa = function(){
        var wdgMObra        = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgDespesas     = templateManager.container.getWidget("wdgDespesas");
        var wdgInvestimento = templateManager.container.getWidget("wdgInvestimentosInserindo");
        
        var total = 0;
        total += this.getVrTotalServico(null);
        total += this.getSomatorioDataSource(wdgMObra, "VRTOTAL", false);
        total += this.getSomatorioDataSource(wdgDespesas, "VRTOTDESPPERIODO", true);
        total += this.getSomatorioDataSource(wdgInvestimento, "VRPARCELA", true);
        
        return total;
    };
    /**
     * Retorna o somatório dos valores desejados em uma widget
     * @params{Object} widget - widget que terá os dados somados
     * @params{string} nmCampoVr - nome do campo de valor
     * @params{boolean} checked - se os dados precisam estar selecionados ou não
    **/
    this.getSomatorioDataSource = function(widget, nmCampoVr, checked){
        var total = 0;
        var dataset = !checked ? widget.dataSource.data : widget.dataSource.getCheckedRows(); 
        dataset.forEach(function(data){
            total += this.tirarMascaraValores(data[nmCampoVr]);
        }.bind(this));
        return total;
    };
    /**
     * Retira a mascara de formatação dos valores 
     * e o inicializa com 0 caso seja null
     * @params{string} vrUnitario - valor a ser formatado 
    **/
    this.tirarMascaraValores = function(vrUnitario){
        if (!vrUnitario) {
            vrUnitario = 0;
        } else {
            vrUnitario      = vrUnitario.toString();
            var primeiraPos = vrUnitario.indexOf(".");
            var ultimaPos   = vrUnitario.lastIndexOf(".");
            if (primeiraPos == ultimaPos) {
                if (vrUnitario.includes(",")) 
                    vrUnitario = parseFloat(vrUnitario.replace(".", "").replace(",", "."));
                else 
                    vrUnitario = parseFloat(vrUnitario.replace(",", "."));
            } else {
                var parteInicial = vrUnitario.substr(0, primeiraPos + 1);
                var parteFinal   = vrUnitario.substr(primeiraPos + 1);
                vrUnitario       = parseFloat(parteInicial.replace(".", "") + parteFinal);
            }
        }
        return parseFloat(vrUnitario.toFixed(2));
    };
    /**
     * Retorna a árvore de serviço a partir de um elemento base
     * @params{Object} widget - widget de serviços grid
     * @params{Object} rowElemento - row do elemento base
    **/
    this.getArvoreServico = function(widget, rowElemento, subArvoreServico, subArvore){
        var elementos   = [];
        var idElementos = [];
        subArvoreServico.push(rowElemento);
        if(rowElemento.IDARVOREPAI){
            widget.dataSource.data.forEach(function(servico) {
                if(servico.IDARVORE === rowElemento.IDARVOREPAI)
                    elementos = this.getArvoreServico(widget, servico, subArvoreServico, subArvore);
            }.bind(this));
        } else {
            idElementos.push(rowElemento.IDARVORE);
            elementos.push(rowElemento);
            widget.dataSource.data.forEach(function(servico) {
                if(servico.IDARVOREPAI === rowElemento.IDARVORE || idElementos.includes(servico.IDARVOREPAI)) {
                    elementos.push(servico);
                    idElementos.push(servico.IDARVORE);
                }
            });
        }
        
        return subArvore ? subArvoreServico : elementos;
    };
    /**
     * Retorna a árvore de serviço a partir de um elemento base
     * @params{Object} widget - widget de serviços grid
     * @params{Object} rowElemento - row do elemento base
    **/
    this.getArvoreInvertidaServico = function(widget, rowElemento, subArvoreServico, subArvore){
        var elementos   = [];
        var idElementos = [];
        
        subArvoreServico.push(rowElemento);
        if(rowElemento._childCount > 0){
            widget.dataSource.data.forEach(function(servico) {
                if(servico.IDARVOREPAI === rowElemento.IDARVORE)
                    elementos = this.getArvoreInvertidaServico(widget, servico, subArvoreServico, subArvore);
            }.bind(this));
        } else {
            idElementos.push(rowElemento.IDARVORE);
            elementos.push(rowElemento);
            widget.dataSource.data.forEach(function(servico) {
                if(servico.IDARVORE === rowElemento.IDARVOREPAI || idElementos.includes(servico.IDARVOREPAI)) {
                    elementos.push(servico);
                    idElementos.push(servico.IDARVORE);
                }
            });
        }
        
        return subArvore ? subArvoreServico : elementos;
    };
    /**
     * Retorna quantas vezes o dia da semana passado como parâmetro aparece no período
    **/
    this.getTotalDiaDaSemanaPeriodo = function() {
        var wdgProposta  = templateManager.container.getWidget("wdgProposta");
        var dataInicial  = moment(wdgProposta.currentRow.DTREFORCAMENTO, 'DD/MM/YYYY');
        var dia          = null;
        var totalDias    = {
            "QTSEGUNDA" : 0,
            "QTTERCA"   : 0,
            "QTQUARTA"  : 0,
            "QTQUINTA"  : 0,
            "QTSEXTA"   : 0,
            "QTSABADO"  : 0, 
            "QTDOMINGO" : 0
        };
        
        var diasDaSemana = {
            "Segunda-feira" : "QTSEGUNDA",
            "Terça-feira"   : "QTTERCA",
            "Quarta-feira"  : "QTQUARTA",
            "Quinta-feira"  : "QTQUINTA",
            "Sexta-feira"   : "QTSEXTA",
            "Sábado"        : "QTSABADO", 
            "Domingo"       : "QTDOMINGO"
        }; 
        
        for(var i = 0; i < wdgProposta.currentRow.QTDIASVALIDADE; i++) {
            dia = dataInicial.add(1, 'day').format("dddd");
            totalDias[diasDaSemana[dia]]++;
        }
        
        return totalDias;
    };
    /**
     * Salva a edição da quantidade de elementos 
    **/
    this.salvarEdicaoServico = function(widget){
        var wdgServico  = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgTabbed   = templateManager.container.getWidget("wdgTabbedServico");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");

        var periodoProposta = parseInt(wdgProposta.currentRow.QTDIASVALIDADE/30);
        var rowQMensal      = Utilities.tirarMascaraValores(widget.currentRow.QTMENSAL);
        var rowQSabado      = Utilities.tirarMascaraValores(widget.currentRow.QTSABADO);
        var rowQdomingo     = Utilities.tirarMascaraValores(widget.currentRow.QTDOMINGO);
        
        var totalFimSemana   = 0;
        var totalDiasSemana  = 0;
        var somatorioPeriodo = 0;
        var totalDiaPeriodo  = this.getTotalDiaDaSemanaPeriodo();
        
        totalFimSemana = (rowQdomingo * totalDiaPeriodo["QTDOMINGO"]) + (rowQSabado * totalDiaPeriodo["QTSABADO"]);
        ["QTSEGUNDA", "QTTERCA", "QTQUARTA", "QTQUINTA", "QTSEXTA"].forEach(function(dia){
            somatorioPeriodo = (!widget.currentRow[dia] ? 0 : Utilities.tirarMascaraValores(widget.currentRow[dia]));
            somatorioPeriodo *= totalDiaPeriodo[dia];
            totalDiasSemana += somatorioPeriodo;
        });
        
        if(rowQMensal && rowQMensal > 0 && rowQMensal < (totalFimSemana + totalDiasSemana)) {
            ScreenService.showMessage("Operação Bloqueada!<br>O valor total mensal não pode ser menor que o somatório das quantidades referentes aos dias da semana.<br>Valor total do período: " + (totalDiasSemana + totalFimSemana));
        } else {
            if(!rowQMensal || rowQMensal == 0) rowQMensal = totalDiasSemana + totalFimSemana;
            if((totalDiasSemana + totalFimSemana) == 0) rowQMensal *= periodoProposta; 
            wdgServico.dataSource.data.forEach(function(row) {
                if(row.IDELEMENTO === widget.currentRow.IDELEMENTO){
                    row.QTMENSAL    = rowQMensal;
                    row.QTDIAUTIL   = totalDiasSemana;
                    row.QTFIMSEMANA = totalFimSemana;
                    row.HRENTRADA   = widget.currentRow.HRENTRADA;
                    row.HRSAIDA     = widget.currentRow.HRSAIDA;
                    row.QTSEGUNDA   = widget.currentRow.QTSEGUNDA;
                    row.QTTERCA     = widget.currentRow.QTTERCA;
                    row.QTQUARTA    = widget.currentRow.QTQUARTA;
                    row.QTQUINTA    = widget.currentRow.QTQUINTA;
                    row.QTSEXTA     = widget.currentRow.QTSEXTA;
                    row.QTSABADO    = widget.currentRow.QTSABADO;
                    row.QTDOMINGO   = widget.currentRow.QTDOMINGO;
                }
            });
            
            this.atualizarSomatorioQuantidadeServico(widget);
            widget.currentRow      = {};
            widget.dataSource.data = [];
            widget.label           = "";
            widget.fields.forEach(function(field){
                field.readOnly = true;
                field.value(null);
            });
            
            wdgTabbed.activate();
            templateManager.updateTemplate();
        }
    };
    /**
     * Atualiza o valor da quantidade dos elementos pai no grid se serviços 
     * @params{Object} widget - widget que contém a row do elemento inicial da contagem
    **/
    this.atualizarSomatorioQuantidadeServico = function(widget){
        var wdgServico          = templateManager.container.getWidget("wdgServicosInserindo");
        var idQuantidades       = ["QTMENSAL", "QTDIAUTIL", "QTFIMSEMANA", "QTSEGUNDA", "QTTERCA", "QTQUARTA", "QTQUINTA", "QTSEXTA", "QTSABADO", "QTDOMINGO"];
        var arvoreServInvertida = [];
        var cdServico           = [];

        this.getArvoreServico(wdgServico, widget.currentRow, [], true).forEach(function(servico){
            cdServico.push(servico.IDARVORE);
        });
        wdgServico.dataSource.data.forEach(function(servico) {
            if(cdServico.includes(servico.IDARVORE)) {
                servico.__isSelected = true;
                wdgServico.dataSource.addCheckedRows(servico);
                    
                if(servico._childCount > 0) {
                    idQuantidades.forEach(function(idQuantidade) {
                        servico[idQuantidade] = 0;
                    });
                    
                    arvoreServInvertida = this.getArvoreInvertidaServico(wdgServico, servico, [], true);
                    arvoreServInvertida.forEach(function(servFilho) {
                        if(servFilho._childCount == 0) {
                            idQuantidades.forEach(function(idQuantidade) {
                                servico[idQuantidade] += servFilho[idQuantidade];
                            });
                        }
                    });
                }
            }
        }.bind(this));
    };
    /**
     * Salva a widget de proposta
     * @params{Object} widget - widget proposta form-view
    **/
    this.salvarWidgetProposta = function(widget){
        var deferred = ZHPromise.defer();
        this.criarParametrosNewCode(widget);
        General.callRestEngine("product#/fct_orcamentoaliment_widget/save", "DataSet", widget.dataSource.data).then(function(response){
            deferred.resolve(response);
        }.bind(this))
        .catch(function(error){
            deferred.reject(error);
            Utilities.showErrorMessage(error);
        });
        return deferred.promise;
    };
    /**
     * Clona o dataSource da tela ignorando os campos determinados 
     * @params{Object} widget - widget que será clonada
     * @params{Array} ignoredKeys - array de chaves que devem ser ignoradas 
     * @params{boolean} check - somente dados selecionados ou não
    **/
    this.clonarData = function(widget, ignoredKeys, check){
        var data = [];
        var dataSet = check ? widget.dataSource.getCheckedRows() : widget.dataSource.data;
        dataSet.forEach(function(row, index){
            data[index] = Util.clone(row, ignoredKeys);
        });
        
        return data;
    };
    /**
     * Retorna um arrai de servicos relacionados da proposta
     * @params{Object} wdgItens - widget de resumo do servico
    **/
    this.getArrayPercServicoRelac = function(wdgItens){
        var wdgCusto = templateManager.container.getWidget("wdgCustoProposta");
        var qtMeses  = templateManager.container.getWidget("wdgProposta").currentRow.QTDIASVALIDADE;
        var arrServicoRelac = [];
        var cdservicos      = [];
        wdgCusto.dataSource.data.forEach(function(custo){
            if(custo.CDSERVORCARELAC)
                cdservicos.push(custo.CDSERVORCARELAC);
        }.bind(this));
        
        qtMeses = qtMeses ? qtMeses/30 : 1;
        cdservicos = this.retirarValoresDuplicados(cdservicos);
        cdservicos.forEach(function(serviRelac){
            if(serviRelac){
                var totalPerElemento = 0;
                totalPerElemento += this.getVrPercentualServico(serviRelac)/qtMeses;
                if(totalPerElemento == 0 && wdgCusto.dataSource.data.length > 0)
                    totalPerElemento = 100 / wdgItens.dataSource.data.length;
                arrServicoRelac.push({
                    "CDSERVORCARELAC": serviRelac,
                    "VRPERCSERVICO": totalPerElemento
                });
            }
        }.bind(this));
        
        return arrServicoRelac;
    };
    /**
     * Coloca nos dados do custo as quantidades de cada elemento
    **/
    this.setQuantidadesServicos = function(){
        var wdgServico = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgCusto   = templateManager.container.getWidget("wdgCustoProposta");
        
        wdgCusto.dataSource.data.forEach(function(custo){
            wdgServico.dataSource.getCheckedRows().forEach(function(servico){
                if(custo.NRCOMPSERVICO === servico.NRCOMPSERVICO){
                    custo.QTMENSAL  = servico.QTMENSAL;
                    custo.QTSEGUNDA = servico.QTSEGUNDA;
                    custo.QTTERCA   = servico.QTTERCA;
                    custo.QTQUARTA  = servico.QTQUARTA;
                    custo.QTQUINTA  = servico.QTQUINTA;
                    custo.QTSEXTA   = servico.QTSEXTA;
                    custo.QTSABADO  = servico.QTSABADO;
                    custo.QTDOMINGO = servico.QTDOMINGO;
                }
            });
        });
    };
    /**
     * Salva todos as widgets ligadas a proposta inserida
     * @params{Object} widget - widget custo final
    **/
    this.salvarProposta = function(widget){
        var wdgProposta    = templateManager.container.getWidget("wdgProposta");
        var wdgCusto       = templateManager.container.getWidget("wdgCustoProposta");
        var wdgMObra       = templateManager.container.getWidget("wdgMaoObraGridInserindo");
        var wdgDespesa     = templateManager.container.getWidget("wdgDespesas");
        var wdgInvest      = templateManager.container.getWidget("wdgInvestimentosInserindo");
        var wdgTaxas       = templateManager.container.getWidget("wdgTaxasInserindo");
        var wdgRServico    = templateManager.container.getWidget("wdgResumoServico");
        var wdgGridServ    = templateManager.container.getWidget("wdgServicosInserindo");
        var wdgRFinaceiro  = templateManager.container.getWidget("wdgResumoGrid");
        var wdgImposto     = templateManager.container.getWidget("wdgImpostoProposta");

        var rowProp = Util.clone(wdgProposta.currentRow, []);
        var nrtipoorcamento = wdgProposta.currentRow.NRTIPOORCAMENTO;
        wdgProposta.currentRow.VRORCAMENTO          = wdgRFinaceiro.dataSource.data[0].VRFINANCEIRO;
        wdgProposta.currentRow.VRPERCMARKUPAPLICADO = wdgRFinaceiro.dataSource.data[8].VRPERCFINANCEIRO;
        this.salvarWidgetProposta(wdgProposta).then(function(response){
            var nrseqorcamento = response.dataset.columnsToReturn[0].NRSEQORCAMENTO; 
            var ignoredKeys = ["changed","changed_VRUNITITEMORCA","__index","__isSelected","__leftActions","__notSelectable","__rightActions","__proto__"];
            var params = {
                "NRTIPOORCAMENTO": nrtipoorcamento,
                "NRSEQORCAMENTO" : nrseqorcamento,
                "SERVICOS"       : wdgGridServ.dataSource.getCheckedRows(),
                "CUSTOS"         : wdgCusto.dataSource.data,
                "MOBRAS"         : this.clonarData(wdgMObra, ignoredKeys, false),
                "INVESTIMENTOS"  : wdgInvest.dataSource.getCheckedRows(),
                "DESPESAS"       : wdgDespesa.dataSource.getCheckedRows(),
                "TAXAS"          : wdgTaxas.dataSource.getCheckedRows(),
                "ITENS"          : this.clonarData(wdgRServico, ignoredKeys, false),
                "IMPOSTO"        : this.clonarData(wdgImposto, ignoredKeys, true),
                "SERVICORELAC"   : this.getArrayPercServicoRelac(wdgRServico)
            };
            General.callRestEngine("product#/fct_orcamentoaliment_widget/inserirTudoProposta", "Row", params).then(function(response) {
                wdgProposta.view();
                wdgProposta.template = "widget/form-view.html";
                this.alterarAcessibilidadeAbasProposta(true);
                this.alterarVisibilidadesWidgets();
                ScreenService.showMessage("Proposta salva com sucesso!");
                codeRelacFormacao = null;
            }.bind(this))
            .catch(function(error){
                wdgProposta.currentRow.NRSEQORCAMENTO = nrseqorcamento;
                wdgProposta.removeRow().then(function(){
                    wdgProposta.currentRow = rowProp;
                    Utilities.showErrorMessage(error);
                });
            }.bind(this))
            .finally(function() {
                this.voltarParaPropostaGrid();
            }.bind(this));
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Muda o field de valor informado e a action do grid 
     * para a edição dos valores
    
    this.editarPrecoResumoServico = function(widget, args, element){
        var wdgCustoFinal  = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        widget.getField("VRUNITITEMORCA").inlineEditable = "number";
        widget.getAction("editar_vrservico").icon = "checked no-border";
        widget.getAction("editar_vrservico").events[0].code = "Sm00001000_LancamentoPropostaAlimenticia_Controller.salvarNewPrecoInformado(args.owner.widget);";
        widget.redraw();
        wdgCustoFinal.activate().then(function() {
            setTimeout(function(){
                if(widget.getAction("save") && widget.getAction("cancel")){
                    widget.getAction("save").isVisible   = false;
                    widget.getAction("cancel").isVisible = false;
                }
            }, 200);
        });
    }*/
    /**
     * Guarda a alterção realizada no valor do serviço completo 
     * durante o resumo final da proposta
    **/
    this.salvarNewPrecoInformado = function(widget){
        var wdgRServico   = templateManager.container.getWidget("wdgResumoServico");
        var wdgCustoFinal = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        var vrInformado   = widget.currentRow.VRUNITITEMORCA_GRID;
        
        if(widget.currentRow.VRPRECOIDEAL.toFixed(2) == vrInformado)
            vrInformado = widget.currentRow.VRPRECOIDEAL;
        
        widget.currentRow.VRUNITITEMORCA = vrInformado;
        this.calcularValoresServico_inline(wdgRServico);
        templateManager.updateTemplate();
        wdgCustoFinal.activate();
    };
    /**
     * Valida em qual momento a widget de resultado está sendo alterada
    **/
    this.validarCalculoAlteracaoServico = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        if(wdgProposta.currentRow.NRSEQORCAMENTO){
            this.calcularNewVrOrcamento(widget, wdgProposta);
        } else {
            this.salvarNewPrecoInformado(widget);
        }
    };
    /**
     * Efetua os calculos da widget de resumo do servico 
     * quando o preço informado é alterado 
    **/
    this.calcularValoresServico_inline = function(widget){
        var wdgFinanceiro = templateManager.container.getWidget("wdgResumoGrid");
        var wdgImpostos   = templateManager.container.getWidget("wdgImpostoProposta");
        var wdgTaxas      = templateManager.container.getWidget("wdgTaxasInserindo");
        var totalServico  = (parseFloat(widget.currentRow.VRTOTIDEALSERV) * parseFloat(widget.currentRow.VRUNITITEMORCA))/ parseFloat(widget.currentRow.VRPRECOIDEAL);
        var vrOrcaBruto   = 0;
    
        var totalSemMarkup = this.calcularValorPropSemTaxa();
        totalSemMarkup *= (1 + (this.getSomatorioDataSource(wdgTaxas, "VRPERCTAXA", true)/100));

        widget.currentRow.VRTOTALITEMORCA = totalServico;
        widget.dataSource.data.forEach(function(servico){
            vrOrcaBruto += parseFloat(servico.VRTOTALITEMORCA);
        });
        
        widget.dataSource.data.forEach(function(servico){
            servico.VRPERCFINANCEIRO = (servico.VRTOTALITEMORCA * 100/vrOrcaBruto);
        });
        
        var vrImposto  = vrOrcaBruto * (this.getSomatorioDataSource(wdgImpostos, "VRPRECIMPOSTO", true)/100);
        var vrNewTotal = vrOrcaBruto - vrImposto;
        wdgFinanceiro.dataSource.data[0].VRFINANCEIRO = vrOrcaBruto;
        wdgFinanceiro.dataSource.data[1].VRFINANCEIRO = vrImposto;
        wdgFinanceiro.dataSource.data[2].VRFINANCEIRO = vrNewTotal;
        wdgFinanceiro.dataSource.data[8].VRFINANCEIRO = totalSemMarkup * ((vrOrcaBruto/totalSemMarkup) - 1);
        wdgFinanceiro.dataSource.data[8].VRPERCFINANCEIRO = ((vrOrcaBruto/totalSemMarkup) - 1) * 100;
        widget.redraw();
        wdgFinanceiro.redraw();
    };
    /**
     * Avança para a widget de Imposto
     * @params{Object} widget - widget servico proposta inserindo 
    **/
    this.avancarImposto = function(widget){
        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(widget.dataSource.data).then(function(mensagemErro) {
            if (mensagemErro && mensagemErro !== "Operação bloqueada.<br/><br/>Nenhum elemento foi selecionado.") {
                ScreenService.showMessage(mensagemErro);
            } else {
                var dataServico = templateManager.container.getWidget("wdgServicosInserindo").dataSource.data;
                widget.enabled  = false;
                Sm00001000_PropostaAlimenticia_Repository.saveDataLocalStorageEngine("wdgServicosInserindo", dataServico);
                var wdgImposto = templateManager.container.getWidget("wdgImpostoProposta");
                wdgImposto.enabled   = true;
                wdgImposto.isVisible = true;
                wdgImposto.mode = "edit";
                wdgImposto.activate().then(function() {
                    this.setFiltroImpostos(wdgImposto);
                }.bind(this));
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
     /**
     * Aplica um filtro na widget de imposto durante a inserção da proposta
     * @params{Object} widget - widget imposto inserindo
    **/
    this.setFiltroImpostos = function(widget){
        var wdgServico  = templateManager.container.getWidget("wdgServicosInserindo");
        var composicoes = [];
        wdgServico.dataSource.getCheckedRows().forEach(function(serv) {
            composicoes.push(serv.NRCOMPSERVICO);
        });
        composicoes = this.retirarValoresDuplicados(composicoes);
        widget.dataSourceFilter[0].value = composicoes;
        widget.reload().then(function(){
            widget.getAction("save").isVisible   = false;
            widget.getAction("cancel").isVisible = false;
            widget.dataSource.selectAll();
        });
    };
    /**
     * Volta para a widget de imposto durante a inserção da proposta
     * @params{Object} widget - widget custo inserindo
    **/
    this.voltarImpostosProposta = function(widget){
        widget.enabled = false;
        var wdgImposto = templateManager.container.getWidget("wdgImpostoProposta");
        wdgImposto.enabled = true;
        wdgImposto.mode    = "edit";
        wdgImposto.activate();
    };
    /**
     * Abre um popup para a seleção de um novo imposto
     * @params{Object} widget - widget impostos inserindo
    **/
    this.adicionarImpostos = function(widget){
        var wdgPopup = templateManager.container.getWidget("wdgPopupImpostos");
        var cdimpostos = [];
        widget.dataSource.data.filter(function(imposto){
            if(imposto.CDIMPOSTO)
                cdimpostos.push(imposto.CDIMPOSTO);
        });
        
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate().then(function(){
                wdgPopup.getField("CDIMPOSTO").dataSourceFilter[0].value = cdimpostos.length > 0 ? cdimpostos : [-1];
                wdgPopup.getField("CDIMPOSTO").reload();
            });
            
            if(widget.name === "wdgItensImpostoProposta"){
                wdgPopup.getAction("save").events[0].code   = "Sm00001000_LancamentoPropostaAlimenticia_Controller.salvarNewImpostoItemOrca(args.owner.widget);";
                wdgPopup.getAction("cancel").events[0].code = "Sm00001000_LancamentoPropostaAlimenticia_Controller.cancelarNewImpostoItemOrca();";
            } else { 
                wdgPopup.getAction("save").events[0].code   = "Sm00001000_LancamentoPropostaAlimenticia_Controller.salvarNovoImposto(args.owner.widget);";
                wdgPopup.getAction("cancel").events[0].code = "Sm00001000_LancamentoPropostaAlimenticia_Controller.cancelarInsercaoImposto(args.owner.widget);";
            }
            
        }.bind(this));
    };
    /**
     * Cancela a inserção de um novo imposto durante a inserção da proposta
     * @params{Object} widget - widget popup de inserção do imposto
    **/
    this.cancelarInsercaoImposto = function(widget){
        ScreenService.closePopup();
        var wdgImposto = templateManager.container.getWidget("wdgImpostoProposta");
        wdgImposto.enabled = true;
        wdgImposto.mode    = "edit";
        wdgImposto.activate().then(function(){
            setTimeout(function() {
                wdgImposto.getAction("save").isVisible   = false;
                wdgImposto.getAction("cancel").isVisible = false;
            }, 300);
        });
    };
    /**
     * Adiciona ao dataSource da tela de impostos um novo imposto
     * durante a inserção da proposta
     * @param{Object} widget - widget popup impostos
    **/
    this.salvarNovoImposto = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgImpostoPro = templateManager.container.getWidget("wdgImpostoProposta");
            widget.currentRow.CDIMPOSTO.forEach(function(imposto){
                var nmimposto;
                widget.getField("CDIMPOSTO").dataSource.data.forEach(function(valueImposto){
                    if(valueImposto.CDIMPOSTO === imposto)
                        nmimposto = valueImposto.NMIMPOSTO;
                });
                var row = {
                    "NMIMPOSTO": nmimposto,
                    "CDIMPOSTO": imposto,
                    "VRPRECIMPOSTO": widget.currentRow.VRPRECIMPOSTO ? widget.currentRow.VRPRECIMPOSTO : "0",
                    "NRCOMPSERVICO": "",
                    "__is_new": false,
                    "__isSelected": true
                };
                wdgImpostoPro.dataSource.data.push(row);
                wdgImpostoPro.dataSource.addCheckedRows(row);
            });
            
            ScreenService.closePopup();
            wdgImpostoPro.mode = "edit";
            wdgImpostoPro.activate().then(function(){
                setTimeout(function() {
                    wdgImpostoPro.getAction("save").isVisible   = false;
                    wdgImpostoPro.getAction("cancel").isVisible = false;
                }, 300);
            });
        } 
    };
    /**
     * Abre um popup para adicionar um novo investimento
     * @params{Object} widget - widget investimento inserindo
    **/
    this.adicionarNovoInvestimento = function(widget){
        var wdgPopup = templateManager.container.getWidget("wdgPopupInvestimento");
        var investimentos = [];
        widget.dataSource.data.filter(function(invest){
            if(invest.NRINVESTIMENTO)
                investimentos.push(invest.NRINVESTIMENTO);
        });
        
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate().then(function(){
                wdgPopup.getField("NRINVESTIMENTO").dataSourceFilter[0].value = investimentos.length > 0 ? investimentos : ["-1"];
                wdgPopup.getField("NRINVESTIMENTO").reload();
            });
        }.bind(this));
    };
    /**
     * Cancela a inserção de um novo investimento durante a inserção da proposta
     * @params{Object} widget - widget popup de inserção do investimento
    **/
    this.cancelarInsercaoInvestimento = function(widget){
        ScreenService.closePopup();
        var wdgIvest = templateManager.container.getWidget("wdgInvestimentosInserindo");
        wdgIvest.enabled = true;
        wdgIvest.mode    = "edit";
        wdgIvest.activate().then(function(){
            setTimeout(function() {
                wdgIvest.getAction("save").isVisible   = false;
                wdgIvest.getAction("cancel").isVisible = false;
            }, 300);
        });
    };
    /**
     * Adiciona ao dataSource da tela de impostos um novo imposto
     * durante a inserção da proposta
     * @param{Object} widget - widget popup impostos
    **/
    this.salvarNovoInvestimento = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgInvest = templateManager.container.getWidget("wdgInvestimentosInserindo");
            widget.currentRow.NRINVESTIMENTO.forEach(function(investimento){
                widget.getField("NRINVESTIMENTO").dataSource.data.forEach(function(valueIvest){
                    if(valueIvest.NRINVESTIMENTO === investimento){
                        var row = {
                            "DSINVESTIMENTO"    : valueIvest.DSINVESTIMENTO,
                            "NRINVESTIMENTO"    : investimento,
                            "VRINVESTIMENTO"    : valueIvest.VRINVESTIMENTO   ? valueIvest.VRINVESTIMENTO : "0",
                            "QTMESARMOTIZACAO"  : valueIvest.QTMESARMOTIZACAO ? valueIvest.QTMESARMOTIZACAO : "0",
                            "TPAMORTIZACAO"     : valueIvest.TPAMORTIZACAO,
                            "NMCATINVEST"       : valueIvest.NMCATINVEST,
                            "VRTOTINVESTIMENTO" : valueIvest.VRINVESTIMENTO ? valueIvest.VRINVESTIMENTO : "0",
                            "VRPARCELA"         : valueIvest.VRINVESTIMENTO/(valueIvest.QTMESARMOTIZACAO ? valueIvest.QTMESARMOTIZACAO : 1),
                            "QTINVESTIMENTO"    : "1",
                            "NRCOMPSERVICO"     : "",
                            "NRFORMPRECOSERVICO": "",
                            "__is_new"          : false,
                            "__isSelected"      : true
                        };
                        wdgInvest.dataSource.data.push(row);
                        wdgInvest.dataSource.addCheckedRows(row);
                    }
                });
            });
            this.cancelarInsercaoInvestimento(widget);
        } 
    };
    /**
     * Efetua o cálculo do valor total da formação de preço 
     * no grid de mao de obra
     * @params{Object} widget - widget mão de obra inserindo
    **/ 
    this.calcularValorTotalMObra = function(widget){
        var qtMObra = (widget.currentRow.QTFORMACAO_inline) ? widget.currentRow.QTFORMACAO_inline : widget.currentRow.QTFORMACAO;
        widget.currentRow.VRTOTAL = parseFloat(qtMObra) * parseFloat(widget.currentRow.VRUNITARIO);
    };
    /**
     * Abre o swipe de visualização das informções dos itens da proposta
    **/
    this.abrirSwipeItensProposta = function(widget){
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipalItens");
        var wdgItens           = templateManager.container.getWidget("wdgItensProposta");
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            setLocalVar("PROPOSTA_NRITEMORCAMENTO", widget.currentRow.NRITEMORCAMENTO);
            wdgItens.list();
            wdgItens.reload().then(function() {
                wdgItens.view();
                wdgItens.activate();
            });
        });
    };
    /**
     * Cancela a edição da quantidade dos itens no campo de inline edit
     * @params{Object} widget - widget itens grid
    **/
    this.cancelarItens = function(widget) {
        widget.list();
        widget.reload();
    };
    /**
     * Prepara a widget para a inserção de um novo item (serviço)
     * @params{Object} widget - widget de serviço
    **/
    this.itensNovoRegistro = function(widget){
        General.widgetNewRow(widget);
        this.controlarCampoValorDoAjuste(widget);
    };
    /**
     * Sintetiza as funções em uma mesma função
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.funcoesDoCampoAjuste = function(widget) {
        this.controlarCampoValorDoAjuste(widget);
        this.calcularValorTotalItem(widget);
        if (widget.currentRow.VRPERCDESCORCA) {
            this.adequaAjusteAoCampo(widget);
        }
    };
    /**
     * Previni possiveis erros com a troca sequida no select native do campo Ajuste
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.adequaAjusteAoCampo = function(widget) {
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
        var ajuste            = widget.currentRow.VRPERCDESCORCA;

        var valorSemajuste = quantidade * valorUnitario;
        var valorTotal = valorSemajuste;
        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
        if (tipoAjuste == "P" && ajuste > 100) {
            widget.currentRow.VRPERCDESCORCA = null;
        }
        if (tipoAjuste == "V" && ajuste > valorTotal) {
            widget.currentRow.VRPERCDESCORCA = null;
        }
    };
    /**
     * Controla o campo de desconto de acordo com o tipo.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.controlarCampoValorDoAjuste = function(widget) {
        var tipoAjuste = widget.currentRow.IDTPDESCORCA;
        var fieldAjuste = widget.getField("VRPERCDESCORCA");
        widget.getField('IDTPDESCORCA').readOnly = false;
        if (tipoAjuste == "N") {
            fieldAjuste.readOnly = true;
            fieldAjuste.value("");
        } else {
            fieldAjuste.readOnly = false;
        }

        fieldAjuste.readOnly = (tipoAjuste == "N") ? true : false;
    };
        /**
     * Calcula o valor total do item da proposta.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.calcularValorTotalItem = function(widget) {
        if (widget.mode != "list" && widget.mode != "inline_edit") {
            var quantidade    = widget.currentRow.QTITEMORCAMENTO;
            var valorUnitario = widget.currentRow.VRUNITITEMORCA;
            /*var tipoAjuste = widget.currentRow.IDTPDESCORCA;
            var ajuste = widget.currentRow.VRPERCDESCORCA;*/
            var qtMedVariavel = widget.currentRow.QTMEDIDAVARIAVEL ? parseFloat(widget.currentRow.QTMEDIDAVARIAVEL.toString().replace(",", ".")) : 1;
    
            if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
            if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
            //if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
            
            var validacao = true;
            if (validacao) {
                var valorSemAjuste = quantidade * valorUnitario ;

                if (widget.currentRow.IDMULTIPVARIACAO == "S") 
                    valorSemAjuste *= qtMedVariavel ;
        
                var valorTotal = valorSemAjuste;
                /*if(!widget.currentRow.IDTIPO || widget.currentRow.IDTIPO !== "I"){
                    if (tipoAjuste == "V" && ajuste <= valorTotal && ajuste >= 0) {
                        valorTotal -= ajuste;
                    }
                    else if (tipoAjuste == "P" && ajuste <= 100 && ajuste >= 0) {
                        valorTotal -= valorTotal * ajuste / 100;
                    }
                    else if (tipoAjuste == "X" && ajuste >= 0) {
                        valorTotal += ajuste;
                    }
                    else if (tipoAjuste == "Y" && ajuste >= 0) {
                        valorTotal += valorTotal * ajuste / 100;
                    }
                }*/
        
                //widget.currentRow.VRITEMSEMDESC = valorSemAjuste;
                widget.currentRow.VRTOTALITEMORCA = valorTotal;
                if(widget.currentRow.TIPO === "M")
                    widget.currentRow.VRCUSTOALIMENTAR = quantidade * valorUnitario;
            }
        }
    };
    /**
     * Abre a widget de itens de formação de preço da proposta 
     * @params{widget} widget - widget mão de obra (tela itens)
    **/
    this.exibirItensFormPrecProposta = function(widget){
        if(!onClickMobraItens || onClickMobraItens !== (widget.currentRow.NRFORMPRECOSERVICO + "_" + widget.currentRow.NRRELACFORMACAOPROP)){
            onClickMobraItens = widget.currentRow.NRFORMPRECOSERVICO + "_" + widget.currentRow.NRRELACFORMACAOPROP;
        } else {
            var wdgTabbed     = templateManager.container.getWidget("wdgTabbedItensMObraForm");
            var wdgItensForm  = templateManager.container.getWidget("wdgItensFormacaoPrecoProposta");
            wdgItensForm.isVisible = true;
            wdgItensForm.enabled   = true;
            ScreenService.openSwipe(wdgTabbed).then(function() {
                wdgItensForm.list();
                wdgItensForm.activate().then(function(){
                    wdgItensForm.dataSourceFilter[2].value = widget.currentRow.NRFORMPRECOSERVICO;
                    wdgItensForm.dataSourceFilter[4].value = widget.currentRow.NRRELACFORMACAOPROP;
                    Utilities.abreArvoreElementos(wdgItensForm);
                });
                onClickMobraItens = null;
            });
        }
    };
    /**
     * Verifica se a widget de resultado está sendo acessada para
     * visualização de dados já inseridos e manda buscar as informações
     * @params{Object} widget - widget Resultado dashboard
    **/
    this.buscarInformacoesResultadoProp = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        if(widget.mode === "list" && !wdgProposta.currentRow.__is_new){
            var wdgServico    = templateManager.container.getWidget("wdgResumoServico");
            var wdgFinanceiro = templateManager.container.getWidget("wdgResumoGrid");
            var params        = {"NRSEQORCAMENTO": getLocalVar("PROPOSTA_NRSEQORCAMENTO")};
            General.callRestEngine("product#/fct_orcamentoaliment_widget/getInfoResultado", "Row", params).then(function(response){
                if(response.dataset && response.dataset.getInfoResultado){
                    wdgServico.dataSource.data = response.dataset.getInfoResultado.SERVICOS;
                    var vrFin = response.dataset.getInfoResultado.FINANCEIRO;
                    this.setValoresWdgRFinanceiro(wdgFinanceiro, vrFin["TOTALBRUTO"], vrFin["IMPOSTO"], vrFin["TOTALLIQ"], vrFin["CALIMENTAR"], vrFin["TOTALCTAXA"], vrFin["MOBRA"], vrFin["DESPESAS"], vrFin["INVESTIMENTOS"], vrFin["TAXAS"], vrFin["MARKUP"], vrFin["TOTALSTAXA"], vrFin["TOTALIDEAL"], vrFin["IMPOSTOIDEAL"], vrFin["TOTALLIQIDEAL"], vrFin["MARKUPIDEAL"]);
                }
                wdgServico.redraw();
                wdgFinanceiro.redraw();
            }.bind(this));
        }
    };
    /**
     * Adiciona na lista fixa da widget de resultados os valores 
     * de cada informação da proposta 
     * @params{Object} widget - widget resumo Financeiro
     * @params{float} (demais parametrôs) - totais de cada elemento
    **/
    this.setValoresWdgRFinanceiro = function(widget, totalOrcaBruto, totalImposto, totalOrcamento, totalServico, totalCmTaxa, totalMObra, totalDespesa, totalInvest, totalTaxa, totalMarkup, totalSTaxa, totalOrcaIdeal, totalImpostoIdeal, totalOrcamentoIdeal, totalMarkupIdeal){
        widget.dataSource.data[0].VRFINIDEAL       = totalOrcaIdeal;
        widget.dataSource.data[0].VRFINANCEIRO     = totalOrcaBruto;
        widget.dataSource.data[0].VRPERCFINIDEAL   = (totalOrcaIdeal * 100/totalOrcaIdeal);
        widget.dataSource.data[0].VRPERCFINANCEIRO = (totalOrcaBruto * 100/totalOrcaBruto);
        
        widget.dataSource.data[1].VRFINANCEIRO     = totalImposto;
        widget.dataSource.data[1].VRFINIDEAL       = totalImpostoIdeal;
        widget.dataSource.data[1].VRPERCFINANCEIRO = (totalImposto * 100/totalOrcaBruto);
        widget.dataSource.data[1].VRPERCFINIDEAL   = (totalImpostoIdeal * 100/totalOrcaIdeal);
        
        widget.dataSource.data[2].VRFINANCEIRO     = totalOrcamento;
        widget.dataSource.data[2].VRFINIDEAL       = totalOrcamentoIdeal;
        widget.dataSource.data[2].VRPERCFINANCEIRO = (totalOrcamento * 100/totalOrcaBruto);
        widget.dataSource.data[2].VRPERCFINIDEAL   = (totalOrcamentoIdeal * 100/totalOrcaIdeal);
        
        widget.dataSource.data[3].VRFINANCEIRO     = totalServico;
        widget.dataSource.data[3].VRFINIDEAL       = totalServico;
        widget.dataSource.data[3].VRPERCFINANCEIRO = (totalServico * 100/totalCmTaxa);
        widget.dataSource.data[3].VRPERCFINIDEAL   = (totalServico * 100/totalCmTaxa);
        
        widget.dataSource.data[4].VRFINANCEIRO     = totalMObra;
        widget.dataSource.data[4].VRFINIDEAL       = totalMObra;
        widget.dataSource.data[4].VRPERCFINANCEIRO = (totalMObra * 100/totalCmTaxa);
        widget.dataSource.data[4].VRPERCFINIDEAL   = (totalMObra * 100/totalCmTaxa);
        
        widget.dataSource.data[5].VRFINANCEIRO     = totalDespesa;
        widget.dataSource.data[5].VRFINIDEAL       = totalDespesa;
        widget.dataSource.data[5].VRPERCFINANCEIRO = (totalDespesa * 100/totalCmTaxa);
        widget.dataSource.data[5].VRPERCFINIDEAL   = (totalDespesa * 100/totalCmTaxa);
        
        widget.dataSource.data[6].VRFINANCEIRO     = totalInvest;
        widget.dataSource.data[6].VRFINIDEAL       = totalInvest;
        widget.dataSource.data[6].VRPERCFINANCEIRO = (totalInvest * 100/totalCmTaxa);
        widget.dataSource.data[6].VRPERCFINIDEAL   = (totalInvest * 100/totalCmTaxa);
        
        widget.dataSource.data[7].VRFINANCEIRO     = totalTaxa;
        widget.dataSource.data[7].VRFINIDEAL       = totalTaxa;
        widget.dataSource.data[7].VRPERCFINANCEIRO = (totalTaxa * 100/totalSTaxa);
        widget.dataSource.data[7].VRPERCFINIDEAL   = (totalTaxa * 100/totalSTaxa);
        
        widget.dataSource.data[8].VRFINANCEIRO     = totalMarkup;
        widget.dataSource.data[8].VRFINIDEAL       = totalMarkupIdeal;
        widget.dataSource.data[8].VRPERCFINANCEIRO = (totalMarkup * 100/ totalCmTaxa);
        widget.dataSource.data[8].VRPERCFINIDEAL   = (totalMarkupIdeal * 100/ totalCmTaxa);
    };
    /**
     * Volta a widget de proposta para o modo de visualização
     * @params{Object} widget - widget de proposta
    **/
    this.voltarParaPropostaView = function(widget){
        var wdgPropostaTabbed     = templateManager.container.getWidget("wdgPropostaTabbed");
        var wdgTabbedPrincipal    = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgPropostaTabbed.enabled = true;
        this.alterarAcessibilidadeAbasProposta(true);
        
        widget.template = "widget/form-view.html";
        widget.currentRow.changed = false;
        widget.getAction("save").isVisible       = false;
        widget.getAction("avancar").isVisible    = true;
        widget.getField("VRPERCMARKUP").readOnly = false;
        
        wdgPropostaTabbed.mode  = "view";
        wdgTabbedPrincipal.mode = "view";
        widget.view();
    };
    /**
     * Valida a edição da proposta
     * @params{Object} widget - widget proposta view
    **/
    this.validarEdicaoProposta = function(widget){
        this.salvarWidgetProposta(widget).then(function(){
            this.voltarParaPropostaView(widget);
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /* Aplica o filtro de situação da proposta na tela de escopo 
    * para aparecer apenas o documento da situação atual
    * @params{Object} widget escopo proposta
    */
    this.setFilterSituacaoEscopo = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var edititem    = templateManager.container.getWidget("wdgEscoposProposta").getAction('edit');
        widget.dataSourceFilter[2].value = wdgProposta.currentRow.IDSITUACAOORCAMENTO;
        widget.reload();

        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO == "CONCLUIDO")
            edititem.isVisible = true;
    };
    /**
     * Salva os escopos da proposta.
     * 
     * @param {widget} widget - Widget de Escopos da Proposta
     */
    this.salvarEscopoProposta = function(widget) {
        widget.currentRow.__columns_to_return = ["NRRELACESCOPOPROPOSTA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        General.widgetSave(widget).then(function(success) {
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                var columnsToReturn = success.dataset.columnsToReturn[0];
                widget.currentRow.NRRELACESCOPOPROPOSTA = columnsToReturn.NRRELACESCOPOPROPOSTA;
                widget.currentRow.NRORG                 = columnsToReturn.NRORG;
                widget.currentRow.DTINCLUSAO            = columnsToReturn.DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO        = columnsToReturn.CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO         = columnsToReturn.NRORGINCLUSAO;
            }
        }).catch(function(error) {
            ScreenService.showMessage(error);
        });
    };
    /**
     * Conclui a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.concluirProposta = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente concluir a proposta?"), 'question', function(){
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            var nrorg          = wdgProposta.currentRow.NRORG;
            
            wdgProposta.currentRow.changed = true;
            wdgProposta.currentRow.IDSITUACAOORCAMENTO = "CONCLUIDO";
            Sm00001000_LancamentoProposta_Service.validarItensPropostaParaConclusao(nrseqorcamento, nrorg).then(function() {
                General.callRestEngine("product#/fct_orcamentoaliment_widget/save", "DataSet", wdgProposta.dataSource.data).then(function() {
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    wdgProposta.currentRow.changed = false;
                    ScreenService.closePopup();
                    var  params = {
                        "NRSEQORCAMENTO"     : wdgProposta.currentRow.NRSEQORCAMENTO,
                        "IDSITUACAOORCAMENTO": "CONCLUIDO"
                    };
                    
                    var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                    wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "CONCLUIDO";
                    this.validacoesGeraisProposta(wdgPropostaGrid);
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }.bind(this),
            function(error) {
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    /**
     * Aceita a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.aceitarProposta = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente aceitar a proposta?"), 'question', function(){
            var params = {
                "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO,
                "NRORG"         : widget.currentRow.NRORG,
                "IDUTILIZACOMP" : widget.currentRow.IDUTILIZACOMP,
                "NRPARCNEGOCIO" : widget.currentRow.NRPARCNEGOCIO
            };
    
            General.callRestEngine("product#/fct_orcamentoaliment_widget/aceitarProposta", "Row", params).then(function() {
                    ScreenService.showMessage("Processo realizado com sucesso!");
                    ScreenService.closePopup();
                    widget.reload().then(function() {
                        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                        wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "ACEITO";
                        wdgPropostaGrid.currentRow.DTACEITEORCA = widget.currentRow.DTACEITEORCA;
                        this.validacoesGeraisProposta(wdgPropostaGrid);
                    }.bind(this));
                }.bind(this),
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }.bind(this));
    };
    /**
     * Copia ou versiona a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param {string} tipo - 'C' para Copiar, 'V' para Versionar
     */
    this.copiarVersionarProposta = function(widget, tipo) {
        var row = widget.currentRow;
        var params = {
            "IDSITUACAOORCAMENTO": widget.currentRow.IDSITUACAOORCAMENTO,
            "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
            "NRORG"              : widget.currentRow.NRORG,
            "TIPO"               : tipo
        };

        General.callRestEngine("product#/fct_orcamentoaliment_widget/copiarVersionarProposta", "Row", params).then(function(response) {
            if(!widget.currentRow.NRCONTATO){
                row.NRSEQORCAMENTO = response.dataset.NRSEORCGERADO[0];
                //this.geraContatoProposta(row);
            }
            
            ScreenService.showMessage("Processo realizado com sucesso!");
            ScreenService.closePopup();
            this.voltarParaPropostaGrid();
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    /**
     * Ação que cancela a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Prposta
     */
    this.cancelarProposta = function(wdgProposta) {
        var params = {
            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
            "NRORG": wdgProposta.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamentoaliment_widget/cancelarProposta", "Row", params).then(function(success) {
            ScreenService.showMessage("Processo realizado com sucesso!");
            ScreenService.closePopup();
            wdgProposta.currentRow.IDSITUACAOORCAMENTO = "CANCELADO";

            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "CANCELADO";
            this.validacoesGeraisProposta(wdgPropostaGrid);
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    /**
     * Verifica o tipo do item para habilitar as ações de edição e exclusão
     * @params{Object} widget - widget de itens 
    **/
    this.validarItens = function(widget){
        var wdProposta = templateManager.container.getWidget("wdgProposta");
        if(["CONCLUIDO", "ELABORACAO"].includes(wdProposta.currentRow.IDSITUACAOORCAMENTO)){
            var tpItens = (widget.currentRow.IDTIPO  && widget.currentRow.IDTIPO !== "I");
            widget.fields.forEach(function(field){
                if(field.fieldGroup === "fdgDadosAdicionais")
                    field.isVisible = !tpItens;
                    
                if(field.fieldGroup === "fdgDadosServico")
                    field.isVisible = tpItens;
            });
            
            widget.getField("VRUNITITEMORCA").isVisible  = tpItens;
            widget.getField("VRUNITITEMORCA").readOnly   = !tpItens;
            widget.getField("QTITEMORCAMENTO").readOnly  = !tpItens;
            widget.getField("NMTIPOORCAMENTO").isVisible = !tpItens;
        }
    };
    /**
     * Coloca a tela de Itens ou de Adicionais em modo de edição.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.prepararEdicaoItens = function(widget) {
        General.widgetEdit(widget);
        this.controlarCampoValorDoAjuste(widget);
        this.validarItens(widget);
    };
    /**
     * Apaga um item do orcamento
     * @params{Object} widget - widget itens 
     */
    this.excluirItem = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            General.callRestEngine("product#/fct_itemorcamentoaliment_widget/deletarItemOrcamento", "Row", widget.currentRow).then(function(){
                ScreenService.closeSwipe().then(function () {
                    widget.activate();
                    widget.reload();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    /**
     * Converte o valor de ajuste para valor percentual
     * 
     * @param {object} widget widget de 
     * Item de Proposta para salvar o novo registro.
     */
    this.converteParaPercentual = function(widget) {
        var percentual        = widget.currentRow.VRPERCDESCORCA;
        var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        var valorSemAjuste    = quantidade * valorUnitario;
        var valorTotal        = valorSemAjuste;
        if (tipoAjuste == "V" || tipoAjuste == "X")
            return (percentual / valorTotal) * 100;
        else
            return percentual;
    };
    /**
     * Atualiza o ajuste dos elementos da arvore de composição de serviço
     * @params{Object} widget widget itens
    **/
    this.salvarNovasQuantidadesElemento = function(widget){
        var paramsElemento = {
            "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO,
            "CDSERVORCA"     : widget.currentRow.CDSERVORCA,
            "IDELEMENTO"     : widget.currentRow.IDELEMENTO,
            "VRCUSTOSERV"    : this.tirarMascaraValores(widget.currentRow.VRUNITITEMORCA),
            "QTSEGUNDA"      : widget.currentRow.QTSEGUNDA,
            "QTTERCA"        : widget.currentRow.QTTERCA,
            "QTQUARTA"       : widget.currentRow.QTQUARTA,
            "QTQUINTA"       : widget.currentRow.QTQUINTA,
            "QTSEXTA"        : widget.currentRow.QTSEXTA,
            "QTSABADO"       : widget.currentRow.QTSABADO,
            "QTDOMINGO"      : widget.currentRow.QTDOMINGO,
            "QTMENSAL"       : widget.currentRow.QTITEMORCAMENTO,
            "HRENTRADA"      : widget.currentRow.HRENTRADA,
            "HRSAIDA"        : widget.currentRow.HRSAIDA
        };
        General.callRestEngine("product#/fct_itemorcamentoaliment_widget/salvarNovasQuantidadesElemento", "Row", paramsElemento)
        .then(function() {
            this.atualizarValorItem();
        }.bind(this)).catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Salva um item ou um adicional.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.salvarItens = function(widget){
        var deferred = ZHPromise.defer();  
        var isNew    = widget.currentRow.__is_new;
        var tipo     = widget.currentRow.IDTIPO;
        
        widget.currentRow.__columns_to_return = ["NRITEMORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        if(widget.currentRow.NMDOCITEMORCAMENTO && widget.currentRow.NMDOCITEMORCAMENTO !== widget.currentRow.NMITEMORCAMENTO){
            widget.currentRow.NMITEMORCAMENTO = widget.currentRow.NMDOCITEMORCAMENTO;
        }
        
        if(!widget.currentRow.VRUNITITEMORCA && tipo == 'I')
            widget.currentRow.VRUNITITEMORCA = widget.currentRow.VRUNITITEMORCA;
            
        General.callRestEngine("product#/fct_itemorcamentoaliment_widget/save", "DataSet", [widget.currentRow]).then(function(success) {
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRITEMORCAMENTO = success.dataset.columnsToReturn[0].NRITEMORCAMENTO;
                widget.currentRow.NRORG           = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO      = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO  = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            if(tipo == 'M')
                this.salvarNovasQuantidadesElemento(widget);
            else 
                this.atualizarValorItem();
            if(isNew){
                ScreenService.closeSwipe().then(function(){
                    widget.list();
                    widget.reload();
                });
            } else {
                widget.view();
            }
            
            deferred.resolve();
        }.bind(this)).catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject();
        });
        
        return deferred.promise;
    };
    /**
     * Realiza a s validaçoes antes de salvar um registro
     * 
     * @param {object} widget widget de 
     * Item de Proposta para salvar o novo registro.
     */
    this.validaPercentualSaveItens = function(widget) {
        var tipoAjuste = widget.currentRow.IDTPDESCORCA;
        var percentual = this.converteParaPercentual(widget);
        var contexto   = "";
        var params;

        if (widget.currentRow.TIPOPRODSERV == 'S')
            widget.currentRow.CDSERVORCA = widget.currentRow.CDPRODSERV;
        else 
            widget.currentRow.CDPRODUTO = widget.currentRow.CDPRODSERV;
            
        if (!widget.currentRow.CDSERVORCA || widget.currentRow.IDTIPO !== "M") {
            this.salvarItens(widget, true);
        } else {
            if (tipoAjuste === "N"){
                this.salvarItens(widget);
            } else {
                params = {
                    "CDSERVORCA": widget.currentRow.CDSERVORCA,
                    "IDTIPO"    : (tipoAjuste == "P" || tipoAjuste == "V") ? "D" : "A"
                };
                
                General.callRestEngine("product#/fct_itemorcamento_widget/validaPorcentualDescontoAcrescimoSave", "Row", params).then(function(response) {
                    var resultado = response.dataset.RESULTADO.VRPERCDESC;
                    if (resultado) {
                        if (percentual <= resultado.VRPERCDESC) {
                            this.salvarItens(widget);
                        } else {
                            if(resultado.TIPO == 1)
                               contexto = "Este";
                            else
                                contexto = "O perfil de acesso deste";
                            var tipo = (tipoAjuste == "P" || tipoAjuste == "V") ? "oferecer desconto" : "propor acréscimo";
                            ScreenService.showMessage(contexto+" operador nao têm autorização para "+ tipo +" superior a " + resultado.VRPERCDESC + "% do valor sem ajuste");
                        }
                    } else {
                        this.salvarItens(widget);
                    }
                }.bind(this)).catch(function(error) {
                    console.error(error);
                    if (error.data)
                        ScreenService.showMessage(error.data.error);
                    else
                        ScreenService.showMessage(error);
                });
            }
        }
    };
    /**
     * Atualiza o valor total da proposta de acordo com a soma dos itens.
     * 
     * @param {object} wdgPropostaGrid - Widget de Proposta (grid)
     */
    this.atualizarValorProposta = function(wdgPropostaGrid) {
        var params = {
            "NRSEQORCAMENTO": wdgPropostaGrid.currentRow.NRSEQORCAMENTO,
            "NRORG": wdgPropostaGrid.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamentoaliment_widget/atualizarValorProposta", "Row", params).then(function(success) {
            var valorProposta = success.dataset.FCT_ORCAMENTO.VRORCAMENTO;
            var wdgProposta = templateManager.container.getWidget("wdgProposta");

            wdgPropostaGrid.currentRow.VRORCAMENTO = valorProposta;
            wdgProposta.currentRow.VRORCAMENTO = valorProposta;
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    /**
     * Valida se o valor semanal do item é menor que o valor mensal
     * @params{Object} widget - widget de itens
    **/
    this.validarQuantidadesItens = function(widget, isMensal){
        var totalQuantidade = 0;
        var totalPeriodos   = this.getTotalDiaDaSemanaPeriodo();
        var arrDias         = ["QTSEGUNDA", "QTTERCA", "QTQUARTA", "QTQUINTA", "QTSEXTA", "QTSABADO", "QTDOMINGO"];
        arrDias.forEach(function(dia) {
            totalQuantidade += (parseInt(widget.currentRow[dia]) * totalPeriodos[dia]);
        });
        
        if(!isMensal) {
            widget.currentRow.QTITEMORCAMENTO = totalQuantidade;
            this.calcularValorTotalItem(widget);
        } else {
            if(totalQuantidade > widget.currentRow.QTITEMORCAMENTO){
                ScreenService.showMessage("Operação Bloqueada!<br>A quantidade mensal não pode ser menor que a quantidade semanal");
                widget.currentRow.QTITEMORCAMENTO = 0;
                this.calcularValorTotalItem(widget);
            }
        }
    };
    /**
     * Insere o imposto num orcamento já inserido
     * @params{Object} widget - widget imposto na tela de itens 
    **/
    this.salvarNewImpostoItemOrca = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgImposto  = templateManager.container.getWidget("wdgItensImpostoProposta");
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            if(widget.name !== "wdgItensImpostoProposta"){
                wdgImposto.currentRow.CDIMPOSTO      = widget.currentRow.CDIMPOSTO;
                wdgImposto.currentRow.VRPERCIMPOSTO  = widget.currentRow.VRPRECIMPOSTO;
                wdgImposto.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
                wdgImposto.currentRow.__is_new       = true;
                Utilities.inserirMultiplosComNovoCodigo(wdgImposto, "CDIMPOSTO").then(function(){
                    this.cancelarNewImpostoItemOrca();
                    this.atualizarValorItem();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            } else {
                wdgImposto.saveRow().then(function(){
                    this.cancelarNewImpostoItemOrca();
                    this.atualizarValorItem();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    /**
     * Cancela a inserção de um novo imposto
     * @params{Object} widget - widget imposto popup
    **/
    this.cancelarNewImpostoItemOrca = function(){
        var widget = templateManager.container.getWidget("wdgItensImpostoProposta");
        ScreenService.closePopup();
        widget.list();
        widget.activate();
        widget.reload();
    };
    /**
     * Atualiza o valor do item da proposta
    **/
    this.atualizarValorItem = function(){
        var wdgProposta    = templateManager.container.getWidget("wdgProposta");
        var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
        var params = {"NRSEQORCAMENTO" : nrseqorcamento};
        General.callRestEngine("product#/fct_itemorcamentoalimenticio_widget/atualizarValorItem", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.fct_orcamentoaliment_widget && response.dataset.fct_orcamentoaliment_widget.VRORCAMENTO){
                wdgProposta.currentRow.VRORCAMENTO = response.dataset.fct_orcamentoaliment_widget.VRORCAMENTO;
            }
        });
    };
    /**
     * Exclui os impostos de um imposto
    **/
    this.excluirImpostoItensOrca = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function() {
            widget.dataSource.getCheckedRows().forEach(function(imposto){
                widget.currentRow = imposto;
                widget.removeRow();
            });
            this.cancelarNewImpostoItemOrca();
            setTimeout(function() {
                this.atualizarValorItem();
            }.bind(this), 100);
        }.bind(this));
    };
    /**
     * Permite ao usuário adicionar uma nova formação de preço
     * @params{Object} widget - widget mão de obra itens
    **/
    this.adicionarItensNovaFormacao = function(widget){
        var wdgItensForm = templateManager.container.getWidget("wdgItensFormacao");
        widget.enabled   = false;
        wdgItensForm.isVisible = true;
        ScreenService.openSwipe(wdgItensForm).then(function() {
            wdgItensForm.enabled = true;
            wdgItensForm.activate();
            wdgItensForm.edit();
        });
    };
    /**
     * Cancela a inserção de uma nova formação de preço no orcamento inserido
     * @params{Object} widget - widget itens de formação de preço  
    **/
    this.voltarWidgetMObraItens = function(widget){
        ScreenService.closePopup();
        widget.isVisible = false;
        widget.enabled   = false;
        
        var wdgMObra = templateManager.container.getWidget("wdgItensMaoObra");
        ScreenService.closeSwipe().then(function(){
            wdgMObra.enabled = true;
            wdgMObra.list();
            wdgMObra.activate().then(function() {
                wdgMObra.reload();
            });
        });
    };
    /**
     * Salva uma nova formação de preço na proposta
     * @params{Object} widget - widget itens formação de preço
    **/
    this.salvaItemFormacaoPreco = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgProposta    = templateManager.container.getWidget("wdgProposta");
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            var formprecoservico = [];
            this.getDataFormacaoServico(widget).forEach(function(newFormacao){
                widget.getField("NRCALCFORMPRECSERV").dataSource.getCheckedRows().forEach(function(elemento){
                    if(newFormacao.NRFORMPRECOSERVICO === elemento.NRFORMPRECOSERVICO){
                        formprecoservico.push({
                            "NRFORMPRECOSERVICO": newFormacao.NRFORMPRECOSERVICO ,
                            "NRCALCFORMPRECSERV": "_" + elemento.NRCALCFORMPRECSERV + "_",
                            "NRSEQORCAMENTO"    : nrseqorcamento,
                            "CDSERVORCA"        : newFormacao.CDSERVORCA,
                            "NRITEMORCAMENTO"   : 0,
                            "QTFORMACAOPROP"    : 0,
                            "__is_new"          : true
                        });
                    }
                });
            });
            General.callRestEngine("product#/fct_itemorcamentoaliment_widget/atualizarQuantidadeMObraItens", "DataSet", formprecoservico).then(function(response){
                ScreenService.showMessage("Processo realizado com sucesso!");
                this.voltarWidgetMObraItens(widget, nrseqorcamento);
                this.atualizarValorItem();
            }.bind(this));
        }
    };
    /**
     * Recalcula o valor total da formação de preço na proposta após a alteração 
     * da quantidade 
     * @params{Object} widget - widget mão de obra (itens)
    **/
    this.calcularVrMObraItens = function(widget){
        var qtFormacao = parseFloat(widget.currentRow.QTFORMACAOPROP_inline);
        var vrunitario = parseFloat(widget.currentRow.VRUNITARIO);
        widget.currentRow.VRTOTAL = qtFormacao * vrunitario;
    };
    /**
     * Edita a quantidade da mão de obra na proposta e manda salvar
     * @params{Object} widget - widget mão de obra (itens)
    **/
    this.atualizarQuantidadeMObraItens = function(widget){
        var nrseqorcamento  = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;
        var formModificadas = [];
        widget.dataSource.data.forEach(function(row){
            if(row.QTFORMACAOPROP_inline && row.QTFORMACAOPROP_inline != row.changed_QTFORMACAOPROP){
                row.NRSEQORCAMENTO = nrseqorcamento;
                row.__is_new = false;
                row.changed  = true;
                formModificadas.push(row);
            }
        }.bind(this));
        General.callRestEngine("product#/fct_itemorcamentoaliment_widget/atualizarQuantidadeMObraItens", "DataSet", formModificadas).then(function(){
            widget.list();
            widget.reload();
            this.atualizarValorItem();
            ScreenService.showMessage("Processo realizado com sucesso!");
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * remove formações de preço da proposta
     * @params{Object} widget - widget mão de obra (itens)
    **/
    this.removerMaoObraItens = function(widget){
        var rows = widget.dataSource.getCheckedRows();
        var nrseqorcamento = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;
        rows.forEach(function(row){
            row.NRSEQORCAMENTO = nrseqorcamento;
        });
        
        General.callRestEngine("product#/fct_itemorcamentoaliment_widget/deletarMaoObraItens", "DataSet", rows).then(function(){
            widget.list();
            widget.reload();
            ScreenService.showMessage("Processo realizado com sucesso!");
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Exclui um elemento da formçaõ de preço auxiliar e manda
     * editar o valor do item
     * @params{Object} widget widget editar itens formação de preço
    **/
    this.excluirFormacaoItens = function(widget){
        var mensagem = "";
        var params = [
            {
                name: "NRCALCFORMPRECSERVPAI",
                operator: "=",
                value: widget.currentRow.NRCALCFORMPRECSERV
            },
            {
                name: "NRSEQORCAMENTO",
                operator: "=",
                value: widget.currentRow.NRSEQORCAMENTO
            },
            {
                name: "NRFORMPRECOSERVICO",
                operator: "=",
                value: widget.currentRow.NRFORMPRECOSERVICO
            },
            {
                name: "NRRELACFORMACAOPROP",
                operator: "=",
                value: widget.currentRow.NRRELACFORMACAOPROP
            }
        ];
        
        General.callRestEngine("product#/crud/fct_calcformprecserv_itpro_widget", "FilterData", params).then(function(response){
            if(response.dataset && response.dataset.fct_calcformprecserv_itpro_widget && response.dataset.fct_calcformprecserv_itpro_widget.length > 0)
                mensagem = "Apagar este elemento levará a exclusão dos elementos inferiores a ele.<br>Confirma exclusão?";
            else 
                mensagem = "Delete Register?";

            ScreenService.confirmMessage(ScreenService.i18n(mensagem), "question", function(){
                params = {
                    "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                    "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
                    "NRRELACFORMACAOPROP": widget.currentRow.NRRELACFORMACAOPROP,
                    "NRFORMPRECOSERVICO" : widget.currentRow.NRFORMPRECOSERVICO
                };

                General.callRestEngine("product#/fct_itemorcamentoaliment_widget/deletarElementosFormacaoProp", "Row", params).then(function(){
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja recalcular a Formação de Preço?"), 'question', function (success) {
                        this.recalcularArvore(widget);
                    }.bind(this));
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        this.atualizarValorItem();
                    }.bind(this));
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }.bind(this));
        }.bind(this));
    };
    /**
     * Salva uma niva despesa inserida na tela de itens 
     * @params{Object} widget - widget despesa(tela itens)
    **/
    this.salvarItemDespesa = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
            General.widgetSave(widget).then(function(response){
                if(response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                    widget.currentRow.NRDESPPROP = response.dataset.columnsToReturn[0].NRDESPPROP;
                    widget.currentRow.NRORG      = response.dataset.columnsToReturn[0].NRORG    ;
                }
                this.atualizarValorItem();
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Salva um novo investimento em uma proposta já inclusa
     * @params{Object} widget - widget investimento (aba itens)
     **/
    this.salvarItemInvestimento = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
            General.widgetSave(widget).then(function(response){
                if(response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                    widget.currentRow.NRINVESTPROP = response.dataset.columnsToReturn[0].NRINVESTPROP;
                    widget.currentRow.NRORG        = response.dataset.columnsToReturn[0].NRORG;
                }
                this.atualizarValorItem();
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Cancela o popup de inserção de uma taxa em um orcamento já inserido
    **/
    this.cancelarPopupItensTaxa = function(){
        var widget = templateManager.container.getWidget("wdgItensTaxasProposta");
        ScreenService.closePopup();
        widget.list();
        widget.activate().then(function(){
            widget.reload();
        });
    };
    /**
     * Salva novas taxas em um orcamento
     * @params{Object} widget - widget taxas (itens)
    **/
    this.salvarItensTaxas = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            if(!widget.currentRow.__is_new){
                widget.saveRow().then(function(){
                    widget.list();
                    widget.reload();
                });
            } else { 
                var wdgProposta = templateManager.container.getWidget("wdgProposta");
                widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
                Utilities.inserirMultiplosComNovoCodigo(widget, "CDTAXA").then(function(){
                    this.cancelarPopupItensTaxa();
                    this.atualizarValorItem();
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    /**
     * Deleta taxas de um orcamento já inserido
     * @params{Object} widget - widget taxas (itens)
    **/
    this.deletarItensTaxas = function(widget){
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function() {
            widget.dataSource.getCheckedRows().forEach(function(imposto){
                widget.currentRow = imposto;
                widget.removeRow();
            });
            this.cancelarPopupItensTaxa();
            setTimeout(function() {
                this.atualizarValorItem();
            }.bind(this), 100);
        }.bind(this));
    };
    /**
     * Abre um popup para a inserção de uma nova taxa
    **/
    this.adicionarNewTaxaItens = function(widget){
        var cdTaxas  = [];
        var wdgPopup = templateManager.container.getWidget("wdgPopupTaxasItens");
        widget.dataSource.data.forEach(function(cdtaxa){
            cdTaxas.push(cdtaxa.CDTAXA);
        });
        
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate().then(function() {
                wdgPopup.getField("CDTAXA").dataSourceFilter[0].value = cdTaxas;
                wdgPopup.getField("CDTAXA").reload();
            });
        }.bind(this));
    };
    /**
     * Exibe mensagem de confirmação para copiar a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.confirmarRenegociarCopiarProposta = function(widget, tipo) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente copiar a proposta?"), 'question', function(){   
            this.copiarVersionarProposta(widget, tipo);
        }.bind(this));
    };
    /**
     * Verifica se a proposta escolhida está aceita
     * Caso esteja abre um popup pedindo a justificativa 
     * @params{Object} widget widget proposta
     * @params{String} action nome da action que acionou a função
    **/
    this.validaSituacaoPropCancReneg = function(widget, action){
        var msgConfirmacao = null;
        if((widget.currentRow.IDSITUACAOORCAMENTO == "CANCELADO" && action === "versionar_proposta") || action === "cancelar_proposta")
            msgConfirmacao = "Deseja realmente "+ (action === "cancelar_proposta" ? "cancelar" : "renegociar") +" a proposta?";
        else 
            msgConfirmacao = "Ao renegociar esta proposta, sua versão atual ficará como cancelada. Deseja continuar?";
        
        if(widget.currentRow.IDSITUACAOORCAMENTO === "ACEITO"){
            var params = {
                "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO
            };
            General.callRestEngine("product#/fct_orcamento_widget/getContratosDifPendentes", "Row", params)
            .then(function(response){
                if(response.dataset && response.dataset.fct_orcamento_widget && response.dataset.fct_orcamento_widget.CONTRATOS){
                    ScreenService.showMessage("Operação bloqueada.<br> Não é permitido "+ (action === "cancelar_proposta" ? "o cancelamento" : "a renegociação") +" de propostas que possuem contratos já aprovados");
                } else {
                    ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function(){
                        this.abrePopupDsJustCancReneg(widget, action);
                    }.bind(this));
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        } else {
            ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function(){
                if(action === "cancelar_proposta")
                    this.cancelarProposta(widget);
                else 
                    this.copiarVersionarProposta(widget, 'V');
            }.bind(this));
        }
    };
    /**
     * Abre o popup de edição do operador responsável da proposta
     * @params{Object} widget widget proposta grid
    **/
    this.abrirPopupAlterarOper = function(widget){
        var wdgPopup = templateManager.container.getWidget("wdgPopupAlteraOper");
        wdgPopup.isVisible = true;
        wdgPopup.getField("CDOPERRESPATUAL").readOnly = false;
        
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.newRow();
            wdgPopup.activate().then(function(){
                this.verificaHierarquia({}, wdgPopup);
                wdgPopup.getField("NRSEQORCAMENTO").dataSourceFilter = [wdgPopup.getField("NRSEQORCAMENTO").dataSourceFilter[0]];
            }.bind(this));
        }.bind(this));
    };
    
    /**
     * filtra as fields de operador atual e propostas do popup de alteração dos
     * operadores responsáveis
     * @params{Object} widget widget popup altera operador responsável
     * @params{Object} field fieldAlterada nome da field de alteraçao
    **/
    this.filtraOperadoresEPropostas = function(widget, fieldAlterada){
        var fieldProposta  = widget.getField("NRSEQORCAMENTO");
        var fieldOperAtual = widget.getField("CDOPERRESPATUAL");

        if(widget.currentRow.CDOPERRESPATUAL && !widget.currentRow.NRSEQORCAMENTO || (widget.currentRow.NRSEQORCAMENTO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "CDOPERRESPATUAL")){
            fieldProposta.clearValue();
            if(widget.currentRow.CDOPERRESPATUAL && widget.currentRow.CDOPERRESPATUAL.length > 0){
                fieldProposta.dataSourceFilter[1] = {
                    "name": "CDOPERRESPONSAVEL",
                    "operator": "IN",
                    "value": widget.currentRow.CDOPERRESPATUAL
                };
                fieldOperAtual.readOnly = false;  
            } else {
                if(fieldProposta.dataSourceFilter.length > 1)
                    fieldProposta.dataSourceFilter.pop();
            }
            
            fieldProposta.reload();
        } else {
            widget.currentRow.CDOPERRESPONSAVEL = [];
            widget.getField("NRSEQORCAMENTO").dataSource.getCheckedRows().forEach(function(proposta){
                widget.currentRow.CDOPERRESPONSAVEL.push(proposta.CDOPERRESPONSAVEL);
            });
            
            if((!widget.currentRow.CDOPERRESPATUAL && widget.currentRow.NRSEQORCAMENTO) || (widget.currentRow.NRSEQORCAMENTO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "NRSEQORCAMENTO")){
                if(widget.currentRow.CDOPERRESPONSAVEL &&  widget.currentRow.CDOPERRESPONSAVEL.length > 0){
                    fieldOperAtual.clearValue();
                    widget.currentRow.CDOPERRESPATUAL = [];
                    fieldOperAtual.dataSourceFilter[0] = {
                        "name": "CDOPERADOR",
                        "operator": "IN",
                        "value": widget.currentRow.CDOPERRESPONSAVEL
                    };
                    
                    fieldOperAtual.readOnly = true;
                    widget.currentRow.CDOPERRESPONSAVEL.forEach(function(operador){
                        if(widget.currentRow.CDOPERRESPATUAL.indexOf(operador) === -1)
                            widget.currentRow.CDOPERRESPATUAL.push(operador);
                    });
                } else {
                    if(fieldOperAtual.dataSourceFilter.length > 0){
                        fieldOperAtual.dataSourceFilter.pop();
                        fieldOperAtual.readOnly = false; 
                    }
                }
                
                fieldOperAtual.reload();
            }
        }
    };
    
    /**
     * Cancela a edição do operador responsável da proposta
     * @params{Object} widget widget popup alterar operador responsavel
    **/
    this.cancelarAlteracaoOper = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgPropostaGrid");
        var fieldProposta  = widget.getField("NRSEQORCAMENTO");
        var fieldOperAtual = widget.getField("CDOPERRESPATUAL");
        fieldOperAtual.readOnly = false;
        fieldOperAtual.dataSource.clearCheckedRows();
        if(fieldOperAtual.dataSourceFilter.length > 0){
            fieldOperAtual.dataSourceFilter.pop();
        }
        fieldProposta.dataSource.clearCheckedRows();
        if(fieldProposta.dataSourceFilter.length > 1){
            fieldProposta.dataSourceFilter.pop();
        }
        widget.dataSource.data = [];
        widget.isVisible = false;
        ScreenService.closePopup();
        wdgProposta.activate().then(function() {
            wdgProposta.reload();
        });
    };
    
    /**
     * Salva as alterações do operador responsável
    **/
    this.salvaAteracoesOperResp = function(widget){
        if(widget.currentRow.NRSEQORCAMENTO.length === 0 && widget.currentRow.CDOPERRESPATUAL.length !== 0){
            widget.currentRow.NRSEQORCAMENTO  = [];
            widget.getField("NRSEQORCAMENTO").dataSource.data.forEach(function(data){
                widget.currentRow.NRSEQORCAMENTO.push(data.NRSEQORCAMENTO);
            });
        }
        
        if(widget.isValid()){
            var params = {
                "NRSEQORCAMENTO"   : widget.currentRow.NRSEQORCAMENTO,
                "CDOPERRESPONSAVEL": widget.currentRow.CDNOVOOPERRESP,
                "ALTERACONTRATO"   : widget.currentRow.ALTERACONTRATO
            };
            General.callRestEngine("product#/fct_orcamento_widget/salvaAlteracaoMassaOper", "Row", params).then(function(){
                ScreenService.showMessage("Propostas atualizados com sucesso.");
                this.cancelarAlteracaoOper(widget);
            }.bind(this));
        }
    };
    /**
     * Verirfica se o elemento que está sendo editado pode receber um preço
     * @params{Object} rowCusto - row que está sendo altera no inlin edit
    **/
    this.validarValorCusto = function(rowCusto){
        if(!rowCusto.CDSERVORCARELAC && rowCusto.VRCUSTOSERV){
            ScreenService.showMessage("Operação bloqueada!<br>Não é possível associar um preço a elementos que não possuem serviços relacionados.");
            rowCusto.VRCUSTOSERV = "";
        }
    };
    /**
     * Cancela o popup e ativa a widget que contem o nome do paramêtro
     * @params{String} widgetName - nome da widget que será ativada
    **/
    this.cancelarPopupDsJustificativa = function(widgetName){
        ScreenService.closePopup();
        templateManager.container.getWidget(widgetName).activate();
    };
    /**
     * Salva a justificativa de cancelamento ou renegociação da proposta
     * @params{Object} widget - widget popup de justificativa de cancelamento ou renegociação
    **/
    this.salvarJustificativaCancReneg = function(widget){
        if(widget.isValid()){
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            wdgProposta.currentRow.DSJUSTIFCANCRENEG = widget.currentRow.DSJUSTIFCANCRENEG;
            wdgProposta.currentRow.DTACEITEORCA      = null;
            wdgProposta.saveRow().then(function(){
                ScreenService.closePopup();
                widget.isVisible = false;
                wdgProposta.activate();
                if(widget.currentRow.ACTION === "cancelar_proposta")
                    this.cancelarProposta(wdgProposta);
                else 
                    this.copiarVersionarProposta(wdgProposta, 'V');
            }.bind(this))
            .catch(function(error){ 
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Recalcula toda a widget de investimento durante a inserção da proposta
     * @params{Oject} widget - widget investimento inserindo
    **/
    this.calcularWidgetInvetimentoInserindo = function(widget){
        widget.dataSource.data.forEach(function(investimento){
            var qtMeses = this.tirarMascaraValores(investimento.QTMESARMOTIZACAO);
            var vrtotal = this.tirarMascaraValores(investimento.VRINVESTIMENTO);
            var quantidade = this.tirarMascaraValores(investimento.QTINVESTIMENTO);
            investimento.VRTOTINVESTIMENTO = vrtotal * quantidade;
            investimento.VRPARCELA = (vrtotal * quantidade)/qtMeses;
        }.bind(this));
    };
    /**
     * Recalcula toda a widget de investimento durante a inserção da proposta
     * @params{Oject} widget - widget investimento inserindo
    **/
    this.calcularWidgetDespesaInserindo = function(widget){
        widget.dataSource.data.forEach(function(despesa){
            despesa.VRDESPESA = this.tirarMascaraValores(despesa.VRDESPESA);
            this.validarECalcularPeriodo(despesa, "VRDESPESA", "VRTOTDESPPERIODO", "CDPERIODO");
        }.bind(this));
    };
    /**
     * Verifica se a fields de inline edit podem ser alteradas 
     * @params{Object} row - row que está sendo alterada
     * @params{Object} widget - widget da alteracao
    **/
    this.validarSituacaoProposta = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta"); 
        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO !== "ELABORACAO"){
            if(wdgProposta.currentRow.IDSITUACAOORCAMENTO !== "CONCLUIDO" || idAlteraPropConc === "N"){
                ScreenService.showMessage("Operação bloqueada!<br>Não é permitido fazer alterações em propostas com situações diferente de Elaboração.");
                widget.list();
                widget.reload();
                return false;
            }
        }
        return true;
    };
    /**
     * Calcula e salva o valor da proposta apos a alteração do valor de um servico
     * @params{Object} widget - widget de resumo do servico (proposta já inserida)
     * @params{Object} wdgProposta - widget de proposta
    **/
    this.calcularNewVrOrcamento = function(widget, wdgProposta){
        var wdgFinanceiro = templateManager.container.getWidget("wdgResumoGrid");
        var wdgTabbed     = templateManager.container.getWidget("wdgTabbedPrincipalResumoProp");
        if(this.validarSituacaoProposta){
            var params = {
                "NEWVRSERVICO"   : widget.currentRow.VRUNITITEMORCA_GRID,
                "QTITEMORCAMENTO": widget.currentRow.QTITEMORCAMENTO,
                "NRCOMPSERVICO"  : widget.currentRow.NRCOMPSERVICO,
                "NRSEQORCAMENTO" : wdgProposta.currentRow.NRSEQORCAMENTO
            };
            General.callRestEngine("product#/fct_orcamentoaliment_widget/alterarValorServicoTotalItem", "Row", params).then(function(response){
                if(response.dataset && response.dataset.getInfoResultado){
                    wdgProposta.currentRow.VRORCAMENTO = response.dataset.getInfoResultado.VRORCAMENTO;
                    widget.dataSource.data = response.dataset.getInfoResultado.SERVICOS;
                    var vrFin              = response.dataset.getInfoResultado.FINANCEIRO;
                    this.setValoresWdgRFinanceiro(wdgFinanceiro, vrFin["TOTALBRUTO"], vrFin["IMPOSTO"], vrFin["TOTALLIQ"], vrFin["CALIMENTAR"], vrFin["TOTALCTAXA"], vrFin["MOBRA"], vrFin["DESPESAS"], vrFin["INVESTIMENTOS"], vrFin["TAXAS"], vrFin["MARKUP"], vrFin["TOTALSTAXA"], vrFin["TOTALIDEAL"], vrFin["IMPOSTOIDEAL"], vrFin["TOTALLIQIDEAL"], vrFin["MARKUPIDEAL"]);
                }
                widget.redraw();
                wdgFinanceiro.redraw();
                wdgTabbed.list();
            }.bind(this));
        }
    };
    /**
     * Fecha a tela de Enviar E-mail.
     * 
     * @param {object} widget
     */
    this.fecharWdgEnviarEmail = function(widget) {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        this.desobrigaCamposEvolucaoEmail();
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;
            wdgProposta.view();
            wdgProposta.activate();
        });
    };
    /**
     * Exclui um item do cálculo da mão de obra
     * @params{Object} widget - widget itens de formação de preço inserindo
    **/
    this.excluiItensMObraArvore = function(widget) {
        idItensMObraAlterado = true;
        UtilitiesPropostaContrato.excluiItensArvore(widget);
    };
    /**
     * Confirma se o lançamento de proposta será cancelado
    **/
    this.confirmarCancelamentoDoLancamento = function() {
        ScreenService.confirmMessage("Cancelar o lançamento da proposta acarretará na perda das informações descritas até o momento.<br>Deseja continuar?", "question", function() {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            this.cancelarSalvamentoProposta(wdgProposta);
        }.bind(this));
    };
    /**
     * Limpa todos os dados de lancamentos de proposta do Local Storage
    **/
    this.removerDataLocalStorageLancamento = function() {
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgImpostoProposta");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgServicosInserindo");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgInvestimentosInserindo");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgDespesas");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgMaoObraGridInserindo");
        Sm00001000_PropostaAlimenticia_Repository.removeDataLocalStorageEngine("wdgCustoProposta");
    };
    /**
     * Volta para o grid de serviços na tela de itens
     * @params{Object} widget - widget de itens 
    **/
    this.voltarGridItensProposta = function(widget) {
        ScreenService.closeSwipe(); 
        widget.reload();
        widget.getField("VRUNITITEMORCA").isVisible  = true;
        widget.getField("QTITEMORCAMENTO").readOnly  = true;
        widget.getField("NMTIPOORCAMENTO").isVisible = true;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001000_LancamentoPropostaAlimenticia_Controller", Sm00001000_LancamentoPropostaAlimenticia_Controller);  
});

// FILE: ./assets/js/controllers/Sm00001000_LancamentoProposta_Controller.js
function Sm00001000_LancamentoProposta_Controller(ScreenService, General, templateManager, Sm00001000_LancamentoProposta_Service, Utilities, ZHPromise, Operador_Repository, Endereco, CRM_Utilities, ger00004500_Prospect_Service, ABEvolucaoController, UtilitiesPropostaContrato, ValidacoesParametrosFaturamentoUtil, ZhFilterPreferences, SelectUtil, restEngine, eventAggregator) {
    var self = this;
    // Campos de saída do modelo de proposta...
    var fieldsModelo = [
        "DSMODPGTO",
        "IDFATURAORCA",
        "NMGRUPOFATURA",
        "QTDIASCONTRATO",
        "QTDIASVALIDADE",
        "DTVALIDADE"
    ];

    var visibilidadeAcoes       = null;
    var opLogado                = null;
    var nomeOperador            = null;
    var idOperMaster            = null;
    var idUtilizaHierOper       = null;
    var nrparcnegocioOperLogado = null;
    var tpOperador              = null;
    var nrOrgOper               = null;
    var is_required             = false;
    var qtdPrenchido            = [];
    var trancado                = false;
    var idAlteraPropConc        = null;
    var servIdMedidaFixa        = [];  
    var idObrigaModProp         = null;

    var cdidiom_padrao = null;
    var nmidioma_padrao = null;
    var nrmoeda_padrao = null;
    var nmmoeda_padrao = null;

    var mudouMoeda = false;

    // widgets
    var getWidgetPropostaGrid = function () {
        return templateManager.container.getWidget('wdgPropostaGrid');
    };
    var getWidgetProposta = function () {
        return templateManager.container.getWidget('wdgProposta');
    };
    var getWidgetEndereco = function () {
        return templateManager.container.getWidget('wdgEndereco');
    };
    var getWidgetItensProposta = function () {
        return templateManager.container.getWidget('wdgItensProposta');
    };
    var getWidgetAnexos = function () {
        return templateManager.container.getWidget('WDGT_ANEXOS_EVOLUCAO');
    };
    var getWidgetEvolucao = function () {
        return templateManager.container.getWidget('evolucao');
    };
    var getWidgetItensPropostaTree = function () {
        return templateManager.container.getWidget('wdgItensPropostaTree');
    };
    var getWidgetWdgElementosProposta = function () {
        return templateManager.container.getWidget('wdgElementosProposta');
    };
    var getWidgetWdgEscopoProposta = function () {
        return templateManager.container.getWidget('wdgEscoposProposta');
    };
    var getWidgetNegociacaoDeValores = function() {
        return templateManager.container.getWidget('wdgNegociacaoValores');
    };
    // Actions
    var getActionEditWidgetProposta = function(){
        return getWidgetProposta().getAction('edit');
    };
    var getActionAddEndereco = function(){
        return getWidgetEndereco().getAction('add');
    };
    var getActionEditEndereco = function(){
        return getWidgetEndereco().getAction('edit');
    };
    var getActionEditEscopoProposta = function(){
        return getWidgetWdgEscopoProposta().getAction('edit');
    };
    var getActionEditEvolucao = function(){
        return getWidgetEvolucao().getAction('EDIT');
    };
    var getActionAdicEvolucao = function(){
        return getWidgetEvolucao().getAction('ADD_EVOLUCAO');
    };
    // fields
    var getFieldModeloDePropostaFromWidgetPropostaGrid = function(){
        return getWidgetProposta().getField('NMORCAMENTOMODELO');
    };
    
    var getDataSourceTipoDesconto = function(somenteP) {
        var data = [
            {
                "VALUE": "P",
                "NAME": "Na quantidade medida total (vendida + adicional)"
            }
        ];
        if (!somenteP) {
            data.push({
                "VALUE": "V",
                "NAME": "Apenas na quantidade vendida"
            });
        }
        
        return data;
    };
    
    var setDataSourceTipoDesconto = function(widget, somenteP) {
        var fieldTipoDesconto = widget.getField("IDTPDESCONTO");
        fieldTipoDesconto.dataSource.data = getDataSourceTipoDesconto(somenteP);
    };
    
    /**
     * Coloca na currentRow os parametrôs de novo código
     * para a inserção de novos registros
     * @params{Object} widget - widget que irá inserir um novo registro
    **/
    this.criarParametrosNewCode = function(widget){
        if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 1;
            widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
            if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
        }
    };
    /** Busca o operador logado a sessão atual 
     * @params{Object} widget de proposta
     */
    // this.wdgPropostaGridOnEnterOnce = function() {
    //     ZhFilterPreferences.wdgPropostaGridOnEnter(templateManager.container.getActiveWidget()).then(function(){
    //         Operador_Repository.getOperadorLogado().then(function(success) {
    //             var filter = templateManager.container.getActiveWidget();
    //             filter.currentRow.IDSITUACAOORCAMENTO = ["ELABORACAO", "CONCLUIDO", "ENVIADO", "ACEITO"];
    //             var operadorResponsavel = filter.getField("CDOPERRESPONSAVEL");
    //             operadorResponsavel.reload().then(function(){
    //                 Utilities.setValoresPadroesParaCampoSelectMultiple(operadorResponsavel, [success[0].CDOPERADOR.toString()], 'CDOPERADOR');
    //                 operadorResponsavel.lazyLoad = true;
    //                 operadorResponsavel.itemsPerPage = 50;
    //             });
                
    //         }.bind(this));
    //     });
    // };
    /** Busca o operador logado a sessão atual 
     * @params{Object} widget de proposta
     */
    this.wdgPropostaGridOnEnterOnce = function() {
        ZhFilterPreferences.openFilterWidget(templateManager.container.getActiveWidget()).then(function(){
            Operador_Repository.getOperadorLogado().then(function(success) {
                var filter = templateManager.container.getActiveWidget();
                filter.currentRow.IDSITUACAOORCAMENTO = ["ELABORACAO", "CONCLUIDO", "ENVIADO", "ACEITO"];
                var operadorResponsavel = filter.getField("CDOPERRESPONSAVEL");
                operadorResponsavel.reload().then(function(){
                    Utilities.setValoresPadroesParaCampoSelectMultiple(operadorResponsavel, [success[0].CDOPERADOR.toString()], 'CDOPERADOR');
                    operadorResponsavel.lazyLoad = true;
                    operadorResponsavel.itemsPerPage = 50;
                });
                eventAggregator.publish("abriuFiltroProposta");
            }.bind(this));
        });
    };

    this.buscaOperadorLogado = function() {
        var deferred = ZHPromise.defer();
        Operador_Repository.getOperadorLogado().then(function(success) {
            opLogado                = success[0].CDOPERADOR;
            nomeOperador            = success[0].NMOPERADOR;
            nrparcnegocioOperLogado = success[0].NRPARCNEGOCIO;
            tpOperador              = success[0].IDTPOPERADOR;
            nrOrgOper               = success[0].NRORG;
            
            this.buscaInformacaoHierarquia();
            deferred.resolve(success);
        }.bind(this));

        return deferred.promise;
    };

    /* Verifica se o usuário tem hierarquia de operadores e caso tenha
     * busca sua posição
     */
    this.verificaHierarquia = function(widgetPropostaGrid, widget) {
        if(idOperMaster === null && idUtilizaHierOper === null)
            this.buscaInformacaoHierarquia();

        if(opLogado) {
            widget.currentRow.CDOPERRESPONSAVEL = opLogado;
            widget.currentRow.CDNOVOOPERRESP    = opLogado;
            widget.currentRow.NMOPERRESP        = nomeOperador;
        } else {
            this.buscaOperadorLogado(widget).then(function(){
                widget.currentRow.CDOPERRESPONSAVEL = opLogado;
                widget.currentRow.CDNOVOOPERRESP    = opLogado;
                widget.currentRow.NMOPERRESP        = nomeOperador;
            });
        }    
    };
    
    /* busca os dados de hierarquia do usuário logado
    * @ params{Object} widget  
    */
    this.buscaInformacaoHierarquia = function() {
        General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", {}).then(function(success) {
            if (success.dataset.fct_parametrogeral && success.dataset.fct_parametrogeral.length > 0) {
                var paramsService = success.dataset.fct_parametrogeral[0];
                
                if(paramsService.IDOBRIGMODORC)
                    idObrigaModProp = (paramsService.IDOBRIGMODORC == 'S');
                
                var paramsFilter = [
                    {
                        name: "CDOPERADOR",
                        operator: "=",
                        value: opLogado
                    }
                ];
                
                idUtilizaHierOper = paramsService.IDUTILIZAHIEROPER;
                General.callRestEngine("product#/crud/crm_hierarquiaoper", "FilterData", paramsFilter).then(function(success) {
                    if (success.dataset.crm_hierarquiaoper && success.dataset.crm_hierarquiaoper.length > 0) 
                        idOperMaster = success.dataset.crm_hierarquiaoper[0].IDOPERMASTER;
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };

    /* Recarrega o field caso o dataSource tenha sido alterado 
     * @params{Object} field Nome Operador
     */
    this.recarregaDadosField = function(fieldNmOp) {
        var widgetAtiva = templateManager.container.getActiveWidget();
        var fieldParc  =  widgetAtiva.getField("NRPARCNEGOCIO");
        
        if (widgetAtiva.name !== "wdgProposta") {
            fieldParc.dataSourceFilter[1] = {
                name: "P_CDOPERADOR",
                operator: "=",
                value: opLogado
            };
        }

        Utilities.iniciarAutocomplete(fieldNmOp);
    };

    this.novaProposta = function() {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");

        wdgProposta.getField("NMIDIOMA").readOnly = false;
        wdgProposta.getField("NMMOEDA").readOnly = false;

        Operador_Repository.getOperadorLogado().then(function(success) {
            var parametro = {
                "NRPARCNEGOCIO" : success[0].NRPARCNEGOCIO,
                "NRORG"         : success[0].NRORG,
                "PROPOUCONT"    : "P"
            };

            return General.callRestEngine('product#/validarPerfilAcesso', "Row", parametro);
        }).then(function(perfilusuario) {
            if (perfilusuario.dataset.validarperfil.PERFILACESSO) {
                var modelosDeContrato = [];
                var field = getFieldModeloDePropostaFromWidgetPropostaGrid();
                perfilusuario.dataset.validarperfil.PERFILACESSO.forEach(function (modeloContrato) {
                    modelosDeContrato.push(modeloContrato.MODELO);
                });
                field.dataSourceFilter[2] = {
                    "name"     : "NRORCAMENTOMODELO",
                    "operator" : "NOT_IN",
                    "value"    : modelosDeContrato
                };
            }

            setLocalVar("PROPOSTA_NRSEQORCAMENTO", "");
            //setLocalVar("PROPOSTA_NRORG", "");
            
            self.validarObrigatoriedadeModelo();
            return ScreenService.openSwipe(wdgTabbedPrincipal);
        }).then(function() {
            wdgProposta.template = "widget/form.html";
            wdgProposta.newRow();
            self.alterarAcessibilidadeAbasProposta(false);

            //wdgProposta.newRow();
            wdgProposta.template = "widget/form.html";
            wdgTabbedPrincipal.edit();

            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            self.verificaHierarquia(wdgPropostaGrid, wdgProposta);
            wdgPropostaGrid.currentRow = {};

            self.validacoesGeraisProposta(wdgPropostaGrid);
            self.filtroInterlocutor(wdgProposta);
            self.setVendedorPadrao(wdgProposta);
            self.controlaCodProposta(wdgProposta);
            self.assinaturaEletronicaOnChange(wdgProposta);

            var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");
            wdgEscoposProposta.isVisible = false;

            //setar idioma e moeda padrao
            return General.callRestEngine("product#/configuracao_regional/getIdiomaMoedaPadrao", "Row", {});
        }).then(function(response) {
            if (response.dataset.retorno.CDIDIOM) {
                cdidiom_padrao = wdgProposta.currentRow.CDIDIOM = response.dataset.retorno.CDIDIOM;
                nmidioma_padrao = wdgProposta.currentRow.NMIDIOMA = response.dataset.retorno.NMIDIOMA;
                nrmoeda_padrao = wdgProposta.currentRow.NRMOEDA = response.dataset.retorno.NRMOEDA;
                nmmoeda_padrao = wdgProposta.currentRow.NMMOEDA = response.dataset.retorno.NMMOEDA;
            }
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    }

    this.validaPerfilEvolucao = function(){
        var wdgProposta = getWidgetProposta();
        var newitem = getActionEditEvolucao();
        var edititem = getActionAdicEvolucao();
        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO == "CONCLUIDO") {
            newitem.isVisible  = visibilidadeAcoes;
            edititem.isVisible = visibilidadeAcoes;
        }
    };

    /**
     * Seta o usuário logado como default no campo vendedor, caso ele seja um.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.setVendedorPadrao = function(wdgProposta) {
        Operador_Repository.getOperadorLogado().then(function(success) {

                var nrparcnegocio = success[0].NRPARCNEGOCIO;
                var params = {
                    "NRPARCNEGOCIO": nrparcnegocio
                };

                General.callRestEngine("product#/fct_orcamento_widget/verificarSeEVendedor", "Row", params).then(function(success) {
                        if (success.dataset.DADOS.vendedor) {
                            wdgProposta.currentRow.NRVENDEDOR = [nrparcnegocio];
                            templateManager.updateTemplate();
                        }
                    },
                    function(error) {
                        console.error(error);
                        ScreenService.showMessage(error.data.error);
                    });
            },
            function(error) {
                console.error(error);
            });
    };

    /**
     * Controla a acessibilidade das abas filhas de Proposta, desabilitando-as em 
     * modo de inserção e habilitando-as em modo de edição.
     * 
     * @param {boolean} enabled
     */
    this.alterarAcessibilidadeAbasProposta = function(enabled) {
        var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");

        wdgPropostaTabbed.widgets.filter(function(wid) {
                return wid.name != "wdgProposta";
            })
            .forEach(function(widget) {
                widget.enabled = enabled;
            });

        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");

        wdgTabbedPrincipal.widgets.filter(function(wid) {
                return wid.name != "wdgPropostaTabbed";
            })
            .forEach(function(widget) {
                widget.enabled = enabled;
            });

        templateManager.updateTemplate();
    };

    /**
     * Volta para o modo list da tela de proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.voltarParaPropostaGrid = function() {
        var deferred    = ZHPromise.defer(); 
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        wdgProposta.dataSource.data = [];
        wdgProposta.currentRow = {};

        ScreenService.closeSwipe().then(function() {
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            wdgPropostaGrid.activate();
            wdgPropostaGrid.refreshWidget().then(function() {
                deferred.resolve();
            });
        });
        return deferred.promise;
    };

    /**
     * Cancela o salvamento da Proposta.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.cancelarSalvamentoProposta = function(wdgProposta) {
        var wdgItensComp              = templateManager.container.getWidget("wdgItensComposicaoProposta");
        var wdgItensProposta          = templateManager.container.getWidget("wdgItensProposta");
        var wdgItensPropostaInserindo = templateManager.container.getWidget("wdgItensPropostaInserindo");
        var wdgTipoMedidaInserindo    = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
        var wdgPropostaTabbed         = templateManager.container.getWidget("wdgPropostaTabbed");
        var wdgTabbedPrincipal        = templateManager.container.getWidget("wdgTabbedPrincipal");
        //var wdgLocalAtendimento = templateManager.container.getWidget("localAtendProposta");
        wdgPropostaTabbed.view();

        wdgItensComp.isVisible              = false;
        wdgItensPropostaInserindo.isVisible = false;
        wdgTipoMedidaInserindo.isVisible    = false;
        //wdgItensProposta.isVisible = true;
        //wdgLocalAtendimento.isVisible = true;
        //wdgProposta.enabled = true;
        //this.desfiltroEscopo(wdgProposta);
        //General.widgetCancelForm(wdgProposta);
        wdgPropostaTabbed.enabled = true;
        this.manipularCamposModeloProposta(wdgProposta, false);
        wdgProposta.currentRow.changed = false;
        this.alterarAcessibilidadeAbasProposta(true);
        wdgProposta.template = "widget/form-view.html";
        if (wdgProposta.currentRow.__is_new) {
            var indexOfCurrentRow = wdgProposta.dataSource.data.indexOf(wdgProposta.currentRow);
            if (indexOfCurrentRow !== -1) {
                wdgProposta.dataSource.data.pop();
            }

            wdgProposta.list();
            this.voltarParaPropostaGrid();
        } else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
            Object.keys(wdgProposta.backupRow).forEach(function(key) {
                wdgProposta.currentRow[key] = wdgProposta.backupRow[key];
            }); //copia dados coluna por coluna para não perder referência do objeto da currentRow
            wdgProposta.view();
            wdgTabbedPrincipal.view();
        }

        var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");
        wdgEscoposProposta.isVisible = true;
    };

    /**
     * Faz o controle geral da tela de Proposta de acordo com o registro selecionado.
     * 
     * @param {object} widget - Widget de Proposta (grid)
     */
    this.validacoesGeraisProposta = function(wdgPropostaGrid) {
        var situacao = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;
        //var utilizaComposicao = wdgPropostaGrid.currentRow.IDUTILIZACOMP;
        var utilizaComposicao = (wdgPropostaGrid.currentRow.QTITENSTRABSERV > 0) ? "S" : "N";
        var utilizaPrecoVariavel = wdgPropostaGrid.currentRow.IDUTILIZAVARPRECO;

        var wdgProposta = templateManager.container.getWidget("wdgProposta");

        //this.controlarAcoesDocumentoPropostaAbaEscopos(situacao);
        this.controlarCamposInlineEditable(situacao);
        this.controlarAcoesDeCrud(wdgProposta, situacao);
        this.controlarAcoesDeOperacaoProposta(wdgProposta, situacao);
        this.controlaActionEnviarEmail(wdgProposta);
        this.controlarAcoesDeOperacaoParcelasNegociacao(situacao);
        this.controlarVisibilidadeAcaoMemoriaDeCalculo(wdgProposta, utilizaComposicao);
        this.controlarVisibilidadeAbaElementoEditando(utilizaComposicao);
        this.controlarVisibilidadeAbaNomeContratualEditando(utilizaComposicao);
        this.controlarVisibilidadeAbaTipoMedidaVariavelEditando(utilizaComposicao, utilizaPrecoVariavel);
        
        this.controlarEscopoProposta(wdgProposta, false, false);

        var wid = templateManager.container.getWidget("wdgItensProposta");
        this.itensVerificarUtilizaComposicao(wid, utilizaComposicao, situacao);
       // this.controlarDataSourceItensProposta(wid, utilizaComposicao);
        this.controlarDataSourceItensProposta(utilizaComposicao);
        this.controlarCampoVariacaoMultiplicavelItensProposta(wid, utilizaComposicao);

        wid = templateManager.container.getWidget("wdgAdicionais");
        //this.controlarDataSourceItensProposta(wid, utilizaComposicao);
        this.itensVerificarUtilizaComposicao(wid, utilizaComposicao, situacao);
        //this.controlarDataSourceItensProposta(wid, utilizaComposicao);
        // this.validaIdCertsign(); // OTIMIZAÇÂO: estava duplicado portanto foi removido
    };
    
    this.controlarCampoVariacaoMultiplicavelItensProposta = function(wdgItensProposta, utilizaComposicao) {
        var campoVariacaoMultiplicavel = wdgItensProposta.getField("QTMEDIDAVARIAVEL");
        if (utilizaComposicao == "S") {
            campoVariacaoMultiplicavel.isVisible = true;
            campoVariacaoMultiplicavel.showOnList = true;
        }
        else {
            campoVariacaoMultiplicavel.isVisible = false;
            campoVariacaoMultiplicavel.showOnList = false;
        }
    };
    
    /**
     * Atribui o dataSource correto na tela de Itens da Proposta. O nome do 
     * dataSource depende se a proposta utiliza ou não composição de serviço.
     */
    this.controlarDataSourceItensProposta = function(/*wid,*/utilizaComposicao) {
        var widItens = templateManager.container.getWidget("wdgItensProposta");
        var widItensTree = templateManager.container.getWidget("wdgItensPropostaTree");
        
        if (utilizaComposicao == "S") {
            widItens.isVisible = false;
            widItensTree.isVisible = true;
            /*
            wid.dataSource.name = "product#/crud/fct_itemorcamento_arvore_medidas_variaveis";
            wid.template = "widget/master_detail/tree/grid.html";
            wid.templateToForm = "widget/master_detail/tree/grid.html";
            wid.primaryProperty = "IDARVORE";
            wid.parentProperty = "IDARVOREPAI";
            wid.expandingProperty = "NMSERVORCA";
            wid.breadcrumbProperty = "NMSERVORCA";
            templateManager.updateTemplate();
            wid.redraw();*/
        } else {
            widItens.isVisible = true;
            widItensTree.isVisible = false;
            
            /*wid.dataSource.name = "product#/crud/fct_itemorcamento_widget";
            wid.template = "widget/master_detail/grid.html";
            delete wid.primaryProperty;
            delete wid.parentProperty;
            delete wid.expandingProperty;
            delete wid.breadcrumbProperty;
            templateManager.updateTemplate();
            wid.redraw();*/
        }
    };

    /**
     * Permite ou bloqueia os campos "inlineEditable" de acordo com a situação da proposta.
     * @param {string} situacao
     */
    this.controlarCamposInlineEditable = function(situacao) {
        var wdgItensProposta  = templateManager.container.getWidget("wdgItensProposta");
        var wdgTipoMedida     = templateManager.container.getWidget("wdgTipoMedidaPropostaEditando");
        var wdgNomeContratual = templateManager.container.getWidget("wdgNomeContratual");
        
        var fieldsInline = [];
        var widgets      = [wdgItensProposta, wdgTipoMedida, wdgNomeContratual];
        
        widgets.forEach(function(widget) {
            fieldsInline = fieldsInline.concat(widget.fields.filter(function(field) {
                return field.editavelGrid == true;
            }));
        });
        
        if (situacao == "ELABORACAO" || (situacao == 'CONCLUIDO' && idAlteraPropConc == 'S' && visibilidadeAcoes)) {
            fieldsInline.forEach(function(field) {
                if (field.name == 'NMCONTRATUALITEMORCA'){
                    field.inlineEditable = "text";  
                }
                else{
                    field.inlineEditable = "number";
                }
            });
        } else {
            fieldsInline.forEach(function(field) {
                delete field.inlineEditable;
            });
        }
        
        /*var fieldsWdgItensProposta = wdgItensProposta.fields.filter(function(field) {
            return field.editavelGrid == true;
        });
        var fieldsWdgTipoMedida = wdgTipoMedida.fields.filter(function(field) {
            return field.editavelGrid == true;
        });
        var fieldsWdgNmContratual = wdgTipoMedida.fields.filter(function(field) {
            return field.editavelGrid == true;
        });
        
        if (situacao == "ELABORACAO") {
            fieldsWdgItensProposta.forEach(function(field) {
                field.inlineEditable = "number";
            });
            fieldsWdgTipoMedida.forEach(function(field) {
                field.inlineEditable = "number";
            });
            fieldsWdgNmContratual.forEach(function(field) {
                field.inlineEditable = "number";
            });
        }
        else {
            fieldsWdgItensProposta.forEach(function(field) {
                delete field.inlineEditable;
            });
            fieldsWdgTipoMedida.forEach(function(field) {
                delete field.inlineEditable;
            });
            fieldsWdgNmContratual.forEach(function(field) {
                delete field.inlineEditable;
            });
        }*/
    };

    /**
     * Habilita a ação de Memória de Cálculo caso a proposta utilize composição de serviço. 
     * Desabilita se não utilizar.
     * 
     * @param {object} wdgProposta - Widget de Proposta (view)
     * @param {boolean} utilizaComposicao
     */
    this.controlarVisibilidadeAcaoMemoriaDeCalculo = function(wdgProposta, utilizaComposicao) {
        var actMemoriaCalculo = wdgProposta.getAction("memoria_calculo_proposta");
        if (utilizaComposicao == "S") {
            actMemoriaCalculo.isVisible = true;
        }
        else {
            actMemoriaCalculo.isVisible = false;
        }
    };

    /**
     * Esconde as ações do documento da proposta da aba de Elementos
     * quando a proposta estiver em elaboração ou cancelada.
     
    this.controlarAcoesDocumentoPropostaAbaEscopos = function(situacao) {
        var wdgEscopos = templateManager.container.getWidget("wdgEscoposProposta");
        //var actDocumento = wdgEscopos.getAction("documento_proposta");
        var actEnviarPorEmail = wdgEscopos.getAction("enviar_documento");

        var visivel = true;
        if (situacao == "ELABORACAO" || situacao == "CANCELADO") {
            visivel = false;
        }

        //actDocumento.isVisible = visivel;
        actEnviarPorEmail.isVisible = visivel;
    }; */

    /**
     * Habilita a aba de Elementos caso a proposta utilize composição de serviço. 
     * Desabilita se não utilizar.
     */
    this.controlarVisibilidadeAbaElementoEditando = function(utilizaComposicao) {
        var widget = templateManager.container.getWidget("wdgElementosProposta");
        widget.isVisible = (utilizaComposicao == "S");
    };
    /**
     * Controla a visibilidade da widget de nome contratual
     * para aparecer apenas quando a proposta for de composição de serviço
     * @params{String} utilizaComposicao - define se utiliza ou não composição
    **/
    this.controlarVisibilidadeAbaNomeContratualEditando = function(utilizaComposicao){
        var widget = templateManager.container.getWidget("wdgNomeContratual");
        if(widget) widget.isVisible = (utilizaComposicao == 'S');
    };
    /**
     * Habilita a aba de Tipo de Medida Variável caso algum escopo da proposta utilize preço variável. 
     * Desabilita se não utilizar.
     */
    this.controlarVisibilidadeAbaTipoMedidaVariavelEditando = function(utilizaComposicao, utilizaPrecoVariavel) {
        var widget = templateManager.container.getWidget("wdgTipoMedidaPropostaEditando");
        if (utilizaComposicao == "S" && utilizaPrecoVariavel.indexOf("S") > -1) {
            this.validarVisibilidadeWidgetTpMedida(widget);
        } else {
            widget.isVisible = false;
        }
    };
    /**
     * Verifica a situação da Proposta para permitir/bloquear sua edição e/ou exclusão.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.controlarAcoesDeCrud = function(wdgProposta, situacao) {
        var actDelete = wdgProposta.getAction("delete");
        var actEdit   = wdgProposta.getAction("edit");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
         
        if(situacao == "CONCLUIDO"){
            actDelete.isVisible = false;
            this.controlarAcoesDeCrudAux(wdgProposta.parent, (idAlteraPropConc == 'S' && visibilidadeAcoes), situacao);
            this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, (idAlteraPropConc == 'S' && visibilidadeAcoes), situacao);
        } else {
            if (situacao == "ELABORACAO") {
                actDelete.isVisible = true;
                actEdit.isVisible   = true;
                this.controlarAcoesDeCrudAux(wdgProposta.parent, true, situacao);
                this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, true, situacao);
                this.verificarEscopo(wdgProposta);
            } else {
                actEdit.isVisible   = false;
                actDelete.isVisible = (situacao == "CANCELADO");
                this.controlarAcoesDeCrudAux(wdgProposta.parent, false, situacao);
                this.controlarAcoesDeCrudAux(wdgTabbedPrincipal, false, situacao);
            }
        }
    };
    
    /**
     * Verifica se as proposta concluídas podem ser alterada
     * @params {Object} widget
    **/
    this.verificaParamAlteraPropConc = function(widget){
        var deferred = ZHPromise.defer();
        if(idAlteraPropConc){
            deferred.resolve();
        } else {
            var nrorg = getLocalVar("PROPOSTA_NRORG");
            var paramsGeral  = [
                {
                    name: "NRORG",
                    operator: "=",
                    value: nrorg
                }
            ];

            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", paramsGeral).then(function(success) {
                if(success.dataset.fct_parametrogeral.length !== 0)
                    idAlteraPropConc = success.dataset.fct_parametrogeral[0].IDALTERAPROPCONC;
                
                if(visibilidadeAcoes !== null) {
                    deferred.resolve();
                } else {
                    this.validarPermissaoAlteracaoDoPerfilAcesso(widget, nrorg).then(function(){
                        deferred.resolve();
                    });
                }     
                
            }.bind(this))
            .catch(function(error) {
                console.log(error);
                deferred.reject();
            });
        }
        
        return deferred.promise;
    };
    

    /**
     * Verifica se a proposta utiliza composição de serviços para bloquear a edição dos escopos.
     * 
     * @param {object} wdgProposta - Widget de Proposta (view)
     */
    this.verificarEscopo = function(wdgProposta) {
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");

        //var utilizaComposicao = wdgPropostaGrid.currentRow.IDUTILIZACOMP;
        var utilizaComposicao = (wdgPropostaGrid.currentRow.QTITENSTRABSERV > 0) ? "S" : "N";
        var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");

        wdgEscoposProposta.getAction("add").isVisible = false;
        wdgEscoposProposta.getAction("delete").isVisible = false;
    };

    /**
     * Muda a visibilidade das ações de crud das widgets filhas de proposta
     */
    this.controlarAcoesDeCrudAux = function(widget, visivel, situacao) {
        var widgetsIgnoradas = ["wdgDocumento", "wdgProposta", "wdgPropostaTabbed"];

        widget.widgets.filter(function(wid) {
                return widgetsIgnoradas.indexOf(wid.name) == -1;
            })
            .forEach(function(wid) {
                this.setVisibilidadeDasAcoesDeCrud(wid, visivel, situacao);
                if (wid.widgets.length > 0) {
                    this.controlarAcoesDeCrudAux(wid, visivel, situacao);
                }
            }.bind(this));
    };

    /**
     * Controla as ações do canto direito da tela de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param string situacao - Situação da Proposta
     */
    this.controlarAcoesDeOperacaoProposta = function(widget, situacao) {
        var actConcluir       = widget.getAction("concluir_proposta");
        var actAceitar        = widget.getAction("aceitar_proposta");
        var actCopiar         = widget.getAction("copiar_proposta");
        var actVersionar      = widget.getAction("versionar_proposta");
        var actCancelar       = widget.getAction("cancelar_proposta");
        var actMemoriaCalculo = widget.getAction("memoria_calculo_proposta");
        /*var actDocProposta = widget.getAction("documento_proposta");
        var actEnviarPorEmail = widget.getAction("enviar_documento");*/

        switch (situacao) {
            case "ELABORACAO":
                {
                    actConcluir.isVisible       = true;
                    actAceitar.isVisible        = false;
                    actCopiar.isVisible         = true;
                    actVersionar.isVisible      = true;
                    actCancelar.isVisible       = true;
                    actMemoriaCalculo.isVisible = true;
                    /*actDocProposta.isVisible  = false;
                    actEnviarPorEmail.isVisible = false;*/
                    break;
                }
            case "CANCELADO":
                {
                   actConcluir.isVisible       = false;
                   actAceitar.isVisible        = false;
                   actCopiar.isVisible         = true;
                   actVersionar.isVisible      = true;
                   actCancelar.isVisible       = false;
                   actMemoriaCalculo.isVisible = false;
                   /* actDocProposta.isVisible = false;
                   actEnviarPorEmail.isVisible = false;*/
                    break;
                }
            case "ACEITO":
                {
                    actConcluir.isVisible       = false;
                    actAceitar.isVisible        = false;
                    actCopiar.isVisible         = true;
                    actVersionar.isVisible      = true;
                    actCancelar.isVisible       = true;
                    actMemoriaCalculo.isVisible = true;
                    /*actDocProposta.isVisible  = true;
                    actEnviarPorEmail.isVisible = true;*/
                    break;
                }
            default:
                { //CONCLUIDO ou ENVIADO
                    actConcluir.isVisible       = false;
                    actAceitar.isVisible        = true;
                    actCopiar.isVisible         = true;
                    actVersionar.isVisible      = true;
                    actCancelar.isVisible       = true;
                    actMemoriaCalculo.isVisible = true;
                    /*actDocProposta.isVisible  = true;
                    actEnviarPorEmail.isVisible = true;*/
                    break;
                }
        }
        // this.validaIdCertsign();
    };

    /**
     * Ativa/desativa todas as actions novo, editar e excluir de uma widget.
     * 
     * @param {object} widget
     * @param {boolean} visivel
     */
    this.setVisibilidadeDasAcoesDeCrud = function(widget, visivel, situacao) {
        var actions               = ["add", "edit", "delete"];
        var wdgsPermiteExclusao   = ["wdgEndereco", "wdgItensProposta"];
        var wdgsPermitemAlteracao = ["wdgEscoposProposta", "wdgElementosProposta", "wdgNegociacaoValores", "wdgEndereco", "wdgItensProposta", "wdgItensPropostaTree"];

        if(situacao === "CONCLUIDO" && wdgsPermitemAlteracao.indexOf(widget.name) !== -1){
            widget.getAction("edit").isVisible = (idAlteraPropConc == 'S' && visibilidadeAcoes);   
            actions = [];

            if (wdgsPermiteExclusao.indexOf(widget.name) !== -1){
                widget.getAction("delete").isVisible = (idAlteraPropConc == 'S' && visibilidadeAcoes);
                widget.getAction("add").isVisible    = (idAlteraPropConc == 'S' && visibilidadeAcoes);
            }
        }
        
        widget.actions.forEach(function(action) {
            if (actions.includes(action.name)) {
                action.isVisible = visivel;
            }
        });
        
    };

    /**
     * Verifica se foi selecionado um Interlocutor para bloquear/liberar o campo Descrição do Interlocutor.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.verificarInterlocutor = function(widget) {
        var fldInterloc = widget.getField("NMPARCINTERLOCUTOR");
        var fldDescInterloc = widget.getField("DSINTERORCA");

        if (fldInterloc.value().length > 0) {
            fldDescInterloc.value("");
            fldDescInterloc.readOnly = true;
        }
        else {
            fldDescInterloc.readOnly = false;
        }
    };

    this.verificarInterlocutorfat = function(widget) {
        var fldInterloc = widget.getField("NMPARCINTERLOCUTORFAT");
        var fldDescInterloc = widget.getField("DSINTERORCA");

        if (fldInterloc.value().length > 0) {
            fldDescInterloc.value("");
            fldDescInterloc.readOnly = true;
        }
        else {
            fldDescInterloc.readOnly = false;
        }
    };

    this.parceiroOnChange = function(wdgProposta) {
        this.filtroInterlocutor(wdgProposta, true);
        this.filtroOportunidade(wdgProposta);
        if (wdgProposta.currentRow.__is_new) {
            if (wdgProposta.currentRow.CDIDIOM_PARC) {
                wdgProposta.currentRow.CDIDIOM = wdgProposta.currentRow.CDIDIOM_PARC;
                wdgProposta.currentRow.NMIDIOMA = wdgProposta.currentRow.NMIDIOMA_PARC;
            }
            else if (cdidiom_padrao) {
                wdgProposta.currentRow.CDIDIOM = cdidiom_padrao;
                wdgProposta.currentRow.NMIDIOMA = nmidioma_padrao;
            }

            if (wdgProposta.currentRow.NRMOEDA_PARC) {
                wdgProposta.currentRow.NRMOEDA = wdgProposta.currentRow.NRMOEDA_PARC;
                wdgProposta.currentRow.NMMOEDA = wdgProposta.currentRow.NMMOEDA_PARC;
                this.moedaOnChange(wdgProposta);
            }
            else if (nrmoeda_padrao) {
                wdgProposta.currentRow.NRMOEDA = nrmoeda_padrao;
                wdgProposta.currentRow.NMMOEDA = nmmoeda_padrao;
                this.moedaOnChange(wdgProposta);
            }
        }
    };

    /**
     * filtra o campo Interlocutor.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.filtroInterlocutor = function(widget, limparCampos) {
        
        var campoInterlocutor = widget.getField("NMPARCINTERLOCUTOR");
        var campoInterlocutorfat = widget.getField("NMPARCINTERLOCUTORFAT"); 
        campoInterlocutorfat.dataSourceFilter[0].value = widget.currentRow.NRPARCNEGOCIO;
        campoInterlocutor.dataSourceFilter[0].value = widget.currentRow.NRPARCNEGOCIO;
        if (limparCampos) {
            widget.currentRow.NMPARCINTERLOCUTOR = null;
            widget.currentRow.DSEMAIL = null;
            widget.currentRow.NRTELEFONE = null;
        }
        
        campoInterlocutor.reload();
        campoInterlocutorfat.reload();
    };

    /**
     * Filtra o campo Oportunidade de Negócio.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.filtroOportunidade = function(widget) {
        var campoOportunidade = widget.getField("NMOPORT");
        campoOportunidade.dataSourceFilter[2].value = widget.currentRow.NRPARCNEGOCIO;
        campoOportunidade.reload();
    };

    /**
     * Salva a proposta.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.salvarProposta = function(wdgProposta) {

        wdgProposta.currentRow.__columns_to_return = ["NRSEQORCAMENTO", "NRORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];

        if (wdgProposta.isValid()) {

            var mudouParceiro = false;
            if (wdgProposta.backupRow && wdgProposta.backupRow.NRPARCNEGOCIO !== wdgProposta.currentRow.NRPARCNEGOCIO) {
                mudouParceiro = true;
            }
            
            var mudouInterlocutor = false;
            if (wdgProposta.backupRow && wdgProposta.backupRow.NRSEQINTERLOCUTOR !== wdgProposta.currentRow.NRSEQINTERLOCUTOR) {
                mudouInterlocutor = true;
                var nrinterlocutorAnte =  wdgProposta.backupRow.NRSEQINTERLOCUTOR;
            }

            var mudouInterlocutorfat = false;
            if (wdgProposta.backupRow && wdgProposta.backupRow.NRSEQINTERLOCUTORFAT !== wdgProposta.currentRow.NRSEQINTERLOCUTORFAT) {
                mudouInterlocutorfat = true;
            }
            
            this.salvarWidgetProposta(wdgProposta).then(function(response) {

                wdgProposta.view();
                templateManager.container.getWidget('wdgTabbedPrincipal').view();
                wdgProposta.template = "widget/form-view.html";
                this.alterarAcessibilidadeAbasProposta(true);

                var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                wdgPropostaGrid.reload();

                wdgProposta.currentRow.changed = false;
                delete wdgProposta.backupRow;

                if (wdgProposta.currentRow.__is_new) {
                    wdgProposta.currentRow.__is_new = false;
                    this.inserirAssinaturasOrca(wdgProposta);
                    this.voltarParaPropostaGrid();
                    
                    var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");
                    wdgEscoposProposta.isVisible = true;
                } else {
                    if (response.dataset && response.dataset.columnsToReturn[0]) {
                        wdgProposta.currentRow.NRSEQORCAMENTO = response.dataset.columnsToReturn[0].NRSEQORCAMENTO;
                        wdgProposta.currentRow.NRORCAMENTO = response.dataset.columnsToReturn[0].NRORCAMENTO;
                        wdgProposta.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                        wdgProposta.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                        wdgProposta.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        wdgProposta.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                    }

                    this.manipularCamposModeloProposta(wdgProposta, false);
                    //this.desfiltroEscopo(wdgProposta);
                    this.verificarObrigatoriedadeOportunidade(wdgProposta);

                    wdgProposta.reload();

                    if (mudouParceiro) {
                        var params = {
                            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
                            "NRPARCNEGOCIO": wdgProposta.currentRow.NRPARCNEGOCIO
                        };
                        General.callRestEngine("product#/fct_orcamento_widget/atualizarParceiroNegociacoesProposta", "Row", params)
                        .then(null)
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                    
                    if (mudouInterlocutor) {
                        var paramsInt = {
                            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
                            "NRSEQINTERLOCUTOR": wdgProposta.currentRow.NRSEQINTERLOCUTOR,
                            "NRINTERLOCUTORANTE": nrinterlocutorAnte,
                            "NRORCAMENTOMODELO": wdgProposta.currentRow.NRORCAMENTOMODELO,
                        };
                        General.callRestEngine("product#/fct_orcamento_widget/insereInterlocutor", "Row", paramsInt)
                        .then(null)
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }

                    if (mudouInterlocutorfat) {
                        var paramsInt = {
                            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
                            "NRPARCINTERLOCUTORFAT": wdgProposta.currentRow.NRSEQINTERLOCUTORFAT,
                            "NRINTERLOCUTORANTE": nrinterlocutorAnte,
                            "NRORCAMENTOMODELO": wdgProposta.currentRow.NRORCAMENTOMODELO,
                        };
                        General.callRestEngine("product#/fct_orcamento_widget/insereInterlocutor", "Row", paramsInt)
                        .then(null)
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                    wdgProposta.reload();
                }
                    
            }.bind(this),
            function(error) {
                console.error(error);
                var errorMessage = 'Timeout';
                if (error.data) errorMessage = error.data.error;
                ScreenService.showMessage(errorMessage);
            });
        } else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };
    
    /**
    * Verifica se a proposta está ligada a uma oportunidade
    * de negócio
    **/
    this.verificaIntegracaoPropOpor = function(rowProposta){
        var params = [
            {
                name: "NRORCAMENTOMODELO",
                operator: "=",
                value: rowProposta.NRORCAMENTOMODELO
            }
        ];
        
        General.callRestEngine("product#/crud/fct_orcamentomodelo_widget", "FilterData", params).then(function(response){
            if(response.dataset.fct_orcamentomodelo_widget[0] && response.dataset.fct_orcamentomodelo_widget[0]["IDOBRIGAOPORTUNIDADE"] === "S"){
                ScreenService.confirmMessage("Deseja gerar uma evolução na oportunidade de negócio vinculada à proposta?", "question", function() {
                    var paramsOrcamento = [
                        {
                            name: "NRSEQORCAMENTO",
                            operator: "=",
                            value: rowProposta.NRSEQORCAMENTO
                        }
                    ];
                            
                    General.callRestEngine("product#/fct_orcamento_widget", "FilterData", paramsOrcamento).then(function(success) {
                        if(success.dataset.fct_orcamento_widget[0]){
                            var paramsService = [
                                {
                                    name: "NRORG",
                                    operator: "=",
                                    value: success.dataset.fct_orcamento_widget[0].NRORG
                                }
                            ];
                            
                            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", paramsService).then(function(response){
                                
                                if(response.dataset.fct_parametrogeral[0] && response.dataset.fct_parametrogeral[0].IDINTEGRACAOCRM  === "S" && response.dataset.fct_parametrogeral[0].URLINTEGRACAOCRM !== null){
                                    this.abrirWdgEvoluirCRM(success.dataset.fct_orcamento_widget[0]);
                                } else {
                                    ScreenService.showMessage("Não há parametrizações cadastradas para a integração com o CRM.");
                                }
                                
                            }.bind(this))
                            .catch(function(error){
                                console.error(error);
                            });
                        }
                        
                    }.bind(this))
                    .catch(function(error){
                        console.error(error);
                    });       
                }.bind(this));
            }
        }.bind(this));
    };
    
    /**
     * Função disparada ao modificar o campo Modelo de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.modeloOnChange = function(widget) {
        this.verificarModeloProposta(widget);
        this.calcularDataValidadeProposta(widget);
        this.controlaCodProposta(widget);
        this.escoposOnChange(widget);
        this.controlaEscopoModelo(widget);
        this.validaIdCertsign();
        this.assinaturaEletronicaOnChange(widget);
        this.habilitaCampoRevenda();
    };
    
    /**
     * Controla a field de escopo de acordo com o modelo de orcamento
     * escolhido. 
     * @param {Object} widget widget propostas
    **/
    this.controlaEscopoModelo = function(widget){
        var nmFieldEscopo = (widget.currentRow.IDUTILIZACOMP == "S") ? "NRTIPOORCAMENTO" : "NMTIPOORCAMENTO";
        var fieldEscopo   = widget.getField(nmFieldEscopo);
        var fieldIsArray  = (fieldEscopo.name === "NRTIPOORCAMENTO");
        
        fieldEscopo.reload().then(function(){
            if(fieldEscopo.dataSource.data.length > 1){
                fieldEscopo.readOnly = false;
                widget.currentRow.NRTIPOORCAMENTO = fieldIsArray ? [] : "";
                widget.currentRow.NMTIPOORCAMENTO = fieldIsArray ? [] : "";
                fieldEscopo.dataSource.clearCheckedRows();
            } else {
                if(fieldEscopo.dataSource.data.length == 1){
                    fieldEscopo.readOnly = true;
                    var nrtipoorcamento = fieldEscopo.dataSource.data[0].NRTIPOORCAMENTO;
                    var nmtipoorcamento = fieldEscopo.dataSource.data[0].NMTIPOORCAMENTO;
                    var utilizavariacao = fieldEscopo.dataSource.data[0].IDUTILIZAVARPRECO;
                    
                    widget.currentRow.NRTIPOORCAMENTO   = fieldIsArray ? [nrtipoorcamento] : nrtipoorcamento;
                    widget.currentRow.NMTIPOORCAMENTO   = fieldIsArray ? [nmtipoorcamento] : nmtipoorcamento;
                    widget.currentRow.IDUTILIZAVARPRECO = fieldIsArray ? [utilizavariacao] : utilizavariacao;
                    
                    fieldEscopo.dataSource.addCheckedRows(fieldEscopo.dataSource.data[0]);
                    fieldEscopo.dataSource.data[0].__isSelected = true;
                }
            }
            UtilitiesPropostaContrato.validarParamsFaturamentoEscopo(widget, nmFieldEscopo, "ORC");
        });
    };

    /**
     * Controla campo "Código" da proposta
     */
    this.controlaCodProposta = function(widget) {
        var modProposta = widget.currentRow.NMORCAMENTOMODELO;
        var fieldCodigo = widget.getField("NRORCAMENTO");

        if (modProposta && widget.currentRow.__is_new) {
            fieldCodigo.readOnly = true;
            widget.currentRow.NRORCAMENTO = "";
            fieldCodigo.validations = {};
        }
        else {
            fieldCodigo.readOnly = false;
            fieldCodigo.validations = {
                "required": true
            };
        }
    };

    /**
     * Ao carregar um modelo, é verificado se seus dados não poderão ser alterados e se será
     * obrigatório informar uma oportunidade de negócio.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.verificarModeloProposta = function(widget) {
        var idbloqalteradados = widget.currentRow.IDBLOQALTERADADOS;
        var readOnly = (idbloqalteradados == "NAO") ? true : false;
        this.manipularCamposModeloProposta(widget, readOnly);

        var utilizaComposicao = (widget.currentRow.IDUTILIZACOMP == "S");
        this.controlarEscopoProposta(widget, utilizaComposicao);
        this.controlarWidgetItens(widget, utilizaComposicao);

        this.verificarObrigatoriedadeOportunidade(widget);
    };

    /**
     * Controla o campo de Escopo de Proposta de acordo com o Modelo (se utiliza composição de serviços ou não).
     * 
     * @param {object} widget - Widget de Proposta
     * @param {boolean} utilizaComposicao
     */
    this.controlarEscopoProposta = function(widget, utilizaComposicao, limparCampos) {
        if (limparCampos == undefined) {
            limparCampos = true;
        }

        var fieldCodEscopo    = widget.getField("NRTIPOORCAMENTO");
        var fieldNomeEscopo   = widget.getField("NMTIPOORCAMENTO");
        var fieldUtilVariacao = widget.getField("IDUTILIZAVARPRECO");
        fieldNomeEscopo.readOnly   = false;
        fieldCodEscopo.readOnly    = false;
        this.filtroEscopo(widget, utilizaComposicao);
        if (utilizaComposicao) {
            fieldNomeEscopo.isVisible  = false;
            //fieldNomeEscopo.showOnView = false;
            fieldNomeEscopo.showOnForm = false;
            fieldNomeEscopo.template   = "field/text-edit.html";
            
            fieldCodEscopo.template    = "field/select-multiple.html";
            fieldCodEscopo.validations = {
                "required": true
            };
            fieldCodEscopo.isVisible  = true;
            //fieldCodEscopo.showOnView = true;
            fieldCodEscopo.showOnForm = true;
            fieldCodEscopo.reload().then(function() {
                if (fieldCodEscopo.dataSource.data.length == 1) {
                    fieldCodEscopo.value([fieldCodEscopo.dataSource.data[0].NRTIPOORCAMENTO]);
                    fieldNomeEscopo.value([fieldCodEscopo.dataSource.data[0].NMTIPOORCAMENTO]);
                    fieldUtilVariacao.value([fieldCodEscopo.dataSource.data[0].IDUTILIZAVARPRECO]);
                }
            });
        } else {
            fieldCodEscopo.isVisible = false;
            //fieldCodEscopo.showOnView = false;
            fieldCodEscopo.showOnForm = false;
            fieldCodEscopo.template = "field/text-edit.html";
            fieldCodEscopo.validations = {};

            fieldNomeEscopo.template = "zh-select-autocomplete#field/select-autocomplete.html";
            fieldNomeEscopo.isVisible = true;
            //fieldNomeEscopo.showOnView = true;
            fieldNomeEscopo.showOnForm = true;
        }

        if (limparCampos) {
            fieldNomeEscopo.value("");
            fieldCodEscopo.value("");
        }
    };

    /**
     * filtra o campo Escopo de Proposta quando tem Modelo.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.filtroEscopo = function(widget, utilizaComposicao) {
        var fieldEscopo       = utilizaComposicao ? widget.getField("NRTIPOORCAMENTO") : widget.getField("NMTIPOORCAMENTO");
        var nrorcamentomodelo = widget.currentRow.NRORCAMENTOMODELO;
        if(fieldEscopo.name == "NRTIPOORCAMENTO"){
            fieldEscopo.dataSourceFilter[1].value = nrorcamentomodelo;
        } else {
            if(!nrorcamentomodelo){
                fieldEscopo.dataSource.name = "product#/crud/fct_tipoorcamento_widget";
                fieldEscopo.dataSourceFilter = [fieldEscopo.dataSourceFilter[0]];
            } else {
                fieldEscopo.dataSource.name = "product#/fct_relacescopomodelo_widget";
                fieldEscopo.dataSourceFilter[1] = {
                    name: "NRORCAMENTOMODELO",
                    operator: "=",
                    value: nrorcamentomodelo
                };
            }
        }
        fieldEscopo.reload();
    };

    /**
     * filtra o campo Escopo de Proposta quando tem Modelo.
     * 
     * @param {object} widget - Widget de Proposta
     */
    /*this.desfiltroEscopo = function(widget) {
        var fieldEscopo = widget.getField("NRTIPOORCAMENTO");
        fieldEscopo.dataSource.name = "product#/crud/fct_tipoorcamento_widget";
        delete fieldEscopo.dataSourceFilter[1];
        fieldEscopo.reload();
    };*/

    /**
     * Escolhe qual widget de Itens irá aparecer de acordo com o parâmetro
     * "Utiliza composição de serviços" do Modelo de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param {boolean} utilizaComposicao
     */
    this.controlarWidgetItens = function(widget, utilizaComposicao) {
        if (utilizaComposicao) {
            widget.getAction("save").isVisible = false;
            widget.getAction("avancar_composicao").isVisible = true;
        }
        else {
            widget.getAction("avancar_composicao").isVisible = false;
            widget.getAction("save").isVisible = true;
        }
    };
    
    /**
     * Valida se o valor alterado na quantidade de medida
     * continua coerente com os demais
     * @params{Object} widget - widget itens proposta
    **/
    this.validarEdicoesInline = function(widget, elementos){
        var deferred = ZHPromise.defer();
        elementos.forEach(function(elemento){
            var quantidade         = elemento.QTITEMORCAMENTO;
            var utilizaQtdeMedida  = elemento.IDMULTMEDVRTOT;
            var qtdeMedida         = elemento.QTMEDIDAORCA_GRID;
            var valorUnitario      = elemento.VRUNITITEMORCA;
            var tipoAjuste         = elemento.IDTPDESCORCA;
            var ajuste             = elemento.VRPERCDESCORCA;
            var qtdeVariacao       = elemento.QTMEDIDAVARIAVEL;
            var multiplicaVariacao = (elemento.IDMULTIPVARIACAO == "S");
            elemento.QTMEDIDAORCA  = elemento.QTMEDIDAORCA_GRID;

            if (qtdeVariacao == undefined || qtdeVariacao == "")
                qtdeVariacao = 1;
        
            if (elemento.QTMEDIDAORCA) {
                if (elemento.IDALTQTDMEDIDAPROP == "4") { //nao altera
                    elemento.QTMEDIDAORCA             = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_inline      = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID        = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID_inline = elemento.QTMEDIDACALC;
                    return deferred.reject("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida do elemento: <br>" + elemento._parent["NMITEMORCAMENTO"] + " -> " + elemento["NMITEMORCAMENTO"]); 
                } else if (elemento.IDALTQTDMEDIDAPROP == "3" && elemento.QTMEDIDAORCA > elemento.QTMEDIDACALC) { //somente reduzir
                    elemento.QTMEDIDAORCA             = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_inline      = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID        = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID_inline = elemento.QTMEDIDACALC;
                    return deferred.reject("Operação bloqueada.<br/>Não é permitido aumentar a Quantidade de Medida do item <br>"+ elemento._parent["NMITEMORCAMENTO"] + " -> " +elemento["NMITEMORCAMENTO"]+". O campo foi atribuído com o valor máximo (" + elemento.QTMEDIDACALC.formatFloat(2) + ").");
                } else if (elemento.IDALTQTDMEDIDAPROP == "2" && elemento.QTMEDIDAORCA < elemento.QTMEDIDACALC) { //somente aumentar
                    elemento.QTMEDIDAORCA             = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_inline      = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID        = elemento.QTMEDIDACALC;
                    elemento.QTMEDIDAORCA_GRID_inline = elemento.QTMEDIDACALC;
                    return deferred.reject("Operação bloqueada.<br/>Não é permitido reduzir a Quantidade de Medida do item<br> "+ elemento._parent["NMITEMORCAMENTO"] + " -> " +elemento["NMITEMORCAMENTO"]+".<br> O campo foi atribuído com o valor mínimo (" + elemento.QTMEDIDACALC.formatFloat(2) + ").");
                }
            }

            if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
            if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
            if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
            if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
            var valorSemAjuste = quantidade * valorUnitario;
            if (utilizaQtdeMedida == "S") 
                valorSemAjuste *= qtdeMedida;
            if (multiplicaVariacao) 
                valorSemAjuste *= qtdeVariacao;

            var valorTotal = valorSemAjuste;
            if (tipoAjuste == "V") {
                valorTotal -= ajuste;
            } else if (tipoAjuste == "P") {
                valorTotal -= valorTotal * ajuste / 100;
            } else if (tipoAjuste == "X") {
                valorTotal += ajuste;
            } else if (tipoAjuste == "Y") {
                valorTotal += valorTotal * ajuste / 100;
            }
    
            if(valorTotal < 0)
                return deferred.reject("Operação bloqueada!<br>Não foi possível atualizar o valor da quantidade de medida do elemento, pois o novo valor total não está coerente com o ajuste do mesmo.");
            else 
                return deferred.resolve();
        }.bind(this));
        return deferred.promise;
    };
    
    this.salvarItensInline = function(widget) {
        var itensTipoI = widget.dataSource.data.filter(function(row) {
            return row.IDTIPO == "I" && row.changed == true;
        });
        var itensTipoM = widget.dataSource.data.filter(function(row) {
            return row.IDTIPO == "M" && row.changed == true;
        });
        
        this.validarEdicoesInline(widget, itensTipoM).then(function(){
            var params = {
                "ELEMENTOS": itensTipoM
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/salvarElementosAlteradosPeloUsuario", "Row", params).then(function() {
                params = {};
                if(itensTipoM.length === 0){
                    return General.callRestEngine(widget.dataSource.name + "/save", "DataSet", itensTipoI);
                } else {
                    widget.reload().then(function(){
                        var eleTipoI = [];
                        itensTipoM.forEach(function(elemento){
                            if(elemento.IDARVOREPAI != null)
                                eleTipoI.push(elemento.IDARVOREPAI);
                        });
                        eleTipoI.forEach(function(elementPai){
                            var newValue = 0;
                            var itemPaiAlterado = {};
                            widget.dataSource.data.forEach(function(item){
                                itemPaiAlterado = (item.IDARVORE !== elementPai) ? itemPaiAlterado : item;
                                if(item.IDARVOREPAI == elementPai)
                                    newValue += item.VRTOTALITEMORCA;
                            });
                            var index = itensTipoI.indexOf(itemPaiAlterado);
                            itemPaiAlterado.VRUNITITEMORCA  = newValue;
                            itemPaiAlterado.VRTOTALITEMORCA = newValue;
                            if(index === -1)
                                itensTipoI.push(itemPaiAlterado);
                            else 
                                itensTipoI[index] = itemPaiAlterado;
                        });
                        General.callRestEngine(widget.dataSource.name + "/save", "DataSet", itensTipoI).then(function(){
                            return widget.reload();
                        });
                    });
                }
            }).then(function() {
                widget.list();
                widget.reload();
                var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                this.atualizarValorProposta(wdgPropostaGrid);
            }.bind(this))
            .catch(function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        }.bind(this)).catch(function(error){
            ScreenService.showMessage(error);
        });
    };
    
    /**
     * Atualiza o ajuste dos elementos da arvore de composição de serviço
     * @params{Object} widget widget itens
    **/
    this.replicaAteracaoParaItemPai = function(widget){    
        var row = {
            "NRSEQORCAMENTO"  : widget.currentRow.NRSEQORCAMENTO,
            "CDSERVORCA"      : widget.currentRow.CDSERVORCA
        };
        
        return General.callRestEngine("product#/fct_itemorcamento_widget/replicaAteracaoParaItemPai", "Row", row);
    };
    
    /**
     * Realiza a s validaçoes antes de salvar um registro
     * 
     * @param {object} widget widget de 
     * Item de Proposta para salvar o novo registro.
     */
    this.validaPercentualSaveItens = function(widget) {
        if (widget.isValid()) {
            var vrunititemorcatblpreco = widget.currentRow.VRUNITITEMORCATBLPRECO;
            var vrunititemorca         = widget.currentRow.VRUNITITEMORCA;
            
            vrunititemorcatblpreco = Utilities.tirarMascaraValores(vrunititemorcatblpreco);
            vrunititemorca         = Utilities.tirarMascaraValores(vrunititemorca);

            if (vrunititemorca > vrunititemorcatblpreco) {
                var nritempai = widget.currentRow.NRITEMORCAMENTOPAI;
                var percAcrescimoServico = 0;
                widget.dataSource.data.filter(function(row) {
                    return row.NRITEMORCAMENTOPAI == nritempai;
                }).forEach(function(row) {
                    var percentual = (Utilities.tirarMascaraValores(row.VRUNITITEMORCA) - Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO)) / Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO) * 100;
                    percAcrescimoServico += percentual;
                });

                var params = {
                    "CDSERVORCA" : widget.currentRow.CDSERVORCA,
                    "IDTIPO"     : "A"
                };
                
                General.callRestEngine("product#/fct_itemorcamento_widget/validaPorcentualDescontoAcrescimoSave", "Row", params).then(function(response) {
                    var resultado = response.dataset.RESULTADO.VRPERCDESC;
                    if (resultado && percAcrescimoServico > resultado.VRPERCDESC) {
                        var contexto = (resultado.TIPO == 1) ? "Este": "O perfil de acesso deste";
                        ScreenService.showMessage("Operação bloqueada.<br/>"+contexto+" operador não tem autorização para propor acréscimo superior a " + resultado.VRPERCDESC + "%.");
                    } else {
                        // this.validarDadosRateioItens(widget); Por hora, não está sendo utilizado rateio   caso necessite voltar, deve ser descomentado e retirado a linha abaixo.
                        this.salvarItens(widget);
                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            } else {
                // this.validarDadosRateioItens(widget); Por hora, não está sendo utilizado rateio   caso necessite voltar, deve ser descomentado e retirado a linha abaixo.
                this.salvarItens(widget);
            }
        }
        else {
            ScreenService.showMessage("Preencha todos os campos corretamente!");
        }
    };

    /**var nritempai = widget.currentRow.NRITEMORCAMENTOPAI;
            var percAcrescimoServico = 0;
            widget.dataSource.data.filter(function(row) {
                return row.NRITEMORCAMENTOPAI == nritempai;
            })
            .forEach(function(row) {
                var percentual = (Utilities.tirarMascaraValores(row.VRUNITITEMORCA) - Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO)) / Utilities.tirarMascaraValores(row.VRUNITITEMORCATBLPRECO) * 100;
                percAcrescimoServico += percentual;
            });

            var params = {
                "CDSERVORCA": widget.currentRow.CDSERVORCA,
                "IDTIPO": "A"
            };
            General.callRestEngine("product#/fct_itemorcamento_widget/validaPorcentualDescontoAcrescimoSave", "Row", params)
     * Converte o valor de ajuste para valor percentual
     * 
     * @param {object} widget widget de 
     * Item de Proposta para salvar o novo registro.
     */
    this.converteParaPercentual = function(widget) {
        var percentual        = widget.currentRow.VRPERCDESCORCA;
        var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida        = widget.currentRow.QTMEDIDAORCA;
        var valorSemAjuste    = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemAjuste *= qtdeMedida;
        }

        var valorTotal = valorSemAjuste;

        if (tipoAjuste == "V" || tipoAjuste == "X") {
            return (percentual / valorTotal) * 100;
        }
        else
            return percentual;
    };

    /**
     * Salva um item ou um adicional.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.salvarItens = function(widget){
        var deferred         = ZHPromise.defer();  
        var wdgProposta      = getWidgetProposta();
        var isnew            = widget.currentRow.__is_new;
        var nrseqorcamento   = widget.currentRow.NRSEQORCAMENTO;
        var quantidade       = widget.currentRow.QTITEMORCAMENTO;
        var quantidadeAntiga = quantidade;

        //widget.currentRow.NRMOEDA = wdgProposta.currentRow.NRMOEDA;
        
        if (widget.backupRow) {
            quantidadeAntiga = widget.backupRow.QTITEMORCAMENTO;
        }
        
        if (widget.currentRow.TIPOPRODSERV == 'S') 
            widget.currentRow.CDSERVORCA = widget.currentRow.CDPRODSERV;
        else 
            widget.currentRow.CDPRODUTO = widget.currentRow.CDPRODSERV;
        
        var idsituacao = wdgProposta.currentRow.IDSITUACAOORCAMENTO;
        widget.currentRow.__columns_to_return = ["NRITEMORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];
        
        if(widget.currentRow.NMDOCITEMORCAMENTO && widget.currentRow.NMDOCITEMORCAMENTO !== widget.currentRow.NMITEMORCAMENTO){
            widget.currentRow.NMITEMORCAMENTO = widget.currentRow.NMDOCITEMORCAMENTO;
        }
        
        if(widget.currentRow.IDTIPO === "I" && !widget.currentRow.VRUNITITEMORCA)
            widget.currentRow.VRUNITITEMORCA = widget.currentRow.VRTOTALITEMORCA;
            
        if(widget.currentRow.VRUNITITEMORCA) widget.currentRow.VRUNITITEMORCA = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMORCA, 8);
        if(widget.currentRow.QTMEDIDAORCA)   widget.currentRow.QTMEDIDAORCA   = Utilities.tirarMascaraValores(widget.currentRow.QTMEDIDAORCA);
        if(widget.currentRow.VRPERCDESCORCA) widget.currentRow.VRPERCDESCORCA = Utilities.tirarMascaraValores(widget.currentRow.VRPERCDESCORCA);
        
        
        this.criarParametrosNewCode(widget);
        General.widgetSave(widget).then(function(success) {
            widget.currentRow.QTMEDIDAORCA_GRID = widget.currentRow.QTMEDIDAORCA;

            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRITEMORCAMENTO = success.dataset.columnsToReturn[0].NRITEMORCAMENTO;
                widget.currentRow.NRORG           = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO      = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO  = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }

            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            
            if (success.dataset.retorno) {
                wdgPropostaGrid.currentRow.VRORCAMENTO = success.dataset.retorno.valorProposta;
                wdgProposta.currentRow.VRORCAMENTO = success.dataset.retorno.valorProposta;
            }
            
            //if (wdgProposta.currentRow.IDUTILIZACOMP === 'S') {
            if (wdgProposta.currentRow.QTITENSTRABSERV > 0) {
                if (widget.currentRow.IDTIPO === "I") {
                    widget.currentRow.VRUNITITEMORCA = "";
                }
                else {
                    this.replicaAteracaoParaItemPai(widget).then(function() {
                        this.atualizarValorItensNegociacaoValores(widget.currentRow).then(function() {
                            this.atualizarValorProposta(wdgPropostaGrid);
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }.bind(this));
                    
                    General.callRestEngine("product#/fct_orcamento_widget/atualizarMedidaVariavelServico", "Row", widget.currentRow).then(null)
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }
            else {
                this.atualizarValoresNegociacaoAoEditarItemSemComposicao(widget.currentRow).then(function() {
                    this.atualizarValorProposta(wdgPropostaGrid);
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            
            // if(["X", "V"].includes(widget.currentRow.IDTPDESCORCA)) 
            //     this.calcularValorTotalItem(widget);

            widget.currentRow.changed = false;
            widget.widgets.forEach(function(wdg){
                wdg.enabled = true;
            });
            
            /*if(!isnew){
                widget.view();
            } else {
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }*/
            
            if(idsituacao === "CONCLUIDO" && isnew) {
                this.insereItensAcaoPreventiva(nrseqorcamento, widget.currentRow.NRITEMORCAMENTO, quantidade, idsituacao);
            } else {
                if (!isnew && quantidade !== quantidadeAntiga) {
                    var params = {
                        "NRSEQORCAMENTO" : nrseqorcamento,
                        "NRITEMORCAMENTO": widget.currentRow.NRITEMORCAMENTO,
                        "QTPENDENCIA"    : quantidade
                    };
                    
                    General.callRestEngine("product#/fct_itemorcamento_widget/atualizaQuantidadeItensAgendPendente", "Row", params)
                    .then(null)
                    .catch(function(error){
                        ScreenService.showMessage(error.data.error);
                    });
                }
            }
            
            deferred.resolve();
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            deferred.reject();
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
        
        return deferred.promise;
    };

    this.atualizarValoresNegociacaoAoEditarItemSemComposicao = function(row) {
        var params = {
            "NRITEMORCAMENTO": row.NRITEMORCAMENTO,
            "VRTOTALITEMORCA": row.VRTOTALITEMORCA
        };

        return General.callRestEngine("product#/fct_orcamento_widget/atualizarValoresNegociacaoAoEditarItemSemComposicao", "Row", params);
    };

    this.excluirItem = function(widget) {
        var widgetProposta  = templateManager.container.getWidget("wdgProposta");
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
        var params = {
            "NRSEQORCAMENTO"  : widgetProposta.currentRow.NRSEQORCAMENTO,
            "NRITEMORCAMENTO" : widget.currentRow.NRITEMORCAMENTO
        };

        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            General.callRestEngine("product#/fct_itemorcamento_widget/DeletaItemOrcamentoConcluido", "Row", widget.currentRow).then(function(){
                General.callRestEngine("product#/fct_relacestruturaprop_widget/deletaAlteracoesInutilizadas", "Row", params).then(function(){
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        widget.reload();
                        this.atualizarValorProposta(wdgPropostaGrid); 
                    }.bind(this));
                }.bind(this))
                .catch(function(error){
                    console.error(error);
                });
            }.bind(this))
            .catch(function(error){
                ScreenService.showMessage(error.data.error);
            });
            
        }.bind(this));
    };

    /**
     * Atualiza o valor total da proposta de acordo com a soma dos itens.
     * 
     * @param {object} wdgPropostaGrid - Widget de Proposta (grid)
     */
    this.atualizarValorProposta = function(wdgPropostaGrid) {
        var params = {
            "NRSEQORCAMENTO": wdgPropostaGrid.currentRow.NRSEQORCAMENTO,
            "NRORG": wdgPropostaGrid.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamento_widget/atualizarValorProposta", "Row", params).then(function(success) {
                var valorProposta = success.dataset.FCT_ORCAMENTO.VRORCAMENTO;
                var wdgProposta = templateManager.container.getWidget("wdgProposta");

                wdgPropostaGrid.currentRow.VRORCAMENTO = valorProposta;
                wdgProposta.currentRow.VRORCAMENTO = valorProposta;
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget - Widget de Itens para pegar o field "IDMULTMEDVRTOT".
     */
    this.bloquearDadosMedida = function(widget) {
        var fieldUsaQtdMedida = widget.getField("IDMULTMEDVRTOT");
        
        if(!fieldUsaQtdMedida)
            fieldUsaQtdMedida = widget.getField("IDUSAMEDIDA");
            
        if (fieldUsaQtdMedida.readOnly == false) {
            /* Bloqueia a Modalidade de pagamento*/
            if (fieldUsaQtdMedida.value() == 'N') {
                /* Medida */
                widget.getField('DSTPMEDIDASERV').readOnly = true;
                widget.getField('DSTPMEDIDASERV').value(null);
                widget.getField('DSTPMEDIDASERV').validations = {};

                widget.getField('CDTPMEDIDASERV').value(null);
                widget.getField('CDTPMEDIDASERV').validations = {};

                /* Quantidade de Medida */
                widget.getField('QTMEDIDAORCA').readOnly = true;
                widget.getField('QTMEDIDAORCA').value(null);
                widget.getField('QTMEDIDAORCA').validations = {};

            } else {
            /* Desbloqueia a Medida*/

                /* Medida */
                widget.getField('DSTPMEDIDASERV').readOnly = false;
                widget.getField('DSTPMEDIDASERV').validations = { "required": true };

                /* Quantidade de Medida */
                widget.getField('QTMEDIDAORCA').readOnly = false;
                widget.getField('QTMEDIDAORCA').validations = { "required": true };
            }
        }
    };


    /**
     * Realiza o controle dos campos de "Dados do item" de acordo para bloquear 
     * os mesmos caso a proposta seja criada à prtir de uma composição.
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDUTILIZACOMP".
     */
    /*this.bloquearDadosItemComposicao = function(widget) {
        var widgetPropostaGrid = getWidgetPropostaGrid();
        var usaComposicao = widgetPropostaGrid.currentRow.IDUTILIZACOMP;

        // Bloqueia a Modalidade de pagamento
        if (usaComposicao == 'S') {
            // Item 
            widget.getField('NMITEMORCAMENTO').readOnly = true;

            // Quantidade  
            widget.getField('QTITEMORCAMENTO').readOnly = true;

            // Utiliza Quantidade de Medida? 
            widget.getField('IDMULTMEDVRTOT').readOnly = true;

            // Medida  
            widget.getField('DSTPMEDIDASERV').readOnly = true;

            // Quantidade de Medida 
            widget.getField('QTMEDIDAORCA').readOnly = true;

            // Valor Unitário  
            widget.getField('VRUNITITEMORCA').readOnly = true;

            // Tipo de Desconto  
            widget.getField('IDTPDESCORCA').readOnly = true;

            // Desconto  
            widget.getField('VRPERCDESCORCA').readOnly = true;
        }
        else {
            // Item 
            widget.getField('NMITEMORCAMENTO').readOnly = false;

            // Quantidade  
            widget.getField('QTITEMORCAMENTO').readOnly = false;

            // Utiliza Quantidade de Medida? 
            widget.getField('IDMULTMEDVRTOT').readOnly = false;

            // Medida  
            widget.getField('DSTPMEDIDASERV').readOnly = false;

            // Quantidade de Medida 
            widget.getField('QTMEDIDAORCA').readOnly = false;

            // Valor Unitário  
            widget.getField('VRUNITITEMORCA').readOnly = false;

            // Tipo de Desconto  
            widget.getField('IDTPDESCORCA').readOnly = false;

            // Desconto  
            widget.getField('VRPERCDESCORCA').readOnly = false;
        }
    };*/

    /**
     * Habilita/desabilita os campos de saída do Modelo de Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param {boolean} readOnly
     */
    this.manipularCamposModeloProposta = function(widget, readOnly) {
        fieldsModelo.forEach(function(field) {
            widget.getField(field).readOnly = readOnly;
        });
    };

    /**
     * Verifica a obrigatoriedade do campo Oportunidade de Negócio de acordo com o modelo de proposta selecionado.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.verificarObrigatoriedadeOportunidade = function(widget) {
        var idobrigaoportunidade = widget.currentRow.IDOBRIGAOPORTUNIDADE;

        var validations = {};
        if (idobrigaoportunidade == "S")
            validations = {
                "required": true
            };

        var fieldOportunidade = widget.getField("NMOPORT");
        fieldOportunidade.validations = validations;
    };

    /**
     * Coloca a tela de Proposta em modo de edição.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.prepararEdicaoProposta = function(wdgProposta) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

        wdgProposta.backupRow = Util.clone(wdgProposta.currentRow, ignoredKeys);
        wdgProposta.edit();
        wdgTabbedPrincipal.edit();

        wdgProposta.template = "widget/form.html";
        //this.bloquearModalidadePagamento(wdgProposta);
        this.filtroInterlocutor(wdgProposta);
        this.filtroOportunidade(wdgProposta);
        //var utilizaComposicao = (wdgProposta.currentRow.IDUTILIZACOMP == "S");
        var utilizaComposicao = (wdgProposta.currentRow.QTITENSTRABSERV > 0);
        this.controlarEscopoProposta(wdgProposta, utilizaComposicao, false);
        wdgProposta.getAction("avancar_composicao").isVisible = false;
        wdgProposta.getAction("save").isVisible = true;

        this.alterarAcessibilidadeAbasProposta(false);
        this.controlaCodProposta(wdgProposta);
        this.assinaturaEletronicaOnChange(wdgProposta);
        mudouMoeda = false;

        var readOnly = wdgProposta.currentRow.IDSITUACAOORCAMENTO !== "ELABORACAO";
        wdgProposta.getField("NMIDIOMA").readOnly = readOnly;
        wdgProposta.getField("NMMOEDA").readOnly = readOnly;
    };

    /**
     * Coloca a tela de Itens ou de Adicionais em modo de edição.
     * 
     * @param {object} widget - Widget de Itens ou de Adicionais
     */
    this.prepararEdicaoItens = function(widget) {
        General.widgetEdit(widget);
        if(templateManager.container.getWidget("wdgProposta").currentRow.QTITENSTRABSERV > 0) {
            this.validarAbreSwipeItens(widget, "W");
            var params = {
                "NRTIPOORCAMENTO": widget.currentRow.NRTIPOORCAMENTO,
                "CDSERVORCA"     : widget.currentRow.CDSERVORCA
            };

            General.callRestEngine("product#/fct_itemorcamento_widget/getMedidaVariavelEscopo", "Row", params).then(function(response) {
                var fieldVrunitario = widget.getField("VRUNITITEMORCA");
                if(response.dataset && response.dataset.retorno) {
                    fieldVrunitario.readOnly = true;
                    fieldVrunitario.help     = "O campo foi desabilitado porque não é possível inserir acréscimos em itens vinculados a serviços faturados por medição.";
                } else {
                    fieldVrunitario.readOnly = false;
                    fieldVrunitario.help     = null;
                }
            });
        } else {
            this.bloquearDadosMedida(widget);
            this.filtrarTipoDeItensManuaisDoEscopo(widget);
            
            var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
            var tipoprodserv    = widget.currentRow.TIPOPRODSERV;
            var cdprodserv      = widget.currentRow.CDPRODSERV;

            if (nrtipoorcamento && tipoprodserv == "S" && cdprodserv) {
                var params = {
                    "NRTIPOORCAMENTO": nrtipoorcamento,
                    "CDSERVORCA"     : cdprodserv
                };

                General.callRestEngine("product#/fct_itemorcamento_widget/getMedidaVariavelEscopo", "Row", params).then(function(response) {
                    if (response.dataset && response.dataset.retorno) {
                        widget.getField("DSTPMEDIDASERV").readOnly = true;
                        widget.currentRow.INSERIR_MEDIDA = true;
                    } else {
                        widget.getField("DSTPMEDIDASERV").readOnly = false;
                        widget.currentRow.INSERIR_MEDIDA = false;
                    }
                });
            }
        }
    };

    /**
     * Salva um Vendedor.
     * 
     * @param {object} widget - Widget de Vendedor
     */
    this.salvarVendedor = function(widget) {
        if (widget.isValid()) {

            var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
            var nrvendedororca = widget.currentRow.NRVENDEDORORCA;
            var nrparcvendedor = widget.currentRow.NRPARCVENDEDOR;

            Sm00001000_LancamentoProposta_Service.validarVendedor(nrseqorcamento, nrvendedororca, nrparcvendedor).then(function(response) {
                    var dados = response.dataset.fct_vendedororca_widget;
                    var idvenpriorca = widget.currentRow.IDVENPRIORCA;

                    if (dados.existeVendedor) {
                        ScreenService.showMessage("Operação bloqueada.<br/><br/>Vendedor já cadastrado para a proposta selecionada.");
                    }
                    else if (dados.existePrincipal && idvenpriorca == "S") {
                        ScreenService.showMessage("Operação bloqueada.<br/><br/>Deve haver apenas um vendedor principal para a proposta selecionada.");
                    }
                    else {
                        widget.currentRow.__columns_to_return = ["NRVENDEDORORCA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];

                        General.widgetSave(widget).then(function(success) {
                            if (success.dataset && success.dataset.columnsToReturn[0]) {
                                widget.currentRow.NRVENDEDORORCA = success.dataset.columnsToReturn[0].NRVENDEDORORCA;
                                widget.currentRow.NRORG = success.dataset.columnsToReturn[0].NRORG;
                                widget.currentRow.DTINCLUSAO = success.dataset.columnsToReturn[0].DTINCLUSAO;
                                widget.currentRow.CDOPERINCLUSAO = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                            }
                        }).catch(function(error) {
                            console.error(error);
                            if (error.data)
                                ScreenService.showMessage(error.data.error);
                            else
                                ScreenService.showMessage(error);
                        });
                    }
                },
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }
    };


    /**
     * Calcula a data de validade da proposta a partir dos dias.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.calcularDataValidadeProposta = function(widget) {
        var qtDias = widget.currentRow.QTDIASVALIDADE;
        if (qtDias != null && qtDias >= 0) {
            var dataAtual = moment();
            widget.currentRow.DTVALIDADE = dataAtual.add(qtDias, "day").format("DD/MM/YYYY");
        }
        else {
            widget.currentRow.DTVALIDADE = "";
            widget.currentRow.QTDIASVALIDADE = "";
        }
    };

    /**
     * Calcula os dias de validade da proposta a partir da data.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.calcularDiasValidadeProposta = function(widget) {
        var dataValidade = widget.currentRow.DTVALIDADE;
        if (dataValidade) {
            dataValidade = moment(dataValidade + " 00:00:00", "DD/MM/YYYY HH:mm:ss");
            var dataAtual = moment().format("DD/MM/YYYY");
            dataAtual = moment(dataAtual + " 00:00:00", "DD/MM/YYYY HH:mm:ss");
            var qtDias = dataValidade.diff(dataAtual) / 86400000;
            if (qtDias >= 0) {
                widget.currentRow.QTDIASVALIDADE = qtDias;
            }
            else {
                ScreenService.showMessage("Data de Validade não pode ser inferior à data de hoje.");
                widget.currentRow.DTVALIDADE = "";
                widget.currentRow.QTDIASVALIDADE = "";
            }
        }
        else {
            widget.currentRow.QTDIASVALIDADE = "";
        }
    };

    /**
     * Seta o Modelo e os Escopos de Proposta para o filtro da Composição.
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.setFiltroComposicao = function(widget) {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var codModelo = wdgProposta.currentRow.NRORCAMENTOMODELO;
        var escopos = wdgProposta.currentRow.NRTIPOORCAMENTO;

        widget.dataSourceFilter[0].value = codModelo;
        widget.dataSourceFilter[1].value = escopos;

        widget.reload();
    };

    /**
     * Avança para a aba de Elementos (inserindo).
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.avancarComposicao = function(widget) {
        if (widget.isValid()) {
            //widget.enabled = false;
            var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");
            wdgPropostaTabbed.enabled = false;

            var wdgItensComp = templateManager.container.getWidget("wdgItensComposicaoProposta");
            wdgItensComp.enabled = true;
            wdgItensComp.isVisible = true;
            wdgItensComp.activate().then(function() {
                this.setFiltroComposicao(wdgItensComp);
            }.bind(this));
        }
        else {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        }
    };

    /**
     * Volta para a aba de Proposta.
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.voltarComposicao = function(widget) {
        widget.enabled = false;
        var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");
        wdgPropostaTabbed.enabled = true;
        wdgPropostaTabbed.activate();
    };

    /**
     * Volta para a aba de Elementos da Proposta (inserindo).
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (inserindo)
     */
    this.voltarTipoMedida = function(widget) {
        widget.enabled = false;
        var wdgItensComposicaoProposta = templateManager.container.getWidget("wdgItensComposicaoProposta");
        wdgItensComposicaoProposta.enabled = true;

        //var wdgProposta = templateManager.container.getWidget("wdgProposta");
        //wdgProposta.openTab("wdgItensComposicaoProposta");
        wdgItensComposicaoProposta.activate();
    };



    //=============== COMENTADO PORQUE FOI RETIRADO O CAMPO DE VALOR DA TELA ===============
    /**
     * Calcula o valor dos itens Pai.
     * 
     * @param {object} widget - Widget de Elementos (inserindo).
     */
    /*this.calcularValorItensPai = function(widget) {
        setTimeout(function() {
            widget.dataSource.data.forEach(function(row) {
                if (row._childCount > 0) {
                    var dataset = widget.dataSource.data;
                    var codItem = row.NRITCOMPSERVICO;
                    
                    var somaValor = this.getSomaValor(dataset, codItem, 0);
                    row.VRTOTAL = somaValor;
                }
            }.bind(this));
        }.bind(this), 100);
    };*/

    /**
     * Calcula a soma dos valores dos filhos do item de código passado.
     * 
     * @param {Object[]} dataset - Dados carregados na tela de Elementos (inserindo)
     * @param {number} codItem - Código do item da Composição
     * @param {floar} somaValor - Somatório dos Itens da Composição filhos do item do código informado
     * 
     * @return {float}
     */
    /*this.getSomaValor = function(dataset, codItem, somaValor) {
        for (var i=0; i < dataset.length; i++) {
            if (dataset[i]._parent && dataset[i]._parent.NRITCOMPSERVICO == codItem) {
                somaValor += dataset[i].VRTOTAL;
                somaValor = this.getSomaValor(dataset, dataset[i].NRITCOMPSERVICO, somaValor);
            }
        }
        return somaValor;
    };*/

    //====================================================================================



    /**
     * Seta o filtro dos Serviços dos Elementos.
     * 
     * @param {object} widget - Widget de Serviço dos Elementos
     */
    this.setFiltroServicoItensComposicaoProposta = function(widget) {
        var wdgItensComposicaoProposta = templateManager.container.getWidget("wdgItensComposicaoProposta");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");

        widget.dataSourceFilter[0].value = wdgItensComposicaoProposta.currentRow.CDELEMENTO;
        widget.dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
        widget.dataSourceFilter[2].value = wdgItensComposicaoProposta.currentRow.NRORG;
        widget.reload();
    };
    
    /**
     * Adiciona os itens da proposta na tabela de agendamentos pendentes 
     * com a pendencia de ação preventiva.
     * @params{number} numero do orcamento
    **/
    this.insereItensAcaoPreventiva = function(nrseqorcamento, nritemorcamento, quantidade, situacao){
        var params = {
            "NRSEQORCAMENTO"     : nrseqorcamento,
            "NRITEMORCAMENTO"    : nritemorcamento,
            "QUANTIDADE"         : quantidade,
            "IDSITUACAOORCAMENTO": situacao,
            "ROTINA"             : "PROPOSTA"
        };
        
        General.callRestEngine("product#/fct_agendrecursocont_widget/salvarAgendamentosPendentes", "Row", params)
        .then(null)
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Conclui a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Proposta
     */
    this.concluirProposta = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente concluir a proposta?"), 'question', function() {
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            var nrorg = wdgProposta.currentRow.NRORG;

            Sm00001000_LancamentoProposta_Service.validarElementosParaConclusao(wdgProposta.currentRow).then(function (validaElemento) {
                if(validaElemento.dataset.validacao) {
                    var msgErro = Sm00001000_LancamentoProposta_Service.validarPropostaParaConclusao(wdgProposta.currentRow);
                    if (msgErro) {
                        ScreenService.showMessage(msgErro);
                        ScreenService.closePopup();
                    } else {
                        Sm00001000_LancamentoProposta_Service.validarMoedaItens(nrseqorcamento).then(function() {
                            Sm00001000_LancamentoProposta_Service.getPorcentagemVendedores(nrseqorcamento, nrorg).then(function(success) {
                                var somaPorcentagem = success.dataset.fct_vendedororca_widget.porcentagem;
                                if (somaPorcentagem == 100 || somaPorcentagem == 0) {

                                    Sm00001000_LancamentoProposta_Service.validarNegociacaoValores(nrseqorcamento).then(function(response) {
                                        if (response) {
                                            //if (wdgProposta.currentRow.IDUTILIZACOMP == "S") {
                                            if (wdgProposta.currentRow.QTITENSTRABSERV > 0) {
                                                var row = {"NRSEQORCAMENTO": nrseqorcamento};
                                                General.callRestEngine("product#/fct_itemorcamento_widget/validarValoresItensProp", "Row", row).then(function(response){
                                                    if(response.dataset && response.dataset.validarValoresItensProp && response.dataset.validarValoresItensProp.VRORCAMENTO){
                                                        wdgProposta.currentRow.VRORCAMENTO = response.dataset.validarValoresItensProp.VRORCAMENTO;
                                                        ScreenService.showMessage("Operação bloqueada!<br><br>O valor dos itens da proposta foram atualizados devido a mudanças recentes na tabela de preço ou na quantidade de medida do serviço.<br><br>Favor checa-los e tentar concluir novamente.");
                                                    }
                                                    else {
                                                        this.salvarConclusaoProposta(wdgProposta, true);
                                                    }
                                                    this.validaIdCertsign();
                                                }.bind(this),
                                                function (error) {
                                                    if (error.data && error.data.error.indexOf('POSSUI COMPOSICAO INATIVA') !== -1) {
                                                        ScreenService.showMessage("Operação bloqueada!<br>Não é possível concluir propostas ligadas a composições de serviço inativas.<br>Ative-as e tente novamente!");
                                                    } else {
                                                        Utilities.showErrorMessage(error);
                                                    }
                                                });
                                            }
                                            else {
                                                this.salvarConclusaoProposta(wdgProposta, false);
                                            }
                                        }
                                        else {
                                            ScreenService.showMessage('Operação bloqueada.<br/><br/>Os itens da aba "Negociação de Valores" que são de parcelamento não estão com as parcelas criadas corretamente: o valor total da soma das parcelas deve ser igual ao valor do item.');
                                        }
                                    }.bind(this))
                                    .catch(function(error) {
                                        Utilities.showErrorMessage(error);
                                    });
                                }
                                else {
                                    ScreenService.closePopup();
                                    ScreenService.showMessage("Operação bloqueada.<br/><br/>Soma da porcentagem de participação na venda dos vendedores da proposta deve ser igual a 100%.");
                                }
                                    
                            }.bind(this))
                            .catch(function (error) {
                                Utilities.showErrorMessage(error);
                            });
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                } else {
                    ScreenService.closePopup();
                }
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    };
    
    /**
     * @param {object} wdgProposta
     * @param {boolean} utilizaComposicao
     */
    this.salvarConclusaoProposta = function(wdgProposta, utilizaComposicao) {
        var params = { "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO };
        General.callRestEngine("product#/fct_orcamento_widget/validarParametrosFaturamentoOrca", "Row", params).then(function(response) {
            var nrseqorcamento    = wdgProposta.currentRow.NRSEQORCAMENTO;
            var nrorcamentomodelo = wdgProposta.currentRow.NRORCAMENTOMODELO;
            var nrorg             = wdgProposta.currentRow.NRORG;
    
            var params = {
                "NRORCAMENTOMODELO" : nrorcamentomodelo,
                "NRORG"             : nrorg
            };
        
            General.callRestEngine("product#/fct_orcamento_widget/validaOrcamentoValidos", "Row", params).then(function(validacao) {
                if (validacao.dataset.modeloInativo.length > 0) {
                    ScreenService.showMessage("Operação bloqueada!<br><br>O modelo de proposta selecionado está inativo");
                } else {
                    
                    wdgProposta.currentRow.changed = true;
                    wdgProposta.currentRow.IDSITUACAOORCAMENTO = "CONCLUIDO";
    
                    wdgProposta.saveRow().then(function () {
                        wdgProposta.currentRow.changed = false;
                        ScreenService.closePopup();
                        
                        // Gera os documentos no ambiente do Certsign
                        
                       var params = {
                            "NRORCAMENTOMODELO" : nrorcamentomodelo,
                        };

                        General.callRestEngine('product#/fct_orcamento_widget/validaGeraDocCertisign', "Row", params).then(function(response) {
                            
                            var idenviadocconclusao = response.dataset.validaGeraDocCertisign.resultado;
                            
                            if (idenviadocconclusao == 'S') {
                            
                                var params = {
                                    "NRSEQORCAMENTO" : nrseqorcamento
                                };
        
                                General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {
                    
                                    var idTokenCertsign = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
                                    var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;
                                    
                                    if ((idTokenCertsign != null ) && (idAssEletrDigital == 'S')) {
                                        
                                        var params = {
                                            "NRSEQORCAMENTO"     : nrseqorcamento,
                                            "NRTIPOORCAMENTO"    : wdgProposta.currentRow.NRTIPOORCAMENTO,
                                            "IDSITUACAOORCAMENTO": wdgProposta.currentRow.IDSITUACAOORCAMENTO,
                                            "NRORCAMENTO"        : wdgProposta.currentRow.NRORCAMENTO
                                        };
                                    
                                        var timeout = 5 * 60000; //5 min
                                        Utilities.callRestEngineWithTimeout("product#/fct_orcamento_widget/gerarDocumentoCertsign", "Row", params, timeout)
                                        .then(function() {
                                            ScreenService.showMessage("Processo realizado com sucesso!");
                                        })
                                        .catch(function(error) {
                                            var message = error;
                                            if (typeof error === 'object') {
                                                if (error.data && error.data.error) {
                                                    message = error.data.error;
                                                }
                                                else if (error.message) {
                                                    message = error.message;
                                                }
                                            }
                                            
                                            ScreenService.showMessage("Ocorreram erros durante o upload dos documentos para o Portal Certisign. <br/>Erro: " + message);
                                        });
                                    }
                                    else {
                                        ScreenService.showMessage("Processo realizado com sucesso!");
                                    }
                                });
                            }
                        });
                        //salva as informações da variação de preço da proposta
                        if (utilizaComposicao) {
                            params = {
                                "NRSEQORCAMENTO": nrseqorcamento,
                                "NRTIPOORCAMENTO": wdgProposta.currentRow.NRTIPOORCAMENTO,
                                "NRMOEDA": wdgProposta.currentRow.NRMOEDA
                            };
                            General.callRestEngine("product#/fct_orcamento_widget/salvarInfoServicoEPrecoOrca", "Row", params).then(null).catch(function(error) {
                                Utilities.showErrorMessage(error);
                            });
                        }
    
                        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                        wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "CONCLUIDO";
                        
                        this.alterarVisibilidadeActionEditProposta(wdgPropostaGrid, nrorg);
                        this.validacoesGeraisProposta(wdgPropostaGrid);
                        this.validaIdCertsign();
                    }.bind(this))
                    .catch(function (error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }.bind(this),
            function (error) {
                ScreenService.closePopup();
                Utilities.showErrorMessage(error);
            });
        }.bind(this),
        function (error) {
            Utilities.showErrorMessage(error);
        });
    };

    /**
     * Abre o popup para  ajustificativa de uma renegociação ou um cancelamento
     * de uma porposta já aceita
     * @params{Object} widget widget proposta
    **/
    this.abrePopupDsJustCancReneg = function(widget, action){
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("PopupDsJustCancReneg");
        if(action === "cancelar_proposta"){
            wdgPopup.label = "Justificativa de Cancelamento da Proposta";
            wdgPopup.getField("DSJUSTIFCANCRENEG").label = "Justificativa de Cancelamento"; 
        } else { 
            wdgPopup.label = "Justificativa de Renegociação da Proposta";
            wdgPopup.getField("DSJUSTIFCANCRENEG").label = "Justificativa de Renegociação"; 
        }
            
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow.NRSEQORCAMENTO   = widget.currentRow.NRSEQORCAMENTO;
            wdgPopup.currentRow.NRORCAMENTO      = widget.currentRow.NRORCAMENTO;
            wdgPopup.currentRow.DSREFERENCIAORCA = widget.currentRow.DSREFERENCIAORCA;
            wdgPopup.currentRow.ACTION           = action;
        }.bind(this));
    };
    
    /**
     * Salva a justificativa de cancelamento ou renegociação da proposta
     * @params{Object} widget widget popup de justificativa de cancelamento ou renegociação
    **/
    this.salvarJustificativaCancReneg = function(widget){
        if(widget.isValid()){
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            wdgProposta.currentRow.DSJUSTIFCANCRENEG = widget.currentRow.DSJUSTIFCANCRENEG;
            wdgProposta.currentRow.DTACEITEORCA      = null;
            wdgProposta.saveRow().then(function(){
                ScreenService.closePopup();
                widget.isVisible = false;
                wdgProposta.activate();
                if(widget.currentRow.ACTION === "cancelar_proposta")
                    this.cancelarProposta(wdgProposta);
                else 
                    this.copiarVersionarProposta(wdgProposta, 'V');
            }.bind(this))
            .catch(function(error){ 
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * Cancela o popup e ativa a widget que contem o nome do paramêtro
     * @params{String} widgetName nome da widget que será ativada
    **/
    this.cancelaPopup = function(widgetName){
        ScreenService.closePopup();
        templateManager.container.getWidget(widgetName).activate();
    };
    
    /**
     * Verifica se a proposta escolhida está aceita
     * Caso esteja abre um popup pedindo a justificativa 
     * @params{Object} widget widget proposta
     * @params{String} action nome da action que acionou a função
    **/
    this.validaSituacaoPropCancReneg = function(widget, action){
        if (action === "versionar_proposta" && (widget.currentRow.ISSUEID || widget.currentRow.NRSOLISERV)) {
            ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>Não é possível renegociar propostas ligadas a uma issue ou solicitação."));
        }
        else {
            var msgConfirmacao = null;
            if((widget.currentRow.IDSITUACAOORCAMENTO == "CANCELADO" && action === "versionar_proposta") || action === "cancelar_proposta")
                msgConfirmacao = "Deseja realmente "+ (action === "cancelar_proposta" ? "cancelar" : "renegociar") +" a proposta?";
            else 
                msgConfirmacao = "Ao renegociar esta proposta, sua versão atual ficará como cancelada. Deseja continuar?";
            
            if(widget.currentRow.IDSITUACAOORCAMENTO === "ACEITO"){
                var params = { "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO };
                General.callRestEngine("product#/fct_orcamento_widget/getContratosDifPendentes", "Row", params).then(function(response){
                    if(response.dataset && response.dataset.fct_orcamento_widget && response.dataset.fct_orcamento_widget.CONTRATOS){
                        ScreenService.showMessage("Operação bloqueada.<br><br> Não é permitido "+ (action === "cancelar_proposta" ? "cancelar" : "renegociar") +" propostas que possuem contratos gerados com situação diferente de \"Pendente\" e \"Reprovado\".");
                    } else {
                        ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function(){
                            if(action == 'cancelar_proposta') 
                                this.abrePopupDsJustCancReneg(widget, action);
                            else 
                                this.validarRenegociacao(widget);
                        }.bind(this));
                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            } else {
                ScreenService.confirmMessage(ScreenService.i18n(msgConfirmacao), 'question', function(){
                    if(action === "cancelar_proposta") 
                        this.cancelarProposta(widget);
                    else 
                        this.validarRenegociacao(widget);
                }.bind(this));
            }
        }
    };
    
    /**
     * Ação que cancela a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Prposta
     */
    this.cancelarProposta = function(wdgProposta) {
        var params = {
            "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO,
            "NRORG": wdgProposta.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamento_widget/cancelarProposta", "Row", params).then(function(success) {
            ScreenService.closePopup();
            wdgProposta.currentRow.IDSITUACAOORCAMENTO = "CANCELADO";
            
            var params = {
                "NRSEQORCAMENTO"  : wdgProposta.currentRow.NRSEQORCAMENTO,
                "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/excluirDocumentoCertsign", "Row", params).then(function() {
                ScreenService.showMessage("Processo realizado com sucesso!");
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });

            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "CANCELADO";
            this.validacoesGeraisProposta(wdgPropostaGrid);
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Exibe mensagem de confirmação para copiar a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.confirmarRenegociarCopiarProposta = function(widget, tipo) {
        if (widget.currentRow.ISSUEID || widget.currentRow.NRSOLISERV) {
            ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>Não é possível copiar propostas ligadas a uma issue ou solicitação."));
        } else {
            ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente copiar a proposta?"), 'question', function(){   
                var params = { "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO };
                General.callRestEngine("product#/fct_itemorcamento_widget/validarAtivacaoComposicao", "Row", params).then(function(response){
                    this.copiarVersionarProposta(widget, tipo);
                }.bind(this))
                .catch(function(error){
                    if (error.data && error.data.error.indexOf('POSSUI COMPOSICAO INATIVA') !== -1) 
                        ScreenService.showMessage("Operação bloqueada!<br>Não é possível copiar propostas ligadas a composições de serviço inativas.<br>Ative-as e tente novamente!");
                    else 
                        Utilities.showErrorMessage(error);
                });
            }.bind(this));
        }
    };
    
    /**
     * Copia ou versiona a Proposta.
     * 
     * @param {object} widget - Widget de Proposta
     * @param {string} tipo - 'C' para Copiar, 'V' para Versionar
     */
    this.copiarVersionarProposta = function(widget, tipo) {
        var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
        var row = widget.currentRow;
        var params = {
            "IDSITUACAOORCAMENTO": widget.currentRow.IDSITUACAOORCAMENTO,
            "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
            "NRORG"              : widget.currentRow.NRORG,
            "TIPO"               : tipo
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/copiarVersionarProposta", "Row", params).then(function(response) {
            if(!widget.currentRow.NRCONTATO){
                row.NRSEQORCAMENTO = response.dataset.NRSEORCGERADO[0];
                this.geraContatoProposta(row);
                
                if (tipo == "V") {
                    var params = {
                        "NRSEQORCAMENTO"  : nrseqorcamento,
                        "NRTIPOORCAMENTO" : widget.currentRow.NRTIPOORCAMENTO
                    };
                    
                    General.callRestEngine("product#/fct_orcamento_widget/excluirDocumentoCertsign", "Row", params);
                }
            }
            
            ScreenService.showMessage("Processo realizado com sucesso!");
            ScreenService.closePopup();
            this.voltarParaPropostaGrid();
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Aceita a Proposta.
     * 
     * @param {object} widget - popup de aceitar proposta
     */
    this.aceitarProposta = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgProposta = getWidgetProposta();
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            
            var params = {
                "NRSEQORCAMENTO" : nrseqorcamento
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/validaAceiteCertsign", "Row", params).then(function(resultado) {
                
                if (resultado.dataset.validaAceiteCertsign.resultado != 'OK') {
                    ScreenService.showMessage("Operação bloqueada!<br><br>Existem assinaturas eletrônicas/digitais pendentes ou rejeitadas.");
                }
                else {
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente aceitar a proposta?"), 'question', function(){
                        var params = {
                            "NRSEQORCAMENTO": nrseqorcamento,
                            "NRORCAMENTO"   : wdgProposta.currentRow.NRORCAMENTO,
                            "NRORG"         : wdgProposta.currentRow.NRORG,
                            //"IDUTILIZACOMP" : wdgProposta.currentRow.IDUTILIZACOMP,
                            "IDUTILIZACOMP" : (wdgProposta.currentRow.QTITENSTRABSERV > 0) ? "S" : "N",
                            "NRPARCNEGOCIO" : wdgProposta.currentRow.NRPARCNEGOCIO,
                            "DTACEITEORCA"  : widget.currentRow.DTACEITE
                        };
                        
                        Sm00001000_LancamentoProposta_Service.validarNegociacaoValores(nrseqorcamento).then(function(response) {
                            if (response) {
                                General.callRestEngine("product#/fct_orcamento_widget/aceitarProposta", "Row", params).then(function() {
                                    
                                    this.cancelaPopup("wdgProposta");
                                    ScreenService.showMessage("Processo realizado com sucesso!");
                                    ScreenService.closePopup();
                                    wdgProposta.reload().then(function() {
                                        var wdgPropostaGrid = getWidgetPropostaGrid();
                                        
                                        wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO = "ACEITO";
                                        wdgPropostaGrid.currentRow.DTACEITEORCA        = wdgProposta.currentRow.DTACEITEORCA;
                                        
                                        this.validacoesGeraisProposta(wdgPropostaGrid);
                                    }.bind(this));
                                
                                }.bind(this))
                                .catch(function(error) {
                                    Utilities.showErrorMessage(error);
                                });
                            }
                            else {
                                ScreenService.showMessage('Operação bloqueada.<br/><br/>Os itens da aba "Negociação de Valores" que são de parcelamento não estão com as parcelas criadas corretamente: o valor total da soma das parcelas deve ser igual ao valor do item.');
                            }
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                        
                    }.bind(this));
                }
            }.bind(this))
            .catch(function (error) {
                Utilities.showErrorMessage(error);
            });
        }
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o País.
     * 
     * @param {object} widget - Widget de Endereço
     */
    this.controlarCamposEnderecoPorPais = function(widget) {
        widget.currentRow.SGESTADO = null;
        widget.currentRow.NMESTADO = null;
        widget.currentRow.CDMUNICIPIO = null;
        widget.currentRow.NMMUNICIPIO = null;
        widget.currentRow.CDBAIRRO = null;
        widget.currentRow.NMBAIRROENDERECO = null;

        var cdpais = widget.currentRow.CDPAIS;

        var campoEstado = widget.getField("NMESTADO");
        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoEstado.dataSourceFilter[0].value = cdpais;
        campoEstado.reload();

        campoMunicipio.dataSourceFilter[0].value = cdpais;

        campoBairro.dataSourceFilter[0].value = cdpais;
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Estado.
     * 
     * @param {object} widget - Widget de Endereço
     */
    this.controlarCamposEnderecoPorEstado = function(widget) {
        widget.currentRow.CDMUNICIPIO = null;
        widget.currentRow.NMMUNICIPIO = null;
        widget.currentRow.CDBAIRRO = null;
        widget.currentRow.NMBAIRROENDERECO = null;

        var sgestado = widget.currentRow.SGESTADO;

        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoMunicipio.dataSourceFilter[1].value = sgestado;
        campoMunicipio.reload();

        campoBairro.dataSourceFilter[1].value = sgestado;
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Município.
     * 
     * @param {object} widget - Widget de Endereço
     */
    this.controlarCamposEnderecoPorMunicipio = function(widget) {
        widget.currentRow.CDBAIRRO = null;
        widget.currentRow.NMBAIRROENDERECO = null;

        var cdmunicipio = widget.currentRow.CDMUNICIPIO;

        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoBairro.dataSourceFilter[2].value = cdmunicipio;
        campoBairro.reload();
    };

    /**
     * Salva o Endereço.
     * 
     * @param {object} widget - Widget de Endereço
     */
    this.salvarEndereco = function(widget) {
        widget.currentRow.__columns_to_return = ["NRENDERECOORCA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];

        var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
        var nrenderecoorca = widget.currentRow.NRENDERECOORCA;
        var cdtipoendereco = widget.currentRow.CDTIPOENDERECO;

        Sm00001000_LancamentoProposta_Service.validarTipoEndereco(nrseqorcamento, nrenderecoorca, cdtipoendereco).then(function(success) {

                if (success.dataset.fct_enderecoorca_widget.existe == false) {

                    General.widgetSave(widget).then(function(success) {
                        if (success.dataset && success.dataset.columnsToReturn[0]) {
                            widget.currentRow.NRENDERECOORCA = success.dataset.columnsToReturn[0].NRENDERECOORCA;
                            widget.currentRow.NRORG = success.dataset.columnsToReturn[0].NRORG;
                            widget.currentRow.DTINCLUSAO = success.dataset.columnsToReturn[0].DTINCLUSAO;
                            widget.currentRow.CDOPERINCLUSAO = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                        }
                    }).catch(function(error) {
                        console.error(error);
                        if (error.data)
                            ScreenService.showMessage(error.data.error);
                        else
                            ScreenService.showMessage(error);
                    });
                }
                else {
                    ScreenService.showMessage("Operação bloqueada.<br/><br/>Já existe um endereço cadastrado para o tipo informado.");
                }
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Abre a widget de Enviar Email.
     */
    this.abrirWdgEnviarEmail = function(widget) {
        ScreenService.closePopup();

        var nrseqorcamento  = widget.currentRow.NRSEQORCAMENTO;
        var nrorg           = widget.currentRow.NRORG;
        var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;

        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        wdgEnviarEmail.isVisible = true;
        this.setDadosPadraoWdgEnviarEmail(wdgEnviarEmail, nrseqorcamento, nrorg, nrtipoorcamento);
    };

    /**
     * Fecha a tela de Enviar E-mail.
     * 
     * @param {object} widget
     */
    this.fecharWdgEnviarEmail = function(widget) {
        this.desobrigaCamposEvolucaoEmail();
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;
            getWidgetProposta().activate();
        });
    };

    /**
     * Seta o Remetente e os Destinatários padrão ao abrir a tela de Enviar E-mail.
     * 
     * @param {object} widget - Widget de Enviar E-mail
     */
    this.setDadosPadraoWdgEnviarEmail = function(widget, nrseqorcamento, nrorg, nrtipoorcamento) {
        var params = {
            "NRSEQORCAMENTO"   : nrseqorcamento,
            "NRORG"            : nrorg,
            "NRTIPOORCAMENTO"  : nrtipoorcamento
        };

        General.callRestEngine("product#/fct_orcamento_widget/getDadosPadraoEnviarEmail", "Row", params).then(function(success) {

                ScreenService.openSwipe(widget).then(function() {
                    widget.edit();

                    widget.currentRow = {
                        "NRSEQORCAMENTO" : nrseqorcamento,
                        "NRORG"          : nrorg,
                        "NRTIPOORCAMENTO": nrtipoorcamento,
                        "REMETENTE"      : success.dataset.DADOS.REMETENTE,
                        "DESTINATARIOS"  : success.dataset.DADOS.DESTINATARIOS,
                        "ASSUNTO"        : success.dataset.DADOS.DSASSUNTOEMAIL,
                        "CORPO"          : success.dataset.DADOS.DSCORPOEMAIL
                    };
                });
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Envia o Documento da Proposta por e-mail.
     * 
     * @param {object} widget
     */
    this.enviarDocumentoPorEmail = function(widget) {
        var widgetProposta = templateManager.container.getWidget("wdgProposta");
        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        
        var nmtipocont     = wdgEnviarEmail.currentRow.NMTIPOCONT;
        var nmmotievocon   = wdgEnviarEmail.currentRow.NMMOTIEVOCON;
        var nmseqresulevol = wdgEnviarEmail.currentRow.NMSEQRESULEVOL;
        
        if (widget.isValid()) {
            var destinatarios = widget.currentRow.DESTINATARIOS;
            if (destinatarios.indexOf(";") == -1) {
                destinatarios += ";";
            }
            
            var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
            if(nrtipoorcamento instanceof Array){
                nrtipoorcamento = 0;
            }
            
            var interlocutor =  widgetProposta.currentRow.NRSEQINTERLOCUTOR;
            var params = {
                "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
                "NRORG"              : widget.currentRow.NRORG,
                "NRTIPOORCAMENTO"    : nrtipoorcamento,
                "REMETENTE"          : widget.currentRow.REMETENTE,
                "DESTINATARIOS"      : widget.currentRow.DESTINATARIOS,
                "ASSUNTO"            : widget.currentRow.ASSUNTO,
                "CORPO"              : widget.currentRow.CORPO,
                "IDSITUACAOORCAMENTO": widgetProposta.currentRow.IDSITUACAOORCAMENTO
            };
            
            var idSituacao = widgetProposta.currentRow.IDSITUACAOORCAMENTO;
            var currentRow = widget.currentRow;
            currentRow.DTHREVOLCONT = (currentRow.DTHREVOLCONT)? currentRow.DTHREVOLCONT : moment(new Date()).format('DD/MM/YYYY HH:mm');
            widget.currentRow.NMPARCINTERLOCUTOR = widgetProposta.currentRow.NMPARCINTERLOCUTOR;
            widget.currentRow.TXEVOLCONT = widget.currentRow.CORPO;
            widget.currentRow.NMOPERADOR = nomeOperador;
            widget.currentRow.NRPARCNEGOCIO = widgetProposta.currentRow.NRPARCNEGOCIO;
           
            var timeout = 5 * 60000; //5 min
            Utilities.callRestEngineWithTimeout("product#/fct_orcamento_widget/enviarDocumentoPorEmail", "Row", params, timeout).then(function() {
                ScreenService.showMessage("Processo realizado com sucesso!").then(function(){
                if(nmtipocont && nmmotievocon && nmseqresulevol){
                    this.actionEvolutionNewRowPropostaEmail(interlocutor,idSituacao);
                }
                
                ScreenService.closePopup();
                ScreenService.closeSwipe();
                widget.isVisible = false;
                getWidgetProposta().activate();
                
            }.bind(this),
            function(error) {
                console.error(error);
                var msg = "Timeout";
                if (error.status !== 0 && error.data)
                    msg = error.data.error;
                ScreenService.showMessage(msg);
            });
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
      /**
     * Cria um nova row de propostaPorEmail
     * 
     * @param {interge} widget - métodi enviarDocumentoPorEmail
     */
    this.actionEvolutionNewRowPropostaEmail = function(interlocutor,idSituacao){
        this.closeActionsPopup().then(function(){
            var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
            var corpo          = wdgEnviarEmail.currentRow.CORPO;
            var nmtipocont     = wdgEnviarEmail.currentRow.NMTIPOCONT;
            var nmmotievocon   = wdgEnviarEmail.currentRow.NMMOTIEVOCON;
            var nmseqresulevol = wdgEnviarEmail.currentRow.NMSEQRESULEVOL;
            var cdtipocont     = wdgEnviarEmail.currentRow.CDTIPOCONT;
            var cdmotievocon   = wdgEnviarEmail.currentRow.CDMOTIEVOCON;
            var nrseqresulevol = wdgEnviarEmail.currentRow.NRSEQRESULEVOL;
            this.evolucaoNewRowEmail(corpo, interlocutor, nmtipocont, nmmotievocon, nmseqresulevol, cdtipocont, cdmotievocon, nrseqresulevol, idSituacao);
        }.bind(this));
    };
    
     /**
     * Forma uma widget para evoluir apos o envio do email
     * 
     * @param {interge} widget - métodi actionEvolutionNewRowPropostaEmail
     */
    this.evolucaoNewRowEmail = function(corpo, interlocutor, nmtipocont, nmmotievocon, nmseqresulevol, cdtipocont, cdmotievocon, nrseqresulevol, idSituacao) {
            var widget = templateManager.container.getWidget("evolucao");
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            
            General.widgetNewRow(widget);
            
            var currentRow = widget.currentRow;
            currentRow.DTHREVOLCONT      = (currentRow.DTHREVOLCONT)? currentRow.DTHREVOLCONT : moment(new Date()).format('DD/MM/YYYY HH:mm');
            currentRow.NMOPERADOR        = nomeOperador;
            currentRow.CDOPEREVOL        = opLogado;
            currentRow.NRORG             = nrOrgOper;
            currentRow.NRSEQINTERLOCUTOR = interlocutor;
            currentRow.NMTIPOCONT        = nmtipocont;
            currentRow.NMMOTIEVOCON      = nmmotievocon;
            currentRow.NMSEQRESULEVOL    = nmseqresulevol;
            currentRow.CDTIPOCONT        = cdtipocont;
            currentRow.CDMOTIEVOCON      = cdmotievocon;
            currentRow.TXEVOLCONT        = corpo;
            currentRow.NRSEQRESULEVOL    = nrseqresulevol;
           //    ABEvolucaoController.setEvolDateChangePermission(widget);
            
            currentRow.DTINCLUSAO = self.momentNow();
            currentRow.NRSEQCONTATO = wdgProposta.currentRow.NRSEQCONTATO;
            currentRow.NRORG = nrOrgOper;
            
            widget.currentRow =  currentRow;
            this.saveEvolucaoAposEmail(widget, idSituacao);
    };

    /**
     * Buscas os dados do anexo para para salvar no anexo da evolução
     */
    this.anexarDocumnetosdoEmail = function(idSituacao, nrseqcontato, nrseqevolcont) {
        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        var nrtipoorcamento = wdgEnviarEmail.currentRow.NRTIPOORCAMENTO;
            
         var params = {
            "NRSEQORCAMENTO": wdgEnviarEmail.currentRow.NRSEQORCAMENTO,
            "NRTIPOORCAMENTO": nrtipoorcamento,
            "IDSITUACAOORCAMENTO": idSituacao,
            "NRORG": wdgEnviarEmail.currentRow.NRORG
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/getDadosAnexo", "Row", params).then(function(response) {
                var params = [];
                response.dataset.dadoAnexos.forEach(function(row){
                
                    var retorno = this.converteNomePDF(row.DSCAMINHODOCGERADOPDF, row.NMARQUIVO);
                
                    params.push({
                        "NRSEQCONTATO": nrseqcontato,
                        "DSCAMINHOARQ": row.DSCAMINHODOCGERADOPDF,
                        "NMARQUIVO": retorno[0],
                        "DSTIPO": retorno[1],
                        "OPERADOR": opLogado,
                        "VRTAMANHO": row.VRTAMANHO,
                        "NRSEQEVOLCONT": nrseqevolcont
                    });
                }.bind(this));
                
                General.callRestEngine("product#/crm_anexocontato/saveAnexo", "DataSet", params).then(function() {
                    
                }.bind(this));
            }.bind(this),
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });

    };
    
     this.converteNomePDF = function(pdf,nomeArq){
        var position;
        var extensao = pdf;
        position = extensao.indexOf(".");
        extensao = extensao.slice(position,extensao.length);
            
        var nome = nomeArq;
        position = nome.indexOf(".");
        nome = nome.substr(0, position);
        
        nome = nome+extensao;
        
        var retorno = [nome,extensao];
        return retorno;
     };
     
     this.setValuesForFileUploadEvolucao = function(widget, dadoAnexo, nrseqcontato, nrseqevolcont){
         var dscamiho = {
            "path": dadoAnexo.DSCAMINHODOCGERADOPDF
        };
        
        var currentRow = angular.copy(widget.currentRow);
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.NRSEQCONTATO = nrseqcontato;
        currentRow.NMARQUIVO = dadoAnexo.NMARQUIVO;
        currentRow.DSTIPO = dadoAnexo.DSTIPO;
        currentRow.VRTAMANHO = dadoAnexo.VRTAMANHO;
        currentRow.DSCAMINHO = dscamiho;
        currentRow.IDATIVO = "S";
        currentRow.NRORG = nrOrgOper;
        currentRow.CDOPERADOR = nrparcnegocioOperLogado;
        currentRow.IDORIGEM = "EVOLUCAO";
        currentRow.NRORIGEM = nrseqevolcont;
        
        return currentRow;
        
    };
   
    /**
     * Salva o Documento.
     * 
     * @param {object} widget - widget de Documento
     */
    this.salvarDocumento = function(widget) {
        widget.currentRow.__columns_to_return = ["NRSEQORCADOC", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO"];

        var isNew = widget.currentRow.__is_new;
        General.widgetSave(widget).then(function(success) {
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRSEQORCADOC = success.dataset.columnsToReturn[0].NRSEQORCADOC;
                widget.currentRow.NRORG = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            }
            if (isNew) {
                this.atualizarDocumento(widget, false);
            }
        }.bind(this));
    };


    /**
     * Atualiza o Documento, chamando a procedure FCT_SUB_MNE_DOC.
     * 
     * @param {object} widget - widget de Documento
     */
    this.atualizarDocumento = function(widget, fecharSwipe) {
        if (fecharSwipe == undefined) {
            fecharSwipe = true;
        }

        var params = {
            "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO,
            "NRSEQORCADOC": widget.currentRow.NRSEQORCADOC,
            "NRORG": widget.currentRow.NRORG
        };

        General.callRestEngine("product#/fct_orcamento_widget/atualizarDocumento", "Row", params).then(function() {
                ScreenService.showMessage("Processo realizado com sucesso!");
                if (fecharSwipe)
                    ScreenService.closeSwipe();
                widget.reload();
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
    };

    /**
     * Marca os elementos que já foram escolhidos ao clicar em editar.
     * 
     * @param {object} widget
     */
    this.marcarElementosEscolhidos = function(widget, elementos) {
        widget.dataSource.data.forEach(function(row) {
            if(row.TIPO == 'C') row.OLDCOMPPROPOSTA = 'N';
            elementos.forEach(function(elemento){
                if(elemento["IDELEMENTO"] == row["IDELEMENTO"] && elemento["NRCOMPSERVICO"] == row["NRCOMPSERVICO"]){
                    if(row.TIPO == 'C') {
                        row.OLDCOMPPROPOSTA = 'S';
                    }
                    row.__isSelected = true;
                    widget.dataSource.addCheckedRows(row);
                    row.CDCENTCUST = elemento["CDCENTCUST"];
                }
            });
        });
    };
    /**
     * Coloca a tela de Elementos da Proposta em modo de edição.
     * 
     * @param {object} widget
     */
    this.editarElementosProposta = function(widget) {
        widget.template  = "widget/master_detail/tree/grid.html";
        var wdgProposta = getWidgetProposta();
        var eleProposta = Util.clone(widget.dataSource.data, []);
        var params = [
            {
                "name": "NRORCAMENTOMODELO",
                "operator": "=",
                "value": wdgProposta.currentRow.NRORCAMENTOMODELO
            },
            {
                "name": "NRTIPOORCAMENTO",
                "operator": "IN",
                "value": wdgProposta.currentRow.NRTIPOORCAMENTO
            },
            {
                "name": "NRSEQORCAMENTO",
                "operator": "=",
                "value": wdgProposta.currentRow.NRSEQORCAMENTO
            }
        ];
        General.callRestEngine("product#/crud/composicao_elemento_proposta_inserida", 'FilterData', params).then(function(response){
            widget.dataSource.data = response.dataset.composicao_elemento_proposta_inserida;
            widget.edit();
            widget.showCheckbox = true;
            this.marcarElementosEscolhidos(widget, eleProposta);
            this.controlarAcessibilidadeTabbed("edit", false, widget);
        }.bind(this));
    };
    /**
     * Ação de cancelar dos Elementos da Proposta (editando).
     * 
     * @param {object} widget - Widget de Elementos da Proposta (editando)
     */
    this.cancelarElementosProposta = function(widget) {
        widget.dataSourceFilter[0].operator = "=";
        widget.dataSourceFilter[0].value    = "S";

        widget.list();
        widget.reload().then(function() {
            widget.showCheckbox = false;
            widget.unSelectAll();
            this.controlarAcessibilidadeTabbed("view", true, widget);
            widget.activate();
        }.bind(this));
    };
    /**
     * Volta para o grid de elementos da proposta inserida
    **/
    this.voltarGridElementos = function(widget){
        widget.dataSourceFilter[0].operator = "=";
        widget.dataSourceFilter[0].value    = "S";
        ScreenService.closeSwipe().then(function(){
            widget.reload().then(function() {
                widget.list();
                widget.showCheckbox = false;
            });
        });
    };

    /**
     * Atualiza os Elementos da Proposta.
     * 
     * @param {object} widget - Widget de Elementos da Proposta (editando)
     */
    this.atualizarComposicao = function(widget, tptratamento) {
        var rowProposta = templateManager.container.getWidget("wdgProposta").currentRow;
        var elementos   = [];
        var composicoes = [];

        widget.dataSource.data.forEach(function(row) {
            if(row.__isSelected){
                if (row.TIPO == "E") {
                    elementos.push({
                        "NRSEQORCAMENTO" : rowProposta.NRSEQORCAMENTO,
                        "NRCOMPSERVICO"  : row.NRCOMPSERVICO,
                        "NRITCOMPSERVICO": row.NRITCOMPSERVICO,
                        "CDSERVORCA"     : row.CDSERVORCA,
                        "IDELEMENTO"     : row.IDELEMENTO,
                        "IDELEMENTOPAI"  : this.getElementoPaiServico(row),
                        "IDESCOLHIDO"    : (row.__isSelected) ? "S" : "N",
                        "NRORG"          : row.NRORG,
                    });
                } else {
                    composicoes.push({
                        "NRSEQORCAMENTO"   : rowProposta.NRSEQORCAMENTO,
                        "NRCOMPSERVICO"    : row.CDELEMENTO,
                        "NMCOMPSERVICO"    : row.NMELEMENTO,
                        "NRCOMPSERVICOPAI" : row.IDARVOREPAI,
                        "IDCOMPOBRIGATORIA": row.IDSERVOBRIGCOMP,
                        "IDESCOLHIDO"      : (row.__isSelected) ? "S" : "N",
                        "NRORG"            : row.NRORG,
                        "OLDCOMPPROPOSTA"  : row.OLDCOMPPROPOSTA,
                        "CDCENTCUST"       : row.CDCENTCUST
                    });
                }
            }
        }.bind(this));

        var params = {
            "PROPOSTA"    : rowProposta,
            "ELEMENTOS"   : elementos,
            "COMPOSICOES" : composicoes,
            "TPTRATAMENTO": tptratamento
        };

        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(widget.dataSource.data, rowProposta.NRSEQORCAMENTO)
        .then(function(msg) {
            if (!msg) {
                General.callRestEngine("product#/fct_orcamento_widget/atualizarComposicaoProposta", "Row", params).then(function(response) {
                        if (response.messages[0].message != null) {
                            ScreenService.showMessage(response.messages[0].message);
                        }
                        this.cancelarElementosProposta(widget);
    
                        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                        this.atualizarValorProposta(wdgPropostaGrid);
    
                    }.bind(this),
                    function(error) {
                        console.error(error);
                        ScreenService.showMessage(error.data.error);
                    });
            }
            else {
                ScreenService.showMessage(msg);
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };

    this.getElementoPaiServico = function(row){
        if(row.__isSelected  && row._parent && row._parent.TIPO == 'C')
            return null;
        return row.IDARVORE;
    };
    /**
     * Seta o filtro da aba de Serviços dos Elementos da Proposta.
     * 
     * @param {object} widget - Widget de Serviços dos Elementos da Proposta
     */
    this.setFiltroServicosElementosProposta = function(widget) {
        var widgetElementosPropostas = getWidgetWdgElementosProposta();
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        widget.dataSourceFilter[0].value = widgetElementosPropostas.currentRow.NRSEQORCAMENTO;
        widget.dataSourceFilter[1].value = widgetElementosPropostas.currentRow.IDELEMENTO;
        widget.dataSourceFilter[2].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
        widget.reload();
    };

    /**
     * Sintetiza as funções em uma mesma função
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.funcoesDoCampoAjuste = function(widget) {
        this.controlarCampoValorDoAjuste(widget);
        this.calcularValorTotalItem(widget);
        if (widget.currentRow.VRPERCDESCORCA) {
            this.adequaAjusteAoCampo(widget);
        }
    };

    /**
     * Previne possiveis erros com a troca seguida no select native do campo Ajuste
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.adequaAjusteAoCampo = function(widget) {
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida        = (widget.currentRow.QTMEDIDAORCA) ? parseFloat(widget.currentRow.QTMEDIDAORCA.replace(",", ".")) : 0;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
        var ajuste            = widget.currentRow.VRPERCDESCORCA;

        var valorSemajuste = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemajuste *= qtdeMedida;
        }
        var valorTotal = valorSemajuste;
        if (ajuste) ajuste = parseFloat(ajuste.toString().replace(",", "."));
        if (tipoAjuste == "P" && ajuste > 100) {
            widget.currentRow.VRPERCDESCORCA = null;
            widget.currentRow.VRDESCORCA     = null;
        }
        if (tipoAjuste == "V" && ajuste > valorTotal) {
            widget.currentRow.VRPERCDESCORCA = null;
            widget.currentRow.VRDESCORCA     = null;
        }
    };
    /**
     * Valida se a alteração na quantidade de medida é permitida
     * @params{Object} row - currentRow itens
    **/
    this.validarAlteracaoQtMedida = function(row, nomeCampoMedida) {
        if (row.QTMEDIDACALC !== "" && row.QTMEDIDACALC !== null) {
            UtilitiesPropostaContrato.validarPermissaoAlteracaoQtMedida().then(function(permiteAlteracao) {  
                var qtmedida = row[nomeCampoMedida];
                if (!qtmedida) qtmedida = row.QTMEDIDAORCA;
    
                qtmedida = Utilities.tirarMascaraValores(qtmedida);
                var qtmedidacalc   = Utilities.tirarMascaraValores(row.QTMEDIDACALCULADA);
                var qtmedidabackup = Utilities.tirarMascaraValores(row.QTMEDIDACALC);
                
                if (qtmedida !== qtmedidacalc && row.IDTIPO == 'M') {
                    if (row.IDALTQTDMEDIDAPROP == "4") { //nao altera
                        ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
                        row.QTMEDIDASERV = qtmedidabackup.formatFloat(2);
                        row.QTMEDIDAORCA = qtmedidabackup.formatFloat(2);
                        return false;
                    } else if (row.IDALTQTDMEDIDAPROP == "3" && qtmedida > qtmedidacalc) { //somente reduzir
                        ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido aumentar a Quantidade de Medida deste item. O campo foi atribuído com o valor máximo (" + qtmedidacalc.formatFloat(2) + ").");
                        row.QTMEDIDASERV = qtmedidabackup.formatFloat(2);
                        row.QTMEDIDAORCA = qtmedidabackup.formatFloat(2);
                        return false;
                    } if (qtmedida < qtmedidacalc) { 
                        if(row.IDALTQTDMEDIDAPROP != "2") {
                            if(!permiteAlteracao) {
                                ScreenService.showMessage("Operação bloqueada.<br/>O operador logado não tem permissão para diminuir a quantidade de medida deste item.");
                                row.QTMEDIDASERV = qtmedidabackup.formatFloat(2);
                                row.QTMEDIDAORCA = qtmedidabackup.formatFloat(2);
                                return false;
                            }
                        } else {
                            if(permiteAlteracao)
                                ScreenService.showMessage("Operação bloqueada.<br/>O escopo de faturamento deste item não permite a diminuição de sua quantidade de medida.<br>Contate um administrador e tente novamente.");
                            else    
                                ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido reduzir a Quantidade de Medida deste item. O campo foi atribuído com o valor mínimo (" + qtmedidacalc.formatFloat(2) + ").");
                            
                            row.QTMEDIDASERV = qtmedidabackup.formatFloat(2);
                            row.QTMEDIDAORCA = qtmedidabackup.formatFloat(2);
                            return false;
                        }
                    }
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        return true;
    };

    /**
     * Calcula o valor total do item da proposta.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.calcularValorTotalItem = function(widget) {
        if (widget.mode != "list" && widget.mode != "inline_edit") {
            var quantidade        = widget.currentRow.QTITEMORCAMENTO;
            var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
            var qtdeMedida        = (widget.getField("QTMEDIDAORCA")) ? widget.currentRow.QTMEDIDAORCA : widget.currentRow.QTMEDIDASERV;
            var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
            var tipoAjuste        = widget.currentRow.IDTPDESCORCA;
            
            if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
            if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
            if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));

            var valorSemAjuste = quantidade * valorUnitario;
            if (utilizaQtdeMedida == "S") 
                valorSemAjuste *= qtdeMedida;
                
            var valorTotal = valorSemAjuste;
            
            var ajuste         = widget.currentRow.VRDESCORCA;
            var vrperdescorca  = widget.currentRow.VRPERCDESCORCA;
            
            if (ajuste) ajuste               = parseFloat(ajuste.toString().replace(",", "."));
            if (vrperdescorca) vrperdescorca = parseFloat(vrperdescorca.toString().replace(",", "."));
            if(!widget.currentRow.__is_new && ["P", "Y"].includes(tipoAjuste))
                ajuste = valorTotal * vrperdescorca/100;
            else
                vrperdescorca = ajuste * 100/valorSemAjuste;
            
            if(!widget.currentRow.IDTIPO || widget.currentRow.IDTIPO !== "I"){
                if (["V", "P"].includes(tipoAjuste) && (ajuste <= valorTotal || ajuste > 0)) {
                    if(ajuste < 0)
                        ajuste *= -1;
                        
                    valorTotal -= ajuste;
                } else if(["X", "Y"].includes(tipoAjuste)) {
                    valorTotal += ajuste;
                }
            }
            
            widget.currentRow.VRITEMSEMDESC   = valorSemAjuste;
            widget.currentRow.VRTOTALITEMORCA = valorTotal;
            widget.currentRow.VRDESCORCA      = (tipoAjuste !== "N" && ajuste) ? ajuste : null;
            widget.currentRow.VRPERCDESCORCA  = (tipoAjuste !== "N" && ajuste) ? vrperdescorca : null;
            
            templateManager.updateTemplate();
        }
    };
    
    /**
     * Realiza as validaçoes no campo conforme o tipo de ajuste selecionado.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.validarCampoPorcentagemValor = function(widget) {
        var tipoAjuste   = widget.currentRow.IDTPDESCORCA;
        var vrpercajuste = widget.currentRow.VRPERCDESCORCA;
        var ajuste       = widget.currentRow.VRDESCORCA;

        if (ajuste) ajuste             = parseFloat(ajuste.toString().replace(",", "."));
        if (vrpercajuste) vrpercajuste = parseFloat(vrpercajuste.toString().replace(",", "."));
        if(['P', 'Y'].includes(tipoAjuste))
            ajuste = vrpercajuste;
            
        var valorSemAjuste = widget.currentRow.VRITEMSEMDESC;
        if (tipoAjuste == "P") {
            if (ajuste < 0 || ajuste > 100) {
                ScreenService.showMessage("O valor percentual do desconto deve ser maior que 0 (zero) e inferior a 100%");
                widget.currentRow.VRPERCDESCORCA = null;
                widget.currentRow.VRDESCORCA     = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        } else if (tipoAjuste == "Y") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor percentual do acréscimo não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA  = null;
                widget.currentRow.VRDESCORCA      = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        } else if (tipoAjuste == "V") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA  = null;
                widget.currentRow.VRDESCORCA      = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            } else if (ajuste > valorSemAjuste) {
                ScreenService.showMessage("O valor do ajuste não pode ser superior ao valor total");
                widget.currentRow.VRPERCDESCORCA  = null;
                widget.currentRow.VRDESCORCA      = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        } else if (tipoAjuste == "X") {
            if (ajuste < 0) {
                ScreenService.showMessage("O valor do ajuste não pode ser inferior a 0 (zero)");
                widget.currentRow.VRPERCDESCORCA  = null;
                widget.currentRow.VRDESCORCA      = null;
                widget.currentRow.VRTOTALITEMORCA = valorSemAjuste;
            }
        }
    };

    /**
     * Controla o campo de desconto de acordo com o tipo.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.controlarCampoValorDoAjuste = function(widget) {
        var tipoAjuste      = widget.currentRow.IDTPDESCORCA ? widget.currentRow.IDTPDESCORCA : "N";
        var fieldAjuste     = widget.getField("VRDESCORCA");
        var fieldPercAjuste = widget.getField("VRPERCDESCORCA");
        var vrItemSAjuste   = widget.currentRow.VRITEMSEMDESC;
        if(vrItemSAjuste) vrItemSAjuste = parseFloat(vrItemSAjuste.toString().replace(",", "."));
        
        widget.getField('IDTPDESCORCA').readOnly = false;
        if (tipoAjuste == "N") {
            fieldAjuste.readOnly     = (tipoAjuste == "N");
            fieldPercAjuste.readOnly = (tipoAjuste == "N");
            fieldAjuste.value("");
            fieldPercAjuste.value("");
        } else {
            var readOnly = ["P", "Y"].includes(tipoAjuste);
            fieldPercAjuste.readOnly = !readOnly;
            fieldAjuste.readOnly     = readOnly;
            
            var ajuste       = fieldAjuste.value();
            var vrpercajuste = fieldPercAjuste.value();
            if(ajuste) ajuste             = parseFloat(ajuste.toString().replace(",", "."));
            if(vrpercajuste) vrpercajuste = parseFloat(vrpercajuste.toString().replace(",", "."));
            if(ajuste < 0) ajuste         = ajuste * -1;
            
            if(!readOnly){
                fieldPercAjuste.value("");
                if(ajuste) fieldPercAjuste.value(parseFloat(((ajuste * 100)/vrItemSAjuste).toFixed(2)));
            } else {
                fieldAjuste.value("");
                //if(vrpercajuste) fieldAjuste.value(parseFloat((vrItemSAjuste * (vrpercajuste/100)).toFixed(2)));
                if(vrpercajuste) fieldAjuste.value((vrItemSAjuste * (vrpercajuste/100)).toFixed(2).replace(".", ","));
            }
        }
    };

    /**
     * Novo Registro da aba de Itens da Proposta ou de Adicionais.
     * 
     * @param {object} widget - Widget de Itens da Proposta ou de Adicionais
     */
    this.itensNovoRegistro = function(widget) {
        
        ScreenService.showCustomDialog("Como deseja realizar a inclusão dos itens na proposta?", 'CONFIRMATION', null, [
            {
                "label": "Manualmente",
                "code": function() {
                    General.widgetNewRow(widget);
                    this.bloquearDadosMedida(widget);
                    var wdgProposta = getWidgetProposta();
                    var codEscopos  = wdgProposta.currentRow.NRTIPOORCAMENTO;
                    var nomeEscopos = wdgProposta.currentRow.NMTIPOORCAMENTO;
                    var idtipoorca  = wdgProposta.currentRow.IDTIPOORCA;
                    
                    if (!Array.isArray(codEscopos)) {
                        widget.currentRow.NRTIPOORCAMENTO = codEscopos;
                        widget.currentRow.NMTIPOORCAMENTO = nomeEscopos;
                    }
                    else if (codEscopos.length <= 1) {
                        widget.currentRow.NRTIPOORCAMENTO = codEscopos[0];
                        widget.currentRow.NMTIPOORCAMENTO = nomeEscopos[0];
                        widget.currentRow.IDTIPOORCA      = idtipoorca[0];
                        
                    } else {
                        var fieldCodEscopo  = widget.getField("NRTIPOORCAMENTO");
                        var fieldNomeEscopo = widget.getField("NMTIPOORCAMENTO");

                        fieldCodEscopo.template   = "field/select-multiple.html";
                        fieldCodEscopo.isVisible  = true;
                        fieldCodEscopo.showOnForm = true;
                        fieldCodEscopo.outData    = {
                            "IDTIPOORCA"      : "IDTIPOORCA",
                            "NRTIPOORCAMENTO" : "NRTIPOORCAMENTO",
                            "NMTIPOORCAMENTO" : "NMTIPOORCAMENTO"
                        };
                        
                        fieldNomeEscopo.isVisible  = false;
                        fieldNomeEscopo.showOnForm = false;
                    }

                    widget.currentRow.NRMOEDA = wdgProposta.currentRow.NRMOEDA;
                    
                    this.filtrarTipoDeItensManuaisDoEscopo(widget);
                }.bind(this)
            },
            {
                "label": "Formação de Preços",
                "code": function() {
                    this.abrirWdgFormacaoPrec();
                }.bind(this)
            },
            {
                "label": "Cancelar",
                "code": function() {}
            }
        ]);
    };

    /**
     * Verifica se a proposta utiliza composição de serviços para impedir alterações nas abas de Itens da Proposta e de Adicionais.
     * 
     * @param {object} widget - Widget de Itens da Proposta (editando) ou de Adicionais
     * @param {string} utilizaComposicao
     * @param {string} situacao
     */
    this.itensVerificarUtilizaComposicao = function(widget, utilizaComposicao, situacao) {
        var readOnly = (utilizaComposicao == "S");

        widget.fields.filter(function(field) {
                return field.composicao == true;
            })
            .forEach(function(field) {
                field.readOnly = readOnly;
            });

        if (situacao == "ELABORACAO") {
            widget.getAction("add").isVisible = !readOnly;
            widget.getAction("delete").isVisible = !readOnly;
        }
    };

    /**
     * Avança para a aba de Tipo de Medida Variável (inserindo).
     * 
     * @param {object} widget - Widget de Elementos (inserindo)
     */
    this.avancarTipoMedida = function(widget) {
        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(widget.dataSource.data)
        .then(function(msg) {
            if (!msg) {
                widget.enabled = false;
    
                var wdgProposta = templateManager.container.getWidget("wdgProposta");
                var idutilizavarpreco = wdgProposta.currentRow.IDUTILIZAVARPRECO;
    
                if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") > -1)) {
                    var wdgTipoMedidaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
                    wdgTipoMedidaInserindo.enabled = true;
                    wdgTipoMedidaInserindo.isVisible = true;
                    wdgTipoMedidaInserindo.activate().then(function() {
                        this.setFiltroTipoMedida(wdgTipoMedidaInserindo);
                    }.bind(this));
                }
                else {
                    this.avancarItensProposta();
                }
            }
            else {
                ScreenService.showMessage(msg);
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };

    /**
     * Seta o filtro da widget de Tipo de Medida.
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (inserindo)
     */
    this.setFiltroTipoMedida = function(widget) {
        var wdgItensComp = templateManager.container.getWidget("wdgItensComposicaoProposta");
        var idElementos = [];

        var idElementosRaiz = [];
        var elementosRaiz = [];
        wdgItensComp.dataSource.data.forEach(function(row) {
            if (row.TIPO == "E" && row.__isSelected && row._parent.TIPO == "C") {
                if (!idElementosRaiz.includes(row.IDELEMENTO)) {
                    idElementosRaiz.push(row.IDELEMENTO);
                    elementosRaiz.push(row);
                }
            }
        });
        
        wdgItensComp.dataSource.data.forEach(function(row) {
            if (row.TIPO == "E" && row.__isSelected) {
                idElementos.push(row.IDELEMENTO);
            }
        });

        idElementos = idElementos.filter(function(elem, i) {
            return idElementos.indexOf(elem) == i;
        });
        
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var nrtipoorcamento = wdgProposta.currentRow.NRTIPOORCAMENTO;
        var nrmoeda = wdgProposta.currentRow.NRMOEDA;
        
        var params = {
            "ID_ELEMENTOS"  : idElementos,
            "ELEMENTOS_RAIZ": elementosRaiz,
            "ESCOPOS"       : nrtipoorcamento,
            "NRMOEDA"       : nrmoeda
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/getTiposMedidaVariaveisParaInserir", "Row", params).then(function(success) {
            widget.dataSource.data = success.dataset.tipos_medida_variavel;
            widget.edit();
            widget.getAction("save").isVisible = false;
            widget.getAction("cancel").isVisible = false;
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Atualiza os tipos de medida variáveis da proposta.
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (editando)
     */
    this.atualizarTipoDeMedidaProposta = function(widget, idTpTratamento) {
        widget.saveRow().then(function() {
            var rowProposta = templateManager.container.getWidget("wdgProposta").currentRow;
            var params = {
                "PROPOSTA"     : rowProposta,
                "TPTRATAMENTO" : idTpTratamento
            };

            General.callRestEngine("product#/fct_orcamento_widget/atualizarItensPropostaPelosTiposDeMedida", "Row", params).then(function() {
                widget.list();
                widget.reload();
                var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                this.atualizarValorProposta(wdgPropostaGrid);
            }.bind(this),
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };

    /**
     * Salva os escopos da proposta.
     * 
     * @param {widget} widget - Widget de Escopos da Proposta
     */
    this.salvarEscopoProposta = function(widget) {
        widget.currentRow.__columns_to_return = ["NRRELACESCOPOPROPOSTA", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];

        General.widgetSave(widget).then(function(success) {
                if (success.dataset && success.dataset.columnsToReturn[0]) {
                    var columnsToReturn = success.dataset.columnsToReturn[0];
                    widget.currentRow.NRRELACESCOPOPROPOSTA = columnsToReturn.NRRELACESCOPOPROPOSTA;
                    widget.currentRow.NRORG = columnsToReturn.NRORG;
                    widget.currentRow.DTINCLUSAO = columnsToReturn.DTINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO = columnsToReturn.CDOPERINCLUSAO;
                    widget.currentRow.NRORGINCLUSAO = columnsToReturn.NRORGINCLUSAO;
                }
            })
            .catch(function(error) {
                console.error(error);
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage(error);
            });
    };

    /**
     * Desabilita a widget e avança para a aba de Itens da Proposta (inserindo).
     * 
     * @param {object} widget - Widget de Tipo de Medida da Proposta (inserindo)
     */
    this.desabilitarWidgetEAvancarItensProposta = function(widget) {
        widget.enabled = false;
        this.avancarItensProposta();
    };

    /**
     * Avança para a aba de Itens da Proposta (inserindo).
     */
    this.avancarItensProposta = function() {
        var wdgItensProposta = templateManager.container.getWidget("wdgItensProposta");
        wdgItensProposta.isVisible = false;

        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgItensComp = templateManager.container.getWidget("wdgItensComposicaoProposta");
        var wdgTipoMedidaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");

        var elementosEscolhidos = [];
        var composicoes = [];

        wdgItensComp.dataSource.data.forEach(function(row) {
            if (row.__isSelected && row.TIPO == "E") {
                elementosEscolhidos.push({
                    "IDELEMENTO": row.IDELEMENTO,
                    "NRCOMPSERVICO": row.NRCOMPSERVICO
                });
            }
            else if (row.TIPO == "C") {
                composicoes.push({
                    "NRCOMPSERVICO": row.CDELEMENTO,
                    "NMCOMPSERVICO": row.NMELEMENTO,
                    "NRCOMPSERVICOPAI": row.IDARVOREPAI,
                    "IDCOMPOBRIGATORIA": row.IDSERVOBRIGCOMP,
                    "CDCENTCUST" : row.CDCENTCUST,
                    "IDESCOLHIDO": row.__isSelected ? "S" : "N"
                });
            }
        });

        var params = {
            "PROPOSTA": wdgProposta.currentRow,
            //"ELEMENTOS": wdgItensComp.dataSource.data,
            "ELEMENTOS_ESCOLHIDOS": elementosEscolhidos,
            "COMPOSICOES": composicoes,
            "MEDIDAS": wdgTipoMedidaInserindo.dataSource.data
        };

        General.callRestEngine("product#/fct_orcamento_widget/getItensPropostaParaInserir", "Row", params).then(function(success) {
                var wdgItensPropostaInserindo = templateManager.container.getWidget("wdgItensPropostaInserindo");
                wdgItensPropostaInserindo.dataSource.data = success.dataset.fct_itemorcamento;
                this.habilitarWidgetNomeContratual(success.dataset.fct_itemorcamento);
        }.bind(this),
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /* Aplica o filtro de situação da proposta na tela de escopo 
    * para aparecer apenas o documento da situação atual
    * @params{Object} widget escopo proposta
    */
    this.setFilterSituacaoEscopo = function(widgetEscopoProposta){
        var wdgProposta = getWidgetProposta();
        var edititem = getActionEditEscopoProposta();
        var widgetProposta = templateManager.container.getWidget("wdgProposta");
        
        widgetEscopoProposta.dataSourceFilter[2].value = widgetProposta.currentRow.IDSITUACAOORCAMENTO;
        widgetEscopoProposta.reload();

        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO == "CONCLUIDO") {
            edititem.isVisible = visibilidadeAcoes;
        }
    };

    /**
     * Coloca a widget de Itens da Proposta em modo de edição.
     * 
     * @param {object} widget - Widget de Itens da Proposta (inserindo)
     */
    this.wdgItensPropostaInserindoOnActivate = function(widget) {
        widget.edit();
        /*setTimeout(function() {
            widget.getAction("save").isVisible = false;
            widget.getAction("cancel").isVisible = false;
        }, 200);*/
    };

    /**
     * Volta para a aba de Tipo de Medida Variável (inserindo) ou para a aba de Elementos (inserindo).
     * 
     * @param {object} widget - Widget de Itens da Proposta (inserindo)
     */
    this.voltarItensProposta = function(widget) {
        widget.enabled = false;
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var idutilizavarpreco = wdgProposta.currentRow.IDUTILIZAVARPRECO;

        if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") > -1)) {
            var wdgTipoMedidaPropostaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
            wdgTipoMedidaPropostaInserindo.enabled = true;

            //wdgProposta.openTab("wdgTipoMedidaPropostaInserindo");
            wdgTipoMedidaPropostaInserindo.activate();
        }
        else {
            var wdgItensComposicaoProposta = templateManager.container.getWidget("wdgItensComposicaoProposta");
            wdgItensComposicaoProposta.enabled = true;

            //wdgProposta.openTab("wdgItensComposicaoProposta");
            wdgItensComposicaoProposta.activate();
        }
    };

    /**
     * Seta o campo IDUTILIZAVARPRECO de acordo com os escopos selecionados.
     * 
     * @param {object} widget - Widget de Proposta
     */
    this.escoposOnChange = function(widget) {
        var fieldEscopo     = widget.getField("NRTIPOORCAMENTO");
        var nrtipoorcamento = fieldEscopo.value();
        var escopos         = fieldEscopo.dataSource.data;

        var idutilizavarpreco = [];
        escopos.forEach(function(row) {
            if (nrtipoorcamento.indexOf(row.NRTIPOORCAMENTO) > -1) {
                idutilizavarpreco.push(row.IDUTILIZAVARPRECO);
            }
        });

        if (idutilizavarpreco.length == 0)
            idutilizavarpreco = "";

        widget.currentRow.IDUTILIZAVARPRECO = idutilizavarpreco;
        if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") == -1)) {
            var wdgTipoMedidaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
            wdgTipoMedidaInserindo.isVisible = false;
            wdgTipoMedidaInserindo.enabled   = false;
        }
    };
    
    /*this.calcularValorTotalItensInlineInserindo = function(row) {
        if (row._parent) {
            this.calcularValorTotalItensInlineInserindoFilho(row);
        }
        else {
            this.calcularValorTotalItensInlineInserindoPai(row);
        }
    };

    this.calcularValorTotalItensInlineInserindoPai = function(row) {
        if (row.QTMEDIDASERV != "" && row.QTMEDIDASERV != row.QTMEDIDACALC) {
            ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
            row.QTMEDIDASERV = row.QTMEDIDACALC;
            row.QTMEDIDASERV_inline = row.QTMEDIDACALC;
        }
        else {
            row.VRTOTALITEMORCA = row.QTITEMORCAMENTO * row.VRUNITITEMORCA * (row.QTMEDIDAVARIAVEL ? parseFloat(row.QTMEDIDAVARIAVEL.toString().replace(",", ".")) : 1);
        }
    };
    var calcularValorUnitarioItemPai = function(row) {
        var widget = templateManager.container.getActiveWidget();
        
        var somaTotalFilhos = 0;
        widget.dataSource.data.forEach(function(item) {
            if (item.IDARVOREPAI == row.IDARVORE) {
                var valorTotal = item.VRTOTALITEMORCA;
                somaTotalFilhos += parseFloat(valorTotal.toString().replace(",", "."));
            }
        });
        
        row.changed = true;
        row.VRUNITITEMORCA = somaTotalFilhos;
    };
    
    /*this.calcularValorTotalItensInlineEditando = function(row, fieldName) {
        if (row._parent) 
            this.calcularValorTotalItensInlineEditandoFilho(row);
        else 
            this.calcularValorTotalItensInlineEditandoPai(row);
    };
    
    this.calcularValorTotalItensInlineEditandoPai = function(row) {
        if (Util.isEmptyOrBlank(row.changed_QTMEDIDAORCA)) {
            row.changed_QTMEDIDAORCA = row.QTMEDIDAORCA;
        }
        
        if (row.QTMEDIDAORCA != row.changed_QTMEDIDAORCA) {
            ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
            row.QTMEDIDAORCA = row.changed_QTMEDIDAORCA;
            row.QTMEDIDAORCA_inline = row.changed_QTMEDIDAORCA;
        }
        else {
            var quantidade = parseFloat(row.QTITEMORCAMENTO.toString().replace(",", "."));
            var quantidadeMedida = row.QTMEDIDAORCA ? row.QTMEDIDAORCA : 1;
            quantidadeMedida = parseFloat(quantidadeMedida.toString().replace(",", "."));
            var vrunititemorca = parseFloat(row.VRUNITITEMORCA.toString().replace(",", "."));

            var utilizaQtdeMedida = row.IDMULTMEDVRTOT;
            var tipoAjuste = row.IDTPDESCORCA;
            var ajuste = (row.VRPERCDESCORCA == null) ? 0 : row.VRPERCDESCORCA;
            ajuste = parseFloat(ajuste.toString().replace(",", "."));
            
            var valorSemAjuste = quantidade * vrunititemorca;
            if (utilizaQtdeMedida == "S") {
                valorSemAjuste *= quantidadeMedida;
            }
            
            var valorTotal = valorSemAjuste;
    
            if (tipoAjuste == "V" && ajuste <= valorTotal && ajuste >= 0) {
                valorTotal -= ajuste;
            }
            else if (tipoAjuste == "P" && ajuste <= 100 && ajuste >= 0) {
                valorTotal -= valorTotal * ajuste / 100;
            }
            else if (tipoAjuste == "X" && ajuste >= 0) {
                valorTotal += ajuste;
            }
            else if (tipoAjuste == "Y" && ajuste >= 0) {
                valorTotal += valorTotal * ajuste / 100;
            }
    
            row.VRITEMSEMDESC = valorSemAjuste;
            row.VRTOTALITEMORCA = valorTotal;
        }
    };
    
    this.calcularValorTotalItensInlineEditandoFilho = function(row) {
        row.QTMEDIDAORCA = row.QTMEDIDAORCA_GRID;
        
        var qtdeVariacao  = row.QTMEDIDAVARIAVEL;
        var multiplicaVariacao = (row.IDMULTIPVARIACAO == "S");
        
        if (qtdeVariacao == undefined || qtdeVariacao == "")
            qtdeVariacao = 1;
        
        if (row.QTMEDIDAORCA != row.QTMEDIDACALC) {
            if (row.IDALTQTDMEDIDAPROP == "4") { //nao altera
                ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido alterar a Quantidade de Medida deste item.");
                row.QTMEDIDAORCA = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_inline = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID_inline = row.QTMEDIDACALC;
            }
            else if (row.IDALTQTDMEDIDAPROP == "3" && row.QTMEDIDAORCA > row.QTMEDIDACALC) { //somente reduzir
                ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido aumentar a Quantidade de Medida deste item. O campo foi atribuído com o valor máximo (" + row.QTMEDIDACALC.formatFloat(2) + ").");
                row.QTMEDIDAORCA = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_inline = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID_inline = row.QTMEDIDACALC;
            }
            else if (row.IDALTQTDMEDIDAPROP == "2" && row.QTMEDIDAORCA < row.QTMEDIDACALC) { //somente aumentar
                ScreenService.showMessage("Operação bloqueada.<br/>Não é permitido reduzir a Quantidade de Medida deste item. O campo foi atribuído com o valor mínimo (" + row.QTMEDIDACALC.formatFloat(2) + ").");
                row.QTMEDIDAORCA = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_inline = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID = row.QTMEDIDACALC;
                row.QTMEDIDAORCA_GRID_inline = row.QTMEDIDACALC;
            }
        }
        
        row.VRTOTALITEMORCA = row.QTITEMORCAMENTO * row.QTMEDIDAORCA * row.VRUNITITEMORCA;
        if (multiplicaVariacao) {
            row.VRTOTALITEMORCA = row.VRTOTALITEMORCA * qtdeVariacao;
        }
        
        calcularValorUnitarioItemPai(row._parent);
        this.calcularValorTotalItensInlineEditando(row._parent);
    };*/
    
    /**
     * Calcula a quantidade de medida do item de acordo com a soma
     * da quantidade de medida dos seus itens filhos.
     */
    this.calcularValorQuantidadeMedidaItem = function(row, nomeCampoMedida) {
        var widget = templateManager.container.getActiveWidget();
        
        var somaMedidaFilhos = 0;
        widget.dataSource.data.forEach(function(item) {
            if (item.IDARVOREPAI == row.IDARVORE) {
                var qtd = item[nomeCampoMedida];
                somaMedidaFilhos += parseFloat(qtd.toString().replace(",", "."));
                //parseFloat(vrunititemorca.toString().replace(",", "."))
            }
        });
        
        row.changed = true;
        row[nomeCampoMedida] = somaMedidaFilhos;
    };

    /**
     * Salva a Proposta e seus Elementos, Itens e Tipos de Medida Variáveis.
     */
    this.inserirTudo = function() {
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        wdgProposta.currentRow.__columns_to_return = ["NRSEQORCAMENTO", "NRORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        // Usar para inserir o interlocutor no final    
        var nrSeqInterLocutor = null; 
        var nrOrcamentoModelo = null; 
        var nrseqorcamento = null;
        
        this.salvarWidgetProposta(wdgProposta).then(function(success) {
            nrseqorcamento = success.dataset.columnsToReturn[0].NRSEQORCAMENTO;

            wdgProposta.currentRow.NRSEQORCAMENTO = nrseqorcamento;
            wdgProposta.currentRow.NRORCAMENTO = success.dataset.columnsToReturn[0].NRORCAMENTO;

            var wdgTipoMedidaInserindo = templateManager.container.getWidget("wdgTipoMedidaPropostaInserindo");
            var medidas = [];

            var idutilizavarpreco = wdgProposta.currentRow.IDUTILIZAVARPRECO;
            if ((idutilizavarpreco) && (idutilizavarpreco.indexOf("S") > -1)) {
                medidas = wdgTipoMedidaInserindo.dataSource.data;
            }

            var wdgItensComp = templateManager.container.getWidget("wdgItensComposicaoProposta");

            var wdgItensPropostaInserindo = templateManager.container.getWidget("wdgItensPropostaInserindo");
            var itens = wdgItensPropostaInserindo.dataSource.data;

            var params = {
                "NRSEQORCAMENTO"   : nrseqorcamento,
                "NRTIPOORCAMENTO"  : wdgProposta.currentRow.NRTIPOORCAMENTO,
                "NRORCAMENTOMODELO": wdgProposta.currentRow.NRORCAMENTOMODELO,
                "MEDIDAS"          : medidas,
                "ITENS"            : itens,
                "INTERLOCUTOR"     : wdgProposta.currentRow.NRSEQINTERLOCUTOR
            };
            
            nrOrcamentoModelo = wdgProposta.currentRow.NRORCAMENTOMODELO;
            nrSeqInterLocutor = wdgProposta.currentRow.NRSEQINTERLOCUTOR;
            nrSeqInterLocutorfat = wdgProposta.currentRow.NRSEQINTERLOCUTORFAT;
            
            var currentRow = {
                "IDSITUACAOORCAMENTO" : "ELABORACAO",
                "NRSEQORCAMENTO"      : nrseqorcamento,
                "NRTIPOORCAMENTO"     : wdgProposta.currentRow.NRTIPOORCAMENTO,
                "IDUTILIZAVARPRECO"   : wdgProposta.currentRow.IDUTILIZAVARPRECO,
                "IDUTILIZACOMP"       : wdgProposta.currentRow.IDUTILIZACOMP,
                "QTITENSTRABSERV"     : 2,
                "NRORG"               : success.dataset.columnsToReturn[0].NRORG
            };
  
            General.callRestEngine("product#/fct_orcamento_widget/inserirTudo", "Row", params).then(function() {
                    this.voltarParaPropostaGrid().then(function() {
                        var wdgPropostaGrid        = getWidgetPropostaGrid();
                        wdgPropostaGrid.currentRow = currentRow;
                        this.wdgPropostaGridAfterMoveRow(wdgPropostaGrid);
                        //this.abrirWidgetNegociacaoValores(currentRow);
                    }.bind(this));
                }.bind(this))
                .catch(function(error) {
                    console.error(error);
                    var msg = "Timeout";
                    if (error.status !== 0 && error.data)
                        msg = error.data.error;

                    ScreenService.showMessage(msg);
                    this.voltarParaPropostaGrid();
                }.bind(this))
                .finally(function() {
                    wdgProposta.view();
                    wdgProposta.template = "widget/form-view.html";
                    this.alterarAcessibilidadeAbasProposta(true);

                    wdgItensComp.isVisible = false;
                    wdgTipoMedidaInserindo.isVisible = false;
                    wdgItensPropostaInserindo.isVisible = false;

                    wdgProposta.enabled = true;

                    var wdgItensProposta = templateManager.container.getWidget("wdgItensProposta");
                    wdgItensProposta.isVisible = true;
                    wdgItensProposta.enabled = true;

                    var wdgPropostaTabbed = templateManager.container.getWidget("wdgPropostaTabbed");
                    wdgPropostaTabbed.isVisible = true;
                    wdgPropostaTabbed.enabled = true;
                    templateManager.container.getWidget("wdgTabbedPrincipal").view();


                    var wdgEscoposProposta = templateManager.container.getWidget("wdgEscoposProposta");
                    wdgEscoposProposta.isVisible = true;
                    
                    var wdgNomeContratual = templateManager.container.getWidget("wdgNomeContratual");
                    wdgNomeContratual.list();
                    wdgNomeContratual.enabled = true;
                    
                    //var wdgLocalAtendimento = templateManager.container.getWidget("localAtendProposta");
                    //wdgLocalAtendimento.isVisible = true;
                     var paramsInt = {
                    "NRSEQORCAMENTO": nrseqorcamento,
                    "NRSEQINTERLOCUTOR": nrSeqInterLocutor,
                    "NRPARCINTERLOCUTORFAT": nrSeqInterLocutorfat,
                    "NRINTERLOCUTORANTE": null,
                    "NRORCAMENTOMODELO": nrOrcamentoModelo
                };
                General.callRestEngine("product#/fct_orcamento_widget/insereInterlocutor", "Row", paramsInt)
                .then(null)
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
                this.validaIdCertsign();
                }.bind(this));
        }.bind(this),
        function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
        // this.salvarProposta(wdgProposta);
    };

    /**
     * Gera o relatório de memória de cálculo da proposta.
     * 
     * @param {object} wdgProposta
     */
    this.gerarRelatorioMemoriaCalculo = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar o relátorio de memória de cálculo?"), 'question', function(){
            var nrseqorcamento = wdgProposta.currentRow.NRSEQORCAMENTO;
            var nrorg          = wdgProposta.currentRow.NRORG;
            var nomeRelatorio  = "memoria_calculo_proposta";
            var params = {
                "NRSEQORCAMENTO": nrseqorcamento,
                "NRORG": nrorg
            };
    
            Utilities.openReport("product#/geraBirtReport", nomeRelatorio, params);
            ScreenService.closePopup();
        }.bind(this));
    };

    this.validaPerfilEndereco = function () {
        var wdgProposta = getWidgetProposta();
        var newitem = getActionAddEndereco();
        var edititem = getActionEditEndereco();
        if(wdgProposta.currentRow.IDSITUACAOORCAMENTO == "CONCLUIDO") {
            newitem.isVisible = visibilidadeAcoes;
            edititem.isVisible = visibilidadeAcoes;
        }
    }

    /**
     * Executa validações da proposta ao trocar de linha.
     * 
     * @param {object} wdgPropostaGrid - Widget de Proposta (grid)
     */
    this.wdgPropostaGridAfterMoveRow = function(wdgPropostaGrid) {
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        wdgTabbedPrincipal.descriptionOrder = wdgPropostaGrid.descriptionOrder;

        var nrseqorcamento = wdgPropostaGrid.currentRow.NRSEQORCAMENTO;
        var nrorg          = wdgPropostaGrid.currentRow.NRORG;

        setLocalVar("PROPOSTA_NRSEQORCAMENTO", nrseqorcamento);
        setLocalVar("PROPOSTA_NRORG", nrorg);

        if(wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO == 'CONCLUIDO')
            this.alterarVisibilidadeActionEditProposta(wdgPropostaGrid, nrorg);
        
        if(idAlteraPropConc) {
            this.validacoesGeraisProposta(wdgPropostaGrid);
        } else {
            this.verificaParamAlteraPropConc(wdgPropostaGrid).then(function(){
                this.validacoesGeraisProposta(wdgPropostaGrid);
            }.bind(this));
        }
        
        
        this.desobrigaCamposEvolucaoEmail();
        this.abrirSwipeProposta();
    };
    /**
     * Valida e muda a visibilidade da action de edição da aba de dados da proposta em 
     * um orcamento concluida
    **/
    this.alterarVisibilidadeActionEditProposta = function(widget, nrorg) {
        if(!idAlteraPropConc) {
            this.verificaParamAlteraPropConc(widget).then(function(){
                this.alterarVisibilidadeActionEditProposta(widget, nrorg);
            }.bind(this));
        } else {
            if(idAlteraPropConc == 'N') {
                getActionEditWidgetProposta().isVisible = false;
            } else {
                if(visibilidadeAcoes !== null) {
                    getActionEditWidgetProposta().isVisible = visibilidadeAcoes;
                } else {
                    this.validarPermissaoAlteracaoDoPerfilAcesso(widget, nrorg).then(function(){
                        getActionEditWidgetProposta().isVisible = visibilidadeAcoes;
                    });
                }
            }
            
            if(idOperMaster === null && idUtilizaHierOper === null)
                this.buscaInformacaoHierarquia();
        }

        this.desobrigaCamposEvolucaoEmail();
    };
    /**
     * valida se o perfil de acesso do operador pode editar as propostas 
     * @params{Object} widget - widget de propostas 
     * @params{integer} nrorg - número da organização 
    **/
    this.validarPermissaoAlteracaoDoPerfilAcesso = function (widget, nrorg) {
        var deferred = ZHPromise.defer();
        var nrparcnegocioOperLogado = ["NRPARCNEGOCIO"];
        Operador_Repository.getOperadorLogado().then(function (success) {
            nrparcnegocioOperLogado[0] = success[0].NRPARCNEGOCIO;
            nrparcnegocioOperLogado[1] = nrorg;
            nrparcnegocioOperLogado[2] = 'P';
            General.callRestEngine('product#/fct_parametrogeral_widget/validarperfilacesso', "Row", nrparcnegocioOperLogado).then(function (perfilusuario) {
                visibilidadeAcoes = perfilusuario.dataset.fct_parametrogeral_widget[0];
                getActionEditWidgetProposta().isVisible = visibilidadeAcoes;
                deferred.resolve();
            }).catch(function(error){
                deferred.reject(error);
            });
        });
        
        return deferred.promise;
    };
    
    this.desobrigaCamposEvolucaoEmail = function() {
       var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        is_required = this.alternaCampo(wdgEnviarEmail, false);
        wdgEnviarEmail.reload();
    };
    /**
     * Abre a widget de Dados da Proposta (view/form)
     */
    this.abrirSwipeProposta = function() {
        var wdgProposta        = templateManager.container.getWidget("wdgProposta");
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");

        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            wdgTabbedPrincipal.openTab("wdgPropostaTabbed");
            wdgProposta.list();
            wdgProposta.reload().then(function() {
                wdgProposta.view();
                wdgProposta.activate();
                wdgTabbedPrincipal.view();
            });
        });
    };

    /**
     * Seta a currentRow com o registro retornado do banco de dados.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.setDadosProposta = function(wdgProposta) {
        wdgProposta.currentRow = wdgProposta.dataSource.data[0];
    };
    
    /**
     * Deleta o contato gerado para as propostas não ligadas a uma oportunidade
     * @params{Object} row currentRow excluida da proposta
    **/
    this.deletaContatoGerado = function(row){
        if(!row.NRSEQOPORT && row.NRSEQCONTATO){
            var params = {
                "NRSEQCONTATO": row.NRSEQCONTATO
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/deletaContatoProposta", "Row", params);
        }
    };

    /**
     * Exclui a Proposta.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.excluirProposta = function(wdgProposta) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function() {
            Utilities.validarCamposDuplicados(wdgProposta, null, ["NRSEQCONTATO", "NRORG"], "product#/evolcont_parcnegocio").then(function(valida){
                if(!valida){
                    ScreenService.confirmMessage("Não é possível excluir a proposta porque ela possui registros filhos. Deseja inativar a proposta?", "question",
                    function() {
                        General.callRestEngine("product#/fct_orcamento_widget/inativaPropContato", "Row", wdgProposta.currentRow)
                        .then(null)
                        .catch(function(error){
                            Utilities.showErrorMessage(error);
                        });
                    });
                } else {
                    var params = {
                        "NRSEQORCAMENTO": wdgProposta.currentRow.NRSEQORCAMENTO
                    };
                    General.callRestEngine("product#/fct_orcamento_widget/buscarLinksDocumentosProposta", "Row", params).then(function(response) {
                        
                        wdgProposta.removeRow().then(function() {
                            this.voltarParaPropostaGrid();
                            var links = response.dataset.links;
                            var params2 = [];
                            links.forEach(function(link) {
                                params2.push({
                                    "CAMINHO": link
                                });
                            });

                            General.callRestEngine("product#/file_server/excluirArquivos", "DataSet", params2).then(null).catch(function(error) {
                                Utilities.showErrorMessage(error);
                            });
                            //this.deletaContatoGerado(row);
                        }.bind(this),
                        function(error) {
                            if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                                //ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                                ScreenService.confirmMessage("Não é possível excluir a proposta porque ela possui registros filhos. Deseja inativar a proposta?", "question",
                                function() {
                                    this.inativarProposta(wdgProposta);
                                }.bind(this),
                                null);
                            } else {
                                Utilities.showErrorMessage(error);
                            }
                        }.bind(this));
                    }.bind(this));
                }
            }.bind(this));
        }.bind(this));
    };
    
    this.inativarProposta = function(wdgProposta) {
        wdgProposta.currentRow.changed = true;
        wdgProposta.currentRow.IDATIVO = "N";
        General.widgetSave(wdgProposta).then(function() {
            this.voltarParaPropostaGrid();
        }.bind(this),
        function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };
    
    /**
     * Gera um contato para a proposta caso ela não esteja ligada a uma oportunidade
     * @params{Object} widget widget proposta
    **/
    this.geraContatoProposta = function(row){
        var params = {
            "NRSEQORCAMENTO"    : row.NRSEQORCAMENTO,
            "DSREFCONTATO"      : row.NRORCAMENTO            ,
            "NRPARCNEGOCIO"     : row.NRPARCNEGOCIO,
            "NRPARCINTERLOCUTOR": row.NRPARCINTERLOCUTOR,
            "NRTIPOORCAMENTO"   : row.NRTIPOORCAMENTO
        };
        
        return General.callRestEngine("product#/fct_orcamento_widget/geraContato", "Row", params);
    };
    
    /**
     * verifica se ha alteração do contato da proposta
     * @params{Object} widget widget proposta
    **/
    this.verificaAlteracaoContato = function(widget){
        var deferred = ZHPromise.defer();
        
        if(!widget.currentRow.__is_new){
            if(widget.currentRow.NRSEQCONTATO !== widget.backupRow.NRSEQCONTATO){
                ScreenService.confirmMessage("A alteração do contato vinculado a essa proposta pode gerar a perda das evoluções atuais. Deseja continuar?", "question", 
                function(){
                    deferred.resolve();
                }, function(){
                    widget.currentRow.NRSEQOPORT   = widget.backupRow.NRSEQOPORT;
                    widget.currentRow.NRSEQCONTATO = widget.backupRow.NRSEQCONTATO;
                    deferred.resolve();
                });
            } else {
                deferred.resolve();
            }
        }
        else {
            deferred.resolve();
        }
        
        return deferred.promise;
    };
    
    /**
     * Salva a proposta.
     * 
     * @param {object} wdgProposta - Widget de Dados da Proposta (view/form)
     */
    this.salvarWidgetProposta = function(wdgProposta) {
        var deferred   = ZHPromise.defer();
        var contato    = wdgProposta.currentRow.NRSEQCONTATO; 
        var row        = wdgProposta.currentRow;
        var isnew      = wdgProposta.currentRow.__is_new;
        
        this.verificaAlteracaoContato(wdgProposta).then(function(){
            if (wdgProposta.currentRow.__is_new) {
                if (wdgProposta.paramsNewCode && wdgProposta.paramsNewCode.__is_new_code) {
                    wdgProposta.currentRow.__using_unit = wdgProposta.paramsNewCode.__using_unit || false;
                    wdgProposta.currentRow.__new_code_pk = wdgProposta.paramsNewCode.__new_code_pk;
                    wdgProposta.currentRow.__table_pk = wdgProposta.paramsNewCode.__table_pk;
                    wdgProposta.currentRow.__new_code_length = wdgProposta.paramsNewCode.__new_code_length || 5;
                    wdgProposta.currentRow.__cast_number = wdgProposta.paramsNewCode.__cast_number || false;
                }
            }
            
            var rowProposta = {
                "NRORCAMENTOMODELO": wdgProposta.currentRow.NRORCAMENTOMODELO,
                "NRSEQORCAMENTO"   : wdgProposta.currentRow.NRPARCNEGOCIO,
                "NRORG"            : wdgProposta.currentRow.NRORG
            };
            
            wdgProposta.saveRow().then(function(response) {
                    // wdgProposta.view();
                    // wdgProposta.template = "widget/form-view.html";
                    // this.alterarAcessibilidadeAbasProposta(true);
                    if(response.dataset){
                        rowProposta.NRSEQORCAMENTO = response.dataset.NRSEQORCAMENTO[0];
                        row.NRSEQORCAMENTO         = response.dataset.NRSEQORCAMENTO[0];
                        row.NRORCAMENTO            = response.dataset.columnsToReturn[0].NRORCAMENTO;
                        this.verificaIntegracaoPropOpor(rowProposta);
                        if(!contato && isnew)
                            this.geraContatoProposta(row);
                    }
                    
                    templateManager.container.getWidget("wdgTabbedPrincipal").view();
                    deferred.resolve(response);
                    
                }.bind(this),
                function(error) {
                    deferred.reject(error);
                });
        }.bind(this));
        
        return deferred.promise;
    };

    /**
     * Ativa a widget de Proposta (grid)
     * 
     * @param {widget} widget - Widget de Proposta (grid)
     */
    this.wdgPropostaGridOnEnter = function(widget) {
        widget.activate();
    };

    /**
     * Busca o endereço pelo CEP.
     */
    this.buscarEndereco = function(widget) {
        var fields = ["CDPAIS", "NMPAIS", "SGESTADO", "NMESTADO", "CDMUNICIPIO", "NMMUNICIPIO", "CDBAIRRO", "NMBAIRROENDERECO", "CDLOGRADOURO", "DSLOGRADOURO", "DSENDERECO"];
        
        if(widget.currentRow.NRCEPENDERECO){
            Utilities.buscarCep(widget.currentRow.NRCEPENDERECO).then(function(response) {
                    var endereco = response.dataset.ENDERECO;
                    fields.forEach(function(field) {
                        widget.currentRow[field] = (endereco[field]) ? endereco[field] : "";
                    });
                    
                    if(response.messages){
                        var msg = ScreenService.i18n(response.messages[0].message);
                        ScreenService.showMessage(msg);
                    }
                    
                    angular.element('#footer').click();;
                },
                function(error) {
                    console.error(error);
                    if (error.data){
                        var mensagem = error.data.error;
                        if(error.data.error === "Erro de configuração de proxy!")
                            mensagem = "Endereço não encotrado.";
                            
                        ScreenService.showMessage(mensagem);
                    } else {
                        ScreenService.showMessage("Falha na conexão.");
                    }
            });
        } else {
            fields.forEach(function(field) {
                widget.currentRow[field] = "";
            });
        }
    };

    this.cancelarItens = function(widget) {
        widget.list();
        widget.reload();
    };

    /**
     * Valida se os documentos podem ser gerados em DOCX
     * e pergunta com qual extensão o usuário deseja baixar
    **/
    this.validaParametrosDoc = function(widget, idescopo){
        General.callRestEngine("product#/fct_orcamento_widget/getParametroExtensaoDoc", "Row", {"NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO}).then(function(response){
            if(!response.dataset || !response.dataset.IDPERMITEDOCX){
                ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar o documento da proposta?"), 'question', function(){
                    this.gerarRelatorioDocumentoProposta(widget, idescopo, 'P');
                }.bind(this));
            } else {
                var permite = false;
                response.dataset.IDPERMITEDOCX.forEach(function(idpermite){
                    if(idpermite.IDPERMITEDOCX && idpermite.IDPERMITEDOCX === "S"){
                        permite = true;
                    }
                }.bind(this));

                if(!permite){
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente gerar o documento da proposta?"), 'question', function(){
                        this.gerarRelatorioDocumentoProposta(widget, idescopo, 'P');
                    }.bind(this));
                } else {
                    ScreenService.showCustomDialog("Alguns documentos da proposta podem ser gerados em DOCX.<br>Como deseja gerá-los?", 'CONFIRMATION', null, [{
                            "label": "Gerar PDF",
                            "code": function() {
                                this.gerarRelatorioDocumentoProposta(widget, idescopo, 'P');
                            }.bind(this)
                        }, {
                            "label": "Gerar DOCX",
                            "code": function() {
                                this.gerarRelatorioDocumentoProposta(widget, idescopo, 'D');
                            }.bind(this)
                        }, {
                            "label": "Cancelar",
                            "code": function() {}
                        }
                    ]);
                }
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    }

    /**
     * Gera o relatório do documento da proposta.
     */
    this.gerarRelatorioDocumentoProposta = function(widget, idescopo, idExtensao) {
        var nrseqorcamento  = widget.currentRow.NRSEQORCAMENTO;
        var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
        var idSituacao      = widget.currentRow.IDSITUACAOORCAMENTO;
        var nrorg           = widget.currentRow.NRORG;

        if (idescopo == "N") {
            nrtipoorcamento = 0;
        }
        
        if(!widget.currentRow.IDSITUACAOORCAMENTO){
            var widgetProposta = templateManager.container.getWidget("wdgProposta");
            idSituacao = widgetProposta.currentRow.IDSITUACAOORCAMENTO;
        }

        var params = {
            "NRSEQORCAMENTO"     : nrseqorcamento,
            "NRTIPOORCAMENTO"    : nrtipoorcamento,
            "NRORG"              : nrorg,
            "IDSITUACAOORCAMENTO": idSituacao,
            "IDPERMITEDOCX"      : idExtensao
        };

        var timeout = 5 * 60000; //5 min
        Utilities.callRestEngineWithTimeout("product#/fct_orcamento_widget/gerarDocumentoProposta", "Row", params, timeout).then(function(success) {
            var links = success.dataset.documentos.links;
            links.forEach(function(link) {
                if(link) window.open(window.backendIndexUrl + link);
            });
            
            if(success && success.messages && success.messages.length > 0 && success.messages[0].message)
                ScreenService.showMessage(success.messages[0].message);
                
            ScreenService.closePopup();
        }).catch(function(error) {
            console.error(error);
            var msg = "Timeout";
            if (error.status !== 0 && error.data)
                msg = error.data.error;
            ScreenService.showMessage(msg);
        });
    };


    /* ---=========  FUNÇÕES DA PÁGINA FORMAÇÃO PREÇO =========--- */


    /**
     * A função "abrirWdgFormacaoPrec" abre a tela "Itens de Formação de Preço"
     */

    this.abrirWdgFormacaoPrec = function() {
        ScreenService.closePopup();
        var formacaoPreco = templateManager.container.getWidget("itemFormacaoPreco");
        formacaoPreco.getField("NRCALCFORMPRECSERV").dataSource.clearCheckedRows();
        formacaoPreco.isVisible = true;
        ScreenService.openSwipe(formacaoPreco).then(function() {
            formacaoPreco.edit();
            this.filtraFormPreco(formacaoPreco);
            this.verificaAcessibilidadeFieldServ(formacaoPreco);
            
            var wdgProposta = getWidgetProposta();
            var codEscopos  = wdgProposta.currentRow.NRTIPOORCAMENTO;
            var nomeEscopos = wdgProposta.currentRow.NMTIPOORCAMENTO;
            
            if (!Array.isArray(codEscopos)) {
                formacaoPreco.currentRow.NRTIPOORCAMENTO = codEscopos;
                formacaoPreco.currentRow.NMTIPOORCAMENTO = nomeEscopos;
            } else if (codEscopos.length == 1) {
                formacaoPreco.currentRow.NRTIPOORCAMENTO = codEscopos[0];
                formacaoPreco.currentRow.NMTIPOORCAMENTO = nomeEscopos[0];
            } else {
                var fieldNomeEscopo = formacaoPreco.getField("NMTIPOORCAMENTO");
                fieldNomeEscopo.readOnly = false;
            }
        }.bind(this));
    };
    
    /**
     * Abre a widget que permite a edição do cálculo da formação de preço
     * @params{Object} widget widget Item Formação de preço
    **/
    this.abreWidgetEdicaoCalculoItens = function(widget) {
        var tabbedFormacao = templateManager.container.getWidget("wdgTabbedPrincipalFormPreco");
        var itensCalculo   = templateManager.container.getWidget("editaCalculoItensFormPreco");
        var widgetProposta = templateManager.container.getWidget("wdgProposta");
        
        ScreenService.closePopup();
        if(!widget.currentRow.NMFORMPRECOSERVICO || widget.currentRow.NMFORMPRECOSERVICO.length === 0){
            ScreenService.showMessage("Não é possível editar os itens do cálculo sem indicar uma Formação de Preço.<br> Para continuar selecione uma Formação de Preço <br>");
        } else {
            if(widget.currentRow.NMFORMPRECOSERVICO && widget.currentRow.NMFORMPRECOSERVICO.length > 1){
                ScreenService.showMessage("A edição do Cálculo de Formação de Preço só é permitida quando apenas um tipo de Formação de Preço é selecionado. <br>Para continuar escolha apenas uma Formação de Preço.<br>");
            } else {
                if(widget.currentRow.NRCALCFORMPRECSERV && widget.currentRow.NRCALCFORMPRECSERV.length > 0){
                    ScreenService.showMessage("Não é possível editar os itens do cálculo após selecionar os Itens de Formação de Preço.<br>Para continuar desfaça a seleção de itens atual.<br>");
                } else {
                    var params = {
                        "NRSEQORCAMENTO": widgetProposta.currentRow.NRSEQORCAMENTO,
                        "NRITEMORCAMENTO": null
                    };
                    
                    General.callRestEngine("product#/fct_relacestruturaprop_widget/deletaAlteracoesInutilizadas", "Row", params).then(function(){
                        this.salvarAlteracoesCalculo(itensCalculo, true, null).then(function(){
                            tabbedFormacao.isVisible = true;
                            itensCalculo.isVisible   = true;

                            ScreenService.openSwipe(tabbedFormacao).then(function() {
                                tabbedFormacao.openTab("editaCalculoItensFormPreco");
                                
                                itensCalculo.dataSourceFilter[0].value = widget.currentRow.NRFORMPRECOSERVICO[0];
                                itensCalculo.dataSourceFilter[2].value = templateManager.container.getWidget('wdgProposta').currentRow.NRSEQORCAMENTO;
                                Utilities.abreArvoreElementos(itensCalculo);
                                
                                UtilitiesPropostaContrato.habilitaEdicaoWidgetItemFormacao(widget, true);
                            });    
                        }.bind(this));    
                    }.bind(this));    
                }
            }
        }
    };
    
    /**
     * Habilita a tela para receber um novo item de formação de preço
     * e seta o número de ordenação na tela
     * @params{Object} widget widget editar formação de preço
    **/
    this.adicionarItensFormacao = function(widget){
        var nritemorcamento    = null;
        var nrformprecoservico = null;
        var widgetItensPropostaTree = getWidgetItensPropostaTree();
        var widgetItensProposta = getWidgetItensProposta();
        
        if(widget.mode === "view"){
            var nmElementoSup = widget.currentRow.DSREFECALCULO;
            var nrCalcServPai = widget.currentRow.NRCALCFORMPRECSERV;
            widget.newRow();
            this.bloquearDadosMedidaEdicao(widget);
            widget.currentRow.DSREFECALCULOPAI      = nmElementoSup;
            widget.currentRow.NRCALCFORMPRECSERVPAI = nrCalcServPai;
        } else {
            widget.newRow();
            this.bloquearDadosMedidaEdicao(widget);
        }

        if(widget.name === "editaItensFormPreco"){
            nritemorcamento    = widgetItensProposta.currentRow.NRITEMORCAMENTO;
            nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
        } else {
            nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            widget.template    = "widget/form.html";
        }
        
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "NRITEMORCAMENTO"    : nritemorcamento,
            "TABELA"             : (widget.dataSource.name === "product#/fct_calcformprecserv_widget") ? null : "FCT_CALCFORMPRECSERV_ITPRO"
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/recebeOrdenacao', "Row", params).then(function(response){
            if(response.dataset.ORDENACAO){
                var ordenacao = response.dataset.ORDENACAO.IDORDENACAO;
                ordenacao = ordenacao ? (ordenacao + 10) : 10;
                widget.currentRow.IDORDENACAO        = ordenacao;
                widget.currentRow.NRFORMPRECOSERVICO = nrformprecoservico;
                widget.currentRow.NRITEMORCAMENTO    = nritemorcamento;
            } else {
                widget.currentRow.IDORDENACAO = 10;
            }
        }, 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
         /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDUSAMEDIDA".
    */
    
    this.bloquearDadosMedidaEdicao = function(widget){
        var usamedida = widget.getField('IDUSAMEDIDA').value();
        
        /* Bloqueia a Modalidade de pagamento*/
        if (usamedida == 'N') {
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = true;
            widget.getField('DSTPMEDIDASERV').setValue(null);
            widget.getField('DSTPMEDIDASERV').validations = {};
            widget.getField('CDTPMEDIDASERV').setValue(null);
            widget.getField('CDTPMEDIDASERV').validations = {};
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = true;
            widget.getField('QTMEDIDACALC').setValue(null);
            widget.getField('QTMEDIDACALC').validations = {};
        } else {
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = false;
            widget.getField('DSTPMEDIDASERV').validations = {"required": true};
            /* Quantidade de Medida */
            widget.getField('QTMEDIDACALC').readOnly = false;
            widget.getField('QTMEDIDACALC').validations = {"required": true};
        }
    };
    
    /**
     * Filtra a field de elemento superior para aperecer apenas os elementos
     * já inseridos na árvore
     * @params{Object} widget widget editar formaçao de preço
    **/
    this.filtraElementoSuperiorItensFormacao = function(widget){
        var widgetPropostaGrid = getWidgetPropostaGrid();
        if(!widget.currentRow.DSREFECALCULOPAI){
            var nrformprecoservico = null;
            widget.currentRow.DSREFECALCULOPAI = "";
            
            if(widget.name === "editaItensFormPreco"){
                nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[3].value = getWidgetItensProposta().currentRow.NRITEMORCAMENTO;
            } else {
                nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            }
            
            if(widget.currentRow.NMELEMENTO){
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[0].value = nrformprecoservico;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[1].value = (widget.currentRow.NRCALCFORMPRECSERV) ? widget.currentRow.NRCALCFORMPRECSERV : -1;
                widget.getField("DSREFECALCULOPAI").dataSourceFilter[2].value = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;;             
                widget.getField("DSREFECALCULOPAI").reload();
            }
        }
    };
    
    /**
     * Salva no grid as alterações feitas no item de formação de preco
     * @params{Object} widget widget editar calculo de formação de preço
    **/
    this.salvarLocalmenteAlteracoesFormPreco = function(widget){
        if(widget.isValid()) {
            var isnew       = widget.currentRow.__is_new;
            var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
            
            widget.currentRow.NRSEQORCAMENTO     = templateManager.container.getWidget("wdgProposta").currentRow.NRSEQORCAMENTO;
            widget.currentRow.NRFORMPRECOSERVICO = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            widget.currentRow.NRITEMORCAMENTO    = null;
            
            if(!widget.currentRow.DSREFECALCULO)
                widget.currentRow.DSREFECALCULO = widget.currentRow.IDITEMCALCULO + " - " + widget.currentRow.NMELEMENTO;
            
            if(!widget.currentRow.NRRELACFORMACAOPROP)
                widget.currentRow.NRRELACFORMACAOPROP = 0;
                
            UtilitiesPropostaContrato.calculaItem(widget, "FCT_CALCFORMPRECSERV_ITPRO", widget.currentRow.NRITEMORCAMENTO, null).then(function(){    
                if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
                    widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
                    widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
                    widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
                    widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                    widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
                    if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                        widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
                }
                
                widget.saveRow().then(function(response) {
                    if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0)
                        widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
            
                    if (tipoCalculo == 'S'){
                        var params = {
                            "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                            "NRITEMORCAMENTO"    : widget.currentRow.NRITEMORCAMENTO,
                            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO"
                        };
                        
                        General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);  
                    }
                    
                    ScreenService.confirmMessage("Deseja recalcular a Formação de Preço?", 'question', 
                        function (success) {
                            widget.template = "widget/tree/grid.html";
                            widget.list();
                            this.recalcularArvore(widget);
                        }.bind(this),
                        function() {
                            if(!isnew) {
                                widget.currentRow.changed = false;
                                UtilitiesPropostaContrato.habilitarViewItensFormacao(widget);
                            } else {
                                widget.template = "widget/tree/grid.html";
                                widget.list();
                                Utilities.abreArvoreElementos(widget);
                            }
                        }
                    );
                }.bind(this));
            }.bind(this));
        }
    };
    
    /**
     * Realiza a validação  do campo "identificador". Não podem haver dois identificadores iguais em uma mesma composição.
     * 
     * @param {object} widget widget
     */

    this.validaIdentificador = function(widget){
        var widgetPropostaGrid = getWidgetPropostaGrid();
        var widgetItensProposta = getWidgetItensProposta();
        var nrformprecoservico = null;
        var nritemorcamento = null;
        var IdItemCalculo = widget.currentRow.IDITEMCALCULO;
        var nrCalcFormPrecServ = widget.currentRow.NRCALCFORMPRECSERV;
        /* === Ordenação === */
        
        if(widget.name === "editaItensFormPreco"){
            nritemorcamento    = widgetItensProposta.currentRow.NRITEMORCAMENTO;
            nrformprecoservico = widget.dataSource.data[0].NRFORMPRECOSERVICO;
        } else {
            nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
        }
        
        var params = {
            "NRFORMPRECOSERVICO": nrformprecoservico,
            "NRITEMORCAMENTO"   : nritemorcamento,
            "TABELA"            : (widget.dataSource.name === "product#/fct_calcformprecserv_widget") ? null : "FCT_CALCFORMPRECSERV_ITPRO",
            "IDITEMCALCULO"     : IdItemCalculo,
            "NRCALCFORMPRECSERV": nrCalcFormPrecServ
        };
        
        General.callRestEngine('product#/fct_calcformprecserv_widget/validaIdentificador', "Row", params).then(function(response){
            var identificador = response.dataset.IDENTIFICADOR.VALIDA;
            if (identificador != 0){
                widget.currentRow.NMELEMENTO = "",
                widget.currentRow.IDITEMCALCULO = null;
                ScreenService.showMessage('O identificador "'+IdItemCalculo+'" já está sendo utilizado para outro elemento.'); 
            }
            
        }.bind(this), 
        function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });    
    };
    
    /**
     * Salva os itens selecionados pelo usuário com as novas formaçoes de preço
     * @params{Object} widget widget editar itens formação de preço
    **/
    this.utilizarItensAlteradosFormPrec = function(widget){
        var widgetItensFomacao = templateManager.container.getWidget("itemFormacaoPreco");
        var widgetItensprop    = templateManager.container.getWidget("wdgItensProposta");
        var widgetProposta     = templateManager.container.getWidget("wdgProposta");
        var rows               = widget.getCheckedRows();
        var itensSelecionados  = [];
        var nrformprecoservico = widgetItensFomacao.currentRow.NRFORMPRECOSERVICO;
        var servico            = null;
        
        if(widgetItensFomacao.currentRow.CDSERVORCA)
            servico = widgetItensFomacao.currentRow.CDSERVORCA;
        else 
            servico = widgetItensFomacao.currentRow.CDSERVORCAFORM;
        
        if(widget.dataSource.data.length !== rows.length)
            rows = UtilitiesPropostaContrato.buscaArvoreFormacaoCompleta(widget, rows);
        
        rows.forEach(function(row) {
            row.NRSEQORCAMENTO = widgetProposta.currentRow.NRSEQORCAMENTO;
            if(rows.indexOf(row._parent) == -1) {
                widgetItensFomacao.getField("NRCALCFORMPRECSERV").dataSource.addCheckedRows(row);
                itensSelecionados.push(row.NRCALCFORMPRECSERV);
            }
        });

        widgetItensFomacao.currentRow.NRCALCFORMPRECSERV = itensSelecionados;
        widget.dataSource.data                           = rows;
        
        this.salvaItemformacaopreco(widgetItensFomacao).then(function(response){
            var nrItensProp = null;
            if(response)
                nrItensProp = response;
            
            UtilitiesPropostaContrato.habilitaEdicaoWidgetItemFormacao(widgetItensFomacao, false);
            this.salvarAlteracoesCalculo(widget, false, nrItensProp).then(function(){
                widget.dataSource.clearCheckedRows();
                ScreenService.closePopup();
                
                ScreenService.closeSwipe().then(function(){
                    ScreenService.closeSwipe().then(function(){
                        widgetItensprop.activate();
                        widgetItensprop.reload();
                    });
                });
            }.bind(this))
            .catch(function(error){
                if(nrItensProp && nrItensProp.length > 0) {
                    var itens = [];
                    nrItensProp.forEach(function(item){
                        itens.push(item.NRITEMORCAMENTO);
                    });
                    
                    var params = {
                        "NRSEQORCAMENTO": widgetProposta.currentRow.NRSEQORCAMENTO,
                        "NRITEMORCAMENTO": itens
                    };
                    
                    General.callRestEngine("product#/fct_relacestruturaprop_widget/deletarItensFormacaoProp", "Row", params).then(function(){
                        widgetItensFomacao.getField("NRFORMPRECOSERVICO").dataSource.addCheckedRows(nrformprecoservico);
                        widgetItensFomacao.currentRow.NRFORMPRECOSERVICO = nrformprecoservico;
                        widgetItensFomacao.currentRow.CDSERVORCAFORM     = servico;
                        
                        ScreenService.closePopup();
                        ScreenService.showMessage("Não foi possivel utilizar o(s) item(ns) selecionados.<br/>Erro durante a inserção do(s) mesmo(s) na proposta.");
                    }.bind(this));
                }
            });
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        }); 
    };
    
    /**
     * Manda recalcular a arvore de formação
     * @params{Object} widget widget formação de preço
     * @params{int} nrformprecoservico nr da formação de preço
    **/
    this.recalculaFormacaoItens = function(widget, nrformprecoservico, nritemorcamento){
        var deferred = ZHPromise.defer();
        var params = {
            "NRFORMPRECOSERVICO" : nrformprecoservico,
            "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO",
            "NRITEMORCAMENTO"    :  nritemorcamento
        };
        
        General.callRestEngine("product#/fct_calcformprecserv_widget/recalcularArvore", "Row", params).then(function() {
            Utilities.abreArvoreElementos(widget).then(function(){
                ScreenService.showMessage("Processo realizado com sucesso!");
                deferred.resolve();
            });
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject(error);
        });
        
        return deferred.promise;
    };
    
    
    /**
     * Recalcula a árvore de elementos
     * @params{Object} widget widget editar itens de formação de preço
     * .parent substituido pela widget da grid principal
    **/
    this.recalcularArvore = function(widget) {
          var widgetItensProposta = getWidgetItensProposta();
        if(widget.name === "editaItensFormPreco"){
            this.recalculaFormacaoItens(widget, widget.dataSource.data[0].NRFORMPRECOSERVICO, widgetItensProposta.currentRow.NRITEMORCAMENTO).then(function(){
                this.recalculaVrItem(widget, null, null);
            }.bind(this));
        } else {
            var nrformprecoservico = templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0];
            this.recalculaFormacaoItens(widget, nrformprecoservico, null);
        }
    };
    
    /**
     * salva as edições feitas nas formações auxiliares dos itens
     * @params{Object} widget widget editar itens formação na aba itens
    **/
    this.salvarEdicaoItFormacao = function(widget){
        var isNew = widget.currentRow.__is_new;
        var tipoCalculo = widget.currentRow.IDTIPOCALCULO;
        if(!widget.currentRow.DSREFECALCULO)
            widget.currentRow.DSREFECALCULO = widget.currentRow.IDITEMCALCULO + " - " + widget.currentRow.NMELEMENTO;
        
        if(!widget.currentRow.NRRELACFORMACAOPROP)
            widget.currentRow.NRRELACFORMACAOPROP = 0; 
            
        UtilitiesPropostaContrato.calculaItem(widget, "FCT_CALCFORMPRECSERV_ITPRO", widget.currentRow.NRITEMORCAMENTO, null).then(function(){
            General.widgetSave(widget).then(function(response){
                if(response.dataset && response.dataset.columnsToReturn.length > 0){
                    widget.currentRow.NRSEQORCAMENTO     = response.dataset.columnsToReturn[0].NRSEQORCAMENTO;
                    widget.currentRow.NRCALCFORMPRECSERV = response.dataset.columnsToReturn[0].NRCALCFORMPRECSERV;
                    widget.currentRow.NRFORMPRECOSERVICO = response.dataset.columnsToReturn[0].NRFORMPRECOSERVICO;
                    widget.currentRow.NRITEMORCAMENTO    = response.dataset.columnsToReturn[0].NRITEMORCAMENTO;
                    widget.currentRow.NRORG              = response.dataset.columnsToReturn[0].NRORG;
                }
                if (tipoCalculo == 'S'){
                    var params = {
                        "NRCALCFORMPRECSERV" : widget.currentRow.NRCALCFORMPRECSERV,
                        "NRITEMORCAMENTO"    : widget.currentRow.NRITEMORCAMENTO,
                        "TABELA"             : "FCT_CALCFORMPRECSERV_ITPRO"
                    };
                    General.callRestEngine('product#/fct_calcformprecserv_widget/calcSomatorioFilhos', "Row", params);  
                }
                ScreenService.confirmMessage("Deseja recalcular a Formação de Preço?", 'question', function (success) {
                    if(!isNew)
                        ScreenService.closeSwipe();
                    widget.list();
                    this.recalcularArvore(widget);
                }.bind(this),
                function(){
                    this.recalculaVrItem(widget, null, null);
                }.bind(this));
                
            }.bind(this))
            .catch(function(error){
                console.error(error);
            });
        }.bind(this));
    };
    
    /**
     * Verifica se o item do orcamento faz parte de uma formação de preço 
     * auxiliar para habilita a widget de edição de formações auxiliares
     * @params{Object} widget widget item orcamento
    **/
    this.validaWidgetEditaFormAux = function(widget){
        //if(getWidgetProposta().currentRow.IDUTILIZACOMP === 'N'){
        if(getWidgetProposta().currentRow.QTITENSTRABSERV == 0){
            templateManager.container.getWidget("wdgAdicionais").enabled = true
            var params = [
                {
                    name    : "NRITEMORCAMENTO",
                    operator: "=",
                    value   : widget.currentRow.NRITEMORCAMENTO
                }
            ];
            
            General.callRestEngine("product#/crud/fct_calcformprecserv_itpro_widget", "FilterData", params).then(function(response){
                var wdgFormacao = templateManager.container.getWidget("editaItensFormPreco");
                var situacao = templateManager.container.getWidget("wdgProposta").currentRow.IDSITUACAOORCAMENTO;
                
                if(response.dataset && response.dataset.fct_calcformprecserv_itpro_widget.length > 0){
                    wdgFormacao.isVisible = true;
                    wdgFormacao.enabled   = true;
                    if(situacao === "ELABORACAO" || (idAlteraPropConc === "S" && situacao === "CONCLUIDO"))
                        wdgFormacao.actions.forEach(function(action){
                            action.isVisible = true;
                        });
                    else
                        wdgFormacao.actions.forEach(function(action){
                            if(action.name !== "VoltarWdgFormPrec")
                                action.isVisible = false;
                        });
                        
                } else {
                    wdgFormacao.isVisible = false;
                }
            });
        }
    };
    
    /**
     * Exclui um elemento da formçaõ de preço auxiliar e manda
     * editar o valor do item
     * @params{Object} widget widget editar itens formação de preço
    **/
    this.excluirFormacaoItens = function(widget){
        var dado = [widget.currentRow];
        var params = [
            {
                name: "NRCALCFORMPRECSERVPAI",
                operator: "=",
                value: widget.currentRow.NRCALCFORMPRECSERV
            },
            {
                name: "NRSEQORCAMENTO",
                operator: "=",
                value: widget.currentRow.NRSEQORCAMENTO
            },
            {
                name: "NRITEMORCAMENTO",
                operator: "=",
                value: widget.currentRow.NRITEMORCAMENTO
            },
            {
                name: "NRFORMPRECOSERVICO",
                operator: "=",
                value: widget.currentRow.NRFORMPRECOSERVICO
            }
        ]

        General.callRestEngine("product#/crud/fct_calcformprecserv_itpro_widget", "FilterData", params).then(function(response){
            var mensagem;
            if(response.dataset && response.dataset.fct_calcformprecserv_itpro_widget && response.dataset.fct_calcformprecserv_itpro_widget.length > 0)
                mensagem = "Apagar este elemento levará a exclusão dos elementos inferiores a ele.<br>Confirma exclusão?";
            else 
                mensagem = "Delete Register?";

            ScreenService.confirmMessage(ScreenService.i18n(mensagem), "question", function(){
                params = {
                    "NRCALCFORMPRECSERV": widget.currentRow.NRCALCFORMPRECSERV,
                    "NRSEQORCAMENTO"    : widget.currentRow.NRSEQORCAMENTO,
                    "NRITEMORCAMENTO"   : widget.currentRow.NRITEMORCAMENTO,
                    "NRFORMPRECOSERVICO": widget.currentRow.NRFORMPRECOSERVICO
                };

                General.callRestEngine("product#/fct_relacestruturaprop_widget/deletarArvoreFormPrecoItPro", "Row", params).then(function(){
                    ScreenService.confirmMessage(ScreenService.i18n("Deseja recalcular a Formação de Preço?"), 'question', function (success) {
                        this.recalcularArvore(widget);
                    }.bind(this));
                    ScreenService.closeSwipe().then(function () {
                        widget.activate();
                        this.recalculaVrItem(widget, dado, "DELETE");
                    }.bind(this));
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }.bind(this));
        }.bind(this));
    };
    
    /**
     * Edita o valor do item quando a formação de preço 
     * é alterada.
     * @params{Object} widget widget editar itens formação de preco
    **/
    this.recalculaVrItem = function(widget, elementos, acao){
        var widgetItensProposta = getWidgetItensProposta();
        var valor  = 0;

        if(!elementos){
            elementos = widget.dataSource.data.filter(function(dado){
                return dado.NRCALCFORMPRECSERVPAI === null;
            });
        }

        elementos.forEach(function(elemento){
            valor = valor + elemento.VRTOTAL;
        });

        if(acao === "DELETE")
            valor = widgetItensProposta.currentRow.VRUNITITEMORCA - elementos[0].VRTOTAL;

        if(widgetItensProposta.currentRow.VRUNITITEMORCA !== valor){
            widgetItensProposta.currentRow.VRUNITITEMORCA = valor;
            this.calcularValorTotalItem(widgetItensProposta);
            this.salvarItens(widgetItensProposta).then(function(){
                widget.activate();
                Utilities.abreArvoreElementos(widget);
            }).catch(function(error){
                ScreenService.showMessage("Não foi possivel atualizar o valor do item da proposta.");
            });
        }
    };
    
    /**
     * Seta o número do item gerado durante a inserção do item na proposta
     * @params{Object} widget widget editar formaçao de preço
     * @params{Array} nrItensProp array de nr de itens gerados
    **/
    this.setNrItemFormacaoAux = function(widget, nrItensProp){
        var rows = [];
        if(nrItensProp === null) {
            widget.dataSource.data.forEach(function(dado){ 
                dado.NRITEMORCAMENTO = null;
            });
        } else {
            if(nrItensProp.length === 1) {
                widget.dataSource.data.forEach(function(dado){
                    dado.NRITEMORCAMENTO = nrItensProp[0].NRITEMORCAMENTO;
                });
            } else {
                nrItensProp.forEach(function(item){
                    widget.dataSource.data.filter(function(data){
                        if(data.IDELEMENTO === item.IDELEMENTO) {
                            data.NRITEMORCAMENTO = item.NRITEMORCAMENTO;
                            rows.push(data);
                        }
                    });
                });
                
                widget.dataSource.data.forEach(function(data){ 
                    rows.filter(function(row){ 
                        if(data._parent && row === data._parent) {
                            if (rows.indexOf(data) === -1) {
                                data.NRITEMORCAMENTO = data._parent.NRITEMORCAMENTO;
                                rows.push(data);
                            }
                        }
                    });
                });
            }
        }
    };
    
    /**
     * Salva as alterações feitas nos elementos do cálculo
     * @params{Object} widget widget editar itens formação de preço
    **/
    this.salvarAlteracoesCalculo = function(widget, isNew, nrItensProp) {
        var deferred       = ZHPromise.defer();
        var nrseqorcamento = getWidgetProposta().currentRow.NRSEQORCAMENTO;
        if(isNew){
            var params = {
                "NRFORMPRECOSERVICO": templateManager.container.getWidget("itemFormacaoPreco").currentRow.NRFORMPRECOSERVICO[0],
                "NRSEQORCAMENTO"    : nrseqorcamento,
                "NRITEMORCAMENTO"   : null
            };
            
            General.callRestEngine("product#/fct_relacestruturaprop_widget/insereAlteracaoFormPrec", "Row", params).then(function(){
                deferred.resolve();
            }.bind(this))
            .catch(function(error){
                deferred.reject();
                Utilities.showErrorMessage(error);
            });
        } else {
            var paramsEdit = [];
            try {
                this.setNrItemFormacaoAux(widget, nrItensProp);
                nrItensProp.forEach(function(item){
                    var idItemCalculo = [];
                    var dadosItem = widget.dataSource.data.filter(function(dado){
                        return dado.NRITEMORCAMENTO === item.NRITEMORCAMENTO;
                    }); 
                    
                    dadosItem.forEach(function(dadoItem){
                        idItemCalculo.push(dadoItem.IDITEMCALCULO);
                    });
                    
                    var param = {
                        "NRFORMPRECOSERVICO": widget.dataSource.data[0].NRFORMPRECOSERVICO,
                        "NRSEQORCAMENTO"    : nrseqorcamento,
                        "IDITEMCALCULO"     : idItemCalculo,
                        "NRITEMORCAMENTO"   : item.NRITEMORCAMENTO
                    };
                    
                    paramsEdit.push(param);
                });
            } catch(error){
                deferred.reject();
            }
            
            General.callRestEngine("product#/fct_relacestruturaprop_widget/editaItensFormacaoAux", "DataSet", paramsEdit).then(function() {
                deferred.resolve();
            }).catch(function(error){
                deferred.reject();    
            });
        }
        
        return deferred.promise;
    };
    
    
    /**
     * Verifica se a formação de preço selecionada está habilitada para 
     * permitir a inserção um serviço
     * @params{Object} widget widget item formação de preço
    **/
    this.verificaAcessibilidadeFieldServ = function(widget){
        if(widget.currentRow.NRFORMPRECOSERVICO && widget.currentRow.NRFORMPRECOSERVICO.length === 1){
            widget.getField("NMSERVORCA").readOnly = false;
        } else {
            widget.getField("NMSERVORCA").readOnly = true;
            widget.currentRow.NMSERVORCA = "";
            widget.currentRow.CDSERVORCA = "";
        } 
    };

    /**
     * A função "fecharWdgFormacaoPrec" fecha a tela "Itens de Formação de Preço"
     */
    this.fecharWdgFormacaoPrec = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;

            /*Limpa os campos*/
            this.limpaCamposFormPrec(widget);

            templateManager.container.getWidget("wdgItensProposta").activate();

        }.bind(this));
    };


    /**
     * A função "filtraItFormPreco" filtra o campo "Itens da Formação de Preço" 
     */

    this.filtraItFormPreco = function(widget) {

        var formPreco = widget.currentRow.NRFORMPRECOSERVICO;

        widget.getField('NRCALCFORMPRECSERV').dataSourceFilter[0].value = formPreco;
        widget.getField('NRCALCFORMPRECSERV').reload();
    };


    /**
     * A Função "filtraFormPreco" filtra o campo "Formação de Preço" 
     */

    this.filtraFormPreco = function(widget) {

        var nrparcnegocio = templateManager.container.getWidget("wdgProposta").currentRow.NRPARCNEGOCIO;

        widget.getField('NRFORMPRECOSERVICO').dataSourceFilter[0].value = nrparcnegocio;
        widget.getField('NRFORMPRECOSERVICO').reload();
    };


    /**
     *  A função "verificaServicosRepetidos" verifica entre as formações que foram marcadas, 
     *  se exitem serviços repetidos.
     */

    this.verificaServicosRepetidos = function(widget) {

        var servRepetidos = [];
        var servicos = widget.currentRow.CDSERVORCAFORM;
        var servAnte = 0;

        servicos.sort();

        servicos.forEach(function(servAux) {

            if (servAux == servAnte) {
                if (!servRepetidos.includes(servAux)) {
                    servRepetidos.push(servAux);
                }
            }

            servAnte = servAux;
        });

        return servRepetidos;
    };


    /**
     *  A função "salvaItemformacaopreco" centraliza o salvamento dos 
     *  itens de formação de preço na proposta.
     */

    this.salvaItemformacaopreco = function(widget) {
        if (widget.isValid()) {
            var servRepetidos = this.verificaServicosRepetidos(widget);
            if (servRepetidos.length != 0) {
                ScreenService.showCustomDialog("Foram selecionados diferentes formações de preço que possuem o mesmo serviço. O que você deseja fazer com os serviços repetidos?", 'CONFIRMATION', null, 
                [
                    {
                        "label": "Somar os Valores",
                        "code": function() {
                            this.validarDadosRateioFormacaoPreco(widget, true);
                        }.bind(this)
                    },
                    {
                        "label": "Incluir Separadamente",
                        "code": function() {
                            this.validarDadosRateioFormacaoPreco(widget, false);
                        }.bind(this)
                    },
                    {
                        "label": "Selecionar Novamente",
                        "code": function() {}
                    }
                ]);
            } else {
                var deferred = ZHPromise.defer();
                this.validarDadosRateioFormacaoPreco(widget, false).then(function(response){
                    deferred.resolve(response);
                }).catch(function(error) {
                    deferred.reject(error);
                });
                
                return deferred.promise;
            }
        }
    };

    /**
     *  A função "insereItemFormPrec" insere os serviços selecionados.
     */
    this.insereItemFormPrec = function(widget, soma) {
        var deferred    = ZHPromise.defer();
        var rowProposta = templateManager.container.getWidget("wdgProposta").currentRow;
        var servicos    = widget.getField("NRCALCFORMPRECSERV").dataSource.getCheckedRows();
        
        if(widget.currentRow.NMSERVORCA){
            // pega o serviço da field serviço da tela e substitui o serviço da formação de preço
            servicos.forEach(function(servico){
                servico.CDSERVORCA = widget.currentRow.CDSERVORCA;
                servico.NMSERVORCA = widget.currentRow.NMSERVORCA;
            });
        }
        
        var params = {
            "SOMA"           : soma,
            "SERVICOS"       : servicos,
            "PROPOSTA"       : rowProposta.NRSEQORCAMENTO,
            "NRTIPOORCAMENTO": widget.currentRow.NRTIPOORCAMENTO
        };

        General.callRestEngine('product#/fct_relacestruturaprop_widget/insereItemFormPrec', "Row", params).then(function(response) {
            this.limpaCamposFormPrec(widget);

            var widgetAtiva     = templateManager.container.getActiveWidget();
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            
            if(widgetAtiva.name === "itemFormacaoPreco"){
                ScreenService.closeSwipe();
                templateManager.container.getWidget("wdgItensProposta").activate();
                templateManager.container.getWidget("wdgItensProposta").reload();
            }
                
            this.atualizarValorProposta(wdgPropostaGrid);
            var itens = response.dataset.fct_relacestruturaprop_widget;
            if(rowProposta.IDSITUACAOORCAMENTO !== "ELABORACAO") {
                itens.forEach(function(item){
                    this.insereItensAcaoPreventiva(rowProposta.NRSEQORCAMENTO, item.NRITEMORCAMENTO, 1, rowProposta.IDSITUACAOORCAMENTO);
                }.bind(this));
            }
            
            deferred.resolve(itens);
        }.bind(this),
        function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject();
        }.bind(this));

        return deferred.promise;
    };

    /**
     *  A função "controlaVisibItFormPrec" controla a visibilidade da
     *  action "Itens de Formação de preço".
     */
    this.controlaVisibItFormPrec = function(widget) {
        var situacao = templateManager.container.getWidget("wdgProposta").currentRow.IDSITUACAOORCAMENTO;
        //var utilizaComposicao = templateManager.container.getWidget("wdgProposta").currentRow.IDUTILIZACOMP;
        var utilizaComposicao = (templateManager.container.getWidget("wdgProposta").currentRow.QTITENSTRABSERV > 0) ? "S" : "N";

        if ((situacao == 'ELABORACAO') && (utilizaComposicao == 'N')) {
            widget.getAction("formacaoPreco").isVisible = true;
        }
        else {
            widget.getAction("formacaoPreco").isVisible = false;
        }
    };

    /**
     *  A função "limpaCamposFormPrec" Limpa os campos após o fechamento do swipe
     */
    this.limpaCamposFormPrec = function(widget) {
        widget.currentRow.NRFORMPRECOSERVICO = null;
        widget.currentRow.NMFORMPRECOSERVICO = null;
        widget.currentRow.CDSERVORCAFORM     = null;

        widget.currentRow.NRCALCFORMPRECSERV = null;
        widget.currentRow.CDSERVORCA         = null;
        widget.currentRow.NMSERVORCA         = null;
        widget.currentRow.NRESTRUTURAM       = null;
        widget.currentRow.CDHABILITSERV      = null;
        widget.currentRow.NMELEMENTO         = null;
        widget.currentRow.VRUNITARIO         = null;
        widget.currentRow.QTCALCULO          = null;
        widget.currentRow.IDUSAMEDIDA        = null;
        widget.currentRow.CDTPMEDIDASERV     = null;
        widget.currentRow.QTMEDIDACALC       = null;
        widget.currentRow.VRTOTAl            = null;
    };

    /* FUNÇÕES DA WIDGET LOCAL DE ATENDIMENTO */

    this.validaCnpj = function(widget) {

        var cnpj = widget.currentRow.NRINSCRICAOPARC;

        var testaCnpj = General.validateCNPJ(cnpj);

        if (testaCnpj) {

            widget.currentRow.CDTIPOINSCRICAO = 'CNPJ';
        }
        else {
            widget.currentRow.CDTIPOINSCRICAO = 'LIVRE';

        }
    };

    /**
     * Realiza à rotina de inserção de todas as informações em suas respectivas tabelas.
     * 
     * @param {object} widget widget 
     */
    this.inserirNvLocalAtend = function(widget) {

        this.validaCnpj(widget);
        var params = {
            "NMESTRUTURAM": widget.currentRow.NMESTRUTURAMNV,
            "NRESTRUTURAMSUP": widget.currentRow.NRESTRUTURAMSUP,
            "NRTIPOESTRUTURA": widget.currentRow.NRTIPOESTRUTURA,
            "NRINSCRICAOPARC": widget.currentRow.NRINSCRICAOPARC.replace(/[\D]+/g, ''),
            "CDCENTCUST": widget.currentRow.CDCENTCUST,
            "CDTIPOINSCRICAO": widget.currentRow.CDTIPOINSCRICAO,
            "DSENDERECO": widget.currentRow.DSENDERECO,
            "NRIMOVELENDERECO": widget.currentRow.NRIMOVELENDERECO,
            "DSCOMPLEENDERECO": widget.currentRow.DSCOMPLEENDERECO,
            "CDPAIS": widget.currentRow.CDPAIS,
            "SGESTADO": widget.currentRow.SGESTADO,
            "CDMUNICIPIO": widget.currentRow.CDMUNICIPIO,
            "CDBAIRRO": widget.currentRow.CDBAIRRO,
            "NMBAIRROENDERECO": widget.currentRow.NMBAIRROENDERECO,
            "NRCEPENDERECO": widget.currentRow.NRCEPENDERECO,
            "DSREFERENCIAENDE": widget.currentRow.DSREFERENCIAENDE
        };

        General.callRestEngine('product#/estruturam_widget/inserirTudo', "Row", params).then(function(response) {

            widget.currentRow.NRESTRUTURAM = response.dataset.estruturam_widget[0];
            General.widgetSave(widget).then(function() {
                ScreenService.closeSwipe().then(function() {
                    widget.currentRow.NOVOLOCALATEND = "N";
                    this.habilitaNovoLocalAtend(widget);
                    widget.reload();
                }.bind(this));
            }.bind(this));


        }.bind(this)
        , function(error) {
            console.log(error);
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Salva o novo registro da página "Local de Atendimento"
     * 
     * @param {object} widget widget de local de atendimento 
     * para salvar o novo registro.
     */
    this.salvarLocalAtend = function(widget) {

        if (widget.isValid()) {
            if (widget.currentRow.NOVOLOCALATEND === 'S') {
                this.inserirNvLocalAtend(widget);
            }
            else {
                Utilities.inserirMultiplos(widget, "NRESTRUTURAM").then(function() {
                        ScreenService.closeSwipe().then(function() {
                            widget.currentRow.NOVOLOCALATEND = "N";
                            this.habilitaNovoLocalAtend(widget);
                            widget.reload();
                        }.bind(this));
                }.bind(this),
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
            }

        }
    };

    this.deletarLocalAtend = function(widget) {

        var possuiFilhos = false;
        if (widget.dataSource.data) {
            widget.dataSource.data.forEach(function(dado) {
                if (dado.NRESTRUTURAMSUP != null && dado.NRESTRUTURAMSUP === widget.currentRow.NRESTRUTURAM) {
                    possuiFilhos = true;
                }
            });
        }

        if (possuiFilhos) {
            this.deletarPaieFilhos(widget);
        }else {
            General.deleteCrud(widget).then(function() {
                    widget.reload();
                },
                function(error) {
                    console.error(error);
                    ScreenService.showMessage(error.data.error);
                });
        }

    };

    /**
     * Exclui os local de atandimento e os filhos.
     * 
     * @param {object} widget widget
     */

    this.deletarPaieFilhos = function(widget) {

        ScreenService.confirmMessage("A exclusão deste local de atendimento também irá excluir os locais de atendimento inferiores a ele. Confirma exclusão?", "question", function() {

            var params = {
                "NRESTRUTURAM": widget.currentRow.NRESTRUTURAM,
                "NRSEQORCAMENTO": widget.currentRow.NRSEQORCAMENTO
            };

            General.callRestEngine('product#/fct_relacestruturaprop_widget/deletarPaieFilhos', "Row", params).then(function() {

                    ScreenService.showMessage('Exclusão realizada com sucesso.');
                    ScreenService.closeSwipe();
                    widget.reload();

                },
                function(error) {
                    console.log(error);
                    ScreenService.showMessage(error.data.error);
                });

        });
    };

    /* Habilita os campos para a inserção de um novo local de atendimento
     * @params{Object} widget local de atendimento
     */
    this.habilitaNovoLocalAtend = function(widgetLocalAtend) {
        var fields = ["NMESTRUTURAMNV", "NRINSCRICAOPARC", "NRTIPOESTRUTURA", "NMTIPOESTRUTURA", "NRESTRUTURAMSUP", "NMESTRUTURAMSUP", "NMCENTCUST", "NRCEPENDERECO", "NMPAIS", "SGESTADO", "NMESTADO", "NMMUNICIPIO", "DSENDERECO", "NRIMOVELENDERECO", "NMBAIRROENDERECO", "DSCOMPLEENDERECO", "DSREFERENCIAENDE"];

        if (widgetLocalAtend.currentRow.NOVOLOCALATEND === 'S') {
            widgetLocalAtend.getField('NRESTRUTURAM').readOnly = true;
            widgetLocalAtend.getField('NRESTRUTURAM').validations = {};
            fields.forEach(function(field) {
                widgetLocalAtend.getField(field).readOnly = false;
            });
            widgetLocalAtend.getField("NMESTRUTURAMNV").validations = {
                "required": true
            };
            widgetLocalAtend.getField("NMTIPOESTRUTURA").validations = {
                "required": true
            };
            this.recuperaEnderecoParcNegocio(widgetLocalAtend);
        }
        else {
            widgetLocalAtend.getField('NRESTRUTURAM').validations = {
                "required": true
            };
            widgetLocalAtend.getField("NMESTRUTURAMNV").validations = {};
            widgetLocalAtend.getField("NMTIPOESTRUTURA").validations = {};
            fields.forEach(function(field) {
                widgetLocalAtend.getField(field).readOnly = true;
                widgetLocalAtend.currentRow[field.name] = "";
            });

            widgetLocalAtend.getField('NRESTRUTURAM').readOnly = false;

        }
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o País.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorPais = function(widget) {
        widget.currentRow.SGESTADO = "";
        widget.currentRow.NMESTADO = "";
        widget.currentRow.CDMUNICIPIO = "";
        widget.currentRow.NMMUNICIPIO = "";
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var cdpais = widget.currentRow.CDPAIS;

        var campoEstado = widget.getField("NMESTADO");
        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoEstado.dataSourceFilter[0].value = cdpais;
        campoEstado.reload();

        campoMunicipio.dataSourceFilter[0].value = cdpais;

        campoBairro.dataSourceFilter[0].value = cdpais;
    };


    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Estado.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorEstado = function(widget) {
        widget.currentRow.CDMUNICIPIO = "";
        widget.currentRow.NMMUNICIPIO = "";
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var sgestado = widget.currentRow.SGESTADO;

        var campoMunicipio = widget.getField("NMMUNICIPIO");
        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoMunicipio.dataSourceFilter[1].value = sgestado;
        campoMunicipio.reload();

        campoBairro.dataSourceFilter[1].value = sgestado;
    };

    /**
     * Controla o filtro e o valor dos campos de endereço ao mudar o Município.
     * 
     * @param {object} widget
     */
    this.controlarCamposEnderecoPorMunicipio = function(widget) {
        widget.currentRow.CDBAIRRO = "";
        widget.currentRow.NMBAIRROENDERECO = "";

        var cdmunicipio = widget.currentRow.CDMUNICIPIO;

        var campoBairro = widget.getField("NMBAIRROENDERECO");

        campoBairro.dataSourceFilter[2].value = cdmunicipio;
        campoBairro.reload();
    };


    this.recuperaEnderecoParcNegocio = function(wdgNvLocalAtend) {
        var widgetLancProposta = templateManager.container.getWidget("wdgProposta");
        var paramsFilter = [
            {
                name: "NRPARCNEGOCIO",
                operator: "=",
                value: widgetLancProposta.currentRow.NRPARCNEGOCIO
            }
        ];

        General.callRestEngine("product#/crud/enderecoparc", "FilterData", paramsFilter).then(function(success) {
            if (success.dataset && success.dataset.enderecoparc.length > 0) {
                success.dataset.enderecoparc.every(function(dado) {
                    if (dado.CDTIPOENDERECO === "PRINCIPAL") {
                        wdgNvLocalAtend.currentRow.NRCEPENDERECO = dado.NRCEPENDERECO;
                        this.buscarEndereco(wdgNvLocalAtend);
                    }
                }.bind(this));
                
                if(!wdgNvLocalAtend.currentRow.NRCEPENDERECO){
                    wdgNvLocalAtend.currentRow.NRCEPENDERECO = success.dataset.enderecoparc[0].NRCEPENDERECO;
                    this.buscarEndereco(wdgNvLocalAtend);
                }
            }
        }.bind(this));
    };
    
    /* Verifica a action acionada */
    this.verificaAction = function(widget, action){
        if(action === "AVANCAR"){
            this.avancarComposicao(widget);
        } else {
            this.salvarProposta(widget);
        }
    };

    /**
     * Busca o parametro de medida fixada da arvore de serviço
     * @params{Object} widget - widget itens
    **/
    this.getUsaMedidaFixada = function(widget, tpElemento, nrseqorcamento){
        var deferred = ZHPromise.defer();
        if(!tpElemento || (servIdMedidaFixa.length > 0 && servIdMedidaFixa[0]["NRSEQORCAMENTO"] === nrseqorcamento)){
            deferred.resolve();
        } else {
            var arrIdElePai = [];
            var cdservorca  = [];
            widget.dataSource.data.forEach(function(item){
                if(item.IDELEMENTO && !arrIdElePai.includes(item.IDELEMENTO))
                    arrIdElePai.push(item.IDELEMENTO);
                if(item.CDSERVORCA && !cdservorca.includes(item.CDSERVORCA))
                    cdservorca.push(item.CDSERVORCA);
            });
            var params = {
                "IDARVORE"  : arrIdElePai,
                "CDSERVORCA": cdservorca
            };
            General.callRestEngine("product#/fct_orcamento_widget/getIdUsaMedidaFixada", "Row", params).then(function(response){
                if(response.dataset && response.dataset.getIdUsaMedidaFixada)
                    servIdMedidaFixa[0] = {"NRSEQORCAMENTO": nrseqorcamento,
                                           "SERVMEDIDAFIXA": response.dataset.getIdUsaMedidaFixada};
                deferred.resolve();
            }).catch(function(error){
                deferred.reject();
            });
        }
        
        return deferred.promise;
    };
    /**
     * Validação para que o swipe do view não seja aberto quando for selecionada
     * uma linha correspondente a uma medida variável, e não um item de proposta.
     */
    this.validarAbreSwipeItens = function(args, tipe) {
        //if(getWidgetProposta().currentRow.IDUTILIZACOMP === 'S'){
        if(getWidgetProposta().currentRow.QTITENSTRABSERV > 0) {
            var nrseqorcamento = getWidgetProposta().currentRow.NRSEQORCAMENTO;
            var idMedidaFixa   = 'N';
            var widget         = (tipe === "W") ? args : args.owner;
            var isVisible      = widget.currentRow.IDTIPO === "M";
            var qtMdVisivel    = (widget.currentRow.IDTIPO === "M" && idMedidaFixa === 'N' && widget.currentRow.IDMULTIPVARIACAO === "S");

            widget.getField("VRUNITITEMORCA").help = null;
            this.getUsaMedidaFixada(widget, isVisible, nrseqorcamento).then(function(){
                if(widget.currentRow.IDTIPO === "M"){
                    servIdMedidaFixa[0]["SERVMEDIDAFIXA"].forEach(function(cdservorca){
                        if(cdservorca.CDSERVORCARELAC === widget.currentRow.CDSERVORCA)
                            idMedidaFixa = cdservorca.IDMEDIDAFIXA;
                    });
                }
                
                widget.getField("QTMEDIDAORCA").isVisible = isVisible;
                widget.getField("QTMEDIDAORCA").readOnly  = qtMdVisivel;
                this.alterarVisibilidadeCamposItens(widget, idMedidaFixa);
                
                if(widget.getField("VRUNITITEMORCATBLPRECO")) {
                    widget.getField("VRUNITITEMORCATBLPRECO").showOnView = isVisible;
                    widget.getField("VRUNITITEMORCATBLPRECO").showOnForm = isVisible;
                }
                
                widget.getAction("edit").isVisible = (widget.getAction("edit").isVisible && widget.currentRow.IDTIPO !== "A");
            }.bind(this));
        }
    };
    /**
     * Altera a visibilidade dos campos da tela de itens
     * @params{Object} widget - widget itens (inserindo ou editando)
    **/
    this.alterarVisibilidadeCamposItens = function(widget, idMedidaFixa){
        var isVisible      = widget.currentRow.IDTIPO === "M";
        var isVisibleAgrup = (widget.currentRow.IDTIPO !== "A" && widget.currentRow.IDTIPO !== "M");
        
        widget.getField("IDMULTMEDVRTOT").isVisible     = isVisible;
        widget.getField("NMDOCITEMORCAMENTO").isVisible = isVisibleAgrup;
        widget.getField("IDIMPRESSAOORCA").isVisible    = isVisibleAgrup;
        widget.getField("TXFORMULAITORCA").isVisible    = isVisibleAgrup;
        widget.getField("NMTIPOORCAMENTO").isVisible    = isVisibleAgrup;
        widget.getField("VRUNITITEMORCA").showOnView    = isVisible;
        widget.getField("VRUNITITEMORCA").showOnForm    = isVisible;
        widget.getField("DSTPMEDIDASERV").showOnView    = isVisible; 
        widget.getField("DSTPMEDIDASERV").showOnForm    = isVisible;
        
        if(widget.getField("VRUNITITEMORCATBLPRECO")) {
            widget.getField("VRUNITITEMORCATBLPRECO").showOnView = isVisible;
            widget.getField("VRUNITITEMORCATBLPRECO").showOnForm = isVisible;
        }
        
        /*widget.getField("IDTPDESCORCA").isVisible          = isVisible; 
        widget.getField("VRITEMSEMDESC").isVisible         = isVisible;
        widget.getField("NMGRUPOFATURA").isVisible         = isVisibleAgrup;
        widget.getField("DTINICFATORC").isVisible          = isVisibleAgrup;
        widget.getField("DSMODPGTO").isVisible             = isVisibleAgrup;
        widget.getField("IDFATURAORCA").isVisible          = isVisibleAgrup;
        widget.getField("QTPARCELASFATU").isVisible        = isVisibleAgrup;
        widget.getField("NRDIAVENCPARCELASFATU").isVisible = isVisibleAgrup;
        
        
        widget.getField("VRPERCDESCORCA").showOnView = isVisible;
        widget.getField("VRPERCDESCORCA").showOnForm = isVisible;
        widget.getField("VRDESCORCA").showOnView = isVisible;
        widget.getField("VRDESCORCA").showOnForm = isVisible;

        widget.getField("VRPERCDESCORCA").readOnly  = (!widget.currentRow.IDTPDESCORCA || widget.currentRow.IDTPDESCORCA === 'N');
        this.controlarCampoValorDoAjuste(widget);*/
    };
    
    /* ================ FUNÇÕES DA WIDGET EVOLUIR CRM ================ */
    
    /**
     * Abre a widget de Evoluir CRM.
     */
    this.abrirWdgEvoluirCRM = function(rowProposta) {
        ScreenService.closePopup();
        
        var widgetEvoluirCRM = templateManager.container.getWidget("wgdEvoluirCRM");

        widgetEvoluirCRM.isVisible = true;
        
        ScreenService.openSwipe(widgetEvoluirCRM).then(function() {
            widgetEvoluirCRM.edit();
            widgetEvoluirCRM.newRow();
            this.setDadosWdgEvoluirCRM(rowProposta, widgetEvoluirCRM);
        }.bind(this));
    };
    
    /**
    * preenche a widget evolução CRM com os dados necessários
    * @params{Object} widget evoluir CRM
    **/
    this.setDadosWdgEvoluirCRM = function(rowProposta, widgetEvoluirCRM){
        widgetEvoluirCRM.currentRow.DTEVOLUCAO  = moment().format('DD/MM/YYYY HH:mm');
        widgetEvoluirCRM.currentRow.DTREALFECHA = moment().format('DD/MM/YYYY HH:mm');
        
        widgetEvoluirCRM.currentRow.NMOPERADOR = nomeOperador;
        widgetEvoluirCRM.currentRow.CDOPEREVOL = opLogado;

        var fieldInterlocutor = widgetEvoluirCRM.getField("NMINTERLOCUTOR");
        fieldInterlocutor.dataSourceFilter[0].value = rowProposta.NRPARCNEGOCIO;
        fieldInterlocutor.reload();
        
        
        if(tpOperador === null)
            tpOperador = "E";

        widgetEvoluirCRM.getField("NMPARCRESP").dataSourceFilter[0].value = tpOperador;
        widgetEvoluirCRM.getField("NMPARCRESP").reload();
        
        widgetEvoluirCRM.getField("NMOPERADORREAL").dataSourceFilter[0].value = tpOperador;
        widgetEvoluirCRM.getField("NMOPERADORREAL").reload();
        
        widgetEvoluirCRM.currentRow.VRPREVOPORT       = rowProposta.VRORCAMENTO;
        widgetEvoluirCRM.currentRow.VROPORT           = rowProposta.VRORCAMENTO;
        widgetEvoluirCRM.currentRow.NRSEQOPORT        = rowProposta.NRSEQOPORT;
        widgetEvoluirCRM.currentRow.NRORG             = rowProposta.NRORG;
        widgetEvoluirCRM.currentRow.CDOPERADORLOGADO  = opLogado;
        widgetEvoluirCRM.currentRow.NRSEQCONTATO      = rowProposta.NRSEQCONTATO;
        widgetEvoluirCRM.currentRow.NRPARCNEGOCIO     = rowProposta.NRPARCNEGOCIO;
        widgetEvoluirCRM.currentRow.NRORCAMENTOMODELO = rowProposta.NRORCAMENTOMODELO;
        widgetEvoluirCRM.currentRow.NRORCAMENTOMODELO = rowProposta.NRORCAMENTOMODELO;

    };
    
    /**
     * Verifica se há proxima tarefa
     * @params{Object} widget Evoluir CRM
    **/
    this.verificaProxTarefa = function(widgetEvoluirCRM){
        if(widgetEvoluirCRM.currentRow.IDGERPROXATIV === "S"){
            this.habilitaProxTarefa(widgetEvoluirCRM, false);
            widgetEvoluirCRM.getField("DTPLANFILACONTA").validations = { "required": true };
        } else {
            this.habilitaProxTarefa(widgetEvoluirCRM, true);
            widgetEvoluirCRM.getField("DTPLANFILACONTA").validations = {};
        }    
    };
    
    /**
     * Habilita/Desabilita o fieldGroup de próxima tarefa
     * @params{Object} widget
     * @params{boolean} permissão
    **/
    this.habilitaProxTarefa = function(widget, isLocked) {

        var dataProxtf = widget.getField("DTPLANFILACONTA");
        var operadorProxTf = widget.getField("NMOPERADORREAL");
        var descricaoProxtf = widget.getField("DSFILACONTATO");

        dataProxtf.readOnly = isLocked;
        operadorProxTf.readOnly = isLocked;
        descricaoProxtf.readOnly = isLocked;
        
        if(isLocked){
            widget.currentRow.DTPLANFILACONTA = "";
            widget.currentRow.NMOPERADORREAL  = "";
            widget.currentRow.DSFILACONTATO   = "";
        }

    };
    
    /**
     * Verifica se há fechamento na fase de nedociação
     * @params{Object} widget Evoluir CRM
    **/
    this.verificaFechamento = function(widgetEvoluirCRM){
        if(widgetEvoluirCRM.currentRow.IDPERMITEFECHA === "S"){
            this.habilitaFechamento(widgetEvoluirCRM, false);
        } else {
            this.habilitaFechamento(widgetEvoluirCRM, true);
        }    
    };
    
    /**
     * Habilita/Desabilita o fieldGroup de fechamento
     * @params{Object} widget
     * @params{boolean} permissão
    **/
    this.habilitaFechamento = function(widget, isLocked) {

        var resultadoFecha      = widget.getField("DESCRESULEVOL");
        var operadorFecha       = widget.getField("NMRAZSOCCONC");
        var valorOpFecha        = widget.getField("VROPORT");
        var dataFecha           = widget.getField("DTREALFECHA");

        resultadoFecha.readOnly = isLocked;
        operadorFecha.readOnly  = isLocked;
        valorOpFecha.readOnly   = isLocked;
        dataFecha.readOnly      = isLocked;
        
        if(isLocked){
            widget.currentRow.DESCRESULEVOL = "";
            widget.currentRow.NMRAZSOCCONC  = "";
            widget.currentRow.VROPORT       = "";
            widget.currentRow.DTREALFECHA   = "";
            
            resultadoFecha.validations = {};
            valorOpFecha.validations   = {};
        } else {
            widget.currentRow.DTREALFECHA = moment().format('DD/MM/YYYY HH:mm');
            resultadoFecha.validations = { "required": true };
            valorOpFecha.validations   = { "required": true };
        }
    };
    
    /**
     * Fecha a tela de Evoluir CRM
     * 
     * @param {object} widget
     */
    this.fecharWdgEvoluirCRM = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;
        });
    };
    
    /**
    * Salvar evolução de oportunidade CRM
    * @params{Object} widget evoluir CRM
    **/
    this.salvarEvolucaoOportunidade = function(widget){
        if(widget.isValid()){
            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
            General.callRestEngine("product#/enviarElementIntegracaoCRM", "Row", widget.currentRow).then(function(response){
                ScreenService.showMessage("Evolução gerada com sucesso.");
                ScreenService.closeSwipe();
                wdgPropostaGrid.reload();
            }.bind(this))
            .catch(function(error){
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        } else {
           ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } 
    };
    
    /** Valida o campo de código da proposta
     * @params{Object} widget 
    **/
    this.validaCodPropostas = function(widget, action){
        if (widget.isValid()) {
            if (widget.currentRow.NRORCAMENTO) {
                var rota = "product#/crud/fct_orcamento";
                Utilities.validarCamposDuplicados(widget, ["NRSEQORCAMENTO"], ["NRORG", "NRORCAMENTO"], rota).then(function(response){
                    if (response) {
                        this.verificaAction(widget, action);
                    } else {
                        ScreenService.showMessage("Operação bloqueada.<br/>Já existe uma proposta com o código informado.");
                    } 
                }.bind(this));
            } else {
                this.verificaAction(widget, action);
            }
        }
    };
    
    /*this.removerEdicaoGrid = function(element) {
        if (element.field && element.field.name == "QTMEDIDAORCA"){
            if (element.context.children[1]) {
                element.context.children[1].remove();
            }
        }
    };*/
    
    this.actionEvolutionNewRowProposta = function(){
        this.closeActionsPopup().then(function(){
            var widget = templateManager.container.getWidget("evolucao");
            widget.activate().then(function(){
                setTimeout(function(){
                    widget.reload().then(function(){
                        this.evolucaoNewRow();
                    }.bind(this));
                }.bind(this), 300);
            }.bind(this));
        }.bind(this));
    };
   
    
    this.closeActionsPopup = function() {
        var deferred = ZHPromise.defer();
        ScreenService.closePopup();
        deferred.resolve();
        return deferred;
    };
    
     this.activateWidget = function(widget){
        var deferred = ZHPromise.defer();
        widget.activate();
        deferred.resolve();
        return deferred;
    };
    
     /**
     * Inicia o mvc para realizar um insert de tarefas
     * na widget de Evolução
    */
    this.evolucaoNewRow = function() {
        var widget      = templateManager.container.getWidget("evolucao");
        var wdgProposta = templateManager.container.getWidget('wdgProposta');
        widget.getField('NMRESULTEVOL').readOnly = false; 
        General.widgetNewRow(widget);

        widget.currentRow.DTHREVOLCONT = (widget.currentRow.DTHREVOLCONT) ? widget.currentRow.DTHREVOLCONT : moment(new Date()).format('DD/MM/YYYY HH:mm');
        widget.currentRow.NMOPERADOR   = nomeOperador;
        widget.currentRow.CDOPEREVOL   = opLogado;
        widget.currentRow.NRORG        = nrOrgOper;
        widget.currentRow.DTINCLUSAO   = self.momentNow();
        widget.currentRow.NRSEQCONTATO = wdgProposta.currentRow.NRSEQCONTATO;
        widget.currentRow.NRCONTATO    = wdgProposta.currentRow.NRSEQCONTATO;
        widget.currentRow.NRORG        = nrOrgOper;
        widget.currentRow.NRSEQEVOLCONT = "";
    };
    
    this.momentNow = function(){
        return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
    };
    
    /*this.setContactReasonFilter = function(field){
        var fieldFilter = field.dataSourceFilter;
        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_NRORG");
        //var cdoperadorFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_CDOPERADOR");
        
        fieldFilter[nrorgFilterIndex].value = nrOrgOper;
        //fieldFilter[cdoperadorFilterIndex].value = opLogado;
        field.reload();
    };
    
     
    this.setResultFilter = function(field){
    
        var fieldFilter = field.dataSourceFilter;
        
        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_NRORG");
        //var cdoperadorFilterIndex = CRM_Utilities.getFilterPosition(fieldFilter, "P_CDOPERADOR");
        
        fieldFilter[nrorgFilterIndex].value = nrOrgOper;
        //fieldFilter[cdoperadorFilterIndex].value = opLogado;
        
        
        field.reload();
    };*/

     /**
     * Organiza os campos para salvar a evolução
     * e verifica tarefa já aberta.
     * 
     * @param {object} widget widget de evolução
    */
    this.saveEvolucao = function(widget) {
        if (widget.isValid()) {
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            var nrseqcontato;
            
            widget.currentRow.CDOPEREVOL      = opLogado;
            widget.currentRow.IDGERAFATURA    = 'NAO';
            widget.currentRow.IDTPEVOLCONT    = 'P';
            widget.currentRow.IDANEXOEVOLCONT = 'N';
            widget.currentRow.IDSENEVOCONT    = "S";
            widget.currentRow.CDFILIAL        = "";
            widget.currentRow.NRPARCNEGOCIO   = wdgProposta.currentRow.NRPARCNEGOCIO;
            
            widget.getField('NMRESULTEVOL').readOnly = true;
            
            if(!widget.currentRow.NRRELACIONAPARC)
                widget.currentRow.NRRELACIONAPARC = null;
            
            if(widget.currentRow.__is_new && !wdgProposta.currentRow.NRSEQCONTATO) {
                this.geraContatoProposta(wdgProposta.currentRow).then(function(response){
                    nrseqcontato                        = response.dataset.NRCONTATO[0];
                    wdgProposta.currentRow.NRSEQCONTATO = response.dataset.NRCONTATO[0];
                    widget.dataSourceFilter[0].value    = nrseqcontato; 
                    widget.currentRow.NRSEQCONTATO      = nrseqcontato;
                    widget.currentRow.NRCONTATO         = nrseqcontato;
                    
                    this.efetuaRequisicaoSaveEvolucao(widget);
                }.bind(this));
            } else {
                widget.currentRow.NRSEQCONTATO  = wdgProposta.currentRow.NRSEQCONTATO;
                widget.currentRow.NRCONTATO     = wdgProposta.currentRow.NRSEQCONTATO;
                this.efetuaRequisicaoSaveEvolucao(widget);
            }
            
            if(widget.currentRow.__is_new)
                ScreenService.closeSwipe();
            else 
                widget.view();
        }
    };
    
    /**
     * Chama a rota de salvar a evolução
     * @params{Object} widget widget de evolução
    **/
    this.efetuaRequisicaoSaveEvolucao = function(widget){
        var deferred = ZHPromise.defer();
        General.callRestEngine("product#/evolcont_parcnegocio/save", "DataSet",  widget.dataSource.data).then(function(response) {
        if (response.dataset && response.dataset.DADOS.NRSEQEVOLCONT)
            widget.currentRow.NRSEQEVOLCONT  = response.dataset.DADOS.NRSEQEVOLCONT;
            
            widget.reload();
            deferred.resolve();
        }.bind(this)).catch(function(error){
            console.log(error);
            deferred.reject();
            if(error === "Column NRSEQCONTATO cannot be null.")
                ScreenService.showMessage(ScreenService.i18n("Não é possível fazer a evolução em propostas que não estão ligadas a uma Oportunidade."));
            else 
                Utilities.showErrorMessage(error);
        });
        
        return deferred.promise;
    };
    
    /**
     * Salva uma evolução após enviar a proposta por email
     * @params{Object} widget widget evolucao
     * @params{String} idSituacao Situação do orcamento
    **/
    this.saveEvolucaoAposEmail = function(widget, idSituacao) {
        var wdgProposta    = templateManager.container.getWidget("wdgProposta");
        var wdgEnviarEmail = templateManager.container.getWidget("wdgEnviarEmail");
        var nrseqcontato;
        
        widget.currentRow.CDOPEREVOL      = opLogado;
        widget.currentRow.IDGERAFATURA    = 'NAO';
        widget.currentRow.IDTPEVOLCONT    = 'P';
        widget.currentRow.IDANEXOEVOLCONT = 'N';
        widget.currentRow.IDSENEVOCONT    = "S";
        widget.currentRow.CDFILIAL        = "";
        
        if(!widget.currentRow.NRPARCNEGOCIO)
            widget.currentRow.NRPARCNEGOCIO = wdgProposta.currentRow.NRPARCNEGOCIO;
        
        if(!widget.currentRow.NRRELACIONAPARC)
            widget.currentRow.NRRELACIONAPARC = null;

        if(widget.currentRow.__is_new && !wdgProposta.currentRow.NRSEQCONTATO) {
            this.geraContatoProposta(wdgProposta.currentRow).then(function(response){
                nrseqcontato                        = response.dataset.NRCONTATO[0];
                wdgProposta.currentRow.NRSEQCONTATO = response.dataset.NRCONTATO[0];
                widget.dataSourceFilter[0].value    = nrseqcontato; 
                widget.currentRow.NRSEQCONTATO      = nrseqcontato;
                widget.currentRow.NRCONTATO         = nrseqcontato;
                
                this.efetuaRequisicaoSaveEvolucao(widget).then(function(){
                    this.anexarDocumnetosdoEmail(idSituacao, nrseqcontato,  widget.currentRow.NRSEQEVOLCONT);
                }.bind(this));
                
            }.bind(this));
        } else {
            var nrseqevolcont = widget.currentRow.NRSEQEVOLCONT; 
            widget.currentRow.NRSEQCONTATO  = wdgProposta.currentRow.NRSEQCONTATO;
            widget.currentRow.NRCONTATO     = wdgProposta.currentRow.NRSEQCONTATO;
            this.efetuaRequisicaoSaveEvolucao(widget).then(function(){
                this.anexarDocumnetosdoEmail(idSituacao, nrseqcontato,  nrseqevolcont);
            }.bind(this));
        }
        
        wdgEnviarEmail.getField("NMTIPOCONT").validations     = {};
        wdgEnviarEmail.getField("NMMOTIEVOCON").validations   = {};
        wdgEnviarEmail.getField("NMSEQRESULEVOL").validations = {};
        
        widget.list();
        wdgProposta.activate();
    };
    /**
     * Inicia o mvc para realizar select de CONTATO
     * 
     * @param {object} widget widget para passar a currentRow
    */
    this.desobrigaCampos = function(widget){
        widget.fields.forEach(function(field){
            field.validations = {};
        }.bind(this));
    };   
    
    this.selectProspectContato = function(widget){        
        return ger00004500_Prospect_Service.callForSelectProspect(widget.currentRow);
    };
    /**
     * Seta o operador para preparar a proxima tarefa
     * da widget de evolução
     * 
     * @param {object} currentRow pega a currentRow
     * da evolução
    */
    this.getParentField = function(widget, nrseqcontato){
        var widgetPropostaGrid = getWidgetPropostaGrid();
        widget.currentRow.NRPARCNEGOCIO = widgetPropostaGrid.currentRow.NRPARCNEGOCIO;
        widget.currentRow.NRSEQCONTATO  = nrseqcontato;
        widget.currentRow.NRCONTATO     = nrseqcontato;
        
    };
    
     this.setEvolutionWidgetFilter = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        widget.dataSourceFilter[0].value = wdgProposta.currentRow.NRSEQCONTATO;
        widget.reload();
    };
    
     /**
     * Função para editar uma evolução.
     * 
     * @param {object} widget widget ativa, no
     * modo de visualização, com as informações
     * da evolução a ser editada.
    */
    this.editEvolution = function(widget){
        General.widgetEdit(widget);
        widget.getField('NMRESULTEVOL').readOnly = true;
        var fieldDTPLANFILACONTA = widget.getField('DTPLANFILACONTA');
        if (fieldDTPLANFILACONTA)
            widget.getField('DTPLANFILACONTA').validations = {};
        //ABEvolucaoController.setEvolDateChangePermission(widget);
    };
    
     /**
     * Função para fazer o download de uma arquivo
     * salvo no servidor.
     * 
     * @param {object} widget widget que contém
     * as informações do arquivo a ser baixado 
     * em sua currentRow.
    */    
    this.downloadFile = function(widget){
        var nmarquivo = widget.currentRow.NMARQUIVO;
        var dscaminho = widget.currentRow.DSCAMINHO;
        var type      = widget.currentRow.DSTIPO;
        
        Utilities.downloadFile(nmarquivo, dscaminho, type);   
    };
    
     /**
     * Função para abrir a widget de seleção
     * de arquivos.
    */
    this.openAnnexSwipe = function(){
        
        var fileUploadWidget = templateManager.container.getWidget('WDGT_SELECAO_ARQUIVOS');
        
        fileUploadWidget.isVisible = true;
        ScreenService.openSwipe(fileUploadWidget);
            
    };
    
     /**
     * Função para salvar os anexos selecionados
     * ao caso em questão.
     * 
     * @param {object} widget widget onde é
     * feito a seleção dos anexos.
    */
    this.uploadEvolutionFiles = function(widget){

        var files = widget.currentRow.DSCAMINHO;
        var widgetAnexos = getWidgetAnexos();
        var rows = [];
        
        files.forEach(function(file, fileIndex) {
            rows.push(this.setValuesForFileUpload(widget, fileIndex));
        }.bind(this));
        
        General.callRestEngine("product#/crm_anexocontato/save", "DataSet", rows).then(function() {
            this.closeAnnexSwipe(widget);
            widgetAnexos.reload();
        }.bind(this));
    };
    
     /**
     * Função para fechar o swipe de upload de imagens.
     * 
     * @param {object} widget widget de upload de imagens.
    */
    this.closeAnnexSwipe = function(widget){
        var widgetEvolucao = getWidgetEvolucao();
        ScreenService.closeSwipe().then(function(){
            
            widget.isVisible = false;
            widgetEvolucao.list();
            widgetEvolucao.activate();
           
        });
    };
    
     /**
     * Função para retornar uma row configurada
     * com as informações do anexo referente
     * ao ID, recebido como parâmetro, no array
     * de anexos na currentRow.
     * 
     * @param {object} widget widget onde foi selecionado
     * os anexos.Array
     * 
     * @param {int} fileIndex index do anexo no array de 
     * anexos presente na currentRow.
     * 
     * @return {object} currentRow row configurada com as informações
     * do anexo referente ao ID recebido.
    */
    this.setValuesForFileUpload = function(widget, fileIndex){
        var currentRow     = angular.copy(widget.currentRow);
        var widgetEvolucao = getWidgetEvolucao();

        currentRow.NRSEQCONTATO = widgetEvolucao.currentRow.NRSEQCONTATO;
        currentRow.NRORIGEM     = widgetEvolucao.currentRow.NRSEQEVOLCONT;
        currentRow.NMARQUIVO    = currentRow.DSCAMINHO[fileIndex].name;
        currentRow.DSTIPO       = this.getFileExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.VRTAMANHO    = currentRow.DSCAMINHO[fileIndex].size;
        currentRow.CDOPERADOR   = nrparcnegocioOperLogado;
        currentRow.NRORG        = nrOrgOper;
        currentRow.IDATIVO      = "S";
        currentRow.IDORIGEM     = "EVOLUCAO";
        
        return currentRow;
    };
     /**
     * Função para retornar o nome do arquivo sem
     * sua extenção.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileNameWithoutExtension = function(fileName){
        return fileName.replace(/^(.*)\.(.*)$/, "$1");
    };
    
     /**
     * Função para retornar a extenção de um arquivo
     * extraida do seu nome.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileExtension = function(fileName){
        return fileName.replace(/^(.*)\.(.*)$/, "$2");
    };
    
    this.tornaCamposObrigatorios = function(widget){
       if(qtdPrenchido.length >= 4)
            qtdPrenchido = [];
        var nmtipocont = widget.currentRow.NMTIPOCONT;
        var nmmotievocon = widget.currentRow.NMMOTIEVOCON;
        var nmseqresulevol = widget.currentRow.NMSEQRESULEVOL;
        
        if(nmtipocont)
            qtdPrenchido.push(1);
        
        if(nmmotievocon)
            qtdPrenchido.push(1);
        
        if(nmseqresulevol)
            qtdPrenchido.push(1);
        
        if(qtdPrenchido.length == 3)
        qtdPrenchido.push(1);
        
        if(qtdPrenchido.length == 1){
            if(!trancado)
                this.aplicaRegra(widget);
        }else
        if(qtdPrenchido.length == 0){
            if(!nmtipocont && !nmmotievocon && !nmseqresulevol)
            this.aplicaRegra(widget);
        }
        if(!nmtipocont && !nmmotievocon && !nmseqresulevol)
            qtdPrenchido = [];
    };
    
     this.aplicaRegra = function(widget){
        if(!is_required){
            is_required = this.alternaCampo(widget, true);
        } else
            is_required = this.alternaCampo(widget, false);
     };
     
     this.alternaCampo = function(widget, setObrg){
         var ocupado = false;
         if(widget.getField("NMTIPOCONT").value() || widget.getField("NMMOTIEVOCON").value() || widget.getField("NMSEQRESULEVOL").value())
            ocupado = true;
         
         if(setObrg){
            if(ocupado) {
                widget.getField("NMTIPOCONT").validations.required     = setObrg;
                widget.getField("NMMOTIEVOCON").validations.required   = setObrg;
                widget.getField("NMSEQRESULEVOL").validations.required = setObrg;
            }
         } else {
            widget.getField("NMTIPOCONT").validations = {};
            widget.getField("NMMOTIEVOCON").validations = {};
            widget.getField("NMSEQRESULEVOL").validations = {};
         }
         
        qtdPrenchido.pop();
        return setObrg;
    };
    
    /**
     * Abre o popup de edição do operador responsável da proposta
     * @params{Object} widget widget proposta grid
    **/
    this.abrirPopupAlterarOper = function(widget){
        var wdgPopup = templateManager.container.getWidget("wdgPopupAlteraOper");
        wdgPopup.isVisible = true;
        wdgPopup.getField("CDOPERRESPATUAL").readOnly = false;
        
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.edit();
            wdgPopup.newRow();
            wdgPopup.activate();
            wdgPopup.getField("NRSEQORCAMENTO").dataSourceFilter = [];
            this.verificaHierarquia({}, wdgPopup);
        }.bind(this));
    };
    
    /**
     * filtra as fields de operador atual e propostas do popup de alteração dos
     * operadores responsáveis
     * @params{Object} widget widget popup altera operador responsável
     * @params{Object} field fieldAlterada nome da field de alteraçao
    **/
    this.filtraOperadoresEPropostas = function(widget, fieldAlterada){
        var fieldProposta  = widget.getField("NRSEQORCAMENTO");
        var fieldOperAtual = widget.getField("CDOPERRESPATUAL");

        if(widget.currentRow.CDOPERRESPATUAL && !widget.currentRow.NRSEQORCAMENTO || (widget.currentRow.NRSEQORCAMENTO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "CDOPERRESPATUAL")){
            fieldProposta.clearValue();
            
            if(widget.currentRow.CDOPERRESPATUAL && widget.currentRow.CDOPERRESPATUAL.length > 0){
                fieldProposta.dataSourceFilter[0] = {
                    "name": "CDOPERRESPONSAVEL",
                    "operator": "IN",
                    "value": widget.currentRow.CDOPERRESPATUAL
                };
                
                fieldOperAtual.readOnly = false;  
            } else {
                if(fieldProposta.dataSourceFilter.length > 0){
                    fieldProposta.dataSourceFilter.pop();
                }
            }
            
            fieldProposta.reload();
            
        } else {
            widget.currentRow.CDOPERRESPONSAVEL = [];
            widget.getField("NRSEQORCAMENTO").dataSource.getCheckedRows().forEach(function(proposta){
                widget.currentRow.CDOPERRESPONSAVEL.push(proposta.CDOPERRESPONSAVEL);
            });
            
            if((!widget.currentRow.CDOPERRESPATUAL && widget.currentRow.NRSEQORCAMENTO) || (widget.currentRow.NRSEQORCAMENTO && widget.currentRow.CDOPERRESPATUAL && fieldAlterada === "NRSEQORCAMENTO")){
                if(widget.currentRow.CDOPERRESPONSAVEL &&  widget.currentRow.CDOPERRESPONSAVEL.length > 0){
                    fieldOperAtual.clearValue();
                    widget.currentRow.CDOPERRESPATUAL = [];
                    fieldOperAtual.dataSourceFilter[0] = {
                        "name": "CDOPERADOR",
                        "operator": "IN",
                        "value": widget.currentRow.CDOPERRESPONSAVEL
                    };
                    
                    fieldOperAtual.readOnly = true;
                    widget.currentRow.CDOPERRESPONSAVEL.forEach(function(operador){
                        if(widget.currentRow.CDOPERRESPATUAL.indexOf(operador) === -1){
                            widget.currentRow.CDOPERRESPATUAL.push(operador);
                        }
                    });
                    
                } else {
                    if(fieldOperAtual.dataSourceFilter.length > 0){
                        fieldOperAtual.dataSourceFilter.pop();
                        fieldOperAtual.readOnly = false; 
                    }
                }
                
                fieldOperAtual.reload();
            }
        }
    };
    
    /**
     * Cancela a edição do operador responsável da proposta
     * @params{Object} widget widget popup alterar operador responsavel
    **/
    this.cancelarAlteracaoOper = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgPropostaGrid");
        var fieldProposta  = widget.getField("NRSEQORCAMENTO");
        var fieldOperAtual = widget.getField("CDOPERRESPATUAL");
                
        fieldOperAtual.readOnly = false;
        fieldOperAtual.dataSource.clearCheckedRows();
        if(fieldOperAtual.dataSourceFilter.length > 0){
            fieldOperAtual.dataSourceFilter.pop();
        }
        
        fieldProposta.dataSource.clearCheckedRows();
        if(fieldProposta.dataSourceFilter.length > 1){
            fieldProposta.dataSourceFilter.pop();
        }
        
        widget.dataSource.data = [];
        widget.isVisible = false;
        ScreenService.closePopup();
        wdgProposta.reload();
        wdgProposta.activate();
    };
    
    /**
     * Salva as alterações do operador responsável
    **/
    this.salvaAteracoesOperResp = function(widget){
        if(widget.currentRow.NRSEQORCAMENTO.length === 0 && widget.currentRow.CDOPERRESPATUAL.length !== 0){
            widget.currentRow.NRSEQORCAMENTO  = [];
            widget.getField("NRSEQORCAMENTO").dataSource.data.forEach(function(data){
                widget.currentRow.NRSEQORCAMENTO.push(data.NRSEQORCAMENTO);
            });
        }
        
        if(widget.isValid()){
            var params = {
                "NRSEQORCAMENTO"   : widget.currentRow.NRSEQORCAMENTO,
                "CDOPERRESPONSAVEL": widget.currentRow.CDNOVOOPERRESP,
                "ALTERACONTRATO"   : widget.currentRow.ALTERACONTRATO
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/salvaAlteracaoMassaOper", "Row", params).then(function(){
                ScreenService.showMessage("Propostas atualizados com sucesso.");
                this.cancelarAlteracaoOper(widget);
            }.bind(this));
        }
    };

    /**
     * Abre a widget para a adição de um novo interlocutor
     * durante a criação/edição da proposta 
     * @params{Object} widget widget prosposta view
    **/ 
   this.abreWidgetAddNewInterloc = function(widget){
       if(!widget.currentRow.NRPARCNEGOCIO){
           ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br>Selecione o parceiro de negócio ligado a proposta."));
       } else {
            ScreenService.closePopup();
            var wdgPopup = templateManager.container.getWidget("wdgNewInterlocutor");
            wdgPopup.isVisible = true;
            ScreenService.openPopup(wdgPopup).then(function(){
                wdgPopup.activate();
                wdgPopup.getField("NMINTERLOCUTOR").readOnly = false;
                wdgPopup.label = "Novo Interlocutor";
                wdgPopup.dataSource.data = [];
                wdgPopup.newRow();
                wdgPopup.currentRow.NRPARCNEGOCIO       = widget.currentRow.NRPARCNEGOCIO;
                wdgPopup.currentRow.DSEMAIL             = widget.currentRow.EMAILINTERLOC;
                wdgPopup.currentRow.NRTELEFONE          = widget.currentRow.TELEFONE1;
                wdgPopup.currentRow.IDNOMEVINDODAAPI    = "N";
                wdgPopup.currentRow.DTNASCIMENTO  = "";
                // wdgPopup.currentRow.NRCPFINTERLOC = widget.currentRow.NRCPFINTERLOC;
                //widget.currentRow.IDTPSIGNATARIO = "N";
                this.mudaTipoSignatario(wdgPopup);
                this.gerarEventosCPF(wdgPopup);
                templateManager.updateTemplate().then(function(){
                    Utilities.clearCalendarField("DTNASCIMENTO");
                });
            }.bind(this));
       }
   };
   
   /**
    * Esse procedimento é responsável por gerar o evento do OnChange do campo de CPF da popup dos interlocutores.
    * @params {Widget} widget da popup do interlocutor
   */
    this.gerarEventosCPF = function(popUp) {
        var cpf = popUp.getField("NRCPFINTERLOC");
        var data = popUp.getField("DTNASCIMENTO");
        cpf.events = [{
            "id": "30185",
            "name": "FieldOnChange",
            "code": "Sm00001000_LancamentoProposta_Controller.fazerConsultaETrocarNome(args.owner.field.widget);"
        }];
        data.events = [{
            "id": "30185543",
            "name": "FieldOnChange",
            "code": "Sm00001000_LancamentoProposta_Controller.fazerConsultaETrocarNome(args.owner.field.widget);"
        }];
    };
    
    /**
    * Procedimento responsável por substituir os dados da popup do Interlocutor com uma row. 
    * @params{Object} - popupWdg widget da popup dos interlocutores 
    * @params{Object} - row onde os dados estao para substituirem a popup 
    */
    this.substituiRowDaPopupDoInterlocutor = function(popupWdg, row) {
        popupWdg.currentRow.NRPARCNEGOCIO = row.NRPARCNEGOCIO;  
        popupWdg.currentRow.NMINTERLOCUTOR = row.NMINTERLOCUTOR;  
        popupWdg.currentRow.NRCPFINTERLOC = row.NRCPFINTERLOC;  
        popupWdg.currentRow.DSEMAIL = row.DSEMAIL;  
        popupWdg.currentRow.EMAILINTERLOC = row.EMAILINTERLOC;
        popupWdg.currentRow.NRSEQINTERLOCUTOR = row.NRSEQINTERLOCUTOR;
        popupWdg.currentRow.IDTPSIGNATARIO = (row.IDTPSIGNATARIO) ? row.IDTPSIGNATARIO :  "N";  
        popupWdg.currentRow.DTNASCIMENTO = row.DTNASCIMENTO;  
        popupWdg.currentRow.IDNOMEVINDODAAPI = (row.IDNOMEVINDODAAPI) ? row.IDNOMEVINDODAAPI :  "N";  
        popupWdg.currentRow.__is_new = false;  
    };
    
   
    /**
    * Função responsável por retornar uma row partindo de um data de um datasource. Usada ao clicar no botão de edição da popup interlocutor.
    * @params{Object} - datasource.data dados que devem ser extraídos
    * @params{boolean} - isNew valor do isnew da row, padrão é falso 
    * @returns {row} - row a ser retornada.
    */
    this.extraiInterlocutorRetornandoRow = function(datasourceData, isNew){
        if (isNew == undefined) {
            isNew = false;
        }

        var row = {
            "NMINTERLOCUTOR": datasouceData.NMINTERLOCUTOR,
            "DSEMAIL" :datasouceData.EMAILINTERLOCUTOR,
            "IDTPSIGNATARIO": datasouceData.IDTPSIGNATARIO,
            "NRPARCNEGOCIO" :datasouceData.NRPARCNEGOCIO,
            "NRCPFINTERLOC" :datasouceData.NRCPFINTERLOC,
            "NRTELEFONE": datasouceData.NRTELEFONE1,
            "IDNOMEVINDODAAPI": datasouceData.IDNOMEVINDODAAPI,
            "__is_new": isNew
        };   
        console.assert( (row) , "Não foi possível extrair uma row do datasource.data");
        return row ;
    };
    
    this.abrirPopupInterlocutorModoEdicao = function(widget){
        if(!widget.currentRow.NRPARCNEGOCIO){
           ScreenService.showMessage(ScreenService.i18n("Operação bloqueada.<br>Selecione o parceiro de negócio ligado a proposta."));
       } else {
            ScreenService.closePopup();
            var wdgPopup = templateManager.container.getWidget("wdgNewInterlocutor");
            wdgPopup.isVisible = true;
            ScreenService.openPopup(wdgPopup).then(function(){
                wdgPopup.activate();
                wdgPopup.dataSource.data = [];
                wdgPopup.newRow();
                this.substituiRowDaPopupDoInterlocutor(wdgPopup, widget.currentRow);
                wdgPopup.getField("NMINTERLOCUTOR").readOnly = (wdgPopup.currentRow.IDNOMEVINDODAAPI != "N");
                wdgPopup.label = "Editar Interlocutor";
                this.gerarEventosCPF(wdgPopup);
                this.mudaTipoSignatario();
                templateManager.updateTemplate();
            }.bind(this));
       }
    };
    
    
    /**
     * Função responsável por criar uma action de editar Interlocutor
     * @return {Object} action - a ação de editar
     */
    this.criarActionEditarInterlocutor = function(){
        return {
            "icon": "pencil",
            "showAsAction": "grid-right",
            "tooltip": "Editar",
            "isVisible": true,
            "condition": true,
            "events": [
                {
                    "name": "ActionEvent",
                    "code": "Sm00001000_LancamentoProposta_Controller.abrirPopupInterlocutorModoEdicao(args.owner.widget)"
                }
            ]
        };
    };
    
    /**
     * Evento chamado para acionar a ação de edit do interlocutor
    */
    this.criarAcaoEditInterlocutor = function(field){
        SelectUtil.construirWidgetSelect(field, this.criarActionEditarInterlocutor());
    };
    
    /**
     * Esse procedimento é responsável por fazer toda a funcionalidade de substituir o nome do interlocutor baseado no CPF
     * @params{Widget} widget da popup do interlocutor
    */
    this.fazerConsultaETrocarNome = function(widget){
        //aplicar a validaçãqo do general
        widget.getField("NMINTERLOCUTOR").readOnly = false;
        widget.currentRow.IDNOMEVINDODAAPI = "N";
        var required = (widget.getField("NRCPFINTERLOC").validations) ? widget.getField("NRCPFINTERLOC").validations.required : false;
        var cpfValido = General.validateCPF(widget.currentRow.NRCPFINTERLOC);
        var data = widget.currentRow.DTNASCIMENTO; 
        var cpf = widget.currentRow.NRCPFINTERLOC; 
        if(cpfValido || required ){
            this.pegarValidacaoCPF().then(function(response){
                if(response.dataset.resulted.IDVALIDACPFRF != 'N'){
                    this.efetuarConsulta(widget, cpfValido);
                }else{
                    if(data && cpf){
                        ScreenService.confirmMessage("Deseja buscar o nome da pessoa de acordo com o CPF e Data de Nascimento?", "question", function(){
                            this.efetuarConsulta(widget, cpfValido);
                        }.bind(this));
                    }
                }
            }.bind(this));
        }else{
           if(required ) 
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br>O valor preenchido no CPF não representa um valor válido."))
        }
    };
    
    /**
     * Função responsável por fazer a consulta do CPF e alterar o nome do campo da popup
    */
    this.efetuarConsulta = function(popUp, cpfValido){
        if (cpfValido == undefined) {
            cpfValido = false;
        }

        var data = popUp.currentRow.DTNASCIMENTO; 
        var cpf = popUp.currentRow.NRCPFINTERLOC; 
        if (data && cpf) {
            if (cpfValido) {
                this.validaNaAPIAntes(data,cpf); 
            }
            else {
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>O CPF informado é inválido."));
            }
        }
        
        /*if(cpfValido && data && cpf)
            this.validaNaAPIAntes(data,cpf); 
        else{
            ScreenService.showMessage(ScreenService.i18n(`Operação bloqueada! <br><br> ${!cpfValido 
            ? "O valor do campo do CPF é inválido, favor inserir um valor válido para o CPF."
            : "O campo de data deve possuir algum valor, favor inserir um valor no campo Data de Nascimento para efetuar a consulta."
            }`));
        }*/
    };
    
    
    /**
     * Essa função vê o estado da consulta automática do CPF parametrizada na tela de Parametrizações SM
     * @returns {Promise} Promise da consulta à tela de Parametrizações
    */
    this.pegarValidacaoCPF = function(){
        return General.callRestEngine("product#/fct_parametrogeral_widget/pegarValidacaoCPF", "Row", {});
    };
    
    /**
     * Esse procedimento efetua a validação e retornar a resposta
     * @params {string} data de nascimento
     * @params {string} número de cpf
    */
    this.validaNaAPIAntes = function(data, cpf){
        var params = {
            "DTNASCIMENTO" : data,
            "NRCPFINTERLOC" : cpf
        };
        General.callRestEngine("product#/consultaDadosCPF", "Row", params).then(function(response){
            var obj = response.dataset.CONSULTA;
            if(obj.Status){
               this.alterarNomeInterlocutor(obj.Nome);
            }else{
                ScreenService.showMessage(ScreenService.i18n('Operação bloqueada! <br><br>Falha na integração com a API de validação do CPF: <br>' + obj.Mensagem));
            }
        }.bind(this)).catch(function(err){
            ScreenService.showMessage(ScreenService.i18n(err));
        });
    };
    
    /**
     * Procedimento responsável por substituir o campo de nome e transformar ele em readonly, chamada somente em uma requisição bem sucedida
     * @params {string} nome a ser substituido
     * @params {Widget} widget da popup do interlocutor, pode ser opcional
    */
    this.alterarNomeInterlocutor = function(nome, wdg) {
        if (wdg == undefined) {
            wdg = null;
        }

        if(wdg == null)
            wdg  = templateManager.container.getWidget("wdgNewInterlocutor");
        wdg.currentRow.NMINTERLOCUTOR = nome;
        wdg.getField("NMINTERLOCUTOR").readOnly = true;
        wdg.currentRow.IDNOMEVINDODAAPI = "S";
    };

   /**
     * Salvar um novo interlocutor 
     * @params{Object} widget popup adicionar interlocutor
    **/
    this.salvarNovoInterlocutor = function(widget){
        var interlocutor = widget.currentRow.NMINTERLOCUTOR;
        var email        = widget.currentRow.EMAILINTERLOC;
        var telefone     = widget.currentRow.NRTELEFONE1;
        var wdgProposta  = templateManager.container.getWidget("wdgProposta");
        
        var podeSalvar = true;
        var isNew = widget.currentRow.__is_new;
        
        if (widget.isValid()) {
            //chamar o valida email e cpf
            var cpf = widget.currentRow.NRCPFINTERLOC;
            if (!Util.isEmptyOrBlank(cpf) && !General.validateCPF(cpf)) {
                podeSalvar = false;
                Utilities.showErrorMessage("Operação bloqueada! <br><br>CPF inválido, favor inserir um valor válido para o CPF.");
            }
            
            /* if(!Utilities.verificaEmail(email)){
                podeSalvar = false;
                Utilities.showErrorMessage("Operação bloqueada! <br>E-mail inválido.");
            }*/
                
            if (podeSalvar) {
                General.widgetSave(widget).then(function(response){
                    if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                        wdgProposta.currentRow.NMPARCINTERLOCUTOR = interlocutor;
                        wdgProposta.currentRow.DSEMAIL            = email;
                        wdgProposta.currentRow.NRTELEFONE         = telefone;
                        wdgProposta.currentRow.NRSEQINTERLOCUTOR  = response.dataset.columnsToReturn[0].NRSEQINTERLOCUTOR ; 
                    }
                    widget.isVisible = false;
                    widget.template = widget.templateToForm;
                    
                    ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                    ScreenService.closePopup();
                    wdgProposta.activate();
                    if(!isNew){
                        var field = wdgProposta.getField("NMPARCINTERLOCUTOR");
                        ScreenService.openPopup(field.selectWidget);
                        field.selectWidget.reload();
                    }
                }.bind(this))
                .catch(function(error){
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    
    /**
     * Volta para o grid de itens
    **/
    this.voltarItensGrid = function(widget){
        if(["V", "P"].includes(widget.currentRow.IDTPDESCORCA) && parseFloat(widget.currentRow.VRDESCORCA) > 0){
            widget.currentRow.VRDESCORCA = parseFloat(widget.currentRow.VRDESCORCA) * -1;
            widget.currentRow.VRDESCORCA = widget.currentRow.VRDESCORCA.toFixed(2).replace(".", ",");
        }
        ScreenService.closeSwipe();
        widget.reload();
    };
    
    this.wdgParametrosFaturamentoItemNewRow = function(wdgParametrosFaturamentoItem) {
        General.widgetNewRow(wdgParametrosFaturamentoItem);
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem);

    };
    
    this.wdgParametrosFaturamentoItemEdit = function(wdgParametrosFaturamentoItem) {
        General.widgetEdit(wdgParametrosFaturamentoItem);
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem);
    };
    
    this.wdgParametrosFaturamentoItemSave = function(wdgParametrosFaturamentoItem) {
        if (wdgParametrosFaturamentoItem.isValid()) {
            Utilities.validarCamposDuplicados(wdgParametrosFaturamentoItem, ["NRPARAMFATITPROP"], ["NRSEQORCAMENTO", "NRITEMORCAMENTO"])
            .then(function(validacao) {
                if (!validacao) {
                    ScreenService.showMessage("Operação bloqueada. <br/>Já existe uma parametrização de faturamento cadastrada para este item.");
                } else {
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgParametrosFaturamentoItem);
                }
            });
        }
    };
    /**
     * valida se o valor informado para a quandidade de medida do item se encontra 
     * na faixa de preco estabelecida na tabela de variação
    **/
    this.validarVariacaoPreco = function(nrseqorcamento, cdservorca, qtmedidaorca, cdtpmedida, idSituacao, nrtipoorcamento){
        var params = {
            "NRSEQORCAMENTO"     : nrseqorcamento,
            "CDSERVORCA"         : cdservorca,
            "QTMEDIDAORCA"       : qtmedidaorca,
            "CDTPMEDIDASERV"     : cdtpmedida,
            "IDSITUACAOORCAMENTO": idSituacao,
            "NRTIPOORCAMENTO"    : nrtipoorcamento
        };
        
        return General.callRestEngine("product#/fct_orcamento_widget/validarVariacaoPreco", "Row", params);
    };
    /**
     * Verifica qual widget está chamando a validação e capta os dados necessários para 
     * a validação do valor da quandidade de medida
     * @params {Object} widget - widget de itens ou de tipo de medida
    **/    
    this.buscaDadosParaValidacaoQtMedida = function(widget, nomeCampoMedida){
        var alteracaoQtMedida = this.validarAlteracaoQtMedida(widget.currentRow, nomeCampoMedida);
        if (alteracaoQtMedida) {
            if(this.validarQtMedidaPropInserindo(widget.currentRow, "QTMEDIDAVARIAVEL_inline")){
                var deferred    = ZHPromise.defer();
                var wdgProposta = getWidgetProposta();
        
                var nrseqorcamento  = wdgProposta.currentRow.NRSEQORCAMENTO;
                var idSituacao      = wdgProposta.currentRow.IDSITUACAOORCAMENTO;
                var nrtipoorcamento = wdgProposta.currentRow.NRTIPOORCAMENTO;
                var cdservorca      = widget.currentRow.CDSERVORCA;
                var qtmedidaorca;
                var cdtpmedida;
                
                if(widget.name === "wdgItensPropostaTree"){
                    qtmedidaorca = widget.currentRow.QTMEDIDAORCA;
                    cdtpmedida   = widget.currentRow.CDTPMEDIDASERV;
                } else {
                    if(widget.name == "wdgItensPropostaInserindo") {
                        qtmedidaorca = widget.currentRow.QTMEDIDASERV;
                        cdtpmedida   = widget.currentRow.CDTPMEDIDASERV;
                    } else {
                        cdservorca   = [];
                        qtmedidaorca = widget.currentRow.QTMEDIDAVARIAVEL_inline;
                        cdtpmedida   = widget.currentRow.CDTPMEDIDAVARIAVEL;
                    }
                }
                
                this.validarVariacaoPreco(nrseqorcamento, cdservorca, qtmedidaorca, cdtpmedida, idSituacao, nrtipoorcamento).then(function(response){
                    if(widget.name === "wdgItensPropostaTree"){
                        var newVrUnitItem = false;
                        if(response.dataset && response.dataset.fct_orcamento_widget && response.dataset.fct_orcamento_widget.NEWVRUNITITEM){
                            newVrUnitItem = response.dataset.fct_orcamento_widget.VRPRECOVARIACAO;
                            if(widget.currentRow.VRUNITITEMORCA !== newVrUnitItem){
                                widget.currentRow.VRUNITITEMORCA = newVrUnitItem;
                            }
                        }
                    }
                    this.calcularValorTotalItem(widget);
                    
                    deferred.resolve();
                }.bind(this)).catch(function(error){
                    deferred.reject();
                    Utilities.showErrorMessage(error);
                    if(widget.name === "wdgTipoMedidaPropostaEditando"){
                        widget.currentRow.QTMEDIDAVARIAVEL_inline = widget.currentRow.QTMEDIDAVARIAVEL;
                    } else {
                        widget.currentRow.QTMEDIDAORCA = widget.backupRow.QTMEDIDAORCA;
                        this.calcularValorTotalItem(widget);
                    }
                }.bind(this));
                return deferred.promise;
            }
        }
        else {
            this.calcularValorTotalItem(widget);
        }
    };
    /**
     * Adiciona o filtro de situação e carrega os dados na tela 
     * @params{Object} widget - widget itens tree
    **/
    this.carregarOnEnterTabItens = function(widget){
        if(!widget.dataSourceFilter[3].value || widget.dataSourceFilter[3].value !== getWidgetProposta().currentRow.IDSITUACAOORCAMENTO)
            widget.dataSourceFilter[3].value = getWidgetProposta().currentRow.IDSITUACAOORCAMENTO;
        Utilities.carregarWidgetOnEnterTab(widget);
    };
    /**
     * Valida se a quantidade de medida informada durante a inserção da proposta
     * é maior ou igual ao valor da franquia
     * @params{Object} row - row do inline edit
    **/
    this.validarQtMedidaPropInserindo = function(rowTpMedida, widget, nameQtMedida){
        if(rowTpMedida.QTFRANQUIA && rowTpMedida.QTFRANQUIA > parseFloat(rowTpMedida[nameQtMedida])){
            ScreenService.showMessage("Operação bloqueada!<br>O valor da franquia não pode ser menor que o valor da quantidade.");
            widget.currentRow.QUANTIDADE_inline = rowTpMedida.QTFRANQUIA;
            return false;
        }
        return true;
    };

    /**
     * Valida se o valor unitário informado no campo é menor que 
     * o valor unitário calculado da tabela de preços.
     * @param {object} widget 
     */
    this.validarValorUnitarioItem = function(widget) {
        this.validarPermissaoAlteracaoValorUnitario(widget).then(null).catch(null).finally(function() {
            widget.name == 'wdgItensProposta' ? this.calcularValoresItemOrcamento(widget) : this.calcularValorTotalItem(widget);
        }.bind(this));
    };
    
    this.validarPermissaoAlteracaoValorUnitario = function(widget) {
        var deferred = ZHPromise.defer();
        
        var vrunititemorca         = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMORCA);
        var vrunititemorcatblpreco = Utilities.tirarMascaraValores(widget.currentRow.VRUNITITEMORCATBLPRECO);
     
        var params = {
            "NRTIPOORCAMENTO" : widget.currentRow.NRTIPOORCAMENTO
        };

        General.callRestEngine("product#/fct_orcamento_widget/validaBloqAlteraValor", "Row", params).then(function(response){

            var idaltvrtabrpecprop = response.dataset.retorno.idaltvrtabrpecprop;
            
            if (idaltvrtabrpecprop == '2' && vrunititemorca < vrunititemorcatblpreco) {
                ScreenService.showMessage("Operação bloqueada.<br/>O campo \"Valor Unitário\" não pode ser menor que o campo \"Valor Unitário da Tabela de Preços\".");
                widget.currentRow.VRUNITITEMORCA = vrunititemorcatblpreco;
                deferred.reject();
            }
            else if (idaltvrtabrpecprop == '3' && vrunititemorca > vrunititemorcatblpreco) {
                ScreenService.showMessage("Operação bloqueada.<br/>O campo \"Valor Unitário\" não pode ser maior que o campo \"Valor Unitário da Tabela de Preços\".");
                widget.currentRow.VRUNITITEMORCA = vrunititemorcatblpreco;
                deferred.reject();
            }
            else if (idaltvrtabrpecprop == '4' && vrunititemorca != vrunititemorcatblpreco) {
                ScreenService.showMessage("Operação bloqueada.<br/>O campo \"Valor Unitário\" não pode ter um valor diferente do valor do o campo \"Valor Unitário da Tabela de Preços\".");
                widget.currentRow.VRUNITITEMORCA = vrunititemorcatblpreco;
                deferred.reject();
            }
            else {
                deferred.resolve();
            }
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
            deferred.reject();
        });
        
        return deferred.promise;
    };

    this.calcularValoresItemOrcamento = function(widget) {
        var qtde          = widget.currentRow.QTITEMORCAMENTO;
        var qtdeMedida    = widget.currentRow.QTMEDIDAORCA;
        var valorUnitario = widget.currentRow.VRUNITITEMORCA;
        
        /*var tipoAjuste    = widget.currentRow.IDTPDESCORCA;
        var valorDoAjuste = widget.currentRow.VRPERCDESCORCA;
        if (typeof valorDoAjuste === "string") {
            valorDoAjuste = (valorDoAjuste.length == 0) ? 0 : parseFloat(widget.currentRow.VRPERCDESCORCA.replace(",", "."));
        }*/

        if (qtdeMedida) {
            qtdeMedida = parseFloat(qtdeMedida);
        }
        else {
            qtdeMedida = 1;
        }
        
        if (valorUnitario && qtde) {
            valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
            qtde = parseFloat(qtde.toString().replace(",", "."));
            var valorSemAjuste = valorUnitario * qtde * qtdeMedida;
            widget.currentRow.VRITEMSEMDESC = valorSemAjuste;

            var valorTotal = null;
        
            /*if (tipoAjuste == "N") {
                valorTotal = valorSemAjuste;
            }
            else if (tipoAjuste == "V") { //desconto valor
                valorTotal = valorSemAjuste - valorDoAjuste;
            }
            else if (tipoAjuste == "P") { //desconto porcentagem
                valorTotal = valorSemAjuste - (valorSemAjuste * valorDoAjuste / 100);
            }
            else if (tipoAjuste == "X") { //acrescimo valor
                valorTotal = valorSemAjuste + valorDoAjuste;
            }
            else if (tipoAjuste == "Y") { //acrescimo porcentagem
                valorTotal = valorSemAjuste + (valorSemAjuste * valorDoAjuste / 100);
            }*/
            
            valorTotal = valorSemAjuste;
            widget.currentRow.VRTOTALITEMORCA = valorTotal;
        }
    };
    
    /**
     * Controla a acessibiliada das abas da widget de tabbed principal
     * @params{string} mode   - modo da widget de tabbed 
     * @params{bool} visible  - visibilidade  
     * @params{Object} widget - widget que efetua a solucitação
    **/
    this.controlarAcessibilidadeTabbed = function(mode, visible, widget){
        var wdgTabbedPrincipal  = templateManager.container.getWidget("wdgTabbedPrincipal");
        if(mode == "edit")
            wdgTabbedPrincipal.edit();
        else 
            wdgTabbedPrincipal.view();
        var widgets = wdgTabbedPrincipal.widgets.filter(function(wid) {
            if(widget.parent && wid.name === widget.parent.name)
                widget.parent.mode = mode;
            return wid.name != widget.name && (widget.parent && wid.name !== widget.parent.name);
        });

        General.changeWidgetAcessibility(widgets, visible);
    };
    /**
     * Abre a widget de nome contratual durante a inserção da proposta
     * @params{Array} dataSource - dataSource da tela
    **/
    this.habilitarWidgetNomeContratual = function(dataSource){
        var widget     = templateManager.container.getWidget("wdgNomeContratual");
        var idarvore   = null;
        var nmInserido = [];
        
        widget.dataSource.data = [];
        dataSource.forEach(function(item) {
            if(item.IDTIPO === 'M') {
                idarvore = item.IDARVORE.split("_");
                idarvore = idarvore[0] + "_" + idarvore[1];
                if(nmInserido.indexOf(idarvore) === -1) {
                    widget.dataSource.data.push({
                        "NRITEMORCAMENTO"      : null,
                        "NMITEMORCAMENTO"      : item.NMSERVORCA,
                        "NMCONTRATUALITEMORCA" : item.NMSERVORCA,
                        "IDELEMENTO"           : idarvore
                    });
                    nmInserido.push(idarvore);
                }
            }
        });
        
        widget.enabled   = true;
        widget.isVisible = true;
        widget.activate().then(function(){
            widget.edit();
        });
    };

    /**
     * Avançar para a widget de itens inserindo
     * @params{Object} widget - widget de nome contratual
    **/
    this.avancarItensInserindo = function(widget){
        var wdgItens   = templateManager.container.getWidget("wdgItensPropostaInserindo");
        var idelemento = null;
        widget.enabled = false;
        
        wdgItens.enabled   = true;
        wdgItens.isVisible = true;
        widget.dataSource.data.forEach(function(itemNmCont){
            wdgItens.dataSource.data.forEach(function(item){
                idelemento = item.IDARVORE.split("_");
                idelemento = idelemento[0] + "_" + idelemento[1];
                if(itemNmCont.IDELEMENTO === idelemento)
                    item.NMCONTRATUALITEMORCA = itemNmCont.NMCONTRATUALITEMORCA;
            });
        });
        
        wdgItens.activate().then(function() {
            wdgItens.list();
            wdgItens.mode = "edit";
            //this.alterarVisibilidadeCamposItens(wdgItens, widget.currentRow.IDMEDIDAFIXA);
        }.bind(this));
    };

    /**
     * Volta para a widget de nome contratual
     * @params{Object} widget - widget de itens da proposta inserindo
    **/
    this.voltarWdgNomeContratual = function(widget){
        var alteracaoItens = widget.dataSource.data.filter(function(item){
            return item.changed;
        });
        
        var wdgNmContratual = templateManager.container.getWidget("wdgNomeContratual");
        if(alteracaoItens.length == 0) {
            widget.enabled = false;
            wdgNmContratual.enabled = true;
            wdgNmContratual.activate();
        } else {
            ScreenService.confirmMessage("Atenção!<br>Voltar para as etapas anteriores pode resultar na perda das alterações realizadas.<br>Deseja continuar?", "question", function(){
                widget.enabled = false;
                wdgNmContratual.enabled = true;
                wdgNmContratual.activate();
            });
        }
    };

    /**
     * Desabilita as fields de "Cancelar" e "Salvar Alterações"
     * da widget de nome contratual quando estiver em modo inserção
     * @params{Object} widget - widget de nome contratual
    **/
    this.desabilitarFieldsInline = function(widget){
        var isVisible = (getWidgetProposta().mode !== "view");
        widget.getAction('cancel').isVisible       = !isVisible;
        widget.getAction('save').isVisible         = !isVisible;
        widget.getAction('back').isVisible         = isVisible;
        widget.getAction('avancarItens').isVisible = isVisible;
        templateManager.updateTemplate();
    };

    /**
     * Valida se a widget de nome contratual está sendo editada para 
     * carregar os dados
     * @params{Object} widget - widget nome contratual
    **/
    this.validarOnEnterWdgNmContratual = function(widget){
        if(getWidgetProposta().mode != "view"){
            widget.floatingControl = false;
            widget.getField('NMCONTRATUALITEMORCA').inlineEditable = 'text';
        } else { 
            Utilities.carregarWidgetOnEnterTab(widget);
            widget.floatingControl = {
                customizationAction: {
                    filter: false,
                    layout: false,
                    view: false
                }
            };
        }
    };

    /**
     * Manda salvar um nome contratual
     * @params{Object} widget - widget nome contratual
    **/
    this.saveNomeContratual = function(widget){
        var rowsAlteradas = widget.dataSource.data.filter(function(row){
            return row.changed;
        });
        
        General.callRestEngine("product#/fct_itemorcamento_widget/salvarNomeContratual", "DataSet", rowsAlteradas).then(function(){
            widget.list();
            widget.reload().then(function(){
                ScreenService.showMessage("Processo realizado com sucesso!");
            });
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    

    this.wdgNegociacaoValoresAfterMoveRow = function(wdgNegociacaoValores) {
        var idtpfatura               = wdgNegociacaoValores.currentRow.IDTPFATURA;
        var wdgParcelas              = templateManager.container.getWidget("wdgParcelas");
        var wdgDescontoPorPeriodo    = templateManager.container.getWidget("wdgDescontoPorPeriodo");
        var wdgCompetenciaNegociacao = templateManager.container.getWidget("wdgCompetenciaNegociacao");

        if (idtpfatura == "P") {
            wdgParcelas.isVisible = true;
            wdgDescontoPorPeriodo.isVisible = false;
            wdgCompetenciaNegociacao.isVisible = false;
        } else {
            wdgParcelas.isVisible = false;
            wdgDescontoPorPeriodo.isVisible = true;
            wdgCompetenciaNegociacao.isVisible = true;
        }
    };
    
    this.wdgDescontoPorPeriodoOnEnterTab = function(wdgDescontoPorPeriodo) {
        if (getWidgetNegociacaoDeValores().currentRow.IDTPFATURA != 'X') {
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").readOnly = true;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").hideEmptyValue = false;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").defaultValue = null;
        }
        else {
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").readOnly = false;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").hideEmptyValue = true;
            wdgDescontoPorPeriodo.getField("IDTPDESCONTO").defaultValue = "P";
        }
        Utilities.carregarWidgetOnEnterTab(wdgDescontoPorPeriodo);
    };
    
    /**
     * Habilita a widget de desconto por período para o recebimento de um novo registro
     * @params{Object} wdgDescontoPorPeriodo - widget de desconto por período
    **/
    this.wdgDescontoPorPeriodoNewRow = function(wdgDescontoPorPeriodo) {
        Utilities.widgetNewRow(wdgDescontoPorPeriodo);
        this.controlarCamposDescontos(wdgDescontoPorPeriodo);
        
        var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
        wdgDescontoPorPeriodo.currentRow.VRTOTNEGPROP = wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC;
    };

    this.atualizarValorItensNegociacaoValores = function(row) {
        var params = {
            "NRSEQORCAMENTO": row.NRSEQORCAMENTO,
            "NRITEMORCAMENTO": row.NRITEMORCAMENTO,
            "NRITEMORCAMENTOPAI": row.NRITEMORCAMENTOPAI,
            "VRTOTALITEMORCA": row.VRTOTALITEMORCA
        };

        return General.callRestEngine("product#/fct_itemorcamento_widget/atualizarValorItensNegociacaoValores", "Row", params);
    };

    this.salvarParcelaNegociacao = function(wdgParcelas) {
        if (wdgParcelas.isValid()) {
            Utilities.validarCamposDuplicados(wdgParcelas, ["NRPARCELANEGOPROP"], ["QTDIASFATURAMENTO", "NRNEGOCIACAOPROPOSTA"]).then(function(response) {
                if (response) {
                    
                    var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores"); 
                    
                    wdgParcelas.currentRow.NRNEGOCIACAOPROPOSTA = wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA;
                    
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgParcelas).then(function(){
                        
                        var params = {
                            "NRNEGOCIACAOPROPOSTA" : wdgParcelas.currentRow.NRNEGOCIACAOPROPOSTA
                        };
                        
                        General.callRestEngine("product#/fct_orcamento_widget/atualizarNrParcelas", "Row", params).then(function(){
                
                            wdgParcelas.reload();
                        }).catch(function(error) {
                            Utilities.showErrorMessage(error);
                        }); 
                    });
                }
                else {
                    ScreenService.showMessage('Operação bloqueada. <br/>Não é possível inserir 2 registros com o mesmo valor para o campo "Quantidade de Dias para Faturamento".');
                }
            });
        }
    };
    /**
     * Salvar um registro de desconto por período
    **/
    this.salvarDescontoPorPeriodo = function(wdgDescontoPorPeriodo) {
        if (wdgDescontoPorPeriodo.isValid()) {
            this.validarNegociacaoValorDescontoOperadorEPerfil(wdgDescontoPorPeriodo.currentRow).then(function(continuar){ //Pode continuar o save?
                if(continuar){
                    Utilities.validarCamposDuplicados(wdgDescontoPorPeriodo, ["NRDESCPERIODONEGOPROP"], ["NRORDEMAPLICADESC", "NRNEGOCIACAOPROPOSTA"]).then(function(response) {
                        if(!response) {
                            ScreenService.showMessage('Operação bloqueada.<br/>Não é possível inserir 2 registros com o mesmo valor para o campo "Ordem de Aplicação".');
                        } else {
                            if(wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM != 100) {
                                wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM   = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM);
                                wdgDescontoPorPeriodo.currentRow.VRDESCITEM       = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRDESCITEM);
                                wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC);
                                Utilities.widgetSaveAndSetColumnsToReturn(wdgDescontoPorPeriodo);
                            } else {
                                Utilities.validarCamposDuplicados(wdgDescontoPorPeriodo, ["NRDESCPERIODONEGOPROP"], ["VRPERCDESCITEM", "NRNEGOCIACAOPROPOSTA"]).then(function(response) {
                                    if(!response) {
                                        ScreenService.showMessage('Operação bloqueada.<br/>Não é possível inserir 2 registros com 100% de desconto.');
                                    } else {
                                        wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM   = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRPERCDESCITEM);
                                        wdgDescontoPorPeriodo.currentRow.VRDESCITEM       = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRDESCITEM);
                                        wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC = Utilities.tirarMascaraValores(wdgDescontoPorPeriodo.currentRow.VRTOTITEMCOMDESC);
                                        Utilities.widgetSaveAndSetColumnsToReturn(wdgDescontoPorPeriodo);
                                    }
                                });
                            }
                        }
                    });
                }
                // else{
                //   ScreenService.showMessage(ScreenService.i18n("O registro não pode ser salvo pois o perfil ou operador atual não pode efetuar mais descontos nesta faixa.")) 
                // }
            });
        }
    };

    this.abrirWdgParcelarNegociacaoPorIgual = function() {
        var wdgParcelarNegociacaoPorIgual = templateManager.container.getWidget("wdgParcelarNegociacaoPorIgual");
        wdgParcelarNegociacaoPorIgual.isVisible = true;
        ScreenService.openPopup(wdgParcelarNegociacaoPorIgual);
        wdgParcelarNegociacaoPorIgual.dataSource.data = [];
        wdgParcelarNegociacaoPorIgual.newRow();
    };

    this.parcelarNegociacaoPorIgual = function(wdgParcelarNegociacaoPorIgual) {
        if (wdgParcelarNegociacaoPorIgual.isValid()) {
            this.validarQuantidadeMaxParcelas(wdgParcelarNegociacaoPorIgual, "PARCELAPORIGUAL").then(function() {
                var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
                var params = {
                    "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA,
                    "QTPARCELAS"          : wdgParcelarNegociacaoPorIgual.currentRow.QTPARCELAS,
                    "QTDIASINTERVALO"     : wdgParcelarNegociacaoPorIgual.currentRow.QTDIASINTERVALO,
                    "NRDIAVENCIMENTO"     : wdgParcelarNegociacaoPorIgual.currentRow.NRDIAVENCIMENTO,
                    "QTCARENCIADIAS"      : wdgParcelarNegociacaoPorIgual.currentRow.QTCARENCIADIAS,
                    "VRTOTITEMCOMDESC"    : wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC
                };
    
                General.callRestEngine("product#/fct_orcamento_widget/parcelarNegociacaoPorIgual", "Row", params).then(function(response) {
                    ScreenService.closePopup();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    templateManager.container.getWidget("wdgParcelas").reload();
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };

    this.controlarAcoesDeOperacaoParcelasNegociacao = function(situacao) {
        var wdgParcelas = templateManager.container.getWidget("wdgParcelas");
        var actParcelarPorIgual = wdgParcelas.getAction("parcelarPorIgual");
        //Controla a visibilidade do Parcelar por Igual
        actParcelarPorIgual.isVisible = (situacao == "ELABORACAO" || (situacao == 'CONCLUIDO' && idAlteraPropConc == 'S' && visibilidadeAcoes));
    };
    
    this.pegarStatusCarenciaMaxima = function(){
        var deferred = ZHPromise.defer();
        var params = {};
        General.callRestEngine('product#/fct_parametrogeral_widget/pegarIdControleCarencia', "Row", params).then(function(response) {
            var resposta =  ( response.dataset.request.resulted !== "N");
            deferred.resolve(resposta);
        }).catch(function(e){
            Utilities.showErrorMessage(e);
            deferred.reject(e);
        });
        return deferred.promise;
    };
    
    this.validarNegociacaoValorDescontoOperadorEPerfil = function(row){
        var deferred = ZHPromise.defer();
        var edit = !row.__is_new;
        this.pegarStatusCarenciaMaxima().then(function(statusCarencia){
            if(statusCarencia){
                var params = {
                  "NRNEGOCIACAOPROPOSTA" : row.NRNEGOCIACAOPROPOSTA  
                };
                return General.callRestEngine("product#/fct_itemorcamento_widget/validarNegDescOperadorPerfil", "Row", params);
            }
        }).then(function(response){
            var carencia =  response ?  response.dataset.request.resulted : response;
            var permitir = true;
            if(carencia){ //* Carencia está ativada?
                // #region SETUP
                var parcelas = parseInt(row.QTMESESDESC); 
                var percentual = parseFloat(row.VRPERCDESCITEM);
                // SETAR OPERADOR
                var  operador = (!carencia.CARENCIAMAXOPERADOR[0]) ? null : {
                    "PARCELAS" : carencia.CARENCIAMAXOPERADOR[0].NRPARCELASMAX,
                    "PERCENTUAL": carencia.CARENCIAMAXOPERADOR[0].VRPERCMAXATE
                };
                // SETAR FAIXA OPERADOR
                var faixaOperador=null;
                var qtdMaximaOperador; // maximo de parcelas
                if(carencia.CARENCIA){
                    var faixaOp = carencia.CARENCIA.filter(function(fop){
                        return fop.IDENTIFICADOR === "OPERADOR"; 
                    });
                   qtdMaximaOperador = (!faixaOp[0]) ? 0 : faixaOp[0].MAXIMO;
                    faixaOperador = (!faixaOp) ? null : this.extrairFaixaDeDescontoPorPeriodo(faixaOp, parcelas, percentual);
                }
                // SETAR PERFIL ACESSO
                var perfilAcesso = (!carencia.CARENCIAMAXPERFIL[0]) ? null : {
                    "PARCELAS" : carencia.CARENCIAMAXPERFIL[0].NRPARCELASMAX,
                    "PERCENTUAL": carencia.CARENCIAMAXPERFIL[0].VRPERCMAXATE
                } ; 
                // SETAR FAIXA PERFIL ACESSO
                var faixaPerfil= null;
                var qtdMaximaPerfil; // quantidade maxima por perfil
                if(carencia.CARENCIA){
                    var faixaPa = carencia.CARENCIA.filter(function(fpa){
                        return fpa.IDENTIFICADOR === "PERFIL"; 
                    });
                    qtdMaximaPerfil = (!faixaPa[0]) ? 0 : faixaPa[0].MAXIMO;
                    faixaPerfil = (!faixaPa) ? null : this.extrairFaixaDeDescontoPorPeriodo(faixaPa, parcelas, percentual);
                }
                // APLICAÇÂO DAS REGRAS
                var permitido = this.validarRegrasDesconto(faixaOperador,operador,faixaPerfil,perfilAcesso, parcelas, percentual);
                if(permitido.status){
                    //validar a quantidade de descontos dadas por este operador ou perfil
                    if(permitido.identificador === "operador" && carencia.OPERADOR[0]){
                        var qtdParcelasMax = (edit) ? parseInt(carencia.OPERADOR[0].QTPARCELAS - templateManager.container.getWidget("wdgDescontoPorPeriodo").backupRow.QTMESESDESC) : parseInt(carencia.OPERADOR[0].QTPARCELAS) ;
                        if(edit){
                            if(permitido.faixa){
                               permitido = this.validarQuantidadePermitidaFaixaOperador(qtdParcelasMax, parcelas, faixaOperador, edit);
                            } else if(!permitido.faixa){
                              permitido =  this.validarQuantidadePermitidaOperador(qtdParcelasMax, parcelas, operador, edit);
                            }
                        }else{ 
                            if(permitido.faixa){
                               permitido = this.validarQuantidadePermitidaFaixaOperador(qtdParcelasMax, parcelas, faixaOperador);
                            } else if(!permitido.faixa){
                              permitido =  this.validarQuantidadePermitidaOperador(qtdParcelasMax, parcelas, operador);
                            }
                        }
                    }else if(permitido.identificador === "perfil" && carencia.PERFIL[0]){
                        var qtdParcelasMax = (edit) ? parseInt(carencia.PERFIL[0].QTPARCELAS - templateManager.container.getWidget("wdgDescontoPorPeriodo").backupRow.QTMESESDESC) : parseInt(carencia.PERFIL[0].QTPARCELAS) ;
                        if(edit){
                            if(permitido.faixa){
                               permitido = this.validarQuantidadePermitidaFaixaPerfil(qtdParcelasMax, parcelas, faixaPerfil, edit);
                            } else if(!permitido.faixa){
                              permitido =  this.validarQuantidadePermitidaPerfil(qtdParcelasMax, parcelas, perfilAcesso, edit);
                            }
                        }else{
                            if(permitido.faixa){
                               permitido = this.validarQuantidadePermitidaFaixaPerfil(qtdParcelasMax, parcelas, faixaPerfil);
                            } else if(!permitido.faixa){
                              permitido =  this.validarQuantidadePermitidaPerfil(qtdParcelasMax, parcelas, perfilAcesso);
                            }
                        }
                    }
                }
                
                permitir = permitido.status;
                if(!permitido.status) 
                    ScreenService.showMessage(ScreenService.i18n('Operação Bloqueada! <br/><br/>' + permitido.mensagem));
            }
                deferred.resolve(permitir); 
        }.bind(this)).catch(function(e){
            Utilities.showErrorMessage(e);
            deferred.reject(e); 
        });
        return deferred.promise;
    };
    
    this.validarObj = function(obj, parcelas,percentual){
        var primeira = false;
        var permitir = obj.PERCENTUAL >= percentual ;
        if(!permitir)
            primeira = true;
        else
            permitir = obj.PARCELAS >= parcelas;
        return {"STATUS":permitir,"PRIMEIRA":primeira}
    };
    
    this.validarQuantidadePermitidaOperador = function(qtParcelas,parcelas,operador) {
        var permitir = (qtParcelas + parcelas) <= operador.PARCELAS;
        var permitido ={
            "status" : permitir,
            "mensagem" : (!permitir)? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + operador.PARCELAS + ' parcelas.': "",
            "identificador":"operador",
            "faixa":false
        };
        return permitido;
    };
    
    this.validarQuantidadePermitidaFaixaOperador = function(qtParcelas,parcelas,faixaOperador) {
        var permitir = (qtParcelas + parcelas) <= faixaOperador.PARCELAS;
        var permitido ={
            "status" : permitir,
            "mensagem" : (!permitir)? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaOperador.PARCELAS + ' parcelas.': "",
            "identificador":"operador",
            "faixa":false
        };
        return permitido;
    };
    
    this.validarQuantidadePermitidaPerfil = function(qtParcelas,parcelas,perfil){
        var permitir = (qtParcelas+ parcelas) <= perfil.PARCELAS;
        var permitido ={
            "status" : permitir,
            "mensagem" : (!permitir)? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + perfil.PARCELAS + ' parcelas.': "",
            "identificador":"perfil",
            "faixa":false
        };
        return permitido;
    };
    
    this.validarQuantidadePermitidaFaixaPerfil = function(qtParcelas,parcelas,faixaPerfil){
        var permitir = (qtParcelas + parcelas) <= faixaPerfil.PARCELAS;
        var permitido ={
            "status" : permitir,
            "mensagem" : (!permitir)? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaPerfil.PARCELAS + ' parcelas.': "",
            "identificador":"perfil",
            "faixa":true
        };
        return permitido;
    };
    
    this.pegarRecebidoNulo = function(){
        return  {
            "status" : false,
            "mensagem" : "",
            "identificador":"",
            "faixa":false
        };
    };
    
    this.validarRegrasDesconto = function(faixaOperador,operador,faixaPerfil,perfilAcesso, parcelas, percentual){
        /*REGRAS
            Na presença de um OPERADOR DEVE IGNORAR O PERFIL DE ACESSO
            Na presença de uma FAIXA DEVE IGNORAR O MAXIMO PARAMETRIZADO ( PERFIL && OPERADOR)
            1. FAIXA OPERADOR
            	1. ANALISAR SOMENTE A FAIXA
            2. OPERADOR 
            	1. ANALISAR SOMENTE O OPERADOR
            3. FAIXA PERFIL ACESSO 
            	1. ANALISAR FAIXA
            	2. ANALISAR MAXIMO PARAMETRIZADO
            4. MAXIMO PARAMETRIZADO
            	1. ANALISAR MAXIMO PARAMETRIZADO
            5.  Ao inserir deve se pegar a quantidade da soma que existe na tabela(operador e perfil)
            	1.  o da faixa é o valor maximo de parcelas da faixa
            6.  Ao editar deve se pegar todas menos a que esta sendo editada
            	1.  o valor da faixa é o valor maximo de parcelas da faixa
        */
        var permitido =this.pegarRecebidoNulo();
        if(faixaOperador){
            permitido = this.validarFaixaOperador(faixaOperador,parcelas,percentual);
        }else if(operador){
            permitido = this.validarOperador(operador,parcelas,percentual);
        }else if(faixaPerfil){
            permitido = this.validarFaixaPerfilAcesso(faixaPerfil,parcelas,percentual)|| this.validarPerfilAcesso(perfilAcesso,parcelas,percentual);
        }else if(perfilAcesso) {
            permitido =  this.validarPerfilAcesso(perfilAcesso,parcelas,percentual);
        }
        return permitido;
    };
    
    this.validarFaixaOperador = function(faixaOperador,parcelas, percentual){
        var permitir = this.validarObj(faixaOperador,parcelas,percentual);
        return {
            "status":permitir.STATUS,
            "mensagem" : (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaOperador.PARCELAS + 'parcelas.' : 'O operador logado não tem permissão para dar desconto maior que ' + faixaOperador.PERCENTUAL + '%' : "",
            "identificador":"operador",
            "faixa":true
        }
    };
    
    this.validarFaixaPerfilAcesso = function(faixaPerfil,parcelas, percentual){
        var permitir = this.validarObj(faixaPerfil,parcelas,percentual);
        return {
            "status":permitir.STATUS,
            "mensagem" : (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que ' + faixaPerfil.PARCELAS + 'parcelas.' : 'O perfil de acesso do operador logado não tem permissão para dar desconto maior que ' + faixaPerfil.PERCENTUAL + '%' : "",
            "identificador":"perfil",
            "faixa":true
        }
    };
    
    this.validarOperador = function(operador,parcelas, percentual){
        var permitir = this.validarObj(operador,parcelas,percentual);
        return {
            "status":permitir.STATUS,
            "mensagem" : (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O operador logado não tem permissão para dar desconto com quantidade maior que ' + operador.PARCELAS +  'parcelas.' : 'O operador logado não tem permissão para dar desconto maior que '+ operador.PERCENTUAL + '%' : "",
            "identificador":"operador",
            "faixa":false
        }
    };
    
    this.validarPerfilAcesso = function(perfil,parcelas, percentual){
        var permitir = this.validarObj(perfil,parcelas,percentual);
        return {
            "status":permitir.STATUS,
            "mensagem" : (!permitir.STATUS) ? (!permitir.PRIMEIRA) ? 'O perfil de acesso do operador logado não tem permissão para dar desconto com quantidade maior que '+ perfil.PARCELAS + 'parcelas.' : 'O perfil de acesso do operador logado não tem permissão para dar desconto maior que ' + perfil.PERCENTUAL + '%' : "",
            "identificador":"perfil",
            "faixa":false
        }
    };
    
    this.extrairFaixaDeDescontoPorPeriodo = function(dados, parcela, valor){
        var obj = null;
        if(dados.length > 0){
           dados.forEach(function(el){
            if(parseFloat(el.VRPERCDESCATE) >= valor && valor >= parseFloat(el.VRPERCDESCDE) && parseInt(el.NRPARCELASMAX) > parcela)
                obj = {
                    "PARCELAS": parseInt(el.NRPARCELASMAX),
                    "PERCENTUAL": parseFloat(el.VRPERCDESCATE)
                };
           });
        }
        return obj;
    };
    
    this.validarDadosCarencia = function(carencia){
        var perfilOperador  = [];    // carencia.PERFIL
        var dadosOperador   = {};    // carencia.OPERADOR
        var dadosPerfil     = [];    // carencia.PERFIS
        var dadosCarencia   = [];    // carencia.CARENCIA
        dadosOperador = carencia.OPERADOR[0];
        this.preencherArray(perfilOperador, carencia.PERFIL);
        this.preencherArray(dadosPerfil, carencia.PERFIS);
        this.preencherArray(dadosCarencia, carencia.CARENCIA);
        return false;
    };
    
    this.preencherArray = function(array, conjunto){
        conjunto.forEach(function(c){
            array.push(c);  
        });
    };

    this.salvarWdgNegociacaoValores = function(wdgNegociacaoValores) {
        if (wdgNegociacaoValores.isValid()) {
            var vrpercdescitem = wdgNegociacaoValores.currentRow.VRPERCDESCITEM;
            if (!vrpercdescitem) vrpercdescitem = 0;
            vrpercdescitem = Utilities.tirarMascaraValores(vrpercdescitem);
            if (vrpercdescitem > 0) {
                var params = {
                    "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA
                };
                General.callRestEngine("product#/fct_itemorcamento_widget/verificarSeExisteDescontoItensNegociacaoProposta", "Row", params)
                .then(function(success) {
                    if (success.dataset.retorno.existe) {
                        ScreenService.showMessage("Operação bloqueada. <br/>Não é possível dar desconto na negociação pois já existe desconto específico informado para seus itens.");
                    }
                    else {
                        General.callRestEngine("product#/fct_itemorcamento_widget/getServicoPercentualMinimoDescontoPermitidoNegociacaoProposta", "Row", params)
                        .then(function(response) {
                            var resultado = response.dataset.retorno.servico;
                            if (resultado && vrpercdescitem > resultado.VRPERCDESC) {
                                var contexto = (resultado.TIPO == 1) ? "Este": "O perfil de acesso deste";
                                ScreenService.showMessage("Operação bloqueada.<br/>"+contexto+" operador não tem autorização para oferecer desconto superior a " + resultado.VRPERCDESC + "% para o serviço \"" + resultado.NMSERVORCA + "\".");
                            }
                            else {
                                salvarNegociacaoValores(wdgNegociacaoValores);
                            }
                        })
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                salvarNegociacaoValores(wdgNegociacaoValores);
            }
        }
    };

    var salvarNegociacaoValores = function(wdgNegociacaoValores) {
        var deletarParcelas = (wdgNegociacaoValores.backupRow && wdgNegociacaoValores.backupRow.VRTOTITEMCOMDESC !== wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC);
        
        Utilities.widgetSaveAndSetColumnsToReturn(wdgNegociacaoValores).then(function() {
            setDataSourceTipoDesconto(wdgNegociacaoValores, false);
            
            var params = {
                "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA,
                "VRTOTITEMCOMDESC"    : wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC
            };
            General.callRestEngine("product#/fct_orcamento_widget/atualizarValoresDescontoPorPeriodoDaNegociacao", "Row", params)
            .then(function() {
                var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                this.atualizarValorProposta(wdgPropostaGrid);
                
                if(deletarParcelas) {
                    params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                    General.callRestEngine("product#/fct_orcamento_widget/excluirParcelasNegociacao", "Row", params)
                    .then().catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                
                // this.atualizarValorRateioItens(wdgNegociacaoValores); por hora, não está sendo utilizado rateio
                if (wdgNegociacaoValores.currentRow.IDTPFATURA == 'P') {
                    var mensagem = "Processo realizado com sucesso! <br><br>Atenção!<br>O valor total da soma das parcelas deve ser igual ao valor do item.";
                    ScreenService.showMessage(mensagem);
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }.bind(this));
    }.bind(this);
    
    this.cancelarSaveNegociacaoValores = function(widget) {
        setDataSourceTipoDesconto(widget, false);
        General.widgetCancelForm(widget);
    };

    this.pegarDadosDeFaturamentoGroup = function(widget){
        return widget.fieldGroups.filter(function(wid){
            return (wid.name == "fdgProdutosFaturamento")
        });
    };

    this.wdgItensNegociacaoAfterMoveRow = function(wdgItensNegociacao) {
        var campoParceiro      = wdgItensNegociacao.getField("NMPRINCIPALPARC");
        var campoValorItem     = wdgItensNegociacao.getField("VRTOTITEM");
        var campoPercDesconto  = wdgItensNegociacao.getField("VRPERCDESCITEM");
        var campoValorDesconto = wdgItensNegociacao.getField("VRDESCITEM");
        var campoTipoDesconto  = wdgItensNegociacao.getField("IDTPDESCONTO");
        var campoTipoMedida    = wdgItensNegociacao.getField("DSTPMEDIDASERV");

        var wdgProposta       = getWidgetProposta();
        var utilizaComposicao = wdgProposta.currentRow.QTITENSTRABSERV > 0;

        var dadosFaturamento =  this.pegarDadosDeFaturamentoGroup(wdgItensNegociacao)[0];
        
        if (wdgItensNegociacao.currentRow.NRITEMORCAMENTOPAI == null && utilizaComposicao) {
            campoValorItem.showOnView     = false;
            campoValorItem.showOnForm     = false;
            campoPercDesconto.showOnView  = false;
            campoPercDesconto.showOnForm  = false;
            campoValorDesconto.showOnView = false;
            campoValorDesconto.showOnForm = false;
            campoTipoDesconto.isVisible   = false;
            campoTipoMedida.showOnView    = false;
            campoTipoMedida.showOnForm    = false;
        } else {
            campoValorItem.showOnView     = true;
            campoValorItem.showOnForm     = true;
            campoPercDesconto.showOnView  = true;
            campoPercDesconto.showOnForm  = true;
            campoValorDesconto.showOnView = true;
            campoValorDesconto.showOnForm = true;
            campoTipoDesconto.isVisible   = true;
            campoTipoMedida.showOnView    = true;
            campoTipoMedida.showOnForm    = true;
        }
        
        dadosFaturamento.isVisible = (wdgItensNegociacao.currentRow._parent == null);
    };
    

    this.salvarWdgItensNegociacao = function(wdgItensNegociacao) {
        if (wdgItensNegociacao.isValid()) {
            var params               = {};
            var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
            var deletarParcelas      = (wdgItensNegociacao.backupRow && wdgItensNegociacao.backupRow.VRTOTITEMCOMDESC !== wdgItensNegociacao.currentRow.VRTOTITEMCOMDESC);
            
            if (!wdgItensNegociacao.currentRow._parent) {
                var mudouParceiro = false;
               
                if (wdgItensNegociacao.currentRow.NRPARCFATURA != wdgItensNegociacao.backupRow.NRPARCFATURA)
                    mudouParceiro = true;

                if (mudouParceiro) {
                    params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                    
                    General.callRestEngine("product#/fct_orcamento_widget/getQuantidadeItensNegociacaoProposta", "Row", params).then(function(response) {
                        var qtdeItens = response.dataset.retorno.quantidade;
                        if (qtdeItens > 1) {
                            ScreenService.confirmMessage("Atenção: <br/>Ao mudar o Parceiro de Negócio deste item, o item será movido deste grupo de Negociação de Valores para um novo grupo. Deseja continuar?", "question", function() {
                                General.widgetSave(wdgItensNegociacao).then(function() {
                                    
                                    setDataSourceTipoDesconto(wdgItensNegociacao, false);
                                    
                                    params = {
                                        "ITEM"      : wdgItensNegociacao.currentRow,
                                        "NEGOCIACAO": wdgNegociacaoValores.currentRow
                                    };
                                    
                                    General.callRestEngine("product#/fct_orcamento_widget/desagruparItemNegociacao", "Row", params).then(function() {
                                        if(deletarParcelas) {
                                            var params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                                            General.callRestEngine("product#/fct_orcamento_widget/excluirParcelasNegociacao", "Row", params)
                                            .then().catch(function(error) {
                                                Utilities.showErrorMessage(error);
                                            });
                                        }
                                        
                                        ScreenService.closeSwipe().then(function() {
                                            ScreenService.closeSwipe();
                                            wdgNegociacaoValores.activate();
                                            wdgNegociacaoValores.reload();
                                        });
                                    }).catch(function(error) {
                                        Utilities.showErrorMessage(error);
                                    });
                                });
                            });
                        }
                        else {
                            General.widgetSave(wdgItensNegociacao);
                            setDataSourceTipoDesconto(wdgItensNegociacao, false);
                        }
                    }).catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                else {
                    General.widgetSave(wdgItensNegociacao).then(function() {
                        params = {
                            "NRNEGOCIACAOPROPOSTA": wdgItensNegociacao.currentRow.NRNEGOCIACAOPROPOSTA
                        };
                        
                        General.callRestEngine("product#/fct_orcamento_widget/atualizarValorNegociacaoComSomaDosItens", "Row", params)
                        .then(function(response) {
                            setDataSourceTipoDesconto(wdgItensNegociacao, false);
                            var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
                            
                            if(deletarParcelas) {
                                params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                                General.callRestEngine("product#/fct_orcamento_widget/excluirParcelasNegociacao", "Row", params)
                                .then().catch(function(error) {
                                    Utilities.showErrorMessage(error);
                                });
                            }
                            
                            var valorNegociacao = response.dataset.retorno.valorNegociacao;
                            wdgNegociacaoValores.currentRow.VRTOTITEM = valorNegociacao;
                            wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC = valorNegociacao;
                            Utilities.calcularValoresDesconto(wdgNegociacaoValores, 'VRTOTITEM', 'VRPERCDESCITEM', 'VRDESCITEM', 'VRTOTITEMCOMDESC', 'VRPERCDESCITEM');

                            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                            this.atualizarValorProposta(wdgPropostaGrid);
                           //this.atualizarValorRateioItens(wdgItensNegociacao);  Por hora, não será utilizado rateio
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }
            else {
                params = {
                    "NRNEGOCIACAOPROPOSTA": wdgItensNegociacao.currentRow.NRNEGOCIACAOPROPOSTA
                };
                
                General.callRestEngine("product#/fct_itemorcamento_widget/verificarSeExisteDescontoNegociacaoProposta", "Row", params)
                .then(function(success) {
                    var percDescontoAtual = wdgItensNegociacao.currentRow.VRPERCDESCITEM;
                    if (!percDescontoAtual) percDescontoAtual = 0;
                    percDescontoAtual = Utilities.tirarMascaraValores(percDescontoAtual);

                    if (success.dataset.retorno.existe && percDescontoAtual > 0) {
                        ScreenService.showMessage("Operação bloqueada. <br/>Não é possível dar desconto diretamente no item pois já existe desconto para o grupo inteiro da negociação de valores.");
                    } else {
                        var nritempai            = wdgItensNegociacao.currentRow.NRITEMORCAMENTOPAI;
                        var valorDescontoServico = 0;
                        var valorServico         = 0;
                        
                        wdgItensNegociacao.dataSource.data.filter(function(row) {
                            return row.NRITEMORCAMENTOPAI == nritempai;
                        }).forEach(function(row) {
                            valorDescontoServico += Utilities.tirarMascaraValores(row.VRDESCITEM);
                            valorServico         += Utilities.tirarMascaraValores(row.VRTOTITEM);
                        });
        
                        params = {
                            "CDSERVORCA": wdgItensNegociacao.currentRow.CDSERVORCA,
                            "IDTIPO"    : "D"
                        };
                        
                        General.callRestEngine("product#/fct_itemorcamento_widget/validaPorcentualDescontoAcrescimoSave", "Row", params).then(function(response) {
                            var resultado = response.dataset.RESULTADO.VRPERCDESC;
                            var percDescontoServico = valorDescontoServico / valorServico * 100;
                            if (resultado && percDescontoServico > resultado.VRPERCDESC) {
                                var contexto = (resultado.TIPO == 1) ? "Este": "O perfil de acesso deste";
                                ScreenService.showMessage("Operação bloqueada.<br/>"+contexto+" operador não tem autorização para oferecer desconto superior a " + resultado.VRPERCDESC + "% para o serviço \"" + resultado.NMSERVORCA + "\".");
                            }
                            else {
                                var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores");
                                General.widgetSave(wdgItensNegociacao).then(function() {
                                    setDataSourceTipoDesconto(wdgItensNegociacao, false);
                                    if(deletarParcelas) {
                                        params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
                                        General.callRestEngine("product#/fct_orcamento_widget/excluirParcelasNegociacao", "Row", params)
                                        .then().catch(function(error) {
                                            Utilities.showErrorMessage(error);
                                        });
                                    }
                                    
                                    params = { "NRSEQORCAMENTO": wdgNegociacaoValores.currentRow.NRSEQORCAMENTO };
                                    if (wdgItensNegociacao.currentRow._parent) {
                                        params = {
                                            "NRNEGOCIACAOPROPOSTA": wdgItensNegociacao.currentRow.NRNEGOCIACAOPROPOSTA,
                                            "NRITEMORCAMENTOPAI"  : wdgItensNegociacao.currentRow.NRITEMORCAMENTOPAI
                                        };
                                        var idParcelamento = wdgNegociacaoValores.currentRow.IDTPFATURA == 'P';
                                        General.callRestEngine("product#/fct_orcamento_widget/atualizarValoresNegociacaoAoEditarElemento", "Row", params).then(function(response2) {
                                            var mensagem = "Processo realizado com sucesso!" + ((idParcelamento) ? "<br><br>Atenção!<br>O valor total da soma das parcelas deve ser igual ao valor do item." : "");
                                            ScreenService.showMessage(mensagem);
                                            
                                            ScreenService.closeSwipe();
                                            wdgItensNegociacao.reload();
                                            
                                            var valorNegociacao      = response2.dataset.retorno.valorNegociacao;
                                            
                                            wdgNegociacaoValores.currentRow.VRTOTITEM = valorNegociacao;
                                            wdgNegociacaoValores.currentRow.VRTOTITEMCOMDESC = valorNegociacao;
                                            Utilities.calcularValoresDesconto(wdgNegociacaoValores, 'VRTOTITEM', 'VRPERCDESCITEM', 'VRDESCITEM', 'VRTOTITEMCOMDESC', 'VRPERCDESCITEM');

                                            var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
                                            this.atualizarValorProposta(wdgPropostaGrid);
                                        }.bind(this))
                                        .catch(function(error) {
                                            Utilities.showErrorMessage(error);
                                        });
                                    }
                                }.bind(this))
                                .catch(function(error) {
                                    Utilities.showErrorMessage(error);
                                });
                            }
                        }.bind(this));
                    }
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }            
        }
    };

    /**
     * Valida as conposições de serviço antes de realizar a renegociação
    **/
    this.validarRenegociacao = function(widget) {
        var params = { "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO };
        General.callRestEngine("product#/fct_itemorcamento_widget/validarAtivacaoComposicao", "Row", params).then(function(response){
            this.abrePopupDsJustCancReneg(widget, 'versionar_proposta');
        }.bind(this))
        .catch(function(error){
            if (error.data && error.data.error.indexOf('POSSUI COMPOSICAO INATIVA') !== -1) 
                ScreenService.showMessage("Operação bloqueada!<br>Não é possível renegociar propostas ligadas a composições de serviço inativas.<br>Ative-as e tente novamente!");
            else 
                Utilities.showErrorMessage(error);
        });
    };
    /**
     * Coloca a widget de itens em mode de edição
     * durante a inserção da proposta
     * @params{Object} widget - widget itens proposta inserindo
    **/
    this.controlarCamposItensPropInserindo = function(widget, isEdit){
        if(isEdit) {
            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
            widget.__is_new = false;
            this.alterarVisibilidadeAcoesItensInserindo(widget, false);
        }
        
        var isVisible   = widget.currentRow.IDTIPO === "M";
        var qtMdVisivel = (widget.currentRow.IDTIPO === "M" && widget.currentRow.IDMEDIDAFIXA === 'N' && widget.currentRow.IDMULTIPVARIACAO === "S");

        widget.getField("QTMEDIDASERV").isVisible  = isVisible;
        widget.getField("QTMEDIDASERV").readOnly   = qtMdVisivel;
        widget.getField("VRUNITITEMORCA").readOnly = (widget.currentRow.CDTPMEDVARESCOPO != null);
        widget.getField("VRUNITITEMORCA").help     = (!widget.currentRow.CDTPMEDVARESCOPO) ? null : "O campo foi desabilitado porque não é possível inserir acréscimos em itens vinculados a serviços faturados por medição.";
        this.alterarVisibilidadeCamposItens(widget, widget.currentRow.IDMEDIDAFIXA);
    };
    /**
     * Altera as actions da tela de itens durante a edição dos mesmo
     * @params{Object} widget  - widget de itens inserindo
     * @params{Bool} isVisible - visibilidade das actions que aparecem no grid 
    **/
    this.alterarVisibilidadeAcoesItensInserindo = function(widget, isVisible){
        widget.getAction("voltar").isVisible       = isVisible;
        widget.getAction("inserir_tudo").isVisible = isVisible;
        widget.getAction("act_cancel").isVisible   = !isVisible;
        widget.getAction("act_salvar").isVisible   = !isVisible;
    };
    /**
     * Volta para o grid de itens durante a insersão de uma proposta de 
     * composição de serviço
     * @params{Object} widget - widget de itens 
    **/
    this.voltarGridItensPropInserindo = function(widget){
        widget.getField("NMTIPOORCAMENTO").isVisible = true;
        widget.getField("VRUNITITEMORCA").isVisible  = true;
        widget.getField("QTMEDIDASERV").isVisible    = true;
        widget.getField("DSTPMEDIDASERV").isVisible  = true;
        
        ScreenService.closeSwipe();
        widget.mode = "edit";
        widget.activate();
        
        this.alterarVisibilidadeAcoesItensInserindo(widget, true);
    };
    /**
     * Salva as alterações realizadas nos itens durante a inserção 
     * de uma proposta de composição de serviço
    **/
    this.salvarAlteracoesItensPropInserindo = function(widget){
        this.replicarVrTotalItemPropInserindo(widget);
        this.alterarVisibilidadeAcoesItensInserindo(widget, true);
        widget.view();
    };
    /**
     * Replica o novo valor do item para seus itens pais na
     * arvore de itens durante a inserção da proposta
     * @params{Object} widget - widget de itens
    **/
    this.replicarVrTotalItemPropInserindo = function(widget){
        if(widget.currentRow.IDTIPO == "M") {
            widget.currentRow.VRUNITITEMORCA_GRID = Utilities.formatarNumero(widget.currentRow.VRUNITITEMORCA);
            widget.currentRow.QTMEDIDASERV_GRID   = Utilities.formatarNumero(widget.currentRow.QTMEDIDASERV);
                
            this.calcularValorTotalItem(widget);
            var cdservorca      = widget.currentRow.CDSERVORCA;
            var indexElSuperior = null;
            var totalServico    = 0;
            widget.dataSource.data.forEach(function(item, index){
                if(cdservorca == item.CDSERVORCA) {
                    if(!item.IDARVOREPAI)
                        indexElSuperior = index;
                    else 
                        totalServico += item.VRTOTALITEMORCA;
                }
            });
            
            if(indexElSuperior !== null) {
                widget.dataSource.data[indexElSuperior].VRTOTALITEMORCA = totalServico;
                widget.dataSource.data[indexElSuperior].VRUNITITEMORCA  = totalServico;
            }
        }
    };
    /**
     * Abre a Widget de negociação de valores para a edição
    **/
    /*this.abrirWidgetNegociacaoValores = function(currentRow) {
        var wdgPropostaGrid    = getWidgetPropostaGrid();
        var wdgProposta        = getWidgetProposta();
        var wdgTabbedPrincipal = templateManager.container.getWidget("wdgTabbedPrincipal");
        var wdgNegociacao      = templateManager.container.getWidget("wdgNegociacaoValores");
        
        wdgPropostaGrid.currentRow = {
            "IDSITUACAOORCAMENTO": "ELABORACAO",
            "IDUTILIZACOMP"      : currentRow.IDUTILIZACOMP,
            "IDUTILIZAVARPRECO"  : currentRow.IDUTILIZAVARPRECO,
            "NRSEQORCAMENTO"     : currentRow.NRSEQORCAMENTO,
            "NRORG"              : currentRow.NRORG
        };
        
        setLocalVar("PROPOSTA_NRSEQORCAMENTO", currentRow.NRSEQORCAMENTO);
        setLocalVar("PROPOSTA_NRORG", currentRow.NRORG);
        
        wdgProposta.list();
        wdgProposta.reload().then(function() {
            wdgProposta.view();
            ["NRORCAMENTO", "NMPRINCIPALPARC", "NMOPERRESPONSAVEL"].forEach(function(nameCampo) {
                if(wdgProposta.currentRow[nameCampo]) 
                    wdgProposta.descriptionOrder += ' - ' + wdgProposta.currentRow[nameCampo];
            });
        });
        
        this.validacoesGeraisProposta(wdgPropostaGrid);
        ScreenService.openSwipe(wdgTabbedPrincipal).then(function() {
            wdgTabbedPrincipal.openTab("wdgPropostaTabbed");
            wdgNegociacao.list();
            wdgNegociacao.reload().then(function() {
                wdgNegociacao.activate();
            });
        });
    };*/
    
    /**
     * Cancela a edição dos itens durante a inserção de uma proposta
     * @params{Object} widget - widget Itens proposta inserindo
    **/
    this.cancelarEdicaoItensPropostaInserindo = function(widget) {
        Utilities.widgetCancelForm(widget); 
        this.alterarVisibilidadeAcoesItensInserindo(widget, true);
    };
    /**
     * Volta para o grid de itens 
     * @params{Object} widget - widget de itens tree
    **/
    this.voltarGridItensPropEditando = function(widget){
        widget.getField("NMTIPOORCAMENTO").isVisible = true;
        ScreenService.closeSwipe();
        widget.reload();
    };
    /**
     * Abre o pop up de data do aceite 
    **/
    this.abrirWdgPopupDataDoAceite = function(widget) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("wdgPopupDataDoAceite");
        wdgPopup.isVisible = true;
        ScreenService.openPopup(wdgPopup).then(function(){
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow.NRSEQORCAMENTO = widget.currentRow.NRSEQORCAMENTO;
            wdgPopup.currentRow.NRORCAMENTO    = widget.currentRow.NRORCAMENTO;
        });
    };
    
    /**
     * valida a quantidade máxima de parcelas da negociação 
     * @params{Object} widget - widget de parcelas
     * @params{String} typeValidacao - tipo de parcela (PARCELAUNICA ou PARCELAPORIGUAL)
    **/
    this.validarQuantidadeMaxParcelas = function(widget, typeValidacao) {
        var deferred      = ZHPromise.defer();
        var rowNegociacao = templateManager.container.getWidget("wdgNegociacaoValores").currentRow;
        var params = {
            "NRNEGOCIACAOPROPOSTA" : rowNegociacao.NRNEGOCIACAOPROPOSTA, 
            "NRTIPOORCAMENTO"      : rowNegociacao.NRTIPOORCAMENTO,
            "IDTPVALIDACAO"        : typeValidacao
        };
        
        General.callRestEngine("product#/fct_parcelanegoprop/validarQuantidadeMaxParcelas", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.fct_orcamento_widget) {
                var maximoparcelas = response.dataset.fct_orcamento_widget.NRMAXPARCELAS;
                
                if (typeValidacao == "PARCELAUNICA") {
                    if (maximoparcelas.indexOf("LIMITE_PARC_INSERIDO") == -1) {
                        deferred.resolve();
                    }
                    else {
                        deferred.reject("Operação bloqueada!<br>Quantidade máxima de parcelas já cadastrada no sistema.<br>Valor máximo permitido: " + maximoparcelas.split(" ")[1]);
                    }
                }
                else if (widget.currentRow.QTPARCELAS > maximoparcelas) {
                    widget.currentRow.QTPARCELAS = "";
                    deferred.reject("Operação bloqueada!<br>A quantidade de parcelas informada excede a quantidade máxima de parcelas da negociação.<br>Valor máximo permitido: " + maximoparcelas);
                }
                else {
                    deferred.resolve();
                }
            }
        }.bind(this));

        return deferred.promise;
    };

    this.novaParcela = function(wdgParcelas) {
        this.validarQuantidadeMaxParcelas(wdgParcelas, 'PARCELAUNICA').then(function() {
            General.widgetNewRow(wdgParcelas);
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * valida se a proposta possui rateio inserido e se haverá alterações nele
    **/
    this.validarDadosRateioProposta = function(widget) {
        var deferred = ZHPromise.defer();
        var params   = { 
            "NRSEQORCAMENTO" : getWidgetProposta().currentRow.NRSEQORCAMENTO,
            "NRTIPOORCAMENTO": getWidgetProposta().currentRow.NRTIPOORCAMENTO,
            "NRESTRUTURAM"   : widget.currentRow.NRESTRUTURAM
        };
        
        General.callRestEngine("product#/fct_relacestruturaprop_widget/validarDadosRateioProposta", "Row", params).then(function(response) {
            var rateiosInseridos = false;
            if(response.dataset && response.dataset.fct_relacestruturaprop_widget && response.dataset.fct_relacestruturaprop_widget.length > 0) 
                rateiosInseridos = response.dataset.fct_relacestruturaprop_widget[0];
            return deferred.resolve(rateiosInseridos);
        }.bind(this))
        .catch(function(error) {
            ScreenService.showMessage("Falha ao validar os rateios da proposta");
            return deferred.resolve();
        });
        
        return deferred.promise;
    };
    /**
     * valida se o item de formação que está sendo inserido pode iterferir no 
     * rateio da proposta
     * @params{Object} widget - widget de itens de formação de preço
     * @params{bool} soma - se os itens serão somados ou não
    **/
    this.validarDadosRateioFormacaoPreco = function(widget, soma) {
        var deferred = ZHPromise.defer();
        this.validarDadosRateioProposta(widget).then(function(rateiosInseridos) {
            if(!rateiosInseridos) {
                this.insereItemFormPrec(widget, soma).then(function(resolve) {
                    deferred.resolve(resolve);
                });
            } else {
                var mensagem = (rateiosInseridos == "SEM_CENTROCUST") 
                    ? "Atenção!<br><br>Os locais de atendimento das formações de preço selecionadas, não está ligados a um centro de custo. Desse modo, não serão gerados rateios para os itens dessa proposta.<br>Deseja continuar?"  
                    : "Atenção!<br><br>O item selecionado não possui local de atendimento ligado a ele. Sua inserção acarretará na exclusão dos rateios criados para essa proposta.<br><br>Deseja continuar?";
                
                ScreenService.confirmMessage(mensagem, "question", function() {
                    this.insereItemFormPrec(widget, soma).then(function(resolve) {
                        deferred.resolve(resolve);
                    });
                }.bind(this));
            }
        }.bind(this));
        return deferred.promise;
    };
    /**
     * valida se o item de formação que está sendo inserido pode iterferir no 
     * rateio da proposta
     * @params{Object} widget - widget de itens de formação de preço
     * @params{bool} soma - se os itens serão somados ou não
    **/
    this.validarDadosRateioItens = function(widget) {
        widget.currentRow.NRESTRUTURAM = [null];
        this.validarDadosRateioProposta(widget).then(function(rateiosInseridos) {
            if(!rateiosInseridos) {
                this.salvarItens(widget);
            } else {
                ScreenService.confirmMessage("Atenção!<br><br>A inserção deste item impedirá a geração de rateios para os faturamentos ligados aos contratos dessa proposta.<br><br>Deseja continuar?", "question", function() {
                    this.salvarItens(widget);
                }.bind(this));
            }
        }.bind(this));
    };
    
    /**
     * Verifica se a proposta utiliza rateio e atualiza os valor deles
     * @params{Object} widget - widget de rateio 
    **/
    this.atualizarValorRateioItens = function(widget) {
        var idutilizacomp = getWidgetProposta().currentRow.IDUTILIZACOMP;
        if(!idutilizacomp || idutilizacomp == 'N') {
            var nritemorcamento = widget.currentRow.NRITEMORCAMENTO ? widget.currentRow.NRITEMORCAMENTO : null;
            var params = {
                "NRITEMORCAMENTO"      : nritemorcamento,
                "NRNEGOCIACAOPROPOSTA" : widget.currentRow.NRNEGOCIACAOPROPOSTA,
                "NRSEQORCAMENTO"       : getWidgetProposta().currentRow.NRSEQORCAMENTO
            };
            
            General.callRestEngine("product#/fct_itemorcamento_widget/atualizarValorRateioItens", "Row", params).then()
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    this.servicoOnChange = function(widget) {
        this.getValorUnitarioTblPrecoItem(widget);
        UtilitiesPropostaContrato.setarMedidaVariavelEscopo(widget, "P");
    };

    /**
     * Busca o valor do serviço na tabela de preço do escopo
     * @params{Object} widget - widget de itens (editando)
    **/
    this.getValorUnitarioTblPrecoItem = function(widget) {
        if (widget.currentRow.CDPRODSERV && widget.currentRow.TIPOPRODSERV == "S") {
            var wdgProposta = getWidgetProposta();
            var params = {
                "NRTIPOORCAMENTO"    : widget.currentRow.NRTIPOORCAMENTO,
                "CDSERVORCA"         : widget.currentRow.CDPRODSERV,
                "QTMEDIDASERV"       : widget.currentRow.QTMEDIDAORCA ? widget.currentRow.QTMEDIDAORCA : null,
                "CDTPMEDIDAVARIAVEL" : widget.currentRow.CDTPMEDIDAVARIAVEL ? widget.currentRow.CDTPMEDIDAVARIAVEL : null,
                "NRMOEDA"            : wdgProposta.currentRow.NRMOEDA
            };
            
            General.callRestEngine("product#/fct_itemorcamento_widget/getValorUnitarioTblPrecoItemProposta", "Row", params).then(function(response) {
                var quantidade = widget.currentRow.QTITEMORCAMENTO ? widget.currentRow.QTITEMORCAMENTO : 1;
                var qtdMedida = widget.currentRow.QTMEDIDAORCA ? widget.currentRow.QTMEDIDAORCA : 1;
                if (!response.dataset || !response.dataset.fct_itemorcamento_widget) {
                    this.calcularValoresItemOrcamento(widget);
                }
                else {
                    if (widget.currentRow.VRUNITITEMORCATBLPRECO != response.dataset.fct_itemorcamento_widget.VRPRECO) {
                        widget.currentRow.VRUNITITEMORCA = response.dataset.fct_itemorcamento_widget.VRPRECO;
                    }
                    widget.currentRow.VRUNITITEMORCATBLPRECO = response.dataset.fct_itemorcamento_widget.VRPRECO;
                    widget.currentRow.VRTOTALITEMORCA = widget.currentRow.VRUNITITEMORCA * quantidade * qtdMedida;
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            this.calcularValoresItemOrcamento(widget);
            widget.currentRow.VRUNITITEMORCATBLPRECO = "";
        }
    };
    
    /**
     * Valida o tipo de tratamento que será dado aos acréscimos e descontos 
     * após a alteração dos elementos e medidas
    **/
    this.validarTpTratamentoDescAcres = function(widget, tpAtualizacao) {
        var params = { "NRSEQORCAMENTO" : getWidgetProposta().currentRow.NRSEQORCAMENTO };
        General.callRestEngine("product#/fct_orcamento_widget/validarExistenciaDescontoAcrescimoProposta", "Row", params).then(function(response) {
            if(!response.dataset || !response.dataset.fct_orcamento_widget || !response.dataset.fct_orcamento_widget[0]) {
                if(tpAtualizacao == 'ELEMENTOS') this.atualizarComposicao(widget, 'S');
                else this.atualizarTipoDeMedidaProposta(widget, 'S');
            } else {
                ScreenService.showCustomDialog("A atualização que deseja fazer resultará na redefinição dos valores dos itens e das negociações."
                    + "<br><br>O que deseja fazer com os acréscimos e descontos desta proposta?", 'CONFIRMATION', null, 
                    [
                        {
                            "label": "Descartá-los",
                            "code": 
                                function() {
                                    if(tpAtualizacao == 'ELEMENTOS') this.atualizarComposicao(widget, 'N');
                                    else this.atualizarTipoDeMedidaProposta(widget, 'N');
                                }.bind(this)
                        },
                        {
                            "label": "Mantê-los",
                            "code": 
                                function() {
                                    if(tpAtualizacao == 'ELEMENTOS') this.atualizarComposicao(widget, 'S');
                                    else this.atualizarTipoDeMedidaProposta(widget, 'S');
                                }.bind(this)
                        },
                        {
                            "label": "Cancelar",
                            "code": function() {}
                        }
                    ]
                );
            }
        }.bind(this))
        .catch(function(error) {
            if(tpAtualizacao == 'ELEMENTOS') this.atualizarComposicao(widget, 'N');
            else this.atualizarTipoDeMedidaProposta(widget, 'N');
        });
    };
    
    /**
     * Valida e habilita a edição da tela de negociação de valores
     * @param {Object} widget - widget de negociação de valores
    **/
    this.editarDescontoPorPeriodo = function(widget) {
        this.controlarCamposDescontos(widget);
        Utilities.widgetEdit(widget);
    };
    
    /**
     * controla as fields de desontonto de acordo com o parametro de tipo de desconto
     * @param {Object} widget
    **/
    this.controlarCamposDescontos = function(widget) {
        var tpDesconto = widget.currentRow.IDTPDESCONTO;
        if (!tpDesconto) {
            if (getWidgetNegociacaoDeValores().currentRow.IDTPFATURA != 'X') {
                widget.getField("VRDESCITEM").readOnly     = false;
                widget.getField("VRPERCDESCITEM").readOnly = false;
            }
            else if (widget.getField("IDTPDESCONTO").isVisible) {
                widget.getField("VRDESCITEM").readOnly     = true;
                widget.getField("VRPERCDESCITEM").readOnly = true;

                widget.getField("VRDESCITEM").value(0);
                widget.getField("VRPERCDESCITEM").value(0);
                Utilities.calcularValoresDesconto(widget, "VRTOTITEM", "VRPERCDESCITEM", "VRDESCITEM", "VRTOTITEMCOMDESC", "VRPERCDESCITEM");
            }
            
            widget.getField("VRDESCITEM").validations     = {};
            widget.getField("VRPERCDESCITEM").validations = {};
        }
        else {
            widget.getField("VRDESCITEM").readOnly     = false;
            widget.getField("VRPERCDESCITEM").readOnly = false;
            
            widget.getField("VRDESCITEM").validations     = {"required": true};
            widget.getField("VRPERCDESCITEM").validations = {"required": true};
        }
        
        templateManager.updateTemplate();
    };
    
    /**
     * Coloca a widget de negociacao ou itens da negociação em modo de edição
     * @params{Object} widget - Widget de negociação ou de itens da negociação
    **/
    this.editarNegociacaoValores = function(widget) {
        this.controlarCamposDescontos(widget);
        var wdgNegociacaoValores = getWidgetNegociacaoDeValores();
        var fieldTipoDesconto = widget.getField("IDTPDESCONTO");
        
        if (wdgNegociacaoValores.currentRow.IDTPFATURA != 'X') {
            fieldTipoDesconto.readOnly = true;
            General.widgetEdit(widget);
        }
        else {
            var params = { "NRNEGOCIACAOPROPOSTA": wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA };
            var rota = (widget.name == 'wdgNegociacaoValores') ? "verificarSeExisteDescontoItensNegociacaoProposta" : "verificarSeExisteDescontoNegociacaoProposta";
            
            General.callRestEngine("product#/fct_itemorcamento_widget/" + rota, "Row", params).then(function(success) {
                var existeOutrosDescontos = success.dataset.retorno.existe;
                fieldTipoDesconto.readOnly = existeOutrosDescontos;
                
                return self.validarMedicaoServico(widget.currentRow.NRNEGOCIACAOPROPOSTA, wdgNegociacaoValores.currentRow.NRTIPOORCAMENTO);
            })
            .then(function(medicaoDeServico) {
                if (medicaoDeServico)
                    setDataSourceTipoDesconto(widget, true);
                
                General.widgetEdit(widget);
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    this.validarMedicaoServico = function(nrnegociacaoproposta, nrtipoorcamento) {
        var deferred = ZHPromise.defer();
        var params = {
            "NRNEGOCIACAOPROPOSTA": nrnegociacaoproposta,
            "NRTIPOORCAMENTO"     : nrtipoorcamento
        };
        General.callRestEngine("product#/fct_orcamento_widget/validarMedicaoServico", "Row", params).then(function(response) {
            var medicaoDeServico = response.dataset.retorno.medicaoServico;
            deferred.resolve(medicaoDeServico);
        })
        .catch(function(error) {
            deferred.reject(error);
        });
        
        return deferred.promise;
    };
    
    /**
     * Filtra a field de serviço/produto para trazer apenas os tipos
     * permitidos pelo escopo.
     * 
     * @params{Object} widget - widget de itens manuais da proposta
    **/
    this.filtrarTipoDeItensManuaisDoEscopo = function(widget) {
        var idtipoorca    = widget.currentRow.IDTIPOORCA;
        var fieldServProd = widget.getField("NMITEMORCAMENTO");
        var posFiltroTipo = undefined;

        if(!fieldServProd.dataSourceFilter) fieldServProd.dataSourceFilter = [];
        
        if(!idtipoorca) {
            // if(fieldServProd.dataSourceFilter && fieldServProd.dataSourceFilter.length)
            //     fieldServProd.dataSourceFilter.pop();
            posFiltroTipo = Utilities.getFilterPosition(fieldServProd.dataSourceFilter, "TIPO");
            if (posFiltroTipo >= 0) {
                delete fieldServProd.dataSourceFilter[posFiltroTipo];
            }
        } else {
            idtipoorca = (idtipoorca == 'T') ? ['S', 'V'] : [idtipoorca];

            posFiltroTipo = Utilities.getFilterPosition(fieldServProd.dataSourceFilter, "TIPO");
            if (posFiltroTipo >= 0) {
                fieldServProd.dataSourceFilter[posFiltroTipo].value = idtipoorca;
            }
            else {
                fieldServProd.dataSourceFilter.push({
                    name     : "TIPO",
                    operator : "IN",
                    value    : idtipoorca
                });
            }
        }
    };
    
    this.validarVisibilidadeWidgetTpMedida = function(wdgTpMedida) {
        var wdgProposta = getWidgetPropostaGrid();
        var params = [
            {
                name: "NRSEQORCAMENTO",
                operator: "=",
                value: wdgProposta.currentRow.NRSEQORCAMENTO
            }/*,
            {
                name: "NRTIPOORCAMENTO",
                operator: "IN",
                value: wdgProposta.currentRow.NRTIPOORCAMENTO
            }*/
        ];
        
        General.callRestEngine("product#/crud/fct_relacmedidaproposta_widget", "FilterData", params).then(function(response) {
            if(response.dataset && response.dataset.fct_relacmedidaproposta_widget && response.dataset.fct_relacmedidaproposta_widget.length)
                wdgTpMedida.isVisible = true;
            else 
                wdgTpMedida.isVisible = false;
        });
    };
    
    /**
     * Busca nas parametrizações se o modelo de proposta é obrigatório
    **/
    this.validarObrigatoriedadeModelo = function() {
        var wdgProposta = getWidgetProposta();
        if(idObrigaModProp != null) {
            wdgProposta.getField("NMORCAMENTOMODELO").validations = (!idObrigaModProp) ? {} : { "required": true };
        } else {
            idObrigaModProp = false;
            General.callRestEngine("product#/crud/fct_parametrogeral", "FilterData", []).then(function(response) {
                if(response.dataset && response.dataset.fct_parametrogeral && response.dataset.fct_parametrogeral.length > 0) {
                    var parametrosService = response.dataset.fct_parametrogeral;
                    idObrigaModProp = (parametrosService.IDOBRIGMODORC == 'S');
                    wdgProposta.getField("NMORCAMENTOMODELO").validations = (idObrigaModProp == "N") ? {} : { "required": true };
                }
            }.bind(this));
        }
    };
    
    this.updateDsAlteracoesContr = function(widget) {
        
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
        
        var params = { 
            "NRSEQORCAMENTO"      : widget.currentRow.NRSEQORCAMENTO,
            "DSALTERACONTRATUAIS" : widget.currentRow.DSALTERACONTRATUAIS
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/updateDsAlteracoesContr", "Row", params).then(function(response) {
            
            ScreenService.showMessage("Atenção! <br><br> A alteração realizada será considerada para todos os escopos. Caso queira definir Alterações Contratuais específicas para cada escopo, favor alterar na aba 'Escopos de Faturamento'.");
            General.widgetCancelForm(widget);
            widget.view();
            widget.reload();
                
        });
    };
    
    this.getAlteracaoContratual = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
         
        widget.currentRow.NRSEQORCAMENTO = wdgProposta.currentRow.NRSEQORCAMENTO;
            
        var params = { 
            "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO
        };
    
        General.callRestEngine("product#/fct_orcamento_widget/getAlteracaoContratual", "Row", params).then(function(response) {
            
            widget.currentRow.DSALTERACONTRATUAIS = response.dataset.getAlteracaoContratual.resultado;
            
        }); 
    };

    /**
     * Valida se o operador tem acesso à página "Alterações Contratuais"
    **/
    this.validaAlteracoesContratuais = function(widget){
        
        var wdgAlteracoesContratuais = templateManager.container.getWidget("wdgAlteracoesContratuais");
        
        var params = {};
        
        General.callRestEngine("product#/fct_orcamento_widget/validaAlteracoesContratuais", "Row", params).then(function(response) {
            if (response.dataset.validaAlteracoesContratuais.resultado == 'HABILITADO') {
            
                wdgAlteracoesContratuais.isVisible = true;
            }
            else{
                wdgAlteracoesContratuais.isVisible = false; 
            }
        });
    };
    
    /**
     * insere as assinaturas da proposta 
    **/
    this.saveAssinaturaOrca = function(widget) {
        if (widget.currentRow.__is_new){
            
            var wdgProposta = templateManager.container.getWidget("wdgProposta");
            
            General.callRestEngine('product#/fct_orcamento_widget/validaAssinaturaOrca', "Row", widget.currentRow).then(function(response) {
                
                var retorno = response.dataset.validaAssinaturaOrca.valida;
                
                /* Recebe os envolvidos para notificação*/
                var envolvidos = null;
                var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
                
                if (arrayEnvolvidos != null && arrayEnvolvidos != ""){
                    envolvidos = arrayEnvolvidos.join('_');
                }

                widget.currentRow.DSEMAILNOTSIGNENV = envolvidos; 
                
                /* Recebe os operdores para notificação */
                var arrayOperadores = widget.currentRow.OPERADORES;
                
                if(retorno == '0') {
                            
                    var params = {
                        "DADOS"           : widget.currentRow,
                        "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO,
                    };
                    
                    General.callRestEngine('product#/fct_orcamento_widget/inserirSignatario', "Row", params).then(function(response) {
            
                        var resultado = response.dataset.inserirSignatario.resultado;
                        
                        if (resultado['STATUS'] == 'OK') {
                            
                            if (Array.isArray(arrayOperadores) && arrayOperadores.length > 0) {
                                var params = {
                                    "NRASSDIGITALORCA" : resultado['NRASSDIGITALORCA'],
                                    "OPERADORES"       : arrayOperadores
                                };
            
                                General.callRestEngine('product#/fct_orcamento_widget/insereOperadoresNotifica', "Row", params).then(function() {
                                    widget.widgets.forEach(function(wid) {
                                        wid.enabled = true;
                                    });
                                    ScreenService.closeSwipe();
                                    widget.reload();
                                })
                                .catch(function(error) {
                                    Utilities.showErrorMessage(error);
                                });
                            }
                            else {
                                ScreenService.closeSwipe();
                                widget.reload();
                            }
                        }
                        else {
                            ScreenService.showMessage(resultado);   
                        }
                        
                    }, function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                else{
                    ScreenService.showMessage("Operação Bloqueada! <br><br> O signatário informado já foi inserido.");
                }
            });
        }
        else{
            
            /* Recebe os envolvidos para notificação*/
            var envolvidos = null;
            var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
            
            if (arrayEnvolvidos != null){
                envolvidos = arrayEnvolvidos.join('_');
            }
            
            widget.currentRow.DSEMAILNOTSIGNENV = envolvidos;
            
            /* Recebe os operdores para notificação */
            var operadoresIguais = false;
            
            var bkpOperadores = widget.backupRow.OPERADORES;
            var arrayOperadores = widget.currentRow.OPERADORES;
            
            if (Utilities.arraysSaoIguais(bkpOperadores, arrayOperadores)) {
                operadoresIguais = true;
            }
            
            General.widgetSave(widget).then(function(response){
                
                if (operadoresIguais == false) {
                    var params = {
                        "NRASSDIGITALORCA" : widget.currentRow.NRASSDIGITALORCA,
                        "OPERADORES"       : arrayOperadores
                    };

                    General.callRestEngine('product#/fct_orcamento_widget/insereOperadoresNotifica', "Row", params).then(function() {
                        widget.reload().then(function() {
                            this.carregaDadosOperadores(widget);
                        }.bind(this));
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
                else {
                    widget.reload();
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * insere as assinaturas da proposta através do Modelo de Proposta
    **/
    this.inserirAssinaturasOrca = function(widget){
        
        General.callRestEngine("product#/fct_orcamento_widget/inserirAssinaturasOrca", "Row", widget.currentRow).then(function() {

        }, function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * Valida se o token da certsign está preenchido
     * Tela: Lançamento de Proposta
     */
    this.validaIdCertsign = function() {
        var wdgAssinaturasOrca = templateManager.container.getWidget("assinaturasOrca");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
        
        var params = {
            "NRSEQORCAMENTO" : wdgPropostaGrid.currentRow.NRSEQORCAMENTO
        };
        General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {
            
            var idTokenCertsign   = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
            var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;
            var iddocumcertsign   = response.dataset.validaIdCertsign.IDDOCUMCERTSIGN;
            
            
            //OTIMIZACAO : Diminuir o numero de acessos ao mesmo campo 
            var campoAssinatura = wdgProposta.getField('IDASSELETRDIGITAL');
            
            if (idTokenCertsign == null){
                
                /* Bloqueia a página "Signatários Padrões" */
                wdgAssinaturasOrca.enabled = false;
                
                /* Bloqueia o campo "Utiliza Assinatura Eletrônica ou Digital" */
                campoAssinatura.readOnly = true;
                campoAssinatura.validations = {};
                if (wdgProposta.currentRow.changed == true){
                    campoAssinatura.value('N');
                }
                this.contolaActionAssEletrDigital(wdgProposta, wdgPropostaGrid,  false, iddocumcertsign);
            }
            else{
                if (idAssEletrDigital == 'N'){
                    /* Bloqueia a página "Signatários Padrões" */
                    wdgAssinaturasOrca.enabled = false;
                    this.contolaActionAssEletrDigital(wdgProposta, wdgPropostaGrid, false, iddocumcertsign);
                }
                else {
                    /* Libera a página "Signatários Padrões" */
                    if (wdgProposta.currentRow.changed != true){
                        wdgAssinaturasOrca.enabled = true;
                        this.contolaActionAssEletrDigital(wdgProposta, wdgPropostaGrid, true, iddocumcertsign);
                    }
                }

                /* Libera o campo "Utiliza Assinatura Eletrônica ou Digital" */
                campoAssinatura.readOnly = false;
                campoAssinatura.validations = {"required" : true};    
            }
        }.bind(this));
    };

    /**
     * Habilita o campo Revenda, caso o campo 'Utiliza Revenda?' da tela de Modelo de Proposta esteja preenchida com 'Sim'". 
     * @param {object} widget widget de 
     * Laçamento de Proposta
    */
    
    this.habilitaCampoRevenda = function(){

        var wdgPropostaForm = templateManager.container.getWidget("wdgProposta");
        
        if(wdgPropostaForm.currentRow.IDUTILIZAREVENDA == 'N'){
            wdgPropostaForm.getField("NMREVENDA").readOnly = true;
        }else {
            wdgPropostaForm.getField("NMREVENDA").readOnly = false; 
        }
       
    };
    
    this.mudaTipoSignatario = function(wdgInterlocutor) {
        if (wdgInterlocutor === undefined) {
            wdgInterlocutor = null;
        }

        if(!wdgInterlocutor)
            wdgInterlocutor = templateManager.container.getWidget("wdgNewInterlocutor");
            
        var tipoSignatario = wdgInterlocutor.currentRow.IDTPSIGNATARIO;
        var obrigatorio = tipoSignatario != 'N';
        wdgInterlocutor.getField("NRCPFINTERLOC").validations = (obrigatorio)?{"required":true}:{}; 
        wdgInterlocutor.getField("DTNASCIMENTO").validations =  (obrigatorio)?{"required":true}:{}; 
        wdgInterlocutor.getField("EMAILINTERLOC").validations =  (obrigatorio)?{"required":true}:{}; 
    };
    
    /**
     * Envia novamente o e-mail de assinatura para os signatários que ainda não realziaram a assinatura
     */
    this.solicitarAssinatura = function() {
        
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        var params = {
            "NRSEQORCAMENTO"  : wdgProposta.currentRow.NRSEQORCAMENTO,
            "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO,
        };
        
        General.callRestEngine('product#/fct_orcamento_widget/solicitarAssinatura', "Row", params).then(function(response) {  
            
            var resultado = response.dataset.solicitarAssinatura.resultado;
            
            if (resultado == 'OK'){
                
                ScreenService.showMessage("Lembrete de assinatura enviado com sucesso!");
            }
            else if (resultado == 'OK_N_DOC'){
                
                ScreenService.showMessage("Lembrete de assinatura enviado com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
            }
            else if (resultado == 'N_DOC'){
                
                ScreenService.showMessage("Operação bloqueada! <br><br> Não foram enviados os documentos para assinatura eletrônica / digital.");
            }
            else{

                ScreenService.showMessage("Ocorreram erros no lembrete de assinatura. <br><br>Favor entrar em contato com o administrador do sistema. <br/>Erro: ".resultado);  
            }
        });
    };
    
    
    /**
     * Exclui o signatário do Certsign e altera a situação para 'Assinado Manualmente'.
     */
    this.assinadoManualmente = function(widget, checked) {
        
        ScreenService.confirmMessage("Atenção!<br><br>Ao confirmar a assinatura manual, todos os documentos vinculados à este signatário serão atualizados para a situação 'Assinado Manualmente'. <br><br>Caso não deseje assinar manualmente todos os documentos, a rotina deve ser realizada individualmente para cada documento na aba 'Documentos'.<br><br>Deseja continuar?", "question", function() {

            var signatario = null;
            var situacao = null;
            
            if (checked == true) {    
                signatario = widget.getCheckedRows();
            }
            else{
                signatario = widget.currentRow;
                situacao = widget.currentRow.IDSITUACAO;
            }
            
            if (((checked  == false) && (situacao == 'P')) || (checked  == true)) {
                
                var params = {
                    "SIGNATARIO" : signatario,
                    "CHECKED" : checked
                };
                
                General.callRestEngine('product#/fct_orcamento_widget/assinadoManualmente', "Row", params).then(function(response) {
                    
                    var resultado = response.dataset.assinadoManualmente.resultado;
                    
                    if (resultado == 'OK') {
                        
                        if (checked == true) {
                            ScreenService.closePopup();
                            ScreenService.showMessage("Situação dos signatários atualizada com sucesso!");
                            widget.dataSource.clearCheckedRows();
                            widget.reload();
                        }
                        else {
                            ScreenService.showMessage("Situação do signatário atualizada com sucesso!");
                            ScreenService.closeSwipe().then(function() {
                                widget.reload();
                            });
                        }
                    }
                    else if (resultado == 'OK_N_DOC'){
                        if (checked == true) {
                            ScreenService.closePopup();
                            ScreenService.showMessage("Situação dos signatários atualizada com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
                            widget.dataSource.clearCheckedRows();
                            widget.reload();
                        }
                        else {
                            ScreenService.showMessage("Situação do signatário atualizada com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
                            ScreenService.closeSwipe().then(function() {
                                widget.reload();
                            });
                        }
                    }
                    else if (resultado == 'N_DOC'){
                        
                        ScreenService.showMessage("Operação bloqueada! <br><br> Não foram enviados os documentos para assinatura eletrônica / digital.");
                    }
                    else{
                        ScreenService.closePopup();
                        ScreenService.showMessage("Ocorreram erros na atualização da situação dos signatários. Favor entrar em contato com o administrador do sistema. <br/>Erro: " . resultado);
                    }
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                
                ScreenService.showMessage("Operação Bloqueada! <br><br> Só é permitida a assinatura manual para signatários que estejam com a situação 'Pendente'."); 
            }
        }.bind(this));
    };
    
    /**
     * Exclui o signatário do Certsign e altera a situação para 'Assinado Manualmente'.
     */
    this.assinadoManualmenteDoc = function(widget, checked) {
        
        var signatario = null;
        var situacao = null;
        
        if (checked == true) {    
            signatario = widget.getCheckedRows();
        }
        else{
            signatario = widget.currentRow;
            situacao = widget.currentRow.IDSITUACAO;
        }
        
        if (((checked  == false) && (situacao == 'P')) || (checked  == true)) {
            
            var params = {
                "SIGNATARIO" : signatario,
                "CHECKED" : checked
            };
            
            General.callRestEngine('product#/fct_orcamento_widget/assinadoManualmenteDoc', "Row", params).then(function(response) {
                
                var resultado = response.dataset.assinadoManualmenteDoc.resultado;
                
                if (resultado == 'OK') {
                    
                    if (checked == true) {
                        ScreenService.closePopup();
                        ScreenService.showMessage("Situação dos signatários atualizada com sucesso!");
                        widget.dataSource.clearCheckedRows();
                        widget.reload();
                    }
                    else {
                        ScreenService.showMessage("Situação do signatário atualizada com sucesso!");
                        ScreenService.closeSwipe().then(function() {
                            widget.reload();
                        });
                    }
                }
                else if (resultado == 'OK_N_DOC'){
                    if (checked == true) {
                        ScreenService.closePopup();
                        ScreenService.showMessage("Situação dos signatários atualizada com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
                        widget.dataSource.clearCheckedRows();
                        widget.reload();
                    }
                    else {
                        ScreenService.showMessage("Situação do signatário atualizada com sucesso! <br><br> Obs: Alguns escopos de faturamento ainda não tiveram os seus documentos enviados para assinatura eletrônica/digital. Estes escopos foram desconsiderados.");
                        ScreenService.closeSwipe().then(function() {
                            widget.reload();
                        });
                    }
                }
                else if (resultado == 'N_DOC'){
                    
                    ScreenService.showMessage("Operação bloqueada! <br><br> Não foram enviados os documentos para assinatura eletrônica / digital.");
                }
                else{
                    ScreenService.closePopup();
                    ScreenService.showMessage("Ocorreram erros na atualização da situação dos signatários. Favor entrar em contato com o administrador do sistema. <br/>Erro: " . resultado);
                }
            });
        }
        else {
            
            ScreenService.showMessage("Operação Bloqueada! <br><br> Só é permitida a assinatura manual para signatários que estejam com a situação 'Pendente'."); 
        }
    };
    
    /**
     * exclui os signatários da proposta e do ambiente Certsign 
    **/
    this.excluirSignatariosCertsign = function(widget){
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        var nrseqorcamento   = widget.currentRow.NRSEQORCAMENTO;
        var idsignatcertsign = widget.currentRow.IDSIGNATCERTSIGN;
        var nrcpfsignatario  = widget.currentRow.NRCPFSIGNATARIO;
        var nrassdigitalorca = widget.currentRow.NRASSDIGITALORCA;
        
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
            var params         = {
                    "NRSEQORCAMENTO"   : nrseqorcamento,
                    "NRASSDIGITALORCA" : nrassdigitalorca,
                    "NRTIPOORCAMENTO"  : wdgProposta.currentRow.NRTIPOORCAMENTO,
                    "IDSIGNATCERTSIGN" : idsignatcertsign,
                    "NRCPFSIGNATARIO"  : nrcpfsignatario
                };
                
            General.callRestEngine("product#/fct_orcamento_widget/excluirSignatariosCertsign", "Row", params).then(function(response) {
                
                var resultado = response.dataset.excluirSignatariosCertsign.resultado;
                
                if (resultado == 'OK') {
                    widget.removeRow().then(function () {
                        
                        ScreenService.closeSwipe().then(function () {
        					widget.activate();
                            widget.reload();
        				});
                    },
                    function (error) {
        				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
        					ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
        				} else {
        					ScreenService.showMessage(error.data.error);
        				}
        			});
                } else {
                    
                    ScreenService.showMessage(resultado);
                }
            },
            function(error) {
                console.error(error);
                ScreenService.showMessage(error.data.error);
            });
        });
    };
    
    this.wdgAssinaturasOnEnterTab = function(widget) {
        Utilities.carregarWidgetOnEnterTab(widget);
        this.filtraParcnegocioAssinat();
        this.controlaEdicaoAssinat(widget);
        this.controlaAcoesAssinat(widget);
    };
    
    this.wdgAssinaturasAfterMoveRow = function(widget) {
        this.controlaAcoesAssinat(widget);
        this.carregaDadosOperadores(widget);
    };
    
    this.wdgAssinaturasVoltarView = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.currentRow.changed = false;
        });
    };
    
    this.wdgAssinaturasCancelar = function(widget) {
        var isNew = widget.currentRow.__is_new;
        General.widgetCancelForm(widget);
        if (!isNew)
            widget.currentRow.changed = false;
            
        this.carregaDadosOperadores(widget);
    };
    
    /**
     * filtra parcnegocio para datasouce do signatário
    **/
    this.filtraParcnegocioAssinat = function(){
        
        var wdgAssinaturasOrca = templateManager.container.getWidget("assinaturasOrca");
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
   
        wdgAssinaturasOrca.getField("NMPRINCIPALPARC").dataSourceFilter[0].value = wdgProposta.currentRow.NRPARCNEGOCIO;
    };
    
    /**
     * A função "contolaActionAssEletrDigital" controla a visibilidade da action de 
     * geração de documento para a assinatura eletrônica e digital.
     * Tela: Lançamento de Proposta
    **/
    this.contolaActionAssEletrDigital = function(wdgProposta, wdgPropostaGrid, visible, iddocumcertsign){
        var situacao = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;

        var assEletrDigital     = wdgProposta.getAction('assinatura_eletronica_digital');
        var downDocEletrDigital = wdgProposta.getAction('download_Doc_Eletr_Digital');
        var actHabAssEletrDig   = wdgProposta.getAction("habilitar_assinatura_eletronica_digital");
        
        if (situacao == 'CONCLUIDO') {
            if (iddocumcertsign == false) {
                downDocEletrDigital.isVisible = false;
                assEletrDigital.isVisible = visible;
                actHabAssEletrDig.isVisible  = true;
            }
            else {
                downDocEletrDigital.isVisible = visible;
                assEletrDigital.isVisible = false;
                actHabAssEletrDig.isVisible = false;
            }
        }
        else if (situacao == 'ELABORACAO' || situacao == 'CANCELADO') {
            downDocEletrDigital.isVisible = false;
            assEletrDigital.isVisible = false;
            actHabAssEletrDig.isVisible  = false;
        }
        /* ACEITO */
        else {
            downDocEletrDigital.isVisible = visible;
            assEletrDigital.isVisible = false;
            actHabAssEletrDig.isVisible = false;
        } 
    };
    
    /**
     * realiza o download do documento do ambiente Certsign 
    **/
    this.downloadDocEletrDigital = function(widget){
        
        var nrseqorcamento  = widget.currentRow.NRSEQORCAMENTO;
        var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
        nrtipoorcamento.forEach(function(codEscopo) {
            window.open(window.backendIndexUrl + "/fct_orcamento_widget/downloadDocEletrDigital?requestType=DataSet&dataset[0][NRSEQORCAMENTO]=" + nrseqorcamento + "&dataset[0][NRTIPOORCAMENTO]=" + codEscopo);
        });
    };
    
    this.controlaEdicaoAssinat = function(widget){
        
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
        
        var situacao = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;
        
        var edit = widget.getAction('EDIT');
        var actAssinaManual = widget.getAction('assinadoManualmente');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteChk');
        var actSolicitaAss = widget.getAction('solicitarAssinatura'); 
        var actAlteraSituAss = widget.getAction('AlteraSituAss');
        var actAlteraSituAssChk = widget.getAction('AlteraSituAssChk');
        
        if (situacao == 'ELABORACAO') {
        
            edit.isVisible = true;
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
        else if ((situacao == 'CONCLUIDO') || (situacao == 'ACEITO')){

            edit.isVisible = false;
            actAssinaManual.isVisible = true;
            actAssinaManualChk.isVisible = true;
            actSolicitaAss.isVisible = true;
            actAlteraSituAss.isVisible = true;
            actAlteraSituAssChk.isVisible = true;
        } 
        else /*CANCELADO*/ {
            edit.isVisible = false;
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
        if(!edit.isVisible && situacao !== "CONCLUIDO" && situacao !== "CANCELADO"){
            var params = {
                "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO
            };
            General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {  
                edit.isVisible = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
                
            });
        }    
    };
    
    this.controlaAcoesAssinat = function(widget){
        var actAssinaManual = widget.getAction('assinadoManualmente');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteChk');
        var actSolicitaAss = widget.getAction('solicitarAssinatura'); 
        var actAlteraSituAss = widget.getAction('AlteraSituAss');
        var actAlteraSituAssChk = widget.getAction('AlteraSituAssChk');
        
        var situacao = widget.currentRow.IDSITUACAO;
        var excluir = widget.getAction('EXCLUIR');
        
        if (situacao == 'P') {
            this.controlaEdicaoAssinat(widget);
            excluir.isVisible = true;
        }
        else {
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = true;
            excluir.isVisible = false;
            actAlteraSituAss.isVisible = false;
            actAlteraSituAssChk.isVisible = false;
        }
    };
    
    
     this.controlaEdicaoAssinatDoc = function(widget){
        
        var wdgPropostaGrid = templateManager.container.getWidget("wdgPropostaGrid");
        
        var situacao = wdgPropostaGrid.currentRow.IDSITUACAOORCAMENTO;
        
        var actAssinaManual = widget.getAction('assinadoManualmenteDoc');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteDocChk');
        var actSolicitaAss = widget.getAction('solicitarAssinaturaDoc'); 
        
        if (situacao == 'ELABORACAO') {
        
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
        }
        else if ((situacao == 'CONCLUIDO') || (situacao == 'ACEITO')){

            actAssinaManual.isVisible = true;
            actAssinaManualChk.isVisible = true;
            actSolicitaAss.isVisible = true;
        } 
        else /*CANCELADO*/ {
            actAssinaManual.isVisible = false;
            actAssinaManualChk.isVisible = false;
            actSolicitaAss.isVisible = false;
        }
        
    };
    
    this.controlaAcoesAssinatDoc = function(widget){
        var actAssinaManual = widget.getAction('assinadoManualmenteDoc');
        var actSolicitaAss = widget.getAction('solicitarAssinaturaDoc');
        var actAssinaManualChk = widget.getAction('assinadoManualmenteDocChk');
        var situacao = widget.currentRow.IDSITUACAO;
        
        if (situacao == 'P') {
            this.controlaEdicaoAssinatDoc(widget);
        }
        else {
            actAssinaManual.isVisible = false;
            actSolicitaAss.isVisible = false;
            actAssinaManualChk.isVisible = false;
        }
    };
    
    /**
     * A função "assinaturaEletrDigital" realiza a geração do documento de contrato no Certsign
    **/
    this.assinaturaEletrDigital = function(widget){
        
       var params = {
            "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO
       };
           
       General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {  
            var idTokenCertsign = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
            var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;
            
            if ((idTokenCertsign != false ) && (idAssEletrDigital == 'S')){
                
                var wdgProposta = templateManager.container.getWidget("wdgProposta");
                
                var params = {
                    "NRSEQORCAMENTO"     : widget.currentRow.NRSEQORCAMENTO,
                    "NRTIPOORCAMENTO"    : wdgProposta.currentRow.NRTIPOORCAMENTO,
                    "IDSITUACAOORCAMENTO": wdgProposta.currentRow.IDSITUACAOORCAMENTO,
                    "NRORCAMENTO"        : wdgProposta.currentRow.NRORCAMENTO
                };
                
                var timeout = 5 * 60000; //5 min
                Utilities.callRestEngineWithTimeout("product#/fct_orcamento_widget/gerarDocumentoCertsign", "Row", params, timeout)
                .then(function(response) {
                    
                    var resultado = response.dataset.gerarDocumentoCertsign.resultado;
                    
                    if (resultado == 'OK') {
                        ScreenService.closePopup();
                        ScreenService.showMessage('Documentos enviados para assinatura eletrônica e digital com sucesso!');
                    } 
                    else {
                       
                       ScreenService.showMessage(resultado);
                    }
                this.validaIdCertsign();
                }.bind(this), function(error) {
                    Utilities.showErrorMessage(error);
                }.bind(this));
            }
       }.bind(this));
    };
    
    this.onEnterTabParcelasNegociacao = function(widget){
        
        var wdgNegociacaoValores = templateManager.container.getWidget("wdgNegociacaoValores"); 
        
        widget.dataSourceFilter[0].value = wdgNegociacaoValores.currentRow.NRNEGOCIACAOPROPOSTA;
        
        Utilities.carregarWidgetOnEnterTab(widget);
        
    };
    
    this.deleteParcelasNegociacao = function(widget){
        
        General.deleteCrud(widget).then(function(){
            
            var params = {
                "NRNEGOCIACAOPROPOSTA" : widget.currentRow.NRNEGOCIACAOPROPOSTA
            };
            
            General.callRestEngine("product#/fct_orcamento_widget/atualizarNrParcelas", "Row", params).then(function(){
                
                widget.reload();
            }).catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        });
   
    };
    
    this.habilitarAssinaturaEletrDigital = function(widget){
        
        var idasseletrdigital = widget.currentRow.IDASSELETRDIGITAL;
        
        General.callRestEngine("product#/fct_orcamento_widget/habilitarAssinaturaEletrDigital", "Row", widget.currentRow).then(function(response){
                
            var resultado = response.dataset.habilitaAssinatura.resultado;
                    
            if (resultado == 'OK') {
                ScreenService.closePopup();
                widget.reload().then(function() {
                    this.validaIdCertsign();
                }.bind(this));
                
                if (idasseletrdigital == 'N'){
                    ScreenService.showMessage('A proposta foi habilitada para assinatura eletrônica e digital.');
                }
                else {
                    ScreenService.showMessage('A proposta foi desabilitada para assinatura eletrônica e digital.');
                }
            } 
            else {
               
               ScreenService.showMessage(resultado);
            }
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
        
    };
    
    
    this.carregaFiltrosServico = function(widget){
        
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        var params = {
            "NRTIPOORCAMENTO" : wdgProposta.currentRow.NRTIPOORCAMENTO
        };
        
        General.callRestEngine("product#/fct_orcamento_widget/validaFiltroServico", "Row", params).then(function(response){
                
            var resultado = response.dataset.validaFiltroServico.resultado;
            
            if (widget.name == 'itemFormacaoPreco') {
                
                widget.getField("NRFORMPRECOSERVICO").dataSourceFilter[1].value = resultado;
                widget.getField("NRFORMPRECOSERVICO").dataSourceFilter[2].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
                
                widget.getField("NMSERVORCA").dataSourceFilter[0].value = resultado;
                widget.getField("NMSERVORCA").dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
                
            }
            else {
            
                widget.getField("NMITEMORCAMENTO").dataSourceFilter[0].value = resultado;
                widget.getField("NMITEMORCAMENTO").dataSourceFilter[1].value = wdgProposta.currentRow.NRTIPOORCAMENTO;
            }
                    
            
        }.bind(this))
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
        
    };
    
    this.controlaActionEnviarEmail = function(widget){
      
        var actEnviarPorEmail = widget.getAction("enviar_documento");
        
        var params = {
            "NRSEQORCAMENTO" : widget.currentRow.NRSEQORCAMENTO
        };
           
        General.callRestEngine('product#/fct_orcamento_widget/validaIdCertsign', "Row", params).then(function(response) {  
            var idTokenCertsign = response.dataset.validaIdCertsign.DSTOKENCERTSIGN;
            var idAssEletrDigital = response.dataset.validaIdCertsign.IDASSELETRDIGITAL;
            
            if ((idTokenCertsign != false ) && (idAssEletrDigital == 'S')){
                
                General.callRestEngine('product#/fct_orcamento_widget/validaStatusAssinaturas', "Row", params).then(function(response) { 
                    
                    var valida = response.dataset.validaStatusAssinaturas.resultado;
                    
                    if (valida == 'N_Pendente') {
                        
                        actEnviarPorEmail.isVisible = false;
                        
                    }
                    else {
                        
                        actEnviarPorEmail.isVisible = true; 
                    }
                })
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
            else {
                
                actEnviarPorEmail.isVisible = true;
            }
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    

    this.recebeOrdenacaoENotificacoes = function(widget){
      
        var wdgProposta = templateManager.container.getWidget("wdgProposta");
        
        var orcamentoModelo = wdgProposta.currentRow.NRORCAMENTOMODELO;
        var nrseqinterlocutor = widget.currentRow.NRSEQINTERLOCUTOR;
        var idtestemunha = widget.currentRow.IDTESTEMUNHA;
        var nomeSignatario = widget.currentRow.NMPRINCIPALPARC;
        var tipoSignat = null;
        
        if (!Util.isEmptyOrBlank(nomeSignatario)) {
            
            if (nrseqinterlocutor != null) {
            
                if (idtestemunha == 'S'){
                    tipoSignat = 'TC';  
                }
                else {
                    tipoSignat = 'SC';  
                }
            }
            else {
             
                if (idtestemunha == 'S'){
                    tipoSignat = 'TP';  
                }
                else {
                    tipoSignat = 'SP';  
                }   
            }
          
            var params = {
                "NRORCAMENTOMODELO" : orcamentoModelo,
                "IDTPSIGNATARIO" : tipoSignat
            };
               
            General.callRestEngine('product#/fct_orcamento_widget/recebeOrdenacaoENotificacoes', "Row", params).then(function(response) {
                var dados = response.dataset.recebeOrdenacaoENotificacoes.resultado;
                
                if (dados['DADOS'] == 'OK'){
                    widget.getField("NRORDENACAO").readOnly = true;
                    widget.currentRow.NRORDENACAO = dados['NRORDENACAO'];
                    
                    var dsemailnotsignenvAux = dados['DSEMAILNOTSIGNENV'];
                    var dsemailnotsignenv = null;
                    if (!Util.isEmptyOrBlank(dsemailnotsignenvAux)) {
                        dsemailnotsignenv = dsemailnotsignenvAux.split('_');
                    }
                    
                    widget.currentRow.DSEMAILNOTSIGNENV_INS = dsemailnotsignenv;
                    widget.currentRow.DSEMAILNOTSIGNENV     = dsemailnotsignenv;
                    widget.currentRow.OPERADORES            = dados['OPERADORES'];
                    widget.currentRow.CDOPERADOR            = dados['OPERADORES'];
                    widget.currentRow.DSEMAILNOTIFOUTROS    = dados['DSEMAILNOTIFOUTROS'];
                }
                else {
                    widget.getField("NRORDENACAO").readOnly = false;
                    widget.currentRow.NRORDENACAO           = 1;
                    widget.currentRow.DSEMAILNOTSIGNENV_INS = null;
                    widget.currentRow.DSEMAILNOTSIGNENV     = null;
                    widget.currentRow.OPERADORES            = null;
                    widget.currentRow.CDOPERADOR            = null;
                    widget.currentRow.DSEMAILNOTIFOUTROS    = null;
                }
                
                this.carregaDadosOperadores(widget);
    
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    this.carregaDadosOperadores = function(widget){
       var operadores = widget.currentRow.OPERADORES;
       if (operadores && operadores.length > 0) {
            widget.getField("OPERADORES").dataSourceFilter[2] = {
                "name": "CDOPERADOR",
                "operator": "IN",
                "value": operadores
            };
                                                                
            widget.getField("OPERADORES").reload();
            widget.getField("OPERADORES").dataSourceFilter.splice(2);
            templateManager.updateTemplate();
        }
    };
    
    this.editarSignatario = function(widget) {
        General.widgetEdit(widget);
        widget.getField("NRORDENACAO").readOnly = true;
    };
    
    this.assinaturaEletronicaOnChange = function(widget) {
        var idasseletrdigital = widget.currentRow.IDASSELETRDIGITAL;
        var fieldEnviaDocOperResp = widget.getField("IDENVIADOCOPERRESPCONCASS");
        if (idasseletrdigital == "N") {
            fieldEnviaDocOperResp.readOnly = true;
            fieldEnviaDocOperResp.value("N");
        }
        else {
            fieldEnviaDocOperResp.readOnly = false;
        }
    };
    
    this.calcularValorUnitario = function(widget){
        var row = widget.currentRow;
        row.VRUNITITEMORCA = this.pegarValorUnitario(row);
        
        this.validarPermissaoAlteracaoValorUnitario(widget).then(null).catch(function() {
            widget.name == 'wdgItensProposta' ? this.calcularValoresItemOrcamento(widget) : this.calcularValorTotalItem(widget);
        }.bind(this));
    };
    
    this.pegarValorUnitario = function(row){
        var valor = Utilities.tirarMascaraValores(row.VRTOTALITEMORCA) / parseFloat(row.QTITEMORCAMENTO);
        if (row.IDMULTMEDVRTOT == "S")
            valor = valor / parseFloat(row.QTMEDIDAORCA);
        
        return parseFloat(valor.toFixed(8));
    };
    
    this.abrirPopupAlteraSituAss = function(widget, check) {
        ScreenService.closePopup();
        var wdgPopup = templateManager.container.getWidget("WdgPopupAlteraSituAssina");
        wdgPopup.isVisible = true;
        wdgPopup.dataSource.data = [];
        wdgPopup.currentRow = {};
        ScreenService.openPopup(wdgPopup).then(function() {
            wdgPopup.activate();
            wdgPopup.newRow();
            wdgPopup.currentRow.IDALTERAEMMASSA = check;
            
            if (check == true){
            
                assinaturas = [];
                rows = widget.getCheckedRows();
                
                rows.forEach(function(row) {
                   
                    assinaturas.push(row['NRASSDIGITALORCA']) ;
                });

            }
            else {
                assinaturas = widget.currentRow.NRASSDIGITALORCA;
                
            }
            
            wdgPopup.getField("DOCUMENTOS").dataSourceFilter[1] = {
                                                                        "name": "NRASSDIGITALORCA",
                                                                        "operator": "IN",
                                                                        "value": assinaturas
                                                                    };
            wdgPopup.getField("DOCUMENTOS").reload();
                
        }.bind(this));
    };
    
    this.validaSituAlteraAss = function(wdgPopup){
        
        if (wdgPopup.currentRow.IDSITUACAO == 'R') {
            
            wdgPopup.getField("DSJUSTIFICARECUSA").readOnly = false;
            wdgPopup.getField("DSJUSTIFICARECUSA").validations = { "required": true };
        }
        else {
         
            wdgPopup.getField("DSJUSTIFICARECUSA").readOnly = true;
            wdgPopup.getField("DSJUSTIFICARECUSA").validations = {};   
        }
        wdgPopup.reload();
    }
    
    this.salvarAlteraSituAssina = function(widget){
        
        if (widget.isValid()) {
            
            var widAssina = templateManager.container.getWidget('assinaturasOrca');
            var idalteraemmassa = widget.currentRow.IDALTERAEMMASSA;
            
            if (idalteraemmassa == true){
            
                var params = {
                    "WIDASSINATURA" : widAssina.getCheckedRows(),
                    "WIDSITUACAO" : widget.currentRow
                };
                    
            }
            else {
                
                var params = {
                    "WIDASSINATURA" : widAssina.currentRow,
                    "WIDSITUACAO" : widget.currentRow
                };
                
            }
            
            General.callRestEngine("product#/fct_orcamento_widget/salvarAlteraSituAssina", "Row", params).then(function(response){
                    
                var resultado = response.dataset.salvarAlteraSituAssina.resultado;
                        
                if (resultado == 'OK') {
                    ScreenService.closePopup();
                    widAssina.activate().then(function() {
                        widAssina.reload().then(function() {
                            this.controlaAcoesAssinat(widAssina);
                        }.bind(this));
                        if (idalteraemmassa == true){
                            ScreenService.showMessage('Situação da(s) assinatura(s) alterada com sucesso.');
                        }
                        else {
                            ScreenService.showMessage('Situação da assinatura alterada com sucesso.');
                        }
                    }.bind(this));
                    
                } 
                else {
                   Utilities.showErrorMessage(resultado);
                }
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        
    };

    this.moedaOnChange = function(widget) {
        if ((!widget.currentRow.__is_new && !mudouMoeda) &&
            (widget.backupRow && widget.backupRow.NRMOEDA != widget.currentRow.NRMOEDA) ) {
            mudouMoeda = true;
            ScreenService.showMessage(ScreenService.i18n("Atenção! <br><br>Mudar a moeda de uma proposta não fará com que o valor dos itens e dos descontos sejam recalculados automaticamente. Será necessário ajustar o valor dos itens manualmente para a nova moeda."));
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001000_LancamentoProposta_Controller", Sm00001000_LancamentoProposta_Controller);
});


// FILE: ./assets/js/controllers/Sm00001001_Documento_Proposta_Controller.js
function Sm00001001_Documento_Proposta_Controller(General, ScreenService, Utilities) {
    
/* --- FUNÇÕES DA PÁGINA DOCUMENTO --- */

    var self = this;
    
    /**
     * Salva o registro da página "Documento"
     * 
     * @param {Object} widget widget de documento
     */
    this.salvarDocumento = function(widget) {
        widget.currentRow.__columns_to_return = ["NRSEQORCADOC", "NRORG", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        widget.currentRow.MUDOU_SITUACAO = false;
        if (widget.currentRow.__is_new || 
            (widget.backupRow && widget.backupRow.IDSITUACAOORCAMENTO.toString() != widget.currentRow.IDSITUACAOORCAMENTO.toString())
            ) {
            widget.currentRow.MUDOU_SITUACAO = true;
        }

        Utilities.widgetSaveAndSetColumnsToReturn(widget);

        /*var nrtipoorcamento = widget.currentRow.NRTIPOORCAMENTO;
        
        if (nrtipoorcamento instanceof Object) {
            nrtipoorcamento = 0;
        }
        
        var isNew = widget.currentRow.__is_new;
        var dscaminhoarq = widget.currentRow.DSCAMINHOARQ;
        
        if (!isNew && !(dscaminhoarq instanceof Array)) {
            widget.currentRow.DSCAMINHOARQ = [{
                "name": widget.currentRow.NMARQUIVO,
                "path": dscaminhoarq,
                "size": widget.currentRow.VRTAMANHO * 1024,
                "type": widget.currentRow.DSTIPO
            }];
        }
        
        General.widgetSave(widget).then(function(response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRSEQORCADOC   = response.dataset.columnsToReturn[0].NRSEQORCADOC;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
            widget.reload().then(function() {
                this.habilitaFieldSituacao(widget, "save");
                this.verificarSeExisteAnexo(widget);
            }.bind(this));
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });*/
    };
    
    this.setFilterSituacaoEscopo = function(widgetDocumento){
        var fieldEscopo = widgetDocumento.getField("NMTIPOORCAMENTO");
        
        if(widgetDocumento.currentRow.NRSEQORCAMENTO !== ""){
            if(widgetDocumento.currentRow.__is_new){
                widgetDocumento.currentRow.NRTIPOORCAMENTO = "";
            }
            
            fieldEscopo.dataSourceFilter[0].operador = "IN"; 
            fieldEscopo.dataSourceFilter[0].value  = widgetDocumento.currentRow.NRSEQORCAMENTO;
            fieldEscopo.reload();
        } else {
            fieldEscopo.dataSourceFilter[0].operador = "IS_NOT_NULL"; 
        }
    };
    
    /**
     * Atualiza o documento de acordo com o modelo encontrado.
     * 
     * @param {Object} widget
     */
    this.atualizaDocumento = function(widget){
        var params = {
            "NRSEQMODDOCFCT" : widget.currentRow.NRSEQMODDOCFCT,
            "NRSEQORCADOC"   : '_' + widget.currentRow.NRSEQORCADOC + "_"
        };
        
        General.callRestEngine("product#/fct_orcamentodoc_widget/atualizaDocumento", "Row", params).then(function() {
            ScreenService.closePopup();
            ScreenService.showMessage("Documento atualizado com sucesso!");
            widget.reload().then(function() {
                this.verificarSeExisteAnexo(widget);
            }.bind(this));
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Realiza o controle de visibilidade das actions.
     * Se a situação for diferente de "Elaboração" as  
     * actions de edit, Atualizar e delete são blqueadas.
     */
    /*this.controlaVisibilidadeDoc = function(widget) {
        this.verificarSeExisteAnexo(widget);
        
        var situacao = widget.currentRow.IDSITUACAOORCAMENTO;
        
        var actAtualizarDocumento = widget.getAction("atualizarDocumento");
        //var actDocumento          = widget.getAction("documento_proposta");
        var actExcluirAnexo       = widget.getAction("excluirAnexo");
        var actEdit               = widget.getAction("edit");
        var actDelete             = widget.getAction("delete");
        
        if (situacao == "ELABORACAO") {
            actAtualizarDocumento.isVisible = true;
            actEdit.isVisible               = true;
            //actDocumento.isVisible          = true;
            actDelete.isVisible             = true;
        }
        else if (situacao == "CANCELADO") {
            actAtualizarDocumento.isVisible = false;
            actEdit.isVisible               = false;
            //actDocumento.isVisible          = false;
            actExcluirAnexo.isVisible       = false;
            actDelete.isVisible             = false;
        }
        else if (situacao == "CONCLUIDO" || situacao == "ACEITO" || situacao == "ENVIADO") {
            actAtualizarDocumento.isVisible = false;
            actEdit.isVisible               = false;
            //actDocumento.isVisible          = true;
            actExcluirAnexo.isVisible       = false;
            actDelete.isVisible             = false;
        }
    };
    */
    
    
    /**
     * Gera o relatório do documento da proposta.
     */
    /*this.gerarRelatorioDocumentoProposta = function(widget) {
        var nrorg = widget.currentRow.NRORG;
        var nrseqorcamento = widget.currentRow.NRSEQORCAMENTO;
        var nrtipoorcamento = (widget.currentRow.NRTIPOORCAMENTO != null) ? widget.currentRow.NRTIPOORCAMENTO : 0;
        
         var params = {
            "NRSEQORCAMENTO" : nrseqorcamento,
            "NRTIPOORCAMENTO": nrtipoorcamento,
            "NRORG"          : nrorg
        };
        
        General.callRestEngine("product#/fct_orcamentodoc_widget/gerarDocumentoProposta", "Row", params).then(function() {
            
            var reportName = "documento_proposta";
            Utilities.openReport("product#/geraBirtReport", reportName, params);
            ScreenService.closePopup();
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };*/
    
    this.verificarSeExisteAnexo = function(wdgDocumento) {
        var actDownload     = wdgDocumento.getAction("download");
        var actExcluirAnexo = wdgDocumento.getAction("excluirAnexo");
        var fieldAnexo      = wdgDocumento.getField("DSCAMINHOARQ");

        var existe = (wdgDocumento.currentRow.NMARQUIVO) ? true : false;
        actDownload.isVisible     = existe;
        actExcluirAnexo.isVisible = existe;
        fieldAnexo.isVisible      = !existe;
    }; 
    
    /**
     * Abre uma nova janela para fazer o download do documento.
     */
    /*this.downloadDocumento = function(wdgDocumento) {
        ScreenService.closePopup();
        var params = { "NRSEQORCADOC": wdgDocumento.currentRow.NRSEQORCADOC };
        General.callRestEngine("product#/fct_orcamentodoc_widget/downloadDocFileServer", "Row", params).then(function(response) {
            if(response.dataset && response.dataset.URLDOCUMENTO && response.dataset.URLDOCUMENTO.length > 0) {
                var urlDocumento = window.backendIndexUrl + response.dataset.URLDOCUMENTO[0]; 
                window.open(urlDocumento);
            }
        }).catch(function(error){
            window.open(error);
        });
    };
    
    this.excluirAnexo = function(wdgDocumento) {
        var params = {
            "NRSEQORCADOC": wdgDocumento.currentRow.NRSEQORCADOC,
            "DSCAMINHOARQ": wdgDocumento.currentRow.DSCAMINHOARQ,
            "NRORG"       : wdgDocumento.currentRow.NRORG
        };
        
        General.callRestEngine("product#/fct_orcamentodoc_widget/excluirAnexo", "Row", params).then(function() {
            ScreenService.showMessage("Anexo excluído com sucesso!");
            ScreenService.closePopup();
            wdgDocumento.currentRow.NMARQUIVO    = null;
            wdgDocumento.currentRow.DSCAMINHOARQ = null;
            wdgDocumento.currentRow.VRTAMANHO    = null;
            this.verificarSeExisteAnexo(wdgDocumento);
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };*/

    /**
     * Coloca a tela em modo de edição 
     * @params{Object} widget - widget documento de proposta
    **/
    this.editarDocumentoProposta = function(widget) {
        this.habilitaFieldSituacao(widget, "edit");
        General.widgetEdit(widget);
    };
    /**
    * Altera a visibilidade dos dados na field de situacao
    * @params{Object} widget documento da proposta
    **/
    this.habilitaFieldSituacao = function(widget, actionName){
        var idSituacao = widget.currentRow.IDSITUACAOORCAMENTO;
        if(idSituacao) {
            if(actionName == "edit" && !Array.isArray(idSituacao)) 
                widget.currentRow.IDSITUACAOORCAMENTO = idSituacao.split(", ");
            else 
                if(actionName != "edit" && Array.isArray(idSituacao))
                    widget.currentRow.IDSITUACAOORCAMENTO = idSituacao.join(", ");
        }
    };
    /**
     * Deleta um documento ligado a uma proposta
     * @params{Object} widget - widget docuementos proposta
    **/
    this.deletarDocumentoProposta = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function () {
            General.callRestEngine("product#/fct_orcamentodoc_widget/deletarDocumentoProposta", "DataSet", [widget.currentRow]).then(function(){
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        });
    };

    this.adicionarAnexoIdioma = function(widget) {
        General.widgetNewRow(widget);
    };
    
    this.editarAnexoIdioma = function(widget) {
        General.widgetEdit(widget);
    };
    
    this.excluirAnexoIdioma = function(widget) {
        General.deleteCrud(widget);
    };
    
    this.salvarAnexoIdioma = function(widget) {
        if (widget.isValid()) {
            Utilities.validarCamposDuplicados(widget, ["NRORCAMENTODOCIDIOMA"], ["CDIDIOM", "NRSEQORCADOC"]).then(function(podeSalvar) {
                if (podeSalvar) {
                    var isNew = widget.currentRow.__is_new;
                    var dscaminhoarq = widget.currentRow.DSCAMINHOARQ;

                    if (!isNew && !(dscaminhoarq instanceof Array)) {
                        widget.currentRow.DSCAMINHOARQ = [{
                            "name": widget.currentRow.NMARQUIVO,
                            "path": dscaminhoarq,
                            "size": widget.currentRow.VRTAMANHO * 1024,
                            "type": widget.currentRow.DSTIPO
                        }];
                    }

                    Utilities.widgetSaveAndSetColumnsToReturn(widget).then(function() {
                        widget.reload();
                    });
                }
                else {
                    ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>Já existe um anexo para o idioma informado."));
                }
            });
        }
    };

    this.cancelarAnexoIdioma = function(widget) {
        General.widgetCancelForm(widget);
    };

    this.downloadDocumento = function(widget) {
        ScreenService.closePopup();
        var params = { "NRORCAMENTODOCIDIOMA": widget.currentRow.NRORCAMENTODOCIDIOMA };
        General.callRestEngine("product#/fct_orcamentodocidioma_completo/downloadDocFileServer", "Row", params).then(function(response) {
            if (response.dataset && response.dataset.URLDOCUMENTO && !Util.isEmptyOrBlank(response.dataset.URLDOCUMENTO[0])) {
                var urlDocumento = window.backendIndexUrl + response.dataset.URLDOCUMENTO[0]; 
                window.open(urlDocumento);
            }
            else {
                ScreenService.showMessage(ScreenService.i18n("Documento não encontrado!"));
            }
        }).catch(function(error) {
            window.open(error);
        });
    };

    this.excluirAnexo = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente excluir o anexo?"), 'question', function() {
            var params = {
                "NRORCAMENTODOCIDIOMA": widget.currentRow.NRORCAMENTODOCIDIOMA,
                "NRSEQORCADOC"        : widget.currentRow.NRSEQORCADOC,
                "DSCAMINHOARQ"        : widget.currentRow.DSCAMINHOARQ,
                "IDPADRAO"            : widget.currentRow.IDPADRAO
            };
            
            General.callRestEngine("product#/fct_orcamentodocidioma_completo/excluirAnexo", "Row", params).then(function() {
                ScreenService.showMessage(ScreenService.i18n("Anexo excluído com sucesso!"));
                ScreenService.closePopup();
                widget.currentRow.NMARQUIVO    = null;
                widget.currentRow.DSCAMINHOARQ = null;
                widget.currentRow.DSTIPO       = null;
                widget.currentRow.VRTAMANHO    = null;
                self.verificarSeExisteAnexo(widget);
            }).catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        });
    };

    this.atualizarDocumentoIdioma = function(wdgAnexosIdioma) {
        ScreenService.confirmMessage("Deseja realmente sobrescrever o documento em anexo para o mesmo documento do Modelo de Documento e Idioma informados?", "question", function() {
            var params = {
                "NRORCAMENTODOCIDIOMA" : wdgAnexosIdioma.currentRow.NRORCAMENTODOCIDIOMA,
                "NRSEQORCADOC"         : wdgAnexosIdioma.currentRow.NRSEQORCADOC,
                "IDPADRAO"             : wdgAnexosIdioma.currentRow.IDPADRAO
            };
            
            General.callRestEngine("product#/fct_orcamentodoc_widget/atualizarDocumentoIdioma", "Row", params).then(function() {
                ScreenService.closePopup();
                ScreenService.showMessage("Documento atualizado com sucesso!");
                return wdgAnexosIdioma.reload();
            }).then(function() {
                self.verificarSeExisteAnexo(wdgAnexosIdioma);
            }).catch(function(error) {
                if (error.data.error.indexOf("SEM_MODELO") === 0) {
                    ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>Não foi encontrado um anexo de acordo com o Modelo de Documento e Idioma informado."));
                }
                else {
                    Utilities.showErrorMessage(error);
                }
            });
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001001_Documento_Proposta_Controller", Sm00001001_Documento_Proposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00001002_HistoricoAlteracoesProposta_Controller.js
function Sm00001002_HistoricoAlteracoesProposta_Controller(templateManager) {
    
    /**
     * Filtra todas as widgets pelas propostas selecionadas.
     * 
     * @param {object} wdgTabbedPrincipal
     */
    this.filtrarPropostas = function(wdgTabbedPrincipal) {
        var propostas = wdgTabbedPrincipal.getField("NRSEQORCAMENTO").value();
        
        var wdgDadosAlteracao = templateManager.container.getWidget("wdgDadosAteracaoProp");
        wdgDadosAlteracao.dataSourceFilter[0].value = propostas;
        wdgDadosAlteracao.reload();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001002_HistoricoAlteracoesProposta_Controller", Sm00001002_HistoricoAlteracoesProposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00001101_RelatorioCustomizavelProposta_Controller.js
function Sm00001101_RelatorioCustomizavelProposta_Controller(templateManager, ScreenService, Sm00001000_LancamentoProposta_Controller, General, ZhFilterPreferences) {
    
    /**
    * Abre a tela de proposta filtrando pelo orcaamento
    * 
    * @param {object} widget widget de dados da proposta  
    **/
    this.abrirProposta = function(widget) {
       ScreenService.openWindow('product#sm_00001_100_lancamento_proposta').then(function() {
            var wdgPropostaGrid = templateManager.container.getWidget('wdgPropostaGrid');
            
            wdgPropostaGrid.reload().then(function() {
                ScreenService.closePopup();
                wdgPropostaGrid.currentRow = {
                    "IDSITUACAOORCAMENTO" : widget.currentRow.IDSITUACAOORCAMENTO,
                    "NRSEQORCAMENTO"      : widget.currentRow.NRSEQORCAMENTO,
                    "IDUTILIZAVARPRECO"   : widget.currentRow.IDUTILIZAVARPRECO,
                    "IDUTILIZACOMP"       : widget.currentRow.IDUTILIZACOMP,
                    "QTITENSTRABSERV"     : widget.currentRow.QTITENSTRABSERV,
                    "NRORG"               : widget.currentRow.NRORG
                };
                Sm00001000_LancamentoProposta_Controller.wdgPropostaGridAfterMoveRow(wdgPropostaGrid);
            });
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00001101_RelatorioCustomizavelProposta_Controller", Sm00001101_RelatorioCustomizavelProposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00002000_Modelo_Proposta_Controller.js
function Sm00002000_Modelo_Proposta_Controller(General, ScreenService, templateManager, Sm00002000_Modelo_Proposta_Service, Utilities, ZhFilterPreferences) {

    var tokenCertisign = null;
    
    var getWidgetPerfilAcesso = function () {
        return templateManager.container.getWidget('ParamPerfilAcesso');
    };
    //field
    var getFieldNrperfilacessoFromWidgetPerfilAcesso = function(){
        return getWidgetPerfilAcesso().getField('NRPERFILACESSO');
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Modelo de proposta para salvar 
     * o novo registro.
    */
    this.saveModeloProposta = function(widget){
        
        widget.currentRow.IDTRANSPORTADORA = null;
        widget.currentRow.IDUTILIZAHIERSERV = null;
        widget.currentRow.NRTIPOORCAMENTO = null; 
        widget.currentRow.__columns_to_return = ["NRORCAMENTOMODELO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "DTULTATU", "CDOPERULTATU"];
        
        General.widgetSave(widget).then(function(response){
            this.bloqueiaItensOuComposicao(widget);
            this.insereSignatariosPadroesOrca(widget);
            
            if (response.dataset){
                widget.currentRow.NRORCAMENTOMODELO = response.dataset.columnsToReturn[0].NRORCAMENTOMODELO;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.DTULTATU = response.dataset.columnsToReturn[0].DTULTATU;
                widget.currentRow.CDOPERULTATU = response.dataset.columnsToReturn[0].CDOPERULTATU;
                
            }
            
        }.bind(this));
        
    };
    
    /**
    * Edita um modelo de documento
    * @params{Object} widget modelo de documento
    **/
    this.documentoEdit = function(widgetModeloDocumento){
        this.habilitaFieldSituacao(widgetModeloDocumento, "edit");
        General.widgetEdit(widgetModeloDocumento);
    };

    this.WdgOnEnterOnce = function(widget){
        ZhFilterPreferences.openFilterWidget(widget).then(function(){
        var widget = templateManager.container.getActiveWidget();
            widget.currentRow.IDATIVO = 'S';
        });
    };

    this.fixSelectMultipleBug = function() {
        var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
        field.reload();
    };
    this.salvarPerfilAcesso = function(widget){

    var parametro = {
        "NRORCAMENTOMODELO"     : widget.currentRow.NRORCAMENTOMODELO,
        "NRORG"   : widget.currentRow.NRORG,
        "PROPOUCONT" : "P"
    };
    General.callRestEngine('product#/procuraPerfilParametrizado', "Row", parametro).then(function (perfilusuario) {
        if(perfilusuario.dataset.validacao){
            var perfisPermitidos = [];
            widget.currentRow.NRPERFILACESSO.forEach(function(perfil){
                if(!perfilusuario.dataset.validacao.includes(perfil)){
                    perfisPermitidos.push(perfil);
                }
            });
            widget.currentRow.NRPERFILACESSO = perfisPermitidos;
        }

            Utilities.inserirMultiplosComNovoCodigo(widget, "NRPERFILACESSO").then(function() {
                ScreenService.closeSwipe().then(function(){
                    getWidgetPerfilAcesso().reload();
                });

          }.bind(this));
        var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
        field.dataSource.data = [];
        })
    };
    /**
     * Exclui a Composição relacionada e seus registros filhos.
     * 
     * @param {object} widget
     */
    this.excluirComposicao = function(widget) {
        General.deleteCrud(widget).then(function() {
            widget.reload();
        },
        function(error) {
            console.error(error);
            ScreenService.showMessage(error);
        });
    };
    
    /**
     * Valida se o modelo utiliza composição para bloquear a inserção 
     * de itens que não sejam item espelho.
     * 
     * @param {object} widget 
    */
    
    this.validaComposicao = function(widget){
    
      var wdgModelo = templateManager.container.getWidget("gridModeloProposta");    
        
      var UsaComposicao = wdgModelo.currentRow.IDUTILIZACOMP;
      var UsaItemEspelho = widget.getField("IDITEMESPELHO").value();
      
      if (UsaComposicao == 'S') {
          
          if (UsaItemEspelho == 'NAO'){
              return true;
          }
          else{
             ScreenService.showMessage("Operação Bloqueada.<br><br>Quando o modelo de proposta utiliza composição de serviços, não é permitido vincular um item espelho."); 
             return false; 
          }
      }
      else{
          
        return true;
      }
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Item para salvar o novo registro.
    */
    this.salvarItem = function(widget){
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta");   
        var validaComposicao = this.validaComposicao(widget);
        
        if (validaComposicao){
            widget.currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
            widget.currentRow.IDADICCOBRITEMORC = 'N';
            this.setFields(widget);
            
            if(widget.currentRow.TIPO == 'S')
                widget.currentRow.CDSERVORCA = widget.currentRow.CODIGO;
            else            
                widget.currentRow.CDPRODUTO = widget.currentRow.CODIGO;
            
            widget.currentRow.__columns_to_return = ["NRITEMORCAMODELO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "DTULTATU", "CDOPERULTATU"];
            General.widgetSave(widget).then(function(response){
                if (response.dataset)
                    widget.currentRow.NRITEMORCAMODELO = response.dataset.columnsToReturn[0].NRITEMORCAMODELO;
                    widget.currentRow.NRORG            = response.dataset.columnsToReturn[0].NRORG;
                    widget.currentRow.DTINCLUSAO       = response.dataset.columnsToReturn[0].DTINCLUSAO;
                    widget.currentRow.CDOPERINCLUSAO   = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                    widget.currentRow.DTULTATU         = response.dataset.columnsToReturn[0].DTULTATU;
                    widget.currentRow.CDOPERULTATU     = response.dataset.columnsToReturn[0].CDOPERULTATU;
                    
                    this.bloqueiaAdicionais(widget);
                    widget.reload();
                    
            }.bind(this));
        }
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget widget de 
     * Adicionais para salvar o novo registro.
    */
    this.salvarAdicItem= function(widget){
        var wdgModelo = templateManager.container.getWidget("wdgItens");   
        widget.currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
        widget.currentRow.NRITEMORCAMODELOPAI = wdgModelo.currentRow.NRITEMORCAMODELO;
        this.setFields(widget);
        
        if(widget.currentRow.TIPO == 'S')
            widget.currentRow.CDSERVORCA = widget.currentRow.CODDIGO;
        else
            widget.currentRow.CDPRODUTO = widget.currentRow.CODIGO;
        
        widget.currentRow.__columns_to_return = ["NRITEMORCAMODELO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "DTULTATU", "CDOPERULTATU"];
        General.widgetSave(widget).then(function(response){
            if (response.dataset)
                widget.currentRow.NRITEMORCAMODELO = response.dataset.columnsToReturn[0].NRITEMORCAMODELO;
                widget.currentRow.NRORG            = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO       = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO   = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.DTULTATU         = response.dataset.columnsToReturn[0].DTULTATU;
                widget.currentRow.CDOPERULTATU     = response.dataset.columnsToReturn[0].CDOPERULTATU;
                widget.reload();
            
        }.bind(this));
    };
    
    /**
    * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo 
     * e salva o escopo no modelo.
     * 
     * * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do escopo relacioando.
     */
    this.validaEscopoProposta = function(widget){
        if (widget.isValid()) {
            var wdgModelo = templateManager.container.getWidget("gridModeloProposta"); 
            var insert = widget.currentRow.__is_new;
          
            if (insert == true){
                widget.currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
                
                var arrayEscopoModelo = this.criaArrayEscoposModelo(widget);
        
                var tipoOrcamento = arrayEscopoModelo.listaValida;
                var orcaModelo = wdgModelo.currentRow.NRORCAMENTOMODELO;
            
               
                Sm00002000_Modelo_Proposta_Service.validaEscopoProposta(tipoOrcamento, orcaModelo).then(function(response){
                    if (response.dataset.VALIDAESCOPO.VALIDA == false) {
                        
                        General.callRestEngine("product#/fct_relacescopomodelo_widget/saveInsert", "DataSet", arrayEscopoModelo.rows).then(function(){
                        
                            ScreenService.closeSwipe().then(function(){
                                widget.reload();
                                widget.widgets.forEach(function(widget){
                                    widget.enabled = true;
                                });
                            });
                        }.bind(this));
                    }
                    else{
                          ScreenService.showMessage('Operação Bloqueada. Alguns dos escopos de faturamento selecionados já estão inseridos.'); 
                    }
        
               }.bind(this)).catch(function(response){
                   ScreenService.showMessage(response.data.error);
               });
            }
            else{
                General.widgetSave(widget);
            }
        }
    };
    
    
    /**
     * Função para criar o array dos escopos que foram 
     * pegos na função "recebeEscoposModelo";
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do escopo relacioando.
     * 
    */
    
    this.criaArrayEscoposModelo = function(widget){
        
        var listaEscoposModelo = widget.currentRow.NRTIPOORCAMENTO;
        var rows = [];
        var listaValida = [];
        var row;
        
        listaEscoposModelo.forEach(function(escopoModelo, indice){
            row = this.recebeEscoposModelo(widget, indice);
            rows.push(row);
            listaValida.push(row.NRTIPOORCAMENTO);
        }.bind(this)); 
        
        return {
            "rows": rows,
            "listaValida": listaValida
        };
    };
    
    /**
     * Função para pegar as informações do array 
     * de escopos na currentRow, e tratá-las em 
     * uma cópia da currentRow para realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do escopo relacioando.
     * 
     * @param {int} indice index do escopo no array de 
     * escopos presente na currentRow.
    */
    this.recebeEscoposModelo = function(widget, indice){
        
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta"); 
        var currentRow = angular.copy(widget.currentRow);
        
        currentRow.__is_new = true;

        currentRow.NRTIPOORCAMENTO = currentRow.NRTIPOORCAMENTO[indice];
        currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
        
        return currentRow;
    };
   
   
     /**
     * A função "salvarCompProposta" salva as 
     * composicoes para o modelo.
     * 
     * * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro da composição relacionada.
     */
    this.salvarCompProposta = function(widget) {
        if(widget.isValid()){
            var arrayComposicaoModelo = this.criaArrayComposicaoModelo(widget);
            
            General.callRestEngine("product#/fct_relorcamodcomposicao_widget/save", "DataSet", arrayComposicaoModelo.rows).then(function() {
                ScreenService.closeSwipe().then(function(){
                    widget.reload();
                });
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    
    /**
     * Função para criar o array das composições que foram 
     * pegos na função "recebeComposicaoModelo";
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro da composição relacionada.
     * 
    */
    
    this.criaArrayComposicaoModelo = function(widget){
        
        var listaComposicaoModelo = widget.currentRow.NRCOMPSERVICO;
        var rows = [];
        var listaValida = [];
        var row;
        
        listaComposicaoModelo.forEach(function(composicaoModelo, indice){
            row = this.recebeComposicaoModelo(widget, indice);
            rows.push(row);
            listaValida.push(row.NRCOMPSERVICO);
        }.bind(this)); 
        
        return {
            "rows": rows,
            "listaValida": listaValida
        };
    };
    
    /**
     * Função para pegar as informações do array 
     * de composições na currentRow, e tratá-las em 
     * uma cópia da currentRow para realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do escopo relacioando.
     * 
     * @param {int} indice index do escopo no array de 
     * escopos presente na currentRow.
    */
    this.recebeComposicaoModelo = function(widget, indice){
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta"); 
        var currentRow = angular.copy(widget.currentRow);
        
        currentRow.__is_new = true;
        
        currentRow.NRCOMPSERVICO = currentRow.NRCOMPSERVICO[indice];
        currentRow.NRORCAMENTOMODELO = wdgModelo.currentRow.NRORCAMENTOMODELO;
        
        return currentRow;
        
    };
    
    /**
     * Seta os campos da widget de itens 
     * que não aparecem na tela para o 
     * usuário
     * 
     * @param {object} widget widget de 
     * Itens
    */
    this.setFields = function(widget){
        widget.currentRow.SGUNIDADE = null;
        widget.currentRow.SGUNIDADEADICIONAL = null;
        widget.currentRow.CDPRECBASESERV = null;
        widget.currentRow.NRPARCTRANSPORTADORA = null;
        widget.currentRow.NRPARCFORNECEDOR = null;
        widget.currentRow.NMREDUITEMORCA = null;
        widget.currentRow.NMMNEMOITEMORCA = null;
        widget.currentRow.DSQTITEMORCA = null;
        widget.currentRow.DSQTITEMORCAADIC = null;
        widget.currentRow.QTITEMORCAADIC = null;
        widget.currentRow.DSAGRUPAITEMORCA = null;
        widget.currentRow.TXFORMULAITORCA = null;
        widget.currentRow.QTDIASCONTRATO = null;
        widget.currentRow.NRTIPOHIERARQSERV = null;
        widget.currentRow.IDSOMASUBITENS = 'N';
        widget.currentRow.IDSOMANOITEMPAI = 'S';
        widget.currentRow.IDSOMANOTOTALORCA = 'S';
        widget.currentRow.IDTRANSPORTADORA = 'TERCEIRO';
        widget.currentRow.IDUTILIZAHIERSERV = 'NAO';
    };
    
    /**
     * Bloqueia ou libera o campo de
     * modalidade de pagamento
     * 
     * @param {object} widget widget de 
     * Itens
    */
    this.blockFieldModPg = function(widget){
        
        var campoNomeModPgto = widget.getField("DSMODPGTO");
        var campoQtdParcelas = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;
        
        if (tipofaturamento == "O") {
            
            campoNomeModPgto.readOnly = false;
            campoNomeModPgto.validations = {"required": true};
            
            campoQtdParcelas.readOnly = true;
            campoQtdParcelas.value(null);
            campoQtdParcelas.validations = {};
            
            campoDiaVencParcelaFatu.validations = {};
        } 
        else {
            campoNomeModPgto.readOnly = true;
            campoNomeModPgto.value(null);
            campoNomeModPgto.validations = {};
            widget.currentRow.CDMODPGTO = null;
            
            if (tipofaturamento == "P") {
                campoQtdParcelas.readOnly = false;
                campoQtdParcelas.validations = {"required": true};
                
                campoDiaVencParcelaFatu.validations = {"required": true};
            }
            else {
                campoQtdParcelas.readOnly = true;
                campoQtdParcelas.value(null);
                campoQtdParcelas.validations = {};
                
                campoDiaVencParcelaFatu.validations = {};
            }
        }
    };
    
    this.blockFieldItModPg = function(widget){
        
        var campoNomeModPgto = widget.getField("DSMODPGTO");
        var campoQtdParcelas = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        
        var tipofaturamento = widget.currentRow.IDFATURAORCA;
        
        if (tipofaturamento == "O") {
            
            campoNomeModPgto.readOnly = false;
            campoNomeModPgto.validations = {"required": true};
            
            campoQtdParcelas.readOnly = true;
            campoQtdParcelas.value(null);
            campoQtdParcelas.validations = {};
            
            campoDiaVencParcelaFatu.validations = {};
        } 
        else {
            campoNomeModPgto.readOnly = true;
            campoNomeModPgto.value(null);
            campoNomeModPgto.validations = {};
            widget.currentRow.CDMODPGTO = null;
            
            if (tipofaturamento == "P") {
                campoQtdParcelas.readOnly = false;
                campoQtdParcelas.validations = {"required": true};
                
                campoDiaVencParcelaFatu.validations = {"required": true};
            }
            else {
                campoQtdParcelas.readOnly = true;
                campoQtdParcelas.value(null);
                campoQtdParcelas.validations = {};
                
                campoDiaVencParcelaFatu.validations = {};
            }
        }
    };
    
    /**
     * Bloqueia ou libera o campo de
     * Serviço/Produto da widget de Itens
     * 
     * @param {object} widget widget de 
     * Itens
    */
    this.blockServProd = function(widget, trocarLabel) {
        if (trocarLabel === undefined) {
            trocarLabel = true;
        }
        var Itemespelho =  widget.currentRow.IDITEMESPELHO;
        
        if (Itemespelho == "SIM") {
            widget.getField("NMITEMORCAMENTO").readOnly = true;
            if (trocarLabel) widget.currentRow.NMITEMORCAMENTO = 'Item Espelho';
            widget.getField('NMITEMORCAMENTO').validations = {};
        }
        else {
            widget.getField("NMITEMORCAMENTO").readOnly = false;
            if (trocarLabel) widget.currentRow.NMITEMORCAMENTO = null;
            widget.getField('NMITEMORCAMENTO').validations = {"required": true};
        }
    };
    
    /**
     * Bloqueia ou libera os campos
     * 'Adicional para cobrança?'  e 
     * 'Item Pai' da widget de Itens
     * 
     * @param {object} widget widget de 
     * Itens
    */
    this.blockAdItmPai = function(widget) {
        
        var itempai = widget.currentRow.IDADICITEMORC;
        
        if (itempai == "S") {
            
            widget.getField("IDADICCOBRITEMORC").readOnly = false;
            widget.getField("NRITEMORCAMODELOPAI").readOnly = false;
        }
        else {
            widget.getField("IDADICCOBRITEMORC").readOnly = true;
            widget.currentRow.IDADICCOBRITEMORC = 'N';
            
            widget.getField("NRITEMORCAMODELOPAI").readOnly = true;
            widget.currentRow.NRITEMORCAMODELOPAI = null;
        }
    };
    
    /**
     * Realiza o controle dos campos de "Medida" de acordo 
     * com o field "Utiliza Medida?"
     * 
     * @param {object} widget widget de 
     * Itens para pegar o field "IDMULTMEDVRTOT".
    */
    this.bloquearDadosMedida = function(widget){
        
        var usamedida = widget.getField('IDMULTMEDVRTOT').value();
        
        /* Bloqueia a Medida*/
        if (usamedida == 'N') {
            
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = true;
            widget.getField('DSTPMEDIDASERV').value(null);
            widget.getField('CDTPMEDIDASERV').value(null);
            widget.getField('DSTPMEDIDASERV').validations = {};
            
            /* Quantidade de Medida */
            widget.getField('QTMEDIDAORCA').readOnly = true;
            widget.getField('QTMEDIDAORCA').value(null);
            widget.getField('QTMEDIDAORCA').validations = {};
            
        } 
        /* Desbloqueia a Medida*/
        else {
            
            /* Medida */
            widget.getField('DSTPMEDIDASERV').readOnly = false;
            widget.getField('DSTPMEDIDASERV').validations = {"required": true};
            
            /* Quantidade de Medida */
            widget.getField('QTMEDIDAORCA').readOnly = false;
            widget.getField('QTMEDIDAORCA').validations = {"required": true};
        }

    };
    
    /**
     * Realiza o cálculo do valor total do item
     * 
     * @param {object} widget widget de 
     * Itens para pegar os fields para cálculo
    */
    
    this.calculaValorTotItem = function(widget){
        var quantidade        = widget.currentRow.QTITEMORCAMENTO;
        var utilizaQtdeMedida = widget.currentRow.IDMULTMEDVRTOT;
        var qtdeMedida        = widget.currentRow.QTMEDIDAORCA;
        var valorUnitario     = widget.currentRow.VRUNITITEMORCA;
        
        if (quantidade) quantidade = parseFloat(quantidade.toString().replace(",", "."));
        if (qtdeMedida) qtdeMedida = parseFloat(qtdeMedida.toString().replace(",", "."));
        if (valorUnitario) valorUnitario = parseFloat(valorUnitario.toString().replace(",", "."));
        
        var valorSemDesconto = quantidade * valorUnitario;
        if (utilizaQtdeMedida == "S") {
            valorSemDesconto *= qtdeMedida;
        }
        
        var valorTotal = valorSemDesconto;
        
        //widget.currentRow.VRITEMSEMDESC   = valorSemDesconto;
        widget.currentRow.VRTOTALITEMORCA = valorTotal;
    };
    
    /**
     * Consolida as chamadas de funções das 
     * Widgets "Itens" e "Adicionais"
     * 
     * @param {object} widget - widget de Itens
     */
    this.funcoesItens = function(widget) {
        this.blockAdItmPai(widget);
        this.blockFieldItModPg(widget);
        this.bloquearDadosMedida(widget);
        this.blockServProd(widget, false);
    };
    
    /**
     * Bloqueia / Desbloqueia as widgets de "Composição de Serviços" e "Adicionais" 
     * ao selecionar um registro.
     * 
     * @param {object} widget 
     */
    this.bloqueiaItensOuComposicao = function(widget) {
        var usaComposicao = widget.currentRow.IDUTILIZACOMP;
        
        if (usaComposicao) {
            if (usaComposicao == 'S') {
                templateManager.container.getWidget("wdgComposicoes").enabled = true;
                templateManager.container.getWidget("wdgItens").enabled = false;
            }
            else {
                templateManager.container.getWidget("wdgComposicoes").enabled = false;
                templateManager.container.getWidget("wdgItens").enabled = true;
            }
        }
    };
    
    /**
     * Bloqueia / Desbloqueia a widget  "Adicionais" ao selecionar ao salvar um item.
     * 
     * @param {object} widget 
     */
    this.bloqueiaAdicionais = function(widget) {
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta");
        
        var UsaComposicao = wdgModelo.currentRow.IDUTILIZACOMP;
      
        if (UsaComposicao == 'S') {
          
            templateManager.container.getWidget("wdgAdicionais").enabled = false;
        }
        else {
          
            templateManager.container.getWidget("wdgAdicionais").enabled = true;
        }
      
    };
    
    /**
     * Bloqueia / Desbloqueiaos campos "Item Obrigatório" e "Item Espelho".
     * 
     * @param {object} widget 
     */
    /*this.bloqCamposComposicao = function(widget) {
      
        var UsaComposicao = widget.parent.currentRow.IDUTILIZACOMP;
      
        var insert = widget.currentRow.__is_new;
     
        if (UsaComposicao == 'S') {
          
            //Item Espelho
            widget.getField('IDITEMESPELHO').readOnly = true;
            widget.currentRow.IDITEMESPELHO = 'NAO';
          
            //Faturamento do Item
            widget.getField('IDFATURAORCA').readOnly = true;
            widget.currentRow.IDFATURAORCA = 'A';
          
            //Grupo de Faturamento
            widget.getField('NMGRUPOFATURA').readOnly = true;
          
            //Dias para início do Faturamento
            widget.getField('QTDIASINICFAT').readOnly = true;
          
            //Item Obrigatório
            widget.getField('IDITEMOBRIGORCA').readOnly = true;
            widget.currentRow.IDITEMOBRIGORCA = 'N';
          
            //Bloqueia alteração
            widget.getField('IDBLOQALTORC').readOnly = true;
            widget.currentRow.IDBLOQALTORC = 'N';
          
            //Exibe no Documento
            widget.getField('IDIMPRESSAOORCA').readOnly = true;
            widget.currentRow.IDIMPRESSAOORCA = 'S';
          
        }
        else {
          
            //Item Espelho
            widget.getField('IDITEMESPELHO').readOnly = false;
          
            //Faturamento do Item
            widget.getField('IDFATURAORCA').readOnly = false;
          
            //Grupo de Faturamento
            widget.getField('NMGRUPOFATURA').readOnly = false;
          
            //Dias para início do Faturamento
            widget.getField('QTDIASINICFAT').readOnly = false;
          
            //Item Obrigatório
            widget.getField('IDITEMOBRIGORCA').readOnly = false;
          
            //Bloqueia alteração
            widget.getField('IDBLOQALTORC').readOnly = false;
          
            //Exibe no Documento
            widget.getField('IDIMPRESSAOORCA').readOnly = false;
          
            if (insert) {
              
                //Item Espelho
                widget.currentRow.IDITEMESPELHO = 'NAO'; 
             
                //Faturamento do Item
                widget.currentRow.IDFATURAORCA = 'A';
             
                //Item Obrigatório*
                widget.currentRow.IDITEMOBRIGORCA = 'N';
             
                //Bloqueia alteração
                widget.currentRow.IDBLOQALTORC = 'N';
             
                //Exibe no Documento
                widget.currentRow.IDIMPRESSAOORCA = 'S';
            }
        }
    };*/
    
     
    /**
     * Salva o novo registro da página "Documento"
     * 
     * @param {object} widget widget de 
     * Documento para salvar o 
     * novo registro.
    */
    this.salvarDocumento = function(widget){
        var wdgModelo = templateManager.container.getWidget("gridModeloProposta");
        
        widget.currentRow.NRORCAMENTOMODELO =wdgModelo.currentRow.NRORCAMENTOMODELO;
        
        widget.currentRow.__columns_to_return = ["NRSEQMODDOCORCA",  "NRORG", "DTINCLUSAO", "NRORGINCLUSAO", "CDOPERINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset)
                widget.currentRow.NRSEQMODDOCORCA = response.dataset.columnsToReturn[0].NRSEQMODDOCORCA;
                widget.currentRow.NRORG = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO  = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.NRORGINCLUSAO = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
            
        }.bind(this));
    };
    
    
    
  /**
     * bloqueia o campo de escopo na edição da página escopo de proposta.
     * 
     * @param {object} widget 
    */
    
    this.preparaCampoEscopo = function(widget){
      
        var insert = widget.currentRow.__is_new;
      
        if (insert == false){
         
            widget.getField('NRTIPOORCAMENTO').showOnForm = false;
            widget.getField('NMTIPOORCAMENTO').showOnForm = true;

        }
        else{
              
            widget.getField('NRTIPOORCAMENTO').showOnForm = true;
            widget.getField('NMTIPOORCAMENTO').showOnForm = false;
        }
    };
    
    this.novoModelo = function(widget) {
        General.widgetNewRow(widget);
        this.blockFieldModPg(widget);
    //    this.validaIdCertsign();
        this.assinaturaEletronicaOnChange(widget);
    };
    
    this.novoItem = function(widget) {
        General.widgetNewRow(widget);
        this.funcoesItens(widget);
    };
    
    /* Salva um modelo de documento */
    this.salvaModeloDocumento = function(widgetModeloDocumento){
        if(!widgetModeloDocumento.isValid()) {
            ScreenService.showMessage("Existem campos obrigatórios a serem preenchidos.");
        } else {
            var isNew          = widgetModeloDocumento.currentRow.__is_new;
            var idSituacao     = widgetModeloDocumento.currentRow.IDSITUACAOORCAMENTO;
            var idSituacaoBckp = widgetModeloDocumento.backupRow ? widgetModeloDocumento.backupRow.IDSITUACAOORCAMENTO : null;
            widgetModeloDocumento.currentRow.__columns_to_return  = ["NRRELACESCOPOMODELO", "NRORCAESCOPODOC", "NRORG"];
            widgetModeloDocumento.currentRow.IDSITUACAOADICIONADA = (isNew || !idSituacaoBckp) ? idSituacao : idSituacao.filter(function (situacao) {
                return widgetModeloDocumento.backupRow.IDSITUACAOORCAMENTO.indexOf(situacao) === -1;
            });
            General.callRestEngine("product#/fct_orcaescopodoc_widget/validarExistenciaIdsituacao", "Row", widgetModeloDocumento.currentRow).then(function(response){
                if(isNew && idSituacao !== ""){
                    Utilities.inserirMultiplosComNovoCodigo(widgetModeloDocumento, "IDSITUACAOORCAMENTO", null, "product#/crud/fct_orcaescopodoc/save").then(function(response){
                        if(response.dataset){
                            widgetModeloDocumento.currentRow.NRRELACESCOPOMODELO = response.dataset.columnsToReturn[0].NRRELACESCOPOMODELO;
                            widgetModeloDocumento.currentRow.NRORG               = response.dataset.columnsToReturn[0].NRORG;
                            widgetModeloDocumento.currentRow.NRORCAESCOPODOC     = response.dataset.columnsToReturn[0].NRORCAESCOPODOC;
                        }
                        
                        ScreenService.closeSwipe();
                        widgetModeloDocumento.reload();
                    }.bind(this))
                    .catch(function(error){
                        Utilities.showErrorMessage(error);
                    });
                } else {
                    widgetModeloDocumento.currentRow.IDSITUACAORETIRADA = (isNew || !idSituacaoBckp) ? [] : widgetModeloDocumento.backupRow.IDSITUACAOORCAMENTO.filter(function (situacao) {
                        return idSituacao.indexOf(situacao) === -1;
                    });
                    
                    General.callRestEngine("product#/fct_orcaescopodoc_widget/editarDocumentosEscopoModelo", "Row", widgetModeloDocumento.currentRow).then(function(response){
                        if(!isNew) {
                            widgetModeloDocumento.view();
                        } else {
                            ScreenService.closeSwipe();
                            widgetModeloDocumento.list();
                        }
                        this.habilitaFieldSituacao(widgetModeloDocumento, "save");
                        widgetModeloDocumento.reload();
                    }.bind(this))
                    .catch(function(error) {
                        Utilities.showErrorMessage(error);
                    });
                }
            }.bind(this))
            .catch(function(error){
                if (error.data && error.data.error.indexOf("SITUACAO_JA_CADASTRADA") !== -1) 
                    ScreenService.showMessage("Operação bloqueada!<br>Este modelo de documento já está vinculado a(s) situação(ões) de: " + error.data.error.split("::")[1]);
                else 
                    Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
    * Habilita fields de situação
    * @params{Object} widget documento
    **/
    this.habilitaFieldSituacao = function(widgetDocumento, actionName){
        var idSituacao = widgetDocumento.currentRow.IDSITUACAOORCAMENTO;
        if(idSituacao) {
            if(actionName == "edit" && !Array.isArray(idSituacao)) 
                widgetDocumento.currentRow.IDSITUACAOORCAMENTO = idSituacao.split(", ");
            else 
                if(actionName != "edit" && Array.isArray(idSituacao))
                    widgetDocumento.currentRow.IDSITUACAOORCAMENTO = idSituacao.join(", ");
        }
    };
    
    /**
    * Cancela o formulário da widget de documento
    * @params{Object} widget
    **/
    this.cancelaFormDocumento = function(widgetDocumento){
        this.habilitaFieldSituacao(widgetDocumento, "cancel");
        General.widgetCancelForm(widgetDocumento);
    };
    
    this.wdgPerfilAcessoCancelar = function() {
        var field = getFieldNrperfilacessoFromWidgetPerfilAcesso();
        field.dataSource.data = [];
    };
    /**
     * Deleta os modelos de documentos ligados a um escopo
     * @params{Object} widget - widget de modelo de documento
    **/
    this.deletarDocumentosEscopo = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), "question", function(){
            General.callRestEngine("product#/fct_orcaescopodoc/deletarDocumentos", "DataSet", [widget.currentRow]).then(function() {
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }, function(error) {
                Utilities.showErrorMessage(error);
            });
        });
    };
    
    this.saveSignatPadr = function(widget) {
        if (widget.isValid() ) {
            if (widget.currentRow.__is_new) {
                
                    
                General.callRestEngine('product#/fct_relacescopomodelo_widget/validaSignatPadrOrca', "Row", widget.currentRow).then(function(response) {  
                    
                    var retorno = response.dataset.validaSignatPadrOrca.valida;
                    
                    /* Recebe os envolvidos para notificação*/
                    var envolvidos = null;
                    var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
                    
                    if (arrayEnvolvidos != null && arrayEnvolvidos != ""){
                        envolvidos = arrayEnvolvidos.join('_');
                    }

                    widget.currentRow.DSEMAILNOTSIGNENV = envolvidos; 
                    
                    /* Recebe os operdores para notificação */
                    var operadores = null;
                    var arrayOperadores = widget.currentRow.CDOPERADORES;
                    
                    // if (arrayOperadores != null && arrayOperadores != ""){
                    //     operadores = arrayOperadores.join('_');
                    // }
                    
                    
                    if(retorno == '0') {
                        General.widgetSave(widget).then(function(response){
                            if(response.dataset && response.dataset.columnsToReturn[0]){
                                widget.currentRow.NRORG             = response.dataset.columnsToReturn[0].NRORG;
                                widget.currentRow.NRSIGNATPADRORCA  = response.dataset.columnsToReturn[0].NRSIGNATPADRORCA;
                                widget.currentRow.NRORCAMENTOMODELO = response.dataset.columnsToReturn[0].NRORCAMENTOMODELO;
                                
                                if (operadores != null) {
                                   var params = { 
                                        "NRSIGNATPADRORCA"  : widget.currentRow.NRSIGNATPADRORCA,
                                        "OPERADORES"        : arrayOperadores
                                   };
                
                                    // if(widget.currentRow.CDOPERADORES.length > 0){
                                       General.callRestEngine('product#/fct_relacescopomodelo_widget/insereOperadoresNotifica', "Row", params).then(function() {
                                       
                                            widget.reload();
                                            
                                       })
                                       .catch(function(response) {
                                           console.error(response);
                                           if (response instanceof Object)
                                               ScreenService.showMessage(response.data.error);
                                           else
                                               ScreenService.showMessage(response);
                                       });
                                    // }
                               }
                               else{
                                   
                                  widget.reload(); 
                               }
                            }
                        }.bind(this))
                        .catch(function(response) {
                            console.error(response);
                            if (response instanceof Object)
                                ScreenService.showMessage(response.data.error);
                            else
                                ScreenService.showMessage(response);
                        });
                    }
                    else{
                        ScreenService.showMessage("Operação Bloqueada! <br><br> O signatário informado já foi inserido.");
                    }
                });
            }
            else{
              
                /* Recebe os envolvidos para notificação*/
                var envolvidos = null;
                var arrayEnvolvidos = widget.currentRow.DSEMAILNOTSIGNENV_INS;
                
                if (arrayEnvolvidos != null){
                    envolvidos = arrayEnvolvidos.join('_');
                }
                
                widget.currentRow.DSEMAILNOTSIGNENV = envolvidos;
                
                /* Recebe os operdores para notificação */
                var operadoresIguais = false;
                
                var bkpOperadores = widget.backupRow.CDOPERADORES;
                var arrayOperadores = widget.currentRow.CDOPERADORES;
                
                if (Utilities.arraysSaoIguais(bkpOperadores, arrayOperadores)) {
                    operadoresIguais = true;
                }
                
                General.widgetSave(widget).then(function(response){
                    if (response.dataset && response.dataset.columnsToReturn[0]) {
                        widget.currentRow.NRORG             = response.dataset.columnsToReturn[0].NRORG;
                        widget.currentRow.NRSIGNATPADRORCA  = response.dataset.columnsToReturn[0].NRSIGNATPADRORCA;
                        widget.currentRow.NRORCAMENTOMODELO = response.dataset.columnsToReturn[0].NRORCAMENTOMODELO;
                    }
                    
                    if (operadoresIguais == false) {
                        // var operadores = arrayOperadores.join('_');
                        
                        var params = { 
                            "NRSIGNATPADRORCA" : widget.currentRow.NRSIGNATPADRORCA,
                            "OPERADORES"       : arrayOperadores
                        };
        
                        // if(widget.currentRow.CDOPERADORES.length > 0){
                            General.callRestEngine('product#/fct_relacescopomodelo_widget/insereOperadoresNotifica', "Row", params).then(function() {
                                widget.reload().then(function() {
                                    this.carregaDadosOperadores(widget);
                                }.bind(this));
                            }.bind(this))
                            .catch(function(error) {
                                Utilities.showErrorMessage(error);
                            });
                        // }
                    }
                    else {
                        this.carregaDadosOperadores(widget);
                    }
                }.bind(this))
                .catch(function(response) {
                    console.error(response);
                    if (response instanceof Object)
                        ScreenService.showMessage(response.data.error);
                    else
                        ScreenService.showMessage(response);
                }); 
            }
        }
    };
    
    /**
     * insere os signatários padrões de modelo de proposta que foram buscados na parametrização SM 
    **/
    this.insereSignatariosPadroesOrca = function(widget){
        if (widget.currentRow.IDASSELETRDIGITAL == "S") {
            General.callRestEngine("product#/fct_relacescopomodelo_widget/insereSignatariosPadroesOrca", "Row", widget.currentRow).then(null).catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * Abre a widget de Cadastramento Automático.
     */
    this.abrirCadAuto = function() {
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");
        var wdgGridModeloProposta = templateManager.container.getWidget("gridModeloProposta");
        
        wdgSignatPadrCadAuto.isVisible = true;
        
        var row = wdgSignatPadrCadAuto.currentRow;
        row.IDTPSIGNATARIO = "SE";
        row.DSEMAILNOTIFOUTROS = "";
        row.DSEMAILNOTSIGNENV = [];
        row.CDOPERADOR = [];
        row.CDOPERADORES = [];
        wdgSignatPadrCadAuto.edit();
        
        wdgSignatPadrCadAuto.getField("NRORCAMENTOMODELO").reload().then(function() {
            row.NRORCAMENTOMODELO = [wdgGridModeloProposta.currentRow.NRORCAMENTOMODELO];
        });
        
        row.NRPARCNEGOCIO     = [];
        row.IDTIPOASSINATURA  = 'E';
        row.IDTESTEMUNHA      = 'N';
        row.NRORDENACAO       = null;
        
        ScreenService.openSwipe(wdgSignatPadrCadAuto);
        this.controlaSignatarioCadastroAutomatico(wdgSignatPadrCadAuto);
        templateManager.updateTemplate();
    };
    
    /**
     * Fecha a widget de Cadastramento Automático.
     */
    this.cancelaCadAutoSignatPadr = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        ScreenService.closeSwipe().then(function() {
            wdgSignatPadrCadAuto.isVisible = false;
        });
    };
    
    /**
     * Coloca a widget "signatariosPadroesCadAuto" invisível
     */
    this.CadAutoInvisivel = function() {
        
        var wdgSignatPadrCadAuto = templateManager.container.getWidget("signatariosPadroesCadAuto");

        wdgSignatPadrCadAuto.isVisible = false;
    };
    
    /**
     * Salva os signatários padrões em cadastramento automático  if (wdgProposta.isValid()) {
     */
    this.saveSignatPadrCadAutoOrca = function(widget) {
        
        if (widget.isValid()) {
            
            var wdgSignatariosPadroes = templateManager.container.getWidget("signatariosPadroes");
            
            General.callRestEngine('product#/fct_relacescopomodelo_widget/cadAutoSignatPadrOrca', "Row", widget.currentRow).then(function(response) {  
                var retorno = response.dataset.cadauto.retorno;
                if(retorno == 'OK') {
                    ScreenService.closeSwipe().then(function() {
                        widget.isVisible = false;
                        wdgSignatariosPadroes.reload();
                        ScreenService.showMessage("Cadastramento automático realizado com sucesso!");
                    });
                } else {
                    ScreenService.showMessage("Ocorreram erros durante o processo de cadastramento automático. Favor entrar em contato o administrador do sistema. <br><br> Erros: <br><br>"+retorno);
                }
            });
        }
    };
    
    /**
     * Valida se o token da certsign está preenchido
     */
    this.validaIdCertsign = function(buscarToken) {
        if (buscarToken === undefined) {
            buscarToken = false;
        }
        if (buscarToken) {
            General.callRestEngine('product#/fct_relacescopomodelo_widget/validaIdCertsign', "Row", {}).then(function(response) {  
                tokenCertisign = response.dataset.validaIdCertsign.retorno;
                this.controlarCamposCertsign();
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            this.controlarCamposCertsign();
        }
    };
    
    this.controlarCamposCertsign = function() {
        var wdgSignatariosPadroes = templateManager.container.getWidget("signatariosPadroes");
        var wdgGridModeloProposta = templateManager.container.getWidget("gridModeloProposta");
        
        if (tokenCertisign == null) {
            /* Bloqueia a página "Signatários Padrões" */
            wdgSignatariosPadroes.enabled = false;
            
            /* Bloqueia o campo "Utiliza Assinatura Eletrônica ou Digital" */
            wdgGridModeloProposta.getField('IDASSELETRDIGITAL').readOnly = true;
            wdgGridModeloProposta.getField('IDASSELETRDIGITAL').validations = {};
            if (wdgGridModeloProposta.mode == "edit") {
               wdgGridModeloProposta.getField('IDASSELETRDIGITAL').value('N');
            }
        }
        else {
            /* Libera o campo "Utiliza Assinatura Eletrônica ou Digital" */
            wdgGridModeloProposta.getField('IDASSELETRDIGITAL').readOnly = false;
            wdgGridModeloProposta.getField('IDASSELETRDIGITAL').validations = {"required" : true};
            
            if (wdgGridModeloProposta.mode != "edit") {
                /* Libera a página "Signatários Padrões" */
                wdgSignatariosPadroes.enabled = true;
            }
        }
    };
    
    this.adicionarSignatarioPadrao = function(widget) {
        General.widgetNewRow(widget);
        this.controlaSignatario(widget);
    };
    
    this.editarSignatarioPadrao = function(widget) {
        General.widgetEdit(widget);
        this.controlaSignatario(widget);
        this.carregaDadosOperadores(widget);
    };
    
    this.controlaSignatario = function (widget){
        var campoNomePrincipal = widget.getField("NMPRINCIPALPARC");
        var campoIdTestemunha =  widget.getField("IDTIPOASSINATURA");
        var campoTipoAssinatura =  widget.getField("IDTESTEMUNHA");
        if (widget.currentRow.IDTPSIGNATARIO == 'SE') {
            campoNomePrincipal.readOnly = false;
            campoNomePrincipal.validations = {"required": true};
            campoIdTestemunha.readOnly = false;
            campoTipoAssinatura.readOnly = false;
        }
        else {
            campoNomePrincipal.readOnly = true;
            campoNomePrincipal.validations = {};
            campoTipoAssinatura.readOnly = true;
            campoIdTestemunha.readOnly = true;
            widget.currentRow.IDTIPOASSINATURA = "E";
            if (["SC", "SP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "N";
            }
            else if (["TC", "TP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "S";
            }
            //Limpando os campos readonly
            widget.currentRow.NRINSCRICAOPARC = widget.currentRow.NMPRINCIPALPARC = "";
        }
    };
    
    this.controlaSignatarioCadastroAutomatico = function(widget){
        var campoNomePrincipal = widget.getField("NRPARCNEGOCIO");
        var campoIdTestemunha =  widget.getField("IDTIPOASSINATURA");
        var campoTipoAssinatura =  widget.getField("IDTESTEMUNHA");
        if (widget.currentRow.IDTPSIGNATARIO == 'SE') {
            campoNomePrincipal.readOnly = false;
            campoNomePrincipal.validations = {"required": true};
            campoIdTestemunha.readOnly = false;
            campoTipoAssinatura.readOnly = false;
        }
        else {
            campoNomePrincipal.readOnly = true;
            campoNomePrincipal.validations = {};
            campoTipoAssinatura.readOnly = true;
            campoIdTestemunha.readOnly = true;
            widget.currentRow.IDTIPOASSINATURA = "E";
            if (["SC", "SP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "N";
            }
            else if (["TC", "TP"].includes(widget.currentRow.IDTPSIGNATARIO)) {
                widget.currentRow.IDTESTEMUNHA = "S";
            }
        }
        templateManager.updateTemplate();
    };    
    
    this.carregaDadosOperadores = function(widget){
       var operadores = widget.currentRow.CDOPERADORES;
       if (operadores && operadores.length > 0) {
            widget.getField("CDOPERADORES").dataSourceFilter[2] = {
                "name": "CDOPERADOR",
                "operator": "IN",
                "value": operadores
            };
                                                                
            widget.getField("CDOPERADORES").reload();
            widget.getField("CDOPERADORES").dataSourceFilter.splice(2);
            templateManager.updateTemplate();
        }
    };
    
    this.controlaCampoEnviaDocConclusao = function (widget){
        
        if(widget.currentRow.IDASSELETRDIGITAL == 'S') {
             widget.getField("IDENVIADOCCONCLUSAO").readOnly = false;
        }
        else {
             widget.getField("IDENVIADOCCONCLUSAO").readOnly  = true;
        }
    };
    
    this.wdgSignatariosPadroesVoltarView = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.currentRow.changed = false;
        });
    };
    
    this.wdgSignatariosPadroesCancelar = function(widget) {
        var isNew = widget.currentRow.__is_new;
        General.widgetCancelForm(widget);
        if (!isNew)
            widget.currentRow.changed = false;
            
        this.carregaDadosOperadores(widget);
    };
    
    this.assinaturaEletronicaOnChange = function(widget) {
        var idasseletrdigital = widget.currentRow.IDASSELETRDIGITAL;
        var fieldEnviaDocOperResp = widget.getField("IDENVIADOCOPERRESPCONCASS");
        var fieldTipoDocumento = widget.getField("NMTIPODOCCERTSIGN");
        
        if (idasseletrdigital == "N") {
            fieldEnviaDocOperResp.readOnly = true;
            fieldEnviaDocOperResp.value("N");
            fieldTipoDocumento.readOnly = true;
            widget.currentRow.NRTIPODOCCERTSIGN = null;
            widget.currentRow.NMTIPODOCCERTSIGN = null;
        }
        else {
            fieldEnviaDocOperResp.readOnly = false;
            fieldTipoDocumento.readOnly = false;
        }
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00002000_Modelo_Proposta_Controller', Sm00002000_Modelo_Proposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00003000_Escopo_Proposta_Controller.js
function Sm00003000_Escopo_Proposta_Controller(General, ScreenService, Sm00003000_Escopo_Proposta_Service, ValidacoesParametrosFaturamentoUtil, Utilities, templateManager) {
    
    /**
     * Salva o novo registro
     * e seta IDATIVO
     * 
     * @param {object} widget widget de 
     * Escopo de proposta para salvar o 
     * novo registro.
    */
    this.saveEscopoProposta = function(widget){
        this.setNull(widget.currentRow);
        
        widget.currentRow.__columns_to_return = ["NRTIPOORCAMENTO", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(response) {
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRTIPOORCAMENTO = response.dataset.columnsToReturn[0].NRTIPOORCAMENTO;
                widget.currentRow.NRORG           = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO      = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO  = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO   = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
        }).catch(function(error){
            
            console.error(error);
            if (error.data){
              ScreenService.showMessage(error.data.error);
            }
            else{
              ScreenService.showMessage(error);
            }
        });
    };
    
    
    /**
     * Seta campos não utilizados
     * com null
     * 
     * @param {object} currentRow 
     * currentRow de Escopo de proposta 
    */
    this.setNull = function(currentRow){
        
        currentRow.IDFATURAORCA = null ;
        currentRow.IDAPURAFATUR = null ;
        currentRow.NRGRUPFATADIC = null;
    };
    
    /**
     * A função "validaServicoEscopo" verifica se o serviço 
     * que está sendo relacionado já está relacionado ao escopo 
     * e salva o serviço no escopo.
     * 
     * * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço relacioando.
     */
     
    this.validaServicoEscopo = function(widget){
        
        if (widget.isValid()) {
            var isNew = widget.currentRow.__is_new;
            if (isNew) {
                var arrayServTabela = this.criaArrayServicosEscopo(widget);

                var servOrca = arrayServTabela.listaValida;
                var escopoOrca = widget.currentRow.NRTIPOORCAMENTO;
               
                Sm00003000_Escopo_Proposta_Service.validaServicoEscopo(servOrca, escopoOrca).then(function(response){
                    if (response.dataset.VALIDASERV.VALIDA == false) {
                        
                        General.callRestEngine("product#/fct_itescopoproposta_widget/save", "DataSet", arrayServTabela.rows).then(function() {
                            General.changeWidgetAcessibility(widget.widgets, true);
                            ScreenService.closeSwipe().then(function() {
                                widget.reload();
                            });
                        }.bind(this))
                        .catch(function(error) {
                            Utilities.showErrorMessage(error);
                        });
                    }
                    else{
                        ScreenService.showMessage('Operação Bloqueada. Alguns dos serviços selecionados já estão inseridos.'); 
                    }
        
               }.bind(this)).catch(function(response){
                   ScreenService.showMessage(response.data.error);
               });
            }
            else {
                General.widgetSave(widget).then(null).catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
        }
    };
    
    
    /**
     * Função para criar o array dos serviços que foram 
     * pegos na função "recebeServicosEscopo";
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço.
     * 
    */
    
    this.criaArrayServicosEscopo = function(widget){
        
        var listaServicosEscopo = widget.currentRow.CDSERVORCA;
        var rows = [];
        var listaValida = [];
        var row;
        
        listaServicosEscopo.forEach(function(servicoRelac , indice){
            row = this.recebeServicosEscopo(widget, indice);
            rows.push(row);
            listaValida.push(row.CDSERVORCA);
        }.bind(this)); 
        
        return {
            "rows": rows,
            "listaValida": listaValida
        };
    };
    
    /**
     * Função para pegar as informações do  array 
     * de serviços na currentRow, e tratá-las em 
     * uma cópia da currentRow para realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do serviço.
     * 
     * @param {int} indice index do serviço no array de 
     * serviços presente na currentRow.
    */
    this.recebeServicosEscopo = function(widget, indice){
        
        var currentRow = angular.copy(widget.currentRow);
        
        currentRow.__is_new = true;

        currentRow.CDSERVORCA = currentRow.CDSERVORCA[indice];
        
        return currentRow;
        
    };
    
    /**
     * Bloqueia ou libera o campo de
     * Modelo de Contrato
     * 
     * @param {object} widget widget de de Escopo de Proposta
    */
    this.idBloqueiaModContrato = function(widget){
        if(widget.getField("NMMODCONTRATO")) {
            var currentRow = widget.currentRow;
            
            if(currentRow.IDGERACONTRATO == "S"){
                widget.getField("NMMODCONTRATO").readOnly = false;
                widget.getField("NMMODCONTRATO").validations = {"required": true};
            } else {
                widget.getField("NMMODCONTRATO").readOnly = true;
                widget.getField("NMMODCONTRATO").validations = {};
                currentRow.NMMODCONTRATO = null;
                currentRow.NRSEQMODCONTRATO = null;
            }
        }
    };
    
    /**
     * Bloqueia ou libera os campos de Tabela de Preços (Produto) ou
     * Tabela de Preços (Serviço) de acordo com o "Tipo de Escopo"
     * 
     * @param {object} widget widget de Escopo de Proposta
    */
    this.idBloqueiaTabPreco = function(widget){
        if(widget.getField("IDTIPOORCA")){        
            var currentRow = widget.currentRow;
            if(currentRow.IDTIPOORCA == "S") {
                widget.getField("DSPRECBASESERV").readOnly = false;
                widget.getField("DSPRECOBASE").readOnly    = true;
                currentRow.DSPRECOBASE    = null;
                currentRow.CDPRECBASEVEND = null;
            }
            else if (currentRow.IDTIPOORCA == "V") {
                widget.getField("DSPRECOBASE").readOnly    = false;
                widget.getField("DSPRECBASESERV").readOnly = true;
                currentRow.DSPRECBASESERV = null;
                currentRow.CDPRECBASESERV = null;
            }
            else { //IDTIPOORCA = "T"
                widget.getField("DSPRECOBASE").readOnly    = false;
                widget.getField("DSPRECBASESERV").readOnly = false;
            }
        }
    };

    this.atualizaNotaFiscal = function(widget, nomeFieldSerie) {
        if (nomeFieldSerie === undefined) {
            nomeFieldSerie = "DSSERIE";
        }

        var fieldSerie = widget.getField(nomeFieldSerie);
        fieldSerie.clearValue();
        fieldSerie.reload().then(function() {
            ScreenService.alertNotification('Atenção: campo "Série da Nota Fiscal" foi recarregado', 5000);
        });
    }
    
      /**
     * Prepara campos para edição
     * 
     * @param {object} widget widget de Escopo de Proposta
    */
    this.PreparaEdicao = function(widget){
        this.idBloqueiaModContrato(widget);
        this.idBloqueiaTabPreco(widget);
        General.widgetEdit(widget);
    };
    
    this.adicionarEscopo = function(widget) {
        General.widgetNewRow(widget);
        this.idBloqueiaModContrato(widget);
        this.idBloqueiaTabPreco(widget);
    };
    
    /**
     * Ao entrar na widget, pega registro, que deve ser único
     * 
     * @param {object} widget widget de  registro único de Parametrizações
    */
    this.WidgetAfterLoadDataSource = function(widget){
        var actAdd = widget.getAction("add");
        var actEdit = widget.getAction("edit");
        
        if (widget.dataSource.data.length > 0) {
            widget.currentRow = widget.dataSource.data[0];
            actAdd.isVisible = false;
            actEdit.isVisible = true;
        }
        else {
            widget.currentRow = {};
            actAdd.isVisible = true;
            actEdit.isVisible = false;
        }
    };
    
    this.novoParametro = function(widget) {
        widget.newRow();
	    widget.template = 'widget/form.html';
	    
	    this.ControlaCamposParamFat(widget);
    };
    
    this.novoParametroServicos = function(widget) {
        widget.newRow();
	    widget.template = 'widget/form.html';
	    
	    this.bloquearFieldsVencimentoItem(widget);
    };
    
    /**
     * Salva o novo registro
     * 
     * @param {object} widget - Widget de registro único de Parametrizações
    */
    this.saveParameterFatura = function(widget, campoPk) {
    
        widget.currentRow.__columns_to_return = [campoPk, "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(success){
            if (success.dataset && success.dataset.columnsToReturn[0]) {
                widget.currentRow.NRSEQFATUESCOPO = success.dataset.columnsToReturn[0].NRSEQFATUESCOPO;
                widget.currentRow.NRORG           = success.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO      = success.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO  = success.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO   = success.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
            widget.view();
            widget.template = "widget/form-view.html";
            this.WidgetAfterLoadDataSource(widget);
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            if (error.data)
                ScreenService.showMessage(error.data.error);
            else
                ScreenService.showMessage(error);
        });
    };
    
    /**
     * Coloca o registro das páginas de parametrização em edição e cria o backup
     * 
     * @param {object} widget widget de registro único de Parametrizações
    */
    
    this.editWidget = function(widget) {
        widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
		
		widget.template = "widget/form.html";
		
	    this.ControlaCamposParamFat(widget);
    };
    
    this.editarParamFatServicos = function(widget) {
        widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
		
		widget.template = "widget/form.html";
		
        this.bloquearFieldsVencimentoItem(widget);
    };

    /**
     * Cancela a edição do registro e volta o registro de Backup
     * 
     * @param {object} widget widget de registro único de Parametrizações
    */
    this.cancelEdit = function(widget) {
    	if (widget.backupRow) {
        	Object.keys(widget.backupRow).forEach(function (key) {
    			widget.currentRow[key] = widget.backupRow[key];
    		});
    	}
    	else {
    	    widget.currentRow = {};
    	}
    	
		widget.view();
		widget.reload();
		widget.template = "widget/form-view.html";
    };
    
    /**
     * Realiza o controle dos campos de vencimento de acordo com o "Tipo de Faturamento"
     * 
     * @param {object} widget widget de Contrato para pegar o field "IDFATURACONTR".
    */
    
    this.bloquearFieldsVencimento = function(widget){
        var campoNomeModPgto        = widget.getField("DSMODPGTO");
        var campoQtdParcelas        = widget.getField("QTPARCELASFATU");
        var campoDiaVencParcelaFatu = widget.getField("NRDIAVENCPARCELASFATU");
        var campoTpCompetencia      = widget.getField("IDQTDIASCOMPCON");
        var campoTpFchCompetencia   = widget.getField("IDTPCOMPETCONT");
        var campoTpEmissaoNF        = widget.getField("IDTPDTEMISSNF");
        var campoTpVencNF           = widget.getField("IDTPDTVENC");
        var campoTpPeriodoInc       = widget.getField("IDTPFATUINCOMP");
        var campoFatItem            = widget.getField("IDSOMAITGRUPFAT");
        var campoTpFat              = widget.getField("IDSOMAFATUCONT");
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;
        
        campoFatItem.defaultValue           = "N";
        widget.currentRow.IDSOMAITGRUPFAT   = "N";
        campoTpFat.defaultValue             = "N";
        widget.currentRow.IDSOMAFATUCONT    = "N";
        
        /*widget.currentRow.NRQTDIASCOMPCON           = null;
        widget.getField("NRQTDIASCOMPCON").value(null);
        widget.getField("NRQTDIASCOMPCON").readOnly = true;*/
        this.bloquearFieldDiasComp(widget);
        
        /*widget.currentRow.NRDIACOMPETCONT           = null;
        widget.getField("NRDIACOMPETCONT").value(null);
        widget.getField("NRDIACOMPETCONT").readOnly = true;*/
        this.bloquearFieldDiasFecha(widget);
        
        /*widget.currentRow.NRDIADTEMISSNF            = null;
        widget.getField("NRDIADTEMISSNF").value(null);
        widget.getField("NRDIADTEMISSNF").readOnly  = true;*/
        this.bloquearFieldDiasEmissao(widget);
        
        /*widget.currentRow.NRDIATPDTVENC           = null;
        widget.getField("NRDIATPDTVENC").value(null);
        widget.getField("NRDIATPDTVENC").readOnly = true;*/
        this.bloquearFieldDiasVenc(widget);
        

        if (tipofaturamento == "A") {
            
            campoNomeModPgto.readOnly           = true;
            campoNomeModPgto.validations        = {};
            widget.currentRow.CDMODPGTO         = null;
            campoNomeModPgto.value(null);
            
            campoQtdParcelas.readOnly           = true;
            campoQtdParcelas.validations        = {};
            campoQtdParcelas.value(null);
            
            campoDiaVencParcelaFatu.readOnly    = true;
            campoDiaVencParcelaFatu.value(null);
            campoDiaVencParcelaFatu.validations = {};
            
            campoTpCompetencia.readOnly         = false;
            campoTpCompetencia.validations      = { "required": true };
            
            campoTpFchCompetencia.readOnly      = false;
            campoTpFchCompetencia.validations   = { "required": true };
            
            campoTpEmissaoNF.readOnly           = false;
            campoTpEmissaoNF.validations        = { "required": true };
            
            campoTpVencNF.readOnly              = false;
            campoTpVencNF.validations           = { "required": true };
            
            campoTpPeriodoInc.readOnly          = false;
            campoTpPeriodoInc.validations       = { "required": true };
            
        } else {
            if(tipofaturamento === "O"){
                
                campoNomeModPgto.readOnly           = false;
                campoNomeModPgto.validations        = { "required": true };
                
                campoQtdParcelas.readOnly           = true;
                campoQtdParcelas.validations        = {};
                campoQtdParcelas.value(null);
                
                campoDiaVencParcelaFatu.readOnly    = true;
                campoDiaVencParcelaFatu.validations = {};
                campoDiaVencParcelaFatu.value(null);
                
                campoTpCompetencia.readOnly         = true;
                campoTpCompetencia.validations      = {};
                campoTpCompetencia.value(null);
                
                campoTpFchCompetencia.readOnly      = true;
                campoTpFchCompetencia.validations   = {};
                campoTpFchCompetencia.value(null);
                
                campoTpEmissaoNF.readOnly           = true;
                campoTpEmissaoNF.validations        = {};
                campoTpEmissaoNF.value(null);
                
                campoTpVencNF.readOnly              = true;
                campoTpVencNF.validations           = {};
                campoTpVencNF.value(null);
                
                campoTpPeriodoInc.readOnly          = true;
                campoTpPeriodoInc.validations       = {};
                campoTpPeriodoInc.value(null);
                
            } else {
                if(tipofaturamento === "X"){
                    
                    campoNomeModPgto.readOnly           = true;
                    campoNomeModPgto.validations        = {};
                    widget.currentRow.CDMODPGTO         = null;
                    campoNomeModPgto.value(null);
                    
                    campoQtdParcelas.readOnly           = true;
                    campoQtdParcelas.validations        = {};
                    campoQtdParcelas.value(null);
                    
                    campoDiaVencParcelaFatu.readOnly    = true;
                    campoDiaVencParcelaFatu.validations = {};
                    campoDiaVencParcelaFatu.value(null);
                    
                    campoTpCompetencia.readOnly         = false;
                    campoTpCompetencia.validations      = { "required": true };
                    
                    campoTpFchCompetencia.readOnly      = false;
                    campoTpFchCompetencia.validations   = { "required": true };
                    
                    campoTpEmissaoNF.readOnly           = false;
                    campoTpEmissaoNF.validations        = { "required": true };
                    
                    campoTpVencNF.readOnly              = false;
                    campoTpVencNF.validations           = { "required": true };
                    
                    campoTpPeriodoInc.readOnly          = false;
                    campoTpPeriodoInc.validations       = { "required": true };
                        
                    campoFatItem.defaultValue            = "S";
                    widget.currentRow.IDSOMAITGRUPFAT   = "S";
                    campoTpFat.defaultValue              = "S";
                    widget.currentRow.IDSOMAFATUCONT    = "S";
                    
                } else {
                    if(tipofaturamento === "P"){
                        campoNomeModPgto.readOnly           = true;
                        campoNomeModPgto.validations        = {};
                        widget.currentRow.CDMODPGTO         = null;
                        campoNomeModPgto.value(null);
                        
                        campoQtdParcelas.readOnly           = false;
                        campoQtdParcelas.validations        = { "required": true};
                        
                        campoDiaVencParcelaFatu.readOnly    = false;
                        campoDiaVencParcelaFatu.validations = { "required": true};
                        
                        campoTpCompetencia.readOnly         = true;
                        campoTpCompetencia.validations      = {};
                        campoTpCompetencia.value(null);
                        
                        campoTpFchCompetencia.readOnly      = true;
                        campoTpFchCompetencia.validations   = {};
                        campoTpFchCompetencia.value(null);
                        
                        campoTpEmissaoNF.readOnly           = true;
                        campoTpEmissaoNF.validations        = {};
                        campoTpEmissaoNF.value(null);
                        
                        campoTpVencNF.readOnly              = true;
                        campoTpVencNF.validations           = {};
                        campoTpVencNF.value(null);
                        
                        campoTpPeriodoInc.readOnly          = true;
                        campoTpPeriodoInc.validations       = {};
                        campoTpPeriodoInc.value(null);
                    }
                }
            }
        }
    };
    
    /**
     * Realiza o controle dos campos de vencimento de acordo com o "Tipo de Faturamento"
     * 
     * @param {object} widget widget de Contrato para pegar o field "IDFATURACONTR".
     */
    
    this.bloquearFieldsVencimentoItem = function(widget){
        
        var tipofaturamento = widget.currentRow.IDFATURACONTR;
        
        // Não Periódico: Desbloqueia a Modalidade de Pagamento
        if (tipofaturamento == 'O') {
            
            widget.getField('DSMODPGTO').readOnly = false;
            widget.getField('DSMODPGTO').validations = {"required": true};

        }
        // Periódico / Apuração: Bloqueia a Modalidade de Pagamento
        else{
    
            widget.getField('DSMODPGTO').readOnly = true;
            widget.getField('DSMODPGTO').value(null);
            widget.getField('CDMODPGTO').value(null);
            widget.getField('CDMODPGTO').validations = {};
        }
    };
    
    
     /**
     * Realiza o controle do campo "Dia Para Competência do Contrato" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDQTDIASCOMPCON".
    */
    
    this.bloquearFieldDiasComp = function(widget){
        
        var tipoCompetencia = widget.getField('IDQTDIASCOMPCON').value();
        
        /* Desbloqueia o campo "Dia Para Competência do Contrato" */
        if (tipoCompetencia == "2") {
            
            widget.getField('NRQTDIASCOMPCON').readOnly = false;
            widget.getField('NRQTDIASCOMPCON').validations = {"required": true};
        } 
        /* Bloqueia o campo "Dia Para Competência do Contrato" */
        else {

            widget.getField('NRQTDIASCOMPCON').readOnly = true;
            widget.getField('NRQTDIASCOMPCON').value(null);
            widget.getField('NRQTDIASCOMPCON').validations = {};
        }
    };
   
   
    /**
     * Realiza o controle do campo "Dia Para Competência do Contrato" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDQTDIASCOMPCON".
    */
    
    this.ValidaFieldDiasComp = function(widget){
        
        var dia = widget.getField('NRQTDIASCOMPCON').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRQTDIASCOMPCON = null;
        }

    };
   
        /**
     * Realiza o controle do campo "Dia Para Fechamento da Competência" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPCOMPETCONT".
    */
    
    this.bloquearFieldDiasFecha = function(widget){
        
        var fechaCompetencia = widget.getField('IDTPCOMPETCONT').value();
        
        /* Bloqueia o campo "Dia Para Fechamento da Competência" */
        if (!fechaCompetencia || fechaCompetencia == "2") {
            
            widget.getField('NRDIACOMPETCONT').readOnly = true;
            widget.getField('NRDIACOMPETCONT').value(null);
            widget.getField('NRDIACOMPETCONT').validations = {};
        } 
        /* Desbloqueia o campo "Dia Para Fechamento da Competência" */
        else {

            widget.getField('NRDIACOMPETCONT').readOnly = false;
            widget.getField('NRDIACOMPETCONT').validations = {"required": true};
        }
    };
   
   
   /**
     * Realiza o controle do campo "Dia Para Fechamento da Competência" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPCOMPETCONT".
    */
    
    this.ValidaFieldDiasFecha = function(widget){
        
        var dia = widget.getField('NRDIACOMPETCONT').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIACOMPETCONT = null;
        }

    };
   
    /**
     * Realiza o controle do campo "Dia Para Emissão da NF" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTEMISSNF".
    */
    
    this.bloquearFieldDiasEmissao = function(widget){
        
        var emissao = widget.getField('IDTPDTEMISSNF').value();
        
        /* Bloqueia o campo "Dia Para Emissão da NF" */
        if (!emissao || emissao == "9") {
            
            widget.getField('NRDIADTEMISSNF').readOnly = true;
            widget.getField('NRDIADTEMISSNF').value(null);
            widget.getField('NRDIADTEMISSNF').validations = {};

        } 
        /* Desbloqueia o campo "Dia Para Emissão da NF" */
        else {

            widget.getField('NRDIADTEMISSNF').readOnly = false;
            widget.getField('NRDIADTEMISSNF').validations = {"required": true};
        }
    };
    
    /**
     * Realiza o controle do campo "Dia Para Emissão da NF" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTEMISSNF".
    */
    
    this.ValidaFielDiasEmissao = function(widget){
        
        var dia = widget.getField('NRDIADTEMISSNF').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIADTEMISSNF = null;
        }

    };
    
    /**
     * Realiza o controle do campo "Dia Para Vencimento da Parcela" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTVENC".
    */
    
    this.bloquearFieldDiasVenc = function(widget){
        
        var vencimento = widget.getField('IDTPDTVENC').value();
        
        /* Bloqueia o campo "Dia Para Vencimento da Parcela" */
        if (!vencimento || vencimento == "5") {
            
            widget.getField('NRDIATPDTVENC').readOnly = true;
            widget.getField('NRDIATPDTVENC').value(null);
            widget.getField('NRDIATPDTVENC').validations = {};
            
        } 
        /* Desbloqueia o campo "Dia Para Vencimento da Parcela" */
        else {

            widget.getField('NRDIATPDTVENC').readOnly = false;
            widget.getField('NRDIATPDTVENC').validations = {"required": true};
        }
    };
   
    /**
     * Realiza o controle do campo "Dia Para Vencimento da Parcela" 
     * 
     * @param {object} widget widget de 
     * Faturamento para pegar o field "IDTPDTVENC".
    */
    
    this.ValidaFielDiasVenc = function(widget){
        
        var dia = widget.getField('NRDIATPDTVENC').value();
        
        /* Aplica Max e Min Value*/
        if (dia < 0 || dia > 31) {    

            widget.currentRow.NRDIATPDTVENC = null;
        }

    };
    
    this.ControlaCamposParamFat = function(widget){
        this.bloquearFieldsVencimento(widget);
        /*this.bloquearFieldDiasComp(widget);
        this.bloquearFieldDiasFecha(widget);
        this.bloquearFieldDiasEmissao(widget);
        this.bloquearFieldDiasVenc(widget);*/
    };
    
    this.parametrosOnEnterTab = function(widget) {
        widget.list();
        widget.reload().then(function() {
            widget.view();
        });
    };
    
    this.wdgParametrosFaturamentoItemNewRow = function(wdgParametrosFaturamentoItem) {
        Utilities.formWidgetNewRow(wdgParametrosFaturamentoItem);
        General.changeWidgetAcessibility(getWidgetsParaControlarAcessibilidadeFaturamentoItem(wdgParametrosFaturamentoItem), false);

        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem, 'wdgCompetenciaServico');
    };

    this.wdgParametrosFaturamentoItemEdit = function(wdgParametrosFaturamentoItem) {
        Utilities.formWidgetEdit(wdgParametrosFaturamentoItem);
        General.changeWidgetAcessibility(getWidgetsParaControlarAcessibilidadeFaturamentoItem(wdgParametrosFaturamentoItem), false);

        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem, 'wdgCompetenciaServico');
    };

    this.wdgParametrosFaturamentoItemCancel = function(wdgParametrosFaturamentoItem) {
        Utilities.formWidgetCancel(wdgParametrosFaturamentoItem);
        General.changeWidgetAcessibility(getWidgetsParaControlarAcessibilidadeFaturamentoItem(wdgParametrosFaturamentoItem), true);
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(wdgParametrosFaturamentoItem, 'wdgCompetenciaServico');
    };

    var getWidgetsParaControlarAcessibilidadeFaturamentoItem = function(widget) {
        var widgetsToDeactivate = widget.parent.widgets;
        
        return widgetsToDeactivate.filter(function(subwidget) {
            return subwidget.name != widget.name;
        }).concat(widget.parent);
    };

    this.wdgParametrosFaturamentoItemSave = function(wdgParametrosFaturamentoItem) {
        if (wdgParametrosFaturamentoItem.isValid()) {
            Utilities.validarCamposDuplicados(wdgParametrosFaturamentoItem, ["NRPARAMFATITESC"], ["NRTIPOORCAMENTO", "NRITESCOPOPROPOSTA"])
            .then(function(validacao) {
                if (!validacao) {
                    ScreenService.showMessage("Operação bloqueada. <br/>Já existe uma parametrização de faturamento cadastrada para este item.");
                }
                else {
                    Utilities.formWidgetSaveAndSetColumnsToReturn(wdgParametrosFaturamentoItem).then(function() {
                        ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoOnEnter(wdgParametrosFaturamentoItem, "wdgCompetenciaServico");
                    });
                }
            });
        }
    };

    this.wdgTabelaPrecosSave = function(wdgTabelaPrecosItem) {
        if (wdgTabelaPrecosItem.isValid()) {
            Utilities.validarCamposDuplicados(wdgTabelaPrecosItem,["NRTABPRECESCOPOFAT"],["NRTIPOORCAMENTO", "NRMOEDA"])
            .then(function(podeSalvar) {
                var idPadraoOld = null;
                if (wdgTabelaPrecosItem.backupRow) {
                    idPadraoOld = wdgTabelaPrecosItem.backupRow.IDPADRAO;
                }
                var wdgEscopo = templateManager.container.getWidget("grid_escopo");
                if(!podeSalvar || wdgTabelaPrecosItem.currentRow.IDPADRAO == 'S' && wdgEscopo.currentRow.CDPRECBASESERV != null && idPadraoOld != "S"){
                    ScreenService.showMessage("Operação bloqueada. <br/><br/>Já existe uma tabela de preço cadastrada para a moeda informada.");
                } else {
                    if(wdgTabelaPrecosItem.currentRow.NRTABPRECESCOPOFAT == null) {
                        wdgTabelaPrecosItem.currentRow.__is_new = true;
                    }
                    
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgTabelaPrecosItem);
                    
                   
                    if (wdgTabelaPrecosItem.currentRow.IDPADRAO == "N" && wdgTabelaPrecosItem.backupRow && wdgTabelaPrecosItem.backupRow.IDPADRAO == "S") {
                        wdgEscopo.currentRow.CDPRECBASESERV = null;
                        wdgEscopo.currentRow.CDPRECBASEVEND = null;
                        wdgEscopo.currentRow.changed = true;
                        wdgEscopo.saveRow();
                    } 
                    
                    if (wdgTabelaPrecosItem.currentRow.IDPADRAO == "S") {
                        wdgEscopo.currentRow.CDPRECBASESERV = wdgTabelaPrecosItem.currentRow.CDPRECBASESERV;
                        wdgEscopo.currentRow.CDPRECBASEVEND = wdgTabelaPrecosItem.currentRow.CDPRECBASEVEND;
                        wdgEscopo.currentRow.changed = true;
                        wdgEscopo.saveRow();
                    }    
                  
                }
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        };
    };

    this.wdgTabelaPrecosDelete = function(wdgTabelaPrecosItem) {
        var idpadrao = wdgTabelaPrecosItem.currentRow.IDPADRAO;
        General.deleteCrud(wdgTabelaPrecosItem).then(function() {
            if (idpadrao == 'S') {
                var wdgEscopo = templateManager.container.getWidget("grid_escopo");
                wdgEscopo.currentRow.CDPRECBASESERV = null;
                wdgEscopo.currentRow.CDPRECBASEVEND = null;
                wdgEscopo.currentRow.changed = true;
                wdgEscopo.saveRow().then(function() {
                    wdgTabelaPrecosItem.reload();
                });
            }
        });
    }
    

    this.salvarDescricaoEscopo = function(wdgDescricaoEscopo) {
        if (wdgDescricaoEscopo.isValid()) {
            var camposValidacao = [
                "NRTIPOORCAMENTO",
                "IDTPFATURA",
                "CDFILIALFATURA",
                "NRPARCFATURA",
                "CDTIPOOPERFATURA",
                "CDSERIENF",
                "CDEMPRESA",
                "CDINSCESTA",
                "CDINSCMUNI",
                "IDPREVCOBRFUT"
            ];

            Utilities.validarCamposDuplicados(wdgDescricaoEscopo, ["NRDESCAGRUPESCOPO"], camposValidacao)
            .then(function(podeSalvar) {
                if (!podeSalvar) {
                    ScreenService.showMessage("Operação bloqueada. <br/><br/>Já existe uma descrição cadastrada para estas parametrizações.");
                }
                else {
                    Utilities.widgetSaveAndSetColumnsToReturn(wdgDescricaoEscopo);
                }
            })
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
        else {
            ScreenService.showMessage("Preencha todos os campos corretamente!");
        }
    };
    
    /**
     * Preenchee o campo de percentual com o valor restante
     * 
     * @params{Object} widget - widget de índice
    **/    
    this.completaPercentual = function(widget){
        General.widgetNewRow(widget);
        var params = {
           "NRTIPOORCAMENTO" : widget.currentRow.NRTIPOORCAMENTO
        };
        
        General.callRestEngine('product#/fct_paramfatescopo_widget/completaPercentual', "Row", params).then(function(response){  
            var percentual = response.dataset.completaPercentual.PERCENTUAL;
            widget.currentRow.NRPERCENTINDICE = percentual;
        });
    };
    
    /**
     * Valida o percentual dos índices financeiros
     * 
     * @params{Object} widget - widget de índices financeiros
    **/
    this.validaSaveIndFinanceiro = function(widget) {
        if(!widget.currentRow.__is_new) {
            this.salvarIndiceFinanceiro(widget);
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["CDINDICE", "NRTIPOORCAMENTO"]).then(function(response) {
                if(!response)
                    ScreenService.showMessage("Operação bloqueada!<br><br>O índice selecionado já está inserido.");
                else
                    this.salvarIndiceFinanceiro(widget);
            }.bind(this))
            .catch(function(error) {
                Utilities.showErrorMessage(error);
            });
        }
    };
    
    /**
     * Salva um índice
     * 
     * @params{object} widget - widget de índice financeiro 
    **/
    this.salvarIndiceFinanceiro = function(widget) {
        var params = {
           "NRTIPOORCAMENTO" : widget.currentRow.NRTIPOORCAMENTO,
           "NRPERCENTINDICE" : widget.currentRow.NRPERCENTINDICE,
           "CDINDICE"        : (widget.currentRow.__is_new) ? null : widget.currentRow.CDINDICE
        };
        
        General.callRestEngine('product#/fct_paramfatescopo_widget/validaPercentual', "Row", params).then(function(response) {  
            var percentual = response.dataset.validaPercentual.PERCENTUAL;
            if (percentual > 100) {
                ScreenService.showMessage("Operação Bloqueada!<br><br>A soma dos percentuais de utilização dos índices financeiros ultrapassou o total de 100%.");
            } else {
                General.widgetSave(widget).then(function(response){
                    widget.reload();
                    
                    if(percentual < 100)
                        ScreenService.showMessage("Atenção! <br><br> A soma dos percentuais de utilização dos índices financeiros não está totalizando os 100% necessários para realização do reajuste dos contratos.");
                });
            }
        }.bind(this));
        
    };
    
    /**
     * Chama as funções necessárias para o cancelamento da edição e insersão 
     * da widget de parametros de faturamento do escopo
     * 
     * @params{Object} widget - widget de parâmetros de faturamento do escopo
    **/
    this.cancelarEdicaoParametrosDeFaturamento = function(widget) {
        ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoCancel(widget, 'grid_escopo', 'wdgParametrosFaturamentoTabbed'); 
        ValidacoesParametrosFaturamentoUtil.controlarCamposTipoFaturamento(widget, 'wdgParametrosFaturamento');  
        
        this.habilitarCamposDeCalculoDtSaida(widget);
        templateManager.container.getWidget('wdgParametrosFaturamentoTabbed').activate(); 
    };
    
    /**
     * Valida o tipo de cálculo da data de saída e habilita o campo 
     * do parametro de cálculo
     * 
     * @params{Object} widget - widget parametros do escopo
    **/
    this.habilitarCamposDeCalculoDtSaida = function(widget) {
        var fieldNrCalculo = widget.getField("NRCALCULODTSAIDANF");
        var idtpcalculo    = widget.currentRow.IDTPCALCDTSAIDANF;
        
        if(!idtpcalculo || idtpcalculo == 1) {
            fieldNrCalculo.readOnly              = true;
            fieldNrCalculo.label                 = "Dia fixo / Quantidade de dias para data de saída";
            fieldNrCalculo.validations           = {};
            widget.currentRow.NRCALCULODTSAIDANF = null;
        } else {
            var nmlabel = ([2, 3].indexOf(idtpcalculo) != -1) ? "Dia fixo para data de saída" : "Quantidade de dias para data de saída";
            
            fieldNrCalculo.readOnly    = false;
            fieldNrCalculo.label       = nmlabel;
            fieldNrCalculo.validations = { "required": true };
        }
        
        templateManager.updateTemplate();
    };
    
    /**
     * Valida se o valor digitado se aplica ao tipo de calculo escolhido
     * 
     * @params{Object} widget - widget parametros de faturamento 
    **/
    this.validarParametrosCalculoDtSaida = function(widget) {
        if([2, 3].indexOf(widget.currentRow.IDTPCALCDTSAIDANF) != -1) {
            if(widget.currentRow.NRCALCULODTSAIDANF > 31) {
                ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível atribuir um dia fixo maior que 31 para o cálculo da data de saída da nota fiscal.");
                widget.currentRow.NRCALCULODTSAIDANF = 0;
            }
        }
    };
    
    /**
     * Valida se o escopo participa de um agupamento e bloqueia a alteração
     * dos parametros de faturamento.
     * 
     * @params{Object} widget - widget de parametros de faturamento do contrato
    **/
    this.salvarDadosGeraisParamFaturamento = function(widget) {
        var escopoAgrupado = templateManager.container.getWidget("grid_escopo").currentRow.ESCOPOAGRUPADO;
        if(escopoAgrupado != "S") {
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoSave(widget, 'grid_escopo', 'wdgParametrosFaturamentoTabbed', 'wdgParametrosFaturamento');
        } else {
            var camposFatura = ["CDTIPOOPERFATURA", "CDSERIENF", "CDEMPRESA", "CDINSCESTA", "CDINSCMUNI", "CDFILIALFATURA", "IDTPCALCDTSAIDANF"];
            var possuiAlteracaoParams = false;
            if (widget.backupRow) {
                camposFatura.forEach(function(campo) {
                    if(widget.currentRow[campo] != widget.backupRow[campo]) {
                        widget.currentRow[campo] = widget.backupRow[campo];
                        possuiAlteracaoParams    = true;
                    }
                });
            }
            
            if(possuiAlteracaoParams)
                ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível alterar as informações de Tipo de Operação, Série da Nota Fiscal, Unidade para Faturamento e Tipo de Cálculo para Data de Saída dos escopos que participam de agrupamentos.");
            else 
                ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoSave(widget, 'grid_escopo', 'wdgParametrosFaturamentoTabbed', 'wdgParametrosFaturamento');
        }
    };
    
    /**
     * Valida se o escopo participa de um agupamento e bloqueia a alteração
     * da competencia.
     * 
     * @params{Object} widget - widget de competencia
    **/
    this.salvarParamsCompetenciaEscopo = function(widget) {
        var escopoAgrupado = templateManager.container.getWidget("grid_escopo").currentRow.ESCOPOAGRUPADO;
        if(escopoAgrupado != "S") {
            ValidacoesParametrosFaturamentoUtil.wdgParametrosFaturamentoSave(widget, 'NRTIPOORCAMENTO');
        } else {
            if(widget.currentRow.__is_new) {
                ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível adicionar informações de competência para escopos que participam de agrupamentos.");
            } else {
                var camposAlterados = false;
                Object.keys(widget.backupRow).forEach(function (key) {
    				if(widget.backupRow[key] && widget.currentRow[key] != widget.backupRow[key]) {
    				    widget.currentRow[key] = widget.backupRow[key];
    				    camposAlterados        = true;
    				}
    			});
    			
    			if(camposAlterados)
                    ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível alterar as informações de competência dos escopos que participam de agrupamentos.");
                else 
                    ValidacoesParametrosFaturamentoUtil.wdgParametrosFaturamentoSave(widget, 'NRTIPOORCAMENTO');
            }
        }
    };
    
    /**
     * Valida se o escopo participa de um agupamento e bloqueia a exclusao
     * da competência
     * 
     * @params{Object} widget - widget de competencia
    **/
    this.excluirCompetenciaEscopo = function(widget) {
        if(widget.currentRow.ESCOPOAGRUPADO != "S")
            General.deleteCrud(widget);
        else 
            ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível deletar as informações de competência dos escopos que participam de agrupamentos.");
    };
    
    /**
     * Valida se o escopo participa de um agupamento e bloqueia a exclusao
     * dos seus parâmetros de faturamento.
     * 
     * @params{Object} widget - widget de parametros de faturamento do contrato
    **/
    this.excluirDadosGeraisParamsFatEscopo = function(widget) {
        if(widget.currentRow.ESCOPOAGRUPADO != "S")
            ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoDelete(widget, 'wdgParametrosFaturamento');
        else 
            ScreenService.showMessage("Operação bloqueada!<br><br>Não é possível deletar os parâmetros de faturamento dos escopos que participam de agrupamentos.");
    };
    
    this.adicionarServicoEscopo = function(wdgServicoEscopo) {
        var fieldCodigoServ = wdgServicoEscopo.getField("CDSERVORCA");
        var fieldNomeServ = wdgServicoEscopo.getField("NMSERVORCA");
        fieldCodigoServ.showOnForm = true;
        fieldCodigoServ.validations = {"required": true};
        fieldNomeServ.showOnForm = false;
        General.widgetNewRow(wdgServicoEscopo);
    };
    
    this.editarServicoEscopo = function(wdgServicoEscopo) {
        var fieldCodigoServ = wdgServicoEscopo.getField("CDSERVORCA");
        var fieldNomeServ = wdgServicoEscopo.getField("NMSERVORCA");
        fieldCodigoServ.showOnForm = false;
        fieldCodigoServ.validations = {};
        fieldNomeServ.showOnForm = true;
        General.widgetEdit(wdgServicoEscopo);
    };
    
    this.alterarVisibilidadeNaturezaOperacao = function(widget) {
        if (widget.currentRow.IDTIPONATUOPER == 'F') {
            widget.getField("IDNATUOPER").readOnly = false;
            widget.getField("IDNATUOPER").validations = {"required": true};
        }
        else {
            widget.getField("IDNATUOPER").readOnly = true;
            widget.getField("IDNATUOPER").validations = {};
            widget.getField("IDNATUOPER").value(null);
        }
    };
    
    this.editarParamFatEscopo = function(wdgCapaParametrosFaturamento) {
        ValidacoesParametrosFaturamentoUtil.wdgCapaParametrosFaturamentoEdit(wdgCapaParametrosFaturamento, 'grid_escopo', 'wdgParametrosFaturamentoTabbed', 'wdgParametrosFaturamento');
        this.habilitarCamposDeCalculoDtSaida(wdgCapaParametrosFaturamento);
        this.alterarVisibilidadeNaturezaOperacao(wdgCapaParametrosFaturamento);
    };
}

Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00003000_Escopo_Proposta_Controller', Sm00003000_Escopo_Proposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00013000_GeracaoProposta_Controller.js
function Sm00013000_GeracaoProposta_Controller(General, ScreenService, templateManager, Utilities, Sm00001000_LancamentoProposta_Service) {
    
    var getWidgetGeracaoProposta = function() {
        return templateManager.container.getWidget("wdgGeracaoProposta");
    };
    
    var getWidgetJsonIntegracao = function() {
        return templateManager.container.getWidget("wdgJson");
    };
    
    this.salvarWdgGeracaoProposta = function(wdgGeracaoProposta) {
        if (wdgGeracaoProposta.currentRow.__is_new) {
            wdgGeracaoProposta.currentRow.MUDOU_ESCOPOS = true;
        }
        else {
            var newEscopos = wdgGeracaoProposta.currentRow.NRTIPOORCAMENTO;
            var bkpEscopos = wdgGeracaoProposta.backupRow.NRTIPOORCAMENTO;
            if (!Utilities.arraysSaoIguais(newEscopos, bkpEscopos)) {
                wdgGeracaoProposta.currentRow.MUDOU_ESCOPOS = true;
            }
        }
        
        Utilities.widgetSaveAndSetColumnsToReturn(wdgGeracaoProposta).then(function() {
            wdgGeracaoProposta.reload();
        });
    };
    
    this.fieldModeloOnChange = function() {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        wdgGeracaoProposta.getField("NRTIPOORCAMENTO").clearValue();
        wdgGeracaoProposta.getField("NRTIPOORCAMENTO").reload();
    };
    
    this.editarWdgGeracaoProposta = function(wdgGeracaoProposta) {
        General.widgetEdit(wdgGeracaoProposta);
        wdgGeracaoProposta.getField("NRTIPOORCAMENTO").reload();
    };
    
    this.wdgTipoMedidaVariavelOnEnterTab = function(wdgTipoMedidaVariavel) {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        var nrgeracaoproposta = wdgGeracaoProposta.currentRow.NRGERACAOPROPOSTA;
        var pos = Utilities.getFilterPosition(wdgTipoMedidaVariavel.dataSourceFilter, "NRGERACAOPROPOSTA");
        wdgTipoMedidaVariavel.dataSourceFilter[pos].value = nrgeracaoproposta;
        Utilities.carregarWidgetOnEnterTab(wdgTipoMedidaVariavel);
    };
    
    this.editarWdgElementos = function(wdgElementos) {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        var elementosSalvos = [];
        if (wdgElementos.dataSource.data)
            elementosSalvos = Util.clone(wdgElementos.dataSource.data, []);
            
        var params = [
            {
                "name": "NRORCAMENTOMODELO",
                "operator": "=",
                "value": wdgGeracaoProposta.currentRow.NRORCAMENTOMODELO
            },
            {
                "name": "NRTIPOORCAMENTO",
                "operator": "IN",
                "value": wdgGeracaoProposta.currentRow.NRTIPOORCAMENTO
            }
        ];
        General.callRestEngine("product#/crud/composicao_elemento_geracao_proposta_edit", 'FilterData', params).then(function(response){
            wdgElementos.dataSource.data = response.dataset.composicao_elemento_geracao_proposta_edit;
            //wdgElementos.edit();
            wdgElementos.editing = true;
            wdgElementos._changeMode(ViewMode.EDIT);
            
            wdgElementos.floatingControl.pdfAction = false;
			wdgElementos.floatingControl.xlsAction = false;
			wdgElementos.floatingControl.csvAction = false;
			wdgElementos.floatingControl.searchAction = false;
			wdgElementos.floatingControl.refreshAction = false;
            
            wdgElementos.showCheckbox = true;
            
            if (elementosSalvos.length > 0)
                this.marcarElementosEscolhidos(wdgElementos, elementosSalvos);
            
            //this.controlarAcessibilidadeTabbed("edit", false, wdgElementos);
        }.bind(this));
    };
    
    this.cancelarWdgElementos = function(wdgElementos) {
        //widget.dataSourceFilter[0].operator = "=";
        //widget.dataSourceFilter[0].value    = "S";

        wdgElementos.list();
        wdgElementos.reload().then(function() {
            wdgElementos.showCheckbox = false;
            wdgElementos.unSelectAll();
            //this.controlarAcessibilidadeTabbed("view", true, widget);
            wdgElementos.activate();
            
            wdgElementos.floatingControl.pdfAction = true;
			wdgElementos.floatingControl.xlsAction = true;
			wdgElementos.floatingControl.csvAction = true;
			wdgElementos.floatingControl.searchAction = true;
			wdgElementos.floatingControl.refreshAction = true;
        }.bind(this));
    };
    
    this.marcarElementosEscolhidos = function(widget, elementos) {
        widget.dataSource.data.forEach(function(row) {
            if (row.TIPO == 'C') row.OLDCOMPPROPOSTA = 'N';
            elementos.forEach(function(elemento) {
                if (elemento["IDELEMENTO"] == row["IDELEMENTO"] && elemento["NRCOMPSERVICO"] == row["NRCOMPSERVICO"]) {
                    if (row.TIPO == 'C') row.OLDCOMPPROPOSTA = 'S';
                    row.__isSelected = true;
                    widget.dataSource.addCheckedRows(row);
                }
            });
        });
    };
    
    this.salvarWdgElementos = function(wdgElementos) {
        Sm00001000_LancamentoProposta_Service.validarObrigatoriedadeItensComposicao(wdgElementos.dataSource.data).then(function(msg) {
            if (msg) {
                ScreenService.showMessage(msg);
            }
            else {
                var elementos   = [];
                var composicoes = [];
                var wdgGeracaoProposta = getWidgetGeracaoProposta();
                var nrgeracaoproposta = wdgGeracaoProposta.currentRow.NRGERACAOPROPOSTA;
                var nrtipoorcamento = wdgGeracaoProposta.currentRow.NRTIPOORCAMENTO;
                
                wdgElementos.dataSource.data.forEach(function(row) {
                    if(row.__isSelected) {
                        if (row.TIPO == "E") {
                            elementos.push({
                                "NRCOMPSERVICO"    : row.NRCOMPSERVICO,
                                "NRTIPOORCAMENTO"  : row.NRTIPOORCAMENTO,
                                "NRITCOMPSERVICO"  : row.NRITCOMPSERVICO,
                                "IDELEMENTO"       : row.IDELEMENTO,
                                "IDELEMENTOPAI"    : row.IDELEMENTOPAI,
                                "IDTIPOELEMENTO"   : row.IDTIPOELEMENTO,
                                "IDSERVOBRIGCOMP"  : row.IDSERVOBRIGCOMP,
                                "IDESCOLHIDO"      : (row.__isSelected) ? "S" : "N",
                                "NRORG"            : row.NRORG
                            });
                        } else {
                            composicoes.push({
                                "NRCOMPSERVICO"    : row.NRCOMPSERVICO,
                                "NRCOMPSERVICOPAI" : row.IDARVOREPAI,
                                "IDCOMPOBRIGATORIA": row.IDSERVOBRIGCOMP,
                                "IDESCOLHIDO"      : (row.__isSelected) ? "S" : "N",
                                "NRORG"            : row.NRORG
                            });
                        }
                    }
                }.bind(this));
        
                var params = {
                    "NRGERACAOPROPOSTA": nrgeracaoproposta,
                    "ELEMENTOS"        : elementos,
                    "COMPOSICOES"      : composicoes,
                    "ESCOPOS"          : nrtipoorcamento
                };
                
                General.callRestEngine("product#/fct_geracaoproposta_modelo/inserirComposicoesElementosMedidas", "Row", params).then(function() {
                    this.cancelarWdgElementos(wdgElementos);
                }.bind(this))
                .catch(function(error) {
                    Utilities.showErrorMessage(error);
                });
            }
        }.bind(this));
    };
    
    /*this.getElementoPaiServico = function(row){
        if (row.__isSelected && row._parent && row._parent.TIPO == 'C')
            return null;
        return row.IDARVORE;
    };*/
    
    this.salvarTipoMedidaVariavel = function(wdgTipoMedidaVariavel) {
        wdgTipoMedidaVariavel.saveRow().then(function() {
            wdgTipoMedidaVariavel.list();
            wdgTipoMedidaVariavel.reload();
        }).catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    this.wdgServicosEscopoOnEnterTab = function(wdgServicosEscopo) {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        var escopos = wdgGeracaoProposta.currentRow.NRTIPOORCAMENTO;
        var pos = Utilities.getFilterPosition(wdgServicosEscopo.dataSourceFilter, "NRTIPOORCAMENTO");
        wdgServicosEscopo.dataSourceFilter[pos].value = escopos;
        Utilities.carregarWidgetOnEnterTab(wdgServicosEscopo);
    };
    
    this.salvarServicoEscopo = function(wdgServicosEscopo) {
        if (wdgServicosEscopo.currentRow.EXISTE == "N") {
            wdgServicosEscopo.currentRow.__is_new = true;
        }
            
        General.widgetSave(wdgServicosEscopo);
    };
    
    this.gerarJson = function(nomeWidgetAtiva) {
        var wdgGeracaoProposta = getWidgetGeracaoProposta();
        var params = {
            "NRGERACAOPROPOSTA": wdgGeracaoProposta.currentRow.NRGERACAOPROPOSTA
        };
        
        General.callRestEngine("product#/fct_geracaoproposta_modelo/gerarJsonIntegracao", "Row", params).then(function(response) {
            var wdgJson = getWidgetJsonIntegracao();
            wdgJson.isVisible = true;
            ScreenService.openSwipe(wdgJson).then(function() {
                wdgJson.activate();
                wdgJson.view();
                
                var txjson = JSON.stringify(response.dataset.retorno.json, null, 4);
                wdgJson.currentRow.TXJSON = txjson;
                wdgJson.currentRow.WIDGET_ANTERIOR = nomeWidgetAtiva;
            });
        })
        .catch(function(error) {
            Utilities.showErrorMessage(error);
        });
    };
    
    this.fecharWdgJson = function(wdgJson) {
        ScreenService.closeSwipe();
        wdgJson.isVisible = false;
        var widgetAnterior = templateManager.container.getWidget(wdgJson.currentRow.WIDGET_ANTERIOR);
        widgetAnterior.activate();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Sm00013000_GeracaoProposta_Controller", Sm00013000_GeracaoProposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00031000_MapaRecursosAgendados_Controller.js
function Sm00031000_MapaRecursosAgendados_Controller(restEngine, ScreenService, templateManager, localStorageEngine, ApplicationContext) {
   
    /**
     * Variaveis globais para o metodos initmap
     * 
     * @param {nulo} widget de 
     * Mapa de Agendamento
    */
    var map;
    var marker;
    var markers = [];
    var locaisAtend = [];
    var latlngbounds;
    var infowindow;
    var primeiraVez = true;
    var self = this;
    var filterRow = {};
    
   /**
     * função que retorna a url do mapa da widget ativa
     * 
     * @param {nulo} widget de 
     * Mapa de Agendamento
     */
    this.getSRC = function() {
        var widget = templateManager.container.getActiveWidget();
        return widget.mapUrl;
    };
     /**
     * função que retorna a url com a chave da API do google maps 
     * 
     * @param {nulo} widget de 
     * Mapa de Agendamento
     */
    this.getMap = function() {
        return "https://maps.googleapis.com/maps/api/js?key=AIzaSyA5vRtFek_17V-Q6t4bhTYpSCSunxDPzzM&callback=initMap()";
    };
    
     /**
     *Função recursiva que gera os marcadores no mapa de acordo com os resultados no dataset
     * 
     * @param {Dataset, interger, string } widget de 
     * Mapa de Agendamento
     **/
    this.adicionarMarcador = function(dataset, indice, geocoder) {
        if (indice < dataset.length) {
            var row = dataset[indice];
            if (locaisAtend.includes(row.NRESTRUTURAM)) {
                self.adicionarMarcador(dataset, indice + 1, geocoder);
            } else {
                locaisAtend.push(row.NRESTRUTURAM);
                if (row.DSLATITUDE && row.DSLONGITUDE) {
                    marker = new google.maps.Marker({
                        position: new google.maps.LatLng(row.DSLATITUDE, row.DSLONGITUDE),
                        title: "Recursos alocados",
                        map: map
                    });
                    latlngbounds.extend(marker.position);
                    markers.push(marker);

                    self.configMarcador(marker, row.AGENDAMENTO, row.NMESTRUTURAM, row.NRESTRUTURAM);
                    self.adicionarMarcador(dataset, indice + 1, geocoder);
                   
                } else {
                    var address = row.DSENDERECOCOMPLETO;
                    if (row.DSENDERECO){
                        geocoder.geocode({
                            'address': address
                        }, function(results, status) {
                            if (status == 'OK') {
                                marker = new google.maps.Marker({
                                    map: map,
                                    position: results[0].geometry.location
                                });
                               latlngbounds.extend(marker.position);
                                markers.push(marker);
                                
                               
                                self.configMarcador(marker, row.AGENDAMENTO, row.NMESTRUTURAM, row.NRESTRUTURAM);
                                self.adicionarMarcador(dataset, indice + 1, geocoder);
                                
                            } else {
                                console.error(status);
                                ScreenService.showMessage("Alguns locais de atendimento não foram mostrados devido à parâmetros de endereço não reconhecidos");
                            }
                        });
                   } else {
                        ScreenService.showMessage("Alguns Locais de Atendimento não possuem endereço cadastrados");
                   } 
                }
            }
             
        } else {
            self.agrupaMarcadores(map,markers);
            if (markers.length == 1) {
                map.setCenter(markers[0].position);
                map.setZoom(18);
            }
            else {
               map.fitBounds(latlngbounds);
            }
        }
    };
      
     /**
     *Função que agrupa os marcadores mais próximos de acordo com o zoom da tela no mapa
     * 
     * @param {object, Array } widget de 
     * Mapa de Agendamento
     **/
    this.agrupaMarcadores = function(map,marker){
        var markerCluster = new MarkerClusterer(map, markers, {imagePath: 'https://developers.google.com/maps/documentation/javascript/examples/markerclusterer/m'});
    };
    
    /**
     *Função que inicializa o  mapa na widget 
     *Inicia a busca da longitude e latitude dos locais de atendimento 
     *mo mapa
     * 
     * @param {Dataset } widget de 
     * Mapa de Agendamento
     **/
    this.initMap = function(dataset) {
        primeiraVez = false;
        var center = {
            lat: -19.931590,
            lng: -43.939584
        };
        
        templateManager.mapPreviousPosition = [];
        map = new google.maps.Map(document.getElementById('map'), {
            center: center,
            zoom: 12
        });
        var geocoder = new google.maps.Geocoder();
        latlngbounds = new google.maps.LatLngBounds();
        
        markers = [];
        locaisAtend = [];
        if (!(dataset instanceof Array)) {
            localStorageEngine.download("product#/crud/fct_mapa_agendamento", []).then(function(dataset) {
                if(dataset.length > 0){
                    self.adicionarMarcador(dataset, 0, geocoder);
                }

            }).catch(function(error) {
                console.error(error);
            });
        } else {
            if(dataset.length > 0){
                self.adicionarMarcador(dataset, 0, geocoder);
            }
        }

    };
    
     /**
     *Função que configura o marcardores com as informações do infobox
     * 
     * @param {object, string, integer, string } widget de 
     * Mapa de Agendamento
     */
    this.configMarcador = function(marker, AGENDAMENTO, NMESTRUTURAM, NRESTRUTURAM) {
       infowindow = new google.maps.InfoWindow();
        marker.addListener('mouseover', function() {
            infowindow.setContent(NMESTRUTURAM);
            infowindow.open(map, marker);
        });
        marker.addListener('mouseout', function() {
            infowindow.setContent(NMESTRUTURAM);
            infowindow.close(map, marker);
        });
        marker.addListener('click', function() {
            map.setCenter(marker.getPosition());
            self.setaParamentros(NRESTRUTURAM);
            var wdgrecursosNoMapaTabbed = templateManager.container.getWidget('wdgrecursosNoMapaTabbed');
            var wdgRecursosNoMapa = templateManager.container.getWidget('wdgrecursosNoMapa');
            ScreenService.openSwipe(wdgrecursosNoMapaTabbed).then(function() {
                wdgRecursosNoMapa.reload();
                ScreenService.openSwipe(wdgrecursosNoMapaTabbed);
            });
        });
    };
    
    /**
     * seta o parametro para o filtro de locais de atedimentos na widget wdgRecursosNoMapa
     * 
     * @param {interger} widget de 
     * wdgRecursosNoMapa
     */
    this.setaParamentros = function(localDeatendimento) {
        var wdgRecursosNoMapa = templateManager.container.getWidget('wdgrecursosNoMapa');
        wdgRecursosNoMapa.dataSourceFilter[0].value = localDeatendimento;
    };
    
     /**
     *Função que abre o filtro na widget mapa de agendamento
     * 
     * @param {nulo} widget de 
     * Mapa
     */
    this.openFilter = function() {
        var filterMap = templateManager.container.getWidget('filterMap');
        var ZhFilterPreferences = ApplicationContext.ZhFilterPreferences;

        ZhFilterPreferences.openFilterWidget(filterMap, filterRow);
    };
    
    /**
     * Guarda os dados da última filtragem 
     * @params{Object} widgetFilter widget de filtro
    **/
    this.backupFilter = function(widgetFilter){
        var fieldsToApplyDefaultValue = widgetFilter.fields;
        fieldsToApplyDefaultValue.forEach(function(fieldName) {
            var field = widgetFilter.getField(fieldName.name);
            if (field) {
                filterRow[fieldName.name] = field.value();
            }
        }.bind(this));
    };
    
     /**
     *Função que fecha o Swipe dos marcadores e reativa a widiget
     * 
     *@param {Object} widget widget de 
     * wdgrecursosNoMapa
     */
    this.voltar = function(widget) {
        var mapa = templateManager.container.getWidget('mapa');
        mapa.activate();
        ScreenService.closeSwipe();
    };
    
     /**
     *Função que recarrega o mapa entre navegaçôes no sistema
     * 
     *@param {Object} widget widget de 
     * Filto
     
    this.recarregar = function(widiget){
        if (primeiraVez == false) {
            var filtro = templateManager.container.getWidget('filterMap');
            filtro.reload();
        }

    };
    **/
}
Configuration(function(ContextRegister) {
    ContextRegister.register('Sm00031000_MapaRecursosAgendados_Controller', Sm00031000_MapaRecursosAgendados_Controller);
});


// FILE: ./assets/js/controllers/Sm00034000_Maximo_Carencia_Proposta_Controller.js
function Sm00034000_Maximo_Carencia_Proposta_Controller(General, ScreenService, Sm00034000_Maximo_Carencia_Proposta_Service, templateManager, localStorageEngine) {
    
    var arrayDasFaixasAtuais = [];
    
    /**
     * Salva o novo registro
     * e seta IDATIVO
     * 
     * @param {object} widget widget de 
     * Carência Máxima por Operador 
     * para salvar o novo registro
    */
    this.saveCarenciaOperador = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRCARENCIAOPER", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRCARENCIAOPER = response.dataset.columnsToReturn[0].NRCARENCIAOPER;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
        }.bind(this));
    };
    

    /**
     * A função "validaCarenciaOperador" verifica se já existe 
     * uma carencia cadastrada para o operador indicado.
     */

    this.validaCarenciaOperador = function(widget){
        
       var parcOperador = widget.currentRow.NRPARCNEGOCIO;
       
       if (widget.currentRow.__is_new == true){
           widget.currentRow.NRDAISMAXCARENCIA = 9999;
           Sm00034000_Maximo_Carencia_Proposta_Service.validaCarenciaOperador(parcOperador).then(function(response){
                if (response.dataset.VALIDAOPER.VALIDA == false) {
                      this.saveCarenciaOperador(widget);  
                }
                else{
                      ScreenService.showMessage('Operação Bloqueada. <br>Já existe carência máxima cadastrada para o operador informado.'); 
                }
    
           }.bind(this)).catch(function(response){
               ScreenService.showMessage(response.data.error);
           });
           
       }
       else{
            this.saveCarenciaOperador(widget); 
       }
       
    };
       /**
     * Função utilizada para validar a current row da widget Faixa de Desconto
     * @params {Row} a current row da widget Faixa de Desconto
     * @returns {Boolean} widget esta valida?
     * 
    */
    this.validaRowSaveFaixaDesconto = function(row){
        var rowPai = this.pegarRowDaWidget("gridCarenciaOperProp");
        var params = {
            "NRCARENCIAOPER" : rowPai.NRCARENCIAOPER,
            "VRINI" : row.VRPERCDESCDE,
            "VRFIN" : row.VRPERCDESCATE
        };
        return General.callRestEngine("product#/fct_carenciaperfil_perfilacesso/pegarBetweenCarencia", "Row", params);
    };
    
    /**
     * Função utilizada para gerar a row quw deverá ser usada no save da Faixa de Desconto
     * @params {Row} a currentRow da widget de Faixa de Desconto
     * @returns {Object} usado para salvar no currentRestEngine
    */
    this.getParamsSaveFaixaDesconto = function(row){
        return (row.__is_new) ? {
            "NRCARENCIAOPER"    : row.NRCARENCIAOPER,
            "VRPERCDESCATE"     : row.VRPERCDESCATE,
            "VRPERCDESCDE"      : row.VRPERCDESCDE,
            "NRCARENCIAFAIXA"   : row.NRPARCELASMAX,
            "NRPARCELASMAX"     : row.NRPARCELASTOTAL,
            "INSERIR"           : row.__is_new
        } : {
            "NRCARENCIAOPER"        : row.NRCARENCIAOPER,
            "VRPERCDESCATE"         : row.VRPERCDESCATE,
            "VRPERCDESCDE"          : row.VRPERCDESCDE,
            "NRCARENCIAFAIXA"       : row.NRPARCELASMAX,
            "NRPARCELASMAX"         : row.NRPARCELASTOTAL,
            "INSERIR"               : false,
            "NRCARENCIAFAIXADESC"   : row.NRCARENCIAFAIXADESC
        };
    };
    
    this.salvarEMostrarMensagem = function(pode,widget,params){
        if(pode.status){
           General.callRestEngine("product#/fct_carenciaperfil_perfilacesso/saveFaixaDesc", "Row", params).then(function(response) {
                widget.list();
                widget.reload();
                var resposta =  response.dataset.request.resulted;
                ScreenService.closeSwipe();
                ScreenService.showMessage(ScreenService.i18n(resposta));
            }).catch(function(err){
                ScreenService.showMessage(ScreenService.i18n(err));
            });
        } else{
            ScreenService.showMessage(ScreenService.i18n(pode.message));
        }
    };
    
    /**
     * Função pública que valida e salva a widget da Faixa de Desconto 
     * @params {Widget} widget da Faixa de Desconto
     * @returns {Object | Error} Um objeto representando a resposta do backend ou então um erro
    */
    this.validaFaixaDeDescontoEInsere = function(widget){
        this.puxarOperador(widget);
        var row = widget.currentRow;
        var params =  this.getParamsSaveFaixaDesconto(row);
        if(row.__is_new){
            this.validaRowSaveFaixaDesconto(row).then(function(response){
                var pode = this.pegarStatusAtualDeFaixaDesconto(row, response);
                this.salvarEMostrarMensagem(pode, widget,params);
            }.bind(this));
        }else{
            var response =  {"dataset":{"request":{"resulted":"0"}}};
            var pode = this.pegarStatusAtualDeFaixaDesconto(row, response);
            this.salvarEMostrarMensagem(pode, widget,params);
        }
    };
    
    /**
     * Salva a faixa de desconto 
     * @params {Widget} widget da faixa de desconto que deve ser salva
    */
    this.saveFaixadeDesconto = function(widget){
        try{
            this.validaFaixaDeDescontoEInsere(widget);
        }catch(e){
            ScreenService.showMessage(ScreenService.i18n(e));
        }
    }; 
    
    /**
     * Puxa o campo do numero e nome do perfil da widget pai. 
     * @params {Widget} widget da faixa de desconto que deve ser alterada
    */
    this.puxarOperador = function(widget){
        var rowPai = this.pegarRowDaWidget("gridCarenciaOperProp");
        if(rowPai){
            widget.currentRow.NRCARENCIAOPER = rowPai.NRCARENCIAOPER;
            widget.currentRow.NMPRINCIPALPARC = rowPai.NMPRINCIPALPARC;
            widget.currentRow.NRPARCELASTOTAL = rowPai.NRPARCELASMAX;
            widget.currentRow.VRPERCMAXDE = rowPai.VRPERCMAXATE;
        }
    };
    
    
    this.filtrarDataSourceFaixaDesconto = function(widget){
        var rowPai = this.pegarRowDaWidget("gridCarenciaOperProp");
        this.pegarDataSourceFiltrado(rowPai.NRCARENCIAOPER).then(function(data){
            widget.dataSource.data = data;
        });
    };
    
    this.pegarNomeAcaoAtivaCarencia = function(widget){
        this.chamadaBackendAcaoCarencia(true);
    };
    
    
    this.alterarNomeAcaoCarencia = function(widget){
        this.chamadaBackendAcaoCarencia(false);
    };
    
    this.pegarDataSourceFiltrado = function(nrparcnegocio){
        var filter = [
            {
                "name" : "NRCARENCIAOPER",
                "operator" : "=",
                "value" : nrparcnegocio 
            }
        ];
        return this.pegarDataSource("fct_carenciafaixadesconto_operador", filter);
    };
    
    
    this.acaoAdicionarFaixaDesconto = function(widget){
        // var quantidadeFaixas = this.pegarWidget("descPorParcelas").dataSource.data.length;    
        // var carenciaMaxima = this.pegarWidget("gridCarenciaOperProp").currentRow.NRPARCELASMAX;
        // if(quantidadeFaixas < carenciaMaxima){
           General.widgetNewRow(widget);
           this.puxarOperador(widget);
        // }else{
        //     ScreenService.showMessage(ScreenService.i18n("Não é possível adicionar mais faixas de descontos pois excedeu o limite máximo para este Operador."));
        // }
    };
    
    /* REGION UTILS*/
    
    this.filterDataSource = function(data, condicao){
        var result = data.filter(function(d){
            return condicao(d);
        }) ;
        return result;
    };
    
    /**
     * Função usada para encapsular a função de pegar uma widget baseada no nome.
     * @params {string} nome da widget
     * @returns Widget widget encontrada pelo nome ou gera erro quando nao encontrada.
    */
    this.pegarWidget = function(nome){
        return templateManager.container.getWidget(nome);
    };
    
    /**
     * Pega a row de uma determinada widget. 
     * @params {string} nome da widget 
     * @returns Object | null  a row ou nulo
    */
    this.pegarRowDaWidget = function(nome){
        var widget = this.pegarWidget(nome);
        return (widget) ? widget.currentRow : null; 
    };
    
    /**
     * Funcao usada para encapsular o procedimento de chamada do backend as rotas são quase iguais e o funcionamento é o mesmo portanto foram unidos em uma unica chamada de metodo.
     * @params {boolean} deve usar a rota de pegar? Senão usara a rota de atualizar
    */
    this.chamadaBackendAcaoCarencia = function(pegar) {
        if (pegar === undefined) {
            pegar = false;
        }
        var params = {};
        var resposta = false;
       General.callRestEngine('product#/fct_parametrogeral_widget/' + ((pegar) ? 'pegar':'atualizar') + 'IdControleCarencia', "Row", params).then(function(response) {
            if (!pegar)
                ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                
            resposta =  ( response.dataset.request.resulted !== "N");
            var ativarWdg = templateManager.container.getWidget('gridCarenciaOperProp').getAction("habCarenciaMaxima");
            ativarWdg.label = resposta ? "Desativ. Controle Carência" : "Ativar Controle de Carência";  
        }).catch(function(err){
            console.log(err);
        });
    };
    
    /**
     * Pega a maior/menor widget baseada em uma condicao
     * @params{Array} array de dados a ser percorrido
     * @params{Function} função usada no teste que retorna boolean e recebe dois parametros
     * @return Widget que passou nos testes 
    */
    this.pegarWidgetPorCondicao = function (arr, condicao) {
        return arr.reduce(function (p, c) {
            return ( condicao(p,c) ? p : c );
        });
    };
    
    this.pegarDataSource = function(rota, filter){
        if (filter === undefined) {
            filter = [];
        }

        var rotaReal = 'product#/crud/' + rota;
        return localStorageEngine.download(rotaReal, filter);
    };
    
    this.pegarStatusAtualDeFaixaDesconto = function(row, response){
        var status = { status : true };   
        const block = "Operação bloqueada! <br> Essa faixa não pode ser inserida pois";
        if(row.NRPARCELASMAX > parseFloat(row.NRPARCELASTOTAL)  || parseFloat(row.VRPERCDESCATE) > parseFloat(row.VRPERCMAXDE)){
            status = { status : false, message: block + 'os valores inseridos representando o número máximo de parcelas ou o percentual máximo de desconto ultrapassam o limite parametrizado na aba da Carência Máxima do Operador.' };
        } else if(response.dataset.request.resulted !== "0"){
            status = { status : false, message: block + 'já existe uma faixa de desconto cadastrado nesse período.'};
        }
        return status;
    };
    
    /* ENDREGION UTILS*/
    
}
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00034000_Maximo_Carencia_Proposta_Controller', Sm00034000_Maximo_Carencia_Proposta_Controller);
});

// FILE: ./assets/js/controllers/Sm00035000_Maximo_Carencia_Proposta_perfilAcesso_Controller.js
function Sm00035000_Maximo_Carencia_Proposta_perfilAcesso_Controller(General, ScreenService, restEngine, templateManager, localStorageEngine) {
    
    var arrayDasFaixasAtuais = [];
    
    /**
     * Salva o novo registro
     * e seta IDATIVO
     * 
     * @param {object} widget widget de 
     * Carência Máxima por Operador 
     * para salvar o novo registro
    */
    this.saveCarenciaPerfilAcesso = function(widget){
        
        widget.currentRow.__columns_to_return = ["NRCARENCIAPERFIL", "NRORG", "DTINCLUSAO", "CDOPERINCLUSAO", "NRORGINCLUSAO"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset && response.dataset.columnsToReturn[0]) {
                widget.currentRow.NRCARENCIAPERFIL = response.dataset.columnsToReturn[0].NRCARENCIAPERFIL;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
                widget.currentRow.DTINCLUSAO     = response.dataset.columnsToReturn[0].DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = response.dataset.columnsToReturn[0].CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = response.dataset.columnsToReturn[0].NRORGINCLUSAO;
            }
            
        }.bind(this));
    };
    

    /**
     * A função "validaCarenciaOperador" verifica se já existe 
     * uma carencia cadastrada para o operador indicado.
     */

    this.validaCarenciaPerfilAcesso = function(widget){
        
       var perfilDeAcesso = widget.currentRow.NRPERFILACESSO;
       
       if (widget.currentRow.__is_new == true){
           this.validaCarencia(perfilDeAcesso).then(function(response){
                if (response.dataset.VALIDAPERF.VALIDA == false) {
                      this.saveCarenciaPerfilAcesso(widget);  
                }
                else{
                      ScreenService.showMessage('Operação Bloqueada. Já existe carência máxima cadastrada para o perfil de acesso selecionado.'); 
                }
    
           }.bind(this)).catch(function(response){
               ScreenService.showMessage(response.data.error);
           });
           
       }
       else{
            this.saveCarenciaPerfilAcesso(widget); 
       }
       
    };
    
     this.validaCarencia = function(perfilDeAcesso){
        
        var paramsRestEngine = {
            'requestType':'Row',
            'row': {'NRPERFILACESSO': perfilDeAcesso}
        };
        
        return restEngine.post('product#/fct_carenciaperfil_perfilacesso/validaCarenciaPerfilAcesso', paramsRestEngine);
        
    };
    
    /**
     * Função utilizada para validar a current row da widget Faixa de Desconto
     * @params {Row} a current row da widget Faixa de Desconto
     * @returns {Boolean} widget esta valida?
     * 
    */
    this.validaRowSaveFaixaDesconto = function(row){
        var rowPai = this.pegarRowDaWidget("gridCarenciaPerfProp");
        var params = {
            "NRPERFILACESSO" : rowPai.NRCARENCIAPERFIL,
            "VRINI" : row.VRPERCDESCDE,
            "VRFIN" : row.VRPERCDESCATE
        };
        return General.callRestEngine("product#/fct_carenciaperfil_perfilacesso/pegarBetweenCarencia", "Row", params);
    };
    
    /**
     * Função utilizada para gerar a row quw deverá ser usada no save da Faixa de Desconto
     * @params {Row} a currentRow da widget de Faixa de Desconto
     * @returns {Object} usado para salvar no currentRestEngine
    */
    this.getParamsSaveFaixaDesconto = function(row){
        return (row.__is_new) ? {
            "NRPERFILACESSO"    : row.NRCARENCIAPERFIL,
            "VRPERCDESCATE"     : row.VRPERCDESCATE,
            "VRPERCDESCDE"      : row.VRPERCDESCDE,
            "NRCARENCIAFAIXA"   : row.NRPARCELASMAX,
            "NRPARCELASMAX"     : row.NRPARCELASTOTAL,
            "INSERIR"           : true 
        } : {
            "NRPERFILACESSO"        : row.NRCARENCIAPERFIL,
            "VRPERCDESCATE"         : row.VRPERCDESCATE,
            "VRPERCDESCDE"          : row.VRPERCDESCDE,
            "NRCARENCIAFAIXA"       : row.NRPARCELASMAX,
            "NRPARCELASMAX"         : row.NRPARCELASTOTAL,
            "INSERIR"               : false,
            "NRCARENCIAFAIXADESC"   : row.NRCARENCIAFAIXADESC
        };
    };
    
    this.salvarEMostrarMensagem = function(pode,widget,params){
        if(pode.status){
           General.callRestEngine("product#/fct_carenciaperfil_perfilacesso/saveFaixaDesc", "Row", params).then(function(response) {
                widget.list();
                widget.reload();
                var resposta =  response.dataset.request.resulted;
                ScreenService.closeSwipe();
                ScreenService.showMessage(ScreenService.i18n(resposta));
            }).catch(function(err){
                ScreenService.showMessage(ScreenService.i18n(err));
            });
        } else{
            ScreenService.showMessage(ScreenService.i18n(pode.message));
        }
    };
    
    /**
     * Função pública que valida e salva a widget da Faixa de Desconto 
     * @params {Widget} widget da Faixa de Desconto
     * @returns {Object | Error} Um objeto representando a resposta do backend ou então um erro
    */
    this.validaFaixaDeDescontoEInsere = function(widget){
        this.puxarPerfilAcesso(widget);
        var row = widget.currentRow;
        var params =  this.getParamsSaveFaixaDesconto(row);
        if(row.__is_new){
            this.validaRowSaveFaixaDesconto(row).then(function(response){
                var pode = this.pegarStatusAtualDeFaixaDesconto(row, response);
                this.salvarEMostrarMensagem(pode,widget,params);
            }.bind(this));
        }else{
            var response =  {"dataset":{"request":{"resulted":"0"}}};
            var pode = this.pegarStatusAtualDeFaixaDesconto(row, response);
            this.salvarEMostrarMensagem(pode,widget,params);
        }
    };
    
    /**
     * Salva a faixa de desconto 
     * @params {Widget} widget da faixa de desconto que deve ser salva
    */
    this.saveFaixadeDesconto = function(widget){
        try{
            this.validaFaixaDeDescontoEInsere(widget);
        }catch(e){
            ScreenService.showMessage(ScreenService.i18n(e));
        }
    };
    
    /**
     * Puxa o campo do numero e nome do perfil da widget pai. 
     * @params {Widget} widget da faixa de desconto que deve ser alterada
    */
    this.puxarPerfilAcesso = function(widget){
        var rowPai = this.pegarRowDaWidget("gridCarenciaPerfProp");
        if(rowPai){
            widget.currentRow.NMPERFILACESSO = rowPai.NMPERFILACESSO;
            widget.currentRow.NRCARENCIAPERFIL = rowPai.NRCARENCIAPERFIL;
            widget.currentRow.NRPARCELASTOTAL = rowPai.NRPARCELASMAX;
            widget.currentRow.VRPERCMAXADE = rowPai.VRPERCMAXATE;
        }
    };
    
    
    this.filtrarDataSourceFaixaDesconto = function(widget){
        var rowPai = this.pegarRowDaWidget("gridCarenciaPerfProp");
        this.pegarDataSourceFiltrado(rowPai.NRPERFILACESSO).then(function(data){
            widget.dataSource.data = data;
        });
    };
    
    this.pegarDataSourceFiltrado = function(nrperfil){
        var filter = [
            {
                "name" : "NRCARENCIAPERFIL",
                "operator" : "=",
                "value" : nrperfil 
            }
        ];
        return this.pegarDataSource("fct_carenciafaixadesconto", filter);
    };
    
    
    this.pegarNomeAcaoAtivaCarencia = function(widget){
        this.chamadaBackendAcaoCarencia(true);
    };
    
    
    this.alterarNomeAcaoCarencia = function(widget){
        this.chamadaBackendAcaoCarencia(false);
    };
    
    this.acaoAdicionarFaixaDesconto = function(widget){
        // var quantidadeFaixas = this.pegarWidget("descPorParcelas").dataSource.data.length;    
        // var carenciaMaxima = this.pegarWidget("gridCarenciaPerfProp").currentRow.NRPARCELASMAX;
        // if(quantidadeFaixas < carenciaMaxima){
           General.widgetNewRow(widget);
           this.puxarPerfilAcesso(widget);
        // }else{
        //     ScreenService.showMessage(ScreenService.i18n("Não é possível adicionar mais faixas de descontos pois excedeu o limite máximo para este Perfil."));
        // }
    };
    
    /* REGION UTILS*/
    
    /**
     * Pega e faz o download de um datasource
     * @params {String} nome do datasource
     * @params {Object} filtro a ser passado
     * @return Promise download do datasource
    */
    this.pegarDataSource = function(rota, filter){
        if (filter === undefined) {
            filter = [];
        }

        var rotaReal = 'product#/crud/'+ rota;
        return localStorageEngine.download(rotaReal, filter);
    };
    
    /**
     * Função usada para encapsular a função de pegar uma widget baseada no nome.
     * @params {string} nome da widget
     * @returns Widget widget encontrada pelo nome ou gera erro quando nao encontrada.
    */
    this.pegarWidget = function(nome){
        return templateManager.container.getWidget(nome);
    };
    
    /**
     * Pega a row de uma determinada widget. 
     * @params {string} nome da widget 
     * @returns Object | null  a row ou nulo
    */
    this.pegarRowDaWidget = function(nome){
        var widget = this.pegarWidget(nome);
        return (widget) ? widget.currentRow : null; 
    };
    
    
    this.chamadaBackendAcaoCarencia = function(pegar){
        if (pegar === undefined) {
            pegar = false;
        }

        var params = {};
        var resposta = false;
       General.callRestEngine('product#/fct_parametrogeral_widget/' + (pegar) ? 'pegar':'atualizar' + 'IdControleCarencia', "Row", params).then(function(response) {
            if (!pegar)
                ScreenService.showMessage(ScreenService.i18n("Processo realizado com sucesso!"));
                
            resposta =  ( response.dataset.request.resulted === "N") ;
            var ativarWdg = templateManager.container.getWidget('gridCarenciaPerfProp').getAction("habCarenciaMaxima");
            ativarWdg.label = resposta ? "Ativar Controle de Carência" : "Desativ. Controle Carência";  
        }).catch(function(err){
            console.log(err);
        });
    };
    
    /**
     * Pega a maior/menor widget baseada em uma condicao
     * @params{Array} array de dados a ser percorrido
     * @params{Function} função usada no teste que retorna boolean e recebe dois parametros
     * @return Widget que passou nos testes 
    */
    this.pegarMaiorWidget = function (arr, condicao) {
      return arr.reduce(function (p, c) {
        return ( condicao(p,c) ? p : c );
      });
    };
    
    this.pegarStatusAtualDeFaixaDesconto = function(row, response){
        var status = { status : true };   
        const block = "Operação bloqueada! <br> Essa faixa não pode ser inserida pois";
        if(row.NRPARCELASMAX > parseFloat(row.NRPARCELASTOTAL)  || parseFloat(row.VRPERCDESCATE) > parseFloat(row.VRPERCMAXADE)){
            status = { status : false, message: block + 'os valores inseridos representando o número máximo de parcelas ou o percentual máximo de desconto ultrapassam o limite parametrizado na aba da Carência Máxima do Operador.' };
        } else if(response.dataset.request.resulted !== "0"){
            status = { status : false, message: block + 'já existe uma faixa de desconto cadastrado nesse período.'};
        }
        return status;
    };
    /* ENDREGION UTILS*/
}
Configuration(function(ContextRegister)
{
    ContextRegister.register('Sm00035000_Maximo_Carencia_Proposta_perfilAcesso_Controller', Sm00035000_Maximo_Carencia_Proposta_perfilAcesso_Controller);
});

// FILE: ./assets/js/controllers/Sm00045000_Despesa_Controller.js
function Sm00045000_Despesa_Controller(General, Utilities,ScreenService, templateManager){
    /**
     * Salva uma nova despesa
     * @params{Object} widget - widget despesas
    **/
    this.SalvarDespesa = function(widget){
        General.widgetSave(widget).then(function(response){
            if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                widget.currentRow.CDDESPESA = response.dataset.columnsToReturn[0].CDDESPESA;
                widget.currentRow.NRORG     = response.dataset.columnsToReturn[0].NRORG;
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Valida e salva um serviço relaciona a uma despesa
     * @params{Object} widget - widget serviço
    **/
    this.salvarServicoDespesa = function(widget){
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["CDDESPESA", "NRCOMPSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>O Serviço selecionado já está ligado a esta despesa.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRCOMPSERVICO").then(function(){
                        widget.reload().then(function(){
                            wdgDespesas.enabled = true;
                            wdgDespesas.widgets.forEach(function(wid){
                                if(wid.name !== widget.name){
                                    wid.enabled = true;
                                }
                            });
                            widget.list();
                        });
                    });
                }
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Deleta mútiplos registros na tela de formação ou de serviço
     * @params{Object} widget - widget serviço
    **/
    this.deletarMultiplos = function(widget){
        widget.dataSource.getCheckedRows().forEach(function(row){
           widget.currentRow = row;
           widget.removeRow(); 
        });
    };
    /**
     * Cancela o formulário de inserção de um novo serviço ou formção 
     * @params{Object} widget - widget servico ou formação de preço
    **/
    this.cancelarFormulario = function(widget){
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        wdgDespesas.enabled = true;
        wdgDespesas.widgets.forEach(function(wid){
            if(wid.name !== widget.name){
                wid.enabled = true;
            }
        });
        widget.dataSource.data.pop();
        widget.list();
    };
    /**
     * Salva uma nova formação de preço ligada a despesa
     * @params{Object} widget - widget formação de preço
    **/
    this.salvarFormacaoDespesas = function(widget){
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["CDDESPESA", "NRFORMPRECOSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>A Formação de Preço selecionada já está ligada a esta despesa.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRFORMPRECOSERVICO").then(function(){
                        widget.reload().then(function(){
                            wdgDespesas.enabled = true;
                            wdgDespesas.widgets.forEach(function(wid){
                                if(wid.name !== widget.name){
                                    wid.enabled = true;
                                }
                            });
                            widget.list();
                        });
                    });
                }
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Habilita a tela para a insersão de um novo serviço ou formação de preço
     * @params{Object} widget - widget formação de preço ou serviço
    **/
    this.adicionarServForm = function(widget){
        var wdgDespesas = templateManager.container.getWidget("wdgDespesas");
        wdgDespesas.enabled = false;
        wdgDespesas.widgets.forEach(function(wid){
            if(wid.name !== widget.name){
                wid.enabled = false;
            }
        });
        General.widgetNewRow(widget);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00045000_Despesa_Controller", Sm00045000_Despesa_Controller);
});

// FILE: ./assets/js/controllers/Sm00046000_Investimentos_Controller.js
function Sm00046000_Investimentos_Controller(templateManager, ScreenService, General, Utilities){
    /**
     * Abre um popup para a inserção de uma nova categoria
    **/
    this.adicionarCategoria = function(){
        ScreenService.closePopup();
        var popupCat = templateManager.container.getWidget("wdgInserirCategoria");
        popupCat.isVisible = true;
        ScreenService.openPopup(popupCat).then(function(){
            popupCat.edit();
            popupCat.newRow();
            popupCat.activate();
        });
    };
    /**
     * Cancela a inserção de uma nova categoria 
     * durante a criação de um investimento
    **/
    this.cancelarInsersaoCategoria = function(){
        var wdgTabbed    = templateManager.container.getWidget("wdgInvertimentogrid");
        var wdgCategoria = templateManager.container.getWidget("wdgInserirCategoria");
        
        wdgCategoria.isVisible = false;
        ScreenService.closePopup();
        wdgTabbed.enabled = true;
        wdgTabbed.activate();
    };
    /**
     * Manda salvar uma nova categoria e a coloca na currentRow 
     * da tela de investimentos
     * @params{Object} widget - widget popup categoria
    **/
    this.salvarNovaCategoria = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var wdgInvest = templateManager.container.getWidget("wdgInvertimentogrid");
            var row = widget.currentRow;
            if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
                widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
                widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
                if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                    widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
            }
            
            widget.saveRow().then(function(response){
                ScreenService.showMessage("Categoria inserida com sucesso!");
                if(response.dataset && response.dataset.columnsToReturn.length > 0)
                    wdgInvest.currentRow.CDCATINVEST = response.dataset.columnsToReturn[0].CDCATINVEST;
                
                wdgInvest.currentRow.NMCATINVEST = row.NMCATINVEST;
                var fieldCat = wdgInvest.getField("NMCATINVEST");
                fieldCat.dataSource.reload().then(function(){
                    fieldCat.dataSource.data.forEach(function(categoria){
                        if(categoria.CDCATINVEST === response.dataset.columnsToReturn[0].CDCATINVEST)
                            categoria.__isSelected = true;
                    });
                });
                
                widget.dataSource.data = [];
                this.cancelarInsersaoCategoria();
            }.bind(this))
            .catch(function(error){
                ScreenService.showMessage("Operação bloqueada!<br>Falha ao inserir categoria.");
            });
        }
    };
    /**
     * Salva um novo investimento
     * @params{Object} widget - widget tabbed investimento
    **/
    this.salvarInvestimento = function(widget){
        General.widgetSave(widget).then(function(response){
            if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0){
                widget.currentRow.NRINVESTIMENTO = response.dataset.columnsToReturn[0].NRINVESTIMENTO;
                widget.currentRow.NRORG          = response.dataset.columnsToReturn[0].NRORG;
            }
        });
    };
    /**
     * Cancela o formulário de inserção de um novo serviço ou formção 
     * @params{Object} widget - widget servico ou formação de preço
    **/
    this.cancelarFormulario = function(widget){
        var wdgInvest = templateManager.container.getWidget("wdgInvertimentogrid");
        wdgInvest.widgets.forEach(function(wid){
            if(wid.name !== widget.name){
                wid.enabled = true;
            }
        });
        wdgInvest.enabled = true;
        widget.dataSource.data.pop();
        widget.list();
    };
    /**
     * Deleta mútiplos registros na tela de formação ou de serviço
     * @params{Object} widget - widget serviço
    **/
    this.deletarMultiplos = function(widget){
        widget.dataSource.getCheckedRows().forEach(function(row){
           widget.currentRow = row;
           widget.removeRow(); 
        });
    };
    /**
     * Valida e salva um serviço relaciona a um investimento
     * @params{Object} widget - widget serviço
    **/
    this.salvarServicoInvestimento = function(widget){
        var wdgInvest = templateManager.container.getWidget("wdgInvertimentogrid");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["NRINVESTIMENTO", "NRCOMPSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>O Serviço selecionado já está ligado a este investimento.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRCOMPSERVICO").then(function(){
                        widget.reload().then(function(){
                            wdgInvest.widgets.forEach(function(wid){
                                if(wid.name !== widget.name){
                                    wid.enabled = true;
                                }
                            });
                            wdgInvest.enabled = true;
                            widget.list();
                        });
                    });
                }
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Salva uma nova formação de preço ligada a um investimento
     * @params{Object} widget - widget formação de preço
    **/
    this.salvarFormacaoInvestimento = function(widget){
        var wdgInvest = templateManager.container.getWidget("wdgInvertimentogrid");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(widget, null, ["NRINVESTIMENTO", "NRFORMPRECOSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>A Formação de Preço selecionada já está ligada a este investimento.");
                } else {
                    Utilities.inserirMultiplosComNovoCodigo(widget, "NRFORMPRECOSERVICO").then(function(){
                        widget.reload().then(function(){
                            wdgInvest.widgets.forEach(function(wid){
                                if(wid.name !== widget.name){
                                    wid.enabled = true;
                                }
                            });
                            wdgInvest.enabled = true;
                            widget.list();
                        });
                    });
                }
            }.bind(this))
            .catch(function(error){
                Utilities.showErrorMessage(error);
            });
        }
    };
    /**
     * Habilita a tela para a insersão de um novo serviço ou formação de preço
     * @params{Object} widget - widget formação de preço ou serviço
    **/
    this.adicionarServForm = function(widget){
        var wdgInvestGrid = templateManager.container.getWidget("wdgInvertimentogrid");
        wdgInvestGrid.widgets.forEach(function(wid){
            if(wid.name !== widget.name){
                wid.enabled = false;
            }
        });
        wdgInvestGrid.enabled = false;
        General.widgetNewRow(widget);
        widget.currentRow.NRINVESTIMENTO = wdgInvestGrid.currentRow.NRINVESTIMENTO;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00046000_Investimentos_Controller", Sm00046000_Investimentos_Controller);
});

// FILE: ./assets/js/controllers/Sm00047000_Taxas_Controller.js
function Sm00047000_Taxas_Controller(General, Utilities, templateManager, ScreenService){
    /**
     * Habilita a tela para receber uma nova taxa
     * @params{Object} widget - widget grid taxas 
    **/
    this.adicionarTaxa = function(widget){
        var wdgTaxasInserindo = templateManager.container.getWidget("wdgTaxasInserindo"); 
        wdgTaxasInserindo.isVisible = true;
        ScreenService.openSwipe(wdgTaxasInserindo).then(function(){
            wdgTaxasInserindo.newRow();
            wdgTaxasInserindo.activate();
        });
    };
    /**
     * Cancela a inserção uma nova taxa
     * @params{Object} widget - widget taxas inserindo 
    **/
    this.cancelarInsercaoTaxas = function(widget){
        var wdgTaxasGrid = templateManager.container.getWidget("wdgTaxas");
        widget.isVisible = false;
        ScreenService.closeSwipe().then(function(){
            wdgTaxasGrid.activate();
            wdgTaxasGrid.reload();
        });
    };
    /**
     * Deleta as taxas selecionadas no grid
     * @params{Object} widget - widget grid taxas 
    **/
    this.deletarTaxas = function(widget){
        widget.dataSource.getCheckedRows().forEach(function(row){
            widget.currentRow = row;
            widget.removeRow();
        });
        
        widget.reload();
    };
    /**
     * Salvar as edições realizadas no grid 
     * @params{Object} widget - widget taxas grid
    **/
    this.salvarEdicaoTaxas = function(widget){
        widget.saveRow().then(function(){
            widget.reload();
            widget.list();
        });
    };
    /**
     * Salva uma nova taxa
     * @params{Object} widget - widget taxas inserindo
    **/    
    this.salvarTaxas = function(widget){
        var wdgTaxasGrid = templateManager.container.getWidget("wdgTaxas"); 
        var isNew = widget.currentRow.__is_new;
        if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit      = widget.paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk     = widget.paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk        = widget.paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
            widget.currentRow.__cast_number     = widget.paramsNewCode.__cast_number || false;
            if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
        }
        
        widget.saveRow().then(function(response){
            if(isNew){
                widget.isVisible = false;
                widget.currentRow = {};
                widget.dataSource.data = [];
                ScreenService.closeSwipe().then(function(){
                    wdgTaxasGrid.activate();
                    wdgTaxasGrid.reload();
                });
            }
        }.bind(this))
        .catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00047000_Taxas_Controller", Sm00047000_Taxas_Controller);
});

// FILE: ./assets/js/controllers/Sm00048000_imposto_Controller.js
function Sm00048000_imposto_Controller(Utilities, templateManager, ScreenService, General, ZHPromise){
    /**
     * cancela o formulário de impostos
     * @params{Object} widget - widget impostos
    **/
    this.cancelarFormImposto = function(widget){
        var wdgCompServ = templateManager.container.getWidget("wdgServicoRelac");
        widget.getAction("avancar_imposto").isVisible           = true;
        widget.getAction("salvar_imposto").isVisible            = false;
        wdgCompServ.getAction("save_impostocomp").isVisible     = false;
        wdgCompServ.getAction("save_composicao").isVisible      = true;
        wdgCompServ.getAction("voltar_compimposto").isVisible   = false;
        wdgCompServ.getAction("cancelar_servimposto").isVisible = true;
        General.widgetCancelForm(widget);
    };
    /**
     * Avança para a widget de composição de serviço para 
     * ser adicionado os serviços ligados ao imposto
     * @params{Object} widget - widget imposto
    **/
    this.avancarImpostoRelacao = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            widget.enabled = false;
            var wdgServico = templateManager.container.getWidget("wdgServicoRelac");
            wdgServico.enabled   = true;
            wdgServico.activate().then(function(){
                wdgServico.getAction("save_impostocomp").isVisible     = true;
                wdgServico.getAction("save_composicao").isVisible      = false;
                wdgServico.getAction("voltar_compimposto").isVisible   = true;
                wdgServico.getAction("cancelar_servimposto").isVisible = false;
                wdgServico.newRow();
            });
        }
    };
    /**
     * Habilita a tela para a edição de um imposto
     * @params{Object} widget - widget imposto
    **/
    this.editarImposto = function(widget){
        widget.getAction("avancar_imposto").isVisible = false;
        widget.getAction("salvar_imposto").isVisible  = true;
        General.widgetEdit(widget);
    };
    /**
     * Deleta mútiplos registros na tela de serviços ou na tela de imposto
     * @params{String} nameWidget - nome da widget que chamou a função 
    **/
    this.deletarMultiplos = function(nameWidget){
        ScreenService.confirmMessage("Excluir Registro?", "question", function(){
            var widget     = templateManager.container.getWidget("wdgImpostos");
            var widgetServ = templateManager.container.getWidget("wdgServicoRelac");
            var nrimposto = [];
            widgetServ.dataSource.getCheckedRows().forEach(function(servico){
                nrimposto.push(servico.NRIMPOSTOSERV);
            });
            var params = {
                "CDIMPOSTO": widget.currentRow.CDIMPOSTO,
                "NRIMPOSTOSERV": (nameWidget === "wdgImpostos") ? null : nrimposto
            };
            General.callRestEngine("product#/fct_relacimpostoserv_widget/deleteImposto", "Row", params).then(function(){
                if(nameWidget === "wdgImpostos"){ 
                    ScreenService.closeSwipe();
                    widget.list();
                    widget.reload();
                } else {
                    widgetServ.getField("NRCOMPSERVICO").dataSource.clearCheckedRows();
                    widgetServ.currentRow = {};
                    widgetServ.reload();
                }
            }).catch(function(error){
                Utilities.showErrorMessage(error);
            });
        });
    };
    /**
     * Salva a edição apenas da widget de imposto
    **/
    this.salvarEdicaoImposto = function(){
        var widget      = templateManager.container.getWidget("wdgImpostos");
        var wdgCompServ = templateManager.container.getWidget("wdgServicoRelac");
        var params = {
            "CDIMPOSTO": widget.currentRow.CDIMPOSTO,
            "VRPRECIMPOSTO": widget.currentRow.VRPRECIMPOSTO
        };
        
        General.callRestEngine("product#/fct_relacimpostoserv_widget/salvarEdicaoImposto", "Row", params).then(function(){
            wdgCompServ.enabled = true;
            widget.view();
        }).catch(function(error){
            Utilities.showErrorMessage(error);
        });
    };
    /**
     * Volta para a widget de imposto
     * @params{Object} widget - widget composição de serviço
    **/
    this.voltarWidgetImposto = function(widget){
        widget.enabled = false;
        var wdgImposto = templateManager.container.getWidget("wdgImpostos");
        wdgImposto.enabled = true;
        wdgImposto.activate().then(function(){
            setTimeout(function(){
                angular.element("label.zh-tab-cell")[0].click();
            }, 100);
        });
    };
    /**
     * salva várias composições de serviço ligadas ao imposto
     * @params{Object} widget - widget composição de serviço
    **/
    this.salvarMultiplasComposicoes = function(widget){
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            var deferred   = ZHPromise.defer();
            var wdgImposto = templateManager.container.getWidget("wdgImpostos");
            widget.currentRow.CDIMPOSTO     = wdgImposto.currentRow.CDIMPOSTO;
            widget.currentRow.VRPRECIMPOSTO = wdgImposto.currentRow.VRPRECIMPOSTO;
            Utilities.validarCamposDuplicados(widget, null, ["CDIMPOSTO", "NRCOMPSERVICO"]).then(function(response){
                if(!response){
                    ScreenService.showMessage("Operação bloqueada.<br/>A composição de serviço selecionada já está ligada ao imposto.");
                } else {
                    Utilities.validarCamposDuplicados(widget, null, ["CDIMPOSTO", "NRCOMPSERVICO"]).then(function(response){
                        if(!response){
                            ScreenService.showMessage("Operação bloqueada.<br/>A composição de serviço selecionada já está ligada ao imposto.");
                        } else {
                            Utilities.inserirMultiplosComNovoCodigo(widget, "NRCOMPSERVICO").then(function(){
                                widget.getField("NRCOMPSERVICO").dataSource.clearCheckedRows();
                                widget.currentRow = {};
                                widget.list();
                                widget.reload();
                                
                                wdgImposto.getAction("avancar_imposto").isVisible  = true;
                                wdgImposto.getAction("salvar_imposto").isVisible   = false;
                                
                                widget.getAction("save_composicao").isVisible      = true;
                                widget.getAction("save_impostocomp").isVisible     = false;
                                widget.getAction("voltar_compimposto").isVisible   = false;
                                widget.getAction("cancelar_servimposto").isVisible = true;
                                ScreenService.showMessage("Composições de Serviço salvas com sucesso.");
                                deferred.resolve();
                            }.bind(this));
                        }
                    });
                }
            }.bind(this))
            .catch(function(error){
                deferred.reject();
            });
            return deferred.promise;
        }
    };
    /**
     * Salva uma nova relação entre imposto e serviço
     * @params{Object} widget - widget composição de serviço
    **/
    this.salvarRelacImpostoServ = function(widget){
        var wdgImposto = templateManager.container.getWidget("wdgImpostos");
        if(!widget.isValid()){
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos.");
        } else {
            Utilities.validarCamposDuplicados(wdgImposto, null, ["CDIMPOSTO"], "product#/crud/fct_relacimpostoserv_widget").then(function(sucess){
                if(!sucess){
                    ScreenService.showMessage("Operação bloqueada.<br/>O Imposto selecionado já foi adicionado.");
                } else {
                    this.salvarMultiplasComposicoes(widget).then(function(){
                        ScreenService.closeSwipe().then(function(){
                            wdgImposto.enabled = true;
                            wdgImposto.list();
                            wdgImposto.reload();
                        }.bind(this));   
                    }.bind(this))
                    .catch(function(error){
                        Utilities.showErrorMessage(error);
                    });
                }
            }.bind(this));
        }
    };
    /**
     * Cancela o formulário de inserção de novas composições de serviço
     * ligadas a um imposto
     * @params{Object} widget - widget composição de serviço
    **/
    this.cancelarInsercaoCompServico = function(widget){
        widget.dataSource.data.pop();
        widget.getField("NRCOMPSERVICO").dataSource.clearCheckedRows();
        widget.currentRow = {};
        widget.list();
    };
    /**
     * controla a acessibilidade, as actions visiveis e habilita a tela
     * para a inserção de uma nova composição
     * @params{Object} widget - widget composição de serviço
    **/
    this.adicionarNovaComposicao = function(widget){
        var wdgImposto = templateManager.container.getWidget("wdgImpostos");
        wdgImposto.widgets.forEach(function(wdg){
            wdg.isVisible = (wdg.name === widget.name);
        });
        widget.getAction("save_composicao").isVisible      = true;
        widget.getAction("save_impostocomp").isVisible     = false;
        widget.getAction("voltar_compimposto").isVisible   = false;
        widget.getAction("cancelar_servimposto").isVisible = true;
        widget.newRow();
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register("Sm00048000_imposto_Controller", Sm00048000_imposto_Controller);    
});

// FILE: ./assets/js/controllers/zhFavorites.js
var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhNavbarDefaultHeader', function(templateManager, ScreenService, eventAggregator, $compile, ApplicationContext) {
	return {
		restrict: 'C',
		link: function($scope, element) {
		    $scope.favoritesList = [];
		    $scope.lastDeletedItem = undefined;
		    
		    var favoritesText = 'Favoritos';
		    var favoritesAddText = 'Adicionar aos Favoritos';
		    var favoritesRemoveText = 'Remover dos Favoritos';
		    var favoritesUndoText = 'DESFAZER';

			// TRANSLATE - Adicionar essas variaveis a lista da aplicação (words.jon)
		    // var favoritesText = ScreenService.i18n('FAVORITES');
		    // var favoritesAddText = ScreenService.i18n('ADD_TO_FAVORITES');
		    // var favoritesRemoveText = ScreenService.i18n('REMOVE_FROM_FAVORITES');
		    // var favoritesUndoText = ScreenService.i18n('UNDO');
		    
		    var favoritesContainer;
		    
		    var startHollowCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M15,4.21l3.51,7.1l7.84,1.14l-5.67,5.53l1.34,7.81L15,22.1l-7.01,3.69l1.34-7.81l-5.67-5.53l7.84-1.14L15,4.21'+
				'			 M15,2.21c-0.76,0-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05'+
				'			l4.92,4.8l-1.16,6.77c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			c0.55-0.53,0.74-1.33,0.51-2.05c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21L15,2.21z"/>'+
				'	</g>'+
				'</svg>';
		    var startFilledCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M28.25,11.83c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21'+
				'			s-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05l4.92,4.8l-1.16,6.77'+
				'			c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			C28.29,13.35,28.48,12.56,28.25,11.83z"/>'+
				'	</g>'+
				'</svg>';
		    
		    function createVisualElements() {
    			var headerContainer = $(element).children()[0];
    			headerContainer.style.width = 'calc(100% - 50px)';
    			
    			favoritesContainer = $('<div class="zh-favorites"></div>').appendTo(element);
    			
    			var favoritesButtonElement = $compile(
    				'<div class="zh-favorites-button" ng-click="toggleList()">'+
    				'  <div class="zh-favorites-button-star" ng-if="!isFavorited">'+
    				'    '+startHollowCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-star" ng-if="isFavorited">'+
    				'    '+startFilledCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-text">'+favoritesText+'</div>'+
    				'</div>'
    			)($scope);
    			var favoritesButton = $(favoritesButtonElement).appendTo(favoritesContainer);
    			
    			var favoritesListElement = $compile(
    				'<div class="zh-favorites-list" ng-class="{\'opened\': opened}">'+
    				'  <div class="zh-favorites-list-button" ng-if="!isFavorited" ng-click="addToFavorites()">'+favoritesAddText+'</div>'+
    				'  <div class="zh-favorites-list-button" ng-if="isFavorited" ng-click="removeFromFavorites()">'+favoritesRemoveText+'</div>'+
    				'  <div class="zh-favorites-list-undo" ng-show="lastDeletedItem" ng-click="undo()">'+favoritesUndoText+'</div>'+
    				'  <div class="zh-favorites-list-items" ng-show="favoritesList.length > 0">'+
    				'	 <div class="zh-favorites-list-item" ng-repeat="item in favoritesList track by item.id" ng-click="openItem(item)">'+
    				'      {{ item.label }}'+
    				'      <span class="zh-favorites-list-item-delete" data-zh-icon="trash icon-white no-border" ng-click="deleteItem(item, $index, $event)"></span>'+
    				'    </div>'+
    				'  </div>'+
    				'</div>'
    			)($scope);
    			var favoritesList = $(favoritesListElement).appendTo(favoritesContainer);
		    }
		    
		    function checkIsFavorited(containerId) {
		    	var oldIsFavorited = $scope.isFavorited;
			    $scope.isFavorited = $scope.favoritesList.findIndex(function(item) { item.id === containerId }) !== -1;
			    if (oldIsFavorited !== $scope.isFavorited) {
			    	templateManager.updateTemplate();
			    }
		    }
		    
		    $scope.addToFavorites = function() {
		    	var item = {
		    		id: templateManager.container.id,
		    		name: templateManager.container.fileName.split('.json')[0],
		    		label: templateManager.container.label,
		    	};
		    	$scope.favoritesList.push(item);
		    	$scope.isFavorited = true;
		    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'add' }});
		    };
		    
		    $scope.removeFromFavorites = function() {
		    	var index = $scope.favoritesList.findIndex(function(item) { item.id === templateManager.container.id });
		    	if (index > -1) {
			    	var item = $scope.favoritesList.splice(index, 1);
			    	$scope.isFavorited = false;
			    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'delete' }});
		    	}
		    };
		    
		    $scope.openItem = function(item) {
		    	ApplicationContext.LibUtilities.openWindow(item.name);
	    		$scope.opened = false;
		    };
		    
		    var undoTimeout;
		    
		    $scope.deleteItem = function(item, index, event) {
		    	$scope.favoritesList.splice(index, 1);
		    	$scope.lastDeletedItem = [item, index ];

		    	clearTimeout(undoTimeout);
		    	undoTimeout = setTimeout(function() {
		    		$scope.lastDeletedItem = undefined;
		    		templateManager.updateTemplate();
		    	}, 10000);

			    eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'delete' }});

		    	if (item.id === templateManager.container.id) {
		    		$scope.isFavorited = false;
		    	}

		    	event.stopImmediatePropagation();
		    };
		    
		    $scope.undo = function() {
		    	if ($scope.lastDeletedItem) {
		    		$scope.favoritesList.splice($scope.lastDeletedItem.index, 0, $scope.lastDeletedItem.item);
		    	}
		    	$scope.lastDeletedItem = undefined;
		    };

		    $scope.toggleList = function() {
		    	$scope.opened = !$scope.opened;
		    };
		    
		    document.addEventListener('mousedown', function (evt) {
		    	if ($(favoritesContainer).find(evt.target).length === 0 && $scope.opened) {
		    		$scope.opened = false;
			    	templateManager.updateTemplate();
		    	}	
		    });
		    
		    eventAggregator.subscribe(eventAggregator.ON_CONTAINER_OPEN, function(args) {
		    	checkIsFavorited(args.container.id);
		    });
		    
		    eventAggregator.subscribe('ZhFavoritesOnLoad', function (favorites) {
		    	
		    	$scope.favoritesList = favorites;
			    if (templateManager.container) {
			    	checkIsFavorited(templateManager.container.id);
			    }
		    });

		    createVisualElements();
		}
	};
});
